<?php
/**
 * Member Templates controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class AdditionalMemberPermissionsController extends ApplicationController {

	var $plugin_name = "additional_member_permissions";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	}
	

	
	function get_additional_member_permission_values() {
		ajx_current("empty");
		
		$values = array();
		
		$pg_id = get_id('pg_id');
		$member_id = get_id('member_id');
		
		$pg = SystemPermissions::findOne(array('conditions' => array("permission_group_id=?", $pg_id)));
		$member = Members::findById($member_id);
		
		$perms = AdditionalMemberPermissions::findAll();
		foreach ($perms as $perm) {
			
			if (!$member instanceof Member) {
				$values[$perm->getName()] = $pg instanceof SystemPermission && $pg->getCanManageDimensionMembers();
			} else {
				$perm_val = $perm->getValueForMemberAndPermissionGroup($member_id, $pg_id);
				if ($perm_val instanceof AdditionalMemberPermissionValue) {
					$values[$perm->getName()] = $perm_val->getValue();
				} else {
					$values[$perm->getName()] = false;
				}
			}
		}
		
		$can_manage_dim_members = false;
		$contact = Contacts::findOne(array('conditions' => 'permission_group_id='.$pg_id));
		$is_user = $contact instanceof Contact && $contact->isUser();
		if ($is_user) {
			$can_manage_dim_members = can_manage_dimension_members($contact);
		} else {
			// let user groups manage these permissions
			$can_manage_dim_members = true;
		}
		
		ajx_extra_data(array('add_perm_values' => $values, 'is_user' => $is_user, 'can_manage_dim_members' => $can_manage_dim_members));
		
	}
	
	
	function save_new_member_additional_permissions() {
		ajx_current("empty");
		
		$pg_id = array_var($_REQUEST, 'pg_id');
		$additional_perms = array_var($_REQUEST, 'additional_perm');
		
		$logged_user_id = logged_user()->getId();
		
		foreach ($additional_perms as $name => $value) {
			$perm = AdditionalMemberPermissions::getByName($name);
			if ($perm instanceof AdditionalMemberPermission) {
				$perm_id = $perm->getId();
				
				DB::execute("INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`) VALUES
				('$perm_id', '0', '$pg_id', 'temp_".$logged_user_id."_$value')
				ON DUPLICATE KEY UPDATE `value`='temp_".$logged_user_id."_$value'");
			}
		}
	}
	
	
	function render_addiotional_permission_inputs() {
		ajx_current("empty");
		
		$member = Members::findById(array_var($_REQUEST, 'member_id'));
		$genid = array_var($_REQUEST, 'genid');
		
		$parameters = array(
				'member' => $member,
				'genid' => $genid,
				'ot_id' => $member->getObjectTypeId(),
				'dim' => $member->getDimension(),
				'onchange' => "og.on_additional_member_perm_change(this, '".$member->getId()."', '$genid')",
		);
		
		ob_start();
		_render_additional_member_permissions($parameters);
		$html = ob_get_clean();
		
		ajx_extra_data(array("html" => $html, "dim_id" => $member->getDimensionId(), 'genid' => $genid));
	}
		
}