<?php
Hook::register("additional_member_permissions");


// verify if user can add, edit or delete the member
function additional_member_permissions_check_additional_member_permissions($parameters, &$ret) {
	$action = array_var($parameters, 'action');
	$member = array_var($parameters, 'member');
	$pg_ids = array_var($parameters, 'pg_ids');
	if (!is_array($pg_ids)) {
		$pg_ids = array($pg_ids);
	}
	$parent_member_id = array_var($parameters, 'parent_member_id');
	
	if ($member instanceof Member && !$member->getDimension()->getDefinesPermissions()) return;
	
	// allow all for superadmins
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) {
		$ret = array();
		return;
	}
	
	$error = array("message" => lang("no access permissions")); // default error message
	
	foreach ($pg_ids as $pg_id) {
		switch ($action) {
			case 'add': 
				$error = _check_additional_add_member_permissions($member, $parent_member_id, $pg_id);
				break;
			case 'edit':
				$error = _check_additional_edit_member_permissions($member, $pg_id);
				break;
			case 'delete':
				$error = _check_additional_delete_member_permissions($member, $pg_id);
				break;
			default:
				break;
		}
		
		// if no error, then user has a permission group that has the permission => return true
		if (!$error || (is_array($error) && count($error)==0)) {
			$ret = null;
			return;
		}
	}
	$ret = $error;
}


// render permissions
function _render_additional_member_permissions($parameters) {
	$member = array_var($parameters, 'member');
	$dim = array_var($parameters, 'dim');
	$genid = array_var($parameters, 'genid');
	$ot_id = array_var($parameters, 'ot_id');
	
	if ($member instanceof Member && $member->getObjectTypeId() > 0) $ot_id = $member->getObjectTypeId();
	if ($member instanceof Member && $member->getDimension() instanceof Dimension) $dim = $member->getDimension();
	
	if ($ot_id) {
		$object_type_name = strtolower(lang(ObjectTypes::findById($ot_id)->getName()));
		$doths = DimensionObjectTypeHierarchies::findAll(array('conditions' => array('dimension_id = ? AND parent_object_type_id = ?', $dim->getId(), $ot_id)));
	} else {
		$object_type_name = lang('member');
		$doths = array();
	}
	$child_ot_names = "";
	
	
	if (count($doths) > 0) {
		$child_ot_ids = array();
		foreach($doths as $doth) {
			$child_ot_ids[] = $doth->getChildObjectTypeId();
		}
		$child_ots = ObjectTypes::findAll(array('conditions' => "id IN (".implode(',', $child_ot_ids).")"));
		$i=0;
		foreach ($child_ots as $child_ot) {
			$i++;
			$child_ot_names .= ($child_ot_names == "" ? "" : ($i==count($child_ots) ? " ".lang('and')." " : ", ")) . strtolower($child_ot->getPluralObjectTypeName());
		}
	}
	
	$perms = AdditionalMemberPermissions::findAll();
	foreach ($perms as $perm) { /* @var $perm AdditionalMemberPermission */
		switch ($perm->getType()) {
			case 'bool':
				$input_options = array('id' => $genid . 'additional_perm['.$perm->getName().']', 'class' => $genid.'single-add-perm');
				if (array_var($parameters, 'onchange')) {
					$input_options['onchange'] = array_var($parameters, 'onchange');
				}
				echo '<div class="single-add-perm bool">'. checkbox_field('additional_perm['.$perm->getName().']', null, $input_options) .
					'<label for="'.$genid .'additional_perm['.$perm->getName().']">'.lang('additional member permission '.$perm->getName(), $object_type_name, $child_ot_names).'</label></div>';
				break;
				
			case 'list':
				$options = array();
				$option_vals = explode(',', $perm->getPossibleValues());
				foreach ($option_vals as $op_val) $options[] = option_tag($op_val, $op_val);
				
				echo '<div class="single-add-perm list"><label for="'.$genid .'additional_perm['.$perm->getName().']">'.lang($perm->getName(), $object_type_name, $child_ot_names).'</label>'.
					select_box('additional_perm['.$perm->getName().']', $options, array('id' => $genid.'additional_perm['.$perm->getName().']')) .'</div>';
				break;
		}
	}
}


function additional_member_permissions_render_additional_member_permissions($parameters, &$ignored) {
	
	_render_additional_member_permissions($parameters);
	
}


function additional_member_permissions_on_create_user_perm_group($pg, &$ret) {
	if ($pg instanceof PermissionGroup) {
		set_user_config_option('recently_added_pg', $pg->getId(), logged_user()->getId());
	}
}

// set additional permissions for new users
function additional_member_permissions_after_save_contact_permissions($pg_id, &$ignored) {
	$user_is_new = user_config_option('recently_added_pg') == $pg_id;
	
	if (is_numeric($pg_id) && $pg_id > 0) {
	  if ($user_is_new) {
		
		set_user_config_option('recently_added_pg', 0, logged_user()->getId());
		$sql = "
				INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`)
				  SELECT (SELECT id FROM ".TABLE_PREFIX."additional_member_permissions WHERE name='[permission_name]' LIMIT 1) as perm_id, m.id as mem_id, sp.permission_group_id as pg_id, 1
				  FROM ".TABLE_PREFIX."system_permissions sp
				    INNER JOIN ".TABLE_PREFIX."contact_member_permissions cmp ON cmp.permission_group_id=sp.permission_group_id
				    INNER JOIN ".TABLE_PREFIX."members m ON m.id=cmp.member_id
				    INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id=m.dimension_id
				  WHERE sp.permission_group_id=". $pg_id ." AND sp.can_manage_dimension_members=1 AND d.defines_permissions=1 AND d.is_manageable=1
				  GROUP BY perm_id, mem_id, pg_id
				ON DUPLICATE KEY UPDATE permission_id=permission_id;
		";

		DB::executeAll(str_replace('[permission_name]', 'can_edit_member_properties', $sql));
		DB::executeAll(str_replace('[permission_name]', 'can_add_submembers', $sql));
		
	  } else {
	  	
	  	// if permissions are saved in background then take the data from the file, else take it from the request
	  	if (isset($_REQUEST['additional_permissions'])) {
	  		$add_perm_str = $_REQUEST['additional_permissions'];
	  	} else {
	  		$filename = ROOT ."/tmp/additional_permissions_$pg_id";
	  		if (file_exists($filename)) {
		  		$add_perm_str = file_get_contents($filename);
				@unlink($filename);
	  		} else {
	  			return;
	  		}
	  	}
	  	$additional_permissions = json_decode($add_perm_str, true);
	  	
	  	// save the additional permissions
	  	foreach ($additional_permissions as $member_id => $perms) {
	  		foreach ($perms as $perm_name => $value) {
	  			
	  			$sql = "INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`) VALUES
	  				((SELECT id FROM ".TABLE_PREFIX."additional_member_permissions WHERE `name`=".DB::escape($perm_name)."), '$member_id', '$pg_id', '$value')
	  				ON DUPLICATE KEY UPDATE value='$value'";
	  			
	  			DB::execute($sql);
	  		}
	  	}
	  	
	  }
	}
}

function additional_member_permissions_before_save_user_permissions_background($params, &$ignored) {
	$pg_id = array_var($params, 'pg_id');
	$request = array_var($params, 'request', array());
	$add_perm_str = array_var($request, 'additional_permissions');
	
	if ($add_perm_str) {
		$add_perm_filename = ROOT ."/tmp/additional_permissions_$pg_id";
		file_put_contents($add_perm_filename, $add_perm_str);
	}
}

function additional_member_permissions_before_add_member($params, &$ignored) {
	if (can_manage_security(logged_user())) {
		$logged_user_id = logged_user()->getId();
		DB::execute("DELETE FROM ".TABLE_PREFIX."additional_member_permission_values WHERE value LIKE 'temp_".$logged_user_id."_%'");
	}
}

function additional_member_permissions_after_member_save($params, &$ignored) {
	if (array_var($params, 'is_new')) {
		$member = array_var($params, 'member');
		if ($member instanceof Member && $member->getDimension() instanceof Dimension && $member->getDimension()->getDefinesPermissions()) {
			set_user_config_option('recently_added_member', $member->getId(), logged_user()->getId());
		}
	}
}

// when adding a member set additional permissions for all users that can_manage_dimension_members 
function additional_member_permissions_after_save_member_permissions($params, &$ignored) {
	$member = array_var($params, 'member');
	$logged_user_id = array_var($params, 'user_id');
	
	$member_is_new = user_config_option('recently_added_member') == $member->getId();
	
	if ($member_is_new) {
		
		set_user_config_option('recently_added_member', 0);
		
		if ($member instanceof Member && $member->getDimension() instanceof Dimension && $member->getDimension()->getDefinesPermissions()) {
			
			DB::execute("UPDATE ".TABLE_PREFIX."additional_member_permission_values SET member_id='".$member->getId()."', value=REPLACE(value, 'temp_".$logged_user_id."_', '') 
					WHERE member_id=0 AND value LIKE 'temp_".$logged_user_id."_%'");
			
			if ($member->getParentMemberId() == 0) {
				// if is root member add permissions for all users that can_manage_dimension_members
				$sql = "
					INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`)
					  SELECT (SELECT id FROM ".TABLE_PREFIX."additional_member_permissions WHERE name='[permission_name]' LIMIT 1) as perm_id, m.id as mem_id, sp.permission_group_id as pg_id, 1
					  FROM ".TABLE_PREFIX."system_permissions sp
					    INNER JOIN ".TABLE_PREFIX."contacts c ON c.permission_group_id=sp.permission_group_id
					    INNER JOIN ".TABLE_PREFIX."contact_member_permissions cmp ON cmp.permission_group_id=sp.permission_group_id
					    INNER JOIN ".TABLE_PREFIX."members m ON m.id=cmp.member_id
					    INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id=m.dimension_id
					  WHERE m.id=". $member->getId() ." AND sp.can_manage_dimension_members=1 AND d.defines_permissions=1 AND d.is_manageable=1
					  GROUP BY perm_id, mem_id, pg_id
					ON DUPLICATE KEY UPDATE permission_id=permission_id;
				";
			
				DB::executeAll(str_replace('[permission_name]', 'can_edit_member_properties', $sql));
				DB::executeAll(str_replace('[permission_name]', 'can_add_submembers', $sql));
				
			} else {
				// if has parent then inherit permissions
				$parent_perms = AdditionalMemberPermissionValues::findAll(array('conditions' => 'member_id='.$member->getParentMemberId()));
				$values_str = "";
				foreach ($parent_perms as $pp) {
					$values_str .= ($values_str == "" ? "" : ",") . "('".$pp->getPermissionId()."', '".$member->getId()."', '".$pp->getPermissionGroupId()."', '".$pp->getValue()."')";
				}
				if ($values_str != "") {
					$sql = "INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`) VALUES
							$values_str
							ON DUPLICATE KEY UPDATE permission_id=permission_id;";
					DB::executeAll($sql);
				}
				
			}
		}
		
	}
}

function additional_member_permissions_after_save_member_permissions_for_pg($request, &$ignored) {
	
	$additional_permissions = array_var($request, 'additional_perm');
	$member_id = array_var($request, 'member_id');
	$pg_id = array_var($request, 'pg_id');
	$apply_submembers = array_var($request, 'apply_submembers');
	
	if (is_numeric($member_id) && is_numeric($pg_id) && $member_id>0 && $pg_id>0) {
		foreach ($additional_permissions as $perm_name => $value) {
			$perm_def = AdditionalMemberPermissions::getByName($perm_name);
			if ($perm_def instanceof AdditionalMemberPermission) {
				$perm_id = $perm_def->getId();
				DB::execute("INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`) VALUES 
					('$perm_id', '$member_id', '$pg_id', '$value') ON DUPLICATE KEY UPDATE value='$value'");
			}
		}
		
		// if the "apply to submembers" checkbox is checked then replicate this member's additional permissions in all its children. 
		if ($apply_submembers) {
			_apply_additional_permissions_to_children($member_id, $pg_id);
		}
	}
	
}



// ******************************* helpers *******************************

function _apply_additional_permissions_to_children($parent_member_id, $pg_id) {
	$sub_member_ids = array();
	// get submembers
	$sub_members = get_all_children_sorted(array($parent_member_id));
	// check if logged user can view each submember, if not remove it from the array to leave its permissions as they are
	foreach ($sub_members as $k => &$sub_mem) {
		$can_view = ContactMemberPermissions::contactCanAccessMemberAll(logged_user()->getPermissionGroupIds(), $sub_mem['id'], logged_user(), ACCESS_LEVEL_READ);
		if (!$can_view) unset($sub_members[$k]);
		else $sub_member_ids[] = $sub_mem['id'];
	}
	
	if (is_array($sub_member_ids) && count($sub_member_ids) > 0) {
		
		$perms = AdditionalMemberPermissions::findAll();
		
		$values = array();
		foreach ($perms as $perm) {
			$value = $perm->getValueForMemberAndPermissionGroup($parent_member_id, $pg_id);
			if ($value instanceof AdditionalMemberPermissionValue) {
				$values[$perm->getId()] = $value->getValue();
			}
		}
	
		try {
			DB::beginWork();
			foreach ($sub_member_ids as $mid) {
				foreach ($perms as $perm) {
					$perm_id = $perm->getId();
					$val = array_var($values, $perm_id);
	
					$sql = "INSERT INTO ".TABLE_PREFIX."additional_member_permission_values (`permission_id`, `member_id`, `permission_group_id`, `value`) VALUES
					('$perm_id', '$mid', '$pg_id', '$val')
					ON DUPLICATE KEY UPDATE `value`='$val';";
					DB::execute($sql);
				}
			}
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
		}
	}
}

function _check_additional_add_member_permissions($member, $parent_member_id, $pg_id) { /* @var $member Member */
	
	// now this is the only check
	return _check_can_add_submembers($member, $parent_member_id, $pg_id);
}

function _check_can_add_submembers($member, $parent_member_id, $pg_id) { /* @var $member Member */
	$ret = array();
	
	$parent_member = Members::findById($parent_member_id);
	if ($parent_member instanceof Member) {
		$permission = AdditionalMemberPermissions::getByName("can_add_submembers");
		if ($permission instanceof AdditionalMemberPermission) {
			
			$value_obj = $permission->getValueForMemberAndPermissionGroup($parent_member->getId(), $pg_id);
			$error = false;
			if ($value_obj instanceof AdditionalMemberPermissionValue) {
				if (!$value_obj->getValue()) {
					$error = true;
				}
			} else {
				$error = true;
			}
			if ($error) {
				$ot_names_str = "submembers";
				$doths = DimensionObjectTypeHierarchies::findAll(array('conditions' => array('dimension_id = ? AND parent_object_type_id = ?', $parent_member->getDimensionId(), $parent_member->getObjectTypeId())));
				if (count($doths) > 0) {
					$child_ot_ids = array();
					foreach($doths as $doth) {
						$child_ot_ids[] = $doth->getChildObjectTypeId();
					}
					$child_ots = ObjectTypes::findAll(array('conditions' => "id IN (".implode(',', $child_ot_ids).")"));
					$ot_names_str = "";
					$i=0;
					foreach ($child_ots as $child_ot) {
						$i++;
						$ot_names_str .= ($ot_names_str == "" ? "" : ($i==count($child_ots) ? " ".lang('or')." " : ", ")) . $child_ot->getPluralObjectTypeName();
					}
				}
				$ret['message'] = lang('cannot add sub members below', $parent_member->getName(), strtolower($ot_names_str));
			}
		}
	}
	
	return $ret;
}


function _check_additional_edit_member_permissions($member, $pg_id) { /* @var $member Member */
	$ret = array();
	
	$permission = AdditionalMemberPermissions::getByName("can_edit_member_properties");
	if ($permission instanceof AdditionalMemberPermission) {
		
		$value_obj = $permission->getValueForMemberAndPermissionGroup($member->getId(), $pg_id);
		$error = false;
		if ($value_obj instanceof AdditionalMemberPermissionValue) {
			if (!$value_obj->getValue()) {
				$error = true;
			}
		} else {
			$error = true;
		}
		if ($error) {
			$ret['message'] = lang('cannot edit member properties', $member->getName());
		} else {
			// if parent has changed check if user can put childs below it
			$ret = _check_can_add_submembers($member, $member->getParentMemberId(), $pg_id);
		}
	}
	
	return $ret;
}


function _check_additional_delete_member_permissions($member, $pg_id) { /* @var $member Member */
	$ret = array();
	
	$permission = AdditionalMemberPermissions::getByName("can_edit_member_properties");
	if ($permission instanceof AdditionalMemberPermission) {
		
		$value_obj = $permission->getValueForMemberAndPermissionGroup($member->getId(), $pg_id);
		$error = false;
		if ($value_obj instanceof AdditionalMemberPermissionValue) {
			if (!$value_obj->getValue()) {
				$error = true;
			}
		} else {
			$error = true;
		}
		if ($error) {
			$ret['message'] = lang('cannot delete member', $member->getName());
		}
	}
	
	return $ret;
}

