<?php

function add_tasks_show_estimated_cost_column($task, &$info){ 
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowEstimatedCost')) {

        $estimated_cost = array_var($task, 'estimated_cost', 0);

        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format estimated cost
        $formatted_estimated_cost = format_money_amount($estimated_cost, $currency_symbol);

        // Build HTML with estimated cost
        $html = build_html_for_task($formatted_estimated_cost);

        // Add HTML to the task
        $info['additional_data']['tasksShowEstimatedCost'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowEstimatedCost'] = array("html" => "");
    }

}

function add_tasks_show_estimated_price_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowEstimatedPrice')) {

        $estimated_price = array_var($task, 'estimated_price', 0);

        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format estimated price
        $formatted_estimated_price = format_money_amount($estimated_price, $currency_symbol);

        // Build HTML with executed cost
        $html = build_html_for_task($formatted_estimated_price);

        // Add HTML to the task
        $info['additional_data']['tasksShowEstimatedPrice'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowEstimatedPrice'] = array("html" => "");
    }

}

function add_tasks_show_executed_cost_column($task, &$info){ 
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowExecutedCost')) {
        $executed_cost = array_var($task, 'executed_cost', 0);
        $estimated_cost = array_var($task, 'estimated_cost', 0);
        $overvalued = $executed_cost > $estimated_cost;
        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format executed cost
        $formatted_executed_cost = format_money_amount($executed_cost, $currency_symbol);

        // Build HTML with executed cost
        $html = build_html_for_task($formatted_executed_cost, $overvalued);

        // Add HTML to the task
        $info['additional_data']['tasksShowExecutedCost'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowExecutedCost'] = array("html" => "");
    }

}

function add_tasks_show_executed_price_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowExecutedPrice')) {
        $executed_price = array_var($task, 'earned_value', 0);
        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);
        // Format executed price
        $formatted_executed_price = format_money_amount($executed_price, $currency_symbol);
        // Build HTML with executed price
        $html = build_html_for_task($formatted_executed_price);
        // Add HTML to the task
        $info['additional_data']['tasksShowExecutedPrice'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowExecutedPrice'] = array("html" => "");
    }

}

function add_tasks_is_billable_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowIsBillable')) {
        $is_billable = array_var($task, 'is_billable', 1);
        $formatted_is_billable = $is_billable == 1 ? lang('yes') : lang('no');
        // Build HTML with is_billable
        $billable_html = '<td class="additional-list-col">'. $formatted_is_billable . '</td>';
        // Add HTML to the task
        $info['additional_data']['tasksShowIsBillable'] = array(
            "html" => $billable_html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowIsBillable'] = array("html" => "");
    }
}

/*
* HELPER FUNCTIONS
*/

function get_currency_symbol($billing_definition) {
    $currency_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getRateCurrencyId() : null;
    if($currency_id){
        $currency = Currencies::instance()->getCurrency($currency_id);
        $currency_symbol = $currency->getSymbol();
    } else {
        $currency = Currencies::instance()->getDefaultCurrencyInfo();
        $currency_symbol = $currency['symbol'];
    }

    return $currency_symbol;
}

/*function get_task_dimension_members($info){
    foreach($info['members_data'] as $member_data){
        if($member_data['object_type_name']=='hour_type'){
            $task_labor_id = $member_data['id'];
            $task_hour_type_dimension_id = $member_data['dimension_id'];
        }
        if($member_data['object_type_name']=='rate_schedule'){
            $task_rate_schedule_id = $member_data['id'];
            $task_rate_schedule_dimension_id = $member_data['dimension_id'];
        }
    }
    $dim_members = array(
        $task_rate_schedule_dimension_id => array($task_rate_schedule_id),
        $task_hour_type_dimension_id => array($task_labor_id),
    );

    return $dim_members;
}*/

function get_task_dimension_members($allowed_dimensions, $task_id){
    $dim_members = array();
    $sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
    INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
    WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$task_id'";

    $dim_member_rows = DB::executeAll($sql);
    if (is_array($dim_member_rows)) {
        foreach ($dim_member_rows as $dmr) {
            if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
            $dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
        }
    }

    return $dim_members;
}

function get_tasks_timeslots($task_id){
    $extra_conditions = " AND rel_object_id = " . $task_id;
    $timeslots_res = Timeslots::instance()->listing(array(
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "extra_conditions" => $extra_conditions 
    ));
    $timeslots = $timeslots_res->objects;

    return $timeslots;
}

function build_html_for_task($formatted_amount, $overvalued = false){
    $red_text = $overvalued ? 'color: #f00;' : '';
    $html = '<td>';
    $html .= '<div style="font-size: 12px; text-align: right;'.$red_text.'">' . $formatted_amount . '</div>';
    $html .= '</td>';

    return $html;
}


function getTaskEstimatedExecutedValues($task_id){ 
    $result = array(
        'executed_cost' => 0,
        'earned_value' => 0,
        'estimated_cost' => 0,
        'estimated_price' => 0
    );
    $task = ProjectTasks::findById($task_id);
    if($task instanceof ProjectTask){
        Env::useHelper('functions', 'advanced_billing');
        $task_children_financials = get_task_children_financials_info($task);

        // Calculate executed_cost and earnded_value
        $timeslots = get_tasks_timeslots($task_id);
        $executed_cost = 0;
        $earned_value = 0;
        if($timeslots){
            foreach($timeslots as $time){
                $executed_cost += $time['fixed_cost'];
                $earned_value += $time['fixed_billing'];
            }
        }
        $result['executed_cost'] = $executed_cost;
        $result['earned_value'] = $earned_value;

        // Assign values to variables to get billing definition
        $allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
        $cant_decide_between_billing_definitions = null;
        $dim_members = get_task_dimension_members($allowed_dimensions, $task_id);
        $assigned_to_contact = $task->getAssignedToContactId();
        $estimated_time = $task->getTimeEstimate();

        // Calculate estimated cost
        $cost_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_billing_definitions); 
        $hourly_cost = $cost_billing_definition instanceof BillingDefinition ? $cost_billing_definition->getHourlyCost() : 0;
        $estimated_cost = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_cost : 0;

        // Calculate estimated price
        $price_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions); 
        $hourly_price = $price_billing_definition instanceof BillingDefinition ? $price_billing_definition->getHourlyRate() : 0;
        $estimated_price = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_price : 0;

        $result['estimated_cost'] = $estimated_cost;
        $result['estimated_price'] = $estimated_price;

        // Add children financials
        foreach($task_children_financials as $k => $v){
            $result[$k] += $v;
        }

    }
    return $result;
}
