<?php

function get_member_budgeted_financials($member_id){

    $member_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
	$labor_financials = get_labor_budgeted_financials_for_member($member_id);
    $member_financials['budgeted_price'] += $labor_financials['budgeted_price'];
    $member_financials['budgeted_cost'] += $labor_financials['budgeted_cost'];
   
    if(Plugins::instance()->isActivePlugin('expenses2')){
        $expenses_financials = get_expenses_budgeted_financials_for_member($member_id);
        $member_financials['budgeted_price'] += $expenses_financials['budgeted_price'];
        $member_financials['budgeted_cost'] += $expenses_financials['budgeted_cost'];
    }
    
    return $member_financials;
}

function get_labor_budgeted_financials_for_member($member_id){
    $labor_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
    $total_estimated_cost = "SUM(estimated_cost) AS total_estimated_cost ";
    $total_estimated_price = "SUM(estimated_price) AS total_estimated_price ";

    $tasks = ProjectTasks::instance()->listing(array(
        "select_columns" => array("estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $total_estimated_cost, $total_estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "member_ids" => array($member_id),
        "extra_conditions" => " AND `parent_id`=0"
    ))->objects;  
    
    $labor_financials['budgeted_price'] += $tasks[0]['total_estimated_price'] ? $tasks[0]['total_estimated_price'] : 0;
    $labor_financials['budgeted_cost'] += $tasks[0]['total_estimated_cost'] ? $tasks[0]['total_estimated_cost'] : 0;

    return $labor_financials;
}

function get_expenses_budgeted_financials_for_member($member_id, $exp_extra_conditions = ''){
    $expenses_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
    
    $expenses = Expenses::instance()->listing(array(
        "select_columns" => array("e.object_id"),
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "extra_conditions" => $exp_extra_conditions,
        "member_ids" => array($member_id)
    ))->objects;

    $expense_ids = array_column($expenses, 'object_id');

    if(count($expense_ids)){
        $expense_ids_string = '(' . implode(',', $expense_ids) . ')';
        $extra_condition = ' AND expense_id IN '.$expense_ids_string;
        $total_estimated_cost = "SUM(total_cost) AS total_estimated_cost ";
        $total_estimated_price = "SUM(total_price) AS total_estimated_price ";

        $expense_item_totals = ExpenseItems::instance()->listing(array(
            "select_columns" => array("total_price", "total_cost"),
            "query_wraper_start" => "SELECT $total_estimated_cost, $total_estimated_price FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "extra_conditions" => $extra_condition,
            "count_results" => false,
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true
        ))->objects;

        $expenses_financials['budgeted_price'] += $expense_item_totals[0]['total_estimated_price'] ? $expense_item_totals[0]['total_estimated_price'] : 0;
        $expenses_financials['budgeted_cost'] += $expense_item_totals[0]['total_estimated_cost'] ? $expense_item_totals[0]['total_estimated_cost'] : 0;
    }

    return $expenses_financials;
}