<?php 
/*
*
* ESTIMATED LABOR INFO
*
*/
function update_estimated_labor_info_for_member($object){
    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_estimated_labor_info_for_member_using_member_id($project_member_id);  
    }
}

function update_estimated_labor_info_for_member_using_member_id($project_member_id){
    $labor_info = get_estimated_labor_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $labor_info);
    update_estimated_overall_totals_for_member($project_member_id);
    $project_member = Members::findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0){
            update_estimated_labor_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_estimated_labor_info_for_member($project_member_id) {

    $labor_info = array(
        'estimated_labor_minutes' => 0, 
        'estimated_labor_billable_price' => 0,
        'estimated_labor_non_billable_price' => 0,
        'estimated_labor_total_price' => 0,
        'estimated_labor_billable_cost' => 0,
        'estimated_labor_non_billable_cost' => 0,
        'estimated_labor_total_cost' => 0
    );
    
    // Get tasks and financials info
    $estimated_hrs = "SUM(time_estimate) AS total_time_estimate ";
    $estimated_cost = "SUM(estimated_cost) AS total_estimated_cost ";
    $estimated_price = "SUM(estimated_price) AS total_estimated_price ";

    $billable_tasks_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("is_billable", "time_estimate", "estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $estimated_hrs, $estimated_cost, $estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 AND `is_billable`=1 "
    ))->objects; 
    
    $non_billable_tasks_info = ProjectTasks::instance()->listing(array(
        "select_columns" => array("is_billable", "time_estimate", "estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $estimated_hrs, $estimated_cost, $estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 AND `is_billable`=0 "
    ))->objects;  

    calculate_labor_values($labor_info, $billable_tasks_info[0], $non_billable_tasks_info[0]);
    
    return $labor_info;
}

function get_object_project_member_id($object) {
    $project_ot_id = ObjectTypes::findByName('project')->getId();
    $project_members = get_object_members_by_type($object, $project_ot_id);
    $project_member = array_var($project_members, 0);
    $project_member_id = 0;
    if($project_member instanceof Member){
        $project_member_id = $project_member->getId();
    }

    return $project_member_id;
}

function calculate_labor_values(&$labor_info, $billable_tasks_info, $non_billable_tasks_info) {
    // Calculate labor hours
    $labor_info['estimated_labor_minutes'] += $billable_tasks_info['total_time_estimate'] ? $billable_tasks_info['total_time_estimate'] : 0;
    $labor_info['estimated_labor_minutes'] += $non_billable_tasks_info['total_time_estimate'] ? $non_billable_tasks_info['total_time_estimate'] : 0;

    // Calculate estimated labor billable price
    $labor_info['estimated_labor_billable_price'] += $billable_tasks_info['total_estimated_price'] ? $billable_tasks_info['total_estimated_price'] : 0;
    // Calculate estimated labor non-billable price
    $labor_info['estimated_labor_non_billable_price'] += $non_billable_tasks_info['total_estimated_price'] ? $non_billable_tasks_info['total_estimated_price'] : 0;
    // Calculate estimated total price
    $labor_info['estimated_labor_total_price'] += $billable_tasks_info['total_estimated_price'] ? $billable_tasks_info['total_estimated_price'] : 0;
    $labor_info['estimated_labor_total_price'] += $non_billable_tasks_info['total_estimated_price'] ? $non_billable_tasks_info['total_estimated_price'] : 0;

     // Calculate estimated labor billable cost
     $labor_info['estimated_labor_billable_cost'] += $billable_tasks_info['total_estimated_cost'] ? $billable_tasks_info['total_estimated_cost'] : 0;
     // Calculate estimated labor non-billable cost
     $labor_info['estimated_labor_non_billable_cost'] += $non_billable_tasks_info['total_estimated_cost'] ? $non_billable_tasks_info['total_estimated_cost'] : 0;
     // Calculate estimated total cost
     $labor_info['estimated_labor_total_cost'] += $billable_tasks_info['total_estimated_cost'] ? $billable_tasks_info['total_estimated_cost'] : 0;
     $labor_info['estimated_labor_total_cost'] += $non_billable_tasks_info['total_estimated_cost'] ? $non_billable_tasks_info['total_estimated_cost'] : 0;
}

function set_new_member_totals_values($project_member_id, $labor_info) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    DB::beginWork();
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    foreach($labor_info as $key => $value){
        $member_totals->setColumnValue($key, $value);
    }
    $member_totals->save();
    DB::commit();
}

/*
*
* EXECUTED LABOR INFO
*
*/
function update_executed_labor_info_for_member($object){
    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_executed_labor_info_for_member_using_member_id($project_member_id);
    }
}

function update_executed_labor_info_for_member_using_member_id($project_member_id){
    $exec_labor_info = get_executed_labor_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $exec_labor_info);
    update_executed_overall_totals_for_member($project_member_id);
    $project_member = Members::findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0){
            update_executed_labor_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_executed_labor_info_for_member($project_member_id) {

    $labor_info = array(
        'executed_labor_minutes' => 0, 
        'executed_labor_billable_price' => 0,
        'executed_labor_non_billable_price' => 0,
        'executed_labor_total_price' => 0,
        'executed_labor_billable_cost' => 0,
        'executed_labor_non_billable_cost' => 0,
        'executed_labor_total_cost' => 0
    );
    
    // Get tasks and financials info
    $executed_hrs = "SUM(worked_time) AS total_worked_time ";
    $executed_cost = "SUM(fixed_cost) AS total_executed_cost ";
    $executed_price = "SUM(fixed_billing) AS total_executed_price ";

    if (Plugins::instance()->isActivePlugin('income')) {

        $billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("invoicing_status", "worked_time", "fixed_billing", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_price, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "count_results" => false,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `invoicing_status` != 'non_billable' "
        ))->objects; 
        
        $non_billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("invoicing_status", "worked_time", "fixed_billing", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_price, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "count_results" => false,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `invoicing_status` = 'non_billable' "
        ))->objects; 

        calculate_executed_labor_values($labor_info, $billable_time_info[0],  $non_billable_time_info[0]);

    } else {
        $billable_time_info = Timeslots::instance()->listing(array(
            "select_columns" => array("worked_time", "fixed_billing", "fixed_cost"),
            "query_wraper_start" => "SELECT $executed_hrs, $executed_price, $executed_cost FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            "count_results" => false,
            "member_ids" => array($project_member_id)
        ))->objects; 

        $non_billable_time_info = array(
            'total_worked_time' => 0,
            'total_executed_cost' => 0,
            'total_executed_price' => 0
        );

        calculate_executed_labor_values($labor_info, $billable_time_info[0], $non_billable_time_info);
    }
    
    return $labor_info;
}

function calculate_executed_labor_values(&$labor_info, $billable_time, $non_billable_time) {
     // Calculate labor hours
     $labor_info['executed_labor_minutes'] += $billable_time['total_worked_time'] ? $billable_time['total_worked_time'] : 0;
     $labor_info['executed_labor_minutes'] += $non_billable_time['total_worked_time'] ? $non_billable_time['total_worked_time'] : 0;
 
     // Calculate labor billable price
     $labor_info['executed_labor_billable_price'] += $billable_time['total_executed_price'] ? $billable_time['total_executed_price'] : 0;
     // Calculate labor non-billable price
     $labor_info['executed_labor_non_billable_price'] += $non_billable_time['total_executed_price'] ? $non_billable_time['total_executed_price'] : 0;
     // Calculate total price
     $labor_info['executed_labor_total_price'] += $billable_time['total_executed_price'] ? $billable_time['total_executed_price'] : 0;
     $labor_info['executed_labor_total_price'] += $non_billable_time['total_executed_price'] ? $non_billable_time['total_executed_price'] : 0;
 
      // Calculate labor billable cost
      $labor_info['executed_labor_billable_cost'] += $billable_time['total_executed_cost'] ? $billable_time['total_executed_cost'] : 0;
      // Calculate labor non-billable cost
      $labor_info['executed_labor_non_billable_cost'] += $non_billable_time['total_executed_cost'] ? $non_billable_time['total_executed_cost'] : 0;
      // Calculate total cost
      $labor_info['executed_labor_total_cost'] += $billable_time['total_executed_cost'] ? $billable_time['total_executed_cost'] : 0;
      $labor_info['executed_labor_total_cost'] += $non_billable_time['total_executed_cost'] ? $non_billable_time['total_executed_cost'] : 0;
} 

/**
 * Update estimated and executed overall values
 */


function update_estimated_overall_totals_for_member($project_member_id) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    DB::beginWork();
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    $estimated_overall_billable_price =  $member_totals->getColumnValue('estimated_labor_billable_price') +  $member_totals->getColumnValue('estimated_expense_billable_price');
    $estimated_overall_non_billable_price = $member_totals->getColumnValue('estimated_labor_non_billable_price') +  $member_totals->getColumnValue('estimated_expense_non_billable_price');
    $estimated_overall_total_price = $member_totals->getColumnValue('estimated_labor_total_price') +  $member_totals->getColumnValue('estimated_expense_total_price');
    $estimated_overall_billable_cost = $member_totals->getColumnValue('estimated_labor_billable_cost') +  $member_totals->getColumnValue('estimated_expense_billable_cost');
    $estimated_overall_non_billable_cost = $member_totals->getColumnValue('estimated_labor_non_billable_cost') +  $member_totals->getColumnValue('estimated_expense_non_billable_cost');
    $estimated_overall_total_cost = $member_totals->getColumnValue('estimated_labor_total_cost') +  $member_totals->getColumnValue('estimated_expense_total_cost');

    $member_totals->setColumnValue('estimated_overall_billable_price', $estimated_overall_billable_price);
    $member_totals->setColumnValue('estimated_overall_non_billable_price', $estimated_overall_non_billable_price);
    $member_totals->setColumnValue('estimated_overall_total_price', $estimated_overall_total_price);
    $member_totals->setColumnValue('estimated_overall_billable_cost', $estimated_overall_billable_cost);
    $member_totals->setColumnValue('estimated_overall_non_billable_cost', $estimated_overall_non_billable_cost);
    $member_totals->setColumnValue('estimated_overall_total_cost', $estimated_overall_total_cost);

    $member_totals->save();
    DB::commit();
}

function update_executed_overall_totals_for_member($project_member_id) {
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    DB::beginWork();
    if(!$member_totals instanceof MemberTotal) {
        $member_totals = new MemberTotal();
        $member_totals->setMemberId($project_member_id);
    }

    $executed_overall_billable_price =  $member_totals->getColumnValue('executed_labor_billable_price') +  $member_totals->getColumnValue('executed_expense_billable_price');
    $executed_overall_non_billable_price = $member_totals->getColumnValue('executed_labor_non_billable_price') +  $member_totals->getColumnValue('executed_expense_non_billable_price');
    $executed_overall_total_price = $member_totals->getColumnValue('executed_labor_total_price') +  $member_totals->getColumnValue('executed_expense_total_price');
    $executed_overall_billable_cost = $member_totals->getColumnValue('executed_labor_billable_cost') +  $member_totals->getColumnValue('executed_expense_billable_cost');
    $executed_overall_non_billable_cost = $member_totals->getColumnValue('executed_labor_non_billable_cost') +  $member_totals->getColumnValue('executed_expense_non_billable_cost');
    $executed_overall_total_cost = $member_totals->getColumnValue('executed_labor_total_cost') +  $member_totals->getColumnValue('executed_expense_total_cost');

    $member_totals->setColumnValue('executed_overall_billable_price', $executed_overall_billable_price);
    $member_totals->setColumnValue('executed_overall_non_billable_price', $executed_overall_non_billable_price);
    $member_totals->setColumnValue('executed_overall_total_price', $executed_overall_total_price);
    $member_totals->setColumnValue('executed_overall_billable_cost', $executed_overall_billable_cost);
    $member_totals->setColumnValue('executed_overall_non_billable_cost', $executed_overall_non_billable_cost);
    $member_totals->setColumnValue('executed_overall_total_cost', $executed_overall_total_cost);

    $member_totals->save();
    DB::commit();
}