<?php

abstract class BaseMemberTotals extends DataManager {

    /**
     * Column name => Column type map
     *
     * @var array
     * @static
     */
    static private $columns = array(
        'id' => DATA_TYPE_INTEGER,
        'member_id'  => DATA_TYPE_INTEGER,
        'estimated_labor_minutes' => DATA_TYPE_FLOAT,
        'estimated_labor_billable_price' => DATA_TYPE_FLOAT,
        'estimated_labor_non_billable_price' => DATA_TYPE_FLOAT,
        'estimated_labor_total_price' => DATA_TYPE_FLOAT,
        'estimated_labor_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_labor_non_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_labor_total_cost' => DATA_TYPE_FLOAT,
        'executed_labor_minutes' => DATA_TYPE_FLOAT,
        'executed_labor_billable_price' => DATA_TYPE_FLOAT,
        'executed_labor_non_billable_price' => DATA_TYPE_FLOAT,
        'executed_labor_total_price' => DATA_TYPE_FLOAT,
        'executed_labor_billable_cost' => DATA_TYPE_FLOAT,
        'executed_labor_non_billable_cost' => DATA_TYPE_FLOAT,
        'executed_labor_total_cost' => DATA_TYPE_FLOAT,
        'estimated_expense_billable_price' => DATA_TYPE_FLOAT,
        'estimated_expense_non_billable_price' => DATA_TYPE_FLOAT,
        'estimated_expense_total_price' => DATA_TYPE_FLOAT,
        'estimated_expense_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_expense_non_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_expense_total_cost' => DATA_TYPE_FLOAT,
        'executed_expense_billable_price' => DATA_TYPE_FLOAT,
        'executed_expense_non_billable_price' => DATA_TYPE_FLOAT,
        'executed_expense_total_price' => DATA_TYPE_FLOAT,
        'executed_expense_billable_cost' => DATA_TYPE_FLOAT,
        'executed_expense_non_billable_cost' => DATA_TYPE_FLOAT,
        'executed_expense_total_cost' => DATA_TYPE_FLOAT,
        'estimated_overall_billable_price' => DATA_TYPE_FLOAT,
        'estimated_overall_non_billable_price' => DATA_TYPE_FLOAT,
        'estimated_overall_total_price' => DATA_TYPE_FLOAT,
        'estimated_overall_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_overall_non_billable_cost' => DATA_TYPE_FLOAT,
        'estimated_overall_total_cost' => DATA_TYPE_FLOAT,
        'executed_overall_billable_price' => DATA_TYPE_FLOAT,
        'executed_overall_non_billable_price' => DATA_TYPE_FLOAT,
        'executed_overall_total_price' => DATA_TYPE_FLOAT,
        'executed_overall_billable_cost' => DATA_TYPE_FLOAT,
        'executed_overall_non_billable_cost' => DATA_TYPE_FLOAT,
        'executed_overall_total_cost' => DATA_TYPE_FLOAT,
        'invoiced_total' => DATA_TYPE_FLOAT,
        'invoiced_subtotal' => DATA_TYPE_FLOAT,
        'invoiced_paid_amount' => DATA_TYPE_FLOAT,
        'invoiced_due_amount' => DATA_TYPE_FLOAT,
    );

    /**
    * Construct
    *
    * @return BaseMemberTotals
    */
    function __construct() {
        parent::__construct('MemberTotal', 'member_totals', true);
    } // __construct

    /**
     * Return array of object columns
     *
     * @access public
     * @param void
     * @return array
     */
    function getColumns() {
        return array_keys(self::$columns);
    }

    /**
     * Return column type
     *
     * @access public
     * @param string $column_name
     * @return string
     */
    function getColumnType($column_name) {
        if(isset(self::$columns[$column_name])) {
            return self::$columns[$column_name];
        } else {
            return DATA_TYPE_STRING;
        } // if
    }

    // getColumnType

    /**
     * Return array of PK columns. If only one column is PK returns its name as string
     *
     * @access public
     * @param void
     * @return array or string
     */
    function getPkColumns() {
        return 'id';
    }

    /**
     * Return name of first auto_incremenent column if it exists
     *
     * @access public
     * @param void
     * @return string
     */
    function getAutoIncrementColumn() {
        return 'id';
    }

// getAutoIncrementColumn


    // -------------------------------------------------------
    //  Finders
    // -------------------------------------------------------

    /**
     * Do a SELECT query over database with specified arguments
     *
     * @access public
     * @param array $arguments Array of query arguments. Fields:
     * 
     *  - one - select first row
     *  - conditions - additional conditions
     *  - order - order by string
     *  - offset - limit offset, valid only if limit is present
     *  - limit
     * 
     * @return one or  objects
     * @throws DBQueryError
     */
    function find($arguments = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::find($arguments);
        } else {
            return MemberTotals::instance()->find($arguments);
        }
    }

    function findAll($arguments = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::findAll($arguments);
        } else {
            return MemberTotals::instance()->findAll($arguments);
        }
    }

    function findOne($arguments = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::findOne($arguments);
        } else {
            return MemberTotals::instance()->findOne($arguments);
        } // if
    }

// findOne

    /**
     * Return object by its PK value
     *
     * @access public
     * @param mixed $id
     * @param boolean $force_reload If true cache will be skipped and data will be loaded from database
     * @return ImType 
     */
    function findById($id, $force_reload = false) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::findById($id, $force_reload);
        } else {
            return MemberTotals::instance()->findById($id, $force_reload);
        } // if
    }

// findById

    /**
     * Return number of rows in this table
     *
     * @access public
     * @param string $conditions Query conditions
     * @return integer
     */
    function count($condition = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::count($condition);
        } else {
            return MemberTotals::instance()->count($condition);
        } // if
    }

// count

    /**
     * Delete rows that match specific conditions. If $conditions is NULL all rows from table will be deleted
     *
     * @access public
     * @param string $conditions Query conditions
     * @return boolean
     */
    function delete($condition = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::delete($condition);
        } else {
            return MemberTotals::instance()->delete($condition);
        } // if
    }

// delete

    /**
     * This function will return paginated result. Result is an array where first element is 
     * array of returned object and second populated pagination object that can be used for 
     * obtaining and rendering pagination data using various helpers.
     * 
     * Items and pagination array vars are indexed with 0 for items and 1 for pagination
     * because you can't use associative indexing with list() construct
     *
     * @access public
     * @param array $arguments Query argumens (@see find()) Limit and offset are ignored!
     * @param integer $items_per_page Number of items per page
     * @param integer $current_page Current page number
     * @return array
     */
    function paginate($arguments = null, $items_per_page = 10, $current_page = 1, $count = null) {
        if (isset($this) && instance_of($this, 'MemberTotals')) {
            return parent::paginate($arguments, $items_per_page, $current_page);
        } else {
            return MemberTotals::instance()->paginate($arguments, $items_per_page, $current_page);
        }
    }

    /**
     * Return manager instance
     *
     * @return MemberTotals
     */
    function instance() {
        static $instance;
        if (!instance_of($instance, 'MemberTotals')) {
            $instance = new MemberTotals();
        } // if
        return $instance;
    } // instance
}
