<?php
Hook::register('advanced_billing');

function advanced_billing_object_definition($type, &$ret) {
	if ($type == 'Timeslot') {
		$ret['is_fixed_cost'] = DATA_TYPE_BOOLEAN;
		$ret['fixed_cost'] = DATA_TYPE_FLOAT;
		$ret['hourly_cost'] = DATA_TYPE_FLOAT;
		$ret['adv_billing_id'] = DATA_TYPE_INTEGER;
		$ret['adv_cost_id'] = DATA_TYPE_INTEGER;
		$ret['cost_currency_id'] = DATA_TYPE_INTEGER;
		$ret['billing_category_status'] = DATA_TYPE_STRING;
	}

    if ($type == 'ProjectTask') {
		$ret['executed_cost'] = DATA_TYPE_FLOAT;
        $ret['earned_value'] = DATA_TYPE_FLOAT;
        $ret['estimated_cost'] = DATA_TYPE_FLOAT;
        $ret['estimated_price'] = DATA_TYPE_FLOAT;
        $ret['is_calculated_executed_cost'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_earned_value'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_cost'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_price'] = DATA_TYPE_BOOLEAN;
        $ret['is_billable'] = DATA_TYPE_BOOLEAN;
	}
    if ($type == 'TemplateTask') {
        $ret['is_billable'] = DATA_TYPE_BOOLEAN;
    }
}


function advanced_billing_additional_system_columns($manager, &$columns) {
	if ($manager instanceof Timeslots) {
		$columns[] = 'is_fixed_cost';
		$columns[] = 'rate_currency_id';
		$columns[] = 'cost_currency_id';
	}
}

function advanced_billing_more_columns_to_aggregate_in_totals($manager, &$columns) {
	if ($manager instanceof Timeslots) {
		$columns['fixed_cost'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'cost_currency_id', 'group_by' => 'cost_currency_id');
	}
}


function advanced_billing_delete_member($member, &$ignored){

	if ($member instanceof Member) {
		DB::execute("DELETE FROM ".TABLE_PREFIX."billing_definition_members WHERE member_id=".$member->getId());
	}
	
}



function advanced_billing_timeslot_list_additional_actions($params, &$actions) {
	if (can_manage_billing(logged_user())) {
		$actions[] = array(
				'url' => '#',
				'onclick' => "og.advanced_billing.recalculate_timeslots_billing('".array_var($params, 'panel_id')."');",
				'cls' => 'ico-billing',
				'text' => lang('recalculate billing'),
		);
	}
}


function advanced_billing_custom_reports_override_column_format($parameters, &$value) {
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');

	if ($manager instanceof Timeslots) {
		if ($field == 'fixed_billing' || $field == 'fixed_cost' || $field == 'hourly_billing' || $field == 'hourly_cost') {
			if ($field == 'fixed_billing' || $field == 'hourly_billing') {
				$currency = Currencies::getCurrency($object->getColumnValue('rate_currency_id'));
			} else if ($field == 'fixed_cost' || $field == 'hourly_cost') {
				$currency = Currencies::getCurrency($object->getColumnValue('cost_currency_id'));
			}
			$csymbol = null;
			if ($currency instanceof Currency) {
				$csymbol = $currency->getSymbol();
			}else{
			    $csymbol = config_option('currency_code');
            }
			$value = format_money_amount($object->getColumnValue($field), $csymbol);
		}

        if ($field == 'bill_total_price' || $field == 'bill_total_cost' || $field=='bill_billing_rate'  ) {
            $object_billing = ObjectBillings::instance()->findByRelObjectId($object->getRelObjectId());
            if(!is_null($object_billing)){
                if ($field == 'bill_total_price') {
                    $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_total_cost') {
                    $currency = Currencies::instance()->findById($object_billing->getCostCurrencyId());
                }

                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                }else{
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }




	}

    if ($manager instanceof ProjectTasks) {

        $object_billing = ObjectBillings::instance()->findByRelObjectId($object->getObjectId());
        if ($field == 'bill_billing_rate' || $field == 'bill_cost_rate') {
            if(!is_null($object_billing)){
                if ($field == 'bill_billing_rate') {
                    $currency = Currencies::getCurrency($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_cost_rate') {
                    $currency = Currencies::getCurrency($object_billing->getCostCurrencyId());
                }
                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                }else{
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }

        if ($field == 'bill_total_price' || $field == 'bill_total_cost') {
            if(!is_null($object_billing)){
                if ($field == 'bill_total_price') {
                    $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_total_cost') {
                    $currency = Currencies::instance()->findById($object_billing->getCostCurrencyId());
                }

                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                }else{
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }
        if ($field == 'bill_profit_margin') {
            if(!is_null($object_billing)){
                $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                }else{
                    $csymbol = config_option('currency_code');
                }
                $value = format_money_amount($value, $csymbol);
            }
        }
        if ($field == 'bill_profit_margin_percentage') {
                $value = '% '.round($value,2);
        }

    }
}

function advanced_billing_custom_reports_get_possible_external_column_values($parameters, &$values) {
	$ot_id = array_var($parameters, 'ot_id');
	$field = array_var($parameters, 'field');
	$ot = ObjectTypes::findById($ot_id);
	if ($ot->getName() == 'timeslot' || $ot->getName() == 'task') {
		if ($field == 'adv_billing_id' || $field == 'adv_cost_id') {
			$all_billing_definitions = BillingDefinitions::findAll();
			foreach ($all_billing_definitions as $def) {
				$values[] = array('id' => $def->getId(), 'name' => $def->getName());
			}
		} else if ($field == 'billing_category_status') {
			$values[] = array('id' => '', 'name' => lang('All'));
			$values[] = array('id' => 'conflict', 'name' => lang('conflict'));
			$values[] = array('id' => 'solved', 'name' => lang('solved'));
		}
	}
}

// remove columns that are not wanted in the group by combo
function advanced_billing_validate_report_group_by_columns($params, &$objectColumns) {
	$ot = array_var($params, 'ot');
	
	if ($ot instanceof ObjectType && $ot->getName() == 'timeslot') {
		$cols = array();
		foreach ($objectColumns as $k => &$col) {
			if (!in_array($col, array('adv_billing_id','adv_cost_id','fixed_billing','fixed_cost','hourly_billing','hourly_cost','worked_time','subtract','description'))) {
				$cols[] = $col;
			}
		}
		$objectColumns = $cols;
	}
}


function advanced_billing_additional_external_columns($manager, &$columns) {
	if ($manager instanceof Timeslots) {
		$columns[] = 'adv_billing_id';
		$columns[] = 'adv_cost_id';
		$columns[] = 'billing_category_status';
	}
}


function advanced_billing_custom_reports_get_external_column_value($parameters, &$value) {
	$external_id = array_var($parameters, 'external_id');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');

	if ($manager instanceof Timeslots) {
		if ($field == 'adv_billing_id' || $field == 'adv_cost_id') {
			$billing_def = BillingDefinitions::instance()->getById($external_id);
			
			if ($billing_def instanceof BillingDefinition) {
				$value = $billing_def->getName();
			} else {
				$value = "";
			}
		} else if ($field == 'billing_category_status') {
			if ($external_id != '') {
				$value = lang($external_id);
			}
		}
	}
}


function advanced_billing_additional_task_timeslot_class($timeslot, &$add_cls) {
	$cat_status = $timeslot->getColumnValue('billing_category_status');
	if ($cat_status == 'conflict') {
		$add_cls = "adv-billing-conflict";
	}
}

//Revise:
//I couldn't find where this function is ever used
//Parameter "ignored" is not used inside the function
function advanced_billing_timeslots_list_additional_sections($ignored) {
	tpl_display(get_template_path('timeslots_with_conflict_section', 'timeslots_list', 'advanced_billing'));
}

function advanced_billing_edit_timeslot_form_additional_fields($object, &$categories) {
	$cat_status = $object->getColumnValue('billing_category_status');
	if ($cat_status == 'conflict') {
		$statuses = array('conflict','solved');
		$options = array();
		foreach ($statuses as $st) {
			$attr = $cat_status == $st ? array('selected' => 'selected') : null;
			$options[] = option_tag(lang($st), $st, $attr);
		}
		$before_inputs = "<br />";
		$before_inputs .= '<span class="bold" style="min-width:150px;display:inline-block;">'. lang('billing status') .':</span>';
		$before_inputs .= select_box('timeslot[billing_category_status]', $options);
		tpl_assign('before_inputs', $before_inputs);
	}
	
	$sep = '<div style="margin-top:25px; border-bottom:1px solid #ccc;" class="bold">'.lang('billing information').'</div>';
	
	tpl_assign('separator', $sep);
	tpl_assign('timeslot', $object);
    $hide_time_billing_tab = config_option('hide_billing_tab_in_timeslot');
    if(!$hide_time_billing_tab){
        echo tpl_fetch(get_template_path('edit_timestlot_billing_tab', 'billing_definition', 'advanced_billing')); 
    }
}

function advanced_billing_object_edit_categories($object, &$categories) {
	if (!can_manage_billing(logged_user()) || Plugins::instance()->isActivePlugin('overtime_calculations')) return;
	if ($object instanceof Timeslot) {
		
		$cat_status = $object->getColumnValue('billing_category_status');
		if ($cat_status == 'conflict') {
			$statuses = array('conflict','solved');
			$options = array();
			foreach ($statuses as $st) {
				$attr = $cat_status == $st ? array('selected' => 'selected') : null;
				$options[] = option_tag(lang($st), $st, $attr);
			}
			$before_inputs = '<label>'. lang('billing status') .':</label>';
			$before_inputs .= select_box('timeslot[billing_category_status]', $options);
			tpl_assign('before_inputs', $before_inputs);
		}
		
		tpl_assign('timeslot', $object);
        $hide_time_billing_tab = config_option('hide_billing_tab_in_timeslot');
        
        if(!$hide_time_billing_tab){
            $categories[] = array(
                    'id' => 'billing_info',
                    'name' => lang('billing'),
                    'required' => false,
                    'visible' => true,
                    'content' => tpl_fetch(get_template_path('edit_timestlot_billing_tab', 'billing_definition', 'advanced_billing')),
            ); 
        }
		
	}

	if ($object instanceof ProjectTask) {
        $show_financials_tab = config_option('show_financial_tab_in_task_form');
        if($show_financials_tab) {
            $can_see_billing_info = true;
            Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
            $can_see_cost_info = true;
            Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
            if($can_see_billing_info || $can_see_cost_info){
                tpl_assign('task', $object);
                $categories[] = array(
                    'id' => 'task_financials_tab',
                    'name' => lang('financials'),
                    'required' => false,
                    'visible' => true,
                    'content' => tpl_fetch(get_template_path('task_form_financials_tab', 'task', 'advanced_billing'))
                );
            }
        }
        
	}else{
    /*
	    $object_billing = ObjectBillings::findByRelObjectId($object->getObjectId());
	    if ($object_billing instanceof ObjectBilling){

            $cat_status = $object_billing->getColumnValue('billing_category_status');
            if ($cat_status == 'conflict') {
                $statuses = array('conflict','solved');
                $options = array();
                foreach ($statuses as $st) {
                    $attr = $cat_status == $st ? array('selected' => 'selected') : null;
                    $options[] = option_tag(lang($st), $st, $attr);
                }
                $before_inputs = '<label>'. lang('billing status') .':</label>';
                $before_inputs .= select_box('timeslot[billing_category_status]', $options);
                tpl_assign('before_inputs', $before_inputs);
            }
        }else{
            $object_billing = new ObjectBilling();

        }
        tpl_assign('objectBilling', $object_billing);
        $categories[] = array(
            'id' => 'billing_info',
            'name' => lang('billing'),
            'required' => false,
            'visible' => true,
            'content' => tpl_fetch(get_template_path('edit_object_billing_tab', 'billing_definition', 'advanced_billing')),
        );*/
    }
}

function advanced_billing_render_billing_categories_admin_link($ignored, &$ret) {
	// link to billing categories list
	
	$onclick = "og.openLink(og.getUrl('billing_definition', 'init')); return false;"; //, {caller:'billing-tab'}
	
	echo '<div class="configCategory odd">
			<h2><a class="internalLink" href="#" onclick="'. $onclick .'">'. lang("billing categories") .'</a></h2>
			<div class="configCategoryDescription">'. lang("billing categories description") .'</div>
		  </div>';
	
	// dimensions to apply billing
	echo '<div class="configCategory">
			<h2>'. lang("dimensions that can be applied to billing categories") .'</h2>';
	
	// get checked dimensions
	$co = ConfigOptions::getByName('adv_billing_dimensions');
	
	// render checked dimensions
	echo $co->getConfigHandler()->do_render('adv_billing_dimensions', array('onchange_fn' => 'og.onAdvancedBillingDimensionChanged'));
	
	echo '<div class="clear"></div></div>';
	
}

function advanced_billing_render_more_total_task_times_report_params($report_data, &$ret) {
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
	if (can_manage_billing(logged_user()) && $can_see_cost_info) { 
		echo "
			<tr style='height:30px;'>
				<td><span class='bold'>". lang('show costs information') ."</span></td>
				<td align='left' style='padding-left:10px;'>
					". checkbox_field('report[show_cost]', array_var($report_data, 'show_cost', false), array("id" => "report[show_cost]")) ."
				</td>
			</tr>
		";
	}
}

function advanced_billing_after_add_to_members($object, &$ret) {
	if ($object instanceof Timeslot && $object->getColumnValue('invoicing_status') != 'invoiced') {
		Env::useHelper('functions', 'advanced_billing');
		calculate_timeslot_rate_and_cost($object);
	}

    // Calculate and set task's executed cost and value 
    if($object instanceof Timeslot){
        $task_id = $object->getRelObjectId();
        $task = ProjectTasks::findById($task_id);
        if($task instanceof ProjectTask){
            Env::useHelper('functions', 'advanced_billing');
            calculate_task_executed_cost_and_price($task);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }

    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
		$subtasks = array_reverse($object->getAllSubTasks(false));
		// if it has subtasks then the subtask calculations will trigger the parent task calculation
		if (count($subtasks) > 0) {
			foreach ($subtasks as $st) {
				calculate_task_estimated_and_executed_financials($st);
			}
		} else {
			// if no subtasks then recalculate the current task financials
			calculate_task_estimated_and_executed_financials($object);
		}
    }
}


/**
 * After classification a task or a timeslot using drag and drop, recalculate task's financials
 */
function advanced_billing_after_dragdrop_classify($parameters, &$member) {
	
	Env::useHelper('functions', 'advanced_billing');
	
	$objects = array_var($parameters, 'objects');
	$prev_classification = array_var($parameters, 'prev_classification');
	
	foreach ($objects as $obj) {
		if ($obj instanceof ProjectTask) {

			calculate_task_estimated_and_executed_financials($obj);

		} else if ($obj instanceof Timeslot) {

			if ($obj->getColumnValue('invoicing_status') != 'invoiced') {
				calculate_timeslot_rate_and_cost($obj);
			}
			
			$task = $obj->getRelObject();
			if($task instanceof ProjectTask){
				calculate_task_executed_cost_and_price($task);
			}

            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            update_executed_labor_info_for_member($object);
		}
	}
}

function advanced_billing_calculate_executed_cost_and_price($params, &$object){
    if($object instanceof ProjectTask){
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_executed_cost_and_price($object);
    }
}

function advanced_billing_after_content_object_untrash($params, $null){
    $object = array_var($params, 'object');
    if($object instanceof Timeslot){
        $task_id = $object->getRelObjectId();
        $task = ProjectTasks::findById($task_id);
        if($task instanceof ProjectTask){
            Env::useHelper('functions', 'advanced_billing');
            calculate_task_executed_cost_and_price($task);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }
}

function advanced_billing_calculate_estimated_and_executed_financials($params, $object) {
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
    }
}

function advanced_billing_after_template_object_instantiation_and_commit($params, $null){
    $object = array_var($params, 'object');
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
    }
}

function advanced_billing_after_object_save($object, &$ret) {
	if ($object instanceof Timeslot && $object->getColumnValue('invoicing_status') != 'invoiced') {

		Env::useHelper('functions', 'advanced_billing');
		calculate_timeslot_rate_and_cost($object);
		
	}

    if ($object instanceof Timeslot) {
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }

    if ($object instanceof ProjectTask) {
        $extra_data = array_var($_REQUEST, 'object_billing');
        if (array_var($extra_data, 'invoicing_status') != 'invoiced'){

            $object_billing = ObjectBillings::findByRelObjectId($object->getObjectId());
            if (!$object_billing instanceof ObjectBilling){
                $object_billing = new ObjectBilling();
            }
            /*if ($object_billing->getContactId() != $object->getAssignedToContactId()) {
                $object_billing->setForceRecalculateBilling(true);
            }*/
            $object_billing->setForceRecalculateBilling(true);

            $object_billing->setRelObjectId($object->getObjectId());
            $object_billing->setTime($object->getTimeEstimate());
            $object_billing->setContactId($object->getAssignedToContactId());
            $object_billing->save();

            // Billing
            $user = Contacts::findById($object->getAssignedToContactId());
            if ($user instanceof Contact && $user->isUser()) {
	            $billing_category_id = $user->getDefaultBillingId();
	            $bc = BillingCategories::findById($billing_category_id);
	            if ($bc instanceof BillingCategory){
	                $object_billing->setAdvBillingId($billing_category_id);
	                $hourly_billing = $bc->getDefaultValue();
	                $object_billing->setBillingRate($hourly_billing);
	                $object_billing->setTotalPrice($hourly_billing * ($object_billing->getTime() /60));
	                $object_billing->setIsTotalPrice(false);
                }
            }
           $object_billing->save();

           Env::useHelper('functions', 'advanced_billing');
           calculate_rate_and_cost($object,$object_billing);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_estimated_labor_info_for_member($object);
    }

}


function advanced_billing_listing_extra_conditions($ignored, &$extra_conditions) {
	$controller = array_var($_GET, 'c');
	if ($controller == 'time' && array_var($_GET, 'type_filter') == 'conflict') {
		$extra_conditions .= " AND billing_category_status='conflict' ";
	}
}



function advanced_billing_view_timeslot_render_more_columns($timeslot, &$additional_columns) {
    $can_see_cost_info = true; 
    Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
	$show_billing = can_manage_billing(logged_user()) && $can_see_cost_info;
	
	if ($show_billing) {
		$additional_columns['fixed_cost'] = "";
		if ($timeslot->getColumnValue('fixed_cost') > 0) {
			$c = Currencies::getCurrency($timeslot->getColumnValue('cost_currency_id'));
			$c_symbol = $c instanceof Currency ? $c->getSymbol() : '';
			$additional_columns['fixed_cost'] = format_money_amount($timeslot->getColumnValue('fixed_cost'), $c_symbol);
		}
	}

    if($can_see_cost_info) {
        $c = Currencies::getCurrency($timeslot->getColumnValue('cost_currency_id'));
        $c_symbol = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');
        $additional_columns['hourly_cost'] = format_money_amount($timeslot->getColumnValue('hourly_cost'), $c_symbol);
    }

    if($can_see_billing_info) {
        $c = Currencies::getCurrency($timeslot->getColumnValue('rate_currency_id'));
        $c_symbol = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');
        $additional_columns['hourly_billing'] = format_money_amount($timeslot->getColumnValue('hourly_billing'), $c_symbol);
    }
}


function advanced_billing_render_view_billings($object, &$additional_html) {

    // load the default html to print the  billing panel

        $cat_status = $object->getColumnValue('billing_category_status');
        if ($cat_status == 'conflict') {
            $statuses = array('conflict','solved');
            $options = array();
            foreach ($statuses as $st) {
                $attr = $cat_status == $st ? array('selected' => 'selected') : null;
                $options[] = option_tag(lang($st), $st, $attr);
            }
            $before_inputs = '<label>'. lang('billing status') .':</label>';
            $before_inputs .= select_box('object_billing[billing_category_status]', $options);
            tpl_assign('before_inputs', $before_inputs);
        }

        tpl_assign('$object', $object);
        $categories[] = array(
            'id' => 'billing_info',
            'name' => lang('billing'),
            'required' => false,
            'visible' => true,
            'content' => tpl_fetch(get_template_path('edit_object_billing_tab', 'billing_definition', 'advanced_billing')),
        );

}

/**
 * Add currency columns to list in reports
 * @param array $params
 * @param mixed $ret
 */
function advanced_billing_get_allow_columns_for_reports($params, &$ret) {
    $allowed_columns = $ret[0];
    $object_type = array_var($params, 'object_type');

    if ($object_type == ProjectTasks::instance()->getObjectTypeId() || $object_type == Timeslots::instance()->getObjectTypeId()) {

        $managerCurrencyInstance = ObjectBillings::instance();
        $columnCurr = $managerCurrencyInstance->getReportsColumns();
        foreach ($columnCurr as $key => $value) {

            $allowed_columns[] = array('id' => 'bill_' . $value, 'name' => lang('bill_' .$value), 'type' => $value);
        }
    }

    $project_ot = ObjectTypes::findByName('project');
    $client_ot = ObjectTypes::findByName('customer');
    if($object_type == $project_ot->getId() || $object_type == $client_ot->getId()){
        $allowed_columns[] = array('id' => 'budgeted_price', 'name' => lang('budgeted revenue'));
        $allowed_columns[] = array('id' => 'budgeted_cost', 'name' => lang('budgeted cost'));
    }

    $ret[0] = $allowed_columns;
}

/**
 *
 * @param array $params
 * @param mixed $results
 */
function advanced_billing_get_columns_to_header_report($params, &$results) {

    $field = array_var($params, 'field');
    if (str_starts_with($field, 'bill_')) {
        $results['columns']['names'][$field] = lang($field);
        $results['columns']['order'][] = $field;
        $results['columns']['types'][$field] = ObjectBillings::instance()->getColumnType(substr($field, 5));
        if ($field == 'bill_profit_margin' || $field == 'bill_profit_margin_percentage'){
            $results['columns']['types'][$field] = DATA_TYPE_INTEGER;
        }
    }
}

/**
 *
 * @param array $params
 * @param mixed $row_values
 */
function advanced_billing_get_value_columns_to_body_report($params, &$value) {

    $field = array_var($params, 'field');
    $object = array_var($params, 'object');

    if (str_starts_with($field, 'bill_')) {
        $fieldAux = substr($field, 5);

        $idObject = $object->getColumnValue('object_id');
        if ($object instanceof Timeslot){
            $idObject = $object->getColumnValue('rel_object_id');
        }
        $task = ProjectTasks::instance()->findById($idObject);
        if (!is_null($task)){
            $object_billing = ObjectBillings::instance()->findByRelObjectId($task->getObjectId());
            if (!is_null($object_billing)){
                if($fieldAux == 'adv_billing_id' || $fieldAux == 'adv_cost_id'){
                    $billing_def = BillingDefinitions::instance()->findById($object_billing->getColumnValue($fieldAux));
                    if ($billing_def instanceof BillingDefinition){
                        $value = $billing_def->getName();
                    }
                }else if($fieldAux == 'profit_margin') {
                    $value = $object_billing->getTotalPrice() - $object_billing->getTotalCost();
                }else if($fieldAux == 'profit_margin_percentage') {
                    $value =  (($object_billing->getTotalPrice() - $object_billing->getTotalCost())*100)/ $object_billing->getTotalPrice();
                }else{
                    $value = $object_billing->getColumnValue($fieldAux);
                }
            }
        }
    }
}

function advanced_billing_get_additional_sql_join($params, &$ref) {
    $class = array_var($params, 'object_class');

    if ($class == 'ProjectTasks') {
        $ref = array(
            'table' => TABLE_PREFIX . "object_billing",
            'jt_field' => 'rel_object_id',
            'e_field' => 'object_id',
            'join_type' => 'left'
        );
    }
    if ($class == 'Timeslots') {
        $ref = array(
            'table' => TABLE_PREFIX . "object_billing",
            'jt_field' => 'rel_object_id',
            'e_field' => 'rel_object_id',
            'join_type' => 'left'
        );
    }
}
function advanced_billing_get_additional_sql_select($params, &$ref) {
    $class = array_var($params, 'object_class');
    if ($class == 'ProjectTasks' || $class == 'Timeslots') {
        $ref = array('o.*,e.*', 'jt.rel_object_id as bill_rel_object_id ');
    }

}

function advanced_billing_additional_totals_column($params,&$columns){
	$object_type = array_var($params, 'object_type');
    if ($object_type->getName() == 'task' || $object_type->getName() == 'timeslot'){
		// prevent adding the same columns more than one time if hook is called again
		if (array_var($_REQUEST, 'additional_totals_column_already_added')) return;

        $columns .= ", SUM( (CASE WHEN jt.total_cost is null THEN 0 ELSE  jt.total_cost END )) as bill_total_cost ";
        $columns .= ", SUM( (CASE WHEN jt.total_price is null THEN 0 ELSE  jt.total_price END )) as bill_total_price ";
        $columns .= ", jt.rate_currency_id as bill_rate_currency_id ";
        $columns .= ", jt.cost_currency_id as bill_cost_currency_id ";
        $columns .= ", SUM(jt.total_price - jt.total_cost) as bill_profit_margin ";
        $columns .= ", ROUND((SUM(jt.total_price - jt.total_cost)*100)/SUM(jt.total_price),2) as bill_profit_margin_percentage";

		$_REQUEST['additional_totals_column_already_added'] = true;
    }

    if ($object_type->getName() == 'timeslot'){
        $columns .= ",GROUP_CONCAT(Distinct(t.object_id)) as tasks_ids ";
    }
}
function advanced_billing_additional_totals_column_as_array($params,&$columns){
    $object_type = array_var($params, 'object_type');
    if ($object_type->getName() == 'task' || $object_type->getName() == 'timeslot'){
        $columns[] = "jt.total_cost as bill_total_cost ";
        $columns[] = "jt.total_price as bill_total_price ";
        $columns[] = "jt.rate_currency_id as bill_rate_currency_id ";
        $columns[] = "jt.cost_currency_id as bill_cost_currency_id ";
    }
}


/*
	REMOVED BECAUSE IT WAS GENERATING ERROR (blank screen) WHEN EXECUTING EXISTING CUSTOM REPORTS
	
function advanced_billing_list_totals_accumulate_total_rows($params, &$info) {
    $ot = array_var($params, 'object_type');
    $is_simple_report = array_var($params, 'is_simple_report');
    if (($ot->getHandlerClass() == 'ProjectTasks' || $ot->getHandlerClass() == 'Timeslots') ){//&& !$is_simple_report
        $info['bill_total_price'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'bill_rate_currency_id', 'group_by' => 'bill_rate_currency_id');
        $info['bill_total_cost'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'bill_cost_currency_id', 'group_by' => 'bill_cost_currency_id');
        $info['bill_profit_margin'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'bill_rate_currency_id', 'group_by' => 'bill_profit_margin');
        $info['bill_profit_margin_percentage'] = array('operation' => 'average', 'format' => 'percent', 'currency_id_col' => '', 'group_by' => 'bill_profit_margin_percentage');
    }
}
*/

function advanced_billing_aditional_join_listing_total_query($params, &$joins_sql) {
    $ot = array_var($params, 'object_type');
    if ($ot->getHandlerClass() == 'ProjectTasks') {
        $joins_sql .= " LEFT JOIN ".TABLE_PREFIX."object_billing ob ON e.object_id = ob.rel_object_id ";
    }
}

function advanced_billing_aditional_columns_listing_total_query($params, &$ref){
    $ot = array_var($params, 'object_type');
    if ($ot->getHandlerClass() == 'ProjectTasks') {
        $columns_to_aggregate = array(
            'total_price' => array('operation' => 'sum', 'format' => 'money'),
            'total_cost' => array('operation' => 'sum', 'format' => 'money'),
            'rate_currency_id' => array('operation' => '', 'format' => ''),
            'cost_currency_id' => array('operation' => '', 'format' => ''),
            'profit_margin' => array('operation' => 'rest', 'format' => 'money'),
            'profit_margin_percentage' => array('operation' => 'porcent', 'format' => 'percent'),

        );
        foreach ($columns_to_aggregate as $col => $col_data) {
            $operation = array_var($col_data, 'operation');

            if ($operation == 'sum') {
                $ref[] = "ROUND(SUM(ob.$col),2) as bill_$col";
            }
            if ($operation == 'rest') {
                if ($col == 'profit_margin'){
                    $ref[] = "ROUND(SUM(ob.total_price - ob.total_cost),2) as bill_$col";
                }
            }
            if ($operation == 'porcent') {
                if ($col == 'profit_margin_percentage'){
                    $ref[] = "ROUND((SUM(ob.total_price - ob.total_cost)*100)/SUM(ob.total_price),2) as bill_$col";
                }
            }
        }
    }
}

/**
 * Calculate the group totals
 */
function advanced_billing_more_report_group_totals($params, &$level_gr_totals){

    $report = array_var($params, 'report');
    $report_ot = array_var($params, 'object_type');

    if ($report_ot && $report_ot->getHandlerClass() == 'Timeslots') {
        foreach ($level_gr_totals as $k => &$lgt) {
            if (isset($lgt['tasks_ids'])){
                $tasks_ids = $lgt['tasks_ids'];
                $tasks_ids = explode(',',$tasks_ids);
                $total_estimated_time = 0.0;
                $total_final_t_price = 0.0;
                $total_final_t_cost = 0.0;
                if (!empty($tasks_ids)){
                    foreach ($tasks_ids as $id){
                        $columns = array();
                        $report_columns = ReportColumns::getAllReportColumns($report->getId());
                        foreach ($report_columns as $col){
                            $columns[] = $col->getFieldName();
                        }

                        if (in_array('time_estimate',$columns)|| in_array('bill_total_price',$columns) || in_array('bill_total_cost',$columns)){

                                $project_task = ProjectTasks::findById($id);
                                if ($project_task instanceof ProjectTask){
                                    $time_estimate = $project_task->getColumnValue('time_estimate');
                                    $total_estimated_time += $time_estimate;

                                    $ob_billing = ObjectBillings::findByRelObjectId($id);
                                    if ($ob_billing instanceof ObjectBilling){
                                        $total_price = $ob_billing->getTotalPrice();
                                        $total_cost = $ob_billing->getTotalCost();
                                        if ($total_price > 0){
                                            $total_final_t_price += $total_price;
                                        }
                                        if ($total_cost > 0){
                                            $total_final_t_cost += $total_cost;
                                        }
                                    }
                                }
                        }

                    }
                }
                $lgt['time_estimate'] = $total_estimated_time;
                $lgt['bill_total_price'] = $total_final_t_price;
                $lgt['bill_total_cost'] = $total_final_t_cost;
            };
        }
    }
}

function advanced_billing_add_dimensions_to_reload_by_object_type($params, &$result){
    /* 'Rate schedule'(rate_schedules) and 'Labor category'(hour_types) dimensions don't have
    association between each other. In some cases we need to simulate association between them. */
    $dimension_id = $params['dimension_id'];
    $rate_schedule_dim = Dimensions::findByCode('rate_schedules');
    if($rate_schedule_dim instanceof Dimension && $dimension_id == $rate_schedule_dim->getId()){
        $adv_billing_dimensions = config_option('adv_billing_dimensions');
        $hour_type_dim = Dimensions::findByCode('hour_types');
        if (($hour_type_dim instanceof Dimension && in_array($hour_type_dim->getId(), $adv_billing_dimensions)) && in_array($rate_schedule_dim->getId(), $adv_billing_dimensions)){
            $rate_schedule_ot_id = ObjectTypes::findByName('rate_schedule')->getId();
            if (!isset($result[$rate_schedule_ot_id])) $result[$rate_schedule_ot_id] = array();
			$result[$rate_schedule_ot_id][] = $hour_type_dim->getId();
        }
    }
} 

/**
 * Filters the labor categoires selectors using the selected rate schedule
 * It looks for the labor categories associated to the rate schedule (or it childs) defined in the billing cateegories settings
 */
function advanced_billing_modify_initial_list_dimension_members($params, &$result_members){
    $dimension_id = $params['dimension_id'];
    $dimension = $params['dimension'];
    $filtered_members = $params['filtered_by_members'];

	// labor cat dim
    $hour_type_dim = Dimensions::findByCode('hour_types');

    if($hour_type_dim instanceof Dimension && $dimension_id == $hour_type_dim->getId()){
        $rate_schedule_member = null;
		$rate_schedule_dim = Dimensions::findByCode('rate_schedules');
		$adv_billing_dimensions = config_option('adv_billing_dimensions');

        if($rate_schedule_dim instanceof Dimension && in_array($hour_type_dim->getId(), $adv_billing_dimensions) && in_array($rate_schedule_dim->getId(), $adv_billing_dimensions)){
            foreach($filtered_members as $filter_m){
                if($filter_m instanceof Member && $filter_m->getDimensionId() == $rate_schedule_dim->getId()){

                    $hour_type_id = $hour_type_dim->getId();
                    $member_id = $filter_m->getId();

					// get rate schedule childs
					$all_children_ids = $filter_m->getAllChildrenIds(true);
					$member_ids_array = array_merge(array($member_id), $all_children_ids);

					// build the rate schedule member ids string to use in the query
					$member_ids_csv = implode(',', $member_ids_array);

                    $sql = "
						SELECT distinct `member_id`
						FROM `".TABLE_PREFIX."billing_definition_members`
						WHERE `dimension_id` = $hour_type_id AND 
                   	 		`definition_id` IN (
								SELECT distinct `definition_id` 
								FROM `".TABLE_PREFIX."billing_definition_members` 
								WHERE `member_id` IN ($member_ids_csv)
							)
					";
                    
                    $sql_result = DB::execute($sql);
                    $rows = $sql_result->fetchAll();
                    $member_ids_array = array_column($rows, 'member_id');
                    if (!empty($member_ids_array)){
                        $member_ids = implode(',', array_column($rows, 'member_id'));
                    } else {
                        $member_ids = '';
                    }
					
					// also include the labor categories parents
                    $children_members = Members::findAll(array("conditions" => " id IN (" . $member_ids . ")"));
                    $all_parents = Members::getAllParentsInHierarchy($member_ids_array);
                    $result_members = array_merge($children_members, $all_parents);
                }
            }
        }
    }

}


function advanced_billing_timeslot_info_additional_data($object, &$info) {
	if ($object instanceof Timeslot) {
		
		$info['billing_category_name'] = '';
		$info['billing_category_id'] = $object->getColumnValue('adv_billing_id');
		
		if ($info['billing_category_id'] > 0) {
			$cat = BaseBillingDefinitions::instance()->getById($info['billing_category_id']);
			if ($cat) {
				$info['billing_category_name'] = $cat->getName();
			}
		}
		
	}
}

function advanced_billing_task_info_additional_data($task, &$info) {
    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
    Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
    if($can_see_cost_info) add_tasks_show_estimated_cost_column($task, $info); 
    if($can_see_billing_info) add_tasks_show_estimated_price_column($task, $info);
    if($can_see_cost_info) add_tasks_show_executed_cost_column($task, $info);
    if($can_see_billing_info) add_tasks_show_executed_price_column($task, $info);
    add_tasks_is_billable_column($task, $info);

    // Add financials info
    $info['executed_cost'] = array_var($task, 'executed_cost', 0);
    $info['estimated_cost'] = array_var($task, 'estimated_cost', 0);
    $info['earned_value'] = array_var($task, 'earned_value', 0);
    $info['estimated_price'] = array_var($task, 'estimated_price', 0);
    $info['is_calculated_executed_cost'] = array_var($task, 'is_calculated_executed_cost', 1);
    $info['is_calculated_estimated_cost'] = array_var($task, 'is_calculated_estimated_cost', 1);
    $info['is_calculated_earned_value'] = array_var($task, 'is_calculated_earned_value', 1);
    $info['is_calculated_estimated_price'] = array_var($task, 'is_calculated_estimated_price', 1);
    $info['is_billable'] = array_var($task, 'is_billable', 1);


}

function advanced_billing_tasks_user_preferences($ignored, &$array){
    $array['tasksShowEstimatedCost'] = (bool) user_config_option('tasksShowEstimatedCost');
    $array['tasksShowEstimatedPrice'] = (bool) user_config_option('tasksShowEstimatedPrice');
    $array['tasksShowExecutedCost'] = (bool) user_config_option('tasksShowExecutedCost');
    $array['tasksShowExecutedPrice'] = (bool) user_config_option('tasksShowExecutedPrice');
    $array['tasksShowIsBillable'] = (bool) user_config_option('tasksShowIsBillable');
}

function advanced_billing_add_task_group_totals($params, &$totals){ 
    $group_totals = array_var($params, 'group_totals_financials');
    $currency_sym = config_option('currency_code');
    $group_estimated_cost = format_money_amount($group_totals[0]['group_estimated_cost'], $currency_sym);
    $group_estimated_price = format_money_amount($group_totals[0]['group_estimated_price'], $currency_sym);
    $group_executed_cost = format_money_amount($group_totals[0]['group_executed_cost'], $currency_sym);
    $group_earned_value = format_money_amount($group_totals[0]['group_earned_value'], $currency_sym);
    $totals['estimated_cost_string'] = $group_estimated_cost;
    $totals['estimated_price_string'] = $group_estimated_price;
    $totals['executed_cost_string'] = $group_executed_cost;
    $totals['earned_value_string'] = $group_earned_value;
}

function advanced_billing_add_estimated_executed_info_to_view($object, &$result){
    if($object instanceof ProjectTask){
        Env::useHelper('functions', 'advanced_billing');
        $result = get_task_estimated_executed_view_info($object);
    }
}

function advanced_billing_update_calculated_and_manual_data($task_data, &$task){
    if($task instanceof ProjectTask){
        // Set executed cost
        $is_calculated_executed_cost = array_var($task_data, 'is_calculated_executed_cost', 1);
        if($is_calculated_executed_cost == 0) {
            $executed_cost_value = clean_formatted_money_amount_for_sql(array_var($task_data, 'executed_cost_value', 0));
            $task->setColumnValue('executed_cost', $executed_cost_value);
        }

        // Set earned value
        $is_calculated_earned_value = array_var($task_data, 'is_calculated_earned_value', 1);
        if($is_calculated_earned_value == 0) {
            $earned_value_amount = clean_formatted_money_amount_for_sql(array_var($task_data, 'earned_value_amount', 0));
            $task->setColumnValue('earned_value', $earned_value_amount);
        }

        // Set estimated cost value
        $is_calculated_estimated_cost = array_var($task_data, 'is_calculated_estimated_cost', 1);
        if($is_calculated_estimated_cost == 0) {
            $estimated_cost_value = clean_formatted_money_amount_for_sql(array_var($task_data, 'estimated_cost_value', 0));
            $task->setColumnValue('estimated_cost', $estimated_cost_value);
        }

        // Set estimated price value
        $is_calculated_estimated_price = array_var($task_data, 'is_calculated_estimated_price', 1);
        if($is_calculated_estimated_price == 0) {
            $estimated_price_value = clean_formatted_money_amount_for_sql(array_var($task_data, 'estimated_price_value', 0));
            $task->setColumnValue('estimated_price', $estimated_price_value);
        }
    }
}

function advanced_billing_task_clone_more_attributes($parameters, &$ignored) {
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');
	
	if (!is_null($copy) && !is_null($original)) {
		$copy->setColumnValue('executed_cost', $original->getColumnValue('executed_cost'));
        $copy->setColumnValue('earned_value', $original->getColumnValue('earned_value'));
        $copy->setColumnValue('estimated_cost', $original->getColumnValue('estimated_cost'));
        $copy->setColumnValue('estimated_price', $original->getColumnValue('estimated_price'));
        $copy->setColumnValue('is_calculated_executed_cost', $original->getColumnValue('is_calculated_executed_cost'));
        $copy->setColumnValue('is_calculated_earned_value', $original->getColumnValue('is_calculated_earned_value'));
        $copy->setColumnValue('is_calculated_estimated_cost', $original->getColumnValue('is_calculated_estimated_cost'));
        $copy->setColumnValue('is_calculated_estimated_price', $original->getColumnValue('is_calculated_estimated_price'));
        $copy->setColumnValue('is_billable', $original->getColumnValue('is_billable'));
	}
}

function advanced_billing_member_listing_additional_data($params, &$listing_object) { 
    Env::useHelper('member_financials_functions', 'advanced_billing');
	$project_ot = ObjectTypes::findByName('project');
	if ($project_ot && array_var($params, 'type_id') == $project_ot->getId()) {
        $csymbol = null;
        if ($currency instanceof Currency) {
            $csymbol = $currency->getSymbol();
        }else{
            $csymbol = config_option('currency_code');
        }
		foreach ($listing_object['members'] as &$member_data) {
            $member_id = $member_data['id'];
            $project_financials = get_member_budgeted_financials($member_id);
            $member_data['budgeted_price'] = format_money_amount($project_financials['budgeted_price'], $csymbol);
            $member_data['budgeted_cost'] = format_money_amount($project_financials['budgeted_cost'], $csymbol);

            // Get project member estimated totals
            $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$member_id.'"')));
            $member_has_totals = $member_totals instanceof MemberTotal;
            $member_data['estimated_labor_minutes'] = format_amount(($member_has_totals ? $member_totals->getColumnValue('estimated_labor_minutes') : 0 ) / 60);
            $member_data['estimated_labor_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_billable_price') : 0, $csymbol);
            $member_data['estimated_labor_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_non_billable_price') : 0, $csymbol);
            $member_data['estimated_labor_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_total_price') : 0, $csymbol);
            $member_data['estimated_labor_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_billable_cost') : 0, $csymbol);
            $member_data['estimated_labor_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_non_billable_cost') : 0, $csymbol);
            $member_data['estimated_labor_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_total_cost') : 0, $csymbol);

            // Get project member executed totals
            $member_data['executed_labor_minutes'] = format_amount(($member_has_totals ? $member_totals->getColumnValue('executed_labor_minutes') : 0 ) / 60);
            $member_data['executed_labor_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_billable_price') : 0, $csymbol);
            $member_data['executed_labor_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_non_billable_price') : 0, $csymbol);
            $member_data['executed_labor_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_total_price') : 0, $csymbol);
            $member_data['executed_labor_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_billable_cost') : 0, $csymbol);
            $member_data['executed_labor_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_non_billable_cost') : 0, $csymbol);
            $member_data['executed_labor_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_total_cost') : 0, $csymbol);

            // Get project member estimated overall totals
            $member_data['estimated_overall_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_billable_price') : 0, $csymbol);
            $member_data['estimated_overall_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_non_billable_price') : 0, $csymbol);
            $member_data['estimated_overall_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_price') : 0, $csymbol);
            $member_data['estimated_overall_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_billable_cost') : 0, $csymbol);
            $member_data['estimated_overall_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_non_billable_cost') : 0, $csymbol);
            $member_data['estimated_overall_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_cost') : 0, $csymbol);

            // Get project member executed overall totals
            $member_data['executed_overall_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_billable_price') : 0, $csymbol);
            $member_data['executed_overall_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_non_billable_price') : 0, $csymbol);
            $member_data['executed_overall_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_total_price') : 0, $csymbol);
            $member_data['executed_overall_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_billable_cost') : 0, $csymbol);
            $member_data['executed_overall_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_non_billable_cost') : 0, $csymbol);
            $member_data['executed_overall_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_total_cost') : 0, $csymbol);
            
		}


	}
}

function advanced_billing_template_task_to_task_columns($parameters, &$new_task) { 
    $template_task = array_var($parameters, 'template_task');
	if ($new_task instanceof ProjectTask) {
		$new_task->setColumnValue('is_calculated_executed_cost', 1);
        $new_task->setColumnValue('is_calculated_earned_value', 1);
        $new_task->setColumnValue('is_calculated_estimated_cost', 1);
        $new_task->setColumnValue('is_calculated_estimated_price', 1);
        if($template_task instanceof TemplateTask){
            $new_task->setColumnValue('is_billable', $template_task->getColumnValue('is_billable'));
        }
	}
}

function advanced_billing_save_additional_subtask_info($parameters, &$new_subtask) {
	if ($new_subtask instanceof ProjectTask) {
		$new_subtask->setColumnValue('is_calculated_executed_cost', 1);
        $new_subtask->setColumnValue('is_calculated_earned_value', 1);
        $new_subtask->setColumnValue('is_calculated_estimated_cost', 1);
        $new_subtask->setColumnValue('is_calculated_estimated_price', 1);
	}
}

function advanced_billing_before_render_main_custom_properties($params, &$categories) {

	$object = array_var($params, 'object');
	$genid = array_var($params, 'genid');
    $task_data = array_var($params, 'task_data');

	if ($object instanceof ProjectTask || $object instanceof TemplateTask) {
		
        tpl_assign('task', $object);
        tpl_assign('genid', $genid);
        tpl_assign('task_data', $task_data);

		echo tpl_fetch(get_template_path('tasks_billable_selector', 'task', 'advanced_billing'));
	}
}

function advanced_billing_view_task_render_more_properties($task, &$null) {
    $show_billable = config_option('use_is_billable_value_in_tasks');
	if ($show_billable && Plugins::instance()->isActivePlugin('hour_types')) {
        $is_billable = $task->getColumnValue('is_billable') ? lang('yes') : lang('no');
		$html = '</br><b>' . lang('billable') . ':</b>&nbsp;<span class="additional-list-col"><b>'.$is_billable.'</b></span>';

		echo $html;
	}
}

function advanced_billing_get_initial_invoicing_status_for_timeslot($params, &$status){
    if(Plugins::instance()->isActivePlugin('income') && Plugins::instance()->isActivePlugin('hour_types')){
        $timeslot = array_var($params, 'timeslot');
        $task_id = array_var($params, 'task_id');
        $selected_member_ids = array_var($params, 'selected_member_ids', array());
        $task = ProjectTasks::findById($task_id);
        $use_task_billable = config_option('use_is_billable_value_in_tasks');
        if($use_task_billable && $status == '' && $timeslot->isNew() && $task instanceof ProjectTask){
            $status = $task->getColumnValue('is_billable') ? 'pending' : 'non_billable';
        } else if($status == '' && $timeslot->isNew()) {
            $labor_cat_ot_id = ObjectTypes::findByName('hour_type')->getId();
            foreach($selected_member_ids as $member_id){
                $member = Members::findById($member_id);
                if($member instanceof Member && $member->getObjectTypeId() == $labor_cat_ot_id){
                    $mcp = MemberCustomProperties::findOne(array('conditions' => '`object_type_id`='.$labor_cat_ot_id.' AND `code`="is_billable"'));
                    if($mcp instanceof MemberCustomProperty){
                        $mcp_id = $mcp->getId();
                        $mcp_value = MemberCustomPropertyValues::findOne(array('conditions' => '`member_id`='.$member_id.' AND `custom_property_id`='.$mcp_id));
                        if($mcp_value instanceof MemberCustomPropertyValue){
                            $status = $mcp_value->getValue() == '-1' ? 'non_billable' : 'pending';
                        }
                    }
                }
            }
        }

    }
}

function advanced_billing_remove_custom_report_columns_from_list($params, &$allowed_columns) {
    $object_type = $params['object_type'];
	$task_ot = ObjectTypes::findByName('task')->getId();
    $columns_to_remove = array();
    if ($object_type == $task_ot ){
        $columns_to_remove = array(
            'is_calculated_executed_cost',
            'is_calculated_earned_value',
            'is_calculated_estimated_cost',
            'is_calculated_estimated_price'
        );
    }
    if(!empty($columns_to_remove)){
        foreach($allowed_columns as $key => $column){
            if(in_array($column['id'], $columns_to_remove)){
                unset($allowed_columns[$key]);
            }
        }
    }

    $allowed_columns = array_values($allowed_columns);
}

function advanced_billing_draw_additional_task_html($genid, &$task){
	$html = '';
    $show_financials_tab = config_option('show_financial_tab_in_task_form');
    if(!$show_financials_tab) {
		$html .= '<input type="hidden" name="task[is_calculated_estimated_cost]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_estimated_price]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_executed_cost]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_earned_value]" value="1"/>';
	}
	
    echo $html;
}

