<?php

/**
 * AdvancedMember controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class AdvancedMemberController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the AdvancedMemberController
	 *
	 * @access public
	 * @param void
	 * @return AdvancedMemberController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct
	
	
	
	function copy_member($id = null) {
		
		if (is_null($id)) {
			$id = get_id();
		}
		
		$original_member = Members::findById($id);
		if (!$original_member instanceof Member) {
			flash_error(lang('member dnx'));
			ajx_current("empty");
			return;
		}
		
		// member type
		$object_type = ObjectTypes::findById($original_member->getObjectTypeId());
		
		// member columns to copy
		$member_columns = Members::instance()->getColumns();
		$ignored_mem_columns = array("id", "object_id");
		
		try {
			DB::beginWork();	
			
			// new member to save
			$member = new Member();
			
			// set member main data
			foreach ($member_columns as $col) {
				if (!in_array($col, $ignored_mem_columns)) {
					$member->setColumnValue($col, $original_member->getColumnValue($col));
				}
			}
			
			// create a copy of the content data object if the member is a dimension object
			if ($object_type->getType() == 'dimension_object') {
				$new_dim_object = null;
				
				$dim_object = Objects::findObject($original_member->getObjectId());
				if ($dim_object instanceof ContentDataObject) {
					// create the copy, including classification and custom properties
					$new_dim_object = $dim_object->copy();
				}
				
				if ($new_dim_object instanceof ContentDataObject) {
					$member->setObjectId($new_dim_object->getId());
				}
			}
			
			// new default name
			$new_name = lang('copy of', $member->getName());
			$exists = Members::findOne(array("conditions" => array("dimension_id=? AND name=?", $original_member->getDimensionId(), $new_name)));
			$i = 0;
			while ($exists instanceof Member) {
				$i++;
				$new_name = lang('copy of', $member->getName()) . " $i";
				$exists = Members::findOne(array("conditions" => array("dimension_id=? AND name=?", $original_member->getDimensionId(), $new_name)));
			}
			$member->setName($new_name);
			
			// save new member
			$member->save();
			
			// copy permissions
			$sql = "
				INSERT INTO ".TABLE_PREFIX."contact_member_permissions (permission_group_id,member_id,object_type_id,can_write,can_delete)
				SELECT permission_group_id,".$member->getId().",object_type_id,can_write,can_delete
				FROM ".TABLE_PREFIX."contact_member_permissions cmp
				WHERE cmp.member_id=".$original_member->getId()."
			";
			DB::execute($sql);

			// copy dimension member associations
			$sql = "
				INSERT INTO ".TABLE_PREFIX."member_property_members (association_id, member_id, property_member_id, is_active, created_on, created_by_id)
				SELECT association_id, ".$member->getId().", property_member_id, is_active, now(), ".logged_user()->getId()."
				FROM ".TABLE_PREFIX."member_property_members mpm
				WHERE mpm.member_id=".$original_member->getId()."
			";
			DB::execute($sql);

			// copy inverse dimension member associations
			$sql = "
				INSERT INTO ".TABLE_PREFIX."member_property_members (association_id, member_id, property_member_id, is_active, created_on, created_by_id)
				SELECT association_id, member_id, ".$member->getId().", is_active, now(), ".logged_user()->getId()."
				FROM ".TABLE_PREFIX."member_property_members mpm
				WHERE mpm.property_member_id=".$original_member->getId()."
			";
			DB::execute($sql);
			
			DB::commit();
			
			ajx_current("empty");
			ajx_extra_data(array("member_id" => $member->getId()));
			
			evt_add("update dimension tree node", array('dim_id' => $member->getDimensionId(), 'member_id' => $member->getId(), 'select_node' => null));

		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
			ajx_current("empty");
			return;
		}
	}
	

} // AdvancedMemberController

