<?php

/**
 * CObjTemplates controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class CObjTemplateController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the CObjTemplatesController
	 *
	 * @access public
	 * @param void
	 * @return CObjTemplateController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct
	

	function get_templates() {

		ajx_current("emtpy");

		$templates_by_type = array();
		$templates = CObjTemplates::findAll();
		foreach ($templates as $template) {

			$ot = $template->getObjectType();
			$ot_name = $ot->getName();

			$type = $ot->getId();
			if (!isset($templates_by_type[$type])) {
				$templates_by_type[$type] = array();
			}

			$templates_by_type[$type][] = array(
				'id' => $template->getId(),
				'name' => $template->getName(),
				'ot_name' => $ot_name,
			);
		}

		ajx_extra_data(array('templates' => $templates_by_type));
	}

	
	function list_all() {
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$templates = CObjTemplates::findAll();

		tpl_assign('templates', $templates);
	}
	
	/**
	 * Creats a new Generic Content Object Template
	 */
	function add() {
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$template_data = array_var($_POST, 'template');
		
		if(!is_array($template_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new CObjTemplate();
			$ot_id = get_id('ot_id');
			$obj->setObjectTypeId($ot_id);

			tpl_assign('template', $obj);

		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$template = $this->save_template($template_data);
				
				DB::commit();
				
				flash_success(lang('success add template'));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	
	/**
	 * Edits Generic Content Object Template
	 */
	function edit() {
		$this->setTemplate('add');
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$template = CObjTemplates::findById(get_id());
		if(!($template instanceof CObjTemplate)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		$template_data = array_var($_POST, 'template');
		
		if(!is_array($template_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('template', $template);

			// get template objects
			$template_objects = CObjTemplateObjects::getByTemplate($template->getId());
			$template_objects_data = array();
			foreach ($template_objects as $template_object) {

				$template_objects_data[] = $template_object->buildObjectData();

			}

			$template_parameters = CObjTemplateParameters::getTemplateParameters($template->getId());
			$template_parameters_data = array();
			foreach ($template_parameters as $template_parameter) {

				$template_parameters_data[] = $template_parameter->getArrayInfo();

			}

			tpl_assign('template_objects_data', $template_objects_data);
			tpl_assign('template_parameters_data', $template_parameters_data);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$template = $this->save_template($template_data, $template);
				
				DB::commit();
				
				flash_success(lang('success edit template'));
				ajx_current("back");
				//ajx_current("reload");// only for testing
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}



	
	/**
	 * Deletes Generic Content Object Template
	 */
	function delete() {
		ajx_current("empty");
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
	
		$template = CObjTemplates::findById(get_id());
		if(!($template instanceof CObjTemplate)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		try {
			DB::beginWork();
			
			$template->delete();
				
			DB::commit();
				
			flash_success(lang('success delete template'));
			evt_add("reload current panel");
				
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
	}
	
	/**
	 * Main function that saves all the template data
	 * Called from add/edit of this controller
	 * Saves the template object along with all the template objects, variables and parameters
	 */
	private function save_template($template_data, $template = null) {
		// Save Form object
		$is_new = false;
		if (is_null($template) || !$template instanceof CObjTemplate) {
			$template = new CObjTemplate();
			$is_new = true;
		}
		
		$template->setFromAttributes($template_data);
		$template->save();

		$template_id = $template->getId();

		// SAVE OBJECTS

		// clean all template objects, then save the ones sent by the form
		$template->deleteAllObjects();

		// create, update each object of the form
		foreach ($template_data['objects'] as $object_id => $object_data) {

			// decode the data string
			$object_data = json_decode(base64_decode($object_data), true);

			// save the object along with its properties
			$template_object = $this->save_template_object($template, $object_id, $object_data);
			
		}

		// SAVE TEMPLATE PARAMETERS
		$template->deleteAllParameters();
		// create, update each object of the form
		foreach ($template_data['parameters'] as $id => $param_data) {

			$param_data['id'] = $id;

			// save the object along with its properties
			$template_param = $this->save_template_parameter($template, $param_data);
			
		}

		// SAVE TEMPLATE VARIABLES
		$template->deleteAllVariables();
		foreach ($template_data['variables'] as $obj_id => $object_variables) {

			foreach ($object_variables as $var_idx => $variable_data) {

				// save the object along with its properties
				$template_variable = $this->save_template_variable($template, $obj_id, $var_idx, $variable_data);
			}
		}

		
		return $template;
	}

	/**
	 * Creates or updates an object that belongs to the template
	 * @param $template CObjTemplate
	 * @param $object_data array
	 */
	private function save_template_object(CObjTemplate $template, $object_id, $object_data) {

		$object = new CObjTemplateObject();
		$object->setTemplateId($template->getId());
		$object->setObjectId($object_id);
		$object->save();

		$manager = $template->getObjectTypeManager();
		$array_properties = array('lines', 'object_custom_properties', 'members', 'linked_objects', 'subscribers');

		// delete all properties and save them again
		CObjTemplateObjectProperties::deleteAllObjectProperties($template->getId(), $object_id);

		// decode the data sent by the form
		$obj_properties = $object_data['properties'];
		
		// save each property
		foreach ($obj_properties as $property => $property_value) {

			$prop = new CObjTemplateObjectProperty();
			$prop->setTemplateId($template->getId());
			$prop->setObjectId($object_id);
			
			$prop->setProperty($property);
			
			if (in_array($property, $manager->getColumns())) {
				$prop->setPropertyType($manager->getColumnType($property));
			} else if (in_array($property, $array_properties)) {
				$prop->setPropertyType('array');
			} else if (in_array($property, Objects::instance()->getColumns())) {
				$prop->setPropertyType(Objects::instance()->getColumnType($property));
			}

			// for expense lines clean the amount columns so the "," character doesn't break the sql or the js logic
			if ($property == 'lines') {
				foreach ($property_value as &$line) {
					foreach ($line as $line_col => &$line_col_value) {
						if (in_array($line_col, array('unit_cost','total_cost','unit_price','total_price','price_multiplier'))) {
							$line_col_value = clean_formatted_money_amount_for_sql($line_col_value);
						}
					}
				}
			}
			
			if (is_array($property_value)) {
				$prop->setPropertyValue(json_encode($property_value));
			} else {
				$prop->setPropertyValue($property_value);
			}

			$prop->save();
		}

		return $object;
	}

	/**
	 * Creates a template parameter
	 * @param $template CObjTemplate
	 * @param $param_data array
	 */
	private function save_template_parameter(CObjTemplate $template, $param_data) {

		// create the parameter
		$object = new CObjTemplateParameter();
		$object->setTemplateId($template->getId());
		$object->setParameterId(array_var($param_data, 'id'));
		$object->setName(clean(array_var($param_data, 'name')));
		$object->setDescription(clean(array_var($param_data, 'description')));
		$object->setType(clean(array_var($param_data, 'type')));
		$object->setRequired(array_var($param_data, 'required'));
		$object->save();
		
		return $object;
	}

	/**
	 * Creates a template parameter
	 * @param $template CObjTemplate
	 * @param $param_data array
	 */
	private function save_template_variable(CObjTemplate $template, $obj_id, $var_id, $variable_data) {

		$variable_value = clean(array_var($variable_data, 'value'));

		$property = array_var($variable_data, 'property');

		$col_type = strtoupper(array_var($variable_data, 'type'));

		/*if (str_starts_with($property, 'line-')) {
			$col_type = strtoupper(array_var($variable_data, 'type'));
		} else {
			$manager = $template->getObjectTypeManager();
			$col_type = $manager->getColumnType($property);
		}*/

		if ($col_type == DATA_TYPE_DATE || $col_type == DATA_TYPE_DATETIME) {
			
			$param = array_var($variable_data, 'param');
			$operation = array_var($variable_data, 'operation');
			$amount = array_var($variable_data, 'amount');
			$unit = array_var($variable_data, 'unit');

			$variable_value = "{".$param."}" . $operation . $amount . $unit;

		} else if ($col_type == DATA_TYPE_FLOAT || $col_type == DATA_TYPE_INTEGER || $col_type == 'NUMERIC') {
			
			$term_string_array = array();

			foreach ($variable_data['num_formula'] as $term) {

				$param = array_var($term, 'param');
				$operation = array_var($term, 'operation');
				$amount = array_var($term, 'amount');
				$term_operation = array_var($term, 'term_operation');

				$term_string = "{".$param."}" . $operation . $amount;

				$term_string_array[] = $term_string;
				$term_string_array[] = $term_operation;
			}

			$formula_unit =  array_var($term, 'unit');
			if ($formula_unit) {
				$term_string_array[] = "unit=" . $formula_unit;
			}

			$variable_value = implode('|', $term_string_array);
		}


		// create the variable
		$var = new CObjTemplateObjectVariableValue();
		$var->setTemplateId($template->getId());
		$var->setObjectId($obj_id);
		$var->setProperty($property);
		$var->setParameterId(array_var($variable_data, 'parameter_id'));
		$var->setValue($variable_value);
		$var->save();
		
		return $var;
	}

	

	// ===================================================================


	function generate_template_object_data() {
		ajx_current("empty");

		$object_data = CObjTemplateObjects::buildObjectDataFromRequest($_REQUEST);

		evt_add("update_template_object_in_form", array("object_data" => $object_data));

	}



/* INSTANTIATION FUNCTIONS */

	function instantiate($template_id = null, $parameter_values = null, $member_ids = null) {
		//ajx_current("empty");
		if (!can_instantiate_templates(logged_user())) {
			flash_error(lang("no access permissions"));
			ajx_current("empty");
			return;
		}
		
		if (is_null($template_id)) {
			$template_id = get_id();
		}

		if (is_null($parameter_values)) {
			$parameter_values = get_id('parameter_values');
		}
	
		
		$template = CObjTemplates::findById($template_id);
		if (!$template instanceof CObjTemplate) {
			flash_error(lang("template dnx"));
			ajx_current("empty");
			return;
		}

		$parameters = CObjTemplateParameters::getTemplateParameters($template_id);
		if(count($parameters) > 0 && !isset($parameter_values)) {
			// we need to ask user for parameters
			//$this->instantiate_parameters($template_id);
			evt_add('instantiate_cobj_template_parameters', array('template_id' => $template_id));
			ajx_current("empty");
			return;
		}

		// get active context to generate objects there
		$active_context = active_context();
		// if memebers have been sent by parameter then use them instead of active context
		if (is_array($member_ids) && count($member_ids) > 0) {
			$param_members = Members::findAll(array("conditions" => "id IN (".implode(',',$member_ids).")"));
			if (is_array($param_members) && count($param_members) > 0) {
				$active_context = $param_members;
			}
		}

		$template_ot = $template->getObjectType();
		$object_controller = new ObjectController();

		$objects_data = $template->getObjectsToInstantiate();

		try{
			DB::beginWork();

			foreach ($objects_data as $object_data) {
				$content_data_object = CObjTemplateObjects::buildObjectFromData($template_ot, $object_data);

				$variables = CObjTemplateObjectVariableValues::instance()->getTemplateObjectVariables($template->getId(), $object_data['obj_id']);


				// apply parameters to the object if it has variables using them
				$this->apply_parameters_to_object($template, $content_data_object, $variables, $parameter_values);

	
				// save the object and generate its id
				$content_data_object->setNew(true);
				$content_data_object->save();



				// Calculate the members where the object is going to be classified
				$template_object_members = $content_data_object->getMembers();
				$object_members = array();
	
				//change members according to context 
				foreach( $active_context as $selection ) {
					if ($selection instanceof Member) { // member selected
						foreach( $template_object_members as $i => $object_member){
							if ($object_member instanceof Member && $object_member->getObjectTypeId() == $selection->getObjectTypeId()) {
								unset($template_object_members[$i]);
							}						
						}
						
						$object_members[] = $selection->getId();
					}
				}
				foreach( $template_object_members as $object_member ) {
					$object_members[] = $object_member->getId();
				}
				
				// add the members that must be autocassified using dimension associations according to the current set of members
				Env::useHelper('dimension');
				append_related_members_to_autoclassify($object_members);
				// END Members calculation

	

				// process object lines if necessary
				if ($template_ot->getName() == 'expense') {

					$this->save_expense_items($content_data_object, $object_controller, $object_members);
	
					$content_data_object->updateCalculatedTotals();
					$content_data_object->addToHistory(logged_user(), 'add');
				}
	
				// add object custom properties
				$object_cps = $content_data_object->getCachedCustomProperties();
				$object_controller->add_custom_properties($content_data_object, $object_cps);
	
	
				// classify the object
				$object_controller->add_to_members($content_data_object, $object_members, null, false);
			}

			ajx_current("reload");
			flash_success(lang('success instatiate template', $template->getName()));

			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			ajx_current("emtpy");
			flash_error($e->getMessage());
		}

	}



	function instantiate_parameters($template_id = null) {
		if (is_array(array_var($_POST, 'parameter_values'))) {
			ajx_current("back");
			
			$parameter_values = array_var($_POST, 'parameter_values');
			$template_id = get_id();

			$this->instantiate($template_id, $parameter_values);
			
			
		} else {

			$this->setTemplate(get_template_path('instantiate_parameters', 'c_obj_template', 'advanced_core'));

			if (is_null($template_id)) {
				$id = get_id();
			} else {
				$id = $template_id;
			}

			$additional_member_ids = array();
			if($add_mem_id = get_id('member_id')){
				$additional_member_ids[] = $add_mem_id;
				
				// ensure that new member is in context before rendering the template parameters form
				if (!in_array($add_mem_id, active_context_members(false))) {
					$current_context = active_context();
					$add_mem = Members::findById($add_mem_id);
					if ($add_mem instanceof Member) $current_context[] = $add_mem;
					CompanyWebsite::instance()->setContext($current_context);
				}
			}

			$linked_objects = array();
			$parameters = CObjTemplateParameters::getTemplateParameters($id);
			$params = array();
			foreach($parameters as $parameter){
				$params[] = $parameter->getArrayInfo();
			}

			$template = CObjTemplates::findById($id);
			if (!$template instanceof CObjTemplate) {
				flash_error(lang("template dnx"));
				ajx_current("empty");
				return;
			}

			if (array_var($_REQUEST, 'additional_member_ids')) {
				$additional_member_ids = array_merge($additional_member_ids,json_decode(array_var($_REQUEST, 'additional_member_ids')));
			}

			if (array_var($_REQUEST, 'linked_objects')) {
				$linked_objects = json_decode(array_var($_REQUEST, 'linked_objects'));
			}

			tpl_assign('id', $id);
			tpl_assign('additional_member_ids', $additional_member_ids);
			tpl_assign('linked_objects', $linked_objects);
			tpl_assign('parameters', $params);
			tpl_assign('template', $template);
		}
	}


	

	private function save_expense_items($content_data_object, $object_controller, $object_members) {

		$expense_items = $content_data_object->items;
		foreach ($expense_items as $expense_item) {
			if ($expense_item->getCurrencyId() == 0) {
				$expense_item->setCurrencyId($content_data_object->getCurrencyId());
			}
			$pt = ProductTypes::getById($expense_item->getProductTypeId());
			if ($pt instanceof ProductType) {
				$expense_item->setObjectName($pt->getName());
			}
			$expense_item->setExpenseId($content_data_object->getObjectId());
			$expense_item->setNew(true);
			$expense_item->save();

			// classify the expense line
			$object_controller->add_to_members($expense_item, $object_members, null, false);

			$expense_item->addToHistory(logged_user(), 'add');
		}

		return $expense_items;
	}



	private function calculate_template_parameter_string($parameter_values, $value) {

		return calculate_template_task_parameter_string($parameter_values, $value);

	}

	private function calculate_template_parameter_numeric($parameter_values, $template_formula) {

		Env::useHelper('functions', 'advanced_templates');
		$final_value = calculate_number_using_template_formula($template_formula, $parameter_values);

		return $final_value;

	}

	private function calculate_template_parameter_date($parameter_values, $value) {

		$exp_value = explode("|", $value);
		$value = $exp_value[0];
		$time_value = null;
		if (isset($exp_value[1])) $time_value = $exp_value[1];
		
		$opPos = strpos($value, '}') + 1;// the operator is placed after variable name

		if ($opPos !== false) {
			
			$operator = substr($value, $opPos, 1);

			// Is parametric
			$dateParam = substr($value, 1, strpos($value, '}') - 1);
			$dateParam = str_replace("'", "", $dateParam);
			$hour_min = null;

			$tz_offset = logged_user()->getUserTimezoneValue();
			
			// get date from parameter, if parameter is defined by user => use that value, if it is the date of task creation => use DateTimeValueLib::now();
			if ($dateParam == 'object_creation') {
				$date = DateTimeValueLib::now();
				$date->advance($tz_offset);

				if ($time_value) {
					$time_value_exp = explode(':', $time_value);
					$hour_min['hours'] = $time_value_exp[0];
					$hour_min['mins'] = $time_value_exp[1];

				}else{
					$hour_min['hours'] = $date->getHour();
					$hour_min['mins'] = $date->getMinute();
				}
			} else {
				$date_str = $parameter_values[$dateParam];
				
				$date = getDateValue($date_str);
				if (!$date instanceof DateTimeValue) {
					// dont set any date if user didn't specify one in the parameters
					return;
				}
			}
			
			$dateUnit = substr($value, strlen($value) - 1); // i, d, w or m (for days, weeks or months, i for minutes)
			if($dateUnit == 'm') {
				$dateUnit = 'M'; // make month unit uppercase to call DateTimeValue::add with correct parameter
			}
			if($dateUnit == 'i') {
				$dateUnit = 'm'; // DateTimeValue::add function needs minute option as 'm'
			}
			$dateNum = (int)substr($value, $opPos+1, strlen($value) - $opPos - 2);

			$value = $date->add($dateUnit, $dateNum);
			
		}else{
			$value = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), $value);
		}

		return array('value' => $value, 'dateUnit' => $dateUnit);

	}


	private function apply_parameters_to_object($template, $content_data_object, $variables, $parameter_values) {

		$manager = $template->getObjectTypeManager();
		$object_type = $template->getObjectType();

		// for each variable, use the paramaters to calculate the final property value and assign it to the object property
		foreach ($variables as $variable) {

			// column name
			$property = $variable->getProperty();

			// $value contains the variable formula specified in the template
			$value = $variable->getValue();

			// used for line items, example: expense lines
			$is_line_property = false;

			// check if the variable is going to be applied to line item property or a property of the main object and get the column type
			if (str_starts_with($property, 'line-')) {
				$is_line_property = true;
				$exploded = explode('-', $property);
				$line_column = end($exploded);
				if ($object_type->getName() == 'expense') {
					$col_type = ExpenseItems::instance()->getColumnType($line_column);
				}
			} else {
				$col_type = $manager->getColumnType($property);
			}

			// depending on the column type apply the correct function to calculate the final value
			if ($col_type == DATA_TYPE_STRING) {
				// is a string column
				$value = $this->calculate_template_parameter_string($parameter_values, $value);

			} else if ($col_type == DATA_TYPE_INTEGER || $col_type == DATA_TYPE_FLOAT) {
				// is a numeric column
				$value = $this->calculate_template_parameter_numeric($parameter_values, $value);
				
			} else if ($col_type == DATA_TYPE_DATE || $col_type == DATA_TYPE_DATETIME) {
				// is a date column
				$result = $this->calculate_template_parameter_date($parameter_values, $value);
				$value = array_var($result, 'value');
				$dateUnit = array_var($result, 'dateUnit');
			}
			
			// Use the paramater value and set it in the copy
			if($value != '') {
				// check if column exists in main object
				$column_exist = $manager->columnExists($property) || $property == 'name';
				if ($column_exist) {

					if (!$content_data_object->setColumnValue($property, $value)){
						$content_data_object->object->setColumnValue($property, $value);
					}

				} else if ($is_line_property) {
					// if it is a line property then browse the lines to apply the value where it corresponds

					if ($object_type->getName() == 'expense') {
						// for expenses the values are applied according if the product type matches with the template var definition

						// get the item column and the product type where we will apply the value
						$exploded = explode('-', $property);
						$line_column = end($exploded);
						$product_type_id = $exploded[1];

						// iterate the items
						foreach ($content_data_object->items as $item) {
							// if product type is the same then apply the value
							if ($item->getProductTypeId() == $product_type_id) {
								$item->setColumnValue($line_column, $value);
	
								// if the quantity or unit cost was modified we have to update the total cost
								if ($line_column == 'quantity' || $line_column == 'unit_cost') {
									$total_cost = $item->getQuantity() * $item->getUnitCost();
									$item->setTotalCost($total_cost);
								}
								// if the quantity or unit price was modified we have to update the total price
								if ($line_column == 'quantity' || $line_column == 'unit_price') {
									$total_price = $item->getQuantity() * $item->getUnitPrice();
									$item->setTotalPrice($total_price);
								}
							}
						}

					}
				}
			}	
		}

	}

} // CObjTemplatesController

