<?php

/**
 * DimensionManagement controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class DimensionManagementController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the DimensionManagementController
	 *
	 * @access public
	 * @param void
	 * @return DimensionManagementController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Access permissions
		if(!(logged_user()->isAdministrator())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
		}
		
	} // __construct
	
	
	
	function render_member_selector() {
		
		$dim_id = array_var($_REQUEST, 'dim_id');
		
		foreach ($_REQUEST as $k => $v) {
			tpl_assign($k, $v);
		}
		
		if (is_numeric($dim_id)) {
			$dim = Dimensions::findById($dim_id);
			tpl_assign('dim', $dim);
		}
	}
	
	
	
	function change_members_dimension_params() {
	
		$all_dimensions = array();
		$enabled_dimensions = config_option('enabled_dimensions');
		if (count($enabled_dimensions) > 0) {
			$all_dimensions = Dimensions::findAll(array("conditions" => "id IN (".implode(',', $enabled_dimensions).")"));
		}
		
		$dim_member_types = array();
		foreach ($all_dimensions as $dim) {
			$dim_member_types[$dim->getId()] = array();
			$dim_member_type_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($dim->getId());
			foreach ($dim_member_type_ids as $ot_id) {
				$ot = ObjectTypes::findById($ot_id);
				$dim_member_types[$dim->getId()][] = $ot->getArrayInfo();
			}
		}
		
		tpl_assign('all_dimensions', $all_dimensions);
		tpl_assign('dim_member_types', $dim_member_types);
	}
	
	
	
	function change_members_dimension() {
		set_time_limit(0);
		ini_set('memory_limit', '1G');
		
		$source_dim_id = array_var($_REQUEST, 'source_dimension');
		$destiny_dim_id = array_var($_REQUEST, 'destiny_dimension');
		$destiny_member_type_id = array_var($_REQUEST, 'destiny_member_type');
		
		$source_member_ids = json_decode(array_var($_REQUEST, 'source_member_ids', ''), true);
		$destiny_member_ids = json_decode(array_var($_REQUEST, 'destiny_member_id', ''), true);
		$destiny_member_id = array_var($destiny_member_ids, 0);
		
		$operation = array_var($_REQUEST, 'operation');
		
		$options = array(
			'keep_source_classficiation_in_parent' => array_var($_REQUEST, 'keep_source_classficiation_in_parent'),
			'include_source_child_members' => array_var($_REQUEST, 'include_source_child_members'),
		);
		
		try {
			if (!$source_dim_id || !$destiny_dim_id) {
				throw new Exception(lang("you have to select source and destiny dimensions"));
			}
			if (!in_array($operation, array('move','merge'))) {
				throw new Exception(lang("invalid operation"));
			}
			if (!is_array($source_member_ids) || count($source_member_ids) == 0) {
				throw new Exception(lang("you have to select at least one source member"));
			}
			if ($operation == 'merge' && !$destiny_member_id) {
				throw new Exception(lang("you have to select the destiny member"));
			}
			if (!$destiny_member_id) $destiny_member_id = 0;
			
			
			DB::beginWork();
			
			if ($operation == 'move') {
				$this->move_members_to_dimension($source_dim_id, $destiny_dim_id, $destiny_member_type_id, $source_member_ids, $destiny_member_id, $options);
				
			} else if ($operation == 'merge') {
				$this->merge_members_in_dimension($source_dim_id, $destiny_dim_id, $source_member_ids, $destiny_member_id, $options);
			}
			
			
			// set the cron_event date as a past date to force the execution of the rebuild contact member cache event
			DB::execute("UPDATE ".TABLE_PREFIX."cron_events SET `date`=ADDDATE(NOW(), INTERVAL -2 DAY) WHERE name='rebuild_contact_member_cache';");
			
			DB::commit();
			
			flash_success(lang("members successfully edited"));
			ajx_current("back");
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			Logger::log_r("ERROR when changing members dimension:\n".$e->getMessage()."\n".$e->getTraceAsString());
			ajx_current("empty");
		}
		
	}
	
	
	
	
	private function move_members_to_dimension($source_dim_id, $destiny_dim_id, $destiny_member_type_id, $source_member_ids, $destiny_member_id, $options) {
		
		$keep_in_old_parent = array_var($options, 'keep_source_classficiation_in_parent');
		$change_member_custom_properties = Plugins::instance()->isActivePlugin('member_custom_properties');
		$member_cp_cache = array();
		
		foreach ($source_member_ids as $source_member_id) {
			$member = Members::getMemberById($source_member_id);
			
			$old_object_type_id = $member->getObjectTypeId();
			$old_parent_members = $member->getAllParentMembersInHierarchy(false, false);
			$old_hierarchy_ids = array($member->getId());
			foreach ($old_parent_members as $old_parent_member) $old_hierarchy_ids[] = $old_parent_member->getId();
			
			$object_id_rows = DB::executeAll("SELECT object_id FROM ".TABLE_PREFIX."object_members WHERE member_id = '$source_member_id' AND is_optimization=0");
			$object_ids = array_filter(array_unique(array_flat($object_id_rows)));
			
			$old_ot = ObjectTypes::instance()->findById($old_object_type_id);
			$new_ot = ObjectTypes::instance()->findById($destiny_member_type_id);
			
			// remove old classification from objects
			$this->remove_old_classification_from_objects($object_ids, $member, $keep_in_old_parent, $old_hierarchy_ids);
			
			
			// calculate the new depth
			$new_depth = 1;
			if ($destiny_member_id > 0) {
				$parent_row = DB::execute("SELECT * FROM ".TABLE_PREFIX."members WHERE id='$destiny_member_id'");
				$new_depth = array_var($parent_row, 'depht', 0) + 1;
			}
			
			// update associated object, create it if necesary.
			$member_object_id = 0;
			if ($new_ot instanceof ObjectType && $new_ot->getId() != $old_ot->getId() && $new_ot->getType() == 'dimension_object' && $new_ot->getTableName()) {
				
				if ($member->getObjectId() > 0) {
					// move associated object to the table of the destiny object type
					$member_object_id = $member->getObjectId();
					DB::execute("DELETE FROM ".TABLE_PREFIX.$old_ot->getTableName()." WHERE object_id='".$member->getObjectId()."';");
					
					DB::execute("UPDATE ".TABLE_PREFIX."objects SET object_type_id = '$destiny_member_type_id' WHERE id='".$member->getObjectId()."';");
					DB::execute("INSERT INTO ".TABLE_PREFIX.$new_ot->getTableName()." (object_id) VALUES (".$member->getObjectId().") ON DUPLICATE KEY UPDATE object_id=object_id;");
				
				} else {
					// create associated object
					DB::execute("INSERT INTO ".TABLE_PREFIX."objects (object_type_id,name,timezone_id,timezone_value, created_by_id, updated_by_id, created_on, updated_on) VALUES
						($destiny_member_type_id, '".$member->getName()."', ".logged_user()->getUserTimezoneId().", ".logged_user()->getUserTimezoneValue().", ".logged_user()->getId().", ".logged_user()->getId().", NOW(), NOW())
						ON DUPLICATE KEY UPDATE id=id;");
					
					$r = DB::executeOne("SELECT MAX(id) as last_id FROM ".TABLE_PREFIX."objects WHERE object_type_id=$destiny_member_type_id");
					$member_object_id = array_var($r, 'last_id');
					
					DB::execute("INSERT INTO ".TABLE_PREFIX.$new_ot->getTableName()." (object_id) VALUES ('".$member_object_id."') ON DUPLICATE KEY UPDATE object_id=object_id;");
				}
			}
			
			// update members table
			DB::execute("UPDATE ".TABLE_PREFIX."members
					SET dimension_id = '$destiny_dim_id',
					object_type_id = '$destiny_member_type_id',
					parent_member_id = '$destiny_member_id',
					depth = '$new_depth',
					object_id = '$member_object_id'
					WHERE id = ".$member->getId()."
			");
			
			
			// change member custom properties
			if ($new_ot->getId() != $old_ot->getId() && $change_member_custom_properties) {
				$this->change_member_custom_properties($member, $old_object_type_id, $destiny_member_type_id, $member_cp_cache);
			}
			
			// add new classification from objects
			$new_mem = new Member();
			$new_mem->setId($member->getId());
			$new_mem->setDimensionId($destiny_dim_id);
			$new_mem->setObjectTypeId($destiny_member_type_id);
			$new_mem->setDepth($new_depth);
			$new_mem->setParentMemberId($destiny_member_id);
			
			$new_parent_members = $new_mem->getAllParentMembersInHierarchy(false, false);
			$new_parent_member_ids = array();
			foreach ($new_parent_members as $new_parent_member) $new_parent_member_ids[] = $new_parent_member->getId();
			
			$this->add_new_classification_to_objects($object_ids, $member, $new_parent_member_ids);
			

			// mark objects in order to be procesed by the rebuild_sharing_table cron event
			$this->add_object_ids_to_sharing_table_flags($object_ids);
		}		
	}
	
	
	
	
	private function merge_members_in_dimension($source_dim_id, $destiny_dim_id, $source_member_ids, $destiny_member_id, $options) {

		$keep_in_old_parent = array_var($options, 'keep_source_classficiation_in_parent');
		$change_member_custom_properties = Plugins::instance()->isActivePlugin('member_custom_properties');
		$member_cp_cache = array();
		
		$destiny_member = Members::getMemberById($destiny_member_id);
		$destiny_member_type_id = $destiny_member->getObjectTypeId();
		
		$new_parent_members = $destiny_member->getAllParentMembersInHierarchy(false, false);
		$new_parent_member_ids = array();
		foreach ($new_parent_members as $new_parent_member) $new_parent_member_ids[] = $new_parent_member->getId();
		
		foreach ($source_member_ids as $source_member_id) {
			$member = Members::getMemberById($source_member_id);
				
			$old_object_type_id = $member->getObjectTypeId();
			$old_parent_members = $member->getAllParentMembersInHierarchy(false, false);
			$old_hierarchy_ids = array($member->getId());
			foreach ($old_parent_members as $old_parent_member) $old_hierarchy_ids[] = $old_parent_member->getId();
				
			$object_id_rows = DB::executeAll("SELECT object_id FROM ".TABLE_PREFIX."object_members WHERE member_id = '$source_member_id' AND is_optimization=0");
			$object_ids = array_filter(array_unique(array_flat($object_id_rows)));
				
			$old_ot = ObjectTypes::instance()->findById($old_object_type_id);
			$new_ot = ObjectTypes::instance()->findById($destiny_member_type_id);
				
			// remove old classification from objects
			$this->remove_old_classification_from_objects($object_ids, $member, $keep_in_old_parent, $old_hierarchy_ids);
			
			// delete old member
			if ($member->getObjectId() > 0) {
				$member_obj = Objects::findObject($member->getObjectId());
				if ($member_obj instanceof ContentDataObject) $member_obj->delete();
			}
			$member->delete();
			
			// add new classification from objects
			$this->add_new_classification_to_objects($object_ids, $destiny_member, $new_parent_member_ids);
			
			// mark objects in order to be procesed by the rebuild_sharing_table cron event
			$this->add_object_ids_to_sharing_table_flags($object_ids);
		}
	}
	
	
	
	private function add_new_classification_to_objects($object_ids, $new_mem, $new_parent_member_ids) {
		$values_array = array();
			
		foreach ($object_ids as $object_id) {
			$values_array[] = "($object_id,".$new_mem->getId().",0)";
			
			foreach ($new_parent_member_ids as $new_p_id) {
				$values_array[] = "(".$object_id.",".$new_p_id.",1)";
			}
		}
		
		if (count($values_array) > 0) {
			$values = implode(',', $values_array);
			DB::execute("INSERT INTO ".TABLE_PREFIX."object_members (object_id,member_id,is_optimization) VALUES $values ON DUPLICATE KEY UPDATE object_id=object_id");
		}
	}
	
	
	private function remove_old_classification_from_objects($object_ids, $member, $keep_in_old_parent, $old_hierarchy_ids) {
		foreach ($object_ids as $object_id) {
			if ($member->getParentMemberId() > 0 && $keep_in_old_parent) {
				// remove classification in this member
				DB::execute("DELETE FROM ".TABLE_PREFIX."object_members WHERE member_id=".$member->getId()." AND object_id='$object_id'");
				// set parent classification as is_optimization=0
				DB::execute("UPDATE ".TABLE_PREFIX."object_members SET is_optimization=0 WHERE member_id='".$member->getParentMemberId()."' AND object_id='$object_id'");
				
			} else {
				// delete classification from this member and its parent
				DB::execute("DELETE FROM ".TABLE_PREFIX."object_members WHERE member_id IN (".implode(',', $old_hierarchy_ids).") AND object_id='$object_id'");
			}
		}
	}
	
	
	
	private function change_member_custom_properties($member, $old_object_type_id, $destiny_member_type_id, &$member_cp_cache) {
		// ids cache for destiny cps
		if (!array_var($member_cp_cache, $destiny_member_type_id)) {
			$cp_rows = DB::executeAll("SELECT id, code FROM ".TABLE_PREFIX."member_custom_properties
					WHERE object_type_id='$destiny_member_type_id' AND code IN ('description_special','color_special')");
			$member_cp_cache[$destiny_member_type_id] = array();
			foreach ($cp_rows as $cp_row) {
				$member_cp_cache[$destiny_member_type_id][$cp_row['code']] = $cp_row['id'];
			}
		}
		
		// ids cache for old member cps
		$mem_cps_info = array_var($member_cp_cache, $old_object_type_id);
		if (!$mem_cps_info) {
			$cp_rows = DB::executeAll("SELECT id, code FROM ".TABLE_PREFIX."member_custom_properties
					WHERE object_type_id='$old_object_type_id' AND code IN ('description_special','color_special')");
			$member_cp_cache[$old_object_type_id] = array();
			foreach ($cp_rows as $cp_row) {
				$member_cp_cache[$old_object_type_id][$cp_row['code']] = $cp_row['id'];
			}
			$mem_cps_info = array_var($member_cp_cache, $old_object_type_id);
		}
		
		// change description cp id
		$old_cp_id = $mem_cps_info['description_special'];
		$new_cp_id = $member_cp_cache[$destiny_member_type_id]['description_special'];
		DB::execute("UPDATE ".TABLE_PREFIX."member_custom_property_values SET custom_property_id='$new_cp_id'
				WHERE custom_property_id='$old_cp_id' AND member_id='".$member->getId()."'");
		
		// change color cp id
		$old_cp_id = $mem_cps_info['color_special'];
		$new_cp_id = $member_cp_cache[$destiny_member_type_id]['color_special'];
		DB::execute("UPDATE ".TABLE_PREFIX."member_custom_property_values SET custom_property_id='$new_cp_id'
				WHERE custom_property_id='$old_cp_id' AND member_id='".$member->getId()."'");
		
	}
	
	
	
	
	private function add_object_ids_to_sharing_table_flags($object_ids) {

		if (is_array($object_ids) && count($object_ids) > 0) {
			
			$already_added_rows = DB::executeAll("SELECT DISTINCT(object_id) FROM ".TABLE_PREFIX."sharing_table_flags WHERE object_id>0");
			$already_added_ids = array_filter(array_unique(array_flat($already_added_rows)));
			
			if (count($already_added_ids) > 0) {
				$insert_object_ids = array_diff($object_ids, $already_added_ids);
			} else {
				$insert_object_ids = $object_ids;
			}
			
			$values_array = array();
			foreach ($insert_object_ids as $oid) {
				$values_array[] = "($oid)";
			}
			
			if (count($values_array) > 0) {
				$values = implode(',', $values_array);
			
				$sql = "INSERT INTO ".TABLE_PREFIX."sharing_table_flags (object_id) VALUES $values ON DUPLICATE KEY UPDATE object_id=object_id;";
				DB::execute($sql);
			}
		}
		
	}
	

} // DimensionManagementController
