<?php

/**
 * DimensionOptions controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class DimensionOptionsController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the DimensionOptionsController
	 *
	 * @access public
	 * @param void
	 * @return DimensionOptionsController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Access permissions
		if(!(logged_user()->isExecutiveGroup())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
		}
		
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------

	
	function dimension_options() {
		if(!can_manage_dimensions(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		DimensionAssociationsConfigs::ensureAllAssociationsHaveConfigOptions();
	
		$enabled_dimensions = config_option('enabled_dimensions');
	
		// custom dimension name
		$dimensions = Dimensions::findAll(array('conditions' => 'id IN ('.implode(',', $enabled_dimensions).')'));
	
		$dimensions_by_id = array();
		$custom_dimension_names = array();
		foreach ($dimensions as $dim) {
			$cdim_name = $dim->getOptionValue('custom_dimension_name');
			if (is_null($cdim_name)) {
				$cdim_name = "";
			}
			$custom_dimension_names[$dim->getId()] = $cdim_name;
			$dimensions_by_id[$dim->getId()] = $dim;
		}
	
		// enabled dimension object types
		$dots = DimensionObjectTypes::findAll(array('conditions' => 'dimension_id IN ('.implode(',', $enabled_dimensions).')'));
		
		// add condition to filter object types only of enabled plugins
		$enabled_object_types = ObjectTypes::getAllObjectTypes();
		$enabled_object_type_ids = array();
		foreach ($enabled_object_types as $enabled_ot) {
			$enabled_object_type_ids[] = $enabled_ot->getId();
		}
		$enabled_object_type_ids_csv = '0';
		if (count($enabled_object_type_ids)) {
			$enabled_object_type_ids_csv = implode(',',$enabled_object_type_ids);
		}
		
		// associations between dimensions
		$associations = DimensionMemberAssociations::findAll(array(
				'order' => 'dimension_id, object_type_id',
				'conditions' => "associated_dimension_id NOT IN (SELECT id FROM ".TABLE_PREFIX."dimensions WHERE code='feng_persons')
					AND associated_object_type_id IN ($enabled_object_type_ids_csv)"
		));
		
		// para cada object type si no hay 'hide_member_selector_in_forms' agregar un reg en false
		
		
		// dimension selectors options (show/hide)
		$enabled_content_object_types_cond = "";
		$enabled_content_object_types_ids = ObjectTypes::getListableObjectTypeIds();
		$enabled_content_object_types_ids[] = Timeslots::instance()->getObjectTypeId();
		if (count($enabled_content_object_types_ids) > 0) {
			$enabled_content_object_types_cond = "AND content_object_type_id IN (".implode(',', $enabled_content_object_types_ids).")";
		}
		$hide_sel_options = DimensionContentObjectOptions::findAll(array("conditions" => "`option`='hide_member_selector_in_forms' $enabled_content_object_types_cond"));
	
		tpl_assign('custom_dimension_names', $custom_dimension_names);
		tpl_assign('dimension_ots', $dots);
		tpl_assign('associations', $associations);
		tpl_assign('hide_sel_options', $hide_sel_options);
		tpl_assign('dimensions_by_id', $dimensions_by_id);
	}
	
	
	
	
	function dimension_options_submit() {
		ajx_current("empty");
		if(!can_manage_dimensions(logged_user())) {
			flash_error(lang('no access permissions'));
			return;
		}
	
		$folder_ots = ObjectTypes::findAll(array('id' => true, 'conditions' => "name IN ('folder','project_folder','customer_folder')"));
	
		$data = array_var($_POST, 'enabled_dots');
		$names_data = array_var($_POST, 'custom_names');
		$names_ot_data = array_var($_POST, 'custom_ot_names');
		$names_ot_plural_data = array_var($_POST, 'custom_ot_names_plural');
		$assoc_data = array_var($_POST, 'assoc_config_option');
		if (is_array($data) || is_array($names_data) || is_array($assoc_data)) {
			try {
				DB::beginWork();
	
				// enabled dimension object types
				foreach ($data as $dim_id => $ots_info) {
					foreach ($ots_info as $ot_id => $enabled) {
						DB::execute("UPDATE ".TABLE_PREFIX."dimension_object_types SET enabled=".DB::escape($enabled)."
							WHERE dimension_id=".DB::escape($dim_id)." AND object_type_id=".DB::escape($ot_id));
	
						// if object type is folder, set the same configuration to all folder types
						if (in_array($ot_id, $folder_ots)) {
							DB::execute("UPDATE ".TABLE_PREFIX."dimension_object_types SET enabled=".DB::escape($enabled)."
								WHERE dimension_id=".DB::escape($dim_id)." AND object_type_id IN (".implode(',', $folder_ots).")");
						}
					}
				}
	
				// custom dimension names
				foreach ($names_data as $dim_id => $custom_name) {
					$cname = trim($custom_name);
					$dimension = Dimensions::getDimensionById($dim_id);
					if ($dimension instanceof Dimension) {
						$dimension->setOptionValue('custom_dimension_name', $cname);
					}
				}
				
				//custom members names
				foreach ($names_ot_data as $dim_id => $ot_ids) {
				    foreach ($ot_ids as $ot_id => $custom_name) {
				        $cname = trim($custom_name);
				        
				        if (in_array($ot_id, $folder_ots)) {
				            foreach ($folder_ots as $f_id){
				                DimensionObjectTypeOptions::setOptionValue($dim_id, $f_id, 'custom_object_type_name', $cname);
				            }				            
				        }else{
				            DimensionObjectTypeOptions::setOptionValue($dim_id, $ot_id, 'custom_object_type_name', $cname);
				        }
				    }
				
				}
				
				//custom members names plural
				foreach ($names_ot_plural_data as $dim_id => $ot_ids) {
				    foreach ($ot_ids as $ot_id => $custom_name) {
				        $cname = trim($custom_name);
				        
				        if (in_array($ot_id, $folder_ots)) {
				            foreach ($folder_ots as $f_id){
				                DimensionObjectTypeOptions::setOptionValue($dim_id, $f_id, 'custom_object_type_name_plural', $cname);
				            }				            
				        }else{
				            DimensionObjectTypeOptions::setOptionValue($dim_id, $ot_id, 'custom_object_type_name_plural', $cname);
				        }
				    }
				
				}
	
				// dimension association options
				foreach ($assoc_data as $assoc_id => $options) {
					foreach ($options as $key => $value) {
						DB::execute("UPDATE ".TABLE_PREFIX."dimension_associations_config SET value=".DB::escape($value)."
							WHERE association_id=".DB::escape($assoc_id)." AND config_name=".DB::escape($key));
					}
				}
				
				// dimension selectors (show/hide)
				$this->save_hidden_dimension_selectors(array_var($_POST, 'show_dim_selector'));
				
				// dimension selectors (show/hide)
				$this->save_default_dimension_values(array_var($_POST, 'default_dimension_value'));
	
				DB::commit();
				flash_success(lang("success edit dimension options"));
				evt_add('tabs changed');
				//ajx_current("back");
	
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
		}
	}
	

	private function save_default_dimension_values($data) {
		
		foreach ($data as $dimension_id => $member_ids_json) {
			$member_ids = json_decode($member_ids_json, true);
			
			$def_mem_id = count($member_ids) > 0 ? $member_ids[0] : 0;
			
			DimensionOptions::setOptionValue($dimension_id, 'default_value', $def_mem_id);
		}
	}
	
	
	private function save_hidden_dimension_selectors($data) {
		$option_name = 'hide_member_selector_in_forms';
		$enabled_dimensions = config_option('enabled_dimensions');
		$enabled_dimensions_cond = count($enabled_dimensions)>0 ? " AND dimension_id IN (".implode(',', $enabled_dimensions).")" : " AND false";
		
		$ot_ids = array();
		
		// build sql for new values
		foreach ($data as $dim_id => $rows) {
			foreach ($rows as $ot_id => $checked) {
				$val = ($checked=='true' ? "0" : "1");
				$values[] = "('$dim_id', '$ot_id', '$option_name', '$val')";
				$ot_ids[] = $ot_id;
			}
		}
		
		if (count($ot_ids) > 0) {
			$ot_cond = "AND content_object_type_id IN (".implode(',', $ot_ids).")";
			
			// clear old values
			DimensionContentObjectOptions::delete("`option`='$option_name' $ot_cond $enabled_dimensions_cond");
		}
		
		// execute the insert query
		if (count($values) > 0) {
			$values_sql = implode(',', $values);
			$sql = "INSERT INTO ".TABLE_PREFIX."dimension_content_object_options VALUES ".$values_sql." ON DUPLICATE KEY UPDATE `value`=`value`;";
			DB::execute($sql);
		}
	}

} // DimensionOptionsController

?>