<?php

/**
 * FileActions controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class FileActionsController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the FileActionsController
	 *
	 * @access public
	 * @param void
	 * @return FileActionsController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct
	
	function send_file_by_notification_email() {

		$file = ProjectFiles::findById(get_id());
		if(!($file instanceof ProjectFile)) {
			flash_error(lang('file dnx'));
			ajx_current("empty");
			return;
		}

		tpl_assign("file", $file);
		
	}

	function queue_notification_email() {

		ajx_current("empty");

		// Vars
		$attachments = array();
		$file_id = array_var($_POST['mail'], 'file_id');
		$file = ProjectFiles::findById($file_id);
		
		if(!($file instanceof ProjectFile)) {
			flash_error(lang('file dnx'));
			ajx_current("empty");
			return;
		}

		// File name
		$fName = utf8_encode_mime_header_value($file->getName());
		$fName = str_replace(':', ' ', $fName);

		if (FileRepository::getBackend() instanceof FileRepository_Backend_FileSystem) {
			$file_path = FileRepository::getBackend()->getFilePath($file->getLastRevision()->getRepositoryId());
		} else {
			$file_path = ROOT . "/tmp/" . $file->getFilename();
			$handle = fopen($file_path, 'wb');
			fwrite($handle, $file->getLastRevision()->getFileContent(), $file->getLastRevision()->getFilesize());
			fclose($handle);
		}

		$att_disposition = 'attachment';

		if (config_option('show images in document notifications') && in_array($file->getTypeString(), ProjectFiles::$image_types)) {
			$att_disposition = 'inline';
		}

		$attachments[] = array(
			//'cid' => gen_id() . "@".$fName,
			'path' => $file_path,
			'type' => $file->getTypeString(),
			'disposition' => $att_disposition,
			'name' => $fName,
		);

		// Email vars
		$to = array(array_var($_POST['mail'], 'to'));
		$array_to = explode(",", $to[0]);
		$title =lang("sent you the file", logged_user()->getUsername(), $fName);

		// Assign vars to tpl
		tpl_assign("title", $title);
		tpl_assign("object", $file);
		tpl_assign("description", array_var($_POST['mail'], 'description'));

		for ($i=0; $i < count($array_to) ; $i++) {
			$to = array($array_to[$i]);
			Notifier::queueEmail(null,
							$to,
							null,
							null,
							logged_user()->getEmailAddress(),
							$title,
							tpl_fetch(get_template_path('attach_by_email', 'file_actions', 'advanced_core')),
							'text/html',
							'8bit',
							$attachments);
		}


		if(filesize($file_path) > 1048576) // 1048576 bits = 10 mb
			flash_error(lang('exeeded file size'));
		else
			flash_success(lang('success massmail'));

	}
	
	function send_file_by_email() {
		
		if (!Plugins::instance()->isActivePlugin('mail')) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		$file = ProjectFiles::findById(get_id());
		if(!($file instanceof ProjectFile)) {
			flash_error(lang('file dnx'));
			ajx_current("empty");
			return;
		}
		
		$fName = utf8_encode_mime_header_value($file->getName());
		$fName = str_replace(':', ' ', $fName);
		
		$type_string = $file->getTypeString();
		
		if (strpos($fName, '.') === FALSE) {
			$split_type = explode('/', $type_string);
			$ext = $split_type[count($split_type)-1];
			if ($ext) $fName .= '.'.$ext;
		}
		
		$attachs = array(
			array(
				'manager' => 'ProjectFiles',
				'object_id' => $file->getId(),
				'name' => $fName,
				'type' => $type_string,
				'ico' => "ico-file ico-" . str_replace("/", "-", $type_string),
			),
		);
		$_SESSION['existing_attachments'] = $attachs;
		
		$this->redirectTo('mail', 'add_mail');
		
	}
	

} // FileActionsController

