<?php

/**
 * ObjectPrint controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ObjectPrintController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the ObjectPrintController
	 *
	 * @access public
	 * @param void
	 * @return ObjectPrintController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');

		if (!Plugins::instance()->isActivePlugin('object_subtypes')) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
		}
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	
	function generate_document() {
		ajx_current("empty");
		
		$object_ids = array_var($_REQUEST, 'ids');
		
		if (!is_array($object_ids)) {
			$object_ids_tmp = explode(',', $object_ids);
			$object_ids = array();
			foreach ($object_ids_tmp as $id) {
				if (is_numeric($id) && $id > 0) $object_ids[] = $id;
			}
		}
		
		if (count($object_ids) == 0) {
			flash_error(lang('no objects selected'));
			return;
		}
		
		$rows = DB::executeAll("SELECT DISTINCT(object_subtype_id) as stype, object_type_id FROM ".TABLE_PREFIX."objects WHERE id IN (".implode(',', $object_ids).")");
		$use_generic_header_footer = count($rows) > 1;
		if (!$use_generic_header_footer) {
			$header_footer_type = $rows[0]['stype'];
		} else {
			$header_footer_type = 0;
		}
		$general_object_type = $rows[0]['object_type_id'];
		
		// header and footer options
		$header_option = ObjectPrintOptions::getObjectPrintHeaderOption($general_object_type, $header_footer_type);
		$footer_option = ObjectPrintOptions::getObjectPrintFooterOption($general_object_type, $header_footer_type);
		
		// generate html
		$html = '<html><head><style>' . file_get_contents(dirname(__FILE__)."/../../public/assets/css/generated_doc.css") . '</style></head><body>';
		
		$html .= $this->generate_header($header_option, $general_object_type);
		
		foreach ($object_ids as $oid) {
			
			$object = Objects::findObject($oid);
			if (!$object) continue;
			
			$property_options = ObjectPrintOptions::getObjectPrintPropertyOptions($object->getObjectTypeId(), $object->getColumnValue('object_subtype_id'));
			
			$html .= $this->generate_object_html($object, $property_options);
			
		}
		
		$html .= $this->generate_footer($footer_option, $general_object_type);
		
		$html .= '</body></html>';
		
		// upload document
		$name = array_var($_REQUEST, 'file_name', 'Autogenerated document.html');
		if (!str_ends_with($name, '.html')) {
			$name .= '.html';
		}
		$members = array();
		foreach (active_context() as $selection) {
			if ($selection instanceof Member) $members[] = $selection;
		}
		$this->upload_generated_document($name, $members, $html);
		
		flash_success(lang('document generated successfully', $name));
		evt_add('reload tab panel', 'documents-panel');
	}
	
	
	private function generate_header($header_option, $general_object_type_id) {
		if ($header_option instanceof ObjectPrintOption) {
			return '<div class="document-header">' . $header_option->getHtmlContent() . '</div>';
		} else {
			$ot = ObjectTypes::findById($general_object_type_id);
			return '<div class="document-header">' . $ot->getName() . '</div>';
		}
	}
	
	private function generate_footer($footer_option, $general_object_type_id) {
		if ($footer_option instanceof ObjectPrintOption) {
			return '<div class="document-footer">' . $footer_option->getHtmlContent() . '</div>';
		} else {
			$ot = ObjectTypes::findById($general_object_type_id);
			return '<div class="document-footer">' . $ot->getName() . '</div>';
		}
	}
	
	private function generate_object_html($object, $property_options) {
		
		$html .= '<div class="document-object" id="object-'.$object->getId().'">';
		$html .= '<div class="object-name">'. $object->getObjectName() .'</div>';
		
		$html .= $this->generate_object_section_properties($object, $property_options, array('section' => 'top', 'style' => ''));
		$html .= $this->generate_object_section_properties($object, $property_options, array('section' => 'left', 'style' => 'float:left; width:50%;'));
		$html .= $this->generate_object_section_properties($object, $property_options, array('section' => 'right', 'style' => 'float:left; width:50%;'));
		$html .= '<div style="clear:both;"></div>';
		$html .= $this->generate_object_section_properties($object, $property_options, array('section' => 'bottom', 'style' => ''));
		
		$html .= '</div>';
		
		return $html;
	}
	
	private function generate_object_section_properties($object, $property_options, $section_options) {
		Env::useHelper('format');
		$html = '<div class="document-object-section '.$section_options['section'].'" id="object-section-'.$section_options['section'].'" style="'.$section_options['style'].'">';
		
		foreach ($property_options as $property) {
			if ($section_options['section'] == $property->getPosition()) {
				$label = "";
				$value = "";
				if (is_numeric($property->getProperty())) {
					
					$cp = CustomProperties::findById($property->getProperty());
					if (!$cp instanceof CustomProperty) continue;
					$label = $cp->getName();
					$value = get_custom_property_value_for_listing($cp, $object);
					
				} else {
					
					$ot = ObjectTypes::findByName($object->getObjectTypeName());
					$label = lang('field '. $ot->getHandlerClass(). ' ' . $property->getProperty());
					$value = format_value_to_print($property->getProperty(), $object->getColumnValue($property->getProperty()), $object->getColumnType($property->getProperty()), $object->getObjectTypeId());
				}
				
				if ($label != '') {
					$html .= '<div class="property"><span class="name">'. $label .': </span><span class="value">'. $value .'</span></div>';
				}
			}
		}
		
		$html .= '</div>';
		
		return $html;
	}
	
	
	private function upload_generated_document($name, $members, $content) {
		
		// create file
		$file = new ProjectFile();
		$file->setFilename($name);
		$file->setIsVisible(true);
		$file->setCreatedById(logged_user()->getId());
		$file->save();
		
		// classify
		$file->addToMembers($members);
		
		// fill sharing table in background
		add_object_to_sharing_table($file, logged_user());
		
		// save content in tmp file
		$userid = logged_user() ? logged_user()->getId() : "0";
		$tempFileName = ROOT ."/tmp/". $userid ."x". gen_id();
		$fh = fopen($tempFileName, 'w');
		fwrite($fh, $content);
		fclose($fh);
		
		// object to generate revision
		$fileToSave = array(
			"name" => $name,
			"type" => 'text/html',
			"tmp_name" => $tempFileName,
			"error" => 0,
			"size" => filesize($tempFileName)
		);
		
		// generate file revision
		$revision = $file->handleUploadedFile($fileToSave, true, '');
		$revision->setCreatedById(logged_user()->getId());
		$revision->save();
		ApplicationLogs::createLog($file, ApplicationLogs::ACTION_ADD);
	}

} // ObjectPrintController

?>