<?php

/**
 * PropertyGroups controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class PropertyGroupsController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the PropertyGroupsController
	 *
	 * @access public
	 * @param void
	 * @return PropertyGroupsController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct
	
	
	
	function add() {
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$property_group_data = array_var($_POST, 'property_group');
		
		if(!is_array($property_group_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new PropertyGroup();
			$ot_id = get_id('ot_id');
			$obj->setObjectTypeId($ot_id);

			$row = DB::executeOne("SELECT (max(`order`)+1) as pg_order FROM ".TABLE_PREFIX."property_groups WHERE object_type_id='$ot_id'"); 
			$obj->setOrder(array_var($row, 'pg_order'));
			
			tpl_assign('property_group', $obj);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$property_group = $this->save_property_group($property_group_data);
				
				DB::commit();
				
				flash_success(lang('success add property group'));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$property_group = PropertyGroups::findById(get_id());
		if(!($property_group instanceof PropertyGroup)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		$property_group_data = array_var($_POST, 'property_group');
		
		if(!is_array($property_group_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('property_group', $property_group);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$property_group = $this->save_property_group($property_group_data, $property_group);
				
				DB::commit();
				
				flash_success(lang('success edit property group'));
				ajx_current("reload");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_property_group($property_group_data, $property_group = null) {
		// Save Form object
		$is_new = false;
		if (is_null($property_group) || !$property_group instanceof PropertyGroup) {
			$property_group = new PropertyGroup();
			$is_new = true;
		}
		
		$property_group->setFromAttributes($property_group_data);
		
		$property_group->save();
		
		Hook::fire('after_property_group_save', array('property_group'=>$property_group, 'data'=>$property_group_data), $property_group);
		
		return $property_group;
	}
	
	function list_all() {
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$all_properties = PropertyGroups::getAllGroupedByObjectType();
		
		// get all object types, exclude object types of disabled plugins
		$all_object_types = ObjectTypes::findAll(array(
			"conditions" => "type IN ('content_object','dimension_object','dimension_group')
				AND IF(plugin_id IS NULL OR plugin_id=0, true, (SELECT p.is_activated FROM ".TABLE_PREFIX."plugins p WHERE p.id=plugin_id) = true)
				AND name NOT IN ('project_folder', 'customer_folder', 'person', 'company','template_task','template_milestone','file revision')",
			"order" => "name"
		));

		tpl_assign('all_object_types', $all_object_types);
		tpl_assign('all_properties', $all_properties);
	}
	
	
	
	

	function delete() {
		ajx_current("empty");
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
	
		$property_group = PropertyGroups::findById(get_id());
		if(!($property_group instanceof PropertyGroup)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		try {
			DB::beginWork();
			
			Hook::fire('before_property_group_delete', $property_group, $property_group);
			
			PropertyGroupProperties::deleteByGroup($property_group->getId());
			$property_group->delete();
				
			DB::commit();
				
			flash_success(lang('success delete property group'));
			evt_add("reload current panel");
				
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
	}

} // PropertyGroupsController

