<?php

/**
 * TimeWeekly controller
 *
 * @version 1.0
 * @author Alex Zonis <alex.zonis@fengoffice.com>
 */
class TimeWeeklyController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the TimeWeeklyController
	 *
	 * @access public
	 * @param void
	 * @return TimeWeeklyController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct

    function get_data_for_time_weekly_view(){
        Env::useHelper('time_weekly_functions', 'advanced_core');
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $from_date = array_var($_REQUEST, 'from_date');
        $until_date = array_var($_REQUEST, 'until_date');
        $user_id = array_var($_REQUEST, 'user_id');
        $genid = array_var($_REQUEST, 'genid');

        $this->update_weekly_time_module_filters($user_id, $from_date, $until_date);

        $filters = array('weekly_user_id' => $user_id, 'from_date' => $from_date, 'until_date' => $until_date);
        $weekly_data = get_weekly_time_data($filters);
        $weekly_html = build_html_for_weekly_view($weekly_data, $genid);
        ajx_current("empty");
        ajx_extra_data(array('weekly_html' => $weekly_html, 'weekly_data' => $weekly_data));
    }

    function format_date_range_for_weekly_view(){

        $from_date = array_var($_REQUEST, 'from_date');
        $until_date = array_var($_REQUEST, 'until_date');
        $from_dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $from_date);
        $until_dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $until_date);

        $user_date_format = user_config_option('date_format');
        $from_formatted = $from_dtv->format($user_date_format);
        $until_formatted = $until_dtv->format($user_date_format);
        $range = $from_formatted . ' - ' . $until_formatted;
        
        ajx_current("empty");
        ajx_extra_data(array('range' => $range));
    }

    function update_time_from_time_weekly_view(){
        $time_info_string = array_var($_REQUEST, 'time_info');
        $time_value = array_var($_REQUEST, 'time_value');
        $user_id = array_var($_REQUEST, 'user_id');
        $genid = array_var($_REQUEST, 'genid');
        $is_multiple = array_var($_REQUEST, 'multiple', false);

        $new_time_added = false;
        $new_time_id = 0;
        $new_view_time_hours_id = '';
        $deleted_timeslot = false;

        $time_info = explode('-', $time_info_string);
        $action = $time_info[1];
        $project_id = $time_info[2];
        $task_id = $time_info[3];
        $date_value = $time_info[4];
        $time_id = $time_info[5];

        $hours_minutes = explode(':', $time_value);
        $hours = $hours_minutes[0];
        $minutes = $hours_minutes[1];
        $total_minutes = $hours * 60 + $minutes;

        if($action == 'edit'){
            $timeslot = Timeslots::findById($time_id);
            if($total_minutes == 0) {
                DB::beginWork();
                $timeslot->trash();
                DB::commit();
                ApplicationLogs::createLog($timeslot, ApplicationLogs::ACTION_TRASH);
                $deleted_timeslot = true;
                $new_view_time_hours_id = $genid.'-new-'.$project_id.'-'.$task_id.'-'.$date_value.'-0';
            } else {
                $end_time = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
                $end_time->add('m', $total_minutes);
                DB::beginWork();
                $timeslot->setEndTime($end_time);
                $timeslot->setColumnValue('ignore_time_start', 1);

                // Round time
	            Hook::fire('round_minutes_to_fifteen', array('timeslot' => $timeslot), $total_minutes);
                $timeslot->save();
                DB::commit();
            }
        } else if($action == 'new'){
            $member_ids = array($project_id);
            $task = ProjectTasks::findById($task_id);
            if($task instanceof ProjectTask){
                $task_members = $task->getMembers();
                foreach ($task_members as $mem) {
                    $member_ids[] = $mem->getId();
                }
            }

            $updated_date_value = str_replace('_', '-', $date_value);
            $date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $updated_date_value); 
            $start_time = user_config_option('work_day_start_time');
            
            // Call hook to round time
            Hook::fire('get_hour_minutes_rounded', $total_minutes, $ret);
            
            // If the time is rounded to 15 minutes update values for timeslot_data
            if($ret){
                $hours = $ret['hours'];
                $minutes = $ret['minutes'];
                $total_minutes = $ret['total_minutes'];
            }

            $timeslot_data = array(
                'hours' => $hours,
                'minutes' => $minutes,
                'date' => $date,
                'start_time' => $start_time,
                'description' => '',
                'contact_id' => $user_id,
                'ignore_time_start' => 1,
                'invoicing_status' => 'pending',
                'additional_member_ids' => array_unique(array_filter($member_ids)),
            );
        
            $parameters = array();
            $parameters['use_current_time'] = false;
            $parameters['object_id'] = $task_id;
            $parameters['timeslot'] = $timeslot_data;

            $time_controller = new TimeController();
            $ts = $time_controller->add_timeslot($parameters, false); 

            if ($ts instanceof Timeslot) {
                $new_time_added = true;
                $new_time_id = $ts->getId();
                $new_view_time_hours_id = $genid.'-edit-'.$project_id.'-'.$task_id.'-'.$date_value.'-'.$new_time_id;
            }
        }

        $description_info = array('popover_id' => '', 'description_button_and_content' => '', 'description_input' => '', 'desc_btn_color' => '');

       if($new_time_added){
            $description_info = $this->get_new_description_html_input($new_view_time_hours_id, $new_time_added, $deleted_timeslot);
        } else if ($deleted_timeslot){
            $description_info = $this->get_new_description_html_input($time_info_string, $new_time_added, $deleted_timeslot);
        }


        $result = array('new_time_added' => $new_time_added, 
                        'new_time_id' => $new_time_id,
                        'new_view_time_hours_id' => $new_view_time_hours_id,
                        'deleted_timeslot' => $deleted_timeslot,
                        'popover_id' => $description_info['popover_id'],
                        'description_button_and_content' => $description_info['description_button_and_content'],
                        'description_input' => $description_info['description_input'],
                        'desc_btn_color' => $description_info['desc_btn_color']);

        ajx_current("empty"); 
        ajx_extra_data($result);
    }

    function get_updated_totals_for_weekly_view(){
        Env::useHelper('time_weekly_functions', 'advanced_core');
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $from_date = array_var($_REQUEST, 'from_date');
        $until_date = array_var($_REQUEST, 'until_date');
        $user_id = array_var($_REQUEST, 'user_id');
        $genid = array_var($_REQUEST, 'genid');
        $filters = array('weekly_user_id' => $user_id, 'from_date' => $from_date, 'until_date' => $until_date);
        $weekly_data = get_weekly_time_data($filters);

        $time_info_string = array_var($_REQUEST, 'time_info');
        $time_info = explode('-', $time_info_string);
        $action = $time_info[1];
        $project_id = $time_info[2];
        $task_id = $time_info[3];
        $date_value = $time_info[4];
        $time_id = $time_info[5];
        $updated_date_value = str_replace('_', '-', $date_value);
        $date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $updated_date_value);
        $weekday = strtolower($date->format('D'));

        $task_total = convert_num_to_hours_mins($weekly_data['projects'][$project_id]['tasks'][$task_id]['task_weekly_total']);
        $day_total = convert_num_to_hours_mins($weekly_data['day_totals'][$weekday]);
        $grand_total =  convert_num_to_hours_mins($weekly_data['weekly_total']);
        $task_day_total = convert_num_to_hours_mins($weekly_data['projects'][$project_id]['tasks'][$task_id]['weekly_timeslots'][$weekday]['task_day_total']);
        $task_total_id = $genid . '-' . 'task_total' . '-' . $project_id . '-' . $task_id;
        $day_total_id = $genid . '-' . 'day_total' . '-' . $user_id . '-' . $weekday;
        $grand_total_id = $genid . '-' . 'grand_total' . '-' . $user_id;
        $task_day_total_id = $genid . '-' . 'multiple_time' . '-' . $project_id . '-' . $task_id . '-' . $date_value;

        $result = array(
            'task_day_total_id' => $task_day_total_id,
            'task_day_total' => $task_day_total,
            'task_total_id' => $task_total_id,
            'task_total' => $task_total,
            'day_total_id' => $day_total_id,
            'day_total' => $day_total,
            'grand_total_id' => $grand_total_id,
            'grand_total' => $grand_total
        );

        ajx_current("empty"); 
        ajx_extra_data($result);
    }

    function update_time_description_from_weekly_view(){
        $desc_info_string = array_var($_REQUEST, 'desc_info');
        $new_desc_value = array_var($_REQUEST, 'desc_value');

        $time_info = explode('-', $desc_info_string);
        $gen_id = $time_info[0];
        $project_id = $time_info[3];
        $task_id = $time_info[4];
        $fullday = $time_info[5];
        $time_id = $time_info[6];

        $popover_id = $gen_id . '-' . 'popover' . '-' . $project_id . '-' . $task_id . '-' . $fullday; 
        $desc_color = "#858585";
        $desc_onchange_func = 'og.updateDescFromWeekly(this)';
        $description_input = '<div>';
        $description_input .= '<div class=\'popover-timeslots\'>';
        if( $new_desc_value == ''){
            $description_input .= '<div><textarea id=\''.$desc_info_string.'\' onchange=\'og.updateDescFromWeekly(this);\' placeholder=\''.lang('description').'...\' class=\'weekly--description-textarea\'></textarea></div>';
            $svg_lines = false;
        } else {
            $description_input .= '<div><textarea id=\''.$desc_info_string.'\' onchange=\'og.updateDescFromWeekly(this);\' class=\'weekly--description-textarea\'>'.$new_desc_value.'</textarea></div>';
            $desc_color = '#22CD92';
            $svg_lines = true;
        }
        $description_input .= '</div>';
        $description_input .= '<button class=\'btn btn-xs btn-primary x-tool x-tool-options add-weekly-time-btn\' onclick=\'og.addWeeklyTimeToPopover(`'.$popover_id.'`);\' type=\'button\'>Add worked hours</button>';
        $description_input .= '</div>';


        // Plus sign by default
        $svg_icon='<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 530 530" 
        fill="none" xmlns="http://www.w3.org/2000/svg">
        <g transform="translate(0,505) scale(0.100000,-0.100000)"
        fill="'.$desc_color.'">
        <path d="M1701 5098 c-36 -26 -1095 -1208 -1105 -1233 -11 -31 -7 -3782 5
        -3808 6 -13 23 -31 39 -40 26 -16 115 -17 1065 -17 1151 0 1082 -4 1111 67 20
        46 10 86 -28 118 l-30 25 -979 0 -979 0 0 1710 0 1710 500 0 500 0 32 29 33
        29 3 611 2 611 1170 0 1170 0 0 -1501 c0 -1664 -5 -1550 65 -1583 41 -20 49
        -20 90 0 70 33 65 -87 65 1638 0 1323 -2 1562 -14 1591 -29 70 67 65 -1379 65
        l-1304 0 -32 -22z m-54 -1255 c-2 -2 -181 -2 -397 -1 l-394 3 395 440 394 440
        3 -439 c1 -242 1 -441 -1 -443z"/>
        <path d="M3855 1381 c-11 -5 -29 -19 -40 -31 -18 -20 -20 -40 -25 -259 l-5
        -236 -237 -5 c-230 -5 -237 -6 -261 -28 -32 -30 -40 -74 -23 -117 25 -60 48
        -65 301 -65 l225 0 0 -236 0 -236 29 -29 c33 -33 83 -39 130 -15 47 25 51 49
        51 291 l0 225 225 0 c246 0 270 5 293 55 21 47 15 94 -17 126 l-29 29 -236 0
        -236 0 0 225 c0 253 -5 275 -65 301 -38 16 -51 16 -80 5z"/>
        </g>
        </svg>';
        if($svg_lines){
            $svg_icon = '<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 1000 1000" 
            fill="none" xmlns="http://www.w3.org/2000/svg">
            <g transform="translate(0,960) scale(0.100000,-0.100000)"
            fill="'.$desc_color.'">
            <path stroke-width="2" d="M2848 9790 c-385 -65 -679 -380 -714 -766 l-7 -74 215 0 215 0 6 56
            c13 130 107 258 236 322 l75 37 1973 3 1972 2 3 -1032 c3 -954 5 -1038 21
            -1092 47 -158 101 -255 201 -363 117 -125 253 -208 421 -255 80 -22 84 -22
            993 -26 l912 -3 0 -2273 c0 -2206 -1 -2275 -19 -2328 -54 -157 -188 -267 -353
            -288 -40 -5 -259 -10 -485 -10 l-413 0 0 -211 0 -210 498 3 c549 4 548 4 698
            73 232 106 392 290 471 542 l28 88 3 2414 2 2415 -36 37 c-33 34 -36 42 -31
            78 l5 39 -1315 1416 -1315 1416 -2107 -1 c-1158 -1 -2127 -5 -2153 -9z m5333
            -1770 l923 -985 -739 -3 c-639 -2 -747 0 -800 13 -160 41 -284 174 -314 339
            -13 73 -16 1629 -2 1624 5 -1 424 -446 932 -988z"/>
            <path stroke-width="2" d="M680 8504 c-351 -75 -620 -361 -670 -712 -8 -55 -10 -1082 -8 -3582
            l3 -3505 22 -72 c50 -158 118 -275 225 -381 108 -109 229 -179 386 -225 l77
            -22 3120 0 3120 0 78 23 c306 88 513 293 609 604 l23 73 3 2414 2 2415 -36 37
            c-33 34 -36 42 -31 78 l5 41 -1314 1415 -1314 1415 -2118 -1 c-1771 0 -2128
            -3 -2182 -15z m4010 -1449 l5 -1040 29 -85 c15 -47 40 -107 53 -135 87 -173
            254 -328 436 -403 150 -63 136 -62 1127 -62 l900 0 -2 -2293 -3 -2293 -37 -75
            c-46 -93 -112 -158 -206 -202 l-67 -32 -3090 0 -3090 0 -66 31 c-114 54 -190
            138 -228 252 l-21 62 2 3502 3 3503 32 66 c59 121 185 216 313 238 32 5 870 8
            1980 7 l1925 -1 5 -1040z m1389 -342 c490 -524 891 -955 891 -958 0 -3 -329
            -5 -731 -5 -821 0 -819 0 -926 69 -104 67 -170 165 -192 285 -7 40 -11 321
            -11 850 l0 791 39 -40 c21 -22 439 -468 930 -992z"/>
            <path stroke-width="2" d="M1492 6178 l3 -213 1063 -3 1062 -2 0 215 0 215 -1065 0 -1065 0 2
            -212z"/>
            <path stroke-width="2" d="M1490 4900 l0 -210 1065 0 1065 0 0 210 0 210 -1065 0 -1065 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 3620 l0 -210 2345 0 2345 0 0 210 0 210 -2345 0 -2345 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 2345 l0 -215 2345 0 2345 0 -2 213 -3 212 -2342 3 -2343 2 0
            -215z"/>
            </g>
            </svg>';
        }

        $description_button = '
            <button class="weekly--desc-button" data-toggle="popover" data-content="'.$description_input.'" data-placement="bottom">
            '.$svg_icon.'
            </button>';


        $timeslot = Timeslots::findById($time_id);
        if($timeslot instanceof Timeslot){
            $timeslot->setDescription($new_desc_value);
            $timeslot->save();
        }

        $result = array(
            'popover_id' => $popover_id,
            'description_button_and_content' => $description_button,
            'description_input' => $description_input,
            'desc_btn_color' => $desc_color
        );
        ajx_current("empty"); 
        ajx_extra_data($result);
    }

    function update_time_description_from_weekly_view_multi_entries(){
        $desc_info_string = array_var($_REQUEST, 'desc_info');
        $new_desc_value = array_var($_REQUEST, 'desc_value');

        $time_info = explode('-', $desc_info_string);
        $time_id = $time_info[6];

        $timeslot = Timeslots::findById($time_id);
        if($timeslot instanceof Timeslot){
            $timeslot->setDescription($new_desc_value);
            $timeslot->save();
        }

        ajx_current("empty"); 
        ajx_extra_data(array('success' => true));
    }

    function get_updated_description_html_multi_entries(){
        Env::useHelper('time_weekly_functions', 'advanced_core');
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $current_filters = array();
        $time_filters = json_decode(user_config_option('current_time_module_filters'), true);
        $current_filters['weekly_user_id'] = $time_filters['user_filter'] == 0 ? logged_user()->getId() : $time_filters['user_filter'];
        get_date_range_for_this_week($current_filters);
        $weekly_data = get_weekly_time_data($current_filters);

        $desc_info_string = array_var($_REQUEST, 'desc_info');
        $time_info = explode('-', $desc_info_string);
        $genid = $time_info[0];
        $project_id = $time_info[3];
        $task_id = $time_info[4];
        $date_value = $time_info[5];
        $time_id = $time_info[6];


        $updated_date_value = str_replace('_', '-', $date_value);
        $date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $updated_date_value); 
        $task_info = $weekly_data['projects'][$project_id]['tasks'][$task_id];

        $time_input_and_desc = get_time_input_and_desc_html($task_info, $task_id, $date, $genid);

        /* 
        RETURN FORMAT
        array(
            'popover_id' => $popover_id,
            'description_content' => $description_input,
        ); 
        */

        ajx_current("empty"); 
        ajx_extra_data($time_input_and_desc);
    }


    function get_new_description_html_input($time_info_string, $new_time_added, $deleted_timeslot){
        $time_info = explode('-', $time_info_string);
        $genid = $time_info[0];
        $action = $time_info[1];
        $project_id = $time_info[2];
        $task_id = $time_info[3];
        $fullday = $time_info[4];
        $timeslot_id = $time_info[5];

        $popover_id = $genid . '-' . 'popover' . '-' . $project_id . '-' . $task_id . '-' . $fullday;
        $desc_color = "#858585";
        $desc_input_disabled = '';

        $desc_input_id = 0;
      
        if($new_time_added){
            $desc_input_id = $genid . '-description-edit-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$timeslot_id;
            $svg_lines = true;
        } else if($deleted_timeslot) {
            $svg_lines = false;
            $desc_input_id = $genid . '-decription-new-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . 0;
            $desc_input_disabled = 'disabled';
            $desc_color = "#D6D6D6";
        }

        $desc_onchange_func = 'og.updateDescFromWeekly(this)';
        $description_input = '<div>';
        $description_input .= '<div class=\'popover-timeslots\'>';
        $description_input .= '<div><textarea id=\''.$desc_input_id.'\' onchange='.$desc_onchange_func.' placeholder=\''.lang('description').'...\' class=\'weekly--description-textarea\'></textarea></div>';
        $description_input .= '</div>';
        $description_input .= '<button class=\'btn btn-xs btn-primary x-tool x-tool-options add-weekly-time-btn\' onclick=\'og.addWeeklyTimeToPopover(`'.$popover_id.'`);\' type=\'button\'>Add worked hours</button>';
        $description_input .= '</div>';

        // Plus sign by default
        $svg_icon='<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 530 530" 
        fill="none" xmlns="http://www.w3.org/2000/svg">
        <g transform="translate(0,505) scale(0.100000,-0.100000)"
        fill="'.$desc_color.'">
        <path d="M1701 5098 c-36 -26 -1095 -1208 -1105 -1233 -11 -31 -7 -3782 5
        -3808 6 -13 23 -31 39 -40 26 -16 115 -17 1065 -17 1151 0 1082 -4 1111 67 20
        46 10 86 -28 118 l-30 25 -979 0 -979 0 0 1710 0 1710 500 0 500 0 32 29 33
        29 3 611 2 611 1170 0 1170 0 0 -1501 c0 -1664 -5 -1550 65 -1583 41 -20 49
        -20 90 0 70 33 65 -87 65 1638 0 1323 -2 1562 -14 1591 -29 70 67 65 -1379 65
        l-1304 0 -32 -22z m-54 -1255 c-2 -2 -181 -2 -397 -1 l-394 3 395 440 394 440
        3 -439 c1 -242 1 -441 -1 -443z"/>
        <path d="M3855 1381 c-11 -5 -29 -19 -40 -31 -18 -20 -20 -40 -25 -259 l-5
        -236 -237 -5 c-230 -5 -237 -6 -261 -28 -32 -30 -40 -74 -23 -117 25 -60 48
        -65 301 -65 l225 0 0 -236 0 -236 29 -29 c33 -33 83 -39 130 -15 47 25 51 49
        51 291 l0 225 225 0 c246 0 270 5 293 55 21 47 15 94 -17 126 l-29 29 -236 0
        -236 0 0 225 c0 253 -5 275 -65 301 -38 16 -51 16 -80 5z"/>
        </g>
        </svg>';
        if($svg_lines){
            $svg_icon = '<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 1000 1000" 
            fill="none" xmlns="http://www.w3.org/2000/svg">
            <g transform="translate(0,960) scale(0.100000,-0.100000)"
            fill="'.$desc_color.'">
            <path stroke-width="2" d="M2848 9790 c-385 -65 -679 -380 -714 -766 l-7 -74 215 0 215 0 6 56
            c13 130 107 258 236 322 l75 37 1973 3 1972 2 3 -1032 c3 -954 5 -1038 21
            -1092 47 -158 101 -255 201 -363 117 -125 253 -208 421 -255 80 -22 84 -22
            993 -26 l912 -3 0 -2273 c0 -2206 -1 -2275 -19 -2328 -54 -157 -188 -267 -353
            -288 -40 -5 -259 -10 -485 -10 l-413 0 0 -211 0 -210 498 3 c549 4 548 4 698
            73 232 106 392 290 471 542 l28 88 3 2414 2 2415 -36 37 c-33 34 -36 42 -31
            78 l5 39 -1315 1416 -1315 1416 -2107 -1 c-1158 -1 -2127 -5 -2153 -9z m5333
            -1770 l923 -985 -739 -3 c-639 -2 -747 0 -800 13 -160 41 -284 174 -314 339
            -13 73 -16 1629 -2 1624 5 -1 424 -446 932 -988z"/>
            <path stroke-width="2" d="M680 8504 c-351 -75 -620 -361 -670 -712 -8 -55 -10 -1082 -8 -3582
            l3 -3505 22 -72 c50 -158 118 -275 225 -381 108 -109 229 -179 386 -225 l77
            -22 3120 0 3120 0 78 23 c306 88 513 293 609 604 l23 73 3 2414 2 2415 -36 37
            c-33 34 -36 42 -31 78 l5 41 -1314 1415 -1314 1415 -2118 -1 c-1771 0 -2128
            -3 -2182 -15z m4010 -1449 l5 -1040 29 -85 c15 -47 40 -107 53 -135 87 -173
            254 -328 436 -403 150 -63 136 -62 1127 -62 l900 0 -2 -2293 -3 -2293 -37 -75
            c-46 -93 -112 -158 -206 -202 l-67 -32 -3090 0 -3090 0 -66 31 c-114 54 -190
            138 -228 252 l-21 62 2 3502 3 3503 32 66 c59 121 185 216 313 238 32 5 870 8
            1980 7 l1925 -1 5 -1040z m1389 -342 c490 -524 891 -955 891 -958 0 -3 -329
            -5 -731 -5 -821 0 -819 0 -926 69 -104 67 -170 165 -192 285 -7 40 -11 321
            -11 850 l0 791 39 -40 c21 -22 439 -468 930 -992z"/>
            <path stroke-width="2" d="M1492 6178 l3 -213 1063 -3 1062 -2 0 215 0 215 -1065 0 -1065 0 2
            -212z"/>
            <path stroke-width="2" d="M1490 4900 l0 -210 1065 0 1065 0 0 210 0 210 -1065 0 -1065 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 3620 l0 -210 2345 0 2345 0 0 210 0 210 -2345 0 -2345 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 2345 l0 -215 2345 0 2345 0 -2 213 -3 212 -2342 3 -2343 2 0
            -215z"/>
            </g>
            </svg>';
        }

        $description_button = '
            <button class="weekly--desc-button" '.$desc_input_disabled.' data-toggle="popover" data-content="'.$description_input.'" data-placement="bottom">
            '.$svg_icon.'
            </button>';

        return array('popover_id' => $popover_id,
                     //'description_html' => $description_button,
                     'description_button_and_content' => $description_button,
                     'description_input' => $description_input,
                     'desc_btn_color' => $desc_color
                     );

    }


    function get_updated_time_and_desc_input_after_mult_time_change(){

        $time_info_string = array_var($_REQUEST, 'time_info');
        $user_id = array_var($_REQUEST, 'user_id');
        $genid = array_var($_REQUEST, 'genid');
        $action = array_var($_REQUEST, 'action');

        Env::useHelper('time_weekly_functions', 'advanced_core');
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $current_filters = array();
        $time_filters = json_decode(user_config_option('current_time_module_filters'), true);
        $current_filters['weekly_user_id'] = $time_filters['user_filter'] == 0 ? logged_user()->getId() : $time_filters['user_filter'];
        get_date_range_for_this_week($current_filters);
        $weekly_data = get_weekly_time_data($current_filters);

        $time_info = explode('-', $time_info_string);
        $genid = $time_info[0];
        $action = $time_info[1];
        $project_id = $time_info[2];
        $task_id = $time_info[3];
        $date_value = $time_info[4];
        $time_id = $time_info[5];

        $updated_date_value = str_replace('_', '-', $date_value);
        $date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $updated_date_value); 
        $task_info = $weekly_data['projects'][$project_id]['tasks'][$task_id];
        $weekday = strtolower($date->format("D"));
        $count_timeslots = count($task_info['weekly_timeslots'][$weekday]['timeslots']);

        $time_input_and_desc = get_time_input_and_desc_html($task_info, $task_id, $date, $genid);
        $time_input_and_desc['count_timeslots'] = $count_timeslots;

        $multi_time_input_id = $genid . '-' . 'multiple_time' . '-' . $project_id . '-' . $task_id . '-' . $date_value;
        $time_input_and_desc['multi_time_input_id'] = $multi_time_input_id;
        /* 
        RETURNS
        array(
            'time_input' => $time_input,
            'description_button' => $description_button,
            'popover_id' => $popover_id,
            'description_content' => $description_input,
            'time_input_id' => $time_input_id,
            'formatted_time_value' => $formatted_time,
            'count_timeslots' => $count_timeslots,
            'multi_time_input_id' => $multi_time_input_id
        );
        */

        ajx_current("empty"); 
        ajx_extra_data($time_input_and_desc);
    }

    function get_new_time_input_for_popover_view(){
        $popover_id_string = array_var($_REQUEST, 'popover_id');

        $popover_id = explode('-', $popover_id_string);
        $genid = $popover_id[0];
        $project_id = $popover_id[2];
        $task_id = $popover_id[3];
        $date_value = $popover_id[4];

        $new_input = '<div class=\'weekly--popover-time-desc-div\'>';
        // Add time input for each timeslot
        $day_time_input_id = $genid . '-' . 'new' . '-' . $project_id . '-' . $task_id . '-' . $date_value . '-' . 0;
        $day_time_onchange_func = 'og.updateTimeFromWeeklyDescDiv(this)';
        $new_input .= "<input class='weekly--time-input-grey' id='".$day_time_input_id."' type='text' value='00:00' prevvalue='00:00' onfocus='og.clearTime(this);' onchange='og.updateTimeFromWeeklyDescDiv(this);' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";

        // Add description input for each timeslot
        $day_desc_input_id = $genid . '-description-edit-' . $project_id . '-' . $task_id . '-' . $date_value . '-' . 0;
        $day_desc_onchange_func = 'og.updateDescFromWeeklyDescDiv(this)';
        $new_input .= '<div><textarea id=\''.$day_desc_input_id.'\' disabled onchange='.$day_desc_onchange_func.' placeholder=\''.lang('description').'...\' class=\'weekly--description-textarea\'></textarea></div>';
        $new_input .= '</div>';

        ajx_current("empty"); 
        ajx_extra_data(array('new_input' => $new_input, 
                            'day_time_input_id' => $day_time_input_id));
    }

    function update_weekly_time_module_filters($user_id, $from_date, $end_date){
        // Set user id for time module
        $time_module_filters = json_decode(user_config_option('current_time_module_filters'), true);
        $time_module_filters['user_filter'] = $user_id;
        set_user_config_option('current_time_module_filters', json_encode($time_module_filters), logged_user()->getId());
        // Set dates for the weekly config options
        set_user_config_option('weekly_time_start_date', $from_date, logged_user()->getId());
        set_user_config_option('weekly_time_end_date', $end_date, logged_user()->getId());
        
    }

    function get_current_week_date_range_for_weekly(){
        $start_monday = user_config_option('start_monday');
        if($start_monday){
            $from = new DateTimeValue(strtotime('monday this week'));
            $until = new DateTimeValue(strtotime('sunday this week'));
        } else {
            $from = new DateTimeValue(strtotime('sunday previous week'));
            $until = new DateTimeValue(strtotime('saturday this week'));
        }
        $from = $from->format('Y-m-d');
        $until = $until->format('Y-m-d');

        $result = array('from' => $from, 
                        'until' => $until);

        ajx_current("empty"); 
        ajx_extra_data($result);
    }

    function get_task_selector_html_for_weekly() {
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $project_member_ids = json_decode(array_var($_REQUEST, 'project_member_id'));
        $user_id = array_var($_REQUEST, 'user_id');
        $selected_task_ids = json_decode(array_var($_REQUEST, 'selected_task_ids'));
        $genid = array_var($_REQUEST, 'genid');

        if(empty($project_member_ids)){
            $selector = build_disabled_task_selector_for_project_in_weekly_view($genid);
        } else {
            $params = array(
                'project_member_id' => $project_member_ids[0],
                'user_id' => $user_id,
                'selected_task_ids' => $selected_task_ids
            );
            $selector = build_task_selector_for_project_in_weekly_view($params, $genid);
        }

        $result = array('selector_html' => $selector);

        ajx_current("empty"); 
        ajx_extra_data($result);
    }

    function get_task_row_for_weekly_view() {
        Env::useHelper('time_weekly_functions', 'advanced_core');
        Env::useHelper('time_weekly_view_functions', 'advanced_core');
        $pm_ids = json_decode(array_var($_REQUEST, 'project_member_id'));
        $project_member_id = array_var($pm_ids, 0, 0);
        $task_id = array_var($_REQUEST, 'task_id');
        $from_date = array_var($_REQUEST, 'from_date');
        $until_date = array_var($_REQUEST, 'until_date');
        $user_id = array_var($_REQUEST, 'user_id');
        $genid = array_var($_REQUEST, 'genid');
        $has_project_selected = array_var($_REQUEST, 'has_project_selected');
        $current_filters = array('weekly_user_id' => $user_id, 'from_date' => $from_date, 'until_date' => $until_date, 'has_project_selected' => $has_project_selected);

        $project_member = Members::findById($project_member_id);
        $task = ProjectTasks::findById($task_id);
        $task_name = $task instanceof ProjectTask ? $task->getName() : lang('no specific task');

        $task_info = array(
            'project_member' => $project_member,
            'task_id' => $task_id,
            'task_name' => $task_name
        );
        $date_range = get_date_range_for_weekly_view($current_filters);

        $task_data = create_task_data_for_view($task_info);
        $task_row = build_task_row_for_weekly_view($task_data, $date_range, $genid, $current_filters);

        $result = array('task_row' => $task_row);

        ajx_current("empty"); 
        ajx_extra_data($result);

    }
}