<?php

function get_default_property_groups_queries($db_connection, $table_prefix) {
	if (class_exists("Env")) {
		Env::useHelper("plugin_installer_functions");
	} else {
		include_once dirname(__FILE__)."/../../../../application/helpers/plugin_installer_functions.php";
	}
	
	$queries = array();
	
	// add default group for custom properties that still doesn't have it
	$queries[] = "INSERT INTO ".$table_prefix."property_groups (object_type_id, name, code, `order`, is_system) 
			SELECT ot.id, 'Details', 'default', 0, 1 FROM ".$table_prefix."object_types ot
			WHERE ot.`type` IN ('content_object','dimension_object','dimension_group')
			AND NOT EXISTS (SELECT pg.id FROM ".$table_prefix."property_groups pg WHERE pg.object_type_id=ot.id AND pg.code='default');";
	


	$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
			SELECT pg.id, 'located_under', '-1'
			FROM ".$table_prefix."property_groups pg
			WHERE pg.code='default'
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('dimension_object', 'dimension_group'))
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp 
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
					WHERE pgp.property_id='located_under' AND px.object_type_id = pg.object_type_id
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	
	// add custom properties to the default property group
	$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
			SELECT pg.id, cp.id, cp.property_order
			FROM ".$table_prefix."property_groups pg
			LEFT JOIN ".$table_prefix."custom_properties cp ON cp.object_type_id=pg.object_type_id
			WHERE pg.code='default' 
				AND pg.object_type_id=cp.object_type_id
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('content_object', 'dimension_object'))
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp 
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
					WHERE pgp.property_id=cp.id
						AND px.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('content_object', 'dimension_object'))
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	
	// add member custom properties to the default property group
	if (check_is_installed_plugin($db_connection, $table_prefix, 'member_custom_properties')) {
		$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
			SELECT pg.id, cp.id, cp.property_order
			FROM ".$table_prefix."property_groups pg
			LEFT JOIN ".$table_prefix."member_custom_properties cp ON cp.object_type_id=pg.object_type_id
			WHERE pg.code='default' 
				AND pg.object_type_id=cp.object_type_id
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` = 'dimension_group')
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp 
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
					WHERE pgp.property_id=cp.id
						AND px.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('dimension_group'))
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	}
	
	// add dimension member associations to the property group
	$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
			SELECT pg.id, concat('assoc_',a.id), a.id
			FROM ".$table_prefix."property_groups pg
			INNER JOIN ".$table_prefix."dimension_member_associations a ON a.object_type_id=pg.object_type_id
			WHERE pg.code='default' 
				AND pg.object_type_id=a.object_type_id
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('dimension_group', 'dimension_object'))
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp 
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
					WHERE pgp.property_id=concat('assoc_',a.id)
						AND px.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `type` IN ('dimension_group', 'dimension_object'))
				)
				
				-- only for enabled dimensions
				AND a.associated_dimension_id IN (
					select substring_index(substring_index(`value`, ',', n), ',', -1) as `value`
					from ".$table_prefix."config_options
					join 
					(SELECT @row := @row + 1 as n FROM 
					(select id from ".$table_prefix."dimensions) t,
					(SELECT @row:=0) r) as numbers
					  on char_length(`value`) 
						- char_length(replace(`value`, ',', ''))  >= n - 1

					where name='enabled_dimensions'
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	
	
	// add contact fields for customers and suppliers
	$contact_fields = array('address','birthday','company_id','department','email','phone','picture_file','webpage');
	foreach ($contact_fields as $cfield) {
		$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
			SELECT pg.id, 'fixedprop_contact_id|$cfield', 10
			FROM ".$table_prefix."property_groups pg
			WHERE pg.code='default' 
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `name` IN ('customer', 'supplier'))
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp 
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
					WHERE pgp.property_id='fixedprop_contact_id|$cfield'
						AND px.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `name` IN ('customer', 'supplier'))
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	}
	

	// add contact fields for managed events
	$event_fields = array('start','duration');
	foreach ($event_fields as $cfield) {
		$queries[] = "INSERT INTO ".$table_prefix."property_group_properties (group_id, property_id, `order`)
		SELECT pg.id, 'fixedprop_event_id|$cfield', 10
		FROM ".$table_prefix."property_groups pg
			WHERE pg.code='default'
				AND pg.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `name` IN ('managed_event'))
				AND NOT EXISTS (
					SELECT pgp.property_id FROM ".$table_prefix."property_group_properties pgp
					INNER JOIN ".$table_prefix."property_groups px ON px.id=pgp.group_id
						WHERE pgp.property_id='fixedprop_event_id|$cfield'
						AND px.object_type_id IN (SELECT id FROM ".$table_prefix."object_types WHERE `name` IN ('managed_event'))
				)
			ON DUPLICATE KEY UPDATE group_id=group_id;";
	}
	
	return $queries;
}
