<?php

function get_weekly_time_data($current_filters){
    $user_id = array_var($current_filters, 'weekly_user_id');
    $from_date = DateTimeValueLib::dateFromFormatAndString("Y-m-d", $current_filters['from_date'])->beginningOfDay();
    $until_date = DateTimeValueLib::dateFromFormatAndString("Y-m-d", $current_filters['until_date'])->endOfDay();
    $from_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
    $until_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);

    $current_filters['formatted_from_date'] = $from_date;
    $current_filters['formatted_until_date'] = $until_date;

    $weekly_time_data = array(
        'current_filters' => $current_filters,
        'projects' => array(),
        'tasks' => array(),
        'weekly_total' => 0,
        'day_totals' => array(
            'mon' => 0,
            'tue' => 0,
            'wed' => 0,
            'thu' => 0,
            'fri' => 0,
            'sat' => 0,
            'sun' => 0,
        )
    );

    $project_ot_id = ObjectTypes::findByName('project')->getId();
    $tasks = get_weekly_view_project_tasks($user_id, $from_date, $until_date);
    foreach($tasks as $task){
        $project_members = get_object_members_by_type($task, $project_ot_id);
        $task_info = array(
            'project_member' => array_var($project_members, 0),
            'task_id' => $task->getId(),
            'task_name' => $task->getName()
        );
        prepare_weekly_task_array($task_info, $weekly_time_data);
    }

    $timeslots = get_weekly_view_timeslots($user_id, $from_date, $until_date);
    foreach($timeslots as $time){
        add_time_to_weekly_data($time, $weekly_time_data);
    }

    group_tasks_for_weekly_view($weekly_time_data);
    add_has_project_selected_info($weekly_time_data);

    return $weekly_time_data;
}

function get_weekly_view_project_tasks($user_id, $from_date, $until_date){
    $task_conditions = ' AND `assigned_to_contact_id`='.$user_id.' AND ((`start_date` >= '.DB::escape($from_date).' AND `start_date` <= ' .DB::escape($until_date). ') OR (`due_date` >= '.DB::escape($from_date).' AND `due_date` <= ' .DB::escape($until_date). ') OR (`start_date` <= '.DB::escape($from_date).' AND `due_date` >= ' .DB::escape($until_date). '))';
    $tasks_res = ProjectTasks::instance()->listing(array(
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'ignore_context' => false,
        'extra_conditions' => $task_conditions,
    ));
    $tasks = $tasks_res->objects;

    return $tasks;
}

function get_weekly_view_timeslots($user_id, $from_date, $until_date){
    $timeslot_conditions = ' AND `contact_id`=' . $user_id . ' AND (`start_time` >= '.DB::escape($from_date).' AND `start_time` <= ' .DB::escape($until_date). ')';
    $time_res = Timeslots::instance()->listing(array(
        'order' => '`e`.`object_id`',
        'order_dir' => 'ASC',
        'ignore_context' => false,
        'extra_conditions' =>  $timeslot_conditions,
    ));
    $timeslots = $time_res->objects;

    return $timeslots;
}

function prepare_weekly_task_array($task_info, &$weekly_time_data){
    $project_member = array_var($task_info, 'project_member');
    $project_id = $project_member instanceof Member ? $project_member->getId() : 0;
    $project_name = $project_member instanceof Member ? $project_member->getName() : '';
    if(!array_key_exists($project_id, $weekly_time_data['projects'])){
        add_project_to_weekly_data($project_id, $project_name, $weekly_time_data);
    }
    $task_id = array_var($task_info, 'task_id');
    $task_name = array_var($task_info, 'task_name');
    
    if(!array_key_exists($task_id, $weekly_time_data['projects'][$project_id]['tasks'])){
        $task_info = array(
            'id' => $task_id,
            'name' => $task_name,
            'project_id' => $project_id,
            'project_name' => $project_name,
            'weekly_timeslots' => array(
                'mon' => array('task_day_total' => 0, 'timeslots' => array()),
                'tue' => array('task_day_total' => 0, 'timeslots' => array()),
                'wed' => array('task_day_total' => 0, 'timeslots' => array()),
                'thu' => array('task_day_total' => 0, 'timeslots' => array()),
                'fri' => array('task_day_total' => 0, 'timeslots' => array()),
                'sat' => array('task_day_total' => 0, 'timeslots' => array()),
                'sun' => array('task_day_total' => 0, 'timeslots' => array()),
            ),
            'task_weekly_total' => 0
        );
        $weekly_time_data['projects'][$project_id]['tasks'][$task_id] = $task_info;
    }
}

function add_time_to_weekly_data($time, &$weekly_time_data){
    $task_id = $time->getRelObjectId();
    if($task_id == 0){
        $task_name = '';
    } else {
        $task = ProjectTasks::findById($task_id);
        $task_name = $task->getName();
    }

    $project_ot_id = ObjectTypes::findByName('project')->getId();
    $project_members = get_object_members_by_type($time, $project_ot_id);
    $project_member = array_var($project_members, 0);
    $project_id = $project_member instanceof Member ? $project_member->getId() : 0;

    $task_info = array(
        'project_member' => $project_member,
        'task_id' => $task_id,
        'task_name' =>  $task_name
    );

    prepare_weekly_task_array($task_info, $weekly_time_data);

    $time_id = $time->getId();
    $start_time = $time->getStartTime();
    $start_time->advance(logged_user()->getUserTimezoneValue(), true);
    $weekday = strtolower($start_time->format('D'));
    $worked_time = $time->getColumnValue('worked_time');
    $description = $time->getDescription();

    

    $weekly_time_data['projects'][$project_id]['tasks'][$task_id]['weekly_timeslots'][$weekday]['task_day_total'] += $worked_time;
    $weekly_time_data['projects'][$project_id]['tasks'][$task_id]['task_weekly_total'] += $worked_time;
    $weekly_time_data['projects'][$project_id]['tasks'][$task_id]['weekly_timeslots'][$weekday]['timeslots'][] = array('timeslot_id' => $time_id, 'description' => $description, 'worked_time' => $worked_time);

    $weekly_time_data['day_totals'][$weekday] += $worked_time;
    $weekly_time_data['weekly_total'] += $worked_time;
}

function get_date_range_for_this_week(&$current_filters){
    $start_date_conf_opt = user_config_option('weekly_time_start_date');
    $end_date_conf_opt = user_config_option('weekly_time_end_date');

    if($start_date_conf_opt == '' || $end_date_conf_opt == ''){
        // Set initial contact config options
        $start_monday = user_config_option('start_monday');
        if($start_monday){
            $from = new DateTimeValue(strtotime('monday this week'));
            $until = new DateTimeValue(strtotime('sunday this week'));
        } else {
            $from = new DateTimeValue(strtotime('sunday previous week'));
            $until = new DateTimeValue(strtotime('saturday this week'));
        }
        $from = $from->format('Y-m-d');
        $until = $until->format('Y-m-d');
        set_user_config_option('weekly_time_start_date', $from, logged_user()->getId());
        set_user_config_option('weekly_time_end_date', $until, logged_user()->getId());
    } else {
        $from = $start_date_conf_opt;
        $until = $end_date_conf_opt;
    }

    $current_filters['from_date'] = $from;
    $current_filters['until_date'] = $until;
}

function get_users_for_the_combo_in_weekly_view(){
     //Get Users Info
     $users = array();
     $context = active_context();
     if (!can_manage_time(logged_user())) {
         $users = logged_user()->getCompanyId() > 0 ? Contacts::getAllUsers(" AND `company_id` = " . logged_user()->getCompanyId()) : array(logged_user());
     } else {
         $users = Contacts::getAllUsers();
     }

     // filter users by permissions only if any member is selected.
     $selected_members = active_context_members(false);
     if (count($selected_members) > 0) {
         $tmp_users = array();
         foreach ($users as $user) {
             if (can_read($user, $context, Timeslots::instance()->getObjectTypeId())) {
                 $tmp_users[] = $user;
             }
         }
         $users = $tmp_users;
     }

     return $users;
}

function  add_project_to_weekly_data($project_id, $project_name, &$weekly_time_data){
    $weekly_time_data['projects'][$project_id] = array('name' => $project_name, 'project_id' =>$project_id,  'tasks' => array());
}

function add_project_row_without_task(&$weekly_time_data) {
    $active_member_ids = active_context_members(false);
    $project_member = null;
    $project_dim_id = Dimensions::findByCode('customer_project')->getId();

    $member_count = 0;
    foreach($active_member_ids as $m_id){
        $member = Members::findById($m_id);
        if($member instanceof Member){
            $member_count += 1;
            if($member->getDimensionId() == $project_dim_id) {
                $project_member = $member;
                break;
            }
        }
    }
    
    if($project_member instanceof Member){
        $task_info = array(
            'project_member' => $project_member,
            'task_id' => 0,
            'task_name' => lang('no specific task')
        );
    } else if($member_count == 0) {
        $task_info = array(
            'project_member' => 0,
            'task_id' => 0,
            'task_name' => lang('no specific task')
        );
    }

    prepare_weekly_task_array($task_info, $weekly_time_data);
}

function add_has_project_selected_info(&$weekly_time_data) {
    $active_member_ids = active_context_members(false);
    $project_member = null;
    $project_dim_id = Dimensions::findByCode('customer_project')->getId();

    $member_count = 0;
    foreach($active_member_ids as $m_id){
        $member = Members::findById($m_id);
        if($member instanceof Member){
            $member_count += 1;
            if($member->getDimensionId() == $project_dim_id) {
                $project_member = $member;
                break;
            }
        }
    }
    $weekly_time_data['selected_project_member_id'] = 0;

    $has_project_selected = $project_member instanceof Member;
    $weekly_time_data['selected_project_member_id'] = $has_project_selected ? $project_member->getId() : 0;
    $weekly_time_data['has_project_selected'] = $has_project_selected;
}

function group_tasks_for_weekly_view(&$weekly_time_data){
    $task_groups = array(
        'assigned_for_the_week' => array(),
        'other_tasks' => array(),
        'no_tasks' => array()
    );
    $selected_task_ids = array();

    $current_filters = $weekly_time_data['current_filters'];
    $filter_from_date = $current_filters['formatted_from_date'];
    $filter_until_date = $current_filters['formatted_until_date'];
    $filter_assigned_to_id = $current_filters['weekly_user_id'];
    $projects_with_no_tasks_rows = array();

    $projects = array_var($weekly_time_data, 'projects');
    foreach($projects as $proj){
        $tasks = $proj['tasks'];
        foreach($tasks as $task){
            $task_id = $task['id'];
            if($task_id == 0){
                $task_groups['no_tasks'][] = $task;
                $projects_with_no_tasks_rows[] = $proj['project_id'];
            } else {
                $selected_task_ids[] = $task_id;
                $project_task = ProjectTasks::findById($task_id);
                $pt_start_date = $project_task->getStartDate();
                $pt_due_date = $project_task->getDueDate();
                $pt_assigned_to_id = $project_task->getAssignedToContactId();
                
                $start_date_in_range = $pt_start_date >= $filter_from_date && $pt_start_date <= $filter_until_date;
                $end_date_in_range = $pt_due_date >= $filter_from_date && $pt_due_date <= $filter_until_date;
                $both_outside_of_range = $pt_start_date <= $filter_from_date && $pt_due_date >= $filter_until_date;
                $contact_id_match = $pt_assigned_to_id == $filter_assigned_to_id;

                if($contact_id_match && ($start_date_in_range || $end_date_in_range || $both_outside_of_range)){
                    $task_groups['assigned_for_the_week'][] = $task;
                } else {
                    $task_groups['other_tasks'][] = $task;
                }
            }
        }
    }

    $weekly_time_data['task_groups'] = $task_groups;
    $weekly_time_data['selected_task_ids'] = $selected_task_ids;
    $weekly_time_data['projects_with_no_tasks_rows'] = $projects_with_no_tasks_rows;
}

function create_task_data_for_view($task_info){
    $project_member = array_var($task_info, 'project_member');
    $project_id = $project_member instanceof Member ? $project_member->getId() : 0;
    $project_name = $project_member instanceof Member ? $project_member->getName() : '';
    $task_id = array_var($task_info, 'task_id');
    $task_name = array_var($task_info, 'task_name');
    $task_data = array(
        'id' => $task_id,
        'name' => $task_name,
        'project_id' => $project_id,
        'project_name' => $project_name,
        'weekly_timeslots' => array(
            'mon' => array('task_day_total' => 0, 'timeslots' => array()),
            'tue' => array('task_day_total' => 0, 'timeslots' => array()),
            'wed' => array('task_day_total' => 0, 'timeslots' => array()),
            'thu' => array('task_day_total' => 0, 'timeslots' => array()),
            'fri' => array('task_day_total' => 0, 'timeslots' => array()),
            'sat' => array('task_day_total' => 0, 'timeslots' => array()),
            'sun' => array('task_day_total' => 0, 'timeslots' => array()),
        ),
        'task_weekly_total' => 0
    );

    return $task_data;
}
