<?php

function build_html_for_weekly_view($weekly_data, $genid){
    $current_filters = array_var($weekly_data, 'current_filters');
    $date_range = get_date_range_for_weekly_view($current_filters);
    $has_project_selected = array_var($weekly_data, 'has_project_selected');

    $weekly_table_class = $has_project_selected ? 'weekly-grid-table weekly-grid-table--has-project' : 'weekly-grid-table';
    $weekly_html = "<table class='".$weekly_table_class."'>";

    $weekly_html .= build_header_for_weekly_view($weekly_data, $date_range);

    $weekly_html .= "<tbody>";
    $weekly_html .= build_body_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= "</tbody>";

    $weekly_html .= "<tbody class='weekly-footer'>";
    $weekly_html .= build_row_adder_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= build_total_row_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= "</tbody>";

    $weekly_html .= "</table>";

    return $weekly_html;
}


function build_header_for_weekly_view($weekly_data, $date_range){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    
    $result = "<thead>";
    $result = "<tr class='grid-row grid-header'>";
    if(!$has_project_selected){
        $result .= "<th>" . lang('project') . "</th>";
    }
    $result .= "<th>" . lang('task') . "</th>";

    foreach($date_range as $date){
        $result .= "<th>";
        $result .= "<span class='weekly-header--weekday'>" . $date->format("D") . "</span>";
        $result .= "<span class='weekly-header--date'>" . $date->format("M d") . "</span>";
        $result .= "</th>";
    }

    $result .= "<th>" . lang('total') . "</th>";
    $result .= "</tr></thead>";

    return $result;
}


function build_body_for_weekly_view($weekly_data, $date_range, $genid){ 
    $task_groups = array_var($weekly_data, 'task_groups');
    $user_name = get_user_name_from_weekly_data($weekly_data);

    $result = "";
    foreach($task_groups as $group_name => $tasks){
        $has_tasks = count($tasks) ? 1 : 0;
        $hidden = !$has_tasks && $group_name == 'no_tasks';
        $style = $hidden ? 'display: none;' : '';

        $result .= build_group_name_row_for_weekly_view($group_name, $genid, $style, $user_name);
        $result .= '<tbody id="'.$genid.'-group_name-'.$group_name.'" hasTasks="'.$has_tasks.'" style="'.$style.'">';
        if($has_tasks){
            foreach($tasks as $task_info){
                $result .= build_task_row_for_weekly_view($task_info, $date_range, $genid, $weekly_data);
            }
        } else {
            $result .= build_empty_group_message_for_weekly_view($group_name);
        }
        $result .= '</tbody>';
    }
   

    return $result;
}


function get_date_range_for_weekly_view($current_filters){
    $from_date = array_var($current_filters, 'from_date');
    $until_date = array_var($current_filters, 'until_date');
    
    $begin = new DateTime( $from_date );
    $end = new DateTime( $until_date );
    $end = $end->modify( '+1 day' );

    $interval = new DateInterval('P1D');
    $daterange = new DatePeriod($begin, $interval ,$end);

    return $daterange;
}

function build_group_name_row_for_weekly_view($group_name, $genid, $style, $user_name){
    $result = "<tbody id='".$genid."-header_group_name-".$group_name."' style='".$style."'>";
    $result .= "<tr>";
    $result .= "<td class='weekly--group-name'>" . lang($group_name, $user_name) . "</td>";
    $result .= "</tr></tbody>";

    return $result;
}

function build_empty_group_message_for_weekly_view($group_name) {

    if($group_name == 'no_tasks'){
        $message = lang('no time entries');
    } else {
        $message = lang('no tasks found weekly view');
    }

    $result = "<tr>";
    $result .= "<td class='weekly--group-message'>" .  $message . "</td>";
    $result .= "</tr>";

    return $result;
}

function build_task_row_for_weekly_view($task_info, $date_range, $genid, $weekly_data){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $task_id = $task_info['id'];
    $project_id = $task_info['project_id'];
    $project_name = array_var($task_info, 'project_name') == '' ? lang('no specific project') : array_var($task_info, 'project_name');
    $task_name = array_var($task_info, 'name') == '' ? lang('no specific task') : array_var($task_info, 'name');
    $result = "<tr class='weekly--task-row'>";
    if(!$has_project_selected){
        if($project_id > 0) {
            $mem = Members::getMemberById($project_id); 
            if ($mem instanceof Member) {
                $mem_html_path = '<div class="task-breadcrumb-container">';
                $mem_html_path .= '<span class="member-path">';

                $mem_html_path .= '<span class="member-path cursor-pointer real-breadcrumb og-wsname-color-'.$mem->getColor().'" data-show-link="true"><span><a onclick="og.projects.onProjectClick('.$project_id.');">' . $mem->getName() . '</a></span></span>';
                $mem_html_path .= '</span>';
                $mem_html_path .= '</div>';
            }
            $result .= "<td class='weekly--task-name' title='". $mem_html_path ."'>" . $mem_html_path . "</td>";
        } else {
            $result .= "<td class='weekly--task-name'  title='". $project_name ."'>" . $project_name . "</td>";
        }
    }
    if($task_id > 0){
        $task_obj = ProjectTasks::findById($task_id);
        if($task_obj instanceof ProjectTask){
            $task_url = $task_obj->getViewUrl();
            $task_name = '<a href="#" class="underline" title="'. $task_name .'" onclick="og.openLink(\''.$task_url.'\');return false;">'.$task_name.'</a>';
        }
    }
    $result .= "<td class='weekly--task-name'>" . $task_name . "</td>";
    foreach($date_range as $date){
        $time_input_and_desc_html = get_time_input_and_desc_html($task_info, $task_id, $date, $genid);
        $result .= "<td class='weekly--td-time-input'>" . $time_input_and_desc_html['time_input'] .  $time_input_and_desc_html['description_button'] ."</td>";
    }
    $total_task_time = array_var($task_info, 'task_weekly_total', 0);
    $formatted_total_task_time = convert_num_to_hours_mins($total_task_time);
    $task_total_id = $genid . '-' . 'task_total' . '-' . $project_id . '-' . $task_id;
    $result .= "<td class='weekly--total-value' id='".$task_total_id."'>" . $formatted_total_task_time . "</td>";
    $result .= "</tr>";

    return $result;
}

function build_row_adder_for_weekly_view($weekly_data, $date_range, $genid) {
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $project_member_id = array_var($weekly_data, 'selected_project_member_id');
    $result = "<tr class='weekly--task-row'>";
    $add_button_style = 'grid-column-start: 2; grid-column-end: -1;';
    $listeners = array();
	$projects_dim = Dimensions::findByCode('customer_project');
    $dimension_id = $projects_dim->getId();

    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $selected_task_ids = array_var($weekly_data, 'selected_task_ids');
    $task_selector_params = array(
        'project_member_id' => $project_member_id,
        'user_id' => $user_id,
        'selected_task_ids' => $selected_task_ids
    );
	
    if(!$has_project_selected) {
        $result .= "<td class='weekly--project-selector'>";
        $result .= "<div id='project-selector-container'></div>";
        $result .= "</td>";
        $add_button_style = 'grid-column-start: 3; grid-column-end: -1;';
    }
    if($has_project_selected){
        $selector = build_task_selector_for_project_in_weekly_view($task_selector_params, $genid);
    } else {
        $selector = build_disabled_task_selector_for_project_in_weekly_view($genid);
    }
    $result .= "<td id='".$genid."task-selector-td' class='weekly--total-value'>" . $selector . "</td>";
    $result .= "<td style='".$add_button_style."'><button id='".$genid."button-add-row-weekly-view' class='btn btn-xs btn-primary' type='button' onclick='og.addTaskRowToWeeklyList()'>".lang('add row')."</button></td>";
    $result .= "</tr>";

    return $result;
}


function build_total_row_for_weekly_view($weekly_data, $date_range, $genid){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $result = "";
    $result .= "<tr class='weekly--total-row'>";
    $result .= "<td class='weekly--total-value'>" . lang('total') . "</td>";
    if(!$has_project_selected){
        $result .= "<td class='weekly--total-value'></td>";
    }
    foreach($date_range as $date){
        $weekday = strtolower($date->format("D"));
        $day_total = $weekly_data['day_totals'][$weekday];
        $formatted_day_total = convert_num_to_hours_mins($day_total);
        $day_total_id = $genid . '-' . 'day_total' . '-' . $user_id . '-' . $weekday;
        $result .= "<td id='".$day_total_id."' class='weekly--total-value'>" . $formatted_day_total . "</td>";
    }
    $grand_total = $weekly_data['weekly_total'];
    $formatted_grand_total = convert_num_to_hours_mins($grand_total);
    $id_grand_total = $genid . '-' . 'grand_total' . '-' . $user_id;
    $result .= "<td id='".$id_grand_total."' class='weekly--total-value'>" . $formatted_grand_total . "</td>";
    $result .= "</tr>";

    return $result;
}


function convert_num_to_hours_mins($time, $format = '%02d:%02d') {
    $hours = floor($time / 60);
    $minutes = ($time % 60);
    return sprintf($format, $hours, $minutes);
}

// Draw time grid on page load
function get_time_input_and_desc_html($task_info, $task_id, $date, $genid){
    $weekday = strtolower($date->format("D"));
    $project_id = $task_info['project_id'];
    $fullday = $date->format("Y_m_d");
    $time = $task_info['weekly_timeslots'][$weekday]['task_day_total'];
    $formatted_time = convert_num_to_hours_mins($time);
    $time_input_disabled = '';
    $desc_data_toggle = 'data-toggle="popover"';
    $time_input_id = '';
    $desc_input_id = '';
    $desc_color = "#7C7C7C"; // 'Active' color
    $description_value = '';
    $popover_id = $genid . '-' . 'popover' . '-' . $project_id . '-' . $task_id . '-' . $fullday;
    $button_add_time_popover = '<button class=\'btn btn-xs btn-primary x-tool x-tool-options add-weekly-time-btn\' onclick=\'og.addWeeklyTimeToPopover(`'.$popover_id.'`);\' type=\'button\'>Add worked hours</button>';
    $onclick = '';
    if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 1){
        $timeslot_id = $task_info['weekly_timeslots'][$weekday]['timeslots'][0]['timeslot_id'];
        $description_value = $task_info['weekly_timeslots'][$weekday]['timeslots'][0]['description'];
        $time_input_id = $genid . '-' . 'edit'  . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . $timeslot_id;
        $desc_input_id = $genid . '-description-edit-' . $project_id  . '-' . $task_id . '-' . $fullday . '-' .$timeslot_id;
    } else if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) > 1){
        $time_input_disabled = 'readonly';
        $time_input_id = $genid . '-' . 'multiple_time' . '-' . $project_id . '-' . $task_id . '-' . $fullday;
        $onclick =" onclick='og.loadTimeInputPopover(".'"'.$time_input_id.'"'.")'";
    }else if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 0) {
        $time_input_id = $genid . '-' . 'new' . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . 0;
        $desc_input_id = $genid . '-decription-new-' . $project_id  . '-' . $task_id . '-' . $fullday . '-' . 0;
        $desc_data_toggle = '';
        $desc_color = "#CCC"; // Disabled color
    }

    // New version time input
    $onchange_func = 'og.updateTimeFromWeekly(this)';
    $class_input = "weekly--time-input";
    if($formatted_time == '00:00'){
        $class_input = "weekly--time-input-grey";
    }

    // Send input_id to generate button_id and simulate click 
    $time_input = "<input ".$onclick." data-toggle='popover' data-content='' class='".$class_input."' ".$time_input_disabled." id='".$time_input_id."' type='text' value='".$formatted_time."' prevvalue='".$formatted_time."' onchange='".$onchange_func."' onfocus='og.closePopovers();og.clearTime(this);' onblur='og.checkTimeValue(this)' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";

    /*
    * Old version time input
    *
    $onchange_func = 'og.updateTimeFromWeekly(this)';
    $time_input = "<input class='weekly--time-input' ".$time_input_disabled." id='".$time_input_id."' type='text' value='".$formatted_time."' prevvalue='".$formatted_time."' onchange='".$onchange_func."' onfocus='og.closePopovers();' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";
    *
    */
    if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) > 1){
        $svg_lines = true;
        $day_timeslots = $task_info['weekly_timeslots'][$weekday]['timeslots'];
        $description_input = '<div>';
        $description_input .= '<div class=\'popover-timeslots\'>';
        foreach($day_timeslots as $day_timeslot){
            $description_input .= '<div class=\'weekly--popover-time-desc-div\'>';
            // Add time input for each timeslots
            $day_formatted_time = convert_num_to_hours_mins($day_timeslot['worked_time']);
            $day_time_input_id = $genid . '-' . 'edit' . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . $day_timeslot['timeslot_id'];
            $day_time_onchange_func = 'og.updateTimeFromWeeklyDescDiv(this)';
            $description_input .= "<input class='weekly--time-input' id='".$day_time_input_id."' type='text' value='".$day_formatted_time."' prevvalue='".$day_formatted_time."' onchange='".$day_time_onchange_func."' onfocus='og.clearTime(this);' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";
            $description_input .= "<a class='list-action ico-delete weekly--delete-time-btn' href='#' onclick='og.deleteTimeEntryWeeklyDescPopover(`".$day_time_input_id."`)';>&nbsp;</a>";
            // Add description input for each timeslot
            $day_desc_input_id = $genid . '-description-edit-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$day_timeslot['timeslot_id'];
            $day_desc_onchange_func = 'og.updateDescFromWeeklyDescDiv(this)';
            $day_description_value = $day_timeslot['description'];
            if($day_description_value == ''){
                $description_input .= '<div><textarea id=\''.$day_desc_input_id.'\' onchange='.$day_desc_onchange_func.' placeholder=\''.lang('description').'...\' class=\'weekly--description-textarea\'></textarea></div>';
            } else {
                $qtty_desc ++;
                $description_input .= '<div><textarea id=\''.$day_desc_input_id.'\' onchange='.$day_desc_onchange_func.' class=\'weekly--description-textarea\'>'.$day_description_value.'</textarea></div>';
            }
            $description_input .= '</div>';

        }
        $description_input .= '</div>';
        $description_input .= $button_add_time_popover;
        $description_input .= '</div>';
        if(count($day_timeslots) == $qtty_desc){
            $desc_color = '#22CD92'; // 'Has description' color
        }else{
            $desc_color = "#7C7C7C"; // 'Active' color
        }
    } else {
        $svg_lines = false;
        $desc_onchange_func = 'og.updateDescFromWeekly(this)';
        if($description_value == ''){
            $description_input = '<div>';

            $description_input .= '<div class=\'popover-timeslots\'>';
            $description_input .= '<div><textarea id=\''.$desc_input_id.'\' onchange='.$desc_onchange_func.' placeholder=\''.lang('description').'...\' class=\'weekly--description-textarea\'></textarea></div>';
            $description_input .= '</div>';

            $description_input .= $button_add_time_popover;

            $description_input .= '</div>';
        } else {
            $description_input = '<div>';

            $description_input .= '<div class=\'popover-timeslots\'>';
            $description_input .= '<div><textarea id=\''.$desc_input_id.'\' onchange='.$desc_onchange_func.' class=\'weekly--description-textarea\'>'.$description_value.'</textarea></div>';
            $description_input .= '</div>';

            $description_input .= $button_add_time_popover;
            
            $description_input .= '</div>';
            if($desc_data_toggle != ''){
                $desc_color = '#22CD92';
            }
        }
    }

     
     // Plus svg by default
    $svg_icon='<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 530 530" 
    fill="none" xmlns="http://www.w3.org/2000/svg">
    <g transform="translate(0,505) scale(0.100000,-0.100000)"
    fill="'.$desc_color.'">
    <path d="M1701 5098 c-36 -26 -1095 -1208 -1105 -1233 -11 -31 -7 -3782 5
    -3808 6 -13 23 -31 39 -40 26 -16 115 -17 1065 -17 1151 0 1082 -4 1111 67 20
    46 10 86 -28 118 l-30 25 -979 0 -979 0 0 1710 0 1710 500 0 500 0 32 29 33
    29 3 611 2 611 1170 0 1170 0 0 -1501 c0 -1664 -5 -1550 65 -1583 41 -20 49
    -20 90 0 70 33 65 -87 65 1638 0 1323 -2 1562 -14 1591 -29 70 67 65 -1379 65
    l-1304 0 -32 -22z m-54 -1255 c-2 -2 -181 -2 -397 -1 l-394 3 395 440 394 440
    3 -439 c1 -242 1 -441 -1 -443z"/>
    <path d="M3855 1381 c-11 -5 -29 -19 -40 -31 -18 -20 -20 -40 -25 -259 l-5
    -236 -237 -5 c-230 -5 -237 -6 -261 -28 -32 -30 -40 -74 -23 -117 25 -60 48
    -65 301 -65 l225 0 0 -236 0 -236 29 -29 c33 -33 83 -39 130 -15 47 25 51 49
    51 291 l0 225 225 0 c246 0 270 5 293 55 21 47 15 94 -17 126 l-29 29 -236 0
    -236 0 0 225 c0 253 -5 275 -65 301 -38 16 -51 16 -80 5z"/>
    </g>
    </svg>';
    if($svg_lines){
        $svg_icon = '<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 1000 1000" 
            fill="none" xmlns="http://www.w3.org/2000/svg">
            <g transform="translate(0,965) scale(0.100000,-0.100000)"
            fill="'.$desc_color.'">
            <path stroke-width="2" d="M2848 9790 c-385 -65 -679 -380 -714 -766 l-7 -74 215 0 215 0 6 56
            c13 130 107 258 236 322 l75 37 1973 3 1972 2 3 -1032 c3 -954 5 -1038 21
            -1092 47 -158 101 -255 201 -363 117 -125 253 -208 421 -255 80 -22 84 -22
            993 -26 l912 -3 0 -2273 c0 -2206 -1 -2275 -19 -2328 -54 -157 -188 -267 -353
            -288 -40 -5 -259 -10 -485 -10 l-413 0 0 -211 0 -210 498 3 c549 4 548 4 698
            73 232 106 392 290 471 542 l28 88 3 2414 2 2415 -36 37 c-33 34 -36 42 -31
            78 l5 39 -1315 1416 -1315 1416 -2107 -1 c-1158 -1 -2127 -5 -2153 -9z m5333
            -1770 l923 -985 -739 -3 c-639 -2 -747 0 -800 13 -160 41 -284 174 -314 339
            -13 73 -16 1629 -2 1624 5 -1 424 -446 932 -988z"/>
            <path stroke-width="2" d="M680 8504 c-351 -75 -620 -361 -670 -712 -8 -55 -10 -1082 -8 -3582
            l3 -3505 22 -72 c50 -158 118 -275 225 -381 108 -109 229 -179 386 -225 l77
            -22 3120 0 3120 0 78 23 c306 88 513 293 609 604 l23 73 3 2414 2 2415 -36 37
            c-33 34 -36 42 -31 78 l5 41 -1314 1415 -1314 1415 -2118 -1 c-1771 0 -2128
            -3 -2182 -15z m4010 -1449 l5 -1040 29 -85 c15 -47 40 -107 53 -135 87 -173
            254 -328 436 -403 150 -63 136 -62 1127 -62 l900 0 -2 -2293 -3 -2293 -37 -75
            c-46 -93 -112 -158 -206 -202 l-67 -32 -3090 0 -3090 0 -66 31 c-114 54 -190
            138 -228 252 l-21 62 2 3502 3 3503 32 66 c59 121 185 216 313 238 32 5 870 8
            1980 7 l1925 -1 5 -1040z m1389 -342 c490 -524 891 -955 891 -958 0 -3 -329
            -5 -731 -5 -821 0 -819 0 -926 69 -104 67 -170 165 -192 285 -7 40 -11 321
            -11 850 l0 791 39 -40 c21 -22 439 -468 930 -992z"/>
            <path stroke-width="2" d="M1492 6178 l3 -213 1063 -3 1062 -2 0 215 0 215 -1065 0 -1065 0 2
            -212z"/>
            <path stroke-width="2" d="M1490 4900 l0 -210 1065 0 1065 0 0 210 0 210 -1065 0 -1065 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 3620 l0 -210 2345 0 2345 0 0 210 0 210 -2345 0 -2345 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 2345 l0 -215 2345 0 2345 0 -2 213 -3 212 -2342 3 -2343 2 0
            -215z"/>
            </g>
            </svg>';
    }

    $popover_time_input_id = 'timeId="'.$time_input_id.'"';
    $description_button = '
        <div id="'.$popover_id.'" class="weekly--popover-div" '.$popover_time_input_id.' onclick="og.closeOtherPopover(this);">
        <button id="'.$popover_id.'_btn" class="weekly--desc-button" '.$desc_data_toggle.' data-content="'.$description_input.'" data-placement="bottom">
        '.$svg_icon.'
        </button>
        </div>';

    $result = array(
        'time_input' => $time_input,
        'description_button' => $description_button,
        'popover_id' => $popover_id,
        'description_content' => $description_input,
        'time_input_id' => $time_input_id,
        'formatted_time_value' => $formatted_time,
        'desc_color' => $desc_color
    );
    return $result;

}

function build_task_selector_for_project_in_weekly_view($task_selector_params, $genid){
    $project_member_id = array_var($task_selector_params, 'project_member_id');
    $user_id = array_var($task_selector_params, 'user_id', 0);
    $selected_task_ids = array_var($task_selector_params, 'selected_task_ids', array());

    $tasks_res = ProjectTasks::instance()->listing(array(
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'ignore_context' => false,
        'member_ids' => array($project_member_id),
        'extra_conditions' => ' AND completed_by_id = 0 '
    ));
    $tasks = $tasks_res->objects;

    $selector = '<select id="'.$genid.'-task-selector-weekly-view" class="time-weekly--task-selector">';
    $selector .= '<option value="0">'.lang('select a task').'</option>';
    $assigned_options = '<optgroup label="'.lang('assigned to the user').'">';
    $not_assigned_options = '<optgroup label="'.lang('not assigned to the user').'">';
    
    foreach($tasks as $task){
        $task_id = $task->getObjectId();
        if(in_array($task_id, $selected_task_ids)) continue;
        $task_name = $task->getName();
        $task_user_id = $task->getAssignedToContactId();
        $task_option = '<option value="' . $task_id . '">' . $task_name . '</option>';
        if($task_user_id == $user_id){
            $assigned_options .= $task_option;
        } else {
            $not_assigned_options .= $task_option;
        }
    }

    $selector .= $assigned_options . '</optgroup>';
    $selector .= $not_assigned_options . '</optgroup>';
    $selector .= '</select>';

    return $selector;

}

function build_disabled_task_selector_for_project_in_weekly_view($genid){
    $selector = '<select id="'.$genid.'-task-selector-weekly-view" class="time-weekly--task-selector" disabled>
                        <option value="0">'.lang('select a task').'</option>
                    </select>';

    return $selector;
}

function get_user_name_from_weekly_data($weekly_data){
    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $user = Contacts::findById($user_id);
    $user_name = $user->getName();
    
    return $user_name;
}
