<?php
	class CObjTemplate extends BaseCObjTemplate {

		private $object_type_manager = null;
		
		
		function delete() {

			$this->deleteAllObjects();

			// delete each template parameter
			$params = CObjTemplateParameters::instance()->findAll(
				array("conditions" => array("template_id=?", $this->getId()))
			);
			foreach ($params as $param) {
				$param->delete();
			}

			// delete the template
			return parent::delete();
		}


		function deleteAllObjects() {

			$objects = CObjTemplateObjects::instance()->findAll(
				array("conditions" => array("template_id=?", $this->getId()))
			);
			// delete each template object with its properties, lines, and variables
			foreach ($objects as $object) {
				$object->delete();
			}
			
		}


		function deleteAllParameters() {

			$params = CObjTemplateParameters::instance()->findAll(
				array("conditions" => array("template_id=?", $this->getId()))
			);
			// delete each template parameter
			foreach ($params as $param) {
				$param->delete();
			}
			
		}


		function deleteAllVariables() {

			return CObjTemplateObjectVariableValues::instance()->deleteTemplateVariables($this->getId());

		}

		function getVariablesData() {

			$data = array();
			$variables = CObjTemplateObjectVariableValues::instance()->getTemplateVariables($this->getId());

			foreach ($variables as $variable) {
				$data[] = $variable->getParsedData($this);
			}

			return $data;
		}

		function getPropertiesForVariables() {

			$manager = $this->getObjectTypeManager();

			$allowed_column_types = array(DATA_TYPE_STRING, DATA_TYPE_DATE, DATA_TYPE_DATETIME);
			$excluded_columns = array('invoicing_status');

			$columns = $manager->getColumns();

			$properties = array();
			$properties[lang('name')] = array(
				'property' => "name",
				'type' => DATA_TYPE_STRING,
				'text' => lang('name'),
			);

			foreach ($columns as $col) {
				$col_type = $manager->getColumnType($col);
				if (in_array($col_type, $allowed_column_types) && !in_array($col, $excluded_columns)) {

					$col_text = Localization::instance()->lang("field ".get_class($manager)." $col");
					if (!$col_text) $col_text = Localization::instance()->lang("field Objects $col");
					if (!$col_text) $col_text = Localization::instance()->lang($col);
					if (!$col_text) $col_text = $col;

					$properties[$col_text] = array(
						'property' => $col,
						'type' => $col_type,
						'text' => $col_text,
					);
				}
			}

			ksort($properties);

			return array_values($properties);
		}


		function getObjectType() {
			$object_type = ObjectTypes::findById($this->getObjectTypeId());

			return $object_type;
		}

		function getObjectTypeManager() {
			if ($this->object_type_manager) {
				return $this->object_type_manager;
			}

			$object_type = $this->getObjectType();
			$manager_cls = $object_type->getHandlerClass();
			$manager = new $manager_cls();

			$this->object_type_manager = $manager;

			return $manager;
		}


		function getTemplateObjectAddUrl($modal = true) {

			$ot = $this->getObjectType();
			$action = 'add';

			if ($modal) {

				$url = "og.render_modal_form('', {c:'".$ot->getName()."', a:'$action', params: {is_template:1, template_ot_id:".$ot->getId().", idx:obj_index, data:obj_data}});";
			//$url = "og.render_modal_form('', {c:'c_obj_template', a:'add_template_object', params: {is_template:1, template_ot_id:".$ot->getId()."}});";

			} else {

				$url = get_url($ot->getName(), $action, array('is_template' => true, 'template_ot_id' => $ot->getId(), 'idx' => "obj_index", 'data' => "obj_data"));

			}

			return $url;

		}


		function getObjectsToInstantiate() {

			$template_objects = CObjTemplateObjects::getByTemplate($this->getId());

			$objects_data = array();
			foreach ($template_objects as $template_object) {
				$template_obj_data = $template_object->buildObjectData();
				$template_obj_data['obj_id'] = $template_object->getObjectId();

				$objects_data[] = $template_obj_data;
			}

			return $objects_data;

		}
		
	}
	 
