<?php 
	class PropertyGroups extends BasePropertyGroups {
		
		static function getAllGroupedByObjectType() {
			$properties = self::findAll(array(
				"order" => "object_type_id, `order`"
			));
			
			$grouped = array();
			foreach ($properties as $p) {
				if (!isset($grouped[$p->getObjectTypeId()])) {
					$grouped[$p->getObjectTypeId()] = array();
				}
				$grouped[$p->getObjectTypeId()][] = $p;
			}
			
			return $grouped;
		}
		
		static function getPropertyOrderInGroup($object_type, $property_id) {
			$current_pg = PropertyGroups::getByObjectTypeAndProperty($object_type->getId(), $property_id);
			$order = 9999;
			if ($current_pg instanceof PropertyGroup) {
				$pgp = PropertyGroupProperties::getByGroupAndProperty($current_pg->getId(), $property_id);
				if ($pgp instanceof PropertyGroupProperty) {
					$order = $pgp->getOrder();
				}
			}
			
			return $order;
		}

		static function getPropertyIsInheritable($object_type, $property_id) { 
			$current_pg = PropertyGroups::getByObjectTypeAndProperty($object_type->getId(), $property_id);
			$is_inheritable = 0;
			if ($current_pg instanceof PropertyGroup) {
				$pgp = PropertyGroupProperties::getByGroupAndProperty($current_pg->getId(), $property_id);
				if ($pgp instanceof PropertyGroupProperty) {
					$is_inheritable = $pgp->getIsInheritable();
				}
			}
			
			return $is_inheritable;
		}

		static function getPropertyGroupPropertyByObjectTypeAndPropertyId($object_type_id, $property_id) { 
			$current_pg = PropertyGroups::getByObjectTypeAndProperty($object_type_id, $property_id);
			$pgp = null;
			if ($current_pg instanceof PropertyGroup) {
				$pgp = PropertyGroupProperties::getByGroupAndProperty($current_pg->getId(), $property_id);
			} 

			return $pgp;
		}
		
		
		static function getByObjectType($object_type_id) {
			return self::findAll(array("conditions" => array("object_type_id = ?", $object_type_id), "order" => "`order`"));
		}
		
		static function getByObjectTypeAndProperty($object_type_id, $property_id) {
			
			return self::findOne(array(
				"conditions" => array(
					"object_type_id = ? AND EXISTS(
						SELECT group_id FROM ".TABLE_PREFIX."property_group_properties WHERE group_id=id AND property_id=?
					)", $object_type_id, $property_id
				)	
			));
		}

        /**
         * Returns all properties that an object type has, grouped by the property group
         *
         * @param $object_type
         * @return array
         *
         */
        static function getAllPropertiesGroupedByPropertyGroup($object_type_id, $visibility='all', $include_disabled=false) {
            $groups = PropertyGroups::getByObjectType($object_type_id);
            $groups_properties = PropertyGroupProperties::getAllGroupedByGroup($object_type_id);

            $groups_array = array();
			$first = true;
            
            foreach ($groups as $group){
                $group_properties = array();
				if($first){
					Hook::fire('get_additional_group_property_to_the_view', array('object_type_id' => $object_type_id), $group_properties);
				}

                if (isset($groups_properties[$group->getId()])) {
                    foreach ($groups_properties[$group->getId()] as $gp){
                        if ($gp instanceof PropertyGroupProperty) {

                            $group_properties[] = array(
                                'property_id' => $gp->getPropertyId(),
                                'order' => $gp->getOrder()
                            );
                        }
                    }
                }

                $groups_array[] = array(
                    'id' => $group->getId(),
                    'name' => $group->getName(),
                	'code' => $group->getCode(),
                	'order' => $group->getOrder(),
                	'alignment' => $group->getAlignment(),
                    'is_system' => $group->getIsSystem(),
                    'properties' => $group_properties
                );
				$first = false;

            }
            
            return $groups_array;
        } //  getAllPropertiesGroupedByPropertyGroup
	}
 