<?php

Hook::register('advanced_core');


function advanced_core_object_definition($type, &$ret) {
	if ($type == 'SystemPermission' || $type == 'MaxSystemPermission') {
		$ret['can_add_timeslots_in_time_tab'] = DATA_TYPE_BOOLEAN;
	}
}


function advanced_core_validate_reminder_minutes($params, &$minutes) {
	$context = array_var($params, 'context');
	$i = array_var($params, 'i');
	$request = array_var($params, 'request');
	
	$duration_before_afterC = array_var($_POST, 'reminder_before_after');
	$duration_before_after = array_var($duration_before_afterC, $context);
	
	if (array_var($duration_before_after, $i) == 'after') {
		$minutes = -1 * $minutes;
	}
}

function advanced_core_time_tab_override_can_add_timeslots_permission($params, &$can_add_timeslots) {
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_add_timeslots_in_time_tab')) {
		$can_add_timeslots = false;
	}
}

function advanced_core_view_document_add_actions($params, &$null) {
	$file = array_var($params, 'file');
	
	if ($file instanceof ProjectFile && Plugins::instance()->isActivePlugin('mail')) {
		
		$action_url = get_url('file_actions', 'send_file_by_email', array('id' => $file->getId()));
		
		add_page_action(lang('attach to email'), $action_url, 'ico-sent', null, null, true);
		
	}

	if ($file instanceof ProjectFile){
		// Send file by email option
		add_page_action(lang('send by email'), "javascript:og.render_modal_form('', {c:'file_actions', a:'send_file_by_notification_email', params: {id:".$file->getId()."}});", 'ico-attachment', null, null, true);
		
	}
}


function advanced_core_render_administration_icons($ignored, &$links) {
	if (logged_user()->isAdministrator()) {
		$show_tool = defined('MEMBER_MIGRATION_TOOL') && MEMBER_MIGRATION_TOOL;
		if ($show_tool) {
			$links[] = array(
				'ico' => 'ico-large-workspace',
				'url' => get_url('dimension_management', 'change_members_dimension_params'),
				'name' => lang('change members dimension'),
				'extra' => '',
			);
		}
	}
}

/**
 * Add the copy member button to the active context widget
 * @param Member $member
 * @param string $edit_onclick
 */
function advanced_core_more_active_context_widget_buttons($params, &$buttons_html) {
	$member = array_var($params, 'member');
	if (!$member instanceof Member) return;
	
	$allowed_ots_to_copy = array("managed_event");
	$object_type = ObjectTypes::findById($member->getObjectTypeId());
	
	if (in_array($object_type->getName(), $allowed_ots_to_copy)) {
		$onclick = "og.advanced_core.copy_member(".$member->getId().");";
		$title = lang('copy');
		$label = lang('copy');
		
		$buttons_html .= '
			<button class="btn" style="margin-left:10px;" onclick="'. $onclick .'" title="'. $title .'">
                <img alt="" style="height: 15px" src="public/assets/themes/default/images/icons-feng-3/16x16/copy.png">
                '. $label .'
            </button>
		';
	}
}


function advanced_core_modify_subtasks_member_ids($params, &$member_ids) {
	$task = array_var($params, 'task');
	
	if (is_array($member_ids) && count($member_ids) > 0) {
		$ignored_dimension_ids = config_option('ignored_dimensions_for_subtasks');
		
		$members = Members::findAll(array('conditions' => "id IN (" . implode(',', $member_ids) . ")"));
		
		$new_member_ids = array();
		foreach ($members as $member) {
			if (!in_array($member->getDimensionId(), $ignored_dimension_ids)) {
				$new_member_ids[] = $member->getId();
			}
		}
		$member_ids = $new_member_ids;
	}
	
}


function advanced_core_get_prevent_adding_time_to_parent($params, &$ret){

	$is_parent_task = $params['is_parent'];
	$time_type = $params['time_type'];
	
    if(!$is_parent_task || config_option('allow_adding_'. $time_type .'_time_to_parent_task')){
        $ret = false;
    } else {
        $ret = true;
    } 
    
}


function advanced_core_task_info_additional_data($task, &$info) {

	$project_task = ProjectTasks::findById($task['id']);
	$prevent_add_time_to_parent = config_option('allow_adding_worked_time_to_parent_task') == false;

	$info['prevent_add_time_to_parent_task'] = $prevent_add_time_to_parent;
	$info['is_parent'] = $project_task->isParent();

}


function advanced_core_custom_reports_additional_columns($args, &$ret) {
	$ot = array_var($args, 'object_type');

	if ($ot instanceof ObjectType) {
		$ret[] = array(
			'id' => 'project_number',
			'name' => lang('field Objects project_number'),
			'type' => 'text'
		);
	}
}


function advanced_core_custom_reports_fixed_additional_columns_def($args, &$results) {
	$ot = array_var($args, 'object_type');
	$field = array_var($args, 'field');
	
	if ($ot instanceof ObjectType) {
		if ($field == "project_number") {
			$results['columns']['names'][$field] = lang('field Objects project_number');
			$results['columns']['types'][$field] = 'text';
			$results['columns']['order'][] = $field;
		}
	}
}


function advanced_core_custom_reports_override_column_format($parameters, &$value) {
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($field == "project_number") {

		$project_member = null;
		$project_ot = ObjectTypes::findByName('project');
		$payment_members = $object->getMembers();
		
		foreach ($payment_members as $member) {
			if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
				$project_member = $member;
				break;
			}
		}

		if ($project_member instanceof Member) {
			$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_id");
			$tmp_project_obj_id = $project_member->getObjectId();

			if ($project_id_cp instanceof CustomProperty) {
				$cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $project_id_cp->getId());
				if ($cpval instanceof CustomPropertyValue) {
					$value = $cpval->getValue();
				}
			}
		}
	}

	return;
}

