<?php

Hook::register('cobj_templates');

/**
 * Adds a new section in the templates configuration list for each ContentDataObject type that is allowed to have templates
 */
function cobj_templates_render_more_type_templates($parameters, &$ret) {
	
	$available_types = CObjTemplates::getAvailableTypes();
	foreach ($available_types as $object_type) {
		
		$custom_ot_name = $object_type->getPluralObjectTypeName();

		$templates = CObjTemplates::getObjectTypeTemplates($object_type->getId());
		
		
		tpl_assign('object_type', $object_type);
		tpl_assign('object_type_name', $custom_ot_name);
		tpl_assign('templates', $templates);
		
		tpl_display(get_template_path('list_by_type', 'c_obj_template', 'advanced_core'));
		
	}
}

/**
 * Overrides the form endpoint so we can manage the request in the CObjTemplateController 
 * and build our template objects instead of saving a new content object
 */
function cobj_templates_override_form_action($parameters, &$form_action) {

	$request = array_var($parameters, 'request');

	if (array_var($request, 'is_template')) {

		$object = array_var($parameters, 'object');
	
		$available_types = CObjTemplates::getAvailableTypeNames();
		if (in_array($object->getObjectTypeName(), $available_types)) {

			$form_action = get_url('c_obj_template', 'generate_template_object_data', array(
				'idx' => array_var($request,'idx'),
				'template_ot_id' => array_var($request,'template_ot_id'),
			));

		}
	}

}

/**
 * Builds a ContentDataObject from the data sent by the form, so we can use it as if it were a real object in its add/edit form
 */
function cobj_templates_override_object_data($parameters, &$object) {

	$request = array_var($parameters, 'request');

	if (array_var($request, 'is_template') && array_var($request, 'data') != '') {
	
		$available_types = CObjTemplates::getAvailableTypeNames();
		if (in_array($object->getObjectTypeName(), $available_types)) {
			$encoded_data = array_var($request, 'data');
			$data = json_decode(base64_decode($encoded_data), true);

			$object_type = ObjectTypes::findById($object->getObjectTypeId());

			$object = CObjTemplateObjects::buildObjectFromData($object_type, $data['properties']);
			
		}
	}

}

/**
 * Override custom property values to use in the form with the ones that have been cached in the built object
 */
function cobj_templates_custom_property_input_default_value($params, &$default_value){
	$object = array_var($params, 'object');
	$custom_prop = array_var($params, 'customProp');

	if ($custom_prop instanceof CustomProperty && $object instanceof ContentDataObject && $object->getUseCachedCustomProperties()) {
		$cp_values = $object->getCachedCustomProperties();
		
		$default_value = array_var($cp_values, $custom_prop->getId());
	}
}


function cobj_templates_member_templates_form_more_template_selectors($params, &$ignored) {

	$genid = array_var($params, 'genid');
	$member_template = array_var($params, 'member_template');
	$cobj_template_ids_csv = '';
	$selected_cobj_templates_info = array();

	if ($member_template->getId() > 0) {
		$reg = MemberTemplatesTaskTemplates::findOne(array('conditions' => "template_id = '".$member_template->getId()."' AND is_initial = 1 "));
		if ($reg) {
			$cobj_template_ids_csv = $reg->getCobjTemplateIds();
		}
	}

	if ($cobj_template_ids_csv) {
		$cobj_templates = CObjTemplates::findAll(array("conditions" => "id IN ($cobj_template_ids_csv)"));
		foreach ($cobj_templates as $cobj_template) {
			if (!isset($cobj_templates_by_type[$cobj_template->getObjectTypeId()])) {
				$cobj_templates_by_type[$cobj_template->getObjectTypeId()] = array();
			}
			$selected_cobj_templates_info[$cobj_template->getObjectTypeId()][] = array('id' => $cobj_template->getId(), 'name' => $cobj_template->getName());
		}
	}

	$cobj_templates_by_type = array();
	$all_cobj_templates = CObjTemplates::findAll();
	foreach ($all_cobj_templates as $cobj_template) {

		$object_type = $cobj_template->getObjectType();

		if (!isset($cobj_templates_by_type[$cobj_template->getObjectTypeId()])) {
			$cobj_templates_by_type[$cobj_template->getObjectTypeId()] = array();
		}
		$cobj_templates_by_type[$cobj_template->getObjectTypeId()][] = array(
			'id' => $cobj_template->getId(),
			'name' => $cobj_template->getName(),
			'ot_id' => $cobj_template->getObjectTypeId(),
			'ot_name' => $object_type->getObjectTypeName(),
		);
	}

	foreach ($cobj_templates_by_type as $ot_id => $cobj_templates_info) {

		tpl_assign('genid', $genid);
		tpl_assign('member_template', $member_template);
		tpl_assign('selected_cobj_templates_info', $selected_cobj_templates_info);
		tpl_assign('cobj_templates_info', $cobj_templates_info);

		tpl_display(get_template_path('selector_for_member_template', 'c_obj_template', 'advanced_core'));

	}
}

