<?php
Hook::register('custom_properties_definition');


function custom_properties_definition_object_definition($type, &$ret) {
	if ($type == 'CustomProperty' || $type == 'MemberCustomProperty') {
		$ret['total_operation'] = DATA_TYPE_STRING;
	}
}


function custom_properties_definition_custom_prop_numeric_options($ignored, &$html) {
	
	$label = lang("operation to do when calculating listing totals");
	$label_desc = "";
	
	$option_tags = array();
	$opts = array("do_nothing", "sum", "average");
	foreach ($opts as $opt) {
		$lang_key = str_replace("_", " ", $opt);
		if ($opt == "do_nothing") $opt = "";
		$option_tags[] = option_tag(lang($lang_key), $opt);
	}
	
	$attributes = array("id" => "total_operation", "style" => "width: auto; margin-left:5px;");
	
	$html = "<label title='$label_desc'>$label:</label>";
	$html .= select_box("custom_properties[{number}][total_operation]", $option_tags, $attributes);
}


function custom_properties_definition_additional_custom_property_fields($params, &$prop) {
	$cp = array_var($params, 'cp');
	$ot = array_var($params, 'ot');
	
	$prop['total_operation'] = $cp['total_operation'];
	
}


function custom_properties_definition_overrides_special_cp($params, &$prop){

	$override_is_required = ['Client', 'Clients', 'Rate schedules'];
	$override_is_multiple_values = ['Client', 'Clients', 'Rate schedules'];

	if(in_array($prop['name'], $override_is_required)) {
		$prop['override_is_required'] = true;
	}  

	if(in_array($prop['name'], $override_is_multiple_values)) {
		$prop['override_is_multiple_values'] = true;
	}  

}



function custom_properties_definition_member_listing_additional_data($params, &$object) {
	if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
		
		Env::useHelper('list_totals', 'advanced_core');
		
		$params['list_type'] = 'member';
		
		if (is_numeric(array_var($params, 'type_id')) && array_var($object, 'totalCount') > 0) {
			$totals = generate_listing_totals_from_query_strings($params);
			if ($totals) {
				$object_type = ObjectTypes::findById(array_var($params, 'type_id'));
				foreach ($totals as $column => &$total_value) {
					$val = get_fixed_column_total($object_type, $totals, $column);
					if ($val) $total_value = $val;
				}
				$object['totals'] = $totals;
			}
		}
	}
}

function custom_properties_definition_object_listing_additional_data($params, &$more_data) {
	
	Env::useHelper('list_totals', 'advanced_core');
 
	$params['list_type'] = 'object';
	
	if (is_numeric(array_var($params, 'type_id')) && array_var($params, 'totalCount') > 0) {
		$mail_ot = ObjectTypes::findByName('mail');
		if ($mail_ot instanceof ObjectType && $mail_ot->getId()==array_var($params, 'type_id')) {
			// dont make any calculations for mail listing
		} else {
			$totals = generate_listing_totals_from_query_strings($params);
			if ($totals) {
				// dont format the totals if it is a report execution, they are already formatted
				if(!in_array(array_var($_GET,'c'), array('reporting', 'excel_export'))) {
					$object_type = ObjectTypes::findById(array_var($params, 'type_id'));
					foreach ($totals as $column => &$total_value) {
						$val = get_fixed_column_total($object_type, $totals, $column);
						if ($val) $total_value = $val;
					}
				}
				if (!$more_data) $more_data = array();
				$more_data['totals'] = $totals;
			}
		}
	}
}


function custom_properties_definition_render_additional_report_rows($params, &$ignored) {
	$results = array_var($params, 'results');
	$report = array_var($params, 'report');
    $final_total = array_var($params, 'final_total');

	Env::useHelper('list_totals', 'advanced_core');
	
	$report_columns = $results['columns']['order'];
	
	$report_ot = null;
	if ($report instanceof Report) {
		$report_ot = ObjectTypes::findById($report->getReportObjectTypeId());
	}
	// check if the report is grouping by and accumulate each group totals to create the general total
	if (!isset($results['totals'])) {
		$group_totals = array_var($results, 'group_totals');
		if (is_array($group_totals)) {
			
			$first_level_totals = array_var($group_totals, 0);
			if (is_array($first_level_totals) && count($first_level_totals) > 0) {
				
				$general_totals_array = accumulate_total_rows($report_ot, $first_level_totals, $report);
				$total_text = lang('total');
				if (!$report->getColumnValue("hide_group_total_counts")) {
					$ot_lang = function_exists('mb_strtolower') ? mb_strtolower($report_ot->getPluralObjectTypeName()) : $report_ot->getPluralObjectTypeName();
					$total_text .= " (".array_var($general_totals_array, 'total_rows', 0)." ".$ot_lang.")";
				}
				$general_totals_array['name'] = $total_text;
				accummulate_custom_property_totals($report, $results, $general_totals_array);

                accummulate_timeslots_totals($report, $results, $general_totals_array,$final_total);


				$results['totals'] = $general_totals_array;
			}
		}
	}

	if (isset($results['totals'])) {
		
		echo "<tr class='list-totals-row'>";
		if (!$report->getColumnValue("hide_group_details")) {
			// empty cell for the link column, show only if details are displayed
			echo "<td class='textrow' style='min-width: 1px !important;width: 1px !important;'></td>";
		}
		foreach ($report_columns as $column) {
			if (is_numeric($column)) { // is cp
				echo "<td>". array_var($results['totals'], "cp_".$column, '') ."</td>";
			} else { // is fixed prop
				if ($column == 'link') continue;
				if ($column == 'name') {
					//echo "<td colspan='2' class='textrow'>".$results['totals']['name']."</td>";
					echo "<td class='textrow'>".$results['totals']['name']."</td>";
				} else {
					
					$formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $results['totals'], $column);
					
					echo "<td>". $formatted_total_value ."</td>";
				}
			}
		}
		
		if ($report_ot instanceof ObjectType) {
			if (in_array($report_ot->getType(), array('dimension_group', 'dimension_object'))) {
				echo "<td></td>"; // located under column
			}
		}
		
		echo "</tr>";
	}
}


function custom_properties_definition_get_additional_report_rows($params, &$val_array) {	
	$results = array_var($params, 'results');
	$report = array_var($params, 'report');
	
	Env::useHelper('list_totals', 'advanced_core');
	
	$report_columns = $results['columns']['order'];
	
	$report_ot = null;
	if ($report instanceof Report) {
		$report_ot = ObjectTypes::findById($report->getReportObjectTypeId());
	}
	
	// check if the report is grouping by and accumulate each group totals to create the general total
	if (!isset($results['totals'])) {
		$group_totals = array_var($results, 'group_totals');
		if (is_array($group_totals)) {
				
			$first_level_totals = array_var($group_totals, 0);
			if (is_array($first_level_totals) && count($first_level_totals) > 0) {
	
				$general_totals_array = accumulate_total_rows($report_ot, $first_level_totals, $report);
				$total_text = lang('total');
				if (!$report->getColumnValue("hide_group_total_counts")) {
					$ot_lang = function_exists('mb_strtolower') ? mb_strtolower($report_ot->getPluralObjectTypeName()) : $report_ot->getPluralObjectTypeName();
					$total_text .= " (".array_var($general_totals_array, 'total_rows', 0)." ".$ot_lang.")";
				}
				$general_totals_array['name'] = $total_text;
	
				accummulate_custom_property_totals($report, $results, $general_totals_array);
	
				$results['totals'] = $general_totals_array;
			}
		}
	}
	
	if (isset($results['totals'])) {

		$data = array();
		foreach ($report_columns as $column) {
			if (is_numeric($column)) {
				$data[] = array_var($results['totals'], "cp_".$column, '');
			} else {
				if ($column == 'link') continue;
				if ($column == 'name') {
					$data[] = $results['totals']['name'];
				} else {
					
					$formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $results['totals'], $column);
						
					$data[] = str_replace("<br/>", " - ", $formatted_total_value);
				}
			}
		}
		
		if ($report_ot instanceof ObjectType) {
			if (in_array($report_ot->getType(), array('dimension_group', 'dimension_object'))) {
				$data[] = ''; // located under column
			}
		}
		
		$val_array[] = $data;
	}
}



function custom_properties_definition_custom_report_group_totals($params, &$group_totals) {
	
	$report = array_var($params, 'report');
	$all_conditions = array_var($params, 'conditions');
	$order_sql_string = array_var($params, 'order');
	$from = array_var($params, 'from');
	$columns = array_var($params, 'columns');
    $group_by_criterias = array_var($params, 'group_by_criterias');


	$report_ot = ObjectTypes::findById($report->getReportObjectTypeId());
	$list_type = in_array($report_ot->getType(), array('dimension_object', 'dimension_group')) ? 'member' : 'object';
	
	Env::useHelper('list_totals', 'advanced_core');
	
	$gb_cols = array();
	$ord_exploded = explode(',', trim(str_replace("ORDER BY", "", $order_sql_string)));
	foreach ($ord_exploded as $oe) {
		if (str_starts_with(trim($oe), "_group")) {
			$oe = trim($oe);
			// if grouping by datetime custom property -> remove the time part to accumulate the total in the same day
			if (str_starts_with($oe, "_group_name_cp_")) {
				$cp_id = str_replace("_group_name_cp_", "", $oe);
				$cp = CustomProperties::findById($cp_id);
				if ($cp instanceof CustomProperty && $cp->getType() == 'datetime') {
					$oe = "DATE($oe)";
				}
			}
			$gb_cols[] = $oe;
		}
	}
	
	$group_totals = array();
	$level = 0;
	$max_level = count($gb_cols);
	
	while ($level < $max_level) {
	

        $group_sql_string = "";
        foreach ($group_by_criterias as $gb) {
            switch ($gb['type']) {
                case 'association': $gkey = '_group_id_a_'.$gb['id']; break;
                case 'custom_property': $gkey = '_group_id_cp_'.$gb['id']; break;
                case 'parent_member': $gkey = '_group_id_pm_'.$gb['id']; break;
                case 'classification': $gkey = '_group_id_dim_'.$gb['id']; break;
                case 'fixed_property': $gkey = '_group_id_fp_'.$gb['id']; break;
                case 'intersection': $gkey = '_group_id_inter_'.$gb['id']; break;
            }

            if (!$gkey) $gkey = "";
            $group_sql_string .= ($group_sql_string == "" ? "GROUP BY " : ", "). $gkey;
        }
		
		$result = build_cols_joins_sql_from_query_strings(array('type_id' => $report->getReportObjectTypeId(), 'list_type' => $list_type, 'report' => $report));
		
		$add_cols = array_filter(array_var($result, 'cols_sql'));
		$add_joins = array_filter(array_var($result, 'cp_joins_sql'));
		$add_group_by = array_filter(array_var($result, 'group_by_sql'));
		
		
		$hook_result = array();
		Hook::fire('modify_group_totals_sql', array('object_type' => $report_ot), $hook_result);
		if (isset($hook_result['cols_sql'])) {
			$add_cols = array_merge($add_cols, $hook_result['cols_sql']);
		}
		if (isset($hook_result['join_sql'])) {
			$add_joins = array_merge($add_joins, $hook_result['join_sql']);
		}
		if (isset($hook_result['gb_sql'])) {
			$add_group_by = array_merge($add_group_by, $hook_result['gb_sql']);
		}

		foreach ($add_group_by as $add_gb) {
			$group_sql_string .= ($group_sql_string == "" ? "GROUP BY " : ", "). $add_gb;
		}
		
		$additional_joins_sql = "";
		Hook::fire('aditional_join_listing_total_query', array('object_type' => $report_ot, 'list_type' => $list_type), $additional_joins_sql);

		Hook::fire('additional_totals_column', array('object_type' => $report_ot), $columns);

		if (count($add_cols) > 0) {
			$add_cols_sql = implode(',', $add_cols);
			$add_joins_sql = implode(' ', $add_joins) . $additional_joins_sql;
			
			if ($list_type == 'member') {
				$add_cols_sql = str_replace('mem.id', 'm.id', $add_cols_sql);
				$add_joins_sql = str_replace('mem.id', 'm.id', $add_joins_sql);
			}
			
			$group_sql = "
				SELECT $columns,
					$add_cols_sql
				FROM $from $add_joins_sql
				WHERE $all_conditions
				$group_sql_string
			";

			$level_gr_totals = DB::executeAll($group_sql);
			
			$more_rep_group_hook_params = array_merge($params, array(
				'object_type' => $report_ot,
				'level' => $level,
				'sql_cols' => "$columns, $add_cols_sql",
				'sql_from' => "$from $add_joins_sql",
				'sql_where' => "$all_conditions",
				'sql_gb' => "$group_sql_string",
                'report' => $report
			));
			
			Hook::fire('more_report_group_totals', $more_rep_group_hook_params, $level_gr_totals);
			
			Hook::fire('modify_group_totals_format', array('object_type' => $report_ot), $level_gr_totals);
			
			modify_group_totals_date_format($level_gr_totals);


			$group_totals[$level] = $level_gr_totals;
		}
		
		$level++;
	}
}

/**
 * Format the total if is a date
 * @param array $totals
 */
function modify_group_totals_date_format(&$totals){
    foreach ($totals as $key => $value) {
        foreach ($value as $k => $v) {
            if(validateDate($v)){
               $totals[$key][$k]= format_date(DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $v),null,0);
            }
        }
    }
}

function validateDate($date, $format = 'Y-m-d')
{
    $d = DateTime::createFromFormat($format, $date);
    return $d && $d->format($format) == $date;
}


function custom_properties_definition_render_additional_report_group_rows($params, &$ignored) {
	$results = array_var($params, 'results');
	$report = array_var($params, 'report');
	$group = array_var($params, 'group');
	$level = array_var($params, 'level');


	Env::useHelper('list_totals', 'advanced_core');
	
	$report_ot = ObjectTypes::findById($report->getReportObjectTypeId());

	$report_columns = ReportColumns::getAllReportColumns($report->getId());
	if (isset($results['group_totals']) && isset($results['group_totals'][$level])) {
		
		// dont show the last group if it is shown as columns
		if ($report->getColumnValue('show_last_group_as_column') && $level == max(array_keys($results['group_totals']))) {
			return;
		}
		
		$all_gtotal = array();
		$g_id_keys = explode(',', $group['original_gkey']);
		$exploded_g_ids = explode('_', $group['id']);
		
		// match group keys
		foreach ($results['group_totals'][$level] as $gt) {
			$all_match = true;
			$i = 0;
			foreach ($g_id_keys as $g_id_key) {

				$temp_group_id = $exploded_g_ids[$i];
				$group_total_key = $gt[$g_id_key];
				// for datetime cps remove the time part to group by
				if (str_starts_with($g_id_key, "_group_id_cp_")) {
					$cp_id = str_replace("_group_id_cp_", "", $g_id_key);
					$cp = CustomProperties::findById($cp_id);
					
					if ($cp instanceof CustomProperty && $cp->getType() == 'datetime') {
						$temp_group_id = gmdate('Y-m-d', strtotime($exploded_g_ids[$i]));
						$group_total_key = gmdate('Y-m-d', strtotime($gt[$g_id_key]));
					}
				}
				
				if ($temp_group_id != $group_total_key) {
					$all_match = false;
				}
				$i++;
				if (!$all_match) break;
			}
			if ($all_match) {
				$all_gtotal[] = $gt;
			}
		}

		$gtotal = accumulate_total_rows($report_ot, $all_gtotal, $report);
		if (count($gtotal) >= 0) {
			$ot_lang = function_exists('mb_strtolower') ? mb_strtolower($report_ot->getPluralObjectTypeName()) : $report_ot->getPluralObjectTypeName();
			
			$additional_class = "";

			$total_text = lang('total in', $group['name']);
			if (!$report->getColumnValue("hide_group_total_counts")) {
				$total_text .= " (".array_var($gtotal, 'total_rows', 0)." ".$ot_lang.")";
			}
			
			if ($report->getColumnValue("hide_group_details")) {
				$total_text = $group['name'];
				if (!$report->getColumnValue("hide_group_total_counts")) {
					$total_text .= " (".array_var($gtotal, 'total_rows', 0)." ".$ot_lang.")";
				}
				$additional_class = "simple-row";
			}
			
			$l = (3 - count($results['group_by_criterias'])) + $level;
			if ($l > 2) $l = 2;
			
			$total_cols = count($results['columns']['order']);

			echo "<tr class='' style='border-top: 1px solid #E9E9E9; background-color: #EEEEEE;'>
				<td colspan='".$total_cols."' style='text-align: left; padding-left: ".(($l+1) * 9)."px;'> ".$total_text." </td>
			</tr>";

			echo "<tr class='list-totals-row level-$l indent-$level $additional_class'>";
			if (!$report->getColumnValue('hide_group_details')) {
				echo "<td></td>"; // empty cell for the link column, show only if details are displayed
			}
			
			/*
			$isset_name_col = false;
			foreach ($report_columns as $col) {
				if ($col->getFieldName() == 'name') $isset_name_col = true;
			}
			if (!$isset_name_col) echo "<td></td>";
			*/
			
			foreach ($results['columns']['order'] as $cl => $column) {
				if (is_numeric($column)) { // is a custom property
					
					$is_a_contact_group = false;
					if (in_array($results['group_by_criterias'][$level]['type'], array('fixed_property','custom_property'))) {
						$exploded_ids = explode('_', $group['id']);
						$group_object_id = array_pop($exploded_ids);
						$contact_group_object = Objects::findObject($group_object_id);
						$is_a_contact_group = $contact_group_object instanceof ContentDataObject;
					}
					
					if ($is_a_contact_group && $report->getColumnValue("hide_group_details")) {
						$contact_cp_value_string = '';
						$contact_cp = CustomProperties::findById($column);
						if ($contact_cp instanceof CustomProperty) {
							$contact_cp_val = CustomPropertyValues::instance()->getCustomPropertyValue($contact_group_object->getId(), $contact_cp->getId());
							if ($contact_cp_val instanceof CustomPropertyValue) {
								$contact_cp_value_string = $contact_cp_val->getValue();
							}
						}
						echo "<td class='textrow'>". $contact_cp_value_string ."</td>";
					} else {
						echo "<td>". array_var($gtotal, "cp_".$column, '') ."</td>";
					}
					
				} else { // is a fixed property or a group total
					if ($column == 'link') continue;
					
					if ($cl == 1) {
						echo "<td class='textrow'></td>";
					} else {
						$dont_show_group_totals = false;
						Hook::fire('dont_show_group_totals', array('report' => $report, 'results' => $results), $dont_show_group_totals);
						$formatted_total_value = "";
						$label_value = "";
						if ($l != 0 || !$dont_show_group_totals) {
							$formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $gtotal, $column);
						}
						echo "<td> ". $label_value.$formatted_total_value ."</td>";
					}
				}
			}
	
			if (in_array($report_ot->getType(), array('dimension_group', 'dimension_object'))) {
				echo "<td></td>"; // located under column if member report
			}
			
			echo "</tr>";
		
		}
	}
}



function custom_properties_definition_get_additional_report_group_rows_csv($params, &$all_rows) {
	$results = array_var($params, 'results');
	$report = array_var($params, 'report');
	$group = array_var($params, 'group');
	$level = array_var($params, 'level');

	$report_ot = ObjectTypes::findById($report->getReportObjectTypeId());
	
	$report_columns = ReportColumns::getAllReportColumns($report->getId());
	if (isset($results['group_totals']) && isset($results['group_totals'][$level])) {
		
		// dont show the last group if it is shown as columns
		if ($report->getColumnValue('show_last_group_as_column') && $level == max(array_keys($results['group_totals']))) {
			return;
		}
		
		Env::useHelper('list_totals', 'advanced_core');
		
		$all_gtotal = array();
		$g_id_keys = explode(',', $group['original_gkey']);
		$exploded_g_ids = explode('_', $group['id']);
		
		// match group keys
		foreach ($results['group_totals'][$level] as $gt) {
			$all_match = true;
			$i = 0;
			foreach ($g_id_keys as $g_id_key) {
				if ($exploded_g_ids[$i] != $gt[$g_id_key]) {
					$all_match = false;
				}
				$i++;
				if (!$all_match) break;
			}
			if ($all_match) {
				$all_gtotal[] = $gt;
			}
		}
		
		// if there are more than one total row (e.g.: amounts separated by currency) for this group then accumulate it in one
		$gtotal = accumulate_total_rows($report_ot, $all_gtotal, $report);
		if ($gtotal) {

		    $ot_lang = function_exists('mb_strtolower') ? mb_strtolower($report_ot->getPluralObjectTypeName()) : $report_ot->getPluralObjectTypeName();
		    
		    if (!$report->getColumnValue("hide_group_details")) {
		        
		        $total_text = lang('total in', $group['name']);
		        if (!$report->getColumnValue("hide_group_total_counts")) {
		            $total_text .= " (".$gtotal['total_rows']." ".$ot_lang.")";
		        }
		        
		        $row_values = array();
		        
		        foreach ($results['columns']['order'] as $cl => $column) {
		            if (is_numeric($column)) { // is a custom property
		                $row_values[] = array_var($gtotal, "cp_".$column, '');
		                
		            } else { // is a fixed property or a group total
		                if ($column == 'link') continue;
		                
		                if ($cl == 1) {
		                    $row_values[] = $group['name'] . $total_text;
		                } else {
		                    
		                    $formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $gtotal, $column);
		                    
		                    $row_values[] = $formatted_total_value;
		                }
		            }
		        }
		        
		        if (in_array($report_ot->getType(), array('dimension_group', 'dimension_object'))) {
		            $row_values[] = ""; // located under column
		        }
		        
		        $all_rows[] = $row_values;
		        $all_rows[] = array();
		        
		    }else{//hidden details
		    	$total_text = "";
		    	if (!$report->getColumnValue("hide_group_total_counts")) {
		        	$total_text = " (".$gtotal['total_rows']." ".$ot_lang.")";
		    	}
		    	
		        $row_index = 0;
		        $index_col_name = 0;
		        foreach ($results['columns']['order'] as $column) {
		            if (is_numeric($column)) { // is a custom property
		                //$row_values[$index_col_name] = array_var($gtotal, "cp_".$column, '');
		                
		                $is_a_contact_group = false;
		                if (in_array($results['group_by_criterias'][$level]['type'], array('fixed_property','custom_property'))) {
		                	$exploded_ids = explode('_', $group['id']);
							$group_object_id = array_pop($exploded_ids);
							$contact_group_object = Objects::findObject($group_object_id);
							
		                	$is_a_contact_group = $contact_group_object instanceof ContentDataObject;
		                }
		                	
		                if ($is_a_contact_group && $report->getColumnValue("hide_group_details")) {
		                	$contact_cp_value_string = '';
		                	$contact_cp = CustomProperties::findById($column);
		                	if ($contact_cp instanceof CustomProperty) {
		                		$contact_cp_val = CustomPropertyValues::instance()->getCustomPropertyValue($contact_group_object->getId(), $contact_cp->getId());
		                		if ($contact_cp_val instanceof CustomPropertyValue) {
		                			$contact_cp_value_string = $contact_cp_val->getValue();
		                		}
		                	}
		                	$row_values[$index_col_name] = $contact_cp_value_string;
		                } else {
		                	$row_values[$index_col_name] = array_var($gtotal, "cp_".$column, '');
		                }
		                
		            } else {
		                if ($column == 'link') continue;
		                
		                if ($column == 'name'){
		                    $index = $index_col_name;
		                }else {
		                    //$formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $gtotal, $column);
		                	$dont_show_group_totals = false;
		                	Hook::fire('dont_show_group_totals', array('report' => $report, 'results' => $results), $dont_show_group_totals);
		                    $formatted_total_value = "";
		                    if ($level != 0 || !$dont_show_group_totals) {
		                    	$formatted_total_value = get_formatted_value_for_fixed_property_totals($report, $results, $gtotal, $column);
		                    }
		                    $row_values[$index_col_name] = $formatted_total_value;
		                }
		            }
		            
		            
		            $index_col_name ++;
		        }
		        
		        foreach($all_rows as $k=>$row){
		        	if ($row[$index] == $group['name']) {
		                $row_index = $k;
		                break;
		            }
		        }
		        
		        //concatenate in column that contain name the total elements
		        $all_rows[$row_index][$index] = $group['name'] . $total_text;    
		        foreach ($row_values as $k=>$v){
		            $all_rows[$row_index][$k] = $v;
		        } 
		        
		    }		    

		}
	}
}


function custom_properties_definition_override_tasks_list_order_by($params, &$result) {
	
	$order = array_var($params, 'order');
	$join_params = array_var($params, 'join_params');
	
	if (str_starts_with($order, "cp_")) {
		$cp_id = str_replace("cp_", "", $order);
		$cp = CustomProperties::getCustomProperty($cp_id);
		
		if (is_null($join_params) || empty($join_params)) {
			$join_params = array();
			$join_params['join_type'] = "LEFT ";
			$join_params['table'] = TABLE_PREFIX . "custom_property_values";
			$join_params['jt_field'] = "object_id";
			$join_params['e_field'] = "object_id";
			$join_params['on_extra'] = " AND custom_property_id='$cp_id'";
			
			$order_field = "jt.value";
		} else {
			$extra = " LEFT JOIN `" . TABLE_PREFIX . "custom_property_values` `cpv` ON `cpv`.`object_id` = `e`.`object_id` AND cpv.custom_property_id='$cp_id' ";
			if (!isset($join_params['on_extra'])) $join_params['on_extra'] = '';
			$join_params['on_extra'] .= $extra;
			
			$order_field = "cpv.value";
		}
		if ($cp->getType() == 'numeric'){
			$order = "CAST($order_field AS DECIMAL(10,4))";
		}else{
			$order = $order_field;
		}
		$order = array($order, array('col' => 'o.name', 'dir' => 'ASC')); // second order by name
		$order_dir = "ASC";
		
		$result = array('order' => $order, 'order_dir' => $order_dir, 'join_params' => $join_params);
	}
}



