<?php 

function advanced_core_update_1_2() {
	if (!check_column_exists(TABLE_PREFIX."object_subtypes", "code")) {
		DB::execute("ALTER TABLE `".TABLE_PREFIX."object_subtypes` ADD COLUMN `code` VARCHAR(255) NOT NULL DEFAULT '';");
	}
}

function advanced_core_update_2_3() {
	// after separating object subtypes from this plugin - set "object_subtypes" plugin as installed if this plugin was previously installed
	if (checkTableExists(TABLE_PREFIX.'object_subtypes')) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."plugins` (name, is_installed, is_activated, priority, activated_on, activated_by_id, version)
			VALUES ('object_subtypes', 1, 1, 10, NOW(), 0, 1)
			ON DUPLICATE KEY UPDATE is_installed=1, is_activated=1;
		");
	}
	
	if (!check_column_exists(TABLE_PREFIX."custom_properties", "total_operation")) { 
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."custom_properties` ADD COLUMN `total_operation` varchar(50) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '';
		");
	}
	
	if (Plugins::instance()->isInstalledPlugin("member_custom_properties") && !check_column_exists(TABLE_PREFIX."member_custom_properties", "total_operation")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."member_custom_properties` ADD COLUMN `total_operation` varchar(50) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '';
		");
	}
}

function advanced_core_update_3_4() {
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."object_print_options` (
		  `object_type_id` INTEGER UNSIGNED NOT NULL DEFAULT 0,
		  `object_subtype_id` INTEGER UNSIGNED NOT NULL DEFAULT 0,
		  `option_type` VARCHAR(255) NOT NULL DEFAULT '',
		  `property` VARCHAR(255) NOT NULL DEFAULT '',
		  `html_content` TEXT NOT NULL,
		  `position` VARCHAR(50) NOT NULL DEFAULT 'left',
		  `order` int(10) unsigned NOT NULL DEFAULT 0,
		  PRIMARY KEY (`object_type_id`, `object_subtype_id`, `option_type`, `property`)
		) ENGINE = InnoDB;
	");
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."object_subtype_task_templates` (
		  `object_type_id` INTEGER UNSIGNED NOT NULL,
		  `object_subtype_id` INTEGER UNSIGNED NOT NULL,
		  `task_template_id` INTEGER UNSIGNED NOT NULL,
		  PRIMARY KEY (`object_type_id`, `object_subtype_id`, `task_template_id`)
		) ENGINE = InnoDB;
	");
}

function advanced_core_update_4_5() {
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."dimension_content_object_options` (
		  `dimension_id` int(11) NOT NULL DEFAULT 0,
		  `content_object_type_id` int(11) NOT NULL DEFAULT 0,
		  `option` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT '',
		  `value` text COLLATE utf8_unicode_ci,
		  PRIMARY KEY (`dimension_id`,`content_object_type_id`,`option`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
	
	DB::execute("
		INSERT INTO `".TABLE_PREFIX."dimension_content_object_options` (`dimension_id`,`content_object_type_id`,`option`,`value`) 
			SELECT d.id, ot.id, 'hide_member_selector_in_forms', d.code IN (
				'project_types', 'project_statuses', 'customer_types', 'project_phases', 'job_phases', 
				'customer_sectors', 'customer_stages', 'project_managers', 'account_executives',
				'analysis_types', 'sample_statuses', 'sample_types'
			)
			FROM ".TABLE_PREFIX."dimensions d LEFT JOIN ".TABLE_PREFIX."object_types ot ON true
			WHERE ot.type IN ('content_object', 'located', 'comment') 
		ON DUPLICATE KEY UPDATE dimension_id=dimension_id;
	");
}

function advanced_core_update_5_6() {
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."member_history` (
		  `id` int(11) NOT NULL AUTO_INCREMENT,
		  `member_id` int(11) NOT NULL,
		  `type` varchar(50) COLLATE utf8_unicode_ci NOT NULL,
		  `property` varchar(50) COLLATE utf8_unicode_ci NOT NULL,
		  `action` varchar(50) COLLATE utf8_unicode_ci NOT NULL,
		  `value` text COLLATE utf8_unicode_ci NOT NULL,
		  `created_by_id` int(11) NOT NULL,
		  `created_on` datetime NOT NULL,
		  PRIMARY KEY (`id`),
		  KEY `member_id_created_on` (`member_id`,`created_on`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
	
	// history widgets
	DB::execute("
		INSERT INTO `".TABLE_PREFIX."widgets` (`name`,`title`,`plugin_id`,`path`,`default_options`,`default_section`,`default_order`,`icon_cls`) VALUES 
		 ('member_history','member_history',(SELECT id from `".TABLE_PREFIX."plugins` WHERE name = 'advanced_core'),'','','left',10,'ico-history')
		ON DUPLICATE KEY UPDATE name=name;
	");
	
	// Inicializar el historico de cada member
	$all_members = Members::findAll(array("conditions" => "dimension_id NOT IN (SELECT id FROM ".TABLE_PREFIX."dimensions WHERE code = 'feng_persons')"));
	foreach ($all_members as $member) {
		$last_app_log = ApplicationLogs::findOne(array("conditions" => "member_id=".$member->getId()." AND action IN ('add','edit')", "order" => "created_on DESC"));
		if (!$last_app_log instanceof ApplicationLog) continue;
		
		$last_edit_date = $last_app_log->getCreatedOn();
		$user_id = $last_app_log->getCreatedById();
		
		MemberHistories::instance()->saveMemberRelationsHistory($member, array(), true, $last_edit_date, $user_id);
	}
	
}

function advanced_core_update_6_7() {
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."property_groups` (
		  `id` int(10) NOT NULL AUTO_INCREMENT,
		  `object_type_id` int(10) NOT NULL DEFAULT 0,
		  `name` varchar(255) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '',
		  `order` int(10) NOT NULL DEFAULT 0,
		  `is_system` tinyint(1) NOT NULL DEFAULT 0,
		  PRIMARY KEY (`id`),
		  KEY `object_type_id_order` (`object_type_id`,`order`),
		  KEY `object_type_id_name` (`object_type_id`,`name`)
		) ENGINE='InnoDB' COLLATE 'utf8_unicode_ci';
	");
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."property_group_properties` (
		  `group_id` int(10) NOT NULL DEFAULT 0,
		  `property_id` varchar(255) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '',
		  `order` int(10) NOT NULL DEFAULT 0,
		  PRIMARY KEY (`group_id`,`property_id`),
		  KEY `group_id_order` (`group_id`,`order`)
		) ENGINE='InnoDB' COLLATE 'utf8_unicode_ci';
	");
}

function advanced_core_update_7_8() {
	if (!check_column_exists(TABLE_PREFIX."property_groups", "code")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."property_groups` ADD `code` varchar(255) COLLATE 'utf8_unicode_ci' NOT NULL;
		");
	}
	
	Env::useHelper('functions', 'advanced_core');
	fill_property_groups_default_data();
}

function advanced_core_update_8_9() {
	$config_option_exists = ConfigOptions::getByName("allow_duplicated_contact_emails");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`)
			VALUES ('clients_and_contacts', 'allow_duplicated_contact_emails', '', 'BoolConfigHandler', 0, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
        ");
	}
}

function advanced_core_update_9_10() {
	// create table to specify which properties are enabled to be searched for each object type
	if (!checkTableExists(TABLE_PREFIX . "api_searchable_properties")) {
		DB::execute("
			CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."api_searchable_properties` (
			  `object_type_id` int(10) unsigned NOT NULL,
			  `object_subtype_id` int(10) unsigned NOT NULL,
			  `property` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
			  PRIMARY KEY (`object_type_id`,`object_subtype_id`,`property`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci
		");
	}
	// insert default configuration for each object type (only name property)
	DB::execute("
		INSERT INTO ".TABLE_PREFIX."api_searchable_properties (object_type_id, object_subtype_id, property)
			SELECT ot.id, 0, 'name' FROM ".TABLE_PREFIX."object_types ot 
			WHERE ot.type IN ('content_object', 'dimension_object', 'dimension_group')
		ON DUPLICATE KEY UPDATE object_type_id=object_type_id;
	");
	
	// insert into searchable objects the properties of each dimension object
	ini_set("memory_limit", "1G");
	set_time_limit(0);
	$dimension_object_ids = Objects::findAll(array(
			'id' => true,
			'conditions' => "object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE `type`='dimension_object')",
	));
	foreach ($dimension_object_ids as $id) {
		$object = Objects::findObject($id);
		if ($object instanceof DimensionObject) {
			$object->addToSearchableObjects(true);
		}
	}
}


function advanced_core_update_10_11() {
	DimensionAssociationsConfigs::ensureAllAssociationsHaveConfigOptions();
}


function advanced_core_update_11_12() {
	$config_option_exists = ConfigOptions::getByName("send_object_email_default_mail_account");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`)
			VALUES ('mailing', 'send_object_email_default_mail_account', '', 'MailAccountConfigHandler', 0, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
        ");
	}
	$config_option_exists = ConfigOptions::getByName("send_object_email_default_to");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`)
			VALUES ('mailing', 'send_object_email_default_to', '', 'StringConfigHandler', 0, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
        ");
	}
}

function advanced_core_update_12_13() {
	DB::execute("
		INSERT INTO ".TABLE_PREFIX."property_group_properties (group_id, property_id, `order`)
		SELECT pg.id, 'located_under', '-1'
		FROM ".TABLE_PREFIX."property_groups pg
		WHERE pg.code='default'
			AND pg.object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE `type` IN ('dimension_object', 'dimension_group'))
		ON DUPLICATE KEY UPDATE group_id=group_id;
	");
}

function advanced_core_update_13_14() {
	$config_option_exists = ContactConfigOptions::getByName("additional_data_in_name");
	if (!$config_option_exists instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`)
			VALUES ('calendar panel', 'additional_data_in_name', '', 'SingleMemberTypeConfigHandler', '0', '0', NULL, NULL);
		");
	}
}

function advanced_core_update_14_15() {
	if (!check_column_exists(TABLE_PREFIX."report_conditions", "group_id")) { 
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."report_conditions`
			ADD `group_id` int(10) NOT NULL;
        ");
	}
}

function advanced_core_update_15_16() {
	if (!check_column_exists(TABLE_PREFIX."system_permissions", 'can_add_timeslots_in_time_tab')) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."system_permissions`
			 ADD COLUMN `can_add_timeslots_in_time_tab` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0;
		");
		DB::execute("
				UPDATE `".TABLE_PREFIX."system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
				SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive')
			);
		");
		DB::execute("
			UPDATE `".TABLE_PREFIX."system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
				SELECT permission_group_id FROM `".TABLE_PREFIX."contacts` WHERE `user_type` IN (
					SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive')
				)
			);
		");
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."max_system_permissions`
			 ADD COLUMN `can_add_timeslots_in_time_tab` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0;
		");
		DB::execute("
			UPDATE `".TABLE_PREFIX."max_system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
				SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive')
			);
		");
	}
}

function advanced_core_update_16_17() {
	DB::execute("
		UPDATE `".TABLE_PREFIX."system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
			SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive','Internal Collaborator')
		);
	");
	DB::execute("
		UPDATE `".TABLE_PREFIX."system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
			SELECT permission_group_id FROM `".TABLE_PREFIX."contacts` WHERE `user_type` IN (
				SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive','Internal Collaborator')
			)
		);
	");
	DB::execute("
		UPDATE `".TABLE_PREFIX."max_system_permissions` SET `can_add_timeslots_in_time_tab`=1 WHERE `permission_group_id` IN (
			SELECT id FROM `".TABLE_PREFIX."permission_groups` WHERE `type`='roles' AND `name` IN ('Super Administrator','Administrator','Manager','Executive','Internal Collaborator')
		);
	");
}


function advanced_core_update_17_18() {
	$config_option_exists = ContactConfigOptions::getByName("dimension_that_dictates_color");
	if (!$config_option_exists instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('calendar panel', 'dimension_that_dictates_color', '', 'SingleDimensionConfigHandler', '0', '0', NULL, NULL);
		");
	}
}


function advanced_core_update_18_19() {
	$config_option_exists = ConfigOptions::getByName("ignored_dimensions_for_subtasks");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('general', 'ignored_dimensions_for_subtasks', '', 'ManageableDimensionsConfigHandler', '0', '0', NULL, NULL);
		");
	}
}


function advanced_core_update_19_20() {
	$config_option_exists = ContactConfigOptions::getByName("sort_member_trees_by_member_type");
	if (!$config_option_exists instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('listing preferences', 'sort_member_trees_by_member_type', '0', 'BoolConfigHandler', '0', '50', NULL, NULL);
		");
	}
}



function advanced_core_update_20_21() {
	
	if (!check_column_exists(TABLE_PREFIX."property_groups", 'alignment')) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."property_groups`
			 ADD COLUMN `alignment` varchar(255) collate utf8_unicode_ci NOT NULL DEFAULT 'left';
		");
	}
}

function advanced_core_update_21_22() {
	DimensionAssociationsConfigs::ensureAllAssociationsHaveConfigOptions();
}

function advanced_core_update_22_23() {
	
	if (!check_column_exists(TABLE_PREFIX."property_group_properties", 'is_inheritable')) {
		DB::execute("
				ALTER TABLE `".TABLE_PREFIX."property_group_properties` ADD COLUMN `is_inheritable` TINYINT(1) NOT NULL DEFAULT 0;
			");
	}
}

function advanced_core_update_23_24() {
	$config_option_exists = ConfigOptions::getByName("allow_adding_worked_time_to_parent_task");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('task_workflow', 'allow_adding_worked_time_to_parent_task', 1, 'BoolConfigHandler', '0', '1', NULL, NULL);
		");
	}

	$config_option_exists = ConfigOptions::getByName("allow_adding_estimated_time_to_parent_task");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('task_workflow', 'allow_adding_estimated_time_to_parent_task', 1, 'BoolConfigHandler', '0', '2', NULL, NULL);
		");
	}
}

function advanced_core_update_24_25() {
	DB::execute("
		UPDATE `".TABLE_PREFIX."dimension_member_associations` SET `is_required` = 1  WHERE `code` IN ('project_customer','project_rate_schedule');
	");
}

function advanced_core_update_25_26() {
	if (!check_column_exists(TABLE_PREFIX."timeslots", "ignore_time_start")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots` ADD `ignore_time_start` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0;
		");
	}

	$time_list_view_config_option = ContactConfigOptions::getByName('time_list_view');
	if (!$time_list_view_config_option instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`)
			VALUES ('general', 'time_list_view', 'list', 'StringConfigHandler', '1', '0', NULL, NULL);
		");
	}

	$weekly_start_date_config_option = ContactConfigOptions::getByName('weekly_time_start_date');
	if (!$weekly_start_date_config_option instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`)
			VALUES ('general', 'weekly_time_start_date', '', 'StringConfigHandler', '1', '0', NULL, NULL);
		");
	}

	$weekly_end_date_config_option = ContactConfigOptions::getByName('weekly_time_end_date');
	if (!$weekly_start_date_config_option instanceof ContactConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."contact_config_options` (`category_name`, `name`, `default_value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`)
			VALUES ('general', 'weekly_time_end_date', '', 'StringConfigHandler', '1', '0', NULL, NULL);
		");
	}
}

function advanced_core_update_26_27() {
	// Add config option 'allow_overlapped_time_entries'
	$config_option_exists = ConfigOptions::getByName("allow_overlapped_time_entries");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`)
			VALUES ('general', 'allow_overlapped_time_entries', '1', 'BoolConfigHandler', '0', '1000', '', '');
		");
	}
}

function advanced_core_update_27_28() {
	// GENERIC CONTENT OBJECT TEMPLATES STRUCTURE

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_templates` (
			`id` int(11) NOT NULL AUTO_INCREMENT,
			`name` varchar(255) DEFAULT NULL,
			`description` text,
			`object_type_id` int(11) DEFAULT NULL,
			PRIMARY KEY (`id`),
			KEY `name` (`name`),
			KEY `object_type_id` (`object_type_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_objects` (
			`template_id` int(11) NOT NULL,
			`object_id` int(11) NOT NULL,
			PRIMARY KEY (`template_id`,`object_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_parameters` (
			`template_id` int(11) NOT NULL,
			`parameter_id` int(11) NOT NULL,
			`name` varchar(255) DEFAULT NULL,
			`description` varchar(255) DEFAULT NULL,
			`type` varchar(255) DEFAULT NULL,
			`required` tinyint(3) unsigned NOT NULL DEFAULT '0',
			PRIMARY KEY (`template_id`,`parameter_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_object_properties` (
			`template_id` int(11) NOT NULL,
			`object_id` int(11) NOT NULL,
			`property` varchar(255) NOT NULL,
			`property_type` varchar(255) DEFAULT NULL,
			`property_value` text,
			PRIMARY KEY (`template_id`,`object_id`,`property`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_object_lines` (
			`template_id` int(11) NOT NULL,
			`object_id` int(11) NOT NULL,
			`line_id` int(11) NOT NULL,
			PRIMARY KEY (`template_id`,`object_id`,`line_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_object_line_values` (
			`template_id` int(11) NOT NULL,
			`object_id` int(11) NOT NULL,
			`line_id` int(11) NOT NULL,
			`line_property` varchar(255) NOT NULL,
			`line_property_value` text,
			PRIMARY KEY (`template_id`,`object_id`,`line_id`,`line_property`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."cobj_template_object_variable_values` (
			`template_id` int(11) NOT NULL,
			`object_id` int(11) NOT NULL,
			`property` varchar(255) NOT NULL,
			`parameter_id` int(11) NOT NULL,
			`value` text,
			PRIMARY KEY (`template_id`,`object_id`,`property`),
			KEY `parameter_id` (`parameter_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");

}

function advanced_core_update_28_29() {

	// generic object templates feature have some functions in common with advanced_templates plugin
	$p_controller = new PluginController();
	$p_controller->ensure_installed_and_activated('advanced_templates');

}