<?php

Hook::register("adv_mail_imap_folder");


function adv_mail_imap_folder_mail_account_edit_categories($account, &$categories) {

	tpl_assign('account', $account);
	$categories[] = array(
		'id' => 'member_data',
		'name' => lang('additional information'),
		'required' => false,
		'visible' => true,
		'content' => tpl_fetch(get_template_path('member_data_tab', 'mail_account_member', 'advanced_mail_imap_folders')),
	);

}


function adv_mail_imap_folder_on_page_load(){
	if (class_exists('MailAccounts')) {
		// generate the members and classify the mails in each imap folder member
		Env::useHelper('functions', 'advanced_mail_imap_folders');
		Env::useHelper('permissions');
		Env::useHelper('dimension');
		
		try {
			$accounts = MailAccounts::findAll();
			foreach ($accounts as $account) {
				check_and_create_mail_account_imap_folder_members($account);
			}
		} catch (Exception $e) {
			Logger::log_r("ERROR creating account imap folders: ".$e->getMessage());
			Logger::log_r($e->getTraceAsString());
		}
	}
}


function adv_mail_imap_folder_mail_account_deleted($params, &$ignored) {
	$account_id = array_var($params, "id");
	
	$imap_folder_objects = MailImapFolderMembers::findAll(array("conditions" => "account_id='$account_id'"));
	$account_member_object = MailAccountMembers::findOne(array("conditions" => "account_id='$account_id'"));
	
	$dimension = Dimensions::findByCode("mail_imap_folders");
	
	if (is_array($imap_folder_objects)) {
		foreach ($imap_folder_objects as $folder_obj) {
			$folder_member = Members::findOneByObjectId($folder_obj->getId(), $dimension->getId());
			if ($folder_member instanceof Member) {
				$folder_member->delete();
			}
		}
	}
	if ($account_member_object instanceof MailAccountMember) {
		$account_member = Members::findOneByObjectId($account_member_object->getId(), $dimension->getId());
		if ($account_member instanceof Member) {
			$account_member->delete();
		}
	}
	
	evt_add("reload dimension tree", array('dim_id' => $dimension->getId()));
}


function adv_mail_imap_folder_before_classify_additional_verifications($params, &$ignored) {
	$email = array_var($params, 'object');
	$member_ids = array_filter(array_var($params, 'member_ids'));
	$after_receiving = array_var($params, 'after_receiving');
	
	if ($email instanceof MailContent && count($member_ids) > 0) {
		
		$dimension = Dimensions::findByCode("mail_imap_folders");
		$mail_account_ot = ObjectTypes::findByName('mail_account_member');
		$imap_folder_ot = ObjectTypes::findByName('mail_imap_folder_member');
		
		$members = Members::findAll(array("conditions" => "id IN(".implode(',', $member_ids).")"));
		foreach ($members as $member) {
			
			if ($member->getDimensionId() == $dimension->getId()
					&& ($member->getObjectTypeId() == $mail_account_ot->getId() || $member->getObjectTypeId() == $imap_folder_ot->getId())) {
				
				$mem_obj = Objects::findObject($member->getObjectId());
				if ($mem_obj instanceof MailAccountMember || $mem_obj instanceof MailImapFolderMember) {
					
					if ($mem_obj->getAccountId() != $email->getAccountId()) {
						$err_msg = lang('the email cant be classified in folder x because that folder belongs to another account', clean($member->getName()));
						Logger::log_r("Mail ".$email->getId()." - ".$err_msg);
						if (!$after_receiving) {
							throw new Exception($err_msg);
						}
					}
				}
			}
		}
	}
}

function adv_mail_imap_folder_modify_mail_classify_selector_options($params, &$options) {
	$mail = array_var($params, 'object');
	
	if ($mail instanceof MailContent && $mail->getAccountId() > 0) {
		$dimension = Dimensions::findByCode("mail_imap_folders");
		$acc_id = $mail->getAccountId();
		
		$sql = "SELECT m.id FROM ".TABLE_PREFIX."members m
				LEFT JOIN ".TABLE_PREFIX."mail_imap_folder_members f ON f.object_id=m.object_id
				LEFT JOIN ".TABLE_PREFIX."mail_account_members a ON a.object_id=m.object_id
				WHERE m.dimension_id=".$dimension->getId()." 
					AND (f.account_id=$acc_id OR a.account_id=$acc_id);";
		$mem_ids_rows = DB::executeAll($sql);
		$mem_ids = array_flat($mem_ids_rows);
		
		if (count($mem_ids) > 0) {
			if (!is_array($options)) $options = array();
			if (!isset($options['filter_by_ids'])) $options['filter_by_ids'] = array();
			
			$options['filter_by_ids'][$dimension->getId()] = $mem_ids;
		}
	}
}


function adv_mail_imap_folder_member_quick_add_urls($parameters, &$urls) {
	$dimension = array_var($parameters, 'dimension');
	$object_types = array_var($parameters, 'object_types');
	$parent_member = array_var($parameters, 'parent_member');
	
	$mail_account_ot = ObjectTypes::findByName('mail_account_member');
	$imap_folder_ot = ObjectTypes::findByName('mail_imap_folder_member');
	
	if ($dimension->getCode() == 'mail_imap_folders' && !$parent_member instanceof Member) {
		$urls = array(
			array('link_text' => lang('add account'), 'url' => get_url('mail', 'add_account'), 'iconcls' => 'mail'),
		);
	} else {
		// by default adds an imap folder member
	}
}

/* dont allow to change mail imap folder parent */
function adv_mail_imap_folder_member_add_render_parent_selector($parameters, &$render_parent_sel) {
	$member = array_var($parameters, 'member');
	$member_ot = ObjectTypes::findById($member->getObjectTypeId());
	$imap_folder_ot = ObjectTypes::findByName('mail_imap_folder_member');
	if ($imap_folder_ot->getId() == $member_ot->getId()) {
		$render_parent_sel = false;
	}
}


function adv_mail_imap_folder_override_email_account_filter($parameters, &$attributes) {
	// don't process the count request, only in the mails listing
	if (array_var($_REQUEST, 'only_result')) return;
	
	$context = array_var($parameters, 'context');
	
	$dim = Dimensions::findByCode('mail_imap_folders');
	$account_member_filter = nulL;
	
	foreach ($context as $selection) {
		if ($selection instanceof Member && $selection->getDimensionId() == $dim->getId()) {
			$account_member_filter = $selection;
			break;
		}
	}
	
	if ($account_member_filter instanceof Member) {
		// filter by selected account or folder
		$object = Objects::findObject($account_member_filter->getObjectId());
		if ($object instanceof ContentDataObject) {
			$account_id = $object->getColumnValue('account_id');
			if ($account_id) {
				$acc = MailAccounts::findById($account_id);
				$account_name = $acc instanceof MailAccount ? $acc->getName() : '';
				
				$attributes['accountId'] = $account_id;
				
				evt_add('refresh_mail_list_account_filter', array('account_id' => $account_id, 'account_name' => $account_name));
			}
		}
		// if it is a folder then ignore the archived filter, show all
		if ($object instanceof MailImapFolderMember) {
			$_REQUEST['archived_filter'] = 'all';
			$attributes['archivedType'] = 'all';
		}
	} else {
		// if context changed and view all clicked -> clear the filter
		if (array_var($_REQUEST, 'prev_context')) {
			$prev_mem_selected = null;
			$prev_context = build_context_array(array_var($_REQUEST, 'prev_context'));
			foreach ($prev_context as $selection) {
				if ($selection instanceof Member && $selection->getDimensionId() == $dim->getId()) {
					$prev_mem_selected = $selection;
					break;
				}
			}

			if ($prev_mem_selected instanceof Member) {
				$attributes['accountId'] = "";
				evt_add('refresh_mail_list_account_filter', array('account_id' => ''));
			}
		}
	}
}



function adv_mail_imap_folder_after_mail_account_save($params, &$ignored) {
	$account = array_var($params, 'account');
	$is_new = array_var($params, 'is_new');
	$save_permissions = array_var($params, 'save_permissions', true);
	
	if ($account instanceof MailAccount) {
		Env::useHelper('functions', 'advanced_mail_imap_folders');
		Env::useHelper('permissions');
		
		$dim = Dimensions::findByCode('mail_imap_folders');
		
		// create member for account if not exists 
		$account_changed = check_and_create_mail_account_imap_folder_members($account);
		
		// get member
		$mail_account_member = MailAccountMembers::findOne(array('conditions' => "account_id=".$account->getId()));
		if ($mail_account_member instanceof MailAccountMember) {
			$member = Members::findOneByObjectId($mail_account_member->getId(), $dim->getId());
		}
				
		// save member custom properties
		$has_custom_properties = false;
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			
			$ot = ObjectTypes::findByName('mail_account_member');
			$has_custom_properties = member_has_custom_properties($ot->getId());
			
			if ($has_custom_properties && $member instanceof Member) {
				$mcp_controller = new MemberCustomPropertiesController();
				$mcp_controller->add_custom_properties($member);
			}
		}
		
		// update account member permissions
		if (!$is_new && $member instanceof Member && $save_permissions) {
			if ($account->getName() != $member->getName()) {
				$member->setName($account->getName());
				$member->save();
			}
			$permissions_string = generate_permissions_string_for_account_users($account);
			$sql = "DELETE FROM ".TABLE_PREFIX."contact_member_permissions WHERE member_id=".$member->getId()." AND object_type_id=".MailContents::instance()->getObjectTypeId();
			DB::execute($sql);
			save_member_permissions_background(logged_user(), $member, $permissions_string, $member->getParentMemberId());
		
			// update imap folder members permissions
			$child_members = $member->getAllChildren(true);
			foreach ($child_members as $child_member) {
				$sql = "DELETE FROM ".TABLE_PREFIX."contact_member_permissions WHERE member_id=".$child_member->getId()." AND object_type_id=".MailContents::instance()->getObjectTypeId();
				DB::execute($sql);
				save_member_permissions_background(logged_user(), $child_member, $permissions_string, $child_member->getParentMemberId());
			}
			
			// apply color custom property to child members
			$color = $member->getColor();
			
			$folder_ot = ObjectTypes::findByName('mail_imap_folder_member');
			$cp = CustomProperties::instance()->getCustomPropertyByCode($folder_ot->getId(), 'color_special');
			if ($cp instanceof CustomProperty) {
				foreach ($child_members as $child_member) {
					DB::execute("
						INSERT INTO ".TABLE_PREFIX."custom_property_values (object_id, custom_property_id, value) VALUES
						('".$child_member->getObjectId()."','".$cp->getId()."','$color')
						ON DUPLICATE KEY UPDATE value='$color';
					");
				}
			}
			
			foreach ($child_members as $child_member) {
				DB::execute("UPDATE ".TABLE_PREFIX."members SET color='$color' WHERE id='".$child_member->getId()."';");
			}
			
			// update contact member cache
			$macs = MailAccountContacts::instance()->getByAccount($account);
			$mac_user_ids = array();
			foreach ($macs as $mac) $mac_user_ids[] = $mac->getContactId();
			$mac_user_ids = array_filter($mac_user_ids);
			if (count($mac_user_ids) > 0) {
				$mac_users = Contacts::findAll(array('conditions' => "id IN (".implode(',', $mac_user_ids).")"));
				foreach ($mac_users as $mac_user) {
					ContactMemberCaches::updateContactMemberCache($mac_user, $member->getId(), $member->getParentMemberId());
				}
			}
		}
		
		if ($account_changed || $has_custom_properties) {
			evt_add("reload dimension tree", array('dim_id' => $dim->getId()));
		}
	}
	
}


function adv_mail_imap_folder_after_mail_imap_folder_inbox_forced($params, &$ignored) {
	$account = array_var($params, 'account');
	
	if ($account instanceof MailAccount) {
		Env::useHelper('functions', 'advanced_mail_imap_folders');
		
		check_and_create_mail_account_imap_folder_members($account);
	}
	
}


function adv_mail_imap_folder_after_mail_content_imap_folder_saved($params, &$ignored) {
	$account = array_var($params, 'account');
	$mail_id = array_var($params, 'mail_id');
	$folder = array_var($params, 'folder');
	$uid = array_var($params, 'uid');
	
	$mail = MailContents::findById($mail_id);
	
	if ($mail instanceof MailContent) {
		$dim = Dimensions::findByCode('mail_imap_folders');
		Env::useHelper('functions', 'advanced_mail_imap_folders');
		
		$folder_name = escape_character($folder);
		if (!isset($_SESSION['imap_folder_member_cache'])) {
			$_SESSION['imap_folder_member_cache'] = array();
		}
		
		$folder_member = array_var($_SESSION['imap_folder_member_cache'], $account->getId().'_'.$folder);
		
		if (!$folder_member instanceof Member) {
			$folder_obj = MailImapFolderMembers::findOne(array('conditions' => "account_id=".$account->getId()." AND folder_name='$folder_name'"));
			if ($folder_obj instanceof MailImapFolderMember) {
				$folder_member = Members::findOneByObjectId($folder_obj->getId(), $dim->getId());
				$_SESSION['imap_folder_member_cache'][$account->getId().'_'.$folder] = $folder_member;
			} else {
				$acc_folder_obj = MailAccountMembers::findOne(array('conditions' => "account_id=".$account->getId()));
				if ($acc_folder_obj instanceof MailAccountMember) {
					$folder_member = Members::findOneByObjectId($acc_folder_obj->getId(), $dim->getId());
					$_SESSION['imap_folder_member_cache'][$account->getId().'_'.$folder] = $folder_member;
				}
			}
		}
		
		if ($folder_member instanceof Member) {
			classify_single_mail_in_imap_folder_member($account, $folder_name, $folder_member, $mail);
		}
		
	}
	
}


function adv_mail_imap_folder_breadcrumbs_extra_conditions($params, &$extra_cond) {
	$dim = array_var($params, 'dim');
	if ($dim instanceof Dimension && $dim->getCode() == 'mail_imap_folders') {
		$extra_cond .= " AND (select oot.name from ".TABLE_PREFIX."object_types oot where oot.id=m.object_type_id) != 'mail_account_member'";
	}
}


function adv_mail_imap_folder_after_delete_feng_imap_folder($params, &$ignored) {
	$account = array_var($params, 'account');
	$folder = array_var($params, 'folder');
	
	$folder_mem_object = MailImapFolderMembers::findOne(array(
		"conditions" => array("account_id=? AND folder_name=?", $account->getId(), $folder)
	));
	
	if ($folder_mem_object instanceof MailImapFolderMember) {
		$dim = Dimensions::findByCode("mail_imap_folders");
		$f_member = Members::findOneByObjectId($folder_mem_object->getId(), $dim->getId());
		if ($f_member instanceof Member) {
			// delete the member
			$f_member->delete(false);
		}
	}
}



function adv_mail_imap_folder_mail_list_dim_ids_excluded_from_classified_filder($params, &$extra_cond) {
	$dim = Dimensions::findByCode("mail_imap_folders");
	$ot = ObjectTypes::findByName('mail_account_member');
	
	$extra_cond .= " AND ( SELECT m.object_type_id FROM ".TABLE_PREFIX."members m WHERE m.id=om.member_id ) <> ".$ot->getId();
}
