<?php

/**
 * ImapSynchronization controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ImapSynchronizationController extends ApplicationController {

	var $plugin_name = "advanced_mail_imap_sync";
	
	/**
	 * Construct the ImapSynchronizationController
	 *
	 * @access public
	 * @param void
	 * @return ImapSynchronizationController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('dimension');
		Env::useHelper('permissions');
		Env::useHelper('functions', 'advanced_mail_imap_sync');
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	/**
	 * Array with a cache of the user ids of each mail account by account id
	 * @var array $account_users_cache
	 */
	private $account_users_cache = array();
	
	/**
	 * Amount of days to calculate the range of the synchronization
	 * @var int $days_to_retrieve_changes
	 */
	private $days_to_retrieve_changes = 30;
	
	
	
	


	/**
	 * Requests to the email server the latest changes for every folder in an email account
	 * Applies the changes in feng
	 * @param integer $account_id
	 */
	function search_imap_folder_changes($account_id = 0) {
		ajx_current("empty");
		set_time_limit(0);
		
	
		if (is_numeric(array_var($_REQUEST, 'account_id'))) {
			$account = MailAccounts::findById(array_var($_REQUEST, 'account_id'));
		} else {
			$account = MailAccounts::findById($account_id);
		}
		if (!$account instanceof MailAccount) {
			return;
		}
		
		// connect and login
		$imap = $account->imapConnect();
		$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
		if (PEAR::isError($ret)) {
			Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
			return;
		}
	
		// init capabilities
		$capabilites = $imap->cmdCapability();
		//Logger::log_r($capabilites);
		
		// check for new mailboxes and add them
		$mailbox_changes = $this->check_for_mailbox_changes($account, $imap);
	
		// check if condstore and qresync sync capabilities are available
		$has_condstore = $imap->hasCapability('CONDSTORE');
		$has_qresync = $imap->hasCapability('QRESYNC');
	
		// enable available capabilites
		if ($has_condstore) $imap->_genericCommand('ENABLE CONDSTORE');
		if ($has_qresync) $imap->_genericCommand('ENABLE QRESYNC');
	
		// get mailboxes to sync
		$mailboxes = MailAccountImapFolders::getMailAccountImapFolders($account->getId());
	
		$updated_uids_count = 0;
	
		if (is_array($mailboxes)) {
			foreach ($mailboxes as $mailbox) {
				if ($mailbox->getCheckFolder()) {
					//file_put_contents(CACHE_DIR."/synclog.txt", date("H:i:s")." - ".$account->getId()." - ".$mailbox->getFolderName()."\n", FILE_APPEND);
					// examine the folder -> select and get status
					$status = $this->examine_folder($imap, utf8_decode($mailbox->getFolderName()));

					if ($status) {

						$last_changes = $this->get_imap_folder_last_changes(array(
								'account' => $account,
								'imap' => $imap,
								'has_condstore' => $has_condstore,
								'has_qresync' => $has_qresync,
								'mailbox' => $mailbox,
								'status' => $status,
						));

						$processed_result = $this->process_imap_folder_last_changes(array(
								'account' => $account,
								'imap' => $imap,
								'has_condstore' => $has_condstore,
								'has_qresync' => $has_qresync,
								'mailbox' => $mailbox,
								'status' => $status,
								'last_changes' => $last_changes,
						));
	
						$updated_uids_count += array_var($processed_result, 'affected_count');
					}
				}
			}
		}
	
		// close imap connection
		$imap->disconnect();
	
		// if any changes applied => reload mails panel
		if ($updated_uids_count > 0) {
			evt_add('reload tab panel', 'mails-panel');
		}
		
		return array('mailbox_changes'=>$mailbox_changes, 'updated_uids_count'=>$updated_uids_count);
	}
	
	
	/**
	 * Requests to the server for the latest changes in the mail account folder
	 * @param array $parameters
	 */
	private function get_imap_folder_last_changes($parameters) {
	
		$account = array_var($parameters, 'account');
		$imap = array_var($parameters, 'imap');
		$has_condstore = array_var($parameters, 'has_condstore');
		$has_qresync = array_var($parameters, 'has_qresync');
		$mailbox = array_var($parameters, 'mailbox');
		$status = array_var($parameters, 'status');
	
		$uidvalidity = array_var($status, 'UIDVALIDITY');
		$highestmodseq = array_var($status, 'HIGHESTMODSEQ');
	
		/* @var $mailbox MailAccountImapFolder */
		$last_uidvalidity = $mailbox->getColumnValue('last_uidvalidity');
		$last_highestmodseq = $mailbox->getColumnValue('last_highestmodseq');
		$last_uid_in_feng = trim($mailbox->getColumnValue('last_uid_in_folder'));
	
		$range_start = "1";
		$range_end = is_numeric($last_uid_in_feng) && $last_uid_in_feng > 0 ? $last_uid_in_feng : "*";
	
		$sync_from_date_obj = DateTimeValueLib::now();
		$sync_from_date_obj->add('d', - $this->days_to_retrieve_changes);
		$sync_from_date = $sync_from_date_obj->format('d-M-Y');
		$sync_from_date_sql = $sync_from_date_obj->format('Y-m-d');
		
		$query_range_start = false;
	
		if ($last_uidvalidity && $last_uidvalidity == $uidvalidity) {
			// rely on the uids and status has the highest mod_seq -> try to fetch only last changes
			if ($has_condstore) {
				if (is_numeric($last_highestmodseq) && $last_highestmodseq > 0) {
					$last_mod_seq_checked = $last_highestmodseq;
				} else {
					$last_mod_seq_checked = '0';
					
					// check last changes since $this->days_to_retrieve_changes days
					$query_range_start = true;
				}
	
				$fetch_params = ' (CHANGEDSINCE '.$last_mod_seq_checked . ($has_qresync ? ' VANISHED' : '') . ')';
	
			} else {
				// check last changes since $this->days_to_retrieve_changes days
				$query_range_start = true;
				
				$fetch_params = '';
			}
	
		} else {
			// uids are not reliable -> fetch all changes since $days_range_to_fetch_changes days
			$query_range_start = true;
			
			$fetch_params = '';
			if ($has_condstore) {
				$fetch_params = ' (CHANGEDSINCE 0' . ($has_qresync ? ' VANISHED' : '') . ')';
			}
	
		}
		
		if ($query_range_start) {
			$first_uid = $this->get_first_uid_since_date($account, $mailbox, $sync_from_date_sql);
			if ($first_uid) {
				$range_start = $first_uid;
			}
		}
	
		$raw_response = '';
		// execute the command to fetch the changes, keep the raw response
		$fetch_result = $this->imap_command_uid_fetch("$range_start:$range_end", '(FLAGS)' . $fetch_params, $imap, $raw_response);
	
		// get the last processed MODSEQ from the raw response, to start next synchronization from there
		if ($has_condstore) {
			preg_match_all("/MODSEQ \(((?>[^)(]+|(?R))*)\)/", $raw_response, $matches);
	
			if (isset($matches[1])) {
				$all_received_mod_seqs = $matches[1];
				if (is_array($all_received_mod_seqs) && count($all_received_mod_seqs) > 0) {
					$last_processed_modseq = max($all_received_mod_seqs);
					if ($last_processed_modseq) {
						$highestmodseq = $last_processed_modseq;
					}
				}
					
			}
		}
	
		/*	Logger::log_r("-----------------------------------------------------");
		 Logger::log_r($mailbox->getFolderName()."\n");
		 Logger::log_r("UID FETCH $range_start:$range_end" . ' (FLAGS)' . $fetch_params);
		 Logger::log_r($fetch_result);
		 */
		// update mailbox uidvalidity and highestmodseq tokens
		if ($uidvalidity || $highestmodseq) {
			if ($uidvalidity) $mailbox->setColumnValue('last_uidvalidity', $uidvalidity);
			if ($highestmodseq) $mailbox->setColumnValue('last_highestmodseq', $highestmodseq);
			$mailbox->save();
		}
	
		return array('fetch_result' => $fetch_result, 'raw_response' => $raw_response);
	}
	
	
	/**
	 * Processes the changes that the server responded
	 * @param array $parameters
	 */
	private function process_imap_folder_last_changes($parameters) {
	
		$account = array_var($parameters, 'account');
		$imap = array_var($parameters, 'imap');
		$has_condstore = array_var($parameters, 'has_condstore');
		$has_qresync = array_var($parameters, 'has_qresync');
		$mailbox = array_var($parameters, 'mailbox');
		$status = array_var($parameters, 'status');
		$last_changes = array_var($parameters, 'last_changes');
		
		$days_from = $account->getDelFromServer() > 0 ? $account->getDelFromServer() : $this->days_to_retrieve_changes;
		$sync_from_date_obj = DateTimeValueLib::now();
		$sync_from_date_obj->add('d', - $days_from);
		$sync_from_date = $sync_from_date_obj->format('d-M-Y');
		$sync_from_date_sql = $sync_from_date_obj->format('Y-m-d');
	
		$fetch_result = $last_changes['fetch_result'];
		$raw_response = $last_changes['raw_response'];
	
		$possible_flags = array_var($status, "FLAGS");
	
		$all_uids = array();
		$uids_by_flag = array();
			
		foreach ($possible_flags as $flag) {
			$uids_by_flag[$flag] = array();
	
			$parsed  = array_var($fetch_result, 'PARSED', array());
			foreach ($parsed as $k => $item) {
	
				if (array_var($item, 'COMMAND') == 'FETCH') {
	
					$ext = array_var($item, 'EXT', array());
					$uid = array_var($ext, 'UID');
					if (!$uid) continue;
	
					$all_uids[$uid] = $uid;
					$flags = array_var($ext, 'FLAGS');
	
					if (is_array($flags)) {
						if (in_array($flag, $flags)) {
							$uids_by_flag[$flag][] = $uid;
						}
					} else {
						// quickfix for parser errors: when it puts the flags in the command key or separated from the item
						for ($j = $k+1; $j<count($parsed); $j++) {
							$next_item = $parsed[$j];
	
							if (array_var($next_item, 'COMMAND') == 'FETCH') break; // reached the next log
	
							if (array_var($next_item, 'COMMAND') == $flag) {
								$uids_by_flag[$flag][] = $uid;
								break;
							}
							if (array_var($next_item, 'COMMAND') == 'FLAGS') {
								$next_ext = array_var($next_item, 'EXT', array());
								if (in_array($flag, array_var($next_ext, 'FLAGS'))) {
									$uids_by_flag[$flag][] = $uid;
									break;
								}
							}
						}
					}
				}
	
			}
		}
	
		$seen_uids = array_var($uids_by_flag, "\\Seen");
		$draft_uids = array_var($uids_by_flag, "\\Draft");
		$deleted_uids = array_var($uids_by_flag, "\\Deleted");
		$vanished_uids = array();
		$unseen_uids = array_diff($all_uids, $seen_uids);
	
		// get vanished uids to remove from folder
		if ($has_condstore && $has_qresync) {
			preg_match_all("/VANISHED \(EARLIER\) (.*)/", $raw_response, $matches);
			if (isset($matches[1]) && count($matches[1]) > 0) {
				$vanished_uids = $this->expand_imap_uid_ranges($matches[1][0]);
			}
		} else {
			//@TODO: ask all uids since a date and check in database, if we have one that not exists in server => remove it
			$sql = "SELECT f.uid FROM ".TABLE_PREFIX."mail_content_imap_folders f
					INNER JOIN ".TABLE_PREFIX."mail_contents m ON m.object_id=f.object_id
					WHERE f.account_id='".$account->getId()."' AND f.folder=".DB::escape($mailbox->getFolderName())."
						AND m.received_date > '$sync_from_date_sql'";
	
			$current_uids = DB::executeAll($sql);
			if (!$current_uids) {
				$current_uids = array();
			} else {
				$current_uids = array_filter(array_flat($current_uids));
			}
			$all_server_uids = $imap->search("SINCE $sync_from_date", true);
			if (!$all_server_uids) $all_server_uids = array();
	
			//$all_server_uids = $imap->search("ALL", true);
	
			$vanished_uids = array_diff($current_uids, $all_server_uids);
		}
		
		$affected_count = 0;
	
		// mark as read in feng
		$affected_count += $this->mark_mails_as_read($account, $mailbox, $seen_uids);
	
		// mark as unread un feng
		$affected_count += $this->mark_mails_as_unread($account, $mailbox, $unseen_uids);
		
		// process special folders
		if ($mailbox->getSpecialUse() != '') {
			$affected_count += $this->process_special_folder($account, $mailbox, $all_uids);
		}
	
		// remove vanished mails from this folder
		$affected_count += $this->remove_vanished_mails($account, $mailbox, $vanished_uids, $imap);
		
		return array(
				'affected_count' => $affected_count,
				'uids_updated' => $all_uids,
				'seen_uids' => $seen_uids,
				'unseen_uids' => $unseen_uids,
				'draft_uids' => $draft_uids,
				'deleted_uids' => $deleted_uids,
				'vanished_uids' => $vanished_uids,
		);
	}
	
	
	
	function process_special_folder($account, $mailbox, $all_uids) {
		if (count($all_uids) == 0) return;
		
		$sql = "SELECT f.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders f
				WHERE f.folder=".DB::escape($mailbox->getFolderName())." AND f.account_id=".$account->getId()."
					AND f.uid IN (".implode(',', $all_uids).")";
		$object_id_rows = DB::executeAll($sql);
		$object_ids = array_unique(array_filter(array_flat($object_id_rows)));
		
		if (is_array($object_ids) && count($object_ids) > 0) {
			
			switch ($mailbox->getSpecialUse()) {
				case "\\Sent": 
					$state = 1; 
					DB::execute("UPDATE ".TABLE_PREFIX."mail_contents SET state='$state' WHERE object_id IN (".implode(',', $object_ids).")");
					break;
				case "\\Drafts": 
					$state = 2; 
					DB::execute("UPDATE ".TABLE_PREFIX."mail_contents SET state='$state' WHERE object_id IN (".implode(',', $object_ids).")");
					break;
				case "\\Junk": 
					$state = 4;
					DB::execute("UPDATE ".TABLE_PREFIX."mail_contents SET state='$state' WHERE object_id IN (".implode(',', $object_ids).")");
					break;
				case "\\Trash": 
					DB::execute("UPDATE ".TABLE_PREFIX."objects SET trashed_by_id='".$account->getContactId()."', trashed_on=NOW()
						WHERE id IN (".implode(',', $object_ids).") AND trashed_by_id=0");
					break;
				case "\\Archive": 
					DB::execute("UPDATE ".TABLE_PREFIX."objects SET archived_by_id='".$account->getContactId()."', archived_on=NOW()
						WHERE id IN (".implode(',', $object_ids).") AND archived_by_id=0");
					break;
				default: break;
			}
			
		}
	}
	
	
	
	
	/**
	 * Executes the imap "EXAMINE" command and returns the parsed response, also parses the raw response to look for the HIGHESTMODSEQ
	 * @param Net_IMAP $imap
	 * @param MailAccountImapFolder $folder
	 */
	private function examine_folder($imap, $folder) {
		/* @var $imap Net_IMAP */
		$cmdid = $imap->_getCmdId();
		$command = 'EXAMINE "'.$folder.'"';
		$imap->_putCMD($cmdid, $command);
		$raw_response = $imap->_getRawResponse($cmdid);
		/*Logger::log_r($command);
		Logger::log_r($raw_response);
		*/
		$original_raw_response = $raw_response;
		$parsed = array();
		
		$ret = $imap->_genericImapResponseParser($raw_response, $cmdid);
		
		if (isset($ret['PARSED'])) {
			for ($i=0; $i<count($ret['PARSED']); $i++) {
				if (array_key_exists('EXT', $ret['PARSED'][$i]) && is_array($ret['PARSED'][$i]['EXT'])) {
					$command = $ret['PARSED'][$i]['EXT'];
					if (key($command)) {
						$parsed[key($command)] = $command[key($command)];
					}
				}
			}
		}
		
		if (!isset($parsed['HIGHESTMODSEQ']) && strpos($original_raw_response, "OK [HIGHESTMODSEQ") !== false) {
			$s = strpos($original_raw_response, "OK [HIGHESTMODSEQ") + strlen("OK [HIGHESTMODSEQ");
			$e = strpos($original_raw_response, "]", $s);
			$v = substr($original_raw_response, $s, ($e-$s));
			
			$parsed['HIGHESTMODSEQ'] = trim($v);
		}
		
		return $parsed;
	}
	
	/**
	 * Executes the imap UID FETCH command and returns the parsed response, also returns the raw response in the variable $rawresponse
	 * @param string $msgset
	 * @param string $fetchparam
	 * @param Net_IMAP $imap
	 * @param string $rawresponse
	 * @return array
	 */
	private function imap_command_uid_fetch($msgset, $fetchparam, $imap, &$rawresponse) {
		if (!$imap->_connected) {
			return new PEAR_Error('not connected! (CMD:$command)');
		}
		
		$cmdid = $imap->_getCmdId();
		$imap->_putCMD($cmdid, 'UID FETCH', sprintf("%s %s", $msgset, $fetchparam));
		$args = $imap->_getRawResponse($cmdid);
		$rawresponse = $args;
		//Logger::log_r($args);
		return $imap->_genericImapResponseParser($args, $cmdid);
	}
	
	/**
	 * Returns a clean array with every uid specified in the ranges
	 * e.g.: "7,15,18:23" returns array(7,15,18,19,20,21,22,23)
	 * @param string $ranges_str
	 */
	private function expand_imap_uid_ranges($ranges_str) {
		
		$all_uids = array();
		$ranges = explode(',', $ranges_str);
		
		foreach ($ranges as $range) {
			if (strpos($range, ":") !== false) {
				$exploded = explode(":", $range);
				$s = $exploded[0];
				$e = $exploded[1];
				for ($i=$s; $i<=$e; $i++) $all_uids[] = $i;
			} else {
				$all_uids[] = $range;
			}
		}
		
		return array_filter(array_unique($all_uids));
	}
	
	/**
	 * Retrieves the first uid stored in feng since a date
	 * @param MailAccount $account
	 * @param MailAccountImapFolder $mailbox
	 * @param string $sync_from_date_sql
	 */
	private function get_first_uid_since_date($account, $mailbox, $sync_from_date_sql) {
		$first_uid = null;
		
		$sql = "SELECT f.uid FROM ".TABLE_PREFIX."mail_content_imap_folders f
				INNER JOIN ".TABLE_PREFIX."mail_contents m ON m.object_id=f.object_id
				WHERE f.account_id='".$account->getId()."' AND f.folder=".DB::escape($mailbox->getFolderName())."
					AND m.received_date > '$sync_from_date_sql'";
		
		$first_uid_row = DB::executeOne($sql);
		if ($first_uid_row && $first_uid_row['uid']) {
			$first_uid = $first_uid_row['uid'];
		}
		
		return $first_uid;
	}
	
	/**
	 * Get the user ids with permissions to the account $account
	 * @param MailAccount $account
	 */
	private function get_mail_account_user_ids($account) {
		$account_user_ids = array_var($this->account_users_cache, $account->getId());
		if (is_null($account_user_ids)) {
			$macs = MailAccountContacts::instance()->getByAccount($account);
			$account_user_ids = array();
			foreach ($macs as $mac) $account_user_ids[] = $mac->getContactId();
		
			$this->account_users_cache[$account->getId()] = $account_user_ids;
		}
		
		return $account_user_ids;
	}
	
	
	
	
	
	/**
	 * Removes from feng folder the emails removed from the folder in the server
	 * @param MailAccount $account
	 * @param MailAccountImapFolder $mailbox
	 * @param array $vanished_uids
	 */
	private function remove_vanished_mails($account, $mailbox, $vanished_uids, $imap) {
		
		/* 
		 * If mail is in any other imap folder:
		 *     remove from mail_contents_imap_folders for this folder
		 *     unclassify from imap folder member
		 * else:
		 *     delete the email
		 */
		
		if (!is_array($vanished_uids) || count($vanished_uids)==0) return 0;
		
		$affected_rows = 0;
		
		$sql = "
			SELECT f.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders f
			WHERE f.folder=".DB::escape($mailbox->getFolderName())." AND f.account_id=".$account->getId()."
				AND f.uid IN (".implode(',', $vanished_uids).")
		";
		$vanished_object_id_rows = DB::executeAll($sql);
		$vanished_object_ids = array_unique(array_filter(array_flat($vanished_object_id_rows)));
		
		$current_folder_member = null;
		if (PLugins::instance()->isActivePlugin('advanced_mail_imap_folders')) {
			$folders_dim = Dimensions::findByCode('mail_imap_folders');
			$folder_mem_obj = MailImapFolderMembers::findOne(array(
				"conditions" => "account_id='".$account->getId()."' AND folder_name=".DB::escape($mailbox->getFolderName()).""
			));
			if ($folder_mem_obj instanceof MailImapFolderMember) {
				$current_folder_member = Members::findOneByObjectId($folder_mem_obj->getId(), $folders_dim->getId());
			}
		}
		
		$account_user = Contacts::findById($account->getContactId());
		
		$vanished_objects = array();
		if (count($vanished_object_ids) > 0) {
			$vanished_objects = MailContents::findAll(array("conditions" => "id IN (".implode(',', $vanished_object_ids).")"));
		}
		
		$from_date = null;
		if ($account->getDelFromServer() > 0) {
			$del_in_feng_old_mails_when_del_in_server = $account->getColumnValue('sync_old_deleted_in_server_delete_in_feng');
			if (!$del_in_feng_old_mails_when_del_in_server) {
				$from_date = DateTimeValueLib::now();
				$from_date->add('d', -1 * $account->getDelFromServer());
			}
		}
		if (is_null($from_date)) {
			$from_date = DateTimeValueLib::now();
			$from_date->add('d', -1 * $this->days_to_retrieve_changes);
		}
		
		$archive_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Archive"))
		);
		$trash_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Trash"))
		);
		$all_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\All"))
		);
		$inbox_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND folder_name LIKE 'INBOX'", $account->getId()))
		);
		
		foreach ($vanished_objects as $mail) { /* @var $mail MailContent */
			// if $from_date is set => dont delete mails older than this date
			if ($from_date instanceof DateTimeValue && $from_date->getTimestamp() >= $mail->getReceivedDate()->getTimestamp()) {
				continue;
			}
			
			$mail_modified = false;
			if ($mail->isArchived() && $mailbox->getSpecialUse() != "\\Archive") {
				continue;
			}
			
			$oid = $mail->getId();
			
			$count_sql = "SELECT count(f.uid) as cant FROM ".TABLE_PREFIX."mail_content_imap_folders f 
				WHERE f.object_id='$oid' AND f.account_id='".$account->getId()."' AND folder <> '' AND f.folder<>".DB::escape($mailbox->getFolderName())."";
			$count_row = DB::executeOne($count_sql);
			
			$exists_in_other_folder = array_var($count_row, 'cant') > 0;
			
			if ($exists_in_other_folder) {
				// unclassify
				if ($current_folder_member instanceof Member) {
					ObjectMembers::removeObjectFromMembers($mail, $account_user, null, array($current_folder_member->getId()), false);
				}
				
				$delete_the_mail = false;
				$mail_modified = true;
				
			} else {
				// if not in any other feng folder then delete the email because
				// the user deleted the email or sent it to a folder not being downloaded in feng
				$delete_the_mail = true;
				$mail_modified = true;
			}
			
			// remove from mail_content_imap_folders for this folder
			DB::execute("DELETE FROM ".TABLE_PREFIX."mail_content_imap_folders
				WHERE object_id='$oid' AND account_id='".$account->getId()."' AND folder=".DB::escape($mailbox->getFolderName())."");
			
			
			// TRASH & UNTRASH
			$mail_is_trashed = false;
			if ($trash_folder instanceof MailAccountImapFolder) {
			// if mail is trahsed -> send to trash in feng
                if (!$mail->isTrashed() && $mailbox->getFolderName() != $trash_folder->getFolderName()) {
				//if exists in \Trash then it is trashed
				$imap->selectMailbox(utf8_encode($trash_folder->getFolderName()));
				$uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
				$is_trashed = count($uids) > 0;
				if ($is_trashed && !$mail->isTrashed()) {
					$mail->trash(null, false);
					$new_uid = $uids[0];
					$sql = "INSERT INTO ".TABLE_PREFIX."mail_content_imap_folders (account_id, message_id, folder, uid, object_id) VALUES
							(".$account->getId().",".DB::escape($mail->getMessageId()).",".DB::escape($trash_folder->getFolderName()).", '$new_uid', ".$mail->getId().")
							ON DUPLICATE KEY UPDATE uid='$new_uid'";
					DB::execute($sql);
					
					$delete_the_mail = false;
					$mail_modified = true;
					
					$mail_is_trashed = true;
				}
			}
			
                // if mail is trahsed in feng and is removed from tash folder in server -> untrash mail in feng
                if ($mail->isTrashed() && $mailbox->getFolderName() == $trash_folder->getFolderName() && $exists_in_other_folder) {
                        $mail->untrash(false);

                        $delete_the_mail = false;
                        $mail_modified = true;

                        $mail_is_trashed = false;

                }
			}



			// check if it is archived, if archived mark as archived in feng
			if (!$mail_is_trashed) {
				
				if (!$mail->isArchived() && !$archive_folder instanceof MailAccountImapFolder) {
					
					if ($all_folder instanceof MailAccountImapFolder) {
						$imap->selectMailbox(utf8_encode($all_folder->getFolderName()));
						$uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
						$is_in_all_folder = count($uids) > 0;
					} else {
						$is_in_all_folder = true;
					}
					
					// if server does not use archive folder, then check if it has been removed from inbox, if not then archive it
					if ($is_in_all_folder && $inbox_folder instanceof MailAccountImapFolder 
							&& $inbox_folder->getFolderName() == $mailbox->getFolderName() 
							&& $account->getColumnValue('sync_removed_from_inbox_archive_in_feng')) {
								// if removed from inbox => archive the email in feng
								$mail->archive(null, false);
								$delete_the_mail = false;
								$mail_modified = true;
						
					}
					
				} else if (!$mail->isArchived() && $archive_folder instanceof MailAccountImapFolder
						&& $mailbox->getFolderName() != $archive_folder->getFolderName()) {
							
					//if exists in \Archive then it is archived
					$imap->selectMailbox(utf8_encode($archive_folder->getFolderName()));
					$uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
					$is_archived = count($uids) > 0;
					if ($is_archived && !$mail->isArchived()) {
						$mail->archive(null, false);
						$new_uid = $uids[0];
						$sql = "INSERT INTO ".TABLE_PREFIX."mail_content_imap_folders (account_id, message_id, folder, uid, object_id) VALUES
								(".$account->getId().",".DB::escape($mail->getMessageId()).",".DB::escape($archive_folder->getFolderName()).", '$new_uid', ".$mail->getId().")
								ON DUPLICATE KEY UPDATE uid='$new_uid'";
						DB::execute($sql);
						
						$delete_the_mail = false;
						$mail_modified = true;
					}
					
				}
			}
			
			
			if ($delete_the_mail) {
				$mail->delete(true);
			}
			if ($mail_modified) {
				$affected_rows += 1;
			}
		}
		//file_put_contents(CACHE_DIR."/sync.txt", date("H:i:s")." - ".__FUNCTION__." ($affected_rows)\n", FILE_APPEND);
		
		return $affected_rows;
	}
	
	
	
	/**
	 * Marks as read in feng the emails marked as read in the server
	 * @param MailAccount $account
	 * @param MailAccountImapFolder $mailbox
	 * @param array $vanished_uids
	 */
	private function mark_mails_as_read($account, $mailbox, $seen_uids) {
		
		$affected_rows = 0;
		
		if (is_array($seen_uids) && count($seen_uids) > 0) {
			$sql = "
				SELECT f.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders f
				WHERE f.folder=".DB::escape($mailbox->getFolderName())." AND f.account_id=".$account->getId()."
					AND f.uid IN (".implode(',', $seen_uids).")
			";
			$seen_object_id_rows = DB::executeAll($sql);
			$seen_object_ids = array_unique(array_filter(array_flat($seen_object_id_rows)));
			
			$account_user_ids = $this->get_mail_account_user_ids($account);
			
			if (count($seen_object_ids) > 0) {
				$sql = "INSERT INTO ".TABLE_PREFIX."read_objects (rel_object_id, contact_id, is_read, created_on) VALUES ";
				$sql_vals = array();
				foreach ($seen_object_ids as $id) {
					foreach ($account_user_ids as $user_id) {
						$sql_vals[] = "('$id', '".$user_id."', 1, NOW())";
					}
				}
				$sql .= implode(',', $sql_vals);
				$sql .= " ON DUPLICATE KEY UPDATE is_read=1";
				DB::execute($sql);
				
				$affected_rows = DB::affectedRows();
			}
		}
		//file_put_contents(CACHE_DIR."/sync.txt", date("H:i:s")." - ".__FUNCTION__." ($affected_rows)\n", FILE_APPEND);
		
		return $affected_rows;
	}
	
	
	/**
	 * Marks as unread in feng the emails marked as unread in the server
	 * @param MailAccount $account
	 * @param MailAccountImapFolder $mailbox
	 * @param array $vanished_uids
	 */
	private function mark_mails_as_unread($account, $mailbox, $unseen_uids) {
		
		$affected_rows = 0;
		
		if (is_array($unseen_uids) && count($unseen_uids) > 0) {
			$sql = "
				SELECT f.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders f
				WHERE f.folder=".DB::escape($mailbox->getFolderName())." AND f.account_id=".$account->getId()."
					AND f.uid IN (".implode(',', $unseen_uids).")
			";
			$unseen_object_id_rows = DB::executeAll($sql);
			$unseen_object_ids = array_unique(array_filter(array_flat($unseen_object_id_rows)));
		
			if (count($unseen_object_ids) > 0) {
				DB::executeAll("DELETE FROM ".TABLE_PREFIX."read_objects
					WHERE rel_object_id IN (".implode(',', $unseen_object_ids).")");
				
				$affected_rows = DB::affectedRows();
			}
			
		}
		//file_put_contents(CACHE_DIR."/sync.txt", date("H:i:s")." - ".__FUNCTION__." ($affected_rows)\n", FILE_APPEND);
		
		return $affected_rows;
	}

	
	
	
	
	/*************************************************************************/
	
	
	
	/**
	 * Marks as read or unread the selected emails in the mail server
	 * @param array $ids_to_mark Array of feng ids of the emails to mark
	 * @param boolean $is_read true to mark as read, false to unread
	 */
	function mark_as_read_unread_in_mail_server($ids_to_mark, $is_read) {
		
		if (is_array($ids_to_mark) && count($ids_to_mark) > 0) {
			// Group the mails by its account id
			$rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."mail_content_imap_folders 
					WHERE object_id IN (".implode(',', $ids_to_mark).")");
			
			$grouped_by_account = array();
			foreach ($rows as $row) {
				if (!isset($grouped_by_account[$row['account_id']])) $grouped_by_account[$row['account_id']] = array();
				if (!isset($grouped_by_account[$row['account_id']][$row['folder']])) $grouped_by_account[$row['account_id']][$row['folder']] = array();
				$grouped_by_account[$row['account_id']][$row['folder']][] = $row['uid'];
			}
			
			// mark as read/unread foreach account
			foreach ($grouped_by_account as $account_id => $uids_by_folder) {
				
				$account = MailAccounts::instance()->findById($account_id);
				if ($account->getIsImap()) {
					$imap = $account->imapConnect();
					$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
					if (PEAR::isError($ret)) {
						Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
						continue;
					}
				
					// foreach folder send the imap command to add/remove the \Seen flag for the uids
					foreach ($uids_by_folder as $folder => $uids) {
						$folder_name = utf8_decode($folder);
						$imap->selectMailbox($folder_name);
						
						sort($uids, SORT_NUMERIC);
						$uids_str = implode(',', $uids);
						
						$mode = $is_read ? 'add' : 'remove';
						
						$ret = $imap->setFlags($uids_str, "\\Seen", $mode, true);
						
						if (PEAR::isError($ret)) {
							Logger::log_r("Mail sync - ERROR marking uids as ".($is_read?"read":"unread")." in server: \nFOLDER:$folder\nUIDS:$uids_str\n".print_r($ret,1));
							continue;
						}
					}
					
					$imap->disconnect();
					
					// mark as read/unread for all users with permissions in the account
					if ($is_read) {
						// add the record in read_objects table for each user with permissions
						$account_user_ids = $this->get_mail_account_user_ids($account);
						
						if (count($ids_to_mark) > 0) {
							$sql = "INSERT INTO ".TABLE_PREFIX."read_objects (rel_object_id, contact_id, is_read, created_on) VALUES ";
							$sql_vals = array();
							foreach ($ids_to_mark as $id) {
								foreach ($account_user_ids as $user_id) {
									$sql_vals[] = "('$id', '".$user_id."', 1, NOW())";
								}
							}
							$sql .= implode(',', $sql_vals);
							$sql .= " ON DUPLICATE KEY UPDATE is_read=1";
							DB::execute($sql);
						}
						
					} else {
						// remove records in read_objects table foreach users with permissions
						if (count($ids_to_mark) > 0) {
							DB::executeAll("DELETE FROM ".TABLE_PREFIX."read_objects
								WHERE rel_object_id IN (".implode(',', $ids_to_mark).")");
						}
					}
				}
			}
			
		}
		
	}
	
	
	
	
	function change_imap_folders_in_mail_server($mail, $previous_member_ids, $new_member_ids, $remove_from_imbox, $process_conversation=false, $conversation=null) {
		
		if (PLugins::instance()->isActivePlugin('advanced_mail_imap_folders')) {
			
			$account = MailAccounts::instance()->findById($mail->getAccountId());
			if ($account instanceof MailAccount && $account->getIsImap()) {
				$imap = $account->imapConnect();
				$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
				if (PEAR::isError($ret)) {
					Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
					return;
				}
				
				// important to initialize UIDPLUS capability (so new uids are returned properly)
				$capabilites = $imap->cmdCapability();
				//Logger::log_r($capabilites);
			
				$folders_dim = Dimensions::findByCode('mail_imap_folders');
				
				$member_ids_to_add = array_diff($new_member_ids, $previous_member_ids);
				$member_ids_to_remove = array_diff($previous_member_ids, $new_member_ids);
				
				$folder_members_to_add = array();
				$folder_members_to_remove = array();
				$folders_to_add = array();
				$folders_to_remove = array();
				
				if (is_array($member_ids_to_add) && count($member_ids_to_add) > 0) {
					$new_members = Members::findAll(array('conditions' => "dimension_id=".$folders_dim->getId().
							" AND id IN (".implode(',', $member_ids_to_add).")"));
					if (is_array($new_members)) {
						foreach ($new_members as $m) $folder_members_to_add[] = $m;
					}
				}
				
				if (is_array($member_ids_to_remove) && count($member_ids_to_remove) > 0) {
					$prev_members = Members::findAll(array('conditions' => "dimension_id=".$folders_dim->getId().
							" AND id IN (".implode(',', $member_ids_to_remove).")"));
					if (is_array($prev_members)) {
						foreach ($prev_members as $m) $folder_members_to_remove[] = $m;
					}
				}
				
				foreach ($folder_members_to_add as $m) {
					$mem_obj = Objects::findObject($m->getObjectId());
					if ($mem_obj instanceof ContentDataObject) {
						$fname = $mem_obj->getColumnValue('folder_name');
						if ($fname) $folders_to_add[] = $fname;
					}
				}
				
				foreach ($folder_members_to_remove as $m) {
					$mem_obj = Objects::findObject($m->getObjectId());
					if ($mem_obj instanceof ContentDataObject) {
						$fname = $mem_obj->getColumnValue('folder_name');
						if ($fname) $folders_to_remove[] = $fname;
					}
				}
				
				// remove from inbox only if classified in another imap folder
				if ($remove_from_imbox && count($folders_to_add) > 0) {
					$inbox_folder = MailAccountImapFolders::instance()->findOne(
						array("conditions" => array("account_id=? AND folder_name LIKE 'INBOX'", $account->getId()))
					);
					if ($inbox_folder instanceof MailAccountImapFolder) {
						$folders_to_remove[] = $inbox_folder->getFolderName();
					}
					// if removed from inbox are archived when synchronizing => archive this email
					if ($account->getColumnValue('sync_removed_from_inbox_archive_in_feng')) {
						$mail->archive();
					}
				}
				
				// copy to new folders
				$this->copy_mail_to_imap_folders($account, $imap, $mail, $folders_to_add);
				
				// remove from old folders
				$this->remove_mail_from_imap_folders($account, $imap, $mail, $folders_to_remove);
				
				
			}
		}
		
	}
	


	function remove_mail_from_imap_folders($account, $imap, $mail, $folders_to_remove) {
		// Remove the email from old folders
		foreach ($folders_to_remove as $folder_name) {
			// if for any reason we have more than one uid for the same email in the same folder try to delete all of them
			$uids_in_folder_rows = DB::executeAll("SELECT uid FROM ".TABLE_PREFIX."mail_content_imap_folders
						WHERE object_id=".$mail->getId()." AND folder=".DB::escape($folder_name));
				
			$folder = utf8_decode($folder_name);
			$imap->selectMailbox($folder);
				
			$uids_in_folder = array_filter(array_flat($uids_in_folder_rows));
			
			if (!$uids_in_folder || count($uids_in_folder) == 0) {
				$uids_in_folder = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
				if (PEAR::isError($uids_in_folder)) {
					$uids_in_folder = array();
				}
			}
				
			if (is_array($uids_in_folder) && count($uids_in_folder)) {
				$uids_str = implode(',', $uids_in_folder);
				$result = $imap->deleteMessages($uids_str, true);
				$imap->expunge();
		
				// delete regs in mail_content_imap_folders for this folder
				$sql = "DELETE FROM ".TABLE_PREFIX."mail_content_imap_folders
						WHERE object_id=".$mail->getId()." AND folder=".DB::escape($folder_name);
				DB::executeAll($sql);
					
			}
		}
	}
	
	
	
	function copy_mail_to_imap_folders($account, $imap, $mail, $folders_to_add) {
		
		$source_row = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."mail_content_imap_folders
						WHERE object_id=".$mail->getId());
		
		// Add the email to the new folders
		if ($source_row) {
			$source_folder = utf8_decode($source_row['folder']);
			$source_uid = $source_row['uid'];
				
			if ($source_folder && $source_uid) {
				// copy to destiny folders
				$imap->selectMailbox($source_folder);
				
				foreach ($folders_to_add as $folder_name) {
					$dest_folder = utf8_decode($folder_name);
					
					// check if this message already exists in destiny folder
					$imap->selectMailbox($dest_folder);
					$exist_uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
					$exists_in_folder = is_array($exist_uids) && count($exist_uids) > 0;
					
                    $imap->selectMailbox($source_folder);

					// if not exists -> copy it
					if (!$exists_in_folder) {
						$ret = $imap->cmdUidCopy($source_uid, $dest_folder);
						
						$response_string = $ret['RESPONSE']['STR_CODE']; // [COPYUID 1500469762 18 14] Copy completed (0.003 + 0.000 + 0.002 secs).
						$matches = array();
						preg_match_all("/\[COPYUID (.*) (.*) (.*)\]/", $response_string, $matches);
						$new_uid = array_var($matches[3], 0);
						




						// update feng tables
						if ($ret && !Pear::isError($ret)) {
							$this->fill_mail_content_imap_folder_table($imap, $account, $mail, $folder_name, $new_uid);
						}
					} else {
						// if already exists -> update feng info
						$this->fill_mail_content_imap_folder_table($imap, $account, $mail, $folder_name, $exist_uids[0]);
					}
				}
			} else {
				// append to destiny folders
				foreach ($folders_to_add as $folder_name) {
					$dest_folder = utf8_decode($folder_name);
					
					// check if this message already exists in destiny folder
					$imap->selectMailbox($dest_folder);
					$exist_uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
					$exists_in_folder = is_array($exist_uids) && count($exist_uids) > 0;
					
                    $imap->selectMailbox($source_folder);

					// if not exists -> append it
					if (!$exists_in_folder) {
						$eml = $mail->getContent();
						$ret = $imap->cmdAppend($dest_folder, $eml);
						
						$response_string = $ret['RESPONSE']['STR_CODE']; // [APPENDUID 1500469762 18] Copy completed (0.003 + 0.000 + 0.002 secs).
						$matches = array();
						preg_match_all("/\[APPENDUID (.*) (.*)\]/", $response_string, $matches);
						$new_uid = array_var($matches[2], 0);
						
						// update feng tables
						if ($ret && !Pear::isError($ret)) {
							$this->fill_mail_content_imap_folder_table($imap, $account, $mail, $folder_name, $new_uid);
						}
					} else {
						// if already exists -> update feng info
						$this->fill_mail_content_imap_folder_table($imap, $account, $mail, $folder_name, $exist_uids[0]);
					}
				}
			}
			
		}
		
	}
	
	private function fill_mail_content_imap_folder_table($imap, $account, $mail, $folder_name, $new_uid) {
		if (!$new_uid) {
			$imap->selectMailbox(utf8_decode($folder_name));
			$uids = $imap->search('HEADER Message-ID '.$mail->getMessageId(), true);
			$new_uid = array_var($uids, 0);
		}
		
		if ($new_uid) {
			$sql = "INSERT INTO ".TABLE_PREFIX."mail_content_imap_folders (account_id, message_id, folder, uid, object_id) VALUES
						(".$account->getId().",".DB::escape($mail->getMessageId()).",".DB::escape($folder_name).", '$new_uid', ".$mail->getId().")
					ON DUPLICATE KEY UPDATE uid='$new_uid'";
			DB::execute($sql);
			DB::execute("DELETE FROM ".TABLE_PREFIX."mail_content_imap_folders
					WHERE account_id=".$account->getId()." AND object_id=".$mail->getId()." AND folder=''");
		}
		
	}
	
	
	
	function create_mailbox($account, $mailbox_name, $parent_object, &$new_mailbox_name) {
		
		$imap = $account->imapConnect();
		$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
		if (PEAR::isError($ret)) {
			Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
			return;
		}
		
		$mu = new MailUtilities();
		$can_detect_special_folders = false;
		$mbox_list = $mu->get_imap_account_mailboxes($account, $can_detect_special_folders, $imap);
		$delimiter = count($mbox_list) > 0 ? $mbox_list[0]['delimiter'] : '.';
		if ($parent_object instanceof MailImapFolderMember) {
			$mailbox_name = $parent_object->getFolderName() . $delimiter . $mailbox_name;
		}
		$new_mailbox_name = $mailbox_name;
		
		$created = $imap->createMailbox($mailbox_name);
		if (PEAR::isError($created)) {
			$needs_inbox_prefix_text = "Mailbox name should probably be prefixed with: INBOX.";
			if (strpos($created->getMessage(), $needs_inbox_prefix_text) !== false) {
				$new_mailbox_name = "INBOX." . $mailbox_name;
				$created = $imap->createMailbox($new_mailbox_name);
			}
		}
		
		if (!PEAR::isError($created)) {
			// some servers need this to activate the folder
			$subscribed = $imap->subscribeMailbox($new_mailbox_name);
		}
		
		$imap->disconnect();
		
		if (PEAR::isError($created)) {
			Logger::log_r($created->getMessage());
			flash_error($created->getMessage());
			return;
		}
		
		if (!PEAR::isError($created)) {
			DB::execute("INSERT INTO ".TABLE_PREFIX."mail_account_imap_folder (account_id,folder_name,check_folder)
						VALUES ('".$account->getId()."',".DB::escape($new_mailbox_name).",1)
						ON DUPLICATE KEY UPDATE account_id=account_id;");
		}
		
		return $created;
	}
	
	
	function delete_mailbox($account, $mailbox_name) {
		$imap = $account->imapConnect();
		$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
		if (PEAR::isError($ret)) {
			Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
			return;
		}
		
		$deleted = $imap->deleteMailbox($mailbox_name);
		$imap->disconnect();
		
		if (PEAR::isError($deleted)) {
			Logger::log_r($deleted->getMessage());
			flash_error($deleted->getMessage());
			return false;
		}
		
		if (!PEAR::isError($deleted)) {
			DB::execute("DELETE FROM ".TABLE_PREFIX."mail_account_imap_folder 
				WHERE folder_name=".DB::escape($mailbox_name)." AND account_id=".$account->getId());
			
			DB::execute("DELETE FROM ".TABLE_PREFIX."mail_content_imap_folders 
				WHERE folder=".DB::escape($mailbox_name)." AND account_id=".$account->getId());
		}
	}
	
	
	function rename_mailbox($account, $old_mailbox_name, $new_mailbox_name) {
		
		$imap = $account->imapConnect();
		$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
		if (PEAR::isError($ret)) {
			Logger::log_r("Mail sync - ".__FUNCTION__." -  LOGIN ERROR\n".print_r($ret,1));
			return;
		}
		
		$renamed = $imap->renameMailbox($old_mailbox_name, $new_mailbox_name);
		if (PEAR::isError($renamed)) {
			$needs_inbox_prefix_text = "Mailbox name should probably be prefixed with: INBOX.";
			if (strpos($renamed->getMessage(), $needs_inbox_prefix_text) !== false) {
				$renamed = $imap->renameMailbox($old_mailbox_name, "INBOX." . $new_mailbox_name);
				$new_mailbox_name = "INBOX." . $new_mailbox_name;
			}
		}
		
		if (!PEAR::isError($renamed)) {
			// some servers need this to activate the folder
			$subscribed = $imap->subscribeMailbox($new_mailbox_name);
		}
		
		$imap->disconnect();
		
		if (PEAR::isError($renamed)) {
			Logger::log_r($renamed->getMessage());
			flash_error($renamed->getMessage());
			return false;
		}
		
		if (!PEAR::isError($renamed)) {
			// update fo_mail_account_imap_folder
			DB::execute("UPDATE ".TABLE_PREFIX."mail_account_imap_folder SET folder_name=".DB::escape($new_mailbox_name)." 
					WHERE folder_name=".DB::escape($old_mailbox_name)." AND account_id=".$account->getId());
			// update all in fo_mail_content_imap_folders
			DB::execute("UPDATE ".TABLE_PREFIX."mail_content_imap_folders SET folder=".DB::escape($new_mailbox_name)." 
					WHERE folder=".DB::escape($old_mailbox_name)." AND account_id=".$account->getId());
			
		}
		
		return $renamed;
	}
	
	
	



	private function check_for_mailbox_changes($account, $imap) {
		$mu = new MailUtilities();
		
		$all_server_mailboxes = array();
		$ignored = false;
		try {
			$all_server_mailboxes = $mu->get_imap_account_mailboxes($account, $ignored, $imap);
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__.": when getting imap account mailboxes: ".$e->getMessage());
		}
		
		$current_feng_mailboxes = MailAccountImapFolders::getMailAccountImapFolders($account->getId());
		$current_feng_mailbox_names = array();
		foreach ($current_feng_mailboxes as $feng_mailbox) {
			$current_feng_mailbox_names[] = $feng_mailbox->getFolderName();
		}
		
		// insert the new mailboxes records in feng
		$new_mailbox_names = array();
		$all_server_mailbox_names = array();
		
		foreach ($all_server_mailboxes as $server_mailbox) {
			if ($server_mailbox['special_use'] == "") {
				
				$all_server_mailbox_names[] = $server_mailbox['name'];
				
				if (!in_array($server_mailbox['name'], $current_feng_mailbox_names)) {
					$new_mailbox_names[] = $server_mailbox['name'];
					
					DB::execute("INSERT INTO ".TABLE_PREFIX."mail_account_imap_folder (account_id,folder_name,check_folder)
							VALUES ('".$account->getId()."',".DB::escape($server_mailbox['name']).",1)
							ON DUPLICATE KEY UPDATE account_id=account_id;");
				}
			}
		}
	
		// create the members
		$null = null; Hook::fire("after_mail_account_save", array('account' => $account, 'is_new' => false, 'save_permissions' => false), $null);
		
		if (count($new_mailbox_names) > 0) {
			// download some mails of new folders
			$mu->getNewImapMails($account, config_option('user_email_fetch_count'), $new_mailbox_names);
		}
	
		$deleted_mailboxes = array();
		foreach ($current_feng_mailboxes as $feng_mailbox) {
			
			if ($feng_mailbox->getSpecialUse() == "" && !in_array($feng_mailbox->getFolderName(), $all_server_mailbox_names)) {
				$deleted_mailboxes[] = $feng_mailbox->getFolderName();
				$this->delete_feng_imap_folder($account, $feng_mailbox->getFolderName(), true);
	
			}
		}
	
		$dim = Dimensions::findByCode("mail_imap_folders");
		if ($dim instanceof Dimension && (count($new_mailbox_names) > 0 || count($deleted_mailboxes) > 0)) {
			evt_add("reload dimension tree", array('dim_id' => $dim->getId()));
		}
	
		return array('new_mailbox_names' => $new_mailbox_names, 'deleted_mailboxes' => $deleted_mailboxes);
			
	}
	
	private function delete_feng_imap_folder($account, $mailbox_name, $delete_member = false) {
	
		DB::execute("DELETE FROM ".TABLE_PREFIX."mail_account_imap_folder
			WHERE account_id=".$account->getId()." AND folder_name=".DB::escape($mailbox_name));
	
		DB::execute("DELETE FROM ".TABLE_PREFIX."mail_content_imap_folders
			WHERE account_id=".$account->getId()." AND folder=".DB::escape($mailbox_name));
		
		$mail_ids_with_no_folder_rows = DB::executeAll("
			SELECT mc.object_id FROM ".TABLE_PREFIX."mail_contents mc
			WHERE mc.account_id=".$account->getId()." AND NOT EXISTS (
				SELECT mcf.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders mcf WHERE mcf.object_id=mc.object_id
			)
		");
		
		// for mails that doesn't have a folder in feng create a dummy record in mail_content_imap_folders
		$mail_ids_with_no_folder = array_filter(array_flat($mail_ids_with_no_folder_rows));
		if (count($mail_ids_with_no_folder)) {
			$mails = MailContents::findAll(array("conditions" => "id IN (".implode(',',$mail_ids_with_no_folder).")"));
			
			$sql = "INSERT INTO ".TABLE_PREFIX."mail_content_imap_folders (account_id, message_id, object_id) VALUES ";
			$vals = array();
			foreach ($mails as $mail) $vals[] = "(".$account->getId().",".DB::escape($mail->getMessageId()).",".$mail->getId().")";
			$sql .= implode(',', $vals);
			$sql .= " ON DUPLICATE KEY UPDATE account_id=account_id;";
			DB::execute($sql);
		}
		
		if ($delete_member) {
			Hook::fire('after_delete_feng_imap_folder', array('account' => $account, 'folder' => $mailbox_name));
		}
	
	}
	
	
	
	function show_capabilities() {
		ajx_current("empty");
		
		if (is_numeric(array_var($_REQUEST, 'account_id'))) {
			$account = MailAccounts::findById(array_var($_REQUEST, 'account_id'));
		} else {
			$account = MailAccounts::findById($account_id);
		}
		if (!$account instanceof MailAccount) {
			return;
		}
		
		
		
		$imap = $account->imapConnect();
		$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
		if (PEAR::isError($ret)) {
			Logger::log_r("Mail sync - LOGIN ERROR");
			Logger::log_r($ret);
			return;
		}
		
		$capabilites = $imap->cmdCapability();
		
		$mailboxes = $mu->get_imap_account_mailboxes($account);
		
		$imap->disconnect();
	}
	
	
	

} // ImapSynchronizationController

