<?php
Hook::register("mail_imap_sync_hooks");

function mail_imap_sync_hooks_object_definition($type, &$ret) {
	if ($type == 'MailAccountImapFolder') {
		$ret['last_uidvalidity'] = DATA_TYPE_STRING;
		$ret['last_highestmodseq'] = DATA_TYPE_STRING;
	}
	
	if ($type == 'MailAccountContact') {
		$ret['need_ui_update'] = DATA_TYPE_INTEGER;
	}

	if ($type == 'MailAccount') {
		$ret['sync_archive_remove_from_inbox_in_server'] = DATA_TYPE_BOOLEAN;
		$ret['sync_classify_remove_from_inbox_in_server'] = DATA_TYPE_BOOLEAN;
		$ret['sync_removed_from_inbox_archive_in_feng'] = DATA_TYPE_BOOLEAN;
		$ret['sync_old_deleted_in_server_delete_in_feng'] = DATA_TYPE_BOOLEAN;
	}
}


function mail_imap_sync_hooks_more_mail_account_imap_options($params, &$return) {
	$account = array_var($params, 'account');
	$genid = array_var($params, 'genid');
	/* @var $account MailAccount */

	tpl_assign('account', $account);
	tpl_assign('genid', $genid);
	
	tpl_display(get_template_path('account_options', 'mail_account', 'advanced_mail_imap_sync'));
}


function mail_imap_sync_hooks_after_get_new_imap_mails($params, &$return) {
	$account = array_var($params, 'account');
	/* @var $account MailAccount */
	
	if ($account->getIsImap()) {
		
		$folders = MailAccountImapFolders::instance()->findAll(array("conditions" => array("account_id=? AND check_folder=1", $account->getId())));
		
		if (count($folders) < 30) {
			if (!logged_user() instanceof Contact) {
				CompanyWebsite::instance()->logUserIn(Contacts::findById($account->getContactId()));
			}
			
			$sync_controller = new ImapSynchronizationController();
			$sync_controller->search_imap_folder_changes($account->getId());
			
		} else {
			// if there are more than 30 folders -> execute in background
			$null = null;
			mail_imap_sync_hooks_after_check_if_new_mails(array('accountId' => $account->getId()), $null);
		}
	}
}


function mail_imap_sync_hooks_after_check_if_new_mails($attributes, &$return) {
	$account_ids = explode(',', array_var($attributes, 'accountId'));
	$account_ids = array_filter($account_ids);
	$accounts = array();

	if (count($account_ids) > 0) {
		$accounts = MailAccounts::findAll(array('conditions' => "id IN (".implode(',', $account_ids).")"));
	} else {
		if (logged_user() instanceof Contact) {
			$macs = MailAccountContacts::instance()->getByContact(logged_user());
			foreach ($macs as $mac) {/* @var $mac MailAccountContact */
				$accounts[] = $mac->getAccount();
			}
		}
	}
	
	$need_ui_update = 0;
	$execute_in_background = is_exec_available();
	$account_ids_to_exec = array();
	
	foreach ($accounts as $account) {
		/* @var $account MailAccount */
		if ($account instanceof MailAccount && $account->getIsImap()) {
	
			if ($execute_in_background) {
				// account ids to sync in background
				$account_ids_to_exec[] = $account->getId();
				
				$user = logged_user() instanceof Contact ? logged_user() : Contacts::findById($account->getContactId());
				
				// if account is marked as "need_ui_update" then trigger event to update the ui for logged user
				$mac = MailAccountContacts::instance()->getByAccountAndContact($account, $user);
				$need_update = $mac->getColumnValue("need_ui_update");

				if ($need_update > $need_ui_update) {
					$need_ui_update = $need_update;
				}
				
			} else {
				$sync_controller = new ImapSynchronizationController();
				$sync_controller->search_imap_folder_changes($account->getId());
			}
			
		}
	}
	
	// execute synchronization in background, this process also marks the account to "need_ui_update"
	if ($execute_in_background && count($account_ids_to_exec) > 0) {
		
		Env::useHelper('functions', 'advanced_mail_imap_sync');
		$command = PHP_PATH." ". ROOT . "/public/mail_imap_sync.php " . implode(',', $account_ids_to_exec);
		execute_sync_script_in_background("mail_imap_sync.php", $command);
	}
	
	if ($need_ui_update > 0) {
		if (logged_user() instanceof Contact) {
			// reset the "need_ui_update" variable for logged user
			DB::execute("UPDATE ".TABLE_PREFIX."mail_account_contacts 
					SET need_ui_update=0 WHERE contact_id=".logged_user()->getId());
		}
		
		// trigger reload panel event
		if ($need_ui_update >= 1) {
			evt_add("update email list", array('ids' => array(-1)));
		}
		// trigger reload dimension event
		if ($need_ui_update >= 2) {
			$dim = Dimensions::findByCode("mail_imap_folders");
			if ($dim instanceof Dimension) {
				evt_add("reload dimension tree", array('dim_id' => $dim->getId()));
			}
		}
	}
}




function mail_imap_sync_hooks_do_mark_as_read_unread_objects($ids_to_mark, $read) {
	
	$sync_controller = new ImapSynchronizationController();
	$sync_controller->mark_as_read_unread_in_mail_server($ids_to_mark, $read);
	
}


function mail_imap_sync_hooks_after_classify_mail($params, &$return) {
	
	$mail = array_var($params, 'email');
	$previous_member_ids = array_var($params, 'previous_member_ids');
	$new_member_ids = array_var($params, 'new_member_ids');
	$process_conversation = array_var($params, 'process_conversation');
	$conversation = array_var($params, 'conversation');
	
	$account = MailAccounts::instance()->findById($mail->getAccountId());
	$remove_from_imbox = $account->getColumnValue("sync_classify_remove_from_inbox_in_server");
	
	$sync_controller = new ImapSynchronizationController();
	$sync_controller->change_imap_folders_in_mail_server($mail, $previous_member_ids, $new_member_ids, $remove_from_imbox, $process_conversation, $conversation);
	
}


function mail_imap_sync_hooks_after_dragdrop_classify($parameters, &$member) {
	
	$objects = array_var($parameters, 'objects');
	$prev_classification = array_var($parameters, 'prev_classification');
	
	$sync_controller = new ImapSynchronizationController();
	$accounts = array();
	
	foreach ($objects as $obj) {
		if ($obj instanceof MailContent) {
			$mail = $obj;
			$new_member_ids = $mail->getMemberIds();
			$previous_member_ids = array_var($prev_classification, $mail->getId());
			$process_conversation = false;
			$conversation = null;
			
			$account = array_var($accounts, $mail->getAccountId());
			if (!$account) {
				$account = MailAccounts::instance()->findById($mail->getAccountId());
				$accounts[$mail->getAccountId()] = $account;
			}
			$remove_from_imbox = $account->getColumnValue("sync_classify_remove_from_inbox_in_server");
			
			$sync_controller->change_imap_folders_in_mail_server($mail, $previous_member_ids, $new_member_ids, $remove_from_imbox, $process_conversation, $conversation);
		}
	}
}



function mail_imap_sync_hooks_after_content_object_trash($parameters, &$ignored) {
	$mail = array_var($parameters, 'object');
	if ($mail instanceof MailContent) {
		
		$account = MailAccounts::instance()->findById($mail->getAccountId());
		
		$trash_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Trash"))
		);
		if ($trash_folder instanceof MailAccountImapFolder) {
			
			$sync_controller = new ImapSynchronizationController();
			
			$imap = $account->imapConnect();
			$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
			if (PEAR::isError($ret)) return;
			
			$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, array($trash_folder->getFolderName()));
			
			$imap->disconnect();
			
		}
		
	}

}


function mail_imap_sync_hooks_after_object_controller_trash($ids, &$ignored) {
	if (!is_array($ids)) {
		$ids = explode(',', $ids);
	}
	$ids = array_filter($ids, 'is_numeric');
	if (count($ids) > 0) {
		$trashed_emails = MailContents::findAll(array("conditions" => "id IN (".implode(',', $ids).")"));
		$account_mails = array();
		foreach ($trashed_emails as $e) {
			if (!isset($account_mails[$e->getAccountId()])) {
				$account_mails[$e->getAccountId()] = array();
				$account_mails[$e->getAccountId()][] = $e;
			}
		}
		
		foreach ($account_mails as $account_id => $mails) {
				
			$account = MailAccounts::instance()->findById($account_id);
				
			$trash_folder = MailAccountImapFolders::instance()->findOne(
					array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Trash"))
			);
			if ($trash_folder instanceof MailAccountImapFolder) {
					
				$sync_controller = new ImapSynchronizationController();
					
				$imap = $account->imapConnect();
				$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
				if (PEAR::isError($ret)) return;

				foreach ($mails as $mail) {
					$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, array($trash_folder->getFolderName()));
				}
				
				$imap->disconnect();
			}
		}
	}
}


function mail_imap_sync_hooks_after_content_object_untrash($parameters, &$ignored) {
	$mail = array_var($parameters, 'object');
	if ($mail instanceof MailContent) {

		$account = MailAccounts::instance()->findById($mail->getAccountId());
		if ($account instanceof MailAccount && $account->getIsImap()) {
			
			$trash_folder = MailAccountImapFolders::instance()->findOne(
				array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Trash"))
			);
			if ($trash_folder instanceof MailAccountImapFolder) {
				$trash_name = $trash_folder->getFolderName();
				
				$other_rows = DB::executeAll("SELECT folder FROM ".TABLE_PREFIX."mail_content_imap_folders
						WHERE object_id=".$mail->getId(). " AND folder!=".DB::escape($trash_name)."");
				
				$folders_to_add = array();
				foreach ($other_rows as $or) {
					$folders_to_add[] = $or['folder'];
				}
				
				$sync_controller = new ImapSynchronizationController();
				
				$imap = $account->imapConnect();
				$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
				if (PEAR::isError($ret)) return;
				
				// copy to normal mail folders
				$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, $folders_to_add);
				
				// remove from trash
				$sync_controller->remove_mail_from_imap_folders($account, $imap, $mail, array($trash_name));
				
				$imap->disconnect();

			}
		}
	}

}



function mail_imap_sync_hooks_after_content_object_archive($parameters, &$ignored) {
	
	$mail = array_var($parameters, 'object');
	if ($mail instanceof MailContent) {
		$unclassify = false;
		
		$account = MailAccounts::instance()->findById($mail->getAccountId());
		if ($account instanceof MailAccount && $account->getIsImap()) {
			
			$archive_folder = MailAccountImapFolders::instance()->findOne(
				array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Archive"))
			);
			$inbox_folder = MailAccountImapFolders::instance()->findOne(
				array("conditions" => array("account_id=? AND folder_name LIKE 'INBOX'", $account->getId()))
			);
			if ($archive_folder instanceof MailAccountImapFolder) {
				$sync_controller = new ImapSynchronizationController();
				
				$imap = $account->imapConnect();
				$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
				if (PEAR::isError($ret)) return;
				
				// put in archived folder
				$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, array($archive_folder->getFolderName()));
				
				// remove from inbox folder
				if ($account->getColumnValue("sync_archive_remove_from_inbox_in_server")) {
					$current_folder_names = DB::executeAll("SELECT folder FROM ".TABLE_PREFIX."mail_content_imap_folders
							WHERE object_id=".$mail->getId()." AND folder = ".DB::escape($inbox_folder->getFolderName()));
					$current_folder_names = array_flat($current_folder_names);
					
					$sync_controller->remove_mail_from_imap_folders($account, $imap, $mail, $current_folder_names);
				}
				$unclassify = false;
				
				$imap->disconnect();
				
			} else {
				
				// unclassify and move mail to \All folder
				$all_folder = MailAccountImapFolders::instance()->findOne(
					array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\All"))
				);
				if ($all_folder instanceof MailAccountImapFolder) {
					$sync_controller = new ImapSynchronizationController();
					
					$imap = $account->imapConnect();
					$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
					if (PEAR::isError($ret)) return;
					
					// put in all folder
					$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, array($all_folder->getFolderName()));
					
					// remove from inbox folder
					if ($account->getColumnValue("sync_archive_remove_from_inbox_in_server")) {
						$current_folder_names = DB::executeAll("SELECT folder FROM ".TABLE_PREFIX."mail_content_imap_folders
							WHERE object_id=".$mail->getId()." AND folder = ".DB::escape($inbox_folder->getFolderName()));
						$current_folder_names = array_flat($current_folder_names);
							
						$sync_controller->remove_mail_from_imap_folders($account, $imap, $mail, $current_folder_names);
					}
					
					$unclassify = false;
					
					$imap->disconnect();
				}
			}
			
			$dim = Dimensions::findByCode("mail_imap_folders");
			if ($dim instanceof Dimension && $unclassify) {
				$members = $mail->getMembers();
				foreach ($members as $member) {
					if ($member->getDimensionId() == $dim->getId()) {
						$mem_ids[] = $member->getId();
					}
				}
				if (count($mem_ids) > 0) {
					$user = logged_user() instanceof Contact ? logged_user() : Contacts::findById($account->getContactId());
					ObjectMembers::removeObjectFromMembers($mail, $user, null, $mem_ids);
				}
				// classify in account member
				$acc_mem_obj = MailAccountMembers::findOne(array("conditions" => "account_id=".$account->getId()));
				if ($acc_mem_obj instanceof MailAccountMember) {
					$acc_mem = Members::findOneByObjectId($acc_mem_obj->getId(), $dim->getId());
					if ($acc_mem instanceof Member) {
						$mail->addToMembers(array($acc_mem));
					}
				}
			}
		}
	}
}



function mail_imap_sync_hooks_after_content_object_unarchive($parameters, &$ignored) {
	try {
		$mail = array_var($parameters, 'object');
		if ($mail instanceof MailContent) {
	
			$account = MailAccounts::instance()->findById($mail->getAccountId());
			if ($account instanceof MailAccount && $account->getIsImap()) {
				
				$archive_folder = MailAccountImapFolders::instance()->findOne(
					array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Archive"))
				);
				$inbox_folder = MailAccountImapFolders::instance()->findOne(
					array("conditions" => array("account_id=? AND folder_name LIKE 'INBOX'", $account->getId()))
				);
				if (!$inbox_folder instanceof MailAccountImapFolder) return;
				
				$sync_controller = new ImapSynchronizationController();
				
				$imap = $account->imapConnect();
				$ret = $imap->login($account->getEmail(), MailUtilities::ENCRYPT_DECRYPT($account->getPassword()),null,false);
				if (PEAR::isError($ret)) return;
				
				// put in inbox folder
				$sync_controller->copy_mail_to_imap_folders($account, $imap, $mail, array($inbox_folder->getFolderName()));
					
				if ($archive_folder instanceof MailAccountImapFolder) {
					// remove from archived
					$sync_controller->remove_mail_from_imap_folders($account, $imap, $mail, array($archive_folder->getFolderName()));	
				}
				
				$imap->disconnect();
				
			}
		}
	} catch (Exception $e) {
	
		Logger::log_r($e->getMessage());
		Logger::log_r($e->getTraceAsString());
	}
}




function mail_imap_sync_hooks_after_mail_content_imap_folder_saved($params, &$ignored) {
	
	$account = array_var($params, 'account');
	$mail_id = array_var($params, 'mail_id');
	$folder = array_var($params, 'folder');
	$uid = array_var($params, 'uid');

	$mail = MailContents::findById($mail_id);
	if ($mail instanceof MailContent) {
		$has_to_unarchive = false;
		$has_to_archive = false;
		
		$archive_folder = MailAccountImapFolders::instance()->findOne(
			array("conditions" => array("account_id=? AND special_use=?", $account->getId(), "\\Archive"))
		);
		if ($archive_folder instanceof MailAccountImapFolder) {
			if ($folder == $archive_folder->getFolderName()) {
				if (!$mail->isArchived()) $has_to_archive = true;
			} else {
				if ($mail->isArchived()) $has_to_unarchive = true;
			}
			
		} else {
			
			$inbox_folder = MailAccountImapFolders::instance()->findOne(
				array("conditions" => array("account_id=? AND folder_name LIKE 'INBOX'", $account->getId()))
			);
			if ($inbox_folder instanceof MailAccountImapFolder && $account->getColumnValue('sync_removed_from_inbox_archive_in_feng')) {
				
				$inbox_folder_rows = DB::executeAll("SELECT folder FROM ".TABLE_PREFIX."mail_content_imap_folders
					WHERE object_id=".$mail->getId()." AND folder = ".DB::escape($inbox_folder->getFolderName()));
				
				$inbox_folder_rows = array_filter(array_flat($inbox_folder_rows));
				if (count($inbox_folder_rows) == 0) {// is archived
					if (!$mail->isArchived()) $has_to_archive = true;
				} else {
					if ($mail->isArchived()) $has_to_unarchive = true;
				}
			}
		}
		
		if ($has_to_unarchive) { 
			$mail->unarchive(false);
			
		} else if ($has_to_archive) {
			$mail->archive(null, false);
			
		}
	}

}





function mail_imap_sync_hooks_delete_member($member, &$ignored){
	
	$dim = Dimensions::findByCode("mail_imap_folders");
	if (!$dim instanceof Dimension) return;
	$ot = ObjectTypes::findByName('mail_imap_folder_member');
	
	if ($member->getDimensionId() == $dim->getId() && $member->getObjectTypeId() == $ot->getId()) {
		if (strtoupper($member->getName()) == "INBOX") {
			return;
		}
		
		$mem_object = Objects::findObject($member->getObjectId());
		if ($mem_object instanceof MailImapFolderMember) {
		
			$account = MailAccounts::findById($mem_object->getAccountId());
			if ($account instanceof MailAccount && $account->getIsImap()) {
					
				$sync_controller = new ImapSynchronizationController();
				$deleted = $sync_controller->delete_mailbox($account, $mem_object->getFolderName());
				
			}
		}
	}
}


function mail_imap_sync_hooks_after_member_save($params, &$ignored) {
	$previous_data = array_var($params, 'previous_data');
	$member = array_var($params, 'member');
	$is_new = array_var($params, 'is_new');
	
	$dim = Dimensions::findByCode("mail_imap_folders");
	if (!$dim instanceof Dimension) return;
	$ot = ObjectTypes::findByName('mail_imap_folder_member');
	
	if ($member->getDimensionId() == $dim->getId() && $member->getObjectTypeId() == $ot->getId()) {
		
		if (strtoupper($member->getName()) == "INBOX") {
			throw new Exception("invalid name");
		}
		
		if (!$is_new && $member->getName() != $previous_data['original_member_data']['name']) {
			// rename
			$mem_object = Objects::findObject($member->getObjectId());
			if ($mem_object instanceof MailImapFolderMember) {
				
				$account = MailAccounts::findById($mem_object->getAccountId());
				if ($account instanceof MailAccount && $account->getIsImap()) {
					
					$prev_folder_name = $mem_object->getFolderName();
					
					$sync_controller = new ImapSynchronizationController();
					$renamed = $sync_controller->rename_mailbox($account, $prev_folder_name, $member->getName());
					if ($renamed) {
						$new_fname = str_replace($previous_data['original_member_data']['name'], $member->getName(), $mem_object->getFolderName());
						DB::execute("UPDATE ".TABLE_PREFIX."mail_imap_folder_members SET folder_name=".DB::escape($new_fname)." WHERE object_id=".$mem_object->getId());
					}
				}
			}
		} else if ($is_new) {
			// create
			$mem_object = Objects::findObject($member->getObjectId());
			if ($mem_object instanceof MailImapFolderMember) {
				if ($mem_object->getFolderName() != '') return; // mailbox already exists in server.
				
				$parent_member = Members::findById($member->getParentMemberId());
				if ($parent_member instanceof Member) {
					
					$parent_object = Objects::findObject($parent_member->getObjectId());
					$account = MailAccounts::findById($parent_object->getAccountId());
					
					if ($account instanceof MailAccount && $account->getIsImap()) {
							
						$sync_controller = new ImapSynchronizationController();
						$new_fname = $member->getName();
						
						$created = $sync_controller->create_mailbox($account, $member->getName(), $parent_object, $new_fname);
						if ($created) {
							
							$mem_object->setFolderName($new_fname);
							$mem_object->setAccountId($account->getId());
							$mem_object->save();
						}
					}
				}
			}
		}
	}
	
}


