<?php

/**
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */

class AdvancedTimeslotReportsController extends ApplicationController {

	/**
	 * Construct the ReportingController
	 *
	 * @access public
	 * @param void
	 * @return AdvancedTimeslotReportsController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('reporting');
		Env::useHelper('grouping');
		Env::useHelper('advanced_reports', 'advanced_reports');
	} // __construct

	
	
	
	function grouped_timeslots_p() {
		$dimensions = array();
		$enabled_dimensions = config_option('enabled_dimensions');
		if (count($enabled_dimensions) > 0) {
			$dimensions = Dimensions::findAll(array("conditions" => "id IN (".implode(',', $enabled_dimensions).")"));
		}
		
		$ts_ot = ObjectTypes::findByName('timeslot');
		$reporting_controller = new ReportingController();
		$column_options = $reporting_controller->get_allowed_columns($ts_ot->getId());
		
		
		tpl_assign('preset_parameters', array_var($_REQUEST, 'preset', ''));
		tpl_assign('dimensions', $dimensions);
		tpl_assign('column_options', $column_options);
	}
	
	
	function grouped_timeslots($function_params = null, $replace_session_params = true) {
		if (!$function_params) $function_params = $_POST;
		
		// get report params
		$report_params = array_var($function_params, 'report');
		
		$preset_parameters = array_var($report_params, 'preset_parameters');
		
		if ($replace_session_params && (!$preset_parameters || $preset_parameters == 'billing_and_cost_by_client')) {
			// save params in session for future requests
			$_SESSION['grouped_timeslots_report_data'] = $report_params;
		}
		
		$report_name = lang('grouped timeslots report');
		if ($preset_parameters) {
			$preset = get_grouped_timeslots_report_preset_config($preset_parameters);
			if ($preset && array_var($preset, 'report_name')) {
				$report_name = array_var($preset, 'report_name');
			}
		}
		
		// parse report group by 
		$gb_params = array_var($report_params, 'group_by_type');
		$gb_options = array_var($report_params, 'group_by');
		$groups_config = array();
		$dimension_ids = array();
		
		foreach ($gb_params as $k => $gb_param) {
			if ($gb_param == 'contact_id') {
				$groups_config[] = $gb_param;
				
			} else if ($gb_param == 'dim') {
				$groups_config[] = $gb_options[$k][0];
				
			} else if ($gb_param == 'member_ids') {
				$groups_config[] = $gb_param;
				foreach ($gb_options[$k] as $opt) {
					$did = str_replace("dim_", "", $opt);
					$dimension_ids[] = $did;
				}
			}
		}
		
		// dimension conditions
		$dimension_conditions = "";
		if (count($dimension_ids) > 0) {
			$dimension_conditions = " AND m.dimension_id IN (".implode(',', $dimension_ids).")";
		}
		
		// temp table columns and sql
		$temp_table_column_names = array("object_id", "member_ids", "contact_id");
		$temp_table_columns = array("object_id int", "member_ids varchar(255)", "contact_id int");
		$dim_group_sqls = array();
		foreach ($groups_config as $gconf) {
			if (str_starts_with($gconf, "dim_")) {
				$did = str_replace("dim_", "", $gconf);
				$temp_table_columns[] = $gconf." int";
				$temp_table_column_names[] = $gconf;
				$dim_group_sqls[] = "
					(SELECT oo$did.member_id FROM ".TABLE_PREFIX."object_members oo$did 
						INNER JOIN ".TABLE_PREFIX."members mm$did ON mm$did.id=oo$did.member_id 
						WHERE mm$did.dimension_id=$did AND oo$did.object_id=o.id ORDER BY is_optimization LIMIT 1) AS dim_$did";
			}
		}
		$temp_table_columns_sql = implode(', ', $temp_table_columns);
		$temp_table_column_names_sql = implode(', ', $temp_table_column_names);
		$dim_group_sql = "";
		if (count($dim_group_sqls) > 0) $dim_group_sql = ", " . implode(', ', $dim_group_sqls);
		
		
		// dates config
		$report_dates = parse_report_param_date_type($report_params);
		$start_date = array_var($report_dates, 'st');
		$end_date = array_var($report_dates, 'et');
		
		// date conditions
		$date_conditions = "";
		if ($start_date instanceof DateTimeValue) $date_conditions .= " AND t.start_time >= '".$start_date->toMySQL()."'";
		if ($end_date instanceof DateTimeValue) $date_conditions .= " AND t.start_time <= '".$end_date->toMySQL()."'";
		
		
		// create memory table
		$table_name = TABLE_PREFIX. "tmp_table_" . logged_user()->getId();
		DB::execute("DROP TABLE IF EXISTS $table_name;");
		DB::execute("CREATE TABLE IF NOT EXISTS $table_name ($temp_table_columns_sql) ENGINE=MEMORY;");
		DB::execute("TRUNCATE TABLE $table_name;");
		
		// filter by current context
		$context_conditions = "";
		$context_member_ids = active_context_members(false);
		if (count($context_member_ids) > 0) {
			$context_conditions = " AND (EXISTS (SELECT om2.object_id
					FROM ".TABLE_PREFIX."object_members om2
					WHERE om2.member_id IN (" . implode ( ',', $context_member_ids ) . ") AND o.id = om2.object_id
					GROUP BY om2.object_id
					HAVING count(om2.member_id) = ".count($context_member_ids)."
				))
			";
		}
		
		
		
		// main sql to insert int tmp table
		$insert_temp_sql = "
			INSERT INTO $table_name ($temp_table_column_names_sql) 
				SELECT om.object_id, group_concat(om.member_id), t.contact_id $dim_group_sql
				FROM ".TABLE_PREFIX."object_members om 
					INNER JOIN ".TABLE_PREFIX."timeslots t ON t.object_id=om.object_id 
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
					INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
					LEFT JOIN ".TABLE_PREFIX."project_tasks pt ON pt.object_id=t.rel_object_id
					LEFT JOIN ".TABLE_PREFIX."objects ot ON ot.id=pt.object_id
				WHERE 
					om.is_optimization=0 AND (ot.trashed_by_id IS NULL OR ot.trashed_by_id=0) AND o.trashed_by_id=0
					$dimension_conditions
					$context_conditions
					$date_conditions
				GROUP BY om.object_id 
				ORDER BY om.member_id;
		";
		// insert int tmp table
		DB::execute($insert_temp_sql);
		
		// fix gruop_id if needed
		$temp_rows = DB::executeAll("SELECT * FROM $table_name t");
		foreach ($temp_rows as $temp_row) {
			$group_id = $temp_row['member_ids'];
			$tmp_arr = explode(',', $group_id);
			sort($tmp_arr, SORT_NUMERIC);
			$group_id = implode(',', $tmp_arr);
			if ($temp_row['member_ids'] != $group_id) {
				DB::execute("UPDATE $table_name SET member_ids='$group_id' WHERE object_id=".$temp_row['object_id']);
			}
		}
		
		// select timeslots data
		$temp_rows = DB::executeAll("
			SELECT ts.object_id as id, t.*, o.*, ts.*, pt.* 
			FROM $table_name t 
			INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
			INNER JOIN ".TABLE_PREFIX."timeslots ts ON ts.object_id=t.object_id
			LEFT JOIN ".TABLE_PREFIX."project_tasks pt ON pt.object_id=ts.rel_object_id;
		");
		
		$user_ids = array();
		$related_task_ids = array();
		$grouped_timeslots = array();
		$timeslot_ids = array();
		
		foreach ($temp_rows as $temp_row) {
			$first_group_id = isset($groups_config[0]) ? $temp_row[$groups_config[0]] : "0";
			$second_group_id = isset($groups_config[1]) ? $temp_row[$groups_config[1]] : "0";
			$third_group_id = isset($groups_config[2]) ? $temp_row[$groups_config[2]] : "0";
			
			if (!isset($grouped_timeslots[$first_group_id])) $grouped_timeslots[$first_group_id] = array();
			if (!isset($grouped_timeslots[$first_group_id][$second_group_id])) $grouped_timeslots[$first_group_id][$second_group_id] = array();
			if (!isset($grouped_timeslots[$first_group_id][$second_group_id][$third_group_id])) $grouped_timeslots[$first_group_id][$second_group_id][$third_group_id] = array();
			
			$grouped_timeslots[$first_group_id][$second_group_id][$third_group_id][] = $temp_row;
			
			if ($temp_row['rel_object_id'] > 0) $related_task_ids[] = $temp_row['rel_object_id'];
			$user_ids[] = $temp_row['contact_id'];
			
			$timeslot_ids[] = $temp_row['id'];
		}
		$related_task_ids = array_unique($related_task_ids);
		$user_ids = array_unique($user_ids);
		
		
		// task names
		$related_task_name_rows = array();
		$related_task_names = array();
		if (count($related_task_ids) > 0) {
			$related_task_name_rows = DB::executeAll("SELECT id, name FROM ".TABLE_PREFIX."objects WHERE id IN (".implode(',', $related_task_ids).")");
			foreach ($related_task_name_rows as $r) {
				$related_task_names[$r['id']] = $r['name'];
			}
		}
		
		// user names
		$user_name_rows = array();
		$user_names = array();
		if (count($user_ids) > 0) {
			$user_name_rows = DB::executeAll("SELECT id, name FROM ".TABLE_PREFIX."objects WHERE id IN (".implode(',', $user_ids).")");
			foreach ($user_name_rows as $r) {
				$user_names[$r['id']] = $r['name'];
			}
		}
		
		// get timeslot member paths if there are any dimension columns to show
		$active_columns = array();
		$report_has_dim_cols = false;
		foreach ($report_params['columns'] as $col => $ord) {
			if ($ord > 0 && str_starts_with($col, 'dim_')) {
				$report_has_dim_cols = true;
				break;
			}
		}
		$timeslot_member_names = array();
		if ($report_has_dim_cols) {
			$timeslot_member_names = get_members_info_for_object_list($timeslot_ids);
		}
		
		
		
		// calculate totals
/*		$group_field = 'member_ids';
		if ($groups_config[0] == 'member_ids') {
			$group_field = 'member_ids';
		} else if ($groups_config[0] == 'contact_id') {
			$group_field = 'contact_id';
		}
		$totals_sql = "
			SELECT tmp.$group_field as group_field, SUM(worked_time) as group_worked_time FROM $table_name tmp
				INNER JOIN ".TABLE_PREFIX."timeslots t ON t.object_id=tmp.object_id
				GROUP BY group_field;
		";
		$total_rows_tmp = DB::executeAll($totals_sql);
		$total_rows = array();
		foreach ($total_rows_tmp as $r) {
			$total_rows[$r['group_field']] = $r;
		}*/
		
		// set variables in view
		tpl_assign('related_task_names', $related_task_names);
		tpl_assign('user_names', $user_names);
		tpl_assign('grouped_timeslots', $grouped_timeslots);
		tpl_assign('groups_config', $groups_config);
		tpl_assign('report_data', $report_params);
		tpl_assign('timeslot_member_names', $timeslot_member_names);
		
		tpl_assign('title', $report_name);
		tpl_assign('post', $report_params);
		tpl_assign('allow_export', true);
		
		tpl_assign('template_name', 'grouped_timeslots');
		$this->setTemplate('report_wrapper');
		
		// remove temp table
	//	DB::execute("DROP TABLE $table_name;");
		
		return array(
			'related_task_names' => $related_task_names,
			'user_names' => $user_names,
			'grouped_timeslots' => $grouped_timeslots,
			'groups_config' => $groups_config,
			'report_data' => $report_params,
			'timeslot_member_names' => $timeslot_member_names,
		);
	}
	
	
	
	
	function grouped_timeslots_to_pdf() {
		ajx_current("empty");
		set_time_limit(0);
		ini_set("memory_limit", "512M");
		
		$html_filename = gen_id().'pdf.html';
		$html_filepath = ROOT . "/tmp/$html_filename";
		
		// include all css
		$html = stylesheet_tag('website.css');
		$css_html = "";
		Hook::fire('additional_report_print_css', null, $css_html);
		if ($css_html) $html .= $css_html;
		
		
		$preset_parameters = array_var($_REQUEST, 'preset_parameters');
		
		$preset = null;
		$report_name = lang('grouped timeslots report');
		if ($preset_parameters) {
			$preset = get_grouped_timeslots_report_preset_config($preset_parameters);
			if ($preset && array_var($preset, 'report_name')) {
				$report_name = array_var($preset, 'report_name');
			}
		}
		if ($preset) {
			$report_params = $preset;
		} else {
			$report_params = $_SESSION['grouped_timeslots_report_data'];
		}
		
		
		// title html
		$html .= '<div class="report-print-header"><div class="title-container"><h1>'.$report_name.'</h1></div></div><div class="clear"></div>';
		
		// start output buffer
		ob_start();
		
		// execute the report
		$this->grouped_timeslots(array('report' => $report_params), false);
		// include the report html to print
		tpl_display(get_template_path('grouped_timeslots', 'advanced_timeslot_reports', 'advanced_reports'));
		
		// get the html from buffer
		$html .= ob_get_clean();
	
		file_put_contents($html_filepath, $html);
	
		// convert html to pdf
		$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
		$page_size = array_var($_REQUEST, 'pdfPageSize');
		
		$pdf_data = convert_to_pdf($html, $orientation, "grouped_timeslots", $page_size);
		$real_pdf_filename = $pdf_data['name'];
		
		ajx_extra_data(array("filename" => $real_pdf_filename, 'size' => $pdf_data['size']));
	
	}
	
	
	
	function grouped_timeslots_print(){
		ajx_current("empty");
		ini_set("memory_limit", "1G");
		
		if (array_var($_REQUEST, 'exportCSV') || array_var($_REQUEST, 'exportExcel')) {
			
			$preset_parameters = array_var($_REQUEST, 'preset_parameters');
			
			$preset = null;
			$report_name = lang('grouped timeslots report');
			if ($preset_parameters) {
				$preset = get_grouped_timeslots_report_preset_config($preset_parameters);
				if ($preset && array_var($preset, 'report_name')) {
					$report_name = array_var($preset, 'report_name');
				}
			}
			if ($preset && $preset_parameters != 'billing_and_cost_by_client') {
				$report_params = $preset;
			} else {
				$report_params = $_SESSION['grouped_timeslots_report_data'];
			}
			
			$result = $this->grouped_timeslots(array('report' => $report_params), false);
			$result['report_params'] = $report_params;
			
			$array_rows = grouped_timeslots_result_to_arrays($result);
			
			if (array_var($_REQUEST, 'exportExcel') && Plugins::instance()->isActivePlugin('advanced_core')) {
				
				$excel_export_controller = new ExcelExportController();
				
				$headers = array_shift($array_rows);
				$all_data_rows = $array_rows;
				
				// build a sheet for the excel file with the report data
				$sheets = array(
					array('headers' => $headers, 'data' => $all_data_rows, 'options' => array('title' => $report_name)),
				);
				
				// generate the excel file
				$filename = "grouped_timeslots.xlsx";
				$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
				$filesize = filesize($filepath);
				
			} else {
			
				$csv_text = "";
				foreach ($array_rows as $row) {
					$csv_text .= '"' . implode('","', $row) . '"' . "\n";
				}
				
				$filename = "grouped_timeslots.csv";
				$filepath = ROOT . "/tmp/grouped_timeslots.csv";
				file_put_contents($filepath, $csv_text);
				$filesize = filesize($filepath);
			}
			
			ajx_extra_data(array("filename" => $filename, 'size' => $filesize));
		}
	}
	
}






















