<?php

function parse_report_condition_date_range($value_obj, $apply_timezone_dates = true, $apply_timezone_hours = true) {
	
	$from = array_var($value_obj, 'from');
	$to = array_var($value_obj, 'to');
	$tz_offset = logged_user()->getUserTimezoneValue();
	
	if (preg_match('/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/', $from)) {
		$dt1 = DateTimeValueLib::makeFromString($from);
	} else {
		$dt1 = getDateValue($from);
	}
	if ($dt1 instanceof DateTimeValue && $apply_timezone_dates) {
		$dt1->add('s', -1 * $tz_offset);
	}
		
	if (preg_match('/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/', $to)) {
		$dt2 = DateTimeValueLib::makeFromString($to);
	} else {
		$dt2 = getDateValue($to);
	}
	if ($dt2 instanceof DateTimeValue && $apply_timezone_dates) {
		$dt2->add('s', -1 * $tz_offset);
	}
	
	$ft = getTimeValue(array_var($value_obj, 'from_time'));
	$tt = getTimeValue(array_var($value_obj, 'to_time'));
	
	if (is_array($ft) && $apply_timezone_hours) {
		$ft['hours'] += -1 * ($tz_offset/3600);
		if ($ft['hours'] < 0) $ft['hours'] += 24;
	}
	if (is_array($tt) && $apply_timezone_hours) {
		$tt['hours'] += -1 * ($tz_offset/3600);
		if ($tt['hours'] < 0) $tt['hours'] += 24;
	}
	
	return array('from' => $dt1, 'to' => $dt2, 'from_time' => $ft, 'to_time' => $tt);
}

function parse_report_param_date_type($report_data, $apply_timezone = true, $type_key = 'date_type', $start_key = 'start_value', $end_key = 'end_value', $report_type = '') {
	
	$now = DateTimeValueLib::now();
	$now->advance(logged_user()->getUserTimezoneValue(), true);	

	switch (array_var($report_data, $type_key)){
		case 1: //Today
			$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
			$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
		case 2: //This week
			$monday = $now->getMondayOfWeek();
			$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
			$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
			$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
		case 3: //Last week
			$monday = $now->getMondayOfWeek()->add('w',-1);
			$nextMonday = $now->getMondayOfWeek()->add('d',-1);
			$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
			$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
		case 4: //This month
			$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
			if($report_type == 'utilization_report'){
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			} else {
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			}
		case 5: //Last month
			$now->add('M',-1);
			$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
			$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
		case 6: //Date interval
			$st = getDateValue(array_var($report_data, $start_key));
			$st = $st->beginningOfDay();
	
			$et = getDateValue(array_var($report_data, $end_key));
			$et = $et->endOfDay();
			break;
	}

	if ($apply_timezone) {
		if ($st instanceof DateTimeValue) {
			$st->advance(logged_user()->getUserTimezoneValue() * -1, true);
		}
		if ($et instanceof DateTimeValue) {
			$et->advance(logged_user()->getUserTimezoneValue() * -1, true);
		}		
	}
	
	return array('st' => $st, 'et' => $et);
}


function grouped_timeslots_echo_totals_row($table_columns, $group_name, $group_totals, $total_columns, $level, $report_data, $type='html') {
	$show_details = array_var($report_data, 'show_details');
	$csv_row = array();
	
	if ($type == 'html') {
?><tr class="list-totals-row level-<?php echo ($level+1)?> indent-<?php echo $level?> <?php echo ($show_details ? '' : 'simple-row');?>"><?php
	}

	$all_to_print = "";
	$ts_ot = ObjectTypes::findByName('timeslot');
	
	foreach ($table_columns as $col_data) {
		$col_name = $col_data['id'];
		
		$to_print = "";
		if ($type == 'html') {
			$to_print = '<td'.($col_name == 'name' ? ' class="textrow"': '').'>';
		}
		
		if ($col_name == 'name') {
			
			if ($show_details) {
				$to_print .= lang('total in', $group_name);
			} else {
				$to_print .= $group_name;
			}
			
		} else if (isset($total_columns[$col_name])) {
			$total_info = $total_columns[$col_name];
			
			if (isset($group_totals[$col_name])) {
				$total = $group_totals[$col_name];
				
				if ($total_info['format'] == 'money') {
					$amounts = array();
					foreach ($group_totals[$col_name] as $curr_id => $amount) {
						$c = Currencies::getCurrency($curr_id);
						$symbol = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');
						$amounts[] = format_money_amount($amount, $symbol);
					}
					$sep = ($type == 'html' ? '<br />' : " | ");
					$to_print .= implode($sep, $amounts);
					
				} else {
					$to_print .= format_value_to_print($col_name, $group_totals[$col_name], Timeslots::instance()->getColumnType($col_name), $ts_ot->getId());
				}
			}
		}
		
		if ($type == 'html') {
			$to_print .= '</td>';
		}
		
		if ($type == 'html') {
			$all_to_print .= " | ".$to_print;
			echo $to_print;
		} else {
			$csv_row[] = $to_print;
		}
	}
	
	if ($type == 'html') {
?></tr><?php
	}

	return $csv_row;
}




function grouped_timeslots_result_to_arrays($result) {
	$array_rows = array();

	$related_task_names = array_var($result, 'related_task_names');
	$user_names = array_var($result, 'user_names');
	$grouped_timeslots = array_var($result, 'grouped_timeslots');
	$groups_config = array_var($result, 'groups_config');
	$report_data = array_var($result, 'report_params');
	$timeslot_member_names = array_var($result, 'timeslot_member_names');

	$ts_ot = ObjectTypes::findByName('timeslot');

	$external_columns = Timeslots::instance()->getExternalColumns();
	$total_columns = Timeslots::instance()->getColumnsToAggregateInTotals();
	$total_col_names = array_keys($total_columns);

	$col_align_right = array_keys($total_columns);
	$col_align_right[] = 'hourly_billing';
	$col_align_right[] = 'hourly_cost';

	$money_columns = array(
			'hourly_billing' => 'rate_currency_id', 'fixed_billing' => 'rate_currency_id',
			'hourly_cost' => 'cost_currency_id', 'fixed_cost' => 'cost_currency_id'
	);
	$money_column_names = array_keys($money_columns);

	$table_columns = array();

	foreach ($report_data['columns'] as $col => $order) {
		if ($order > 0) {
			if (str_starts_with($col, 'dim_')) {
				$col_lang = Dimensions::getDimensionById(str_replace('dim_', '', $col))->getName();
			} else {
				$col_lang = Localization::instance()->lang('field Timeslots '.$col);
				if (!$col_lang) $col_lang = lang('field Objects '.$col);
			}
			$col_data = array('id' => $col, 'name' => $col_lang);
			if (in_array($col, $col_align_right)) $col_data['align'] = 'right';

			$table_columns[$order] = $col_data;
		}
	}
	ksort($table_columns, SORT_NUMERIC);

	$header_row = array();
	foreach ($table_columns as $col) {
		$header_row[] = $col['name'];
	}
	$array_rows[] = $header_row;


	$all_totals = array(); // level => array(gid => totals)

	// first level groups
	foreach ($grouped_timeslots as $first_group_id => $second_level_groups) {

		$group_name = grouped_timeslots_get_group_name($first_group_id, $groups_config[0], $user_names);

		if (array_var($report_data, 'show_details')) {
			$array_rows[] = array($group_name);
		}

		$first_total = array();
		// second level groups
		foreach ($second_level_groups as $second_group_id => $third_level_groups) {
				
			if (isset($groups_config[1])) {
				$second_group_name = grouped_timeslots_get_group_name($second_group_id, $groups_config[1], $user_names);

				if (array_var($report_data, 'show_details')) {
					$array_rows[] = array($second_group_name);
				}
			}

			// third level groups
			$second_total = array();
			foreach ($third_level_groups as $third_group_id => $timeslots) {

				if (isset($groups_config[2])) {
					$third_group_name = grouped_timeslots_get_group_name($third_group_id, $groups_config[2], $user_names);
						
					if (array_var($report_data, 'show_details')) {
						$array_rows[] = array($third_group_name);
					}
				}

				$third_total = array();
				foreach ($timeslots as $ts) {
						
					$ts_row = array();
						
					foreach ($table_columns as $col) {
						$field = $col['id'];

						if (in_array($field, $total_col_names)) {
								
							if ($total_columns[$field]['format'] == 'money') {
								if (!isset($third_total[$field])) $third_total[$field] = array();
								$curr_id = $ts[$total_columns[$field]['currency_id_col']];
								if ($curr_id > 0) {
									if (!isset($third_total[$field][$curr_id])) $third_total[$field][$curr_id] = 0;
									$third_total[$field][$curr_id] += $ts[$field];
								}
							} else {
								if (!isset($third_total[$field])) $third_total[$field] = 0;
								$third_total[$field] += $ts[$field];
							}
						}

						if (array_var($report_data, 'show_details')) {
								
							$col_value = "";
								
							if ($field == 'name') {
								$col_value = array_var($related_task_names, $ts['rel_object_id'], array_var($ts, 'description', ''));

							} else if ($field == 'rel_object_id') {
								$col_value = array_var($related_task_names, $ts['rel_object_id'], '');

							} else if (in_array($field, $money_column_names)) {
								/*$currency_col_id = $money_columns[$field];
								$c = Currencies::getCurrency($ts[$currency_col_id]);
								$sym = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');

								$col_value = format_money_amount($ts[$field], $c instanceof Currency ? $c->getSymbol() : null);*/
								$col_value = format_amount($ts[$field]);

							} else if (in_array($field, $external_columns)) {
								$col_value = clean(Reports::instance()->getExternalColumnValue($field, $ts[$field], Timeslots::instance()));

							} else if (str_starts_with($field, "dim_")) {
								$dim_id = str_replace("dim_", "", $field);

								$member_names = array_var($timeslot_member_names, $ts['id']);
								if ($members_info = array_var($member_names, $dim_id)) {
									foreach ($members_info as $mid => $info) {
										$col_value = $info['name'];
									}
								}

							} else if ($field == 'start_time' || $field == 'end_time') {
								$dt = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $ts[$field]);
								$col_value = $dt instanceof DateTimeValue ? format_datetime($dt) : '';

							} else if ($field == 'worked_time' && !$report_data['show_descriptive_worked_time']) {
								$col_value = $ts[$field] / 60;
							} else {
								$col_value = format_value_to_print($field, $ts[$field], Timeslots::instance()->getColumnType($field), $ts_ot->getId());
							}
								
							$ts_row[] = $col_value;
								
						}
					}
					if (array_var($report_data, 'show_details')) {
						$array_rows[] = $ts_row;
					}
				}
					
				if (isset($groups_config[2]) && array_var($report_data, 'show_details')) {
					$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $third_group_name, $third_total, $total_columns, 2, $report_data, 'csv');
				}

				$all_totals[2][$first_group_id ."-". $second_group_id ."-". $third_group_id] = $third_total;

				// accumulate for prev level group
				foreach ($third_total as $k => $v) {
					if (is_array($v)) {
						if (!isset($second_total[$k])) $second_total[$k] = array();
						foreach ($v as $cur => $am) {
							if (!isset($second_total[$k][$cur])) $second_total[$k][$cur] = 0;
							$second_total[$k][$cur] += $am;
						}
					} else {
						if (!isset($second_total[$k])) $second_total[$k] = 0;
						$second_total[$k] += $v;
					}
				}
					
			}

			if (isset($groups_config[1]) && array_var($report_data, 'show_details')) {
				$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $second_group_name, $second_total, $total_columns, 1, $report_data, 'csv');
			}

			$all_totals[1][$first_group_id ."-". $second_group_id] = $second_total;

			// accumulate for prev level group
			foreach ($second_total as $k => $v) {
				if (is_array($v)) {
					if (!isset($first_total[$k])) $first_total[$k] = array();
					foreach ($v as $cur => $am) {
						if (!isset($first_total[$k][$cur])) $first_total[$k][$cur] = 0;
						$first_total[$k][$cur] += $am;
					}
				} else {
					if (!isset($first_total[$k])) $first_total[$k] = 0;
					$first_total[$k] += $v;
				}
			}

		}

		if (isset($groups_config[0]) && array_var($report_data, 'show_details')) {
			$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $group_name, $first_total, $total_columns, 0, $report_data, 'csv');
		}

		$all_totals[0][$first_group_id] = $first_total;

		// not showing details => draw only totals
		if (!array_var($report_data, 'show_details')) {
				
			$group_name = grouped_timeslots_get_group_name($first_group_id, $groups_config[0], $user_names);
			$first_total = array_var($all_totals[0], $first_group_id);
			$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $group_name, $first_total, $total_columns, 0, $report_data, 'csv');
				
			if (isset($groups_config[1])) {
				foreach ($second_level_groups as $second_group_id => $third_level_groups) {
						
					$second_group_name = grouped_timeslots_get_group_name($second_group_id, $groups_config[1], $user_names);
					$second_total = array_var($all_totals[1], $first_group_id ."-". $second_group_id);
					$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $second_group_name, $second_total, $total_columns, 1, $report_data, 'csv');
						
					if (isset($groups_config[2])) {
						foreach ($third_level_groups as $third_group_id => $timeslots) {
							$third_group_name = grouped_timeslots_get_group_name($third_group_id, $groups_config[2], $user_names);
							$third_total = array_var($all_totals[2], $first_group_id ."-". $second_group_id ."-". $third_group_id);
							$array_rows[] = grouped_timeslots_echo_totals_row($table_columns, $third_group_name, $third_total, $total_columns, 2, $report_data, 'csv');
						}
					}
				}
			}
			
			$array_rows[] = array(""); // separator
		}

	}


	return $array_rows;
}





function grouped_timeslots_get_group_name($group_id, $group_type, $user_names) {
	if ($group_type == 'member_ids') {
		$group_name_arr = array();
		$mem_ids = explode(',', $group_id);
		foreach ($mem_ids as $mid) {
			$m = Members::getMemberById($mid); // uses cache
			$group_name_arr[] = $m->getName();
		}
		$group_name = implode(', ', $group_name_arr);
		
	} else if ($group_type == 'contact_id') {
		$group_name = array_var($user_names, $group_id, '');
		
	} else if (str_starts_with($group_type, "dim_")) {
		$mem = Members::getMemberById($group_id);
		$group_name = $mem instanceof Member ? $mem->getName() : '';
	}
	if (!$group_name) {
		$group_name = lang("unclassified");
	}
	return $group_name;
}


function get_grouped_timeslots_report_preset_config($preset_parameters) {
	
	$config = null;
	Hook::fire('grouped_timeslots_report_preset_config', $preset_parameters, $config);
	
	return $config;
}




/**
 * Creates and fills a temporary table with the dimension intersection for each object of the report
 * 
 * @param ObjectType $object_type: report object type
 * @param array $dimension_filters: dimension ids to intersect, can also include specific member ids
 * @param string $all_conditions: report conditions
 */
function fill_custom_report_intersection_table(ObjectType $object_type, $dimension_filters, $all_conditions) {
	
	$temp_table_columns = array("object_id int", "member_ids varchar(255)", "member_names varchar(255)");
	$temp_table_column_names = array("object_id", "member_ids", "member_names");
	
	$temp_table_columns_sql = implode(', ', $temp_table_columns);
	$temp_table_column_names_sql = implode(', ', $temp_table_column_names);
	
	// create memory table
	$table_name = TABLE_PREFIX. "tmp_table_" . logged_user()->getId();
	DB::execute("DROP TABLE IF EXISTS $table_name;");
	DB::execute("CREATE TABLE IF NOT EXISTS $table_name ($temp_table_columns_sql) ENGINE=InnoDB;");
	DB::execute("TRUNCATE TABLE $table_name;");
	
	// filter by current context
	$context_conditions = "";
	$context_member_ids = active_context_members(false);
	if (count($context_member_ids) > 0) {
		$context_conditions = " AND (EXISTS (SELECT om2.object_id
				FROM ".TABLE_PREFIX."object_members om2
				WHERE om2.member_id IN (" . implode ( ',', $context_member_ids ) . ") AND o.id = om2.object_id
				GROUP BY om2.object_id
				HAVING count(om2.member_id) = ".count($context_member_ids)."
			))
		";
	}
	
	$specific_member_ids = array();
	$dimension_ids = array();
	
	foreach ($dimension_filters as $dimension_filter) {
		if (is_numeric($dimension_filter)) {
			$dimension_ids[] = $dimension_filter;
			
		} else if (is_array($dimension_filter) && isset($dimension_filter['dim_id']) && $dimension_filter['dim_id'] > 0) {
			$dimension_ids[] = $dimension_filter['dim_id'];
				
			if (isset($dimension_filter['member_ids'])) {
				$mem_ids_array = is_string($dimension_filter['member_ids']) ? json_decode($dimension_filter['member_ids'], true) : $dimension_filter['member_ids'];
				$mem_ids = array_filter($mem_ids_array);
				if (is_array($mem_ids) && count($mem_ids) > 0) {
					$specific_member_ids[$dimension_filter['dim_id']] = $mem_ids;
				}
			}
		}
	}
	
	if (count($specific_member_ids) > 0) {
		$dimension_conditions = " AND (
					";
		$dim_conds_array = array();
		foreach ($dimension_ids as $did) {
			$tmp_cond = "m.dimension_id = $did";
			if (isset($specific_member_ids[$did])) {
				$tmp_cond .= " AND m.id in (".implode(',', array_unique(array_filter($specific_member_ids[$did]))).")";
			}
			$dim_conds_array[] = $tmp_cond;
		}
		$dim_conds_sql = implode("
				OR
		", $dim_conds_array);
		
		$dimension_conditions .= "$dim_conds_sql
				)";
		
	} else {
		$dimension_ids = array_unique(array_filter($dimension_ids));
		$dimension_conditions = " AND m.dimension_id IN (".implode(',', $dimension_ids).")";
	}
	
	$object_table = TABLE_PREFIX . $object_type->getTableName();
	
	// main sql to insert int tmp table
	$insert_temp_sql = "
		INSERT INTO $table_name ($temp_table_column_names_sql) 
			SELECT om.object_id, group_concat(om.member_id), group_concat(m.name)
			FROM ".TABLE_PREFIX."object_members om 
				INNER JOIN $object_table e ON e.object_id=om.object_id 
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=e.object_id
				INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
			WHERE 
				om.is_optimization < 1
				$dimension_conditions
				$context_conditions
				$all_conditions
			GROUP BY om.object_id 
			ORDER BY m.dimension_id, m.name;
	";
	// insert int tmp table
	DB::execute($insert_temp_sql);
	
	// fix gruop_id if needed
	$temp_rows = DB::executeAll("SELECT * FROM $table_name t");
	foreach ($temp_rows as $temp_row) {
		$group_id = $temp_row['member_ids'];
		$tmp_arr = explode(',', $group_id);
		sort($tmp_arr, SORT_NUMERIC);
		$group_id = implode(',', $tmp_arr);
		if ($temp_row['member_ids'] != $group_id) {
			if ($group_id != '') {
				$mem_name_rows = DB::executeAll("SELECT name FROM ".TABLE_PREFIX."members WHERE id IN ($group_id) ORDER BY id");
				$mem_names = implode(',', array_flat($mem_name_rows));
			} else {
				$mem_names = $temp_row['member_names'];
			}
			DB::execute("UPDATE $table_name SET member_ids='$group_id', member_names='$mem_names' WHERE object_id=".$temp_row['object_id']);
		}
	}
	
	return array('table_name' => $table_name);
}




/**
 * This function applies changes defined in the group by options over the report result.
 * E.g.: adding the empty groups to the result.
 * 
 * @param array $plain_groups: the plain list of groups to process
 * @param Report $report: the report content data object 
 * @param array $group_by_criterias array with the selected group by criteria and options
 */
function apply_gb_options_before_grouping($plain_groups, $report, $group_by_criterias, $main_query_sql=null, $parameters=array()) {
	$to_add = array();
	
	$gb_keys = array();
	foreach ($group_by_criterias as $gb) {
		switch ($gb['type']) {
			case 'association': $gkey = '_group_id_a_'.$gb['id']; break;
			case 'custom_property': $gkey = '_group_id_cp_'.$gb['id']; break;
			case 'parent_member': $gkey = '_group_id_pm_'.$gb['id']; break;
			case 'classification': $gkey = '_group_id_dim_'.$gb['id']; break;
			case 'fixed_property': $gkey = '_group_id_fp_'.$gb['id']; break;
			case 'intersection': $gkey = '_group_id_inter_'.$gb['id']; break;
		}
		
		if (!$gkey) $gkey = "";
		$gname = str_replace("_id_", "_name_", $gkey);
		$gb_keys[] = array('k' => $gkey, 'n' => $gname, 'is_date' => false);
	}
	
	$group_names = array();
	if (is_array($plain_groups)) {
	  foreach ($plain_groups as $row) {
		foreach ($row as $col => $val) {
			if (str_starts_with($col, "_group_name_")) {
				if (!isset($group_names[$col])) $group_names[$col] = array();
				$col_id = str_replace("_name_", "_id_", $col);
				$group_names[$col][$row[$col_id]] = $val;
			}
		}
	  }
	}
	
	foreach ($group_by_criterias as $index => $gb_data) {
		
		if ($gb_data['type'] == 'fixed_property' && isset($gb_data['options']) && array_var($gb_data['options'], 'show_empty_groups') 
				&& in_array($gb_data['id'], array("contact_id", "assigned_to_contact_id"))) {
			
			$put_empty_groups_first = array_var($gb_data['options'], 'put_empty_groups_first') > 0;
			
			$query_limit = array_var($parameters, 'limit');
			$query_offset = array_var($parameters, 'offset', 0);
			
			/* Only add the empty groups 
			 *  - if it is the first page and they go before, 
			 *  - or if it is the last page and they go after
			 *  - or if limit = 0 (print or export)
			 */ 
			if ($put_empty_groups_first && $query_offset == 0 || !$put_empty_groups_first && count($plain_groups) < 50 || $query_limit == 0) {
				
				
				$order_prefix = $put_empty_groups_first ? "0000000000_" : "zzzzzzzzzz_";
				
				$group_conditions_sql = "";
				$group_conditions = array_var($gb_data['options'], 'conditions');
				if (is_array($group_conditions)) {
					$conds_fields = array();
					$conds_cp = array();
					$dummy_report = new Report();
					$dummy_report->setReportObjectTypeId(Contacts::instance()->getObjectTypeId());
					
					foreach ($group_conditions as $group_cond) {
						$cond = new ReportCondition();
						$cond->setCondition($group_cond['condition']);
						$cond->setValue($group_cond['value']);
						if (is_numeric($group_cond['field_name'])) {
							$cond->setCustomPropertyId($group_cond['field_name']);
							$conds_cp[] = $cond;
						} else {
							$cond->setFieldName($group_cond['field_name']);
							$conds_fields[] = $cond;
						}
					}
					
					$group_conditions_data = build_report_conditions_sql(array('report' => $dummy_report, 'conditionsFields' => $conds_fields, 'conditionsCp' => $conds_cp));
					$group_conditions_sql = array_var($group_conditions_data, 'all_conditions');
				}
				
				// get the user ids already added 
				$user_ids = array();
				if ($main_query_sql) {
					$gb_keys_str = "";
					foreach ($gb_keys as $gb_key) $gb_keys_str .= ($gb_keys_str == "" ? "" : ",") . $gb_key['k'];
					$temp_group_rows = DB::executeAll($main_query_sql . " GROUP BY ".$gb_keys_str);
					
					foreach ($temp_group_rows as $data) {
						$user_ids[] = $data["_group_id_fp_" . $gb_data['id']];
					}
				} else {
					foreach ($plain_groups as $k => $data) {
						$user_ids[] = $data["_group_id_fp_" . $gb_data['id']];
					}
				}
				// sql to exclude the already present user ids
				$user_ids_sql = "";
				$user_ids = array_filter($user_ids);
				if (count($user_ids) > 0) $user_ids_sql = " AND object_id NOT IN (".implode(',', $user_ids).")";
				
				// get the other contacts data
				$all_contacts = Contacts::instance()->listing(array(
					"count_results" => false,
					"raw_data" => true,
					"ignore_context" => true,
					"extra_conditions" => " AND user_type > 0 $user_ids_sql $group_conditions_sql",
				))->objects;
				
				if (is_array($all_contacts) && count($all_contacts) > 0) {
					
					$key_len = $index;
					
					foreach ($plain_groups as $k => $data) {
						$exp_k = explode("_", $k);
						
						// use parent keys as base
						if (count($exp_k) == $key_len || $index == 0) {
							
							// add a row for each contact that doesn't have one
							foreach ($all_contacts as $c) {
								$cid = $c['id'];
								$cname = $c['name'];
								
								if ($index == 0) {
									$new_key = $cid;
								} else {
									$new_key = $k."_".$cid;
								}
								
								$to_add[$new_key] = array('total_rows' => 0);
								
								foreach ($gb_keys as $j => $gb_key) {
									if (isset($exp_k[$j]) && $index > 0) {
										
										$to_add[$new_key][$gb_key['k']] = $exp_k[$j];
										$to_add[$new_key][$gb_key['n']] = $group_names[$gb_key['n']][$exp_k[$j]];
										
									} else {
										
										$to_add[$new_key][$gb_key['k']] = $cid;
										$to_add[$new_key][$gb_key['n']] = $cname;
										$to_add[$new_key][$gb_key['k']."_toorder"] = $order_prefix . $cname;
									}
								}
							}
						}
					}
					
				}
				
			}
		}
	}
	
	return array_merge($plain_groups, $to_add);
	
}







function calculate_report_groups_as_columns($group_data, &$results, $report, $level=0, &$group_columns) {

	if (!$report->getColumnValue('show_last_group_as_column')) return;

	$report_columns = array_var($results, 'columns');
	$i = 0;
	foreach ($group_data as $gd) {

		if (isset($gd['groups'])) {
				
			calculate_report_groups_as_columns($gd['groups'], $results, $report, $level+1, $group_columns);
				
		} else {
				
			if ($level > 0) {
				foreach ($results['group_totals'][$level] as $k => &$t) {
						
					// don't process again the already processed total rows
					if (array_var($t, '__TOTALROW_IS_PROCESSED__')) continue;
						
					$gr_id = "";
					$gr_name = "";
					$parent_group_ids = array();
					$parent_group_keys = array();
						
					foreach ($t as $t_name => $t_val) {
						if (str_Starts_with($t_name, "_group_id")) {
							//	$gr_id .= ($gr_id == "" ? "" : "_") . $t_val;
							$gr_id = $t_val;
							$parent_group_ids[] = $t_val;
							$parent_group_keys[] = $t_name;
						}

						if (str_Starts_with($t_name, "_group_name")) {
							$gr_name = $t_val;
						}
					}

					array_pop($parent_group_keys);
					array_pop($parent_group_ids);
						
					$parent_totals_key = null;
					foreach ($results['group_totals'][$level - 1] as $j => $parent_t) {
						$found = false;
						foreach ($parent_group_keys as $idx => $k) {
							$found = $parent_t[$k] == $parent_group_ids[$idx];
							if (!$found) break;
						}
						if ($found) {
							$parent_totals_key = $j;
							break;
						}
					}
						
					// column definition and total values
					foreach ($t as $t_name => $t_val) {
						if ($t_name == '__TOTALROW_IS_PROCESSED__') continue;
						// col definition
						$orig_col_key = str_starts_with($t_name, "cp_") ? str_replace("cp_", "", $t_name) : $t_name;
						$col_key = $gr_id . "|" . $t_name;
						if (!isset($group_columns['names'][$col_key]) && in_array($orig_col_key, $report_columns['order'])) {
							if (is_numeric($gr_id) && $gr_id == 0 && trim($gr_name)=="") {
								$gr_name = lang('unclassified'); // apply only the unclassified word when group is from a dimension
							} else if (trim($gr_name)=="" && $gr_id != '') {
								$last_group = end($results['group_by_criterias']);
								if ($last_group['id'] == 'invoicing_status') {
									// group name is the lang of the invoicing status key
									$gr_name = lang("invoicing_status $gr_id");
								} else {
									// group name is the lang of the key
									$gr_name = lang("$gr_id");
								}
							}
								
							$group_columns['order'][] = $col_key;
							$group_columns['names'][$col_key] = $gr_name /*. " " . $report_columns['names'][$orig_col_key]*/;
							$group_columns['types'][$col_key] = $report_columns['types'][$orig_col_key];
								
						}

						// set total values in parent group totals
						if (!is_null($parent_totals_key)) {
							if (!isset($results['group_totals'][$level - 1][$parent_totals_key][$col_key])) {
								$results['group_totals'][$level - 1][$parent_totals_key][$col_key] = 0;
							}
							if (is_numeric($t_val)) {
								$results['group_totals'][$level - 1][$parent_totals_key][$col_key] += $t_val;
							}
							$t['__TOTALROW_IS_PROCESSED__'] = 1;
						}
					}
				}
			}
		}
	}
}

function report_append_group_columns($report, &$results) {
	$groups = $results['grouped_rows'];

	$group_columns = array('order' => array(), 'names' => array(), 'types' => array());
	foreach($groups as $g) {
		calculate_report_groups_as_columns($g, $results, $report, 0, $group_columns);
	}
	
	// accumulate the second level totals
	if (count($results['group_totals']) > 2) {
		foreach ($results['group_totals'][1] as $second_total_row) {
			
			foreach ($second_total_row as $t_name => $t_val) {
				if (str_Starts_with($t_name, "_group_id")) {
					$gr_id = $t_val;
					$parent_group_id = $t_val;
					$parent_group_key = $t_name;
					break;
				}	
			}
			
			$parent_totals_key = null;
			foreach ($results['group_totals'][0] as $j => $parent_t) {
				if ($parent_t[$parent_group_key] == $parent_group_id) {
					$parent_totals_key = $j;
					break;
				}
			}
			
			if (!is_null($parent_totals_key)) {
				foreach ($second_total_row as $t_name => &$t_val) {
					if (strpos($t_name, "|") !== false) {
						
						if (!isset($results['group_totals'][0][$parent_totals_key][$t_name])) {
							$results['group_totals'][0][$parent_totals_key][$t_name] = 0;
						}
						$results['group_totals'][0][$parent_totals_key][$t_name] += $t_val;
					}
				}
			}
		}
	}

	// report columns
	$columns = array_var($results, 'columns');
	
	if (count($group_columns['order']) > 0) {
		$temp_group_columns_order = array();
		foreach ($group_columns['names'] as $k => $v) {
			$temp_group_columns_order[$v . "-" . $k] = $k;
		}
		ksort($temp_group_columns_order, SORT_STRING);
		
		foreach ($temp_group_columns_order as $k => $v) {
			$columns['order'][] = $v;
		}
		foreach ($group_columns['names'] as $k => $v) {
			$columns['names'][$k] = $v;
		}
		foreach ($group_columns['types'] as $k => $v) {
			$columns['types'][$k] = $v;
		}
		// add the group columns to the report results
		$results['group_columns'] = $group_columns;

		// update report columns
		$results['columns'] = $columns;
	}
}

