<?php
/**
 * @abstract Creates a default report for an object type
 * @param mixed $object_type: Can be the object type's id or name of the object type
 */
function create_default_member_report($object_type) {
	
	if (is_numeric($object_type)) {
		$ot = ObjectTypes::findById($object_type);
	} else {
		$ot = ObjectTypes::findByName($object_type);
	}
	if ($ot instanceof ObjectType) {
		
		$report_exists = Reports::findOne(array("conditions" => "is_default=1 AND report_object_type_id=".$ot->getId()." AND code='".$ot->getName()."_report'"));
		if ($report_exists instanceof Report) return;
		
		$report_ot = ObjectTypes::findByName('report');
		$report_ot_id = $report_ot->getId();
		
		$ot_name = Localization::instance()->lang($ot->getName()." report");
		if (is_null($label_value)) {
			$ot_name = ucfirst($ot->getName()." report");
		}
		$ot_id = $ot->getId();
		
		// fill objects table
		$sql = "
			INSERT INTO ".TABLE_PREFIX."objects (object_type_id, name, created_on, created_by_id, updated_on, updated_by_id, trashed_by_id, archived_by_id) VALUES
			  ($report_ot_id, '$ot_name', NOW(), 0, NOW(), 0, 0, 0);
		";
		DB::execute($sql);
		
		// fill reports table
		$sql = "
			INSERT INTO ".TABLE_PREFIX."reports (object_id, description, report_object_type_id, is_order_by_asc, ignore_context, is_default, code) VALUES
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), '', $ot_id, 1, 1, 1, '".$ot->getName()."_report');
		";
		DB::execute($sql);
		
		// add column: name
		$sql = "
			INSERT INTO ".TABLE_PREFIX."report_columns (report_id, custom_property_id, field_name) VALUES
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'name');
		";
		DB::execute($sql);
		
		// add custom property columns
		if (in_array($ot->getType(), array('dimension_group'))) {
			if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
				$sql = "
					INSERT INTO ".TABLE_PREFIX."report_columns (report_id, custom_property_id, field_name)
					  SELECT (SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), cp.id, ''
					  FROM ".TABLE_PREFIX."member_custom_properties cp WHERE cp.object_type_id=$ot_id and cp.code!='color_special';
				";
				DB::execute($sql);
			}
		} else {
			$sql = "
				INSERT INTO ".TABLE_PREFIX."report_columns (report_id, custom_property_id, field_name)
				  SELECT (SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), cp.id, ''
				  FROM ".TABLE_PREFIX."custom_properties cp WHERE cp.object_type_id=$ot_id and cp.code!='color_special';
			";
			DB::execute($sql);
		}
	
	}
}


function get_default_member_report_queries($db_connection, $table_prefix, $ot_name) {
	
	$report_exists = false;
	$ot_subquery = "SELECT id FROM ".$table_prefix."object_types WHERE name='$ot_name'";
	$sql = "SELECT count(id) as total FROM ".$table_prefix."reports WHERE code='".$ot_name."_report' AND object_type_id = ($ot_subquery)";
	$mysql_res = mysqli_query($db_connection, $sql);
	if ($mysql_res) {
		$rows = mysqli_fetch_assoc($mysql_res);
		if (is_array($rows) && count($rows) > 0) {
			$report_exists = $rows['total'] > 0;
		}
	}
	if ($report_exists) return array();
	
	$queries = array();
	$report_ot_id="(SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='report')";
	
	$queries[] = "
		INSERT INTO ".$table_prefix."objects (object_type_id, name, created_on, created_by_id, updated_on, updated_by_id, trashed_by_id, archived_by_id) VALUES
			($report_ot_id, '".ucfirst($ot_name."s report")."', NOW(), 0, NOW(), 0, 0, 0);
	";
	$queries[] = "
		INSERT INTO ".$table_prefix."reports (object_id, description, report_object_type_id, is_order_by_asc, ignore_context, is_default, code, group_by_options) VALUES
		  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), '', (SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='$ot_name'), 1, 1, 1, '".$ot_name."_report', '');
	";
	
	$queries[] = "
		INSERT INTO ".$table_prefix."report_columns (report_id, custom_property_id, field_name) VALUES
		  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'name');
	  ";
	
	$queries[] = "
		INSERT INTO ".$table_prefix."report_columns (report_id, custom_property_id, field_name)
		  SELECT (SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), cp.id, ''
		  FROM ".$table_prefix."custom_properties cp WHERE cp.object_type_id=(SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='$ot_name');
	  ";
	
	include_once dirname(__FILE__)."/../../../../application/helpers/plugin_installer_functions.php";
	
	if (check_is_installed_plugin($db_connection, $table_prefix, 'member_custom_properties')) {
		$queries[] = "
			INSERT INTO ".$table_prefix."report_columns (report_id, custom_property_id, field_name)
			  SELECT (SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), cp.id, ''
			  FROM ".$table_prefix."member_custom_properties cp
			  WHERE cp.object_type_id=(SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='$ot_name') and cp.code!='color_special';
		";
	}
	
	return $queries;
}


function create_fixed_report($object_type,$ot_name) {

    if (is_numeric($object_type)) {
        $ot = ObjectTypes::findById($object_type);
    } else {
        $ot = ObjectTypes::findByName($object_type);
    }
    if ($ot instanceof ObjectType) {
        $report_ot = ObjectTypes::findByName('report');
        $report_ot_id = $report_ot->getId();

        $ot_id = $ot->getId();

        // fill objects table
        $sql = "
			INSERT INTO ".TABLE_PREFIX."objects (object_type_id, name, created_on, created_by_id, updated_on, updated_by_id, trashed_by_id, archived_by_id) VALUES
			  ($report_ot_id, '$ot_name', NOW(), 0, NOW(), 0, 0, 0);
		";
        DB::execute($sql);

        // fill reports table
        $sql = "
			INSERT INTO ".TABLE_PREFIX."reports (object_id, description, report_object_type_id, is_order_by_asc, ignore_context, is_default,is_system, code) VALUES
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), '', $ot_id, 1, 1, 1,1, '".$ot->getName()."_report');
		";
        DB::execute($sql);

        // add columns
        $sql = "
			INSERT INTO ".TABLE_PREFIX."report_columns (report_id, custom_property_id, field_name) VALUES
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'description'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'worked_time'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'fixed_billing'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'fixed_cost'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'hourly_billing'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'hourly_cost'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'adv_billing_id'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'time_estimate'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'adv_cost_id'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'bill_total_price'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'bill_total_cost'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'bill_adv_billing_id'),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id), 0, 'bill_adv_cost_id');
		";

        DB::execute($sql);

        // add conditions
        $sql = "
			INSERT INTO ".TABLE_PREFIX."report_conditions (`report_id`,`custom_property_id`,`field_name`, `condition`, `value` , `is_parametrizable`) VALUES
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id),0, 'start_time', '>=', '', 1),
			  ((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=$report_ot_id),0, 'end_time', '<=', '', 1);
	    ";

        DB::execute($sql);

    }
}

function get_create_fixed_report_queries($table_prefix, $ot_name,$name) {

    $queries = array();
    $report_ot_id="(SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='report')";

    $queries[] = "
		INSERT INTO ".$table_prefix."objects (object_type_id, name, created_on, created_by_id, updated_on, updated_by_id, trashed_by_id, archived_by_id) VALUES
			($report_ot_id, '".$name."', NOW(), 0, NOW(), 0, 0, 0);
	";
    $queries[] = "
		INSERT INTO ".$table_prefix."reports (object_id, description, report_object_type_id, is_order_by_asc, ignore_context, is_default,is_system, code) VALUES
		  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), '', (SELECT id FROM ".$table_prefix."object_types ot WHERE ot.name='$ot_name'), 1, 1, 1, 1, '".$ot_name."_report');
	";

    $queries[] = "
		INSERT INTO ".$table_prefix."report_columns (report_id, custom_property_id, field_name) VALUES
		 ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'description'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'worked_time'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'fixed_billing'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'fixed_cost'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'hourly_billing'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'hourly_cost'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'adv_billing_id'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'time_estimate'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'adv_cost_id'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'bill_total_price'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'bill_total_cost'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'bill_adv_billing_id'),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id), 0, 'bill_adv_cost_id');
	  ";

    $queries[] = "
		INSERT INTO ".$table_prefix."report_conditions (`report_id`,`custom_property_id`,`field_name`, `condition`, `value` , `is_parametrizable`) VALUES
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id),0, 'start_time', '>=', '', 1),
			  ((SELECT max(id) FROM ".$table_prefix."objects WHERE object_type_id=$report_ot_id),0, 'end_time', '<=', '', 1);
	  ";

    return $queries;
}