<?php

function get_main_permissions_report_data($report_data){
    // Get max depth from members 
    $depth_sql = DB::executeOne("SELECT max(depth) as 'max_depth' FROM ".TABLE_PREFIX."members;");
    $max_depth = $depth_sql['max_depth'];

    $user_id = array_var($report_data, 'user');
    if($user_id){
        $data = array('users' => array());
        $data['users'][] = get_user_permission_data($report_data, $user_id, $max_depth);
    } else {
        $data = get_all_users_permission_data($report_data, $max_depth);
    }
    $data['max_depth'] = $max_depth;

    return $data;
}

function get_user_permission_data($report_data, $user_id, $max_depth){
    $contact = Contacts::findById($user_id);
    $user_data = array();
    if($contact instanceof Contact){
        $user_data['id'] = $user_id;
        $user_data['name'] = $contact->getName();
        $user_data['permissions'] = collect_and_structure_user_permissions($user_id, $report_data, $max_depth);
    }
    return $user_data;
}

function get_all_users_permission_data($report_data, $max_depth){
    $result = array('users' => array());
    $all_users = Contacts::getAllUsers();
    foreach($all_users as $user){
        $user_id = $user->getObjectId();
        $result['users'][] = get_user_permission_data($report_data, $user_id, $max_depth);
    }
    return $result;
}

function collect_and_structure_user_permissions($user_id, $report_data, $max_depth){
    $depth = 1;
    $result = array();
    while($depth <= $max_depth){
        if($depth == 1){
            $result[$depth] = get_permissions_by_depth_dimension_and_object_type($user_id, $depth, $report_data);
        } else {
            $user_permissions = get_permissions_by_depth_dimension_and_object_type($user_id, $depth, $report_data);
            $new_depth_result = array();
            foreach($user_permissions as $permission){
                $parent_id = $permission['parent_id'];
                if(!isset($new_depth_result[$parent_id])) $new_depth_result[$parent_id] = array();
                $new_depth_result[$parent_id][] = $permission;
            }
            $result[$depth] = $new_depth_result;
        }
        $depth++;
    }
    return $result;
}

function get_permissions_by_depth_dimension_and_object_type($user_id, $depth, $report_data){

    $object_type_id = array_var($report_data, 'object_type');
    $dimension_id = array_var($report_data, 'dimension');

    $sql = "
    SELECT cmp.member_id as 'member_id',
        mem.name as 'member_name',
        mem.parent_member_id as 'parent_id',
        cmp.can_write as 'can_write',
        cmp.can_delete as 'can_delete'
    FROM ".TABLE_PREFIX."contact_member_permissions cmp 
        JOIN ".TABLE_PREFIX."members mem ON cmp.member_id=mem.id
        JOIN ".TABLE_PREFIX."contact_permission_groups cpg ON cpg.permission_group_id=cmp.permission_group_id
        JOIN ".TABLE_PREFIX."contacts c ON cpg.contact_id=c.object_id
    WHERE c.object_id=".$user_id." 
        AND cmp.object_type_id=".$object_type_id." 
        AND mem.dimension_id=".$dimension_id." 
        AND mem.depth=".$depth." 
        AND mem.archived_by_id=0
    ORDER BY cmp.can_write DESC, cmp.can_delete DESC, mem.name ASC;";

    $rows = DB::executeAll($sql);
    $result = array();
    $existing_mem_ids = array();
    foreach($rows as $row){
        if(!in_array($row['member_id'], $existing_mem_ids)){
            $existing_mem_ids[] = $row['member_id'];
            $result[] = $row;
        }
    }

    return $result;
}
