<?php

class ReportCategories extends BaseReportCategories {

    public function __construct() {
        parent::__construct();
        $this->object_type_name = 'report_category';
    }

    /**
     * Return a category
     * @param int $id
     * @return ReportCategory
     */
    static function getReportCategory($id) {
        return self::findById($id);
    }

    /**
     * Return all categories
     *
     * @return array
     */
    static function getAllCategories() {
        return self::findAll();
    }

    /**
     * Return all categories
     *
     * @return array
     */
    static function getAllCategoriesAndOrderByName() {
        return self::instance()->findAll(
                array(
                    'order' => 'o.name',
                    'order_dir' => 'asc'
                ));
    }

    /**
     * Return all reports for an object type
     *
     * @param $object_type
     * @return array
     */
    static function getAllReportsForCategory($report_category) {
        $reports_result = Reports::instance()->listing(
                array(
                    'order' => 'o.name',
                    'order_dir' => 'asc',
                    'extra_conditions' => 'AND report_category_id = ' . $report_category ,//array("`report_category_id` = ?", $report_category)
                    'ignore_context' => 1
                )
        );
        return $reports_result->objects;
    }

//  getAllReportsForObjectType

    /**
     * Return all reports for an object type
     *
     * @param $object_type
     * @return array
     */
    static function getAllReportsForCategoryWithoutListing($report_category) {
        return Reports::instance()->findAll(array(
                    'conditions' => array("`report_category_id` = ?", $report_category),
                    'order' => 'name',
                    'ignore_context' => 1
        ));
    }

//  getAllReportsForObjectType

    /**
     * Return all reports for an object type
     *
     * @param $object_type
     * @return array
     */
    static function getAllReportsWithoutCategory() {
        $reports_result = Reports::instance()->listing(
                array(
                    'order' => 'o.name',
                    'order_dir' => 'asc',
                    'extra_conditions' => 'AND report_category_id = 0',
                    'ignore_context' => 1
                )
        );
        return $reports_result->objects;
    }

//  getAllReportsForObjectType
}
