<?php
Hook::register('advanced_reports');


function advanced_reports_additional_report_print_css($ignored, &$css_html) {
	
	$css_html .= stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css");
	
}

function advanced_reports_report_can_delete($report, &$can_delete) {
	// default reports cannot be deleted
	if ($report->getColumnValue('is_default')) {
		$can_delete = false;
	}
    if ($report->getColumnValue('is_system')) {
        $can_delete = false;
    }
}

function advanced_reports_report_can_edit($report, &$can_edit) {
    // default reports cannot be edited
    if ($report->getColumnValue('is_system')) {
            $can_edit = false;
        }
    }


function advanced_reports_override_custom_report_order($ignored, &$report) {
	$order_col = array_var($_REQUEST, 'order');
	if ($order_col) {
		if (str_starts_with($order_col, "cp_")) {
			$order_col = str_replace("cp_", "", $order_col);
		}
		$report->setOrderBy($order_col);
		$report->setIsOrderByAsc(array_var($_REQUEST, 'order_dir')=='ASC');
	}
}


function advanced_reports_custom_reports_fixed_additional_columns($args, &$ret) {

	$ot = array_var($args, 'object_type');
	if ($ot instanceof ObjectType && $ot->getName() == 'contact') {
		$ret[] = array(
			'id' => 'availability',
			'name' => lang('availability'),
			'type' => 'date_range_time_range',	
		);
	}
	
	if ($ot instanceof ObjectType && !in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
		$enabled_dimensions = config_option("enabled_dimensions");
		$dimensions = Dimensions::findAll(array("conditions" => "id IN (".implode(',', $enabled_dimensions).")"));
		foreach ($dimensions as $dim) {
			$ret[] = array(
				'id' => 'dim_' . $dim->getId(),
				'name' => $dim->getName(),
				'type' => 'classification',	
			);
		}
	}
	
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
		$ot_dim_ids = DimensionObjectTypes::instance()->getDimensionIdsByObjectTypeId($ot->getId());
		$ot_dim_id = array_var($ot_dim_ids, 0);
		if (is_numeric($ot_dim_id) && $ot_dim_id > 0) {
			// located under
			$ret[] = array(
					'id' => 'located_under_' . $ot_dim_id,
					'name' => lang('located_under'),
					'type' => 'located_under',
			);
			// associations
			$d = Dimensions::getDimensionById($ot_dim_id);
			$associations = DimensionMemberAssociations::getAssociatations($ot_dim_id, $ot->getId());
			foreach ($associations as $assoc) {/* @var $assoc DimensionMemberAssociation */
				$assoc_info = $assoc->getArrayInfo($d);
				if ($assoc_info) {
					$ret[] = array(
						'id' => 'dimassoc_' . $assoc->getId(),
						'name' => $assoc_info['name'],
						'type' => 'dim_association',
					);
				}
			}
			
		}
	}
}

function advanced_reports_report_conditions_extra_fields($args, &$allConditions) {
	$field = array_var($args, 'field');
	$report = array_var($args, 'report');
	$report_ot = array_var($args, 'report_ot');
	$value = array_var($args, 'value');
	$disabled_params = array_var($args, 'disabled_params', array());
	
	if ($report_ot instanceof ObjectType && $report_ot->getName() == 'contact' && $field instanceof ReportCondition) {
		
		if ($field->getFieldName() == 'availability' && $value != '') {
			Env::useHelper('advanced_reports', 'advanced_reports');
			
			$value_obj = is_array($value) ? $value : json_decode($value, true);
			
			$dates = parse_report_condition_date_range($value_obj);
			
			$from_str = "";
			$fromdate = array_var($dates, 'from');
			if ($fromdate instanceof DateTimeValue) $from_str = $fromdate->toMySQL();
			
			$to_str = "";
			$todate = array_var($dates, 'to');
			if ($todate instanceof DateTimeValue) {
				$todate->add('d', 1); // include last day
				$to_str = $todate->toMySQL();
			}
			
			$from_time_str = "";
			$from_time = array_var($dates, 'from_time');
			if (is_array($from_time)) {
				$tmp = DateTimeValueLib::make($from_time['hours'], $from_time['mins'], 0, 0, 0, 0);
				$from_time_str = $tmp->format('H:i:s');
			}
			$to_time_str = "";
			$to_time = array_var($dates, 'to_time');
			if (is_array($to_time)) {
				$tmp = DateTimeValueLib::make($to_time['hours'], $to_time['mins'], 0, 0, 0, 0);
				$to_time_str = $tmp->format('H:i:s');
			}
			
			
			if ($from_str != "" || $to_str != "") {
				
				// days range
				$from_sql = ($from_str ? "t.start_date >= '$from_str'" : "");
				if ($to_str) $from_sql .= ($from_sql == "" ? "" : " AND ") . "t.start_date <= '$to_str'";
				
				$to_sql = ($from_str ? "t.due_date >= '$from_str'" : "");
				if ($to_str) $to_sql .= ($to_sql == "" ? "" : " AND ") . "t.due_date <= '$to_str'";
				
				// time range
				$time_range_cond = "";
				if ($from_time_str != "" || $to_time_str != "") {
					
					if ($from_time_str != "" && $to_time_str != "") {
						$time_range_cond = "AND (TIME(due_date) >= '$from_time_str' AND TIME(due_date) <= '$to_time_str' 
							OR TIME(start_date) >= '$from_time_str' AND TIME(start_date) <= '$to_time_str'
							OR TIME(start_date) <= '$from_time_str' AND TIME(due_date) >= '$to_time_str')";
						
					} else if ($from_time_str != "") {
						$time_range_cond = "AND (TIME(due_date) >= '$from_time_str' OR TIME(start_date) >= '$from_time_str')";
						
					} else if ($to_time_str != "") {
						$time_range_cond = "AND (TIME(due_date) <= '$to_time_str' OR TIME(start_date) <= '$to_time_str')";
					}
				}
				
				// sql conditions to add to report to exclude the assigned users in the defined days range within the hours range
				$availability_sql = " o.id NOT IN (
					SELECT DISTINCT(assigned_to_contact_id) as user_id
						FROM ".TABLE_PREFIX."project_tasks t
						INNER JOIN ".TABLE_PREFIX."objects oo ON t.object_id=oo.id
					WHERE oo.trashed_by_id=0 AND t.assigned_to_contact_id > 0 
					AND ($from_sql OR $to_sql)
					$time_range_cond
				)";
				
				$allConditions .= $availability_sql;
			}
		}
	}
	
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dim_")) {
		
		$skip_condition = false;
		if ($field->getIsParametrizable() && in_array($field->getId(), $disabled_params)) $skip_condition = true;
		
		if (!$skip_condition && $value != '') {
			$dim_id = str_replace("dim_", "", $field->getFieldName());
			$tmp_mem_ids = json_decode($value, true);
			if (is_array($tmp_mem_ids)) {
				$member_ids = array();
				foreach ($tmp_mem_ids as $mid) {
					if (is_numeric($mid) && $mid > 0) $member_ids[] = $mid;
				}
				if (count($member_ids) > 0) {
					$classification_sql = "";
					$table_alias = "om_" . $field->getId();
					$count = count($member_ids);
					
					if ($field->getCondition() == 'in') {
						// the object must be classified in all the members selected in the condition

						$condition = '';
						foreach($member_ids as $cl => $member_id){
							$condition .= ($cl > 0) ? ' OR ': '';
							if($member_id){
								$condition .= $table_alias.'.member_id = '.$member_id;
							}
						}

						$condition = ($condition) ? "(". $condition .") AND " : '';

						$classification_sql = " 0 < (
							SELECT COUNT($table_alias.member_id) 
							FROM ".TABLE_PREFIX."object_members $table_alias 
							WHERE 
							". $condition ."
							o.id = $table_alias.object_id
						)";
						
					}else if ($field->getCondition() == 'single_in') {
						$member_id = reset($member_ids);
						$classification_sql = " 
							1 = (
								SELECT COUNT($table_alias.member_id) 
								FROM ".TABLE_PREFIX."object_members $table_alias 
								WHERE 
								".$table_alias.".member_id = '$member_id'
								AND o.id = $table_alias.object_id
							)
						";
					}else if ($field->getCondition() == 'not_in') {

						$condition = '';
						foreach($member_ids as $cl => $member_id){
							$condition .= ($cl > 0) ? ' OR ': '';
							if($member_id){
								$condition .= $table_alias.'.member_id = '.$member_id;
							}
						}

						$condition = ($condition) ? "(". $condition .") AND " : '';

						$classification_sql = " NOT EXISTS (
							SELECT $table_alias.member_id 
							FROM ".TABLE_PREFIX."object_members $table_alias
							WHERE 
							". $condition ."
							o.id = $table_alias.object_id
						)";

					}else if ($field->getCondition() == 'not_in_single') {
						$member_id = reset($member_ids);
						$classification_sql = " 0 = (
								SELECT COUNT($table_alias.member_id) 
								FROM ".TABLE_PREFIX."object_members $table_alias 
								WHERE 
								".$table_alias.".member_id = '$member_id'
								AND o.id = $table_alias.object_id
							)
						";

					} else if ($field->getCondition() == 'unclassified') {
						// the object must not be classified in any of the members selected in the condition
						$classification_sql = " o.id not in (
							SELECT $table_alias.object_id 
							FROM ".TABLE_PREFIX."object_members $table_alias
                            INNER JOIN ".TABLE_PREFIX."members AS m1 on $table_alias.member_id = m1.id
							WHERE m1.dimension_id = $dim_id
						)";
					}
					$allConditions .= $classification_sql;
				}
				
			}
		}
	}
	
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "located_under_")) {
		
		$skip_condition = false;
		if ($field->getIsParametrizable() && in_array($field->getId(), $disabled_params)) $skip_condition = true;
		
		if (!$skip_condition && $value != '') {
			$dim_id = str_replace("located_under_", "", $field->getFieldName());
			$tmp_mem_ids = json_decode($value, true);
			if (is_array($tmp_mem_ids)) {
				$member_ids = array();
				foreach ($tmp_mem_ids as $mid) {
					if (is_numeric($mid) && $mid > 0) $member_ids[] = $mid;
				}
				if (count($member_ids) > 0) {
					$classification_sql = "";
					$table_alias = "om_" . $field->getId();
					$count = count($member_ids);
					
					
					$parent_mem_id = $member_ids[0];
					$parent_member = Members::getMemberById($parent_mem_id);
					$all_children_ids = $parent_member->getAllChildrenIds(true);
					$all_children_ids_sql = count($all_children_ids) > 0 ? implode(',',$all_children_ids) : '0';
					
					if ($field->getCondition() == 'in') {
						// the object must be classified in all the members selected in the condition
						$classification_sql = " AND m.id IN ($all_children_ids_sql)";
						
					} else if ($field->getCondition() == 'not_in') {
						// the object must not be classified in any of the members selected in the condition
						$classification_sql = " AND m.id NOT IN ($all_children_ids_sql)";
						
					}
					$allConditions .= $classification_sql;
				}
				
			}
		}
	}
	
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dimassoc_")) {
	
		$skip_condition = false;
		if ($field->getIsParametrizable() && in_array($field->getId(), $disabled_params)) $skip_condition = true;
	
		if (!$skip_condition && $value != '') {
			$assoc_id = str_replace("dimassoc_", "", $field->getFieldName());
			
			$tmp_mem_ids = json_decode($value, true);
			if (is_array($tmp_mem_ids)) {
				$member_ids = array();
				foreach ($tmp_mem_ids as $mid) {
					if (is_numeric($mid) && $mid > 0) $member_ids[] = $mid;
				}
				if (count($member_ids) > 0 || $field->getCondition() == 'unclassified') {
					$classification_sql = "";
					$table_alias = "mpm_" . $field->getId();
					$count = count($member_ids);
						
					if ($field->getCondition() == 'in' && count($member_ids) > 0) {
						// the object must be associated in all members selected in the condition
						$classification_sql = " $count = (
							SELECT COUNT($table_alias.property_member_id)
							FROM ".TABLE_PREFIX."member_property_members $table_alias
							WHERE $table_alias.property_member_id IN (".implode(',', $member_ids).")
							AND $table_alias.association_id='$assoc_id'
							AND m.id = $table_alias.member_id
						)";
	
					} else if ($field->getCondition() == 'not_in' && count($member_ids) > 0) {
						// the object must not be associated to any member selected in the condition
						$classification_sql = " NOT EXISTS (
							SELECT $table_alias.property_member_id
							FROM ".TABLE_PREFIX."member_property_members $table_alias
							WHERE $table_alias.property_member_id IN (".implode(',', $member_ids).")
							AND $table_alias.association_id='$assoc_id'
							AND m.id = $table_alias.member_id
						)";
					} else if ($field->getCondition() == 'unclassified') {
						// the object must not be associated using this association
						$classification_sql = " NOT EXISTS (
							SELECT $table_alias.property_member_id
							FROM ".TABLE_PREFIX."member_property_members $table_alias
							WHERE $table_alias.association_id='$assoc_id'
							AND m.id = $table_alias.member_id
						)";
					}
					$allConditions .= $classification_sql;
				}
	
			}
		}
	}
	
}

function advanced_reports_custom_report_param_name_render($args, &$name) {
	$field = array_var($args, 'field');
	$report = array_var($args, 'report');
	$report_ot = array_var($args, 'report_ot');
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dim_")) {
		$dim_id = str_replace("dim_", "", $field->getFieldName());
		$dim = Dimensions::getDimensionById($dim_id);
		
		if ($dim instanceof Dimension) {
			$name = $dim->getName();
		}
	}
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "located_under_")) {
		$dim_id = str_replace("located_under_", "", $field->getFieldName());
		$dim = Dimensions::getDimensionById($dim_id);
		
		if ($dim instanceof Dimension) {
			$name = $dim->getName();
		}
	}
	
	if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dimassoc_")) {
		
		$assoc_id = str_replace("dimassoc_", "", $field->getFieldName());
		$a = DimensionMemberAssociations::findById($assoc_id);
		
		$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($a->getId(), 'custom_association_name');
		if ($custom_assoc_name) {
			$name = $custom_assoc_name;
			
		} else {
		
			$dim_id = $a->getAssociatedDimensionMemberAssociationId();
			$dim = Dimensions::getDimensionById($dim_id);
		
			if ($dim instanceof Dimension) {
				$name = $dim->getName();
			}
		}
	}
}

function advanced_reports_custom_report_param_condition_render($args, &$name) {
	$field = array_var($args, 'field');
	$report = array_var($args, 'report');
	$report_ot = array_var($args, 'report_ot');

	if ($field instanceof ReportCondition 
			&& (str_starts_with($field->getFieldName(), "dim_") || str_starts_with($field->getFieldName(), "dimassoc_") || str_starts_with($field->getFieldName(), "located_under_"))) {
		if ($field->getCondition() == 'in') {
			$name = lang('classified in');
		} else if ($field->getCondition() == 'not_in') {
			$name = lang('not classified in');
		} else if ($field->getCondition() == 'single_in') {
			$name = lang('classified in this single');
		}else if ($field->getCondition() == 'not_in_single') {
			$name = lang('not classified in single');
		} 
	}
}

function advanced_reports_custom_report_param_render($args, &$already_rendered) {
	$field = array_var($args, 'field');
	$report = array_var($args, 'report');
	$report_ot = array_var($args, 'report_ot');
	
	if ($report_ot instanceof ObjectType && $report_ot->getName() == 'contact' && $field instanceof ReportCondition && $field->getFieldName() == 'availability') {
		
			$already_rendered = true;
			
			echo '<div style="display:inline-block">'. pick_date_widget2("params[".$field->getId()."][from]") .'</div>';
			echo '<div style="display:inline-block; vertical-align:super;">&nbsp;'. lang('and') .'&nbsp;</div>';
			echo '<div style="display:inline-block">'. pick_date_widget2("params[".$field->getId()."][to]") .'</div>';
			
			echo '<div style="display:inline-block; vertical-align:super;">&nbsp;'. lang('between') .'&nbsp;</div>';
			echo '<div style="display:inline-block">'. pick_time_widget2("params[".$field->getId()."][from_time]") .'</div>';
			echo '<div style="display:inline-block; vertical-align:super;">&nbsp;'. lang('and') .'&nbsp;</div>';
			echo '<div style="display:inline-block">'. pick_time_widget2("params[".$field->getId()."][to_time]") .'</div>';
			
			echo '<div class="clear"></div>';
		
		
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dim_")) {
		$already_rendered = true;
		$is_multiple = ($field->getCondition()=="single_in" || $field->getCondition()=="not_in_single") ? false : true;
		
		$dim_id = str_replace("dim_", "", $field->getFieldName());
		$dim = Dimensions::getDimensionById($dim_id);
		
		if ($dim instanceof Dimension) {
			$selector_genid = gen_id();
			$hf_name = "params[".$field->getId()."]";
			
			$selector_params = array('is_multiple' => $is_multiple, 'hide_label' => true, 'root_lang' => lang('none'),
					'hidden_field_name' => $hf_name, 'dont_filter_this_selector' => true);
			
			render_single_member_selector($dim, $selector_genid, null, $selector_params, false);
			
			echo '<div class="clear"></div>';
		}
		
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dimassoc_")) {
		$already_rendered = true;
		
		$assoc_id = str_replace("dimassoc_", "", $field->getFieldName());
		$a = DimensionMemberAssociations::findById($assoc_id);
		
		$dim_id = $a->getAssociatedDimensionMemberAssociationId();
		$dim = Dimensions::getDimensionById($dim_id);
		
		if ($dim instanceof Dimension) {
			$selector_genid = gen_id();
			$hf_name = "params[".$field->getId()."]";
			
			$selector_params = array('is_multiple' => true, 'hide_label' => true, 'root_lang' => lang('none'),
					'hidden_field_name' => $hf_name, 'dont_filter_this_selector' => true);
			
			render_single_member_selector($dim, $selector_genid, null, $selector_params, false);
			
			echo '<div class="clear"></div>';
		}
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "located_under_")) {
		$already_rendered = true;
		
		$dim_id = str_replace("located_under_", "", $field->getFieldName());
		$dim = Dimensions::getDimensionById($dim_id);
		
		if ($dim instanceof Dimension) {
			$selector_genid = gen_id();
			$hf_name = "params[".$field->getId()."]";
			
			$selector_params = array('is_multiple' => false, 'hide_label' => true, 'root_lang' => lang('none'),
					'hidden_field_name' => $hf_name, 'dont_filter_this_selector' => true);
			
			render_single_member_selector($dim, $selector_genid, null, $selector_params, false);
			
			echo '<div class="clear"></div>';
		}
	}
}


function advanced_reports_custom_report_cond_html($args, &$html) {
	$field = array_var($args, 'field');
	$report = array_var($args, 'report');
	$report_ot = array_var($args, 'report_ot');
	$param_value = array_var($args, 'value');
	
	if ($report_ot instanceof ObjectType && $report_ot->getName() == 'contact' && $field instanceof ReportCondition && $field->getFieldName() == 'availability') {
		
			$value = $field->getIsParametrizable() ? $param_value : $field->getValue();
			$value_obj = is_array($value) ? $value : json_decode($value, true);
			
			$dates = parse_report_condition_date_range($value_obj, true, false);
			$from = array_var($dates, 'from');
			$to = array_var($dates, 'to');
			
			if ($from instanceof DateTimeValue || $to instanceof DateTimeValue) {
				$formatted = strtolower(lang('from date')) . " ";
				$formatted .= $from instanceof DateTimeValue ? format_date($from) : '--';
				$formatted .= " " . strtolower(lang('to date')) . " ";
				$formatted .= $to instanceof DateTimeValue ? format_date($to) : '--';
				
				if ($dates['from_time'] || $dates['to_time']) {
					$formatted .= " (";
					if ($dates['from_time']) {
						$formatted .= strtolower(lang('from date')).": ";
						$d = DateTimeValueLib::make($dates['from_time']['hours'], $dates['from_time']['mins'], 0, 0, 0, 0);
						$formatted .= format_time($d, null, 0);
					}
					if ($dates['to_time']) {
						$formatted .= " " . strtolower(lang('to date')).": ";
						$d = DateTimeValueLib::make($dates['to_time']['hours'], $dates['to_time']['mins'], 0, 0, 0, 0);
						$formatted .= format_time($d, null, 0);
					}
					$formatted .= ")";
				}
				
				$html = '<li>'. lang($field->getFieldName()) .' '. $formatted .'</li>';
			} else {
				$html = ' ';
			}
		
		
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dim_")) {
		if (!$field->getIsParametrizable()) {
			$param_value = $field->getValue();
		}
		if (trim($param_value) != '') {
			$dim_id = str_replace("dim_", "", $field->getFieldName());
			$dim = Dimensions::getDimensionById($dim_id);
			
			if ($dim instanceof Dimension) {
				$member_ids = array_filter(json_decode($param_value, true));
				$dim_name = $dim->getName();
				
				if (is_array($member_ids) && count($member_ids) > 0) {
					$member_name_rows = DB::executeAll("SELECT name FROM ".TABLE_PREFIX."members WHERE id IN (".implode(',', $member_ids).")");
					$member_names = implode(', ', array_flat($member_name_rows));
				} else {
					$member_names = "";
				}
				
				$condition = $field->getCondition() == 'not_in' ? lang('not classified in') : lang('classified in');
				$condition = strtolower($condition);
				
				$html = "<li>$dim_name: $condition $member_names</li>";
			}
		}
		
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "dimassoc_")) {
		if (!$field->getIsParametrizable()) {
			$param_value = $field->getValue();
		}
		if (trim($param_value) != '') {
			$assoc_id = str_replace("dimassoc_", "", $field->getFieldName());
			$a = DimensionMemberAssociations::findById($assoc_id);
			
			$dim_id = $a->getAssociatedDimensionMemberAssociationId();
			$dim = Dimensions::getDimensionById($dim_id);
			
			if ($dim instanceof Dimension) {
				$member_ids = array_filter(json_decode($param_value, true));
				$dim_name = $dim->getName();
				
				$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($a->getId(), 'custom_association_name');
				if ($custom_assoc_name) {
					$dim_name = $custom_assoc_name;
				}
				
				if (is_array($member_ids) && count($member_ids) > 0) {
					$member_name_rows = DB::executeAll("SELECT name FROM ".TABLE_PREFIX."members WHERE id IN (".implode(',', $member_ids).")");
					$member_names = implode(', ', array_flat($member_name_rows));
				} else {
					$member_names = "";
				}
				
				$condition = $field->getCondition() == 'not_in' ? lang('not classified in') : lang('classified in');
				$condition = strtolower($condition);
				
				$html = "<li>$dim_name: $condition $member_names</li>";
			}
		}
		
	} else if ($field instanceof ReportCondition && str_starts_with($field->getFieldName(), "located_under_")) {
		if (!$field->getIsParametrizable()) {
			$param_value = $field->getValue();
		}
		if (trim($param_value) != '') {
			$dim_id = str_replace("located_under_", "", $field->getFieldName());
			$dim = Dimensions::getDimensionById($dim_id);
			
			if ($dim instanceof Dimension) {
				$member_ids = array_filter(json_decode($param_value, true));
				$dim_name = $dim->getName();
				
				if (is_array($member_ids) && count($member_ids) > 0) {
					$member_name_rows = DB::executeAll("SELECT name FROM ".TABLE_PREFIX."members WHERE id IN (".implode(',', $member_ids).")");
					$member_names = implode(', ', array_flat($member_name_rows));
				} else {
					$member_names = "";
				}
				
				$condition = $field->getCondition() == 'not_in' ? lang('not in') : lang('in');
				$condition = strtolower($condition);
				
				$html = "<li>$dim_name: $condition $member_names</li>";
			}
		}
		
	} else if($field->getIsParametrizable() && $report_ot->getHandlerClass() == 'ProjectTasks' && $field->getFieldName() == "status") {
	    $html = "<li>".lang($field->getFieldName()) ." = ".$param_value ."</li>";
	}
}


function advanced_reports_override_custom_report_columns($report_id, &$columns) {
	if (array_var($_REQUEST, 'columns')) {
		$rcols = explode(',', array_var($_REQUEST, 'columns'));
		$columns = array();
			
		foreach ($rcols as $colname) {
			if ($colname == 'mem_path') continue;
			
			$rcol = new ReportColumn();
			$rcol->setReportId($report_id);
			if (str_starts_with($colname, 'cp_')) {
				$rcol->setCustomPropertyId(str_replace("cp_", "", $colname));
			} else {
				$rcol->setFieldName($colname);
			}
			$columns[] = $rcol;
		}
	}
	
	// ensure that name column is shown if the report does not show details, so that the report group names are listed.
	$report = Reports::findById($report_id);
	if ($report instanceof Report && $report->getColumnValue("hide_group_details")) {
		$has_name_col = false;
		foreach ($columns as $col) {
			if ($col->getFieldName() == 'name') {
				$has_name_col = true;
				break;
			}
		}
		if (!$has_name_col) {
			$new_col = new ReportColumn();
			$new_col->setFieldName('name');
			$new_col->setReportId($report_id);
			array_unshift($columns, $new_col);
		}
	}
}


function advanced_reports_override_custom_report_group_by($report, &$group_by) {
	if (array_var($_REQUEST, 'group_by')) {
		$group_by = explode(',', array_var($_REQUEST, 'group_by'));
	} else if ($report->getColumnValue('group_by')) {
		$group_by = explode(',', $report->getColumnValue('group_by'));
	}
}

function advanced_reports_object_definition($type, &$ret){
	if ($type == 'Report' && check_column_exists(TABLE_PREFIX.'reports', 'group_by')) {
		$ret['group_by'] = DATA_TYPE_STRING;
		$ret['group_by_options'] = DATA_TYPE_STRING;
		$ret['hide_group_details'] = DATA_TYPE_BOOLEAN;
		$ret['hide_group_total_counts'] = DATA_TYPE_BOOLEAN;
		$ret['show_last_group_as_column'] = DATA_TYPE_BOOLEAN;
		$ret['show_comments'] = DATA_TYPE_BOOLEAN;
		$ret['report_category_id'] = DATA_TYPE_INTEGER;
		$ret['date_format'] = DATA_TYPE_STRING;
        $ret['show_currency'] = DATA_TYPE_BOOLEAN;
        $ret['show_negatives'] = DATA_TYPE_BOOLEAN;
        $ret['is_system'] = DATA_TYPE_BOOLEAN;
        //`is_system` tinyint(1) unsigned NOT NULL default '0',
	}
	if ($type == 'Report' && !isset($ret['code'])) {
		$ret['code'] = DATA_TYPE_STRING;
	}
	if ($type == 'Report' && !isset($ret['dont_show_time'])) {
		$ret['dont_show_time'] = DATA_TYPE_BOOLEAN;
	}
	if ($type == 'Report' && !isset($ret['add_signature_and_date_field'])) {
		$ret['add_signature_and_date_field'] = DATA_TYPE_BOOLEAN;
	}
	if ($type == 'Report' && !isset($ret['calculate_quantity_for_actual_expenses'])) {
		$ret['calculate_quantity_for_actual_expenses'] = DATA_TYPE_BOOLEAN;
	}
}

function advanced_reports_add_more_custom_report_parameters($params, &$ignored) {
	$report = array_var($params, 'report');
	$genid = array_var($params, 'genid');
	
	$ot = ObjectTypes::findById($report->getReportObjectTypeId());
	
	if ($ot /*&& in_array($ot->getType(), array('dimension_object', 'dimension_group'))*/) {
		tpl_assign('report', $report);
		tpl_assign('genid', $genid);
		tpl_display(get_template_path('group_by_parameters', 'advanced_reports', 'advanced_reports'));
		tpl_assign('handlerClass',$ot->getTableName());
		if ($ot->getType() == 'content_object' || $ot->getType() == 'located') {
			tpl_display(get_template_path('show_comments', 'advanced_reports', 'advanced_reports'));
		}
	}
}


function advanced_reports_before_object_save($object, &$ret) {
	if ($object instanceof Report) {
		
		$data = array_var($_REQUEST, 'report');
		$group_by = array_var($data, 'group_by');
		$group_by_options = array_var($data, 'gb_options');
		$hide_group_details = array_var($data, 'hide_group_details');
		$show_last_group_as_column = array_var($data, 'show_last_group_as_column');
		$hide_group_total_counts = array_var($data, 'hide_group_total_counts');
		$show_comments = array_var($data, 'show_comments');
		$report_category_id = array_var($data, 'reportCategorySel');
		$date_format = array_var($data, 'date_format');
		$show_currency = array_var($data, 'show_currency');
		$show_negatives = array_var($data, 'show_negatives');
		$dont_show_time = array_var($data, 'dont_show_time');
		$add_signature_and_date_field = array_var($data, 'add_signature_and_date_field');
		$calculate_quantity_for_actual_expenses = array_var($data, 'calculate_quantity_for_actual_expenses');
		
		$gb_data = "";
		if (is_array($group_by)) {
			$gb_keys = array_unique(array_filter($group_by));
			$gb_data = implode(',', $gb_keys);
			$options = array();
			$i=0;
			foreach ($gb_keys as $gbk) {
				$options[$gbk] = array_var($group_by_options, $i);
				$i++;
			}
			$object->setColumnValue('group_by_options', json_encode(array_filter($options)));
		}
		
		$object->setColumnValue('group_by', $gb_data);
		$object->setColumnValue('hide_group_details', $hide_group_details != '');
		$object->setColumnValue('hide_group_total_counts', $hide_group_total_counts != '');
		$object->setColumnValue('show_last_group_as_column', $show_last_group_as_column != '');
		$object->setColumnValue('show_comments', $show_comments != '');
		$object->setColumnValue('report_category_id', $report_category_id);
		$object->setColumnValue('date_format', $date_format);
		$object->setColumnValue('show_currency', $show_currency != '');
		$object->setColumnValue('show_negatives', $show_negatives != '');
		$object->setColumnValue('dont_show_time', $dont_show_time != '');
		$object->setColumnValue('add_signature_and_date_field', $add_signature_and_date_field != '');
		$object->setColumnValue('calculate_quantity_for_actual_expenses', $calculate_quantity_for_actual_expenses != '');
	}
}


function advanced_reports_more_report_header_info($params, &$ret) {
	
	$report_id = array_var($params, 'report_id');
	$results = array_var($params, 'results');
	$group_by = array_var($results, 'group_by_criterias');
	
	if (is_array($group_by) && count($group_by) > 0) {
		$report = Reports::findById($report_id);
		if (!$report instanceof Report) return;
		$ot = ObjectTypes::findById($report->getReportObjectTypeId());
		?>
		<div class="group-by-info custom-report-info-block">
			<h5><?php echo lang('group by')?>:</h5>
			<ul>
		<?php
		$gb_text = "";
		foreach ($group_by as $gb) {
			switch ($gb['type']) {
				case 'association':
					$a = DimensionMemberAssociations::findById($gb['id']);
					if ($a instanceof DimensionMemberAssociation) {
						$ot = ObjectTypes::findById($a->getAssociatedObjectType());
						if ($ot instanceof ObjectType) {
							$gb_text = lang($ot->getName());
						}
					}
					break;
				case 'parent_member':
					$gb_text = lang('located under');
					break;
				case 'classification':
					if (strpos($gb['id'], '_') !== false) {
						$exp = explode('_', $gb['id']);
						$dim_id = $exp[0];
						$ot_id = $exp[1];
					} else {
						$dim_id = $id;
						$ot_id = null;
					}
					if (is_null($ot_id)) {
						$d = Dimensions::findById($dim_id);
						if ($d instanceof Dimension) {
							$gb_text = $d->getName();
						}
					} else {
						$objtype = ObjectTypes::findById($ot_id);
						if ($objtype instanceof ObjectType) {
							$gb_text = $objtype->getPluralObjectTypeName();
						}
					}
					break;
				case 'fixed_property':
					$lang = Localization::instance()->lang('field '.$ot->getHandlerClass().' '.$gb['id']);
					if (is_null($lang)) $lang = Localization::instance()->lang('field Objects '.$gb['id']);
					if (is_null($lang)) $lang = $gb['id'];
					
					$gb_text = $lang;
					break;
				case 'custom_property': 
					if (in_array($ot->getType(), array('dimension_object','dimension_group'))) {
						$cp = MemberCustomProperties::findById($gb['id']);
					} else {
						$cp = CustomProperties::findById($gb['id']);
					}
					if ($cp) {
						$gb_text = clean($cp->getName());
						if ($cp->getIsSpecial()) {
							$label_code = str_replace("_special", "", $cp->getCode());
							$label_value = Localization::instance()->lang($label_code);
							if (!is_null($label_value)) $gb_text = $label_value;
						}
					}
					break;
				case 'intersection':
					
					$options = $report->getColumnValue('group_by_options');
					if (is_string($options)) {
						$options = json_decode($options, true);
					}
					
					$dimension_filters = array_filter($options['intersection']);
					if (count($dimension_filters) > 0) {
						$dimension_ids = array();
						foreach ($dimension_filters as $dimension_filter) {
							if (is_numeric($dimension_filter)) {
								$dimension_ids[] = $dimension_filter;
							} else if (is_array($dimension_filter) && isset($dimension_filter['dim_id'])) {
								if ($dimension_filter['dim_id']) {
									$dimension_ids[] = $dimension_filter['dim_id'];
								}
							}
						}
						$dim_names = array();
						$dims = Dimensions::findAll(array("conditions" => "id IN (".implode(',',$dimension_ids).")"));
						foreach ($dims as $d) {
							$dim_names[] = $d->getName();
						}
						$gb_text = lang('intersection of', implode(', ', $dim_names));
					} else {
						$gb_text = lang('intersection');
					}
					break;
				default: break;
			}
			?><li><?php echo $gb_text ?></li><?php 
		}
		?>
			</ul>
		</div>
		<?php 
	}
}

function advanced_reports_execute_object_custom_report($parameters, &$results) {
	$report = array_var($parameters, 'report');
	if (!$report instanceof Report) return;
	
	$ot = ObjectTypes::findById($report->getReportObjectTypeId());

	$cond_end_time_not_set = '';
	if($ot->getName() == 'timeslot'){
		$cond_end_time_not_set = "AND `e`.`end_time` <> '0000-00-00 00:00:00'";
	}
	
	if ($ot && !in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
		Env::useHelper('advanced_reports', 'advanced_reports');
		
		$order_join = "";
		if (isset($parameters['join_params'])) {
			$order_join = ContentDataObjects::prepareJoinConditions($parameters['join_params']);
		}
		
		$allConditions = array_var($parameters, 'conditions');
		
		$group_by = explode(',', $report->getColumnValue('group_by'));
		Hook::fire('override_custom_report_group_by', $report, $group_by);
		
		$group_by_options = null;
		$gb_options = trim($report->getColumnValue('group_by_options'));
		if ($gb_options) {
			$group_by_options = json_decode($gb_options, true);
		}
		
		$group_by_criterias = parse_custom_report_group_by($group_by, $group_by_options);
		// don't use this function if it is not grouping by anysthing
		if (!$group_by_criterias || count($group_by_criterias) == 0) return;
		
		$select_columns = "DISTINCT(o.id) as mid ";
		//additional options when order is by CP
		$more_select_columns = array_var($parameters, 'more_select_columns', '');
		$join_str = array_var($parameters, 'join_str', '');
		
		$order_sql_string = "ORDER BY ".$parameters['order']." ".$parameters['order_dir'];
		$limit_string = "";
		if (array_var($parameters, 'limit') > 0) {
			$limit_string = "LIMIT ".$parameters['offset'].", ".$parameters['limit'];
		}

		// permissions
		$permissions_condition = "";
		if (!logged_user()->isAdministrator()) {
			$logged_user_pgs = logged_user()->getPermissionGroupIds();
			$permissions_condition = "AND FALSE";
			if (count($logged_user_pgs) > 0) {
				$permissions_condition = "AND EXISTS (SELECT sh.object_id FROM ".TABLE_PREFIX."sharing_table sh WHERE sh.group_id IN (".implode(',',$logged_user_pgs).") AND sh.object_id=o.id)";
			}
		}

		// active context filter
		$context_condition = "";
		$context_members = active_context_members(false);
		if (count($context_members)) {
			$context_condition = " AND (EXISTS (SELECT om.object_id
				FROM ".TABLE_PREFIX."object_members om
				WHERE om.member_id IN (" . implode ( ',', $context_members) . ") AND o.id = om.object_id
					GROUP BY object_id
					HAVING count(member_id) = ".count($context_members)."
			))";
		}
		
		$group_by_sql = "";
		$original_conditions = $allConditions;
		
		// build group_by variables
		object_reports_build_group_by_sql($group_by_criterias, $more_select_columns, $join_str, $order_sql_string, $group_by_sql, $ot, $allConditions, $original_conditions);
		
		$object_join = "INNER JOIN ".TABLE_PREFIX.$ot->getTableName()." e ON e.object_id=o.id";
        advanced_reports_additional_column(array('ot'=>$ot),$object_join);

		if (!$report->getColumnValue("hide_group_details")) {
			$main_sql = "
				SELECT $select_columns $more_select_columns
					FROM ".TABLE_PREFIX."objects o
					$object_join
					$join_str
					$order_join
				WHERE o.object_type_id=".$ot->getId()." AND o.trashed_by_id=0
					$cond_end_time_not_set $allConditions
					$permissions_condition
					$context_condition
			";
			
			$order_limit_sql = "
				$order_sql_string
				$limit_string
			";
			
			$sql = $main_sql . $group_by_sql . $order_limit_sql;
			$rows = DB::executeAll($sql);
			
			// apply options such as adding the empty groups
			$tmp = apply_gb_options_before_grouping($rows, $report, $group_by_criterias, $main_sql, $parameters);
			
			// make the group structure to print
			$grouped_rows = group_custom_report_results($tmp, $group_by_criterias, $ot);
			
		}
		
		$conditions_sql = "$cond_end_time_not_set $allConditions $permissions_condition $context_condition";
		$joins_sql = "$object_join $join_str $order_join";
		
		$group_totals = array();
		$params_array = array(
				'report' => $report,
				'conditions' => "o.object_type_id=".$ot->getId()."  AND o.trashed_by_id=0 $conditions_sql",
				'columns' => "$select_columns $more_select_columns",
				'from' => "".TABLE_PREFIX."objects o $joins_sql",
				'order' => $order_sql_string,
                'group_by_criterias' => $group_by_criterias
		);
		Hook::fire('custom_report_group_totals', $params_array, $group_totals);
		
		$count_sql = "SELECT count(distinct(o.id)) as totalrows $more_select_columns FROM ".TABLE_PREFIX."objects o $joins_sql
			WHERE o.object_type_id=".$ot->getId()." AND o.trashed_by_id=0 $conditions_sql $order_sql_string";
		
		$time = microtime(true);
		$totalrows = DB::executeAll($count_sql);
		$totalResults = $totalrows[0]['totalrows'];
		
		// if hiding group details, build the result with the totals and return
		if ($report->getColumnValue("hide_group_details")) {


			$tmp = _accumulate_group_totals_hidden_details($report, $ot, $group_totals);
			
			// apply options such as adding the empty groups
			$tmp = apply_gb_options_before_grouping($tmp, $report, $group_by_criterias);
			
			// make the group structure to print
			$grouped_rows = group_custom_report_results($tmp, $group_by_criterias, $ot,false);
			
			$results = array(
				'group_totals' => $group_totals,
				'grouped_rows' => $grouped_rows,
				'hide_group_details' => true,
				'total' => $totalResults,
				'group_by_criterias' => $group_by_criterias,
				'pagination' => '',
				'objects' => array(),
			);
			
			return;
		}
		
		$object_ids = array();
		if (is_array($rows)) {
			foreach ($rows as $row) {
				$object_ids[] = $row['mid'];
			}
		}
		
		$all_objects = array();
		if (count($object_ids) > 0) {
			
			eval('$managerInstance = ' . $ot->getHandlerClass() . "::instance();");
			if ($managerInstance) {
				$objects = $managerInstance->findAll(array('conditions' => 'id IN ('.implode(',', $object_ids).')'));
			}
			
			// follow the order resultant of the first query
			foreach ($object_ids as $oid) {
				foreach ($objects as $object) {
					if ($oid == $object->getId()) {
						if ($object instanceof Timeslot && $object->getRelObjectId()>0) {
							if (trim($object->getDescription())=='') {
								$rel_object = $object->getRelObject();
								if ($rel_object instanceof ContentDataObject) $object->setObjectName($rel_object->getObjectName());
							} else {
								$object->setObjectName($object->getDescription());
							}
						}
						$all_objects[] = $object;
						break;
					}
				}
			}
			
			$results = array(
					'objects' => $all_objects,
					'total' => $totalResults,
			);
			
			$results['group_by_criterias'] = $group_by_criterias;
			$results['grouped_rows'] = $grouped_rows;
			$results['use_obj_id_as_row_key'] = true;
			$results['group_totals'] = $group_totals;

		}
	}
}



function get_external_column_info($object_type, $column) {
	$result = array('table' => 'objects', 'name_col' => 'name', 'link_col' => 'id');
	
	Hook::fire('override_report_grouping_external_column_info', array('object_type'=>$object_type,'column'=>$column), $result);
	
	return $result;
}


function object_reports_build_group_by_sql($group_by_criterias, &$select_columns, &$joins_sql, &$order_by_sql, &$group_by_str, $ot, &$all_conditions, $original_conditions) {

	$order_sql = "";
	foreach ($group_by_criterias as $criteria) {
		$id = $criteria['id'];
		switch ($criteria['type']) {
			case 'intersection':
				$options = array_var($criteria, 'options');
				if (is_string($options)) {
					$options = json_decode($options, true);
				}
				$dimension_ids = array_filter($options);
				
				if (count($dimension_ids) > 0) {
					// fill temp dimension intersection table
					$inter_info = fill_custom_report_intersection_table($ot, $dimension_ids, $original_conditions);
					$inter_table = array_var($inter_info, 'table_name');
					
					$select_columns .= ", inter.member_ids as _group_id_inter_$id, inter.member_names as _group_name_inter_$id";
					
					$joins_sql .= "
						LEFT JOIN $inter_table inter ON inter.object_id=o.id
					";
					$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "_group_name_inter_$id";
				}
				
				break;
				
			case 'classification':
				
				if (strpos($id, '_') !== false) {
					$exp = explode('_', $id);
					$dim_id = $exp[0];
					$ot_id = $exp[1];
				} else {
					$dim_id = $id;
					$ot_id = null;
				}
				
				$member_ids_filter = null;
				
				$options = array_var($criteria, 'options');
				if (is_string($options)) {
					$options = json_decode($options, true);
				}
				if (is_array($options)) {
					$member_ids_filter = array_unique(array_filter($options)); 
				}
				
				// when grouping by folder, include all types of them, user doesn't know that there are three different kinds of folder
				$folder_ot_ids = ObjectTypes::findAll(array("id" => true, "conditions" => "name IN ('folder','project_folder','customer_folder')"));
				if (in_array($ot_id, $folder_ot_ids)) {
					$ot_id = implode(',', $folder_ot_ids);
				}
				
				if (is_null($ot_id)) {
					// use is_optimization to get the last member of the hierarchy
					$opt_join_cond = " AND om_$id.is_optimization=0";
					$ot_id = '0';
                } else {
					if (is_array($member_ids_filter) && count($member_ids_filter) > 0) {
						// only make groups using the specified members
						$more_join_cond = " AND mx_$id.id IN (".implode(',', $member_ids_filter).")";
						
					} else {
						// dont use is_optimization and use depth to group by the last member of the type in the hierarchy
						$more_join_cond = "
						AND NOT EXISTS (
							SELECT my_$id.id FROM ".TABLE_PREFIX."members my_$id
							INNER JOIN ".TABLE_PREFIX."object_members tom_$id ON my_$id.id=tom_$id.member_id
							WHERE my_$id.dimension_id='$dim_id' AND my_$id.object_type_id IN ($ot_id) AND tom_$id.object_id=o.id
								AND my_$id.depth > mx_$id.depth
						)";
						
					}
					$opt_join_cond = $more_join_cond;
				}
				
				$select_columns .= ",
					(if (mx_$id.object_type_id IN ($ot_id), COALESCE(mx_$id.id, 0), 0)) as _group_id_dim_$id,
					(if (mx_$id.object_type_id IN ($ot_id), mx_$id.name, NULL)) as _group_name_dim_$id";
				
				$joins_sql .= "
					LEFT JOIN ".TABLE_PREFIX."object_members om_$id ON om_$id.object_id=o.id
						AND (ISNULL(om_$id.member_id) OR om_$id.member_id IN (SELECT id FROM ".TABLE_PREFIX."members WHERE dimension_id='$dim_id' AND object_type_id IN ($ot_id)))
						
					LEFT JOIN ".TABLE_PREFIX."members mx_$id ON om_$id.member_id=mx_$id.id AND mx_$id.dimension_id='$dim_id'
					$opt_join_cond
				";
				
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "ISNULL(_group_name_dim_$id), _group_name_dim_$id";
				
				// select rows with the group by dimension or if it is null check that not exists classification in this dimension
				$all_conditions .= " AND (
					IF (ISNULL(mx_$id.dimension_id), 
						NOT EXISTS (
							SELECT object_id FROM ".TABLE_PREFIX."object_members 
							WHERE object_id=o.id AND member_id IN (
								SELECT id FROM ".TABLE_PREFIX."members WHERE dimension_id='$dim_id' AND object_type_id IN ($ot_id)
							)
						),
						mx_$id.dimension_id='$dim_id')
					)
				";
				
				break;
				
			case 'fixed_property':
				
				$t_prefix = "e";
				if (in_array($id, Objects::instance()->getColumns())) {
					$t_prefix = "o";
				}
				
				$managerInstance = null;
				eval('$managerInstance = ' . $ot->getHandlerClass() . "::instance();");

				// don't user hour and minutes when grouping by date columns
				if ($managerInstance && in_array($managerInstance->getColumnType($id), array(DATA_TYPE_DATETIME, DATA_TYPE_DATE))) {
					// apply object timezone to the date value to make the group
					$select_columns .= ", TRIM(DATE(ADDDATE($t_prefix.$id, INTERVAL o.timezone_value SECOND))) as _group_id_fp_$id";
				} else {
					$select_columns .= ", $t_prefix.$id as _group_id_fp_$id";
				}
				
				if ($managerInstance) {
					if (in_array($id, $managerInstance->getExternalColumns())) {
						$ext_col_info = get_external_column_info($ot, $id);
						$joins_sql .= "
							left join ".TABLE_PREFIX.$ext_col_info['table']." namestable_$id ON namestable_$id.".$ext_col_info['link_col']." = $t_prefix.$id
						";
						$ext_col_select_columns = ", namestable_$id.".$ext_col_info['name_col']." as _group_name_fp_$id";
						
						$hook_return_vals = null;
						Hook::fire('override_report_group_ext_col_order_sql', array('ext_col_info' => $ext_col_info, 'ot' => $ot, 'col' => $id, 't_prefix'=>$t_prefix), $hook_return_vals);
						if ($hook_return_vals) {
							if ($hook_return_vals['joins_sql']) $joins_sql .= $hook_return_vals['joins_sql'];
							if ($hook_return_vals['ext_col_select_columns']) $ext_col_select_columns .= $hook_return_vals['ext_col_select_columns'];
							if ($hook_return_vals['order_column']) $custom_order_column = $hook_return_vals['order_column'];
						}
						
					} else {
						// don't user hour and minutes when grouping by date columns
						$mysql_date_format = get_mysql_date_format_from_config_option();
						if (in_array($managerInstance->getColumnType($id), array(DATA_TYPE_DATETIME, DATA_TYPE_DATE))) {
							// apply object timezone to the date value to make the group
							$ext_col_select_columns = ", DATE_FORMAT(DATE(ADDDATE($t_prefix.$id, INTERVAL o.timezone_value SECOND)), '$mysql_date_format') as _group_name_fp_$id";
						} else {
							$ext_col_select_columns = ", $t_prefix.$id as _group_name_fp_$id";
						}
					}
				} else {
					$ext_col_select_columns = ", $t_prefix.$id as _group_name_fp_$id";
				}
				
				// gb options
				$options = array_var($criteria, 'options');
				if (is_string($options)) {
					$options = json_decode($options, true);
				}
				$group_conditions = array_var($options, 'conditions');
				if (is_array($group_conditions) && in_array($id, array('contact_id', 'assigned_to_contact_id'))) {
					$conds_fields = array();
					$conds_cp = array();
					$dummy_report = new Report();
					$dummy_report->setReportObjectTypeId(Contacts::instance()->getObjectTypeId());
					
					foreach ($group_conditions as $group_cond) {
						$cond = new ReportCondition();
						$cond->setCondition($group_cond['condition']);
						$cond->setValue($group_cond['value']);
						if (is_numeric($group_cond['field_name'])) {
							$cond->setCustomPropertyId($group_cond['field_name']);
							$conds_cp[] = $cond;
						} else {
							$cond->setFieldName($group_cond['field_name']);
							$conds_fields[] = $cond;
						}
					}
					
					$group_conditions_data = build_report_conditions_sql(array('report' => $dummy_report, 'conditionsFields' => $conds_fields, 'conditionsCp' => $conds_cp));
					$group_conditions_sql = array_var($group_conditions_data, 'all_conditions');
					$group_conditions_sql = str_replace("e.", "eee.", $group_conditions_sql);
					$group_conditions_sql = str_replace("`e`.", "eee.", $group_conditions_sql);
					$group_conditions_sql = str_replace("o.", "ooo.", $group_conditions_sql);
					$group_conditions_sql = str_replace("`o`.", "ooo.", $group_conditions_sql);
					
					$joins_sql .= "
						LEFT JOIN ".TABLE_PREFIX."contacts eee ON eee.object_id=$t_prefix.$id
						LEFT JOIN ".TABLE_PREFIX."objects ooo ON ooo.id=$t_prefix.$id
					";
					$all_conditions .= " $group_conditions_sql ";
				}
				
				
				$select_columns .= $ext_col_select_columns;
				
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . (isset($custom_order_column) ? $custom_order_column : "_group_name_fp_$id");
				
				break;
			case 'custom_property':

				$cp = CustomProperties::findById($id);

				$select_columns .= ", COALESCE(NULLIF(y_$id.value,''), 0) as _group_id_cp_$id";
				$joins_sql .= "
					left join ".TABLE_PREFIX."custom_property_values y_$id on y_$id.custom_property_id=$id and y_$id.object_id=o.id
				";
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "_group_name_cp_$id";

				$select_columns .= ", y_$id.value as _group_name_cp_$id";
				
				break;
			default:
				break;
		}
	}

	if (count($group_by_criterias) > 0) {
		$last_order_by = "";
		if (trim($order_by_sql) != "") {
			$last_order_by = str_replace("ORDER BY ", "", $order_by_sql);
		}
		$order_by_sql = "$order_sql, $last_order_by";
	}

}




function _accumulate_group_totals_hidden_details($report, $ot, $group_totals) {
	
	$columns_to_aggregate = array();
	$manager = null;
	if (trim($ot->getHandlerClass()) != '' && class_exists($ot->getHandlerClass())) {
		eval('$manager = ' . $ot->getHandlerClass() . "::instance();");
	}
		
	if ($manager) {
		$columns_to_aggregate_info = $manager->getColumnsToAggregateInTotals();
		Hook::fire('list_totals_accumulate_total_rows', array('object_type' => $ot, 'report' => $report), $columns_to_aggregate_info);
		$columns_to_aggregate = array_keys($columns_to_aggregate_info);
	}

	$group_totals_accumulated = array();
	
	foreach ($group_totals as $gtotal) {
		foreach ($gtotal as $gt) {
			$group_id = "";
			foreach ($gt as $col => $val) {
				if (str_starts_with($col, '_group_id_')) {
					$group_id .= ($group_id=="" ? "" : "_") . $val;
					if (!isset($group_totals_accumulated[$group_id])) {
						$group_totals_accumulated[$group_id] = array();
					}
				}
			}
		
			foreach ($gt as $col => $val) {
					
				if (in_array($col, $columns_to_aggregate)) {
		
					if ($ot->getName() == 'expense' && str_starts_with($col, 'total_')) {
							
						$currency_id = 0;
						if (str_starts_with($col, 'total_')) {
							$currency_id = array_var($gt, str_replace('total_', 'currency_id_', $col));
						}
						if (is_null($currency_id)) continue;
							
						if (!isset($group_totals_accumulated[$group_id][$col])) {
							$group_totals_accumulated[$group_id][$col] = array($currency_id => 0);
						}
							
						$group_totals_accumulated[$group_id][$col][$currency_id] += $val;
							
					} else {
						if (!isset($group_totals_accumulated[$group_id][$col])) $group_totals_accumulated[$group_id][$col] = 0;
						$group_totals_accumulated[$group_id][$col] += $val;
					}
		
				} else {
					$group_totals_accumulated[$group_id][$col] = $val;
				}
					
			}
		}
	}
	
	return $group_totals_accumulated;
}



function advanced_reports_modify_custom_report_results($params, &$results) {
	
	$report = array_var($params, 'report');
	Env::useHelper('advanced_reports', 'advanced_reports');
	
	report_append_group_columns($report, $results);
	
}

function advanced_reports_set_custom_format_value_date($params, &$row) {
    $report = array_var($params, 'report');

    $formato = $report->getColumnValue('date_format');
    foreach ($row as $key => $value) {
        try {
            $date = getDateValue($value);
            if ($date instanceof DateTimeValue) {
                $formatted = $date->format($formato);
                $row[$key] = $formatted;
            }
        } catch (Exception $ex) {}
    }
}

function advanced_reports_override_action_view($params, &$results) {
    if ($params->getControllerName() == 'reporting' &&$params->getAction() == 'index') {
        $params->setTemplate(get_template_path('index', 'advanced_reports', 'advanced_reports'));
    }
}

function advanced_reports_show_category_selector($report, &$results) {
	if(is_array($report)){
		$params = $report;
		$report = $params['object'];
		$genid = $params['genid'];
	}
    $selected_category = $report->getColumnValue('report_category_id');
    $list_of_categories = ReportCategories::getAllCategories();
    $categories = array(array("", lang("select one")));
    if (!empty($list_of_categories)) {
        foreach ($list_of_categories as $category) {
            $categories[] = array($category->getId(), $category->getName());
        }
    }
    $results = '
    <div class="dataBlock">';
    $results.= label_tag(lang('Category'), $genid . 'reportFormCategory', $report->getFunctionUrl() ? false : true);
    $selected_option = null;
    $optionsCategory = array();
    foreach ($categories as $category) {
        if ($selected_category == $category[0]) {
            $selected = 'selected="selected"';
            $selected_option = $category[1];
        } else {
            $selected = '';
        }
        $optionsCategory[] = '<option value="' . $category[0] . '" ' . $selected . '>' . $category[1] . '</option>';
    }
    $results.= select_box('report[reportCategorySel]', $optionsCategory, array('id' => 'reportCategorySel', 'onchange' => '', 'style' => 'width:200px;', $strDisabled => '', 'tabindex' => '10'));
    $results.="</div>";
}


function advanced_reports_additional_column_timeslots($params,&$results){

    $ot = array_var($params,'ot');
    if ($ot->getTableName() == Timeslots::instance()->getTableName(false,false)){
        $results[] = 'time_estimate';
        $results[] = 'bill_total_price';
        $results[] = 'bill_total_cost';
    }

}

function advanced_reports_additional_column($params,&$object_join){
    $ot = array_var($params,'ot');
    if ($ot->getTableName() == Timeslots::instance()->getTableName(false,false)){

        $object_join = "INNER JOIN ".TABLE_PREFIX.$ot->getTableName()." e ON e.object_id=o.id
                LEFT JOIN ".TABLE_PREFIX.ProjectTasks::instance()->getTableName(false,false)." t ON e.rel_object_id=t.object_id
            ";
    }
}

function advanced_reports_additional_totals_column($params,&$columns){
    $report = array_var($params, 'object_type');
    if ($report->getTableName() == Timeslots::instance()->getTableName(false,false)){
        //$columns .= ", SUM( DISTINCT(CASE WHEN t.time_estimate is null THEN 0 ELSE t.time_estimate END )) as time_estimate ";
        $columns .= ",(if (COUNT(DISTINCT(o.id)) > COUNT(DISTINCT(t.object_id)),  SUM(DISTINCT (CASE WHEN t.time_estimate is null THEN 0 ELSE t.time_estimate END )) ,  SUM( (CASE WHEN t.time_estimate is null THEN 0 ELSE t.time_estimate END )))) as time_estimate ";
    }
}

function advanced_reports_get_allow_columns_for_reports($params, &$columns)
{
    $option_groups = $columns[1];
    $allowed_columns = $columns[0];
    $object_type = array_var($params, 'object_type');

    if ($object_type == Customers::instance()->getObjectTypeId()) {
        $ts_group_count = count($allowed_columns);
        $option_groups[] = array('count' => $ts_group_count-1, 'name' => lang('customer').' '.lang('columns'));
    }

    if ($object_type == Customers::instance()->getObjectTypeId()) {


        $id_object_type_contact = ObjectTypes::findByName('contact');
        $id_object_type_company = ObjectTypes::findByName('company');

        $id_object_type_contact = $id_object_type_contact->getId();
        $id_object_type_company = $id_object_type_company->getId();

        $managerContactsInstance = Contacts::instance();
        $columnContacts = $managerContactsInstance->getReportColumns();

        $langs = Contacts::getContactFieldNames();
        foreach ($columnContacts as $key => $value) {
            $allowed_columns[] = array('id' => 'contact_' . $value, 'name' => $langs['contact['.$value.']'], 'type' => $value);
        }


        $cps_1 = CustomProperties::getAllCustomPropertiesByObjectType($id_object_type_contact);
        foreach ($cps_1 as $cp) {
            if ($cp instanceof CustomProperty) {
                $cp_id = $cp->getId();
                $cp_name = $cp->getName();
                $allowed_columns[] = array('id' => '' . $cp_id, 'name' => $cp_name, 'type' => $cp_name);
            }
        }

        $cps_2 = CustomProperties::getAllCustomPropertiesByObjectType($id_object_type_company);
        foreach ($cps_2 as $cp) {
            if ($cp instanceof CustomProperty) {
                $cp_id = $cp->getId();
                $cp_name = $cp->getName();
                $allowed_columns[] = array('id' => '' . $cp_id, 'name' => $cp_name, 'type' => $cp_name);
            }
        }
        $option_groups[] = array('count' =>  count($columnContacts)+count($cps_1)+count($cps_2), 'name' => lang('contact') . ' ' . lang('columns'));


        $columns_in_list = array_var($params, 'columns');

        $customProperties = CustomProperties::getAllCustomPropertiesByObjectType($object_type);
        $columnContacts = $managerContactsInstance->getReportColumnsCustomPropertyAssc();
        foreach($customProperties as $cp){
            if ($cp->getType() != 'contact') continue;

            $allowed_columns[] = array('id' => $cp->getId(), 'name' => $cp->getName(), 'type' => $cp->getType(), 'values' => $cp->getValues(), 'multiple' => $cp->getIsMultipleValues());
            $total_columns = 1;
            foreach ($columnContacts as $key => $value) {
                $allowed_columns[] = array('id' => 'group_cp_'.$cp->getId().'_' . $value, 'name' => $langs['contact['.$value.']']. ' ('.$cp->getName().')', 'type' => $value);
                if (!in_array('group_cp_'.$cp->getId().'_' . $value, $columns_in_list)) {
                    $total_columns ++;
                }
            }

            $option_groups[] = array('count' =>  $total_columns, 'name' => $cp->getName() . ' ' . lang('columns'));
        }

    } else if ($object_type == Timeslots::instance()->getObjectTypeId()) {
    	
    	// include timeslot's project custom properties
    	$project_cps = CustomProperties::getAllCustomPropertiesByObjectType(Projects::instance()->getObjectTypeId());
    	foreach ($project_cps as $cp) {
    		if ($cp instanceof CustomProperty) {
    			$cp_id = $cp->getId();
    			$cp_name = $cp->getName();
    			$allowed_columns[] = array('id' => '' . $cp_id, 'name' => $cp_name, 'type' => $cp_name);
    		}
    	}
    	$option_groups[] = array('count' => count($project_cps), 'name' => lang('project') . ' ' . lang('columns'));
    	
    }
    $columns[0] = $allowed_columns;
    $columns[1] = $option_groups;
}

/**
 *
 * @param $params
 * @param $results
 */
function advanced_reports_get_more_columns_to_header_report($params, &$results) {
/*
    $object_type = array_var($params, 'object_type');
    $report = array_var($params, 'report');
    if ($object_type instanceof ObjectType){
        if($report->getColumnValue('show_comments')){
            $results['columns']['names']['show_comments'] = lang('comments');
            $results['columns']['order'][] = 'show_comments';
            $results['columns']['types']['show_comments'] = DATA_TYPE_STRING;
        }
    }
*/
}

/**
 *
 * @param $params
 * @param $results
 * @throws DBQueryError
 * @throws Exception
 */
function advanced_reports_report_results_more_columns_value($params, &$results) {
    $report = array_var($params, 'report');
    $object = array_var($params, 'object');

    if ($report->getColumnValue('show_comments')) {
        $comments = array(); // object_id => comments_array
        $oids = array();

        if ($object instanceof ContentDataObject) {
                $oids[] = $object->getId();
        }

        if (count($oids) > 0) {
            $comment_rows = DB::executeAll("SELECT c.*,o.created_on,o.created_by_id FROM ".TABLE_PREFIX."comments c 
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=c.object_id
					WHERE rel_object_id IN (".implode(',', $oids).")");
            if (is_array($comment_rows)) {
	            foreach ($comment_rows as $crow) {
	                $oid = $crow['rel_object_id'];
	                if (!isset($comments)) $comments = array();
	                $comments[] = $crow;
	            }
            }
        }

        $coments_result = array();
        $contacts_cache = array();
        if(isset($comments)){
            foreach ($comments as $comment) {

                $d = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $comment['created_on']);
                $date_str = mb_convert_encoding(friendly_date($d), "HTML-ENTITIES", "UTF-8");

                $c = array_var($contacts_cache, $comment['created_by_id']);
                if (!$c instanceof Contact) {
                    $c = Contacts::findById($comment['created_by_id']);
                    $contacts_cache[$comment['created_by_id']] = $c;
                }

                $user_str = $c instanceof Contact ? $c->getObjectName() : '';

                $content = "$user_str, $date_str: " . $comment['text'];
                $coments_result[] = $content;
            }
        }

		$results['show_comments'] = (count($coments_result) > 0 ? " - " : '') . implode('<br /> - ',$coments_result);
    }
}

function advanced_reports_after_report_table_html_row($params, &$ignored) {
	$row = array_var($params, 'row');
	$report = array_var($params, 'report');
	$results = array_var($params, 'results');
	$is_alt = array_var($params, 'is_alt');
	
	if ($report->getColumnValue('show_comments') && trim(array_var($row, 'show_comments', '')) != '') {
		$colspan = count($results['columns']['names']);
		?>
		<tr class="report-row-tbody report-data comment <?php echo ($is_alt ? 'alt' : "");?>">
			<td colspan="<?php echo $colspan?>"><?php echo trim(array_var($row, 'show_comments', '')) ?></td>
		</tr>
		<?php 
	}
}

function advanced_reports_after_report_table_plain_row($params, &$all_data_rows) {
	$row = array_var($params, 'row');
	$report = array_var($params, 'report');
	$results = array_var($params, 'results');
	
	if ($report->getColumnValue('show_comments') && trim(array_var($row, 'show_comments', '')) != '') {
		$colspan = count($results['columns']['names']);
		$comments = explode('<br />', trim(array_var($row, 'show_comments', '')));
		$padding = "    ";
		foreach ($comments as $c) {
			$new_row = array($padding . trim(strip_tags($c)));
			while (count($new_row) < $colspan-1) $new_row[] = '';
			$all_data_rows[] = $new_row;
		}
	}
}


function advanced_reports_excel_export_additional_header_content($params, &$data_start_row) {

	$report = array_var($params, 'report');
	$sheet = array_var($params, 'sheet');

	if ($report instanceof Report && $report->getColumnValue('add_signature_and_date_field')) {

		$excel_controller = new ExcelExportController();

		if ($excel_controller->can_use_php_spreadsheet_lib()) {
			$right_alingment = \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT;
		} else {
			$right_alingment = PHPExcel_Style_Alignment::HORIZONTAL_RIGHT;
		}

		$row = $sheet->getHighestRow()+1;
		$sheet->insertNewRowBefore($row);
		$sheet->setCellValue('A'.$row, lang('signature label'));
		$sheet->getStyle("A".$row)->applyFromArray(array('alignment' => ['horizontal' => $right_alingment]));
		$excel_controller->apply_borders_to_cells($sheet, "B".$row, '333333', 'bottom');
		
		$row = $sheet->getHighestRow()+1;
		$sheet->insertNewRowBefore($row);
		$sheet->setCellValue('A'.$row, lang('name'));
		$sheet->getStyle("A".$row)->applyFromArray(array('alignment' => ['horizontal' => $right_alingment]));
		$excel_controller->apply_borders_to_cells($sheet, "B".$row, '333333', 'bottom');
		
		$row = $sheet->getHighestRow()+1;
		$sheet->insertNewRowBefore($row);
		$sheet->setCellValue('A'.$row, lang('date label'));
		$sheet->getStyle("A".$row)->applyFromArray(array('alignment' => ['horizontal' => $right_alingment]));
		$excel_controller->apply_borders_to_cells($sheet, "B".$row, '333333', 'bottom');
		
		$data_start_row = $row + 1;
	}
}


function advanced_reports_export_report_pdf_more_content($params, &$html) {
	$results = array_var($params, 'results');
	$report = array_var($params, 'report');

	if ($report->getColumnValue('add_signature_and_date_field')) {
		$html .= '
			<div class="signature-date">
				<div class="col-sig">
					<div class="line"></div>
					<b>'.lang('name').'</b>
				</div>
				<div class="col-sig">
					<div class="line"></div>
					<b>'.lang('signature label').'</b>
				</div>
				<div class="col-sig">
					<div class="line"></div>
					<b>'.lang('date label').'</b>
				</div>
			</div>
		';
	}
}

