<?php

function save_night_hours($night_hours_data) {
	
	if (is_array($night_hours_data) && count($night_hours_data) > 0) {
			
		$start_data = getTimeValue($night_hours_data['start_time']);
		$end_data = getTimeValue($night_hours_data['end_time']);
		$hour_type_ids = json_decode($night_hours_data['hour_type_id'], true);
		$hour_type_id = array_var($hour_type_ids, 0);
		
		set_config_option('min_night_hours', $night_hours_data['min_night_hours']);
		set_config_option('min_percentage_to_mark_night_hours', $night_hours_data['min_percentage_to_mark_night_hours']);
		
		$night_hours_def = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."work_night_hours ORDER BY id DESC");
		if (is_array($night_hours_def) && count($night_hours_def) > 0) {
			$sql = "UPDATE ".TABLE_PREFIX."work_night_hours SET
							start_hour='".$start_data['hours']."', start_minute='".$start_data['mins']."',
							end_hour='".$end_data['hours']."', end_minute='".$end_data['mins']."',
							hour_type_id='".$hour_type_id."'
						WHERE id=".$night_hours_def['id'];
		} else {
			$sql = "INSERT INTO ".TABLE_PREFIX."work_night_hours (start_hour, start_minute, end_hour, end_minute, hour_type_id) VALUES
						('".$start_data['hours']."', '".$start_data['mins']."', '".$end_data['hours']."', '".$end_data['mins']."', '".$hour_type_id."')";
		}
		DB::execute($sql);
	}
	
}

function get_night_hours_config() {
	$night_hours_def = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."work_night_hours ORDER BY id DESC");
	return $night_hours_def;
}


function save_contact_work_shifts($object, $shifts_data) {
	if (!is_array($shifts_data)) return;
	
	ContactShifts::delete("contact_id=".$object->getId());
	foreach ($shifts_data as $shift_data) {
		$shift_id = $shift_data['id'];
		$takes_break = $shift_data['takes_break'];
		
		$contact_shift = new ContactShift();
		$contact_shift->setContactId($object->getId());
		$contact_shift->setShiftId($shift_id);
		$contact_shift->setTakesBreak($takes_break);
		$contact_shift->save();
	}
	
}



/**
 * Splits the timeslot by the hour types defined in the configuration.
 * Uses the contact's work shift to calculate normal hours.
 * Uses the night hours configuration to split the night hours.
 * Verifies all the user's timeslots in current day to split in extra hours timeslots.
 * Users non-working days to classify the resulting timeslots with holiday hour type.
 * 
 * @return array of timeslots data after splitting the original timeslot using the hour types
 * @param ProjectTask $task
 * @param array $original_ts_data
 */
function split_timeslots_by_hour_type($task, $original_ts_data) {
	
	$task_member_ids = $task->getMemberIds();
	
	$worked_hour_type_members = config_option("worked_hour_type_members");
	
	$mem_intersection = array_intersect($task_member_ids, $worked_hour_type_members);
	
	// if timeslot doesnt have any member of the worked hour types then dont split
	if (count($mem_intersection) == 0) {
		return array($original_ts_data);
	}
		
	// initial date
	$timeslot_start_date = $original_ts_data['date'];
	// total minutes
	$total_timeslot_minutes = $original_ts_data['hours'] * 60 + $original_ts_data['minutes'];
	
	$contact = $task->getAssignedTo();
	
	$all_timeslots = array();
	
	// split by night hours
	if (config_option("use_night_hours")) {
		$normal_and_night_timeslots = split_normal_and_night_hours($contact, $timeslot_start_date, $total_timeslot_minutes);
	} else {
		$normal_and_night_timeslots = array($original_ts_data);
	}
	
	// timeslots not classified in night service member must be classified in the day_hours_member
	$night_hours_config = get_night_hours_config();
	$night_hours_mem_id = $night_hours_config['hour_type_id'];
	$day_hours_mem_id = config_option('day_hours_member');
	
	// timeslots classified in the "with weapon service member" must be classified in the "with weapon hours member"
	$with_weapon_service_mem_id = config_option('with_weapon_service_member');
	$with_weapon_hours_mem_id = config_option('with_weapon_hours_member');
	
	foreach ($normal_and_night_timeslots as &$timeslot) {
		// if not in night_hour_type then apply day_hours_member to the timeslot
		$n_member_ids = array_filter(array_var($timeslot, 'additional_member_ids'));
		if (!in_array($night_hours_mem_id, $n_member_ids)) {
			$timeslot['additional_member_ids'][] = $day_hours_mem_id;
		}
		// check services, if with_weapon_service is in task, then apply with_weapon_hour_type to the timeslot
		if (in_array($with_weapon_service_mem_id, $task_member_ids)) {
			$timeslot['additional_member_ids'][] = $with_weapon_hours_mem_id;
		}
	}
	
		
	// split by extra hours
	$all_extra_timeslots = array();
	$extra_hours_member_id = config_option('extra_hours_member');
	$processed_normal_minutes = 0;
	
	foreach ($normal_and_night_timeslots as $ts_data) {
		$current_ts_minutes = $ts_data['hours'] * 60 + $ts_data['minutes'];
		
		// make the split by extra hours
		$extra_timeslots = split_normal_and_extra_hours($contact, $ts_data['date'], $current_ts_minutes, $processed_normal_minutes, $ts_data['additional_member_ids']);
		
		// accumulate normal processed minutes
		if (!in_array($extra_hours_member_id, $ts_data['additional_member_ids'])) {
			$processed_normal_minutes += $current_ts_minutes;
		}
		
		$all_extra_timeslots = array_merge($all_extra_timeslots, $extra_timeslots);
	}
	
	$all_holiday_timeslots = array();
	// make the split by holidays
	foreach ($all_extra_timeslots as $ts_data) {
		$current_ts_extra_minutes = $ts_data['hours'] * 60 + $ts_data['minutes'];
			
		$holiday_timeslots = split_normal_and_holiday_hours($contact, $ts_data['date'], $current_ts_extra_minutes, $ts_data['additional_member_ids']);
			
		$all_holiday_timeslots = array_merge($all_holiday_timeslots, $holiday_timeslots);
	}
	
	
	$all_timeslots = $all_holiday_timeslots;
	
	// timeslots with no hour type must be classified in the normal hours member or days off member
	$normal_hours_member_id = config_option('normal_hours_member');
	
	$logged_user_tz_offset = logged_user()->getUserTimezoneValue();
	$timeslot_start_date_with_tz = new DateTimeValue($timeslot_start_date->getTimestamp());
	$timeslot_start_date_with_tz->add('s', $logged_user_tz_offset);
	
	
	// create timeslots for generated break hours
	$break_timeslots = create_break_hours_timeslots($contact, $timeslot_start_date, $total_timeslot_minutes, $ts_data['additional_member_ids'], $task, $all_timeslots);
	if (is_array($break_timeslots)) {
		$all_timeslots = array_merge($all_timeslots, $break_timeslots);
	}
	
	
	// Add default hour types for working hours timeslots --
	
	// if timeslot has no member in hour types, then assign the member defined as common_hours_member
	$hour_types_dim = Dimensions::findByCode('hour_types');
	$common_hours_member_id = config_option('common_hours_member');
	if ($common_hours_member_id > 0) {
		assign_default_hour_type_member($all_timeslots, $hour_types_dim, $common_hours_member_id, $task);
	}
	
	// if timeslot has no member in hour types night, then assign the member defined as day_hours_member
	$hour_types_dim = Dimensions::findByCode('hour_types_night');
	$day_hours_member_id = config_option('day_hours_member');
	if ($day_hours_member_id > 0) {
		assign_default_hour_type_member($all_timeslots, $hour_types_dim, $day_hours_member_id, $task);
	}
	
	// if timeslot has no member in hour types extra, then assign the member defined as normal_hours_member
	$hour_types_extra_dim = Dimensions::findByCode('hour_types_extra');
	$normal_hours_member_id = config_option('normal_hours_member');
	if ($normal_hours_member_id > 0) {
		assign_default_hour_type_member($all_timeslots, $hour_types_extra_dim, $normal_hours_member_id, $task);
	}
	
	// if timeslot has no member in hour types weapon, then assign the member defined as no_weapon_hours_member
	$hour_types_weapon_dim = Dimensions::findByCode('hour_types_weapon');
	$no_weapon_hours_member_id = config_option('no_weapon_hours_member');
	if ($no_weapon_hours_member_id > 0) {
		assign_default_hour_type_member($all_timeslots, $hour_types_weapon_dim, $no_weapon_hours_member_id, $task);
	}
	
	// --
	
	return $all_timeslots;
}


/**
 * Foreach $all_timeslots: if timeslot has any of the worked hours members then assign the default member id
 * of the other hour type dimensions if it doesn't have any
 * @param array $all_timeslots
 * @param Dimension $hour_types_dim
 * @param integer $default_hours_member_id
 * @param ProjectTask $task
 * @return array $all_timeslots
 */
function assign_default_hour_type_member(&$all_timeslots, $hour_types_dim, $default_hours_member_id, $task) {
	
	$worked_hour_type_members = config_option("worked_hour_type_members");
	$break_hours_member_id = config_option('break_hours_member');
	
	foreach ($all_timeslots as &$timeslot) {
		
		$member_ids = array_filter(array_var($timeslot, 'additional_member_ids'));
		$task_mem_ids = $task->getMemberIds();
		$member_ids = array_unique(array_merge($member_ids, $task_mem_ids));
		
		$mem_intersection = array_intersect($member_ids, $worked_hour_type_members);
		
		// if timeslot doesnt have any member of the worked hour types then dont assign anything
		if (count($mem_intersection) > 0 && !in_array($break_hours_member_id, $member_ids)) {
			$count = 0;
			if (count($member_ids) > 0) {
				
				$sql = "SELECT count(id) as num FROM ".TABLE_PREFIX."members
						WHERE dimension_id=".$hour_types_dim->getId()."
							AND id IN (".implode(',',$member_ids).")";
				$row = DB::executeOne($sql);
				$count = array_var($row, 'num');
			}
			if ($count == 0) {
				// if timeslot deoes not have any member in the dimension then assign the default member id
				$timeslot['additional_member_ids'][] = $default_hours_member_id;
			}
		}
	}
	
	return $all_timeslots;
}



/**
 * Splits the timeslot if part of it belongs to a holiday
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param int $current_timeslot_minutes
 * @param array $member_ids
 */
function split_normal_and_holiday_hours($contact, $timeslot_start_date, $current_timeslot_minutes, $member_ids=array()) {
	
	// user timezone offset
	$tz_diff = $contact->getUserTimezoneValue();
	
	// calculate the current timeslot end time
	$timeslot_end_date = new DateTimeValue($timeslot_start_date->getTimestamp());
	$timeslot_end_date->add('m', $current_timeslot_minutes);
	
	$timeslot_start_date->add('s', $tz_diff);
	$timeslot_end_date->add('s', $tz_diff);
	
	$is_start_nw_day = NonWorkingDays::instance()->isNonWorkingDay($timeslot_start_date, false);
	$is_end_nw_day = NonWorkingDays::instance()->isNonWorkingDay($timeslot_end_date, false);
	
	$common_hours_member_id = config_option('common_hours_member');
	$remove_from_mem_ids = array();
	
	if ($timeslot_start_date->getDay() == $timeslot_end_date->getDay() || $is_start_nw_day == $is_end_nw_day) {
		// when timeslot is only in one day or both days are of the same kind then only add the classification in holiday hour type member if it corresponds
		if ($is_start_nw_day) {
			$h_member_ids = array_merge($member_ids, array(config_option("holiday_hours_member")));
			$remove_from_mem_ids = array($common_hours_member_id);
		} else {
			$h_member_ids = $member_ids;
		}
		
		// return date to original value
		$timeslot_start_date->add('s', -1 * $tz_diff);
		
		$ts_data = build_timeslot_data($contact, $timeslot_start_date, $current_timeslot_minutes, $h_member_ids, $remove_from_mem_ids);
		
		return array($ts_data);
		
	} else {
		$all_timeslots = array();
		// if start or end is a non working day and the other is not then we have to split the timeslot
		if ($is_start_nw_day != $is_end_nw_day) {
			if ($is_start_nw_day) {
				$start_member_ids = array_merge($member_ids, array(config_option("holiday_hours_member")));
				$end_member_ids = $member_ids;
				$remove_from_mem_ids = array($common_hours_member_id);
			} else {
				$end_member_ids = array_merge($member_ids, array(config_option("holiday_hours_member")));
				$start_member_ids = $member_ids;
				$remove_from_mem_ids = array($common_hours_member_id);
			}
			
			// calculate the minutes until midnight
			$start_dt_tmp = new DateTimeValue($timeslot_start_date->getTimestamp());
			$start_dt_tmp->endOfDay(); //23:59:59
			$start_dt_tmp->add('s', 1);
			$start_ts_minutes = floor(($start_dt_tmp->getTimestamp() - $timeslot_start_date->getTimestamp())/60);
			
			if ($start_ts_minutes > 0) {
				// return date to original value
				$timeslot_start_date->add('s', -1 * $tz_diff);
			
				$ts_start_data = build_timeslot_data($contact, $timeslot_start_date, $start_ts_minutes, $start_member_ids, $remove_from_mem_ids);
				$all_timeslots[] = $ts_start_data;
			}
			
			// calculate the minutes past midnight
			$end_ts_minutes = $current_timeslot_minutes - $start_ts_minutes;
			if ($end_ts_minutes > 0) {
				// return date to original value
				$timeslot_end_date->add('s', -1 * $tz_diff);
				$end_dt_tmp = new DateTimeValue($timeslot_end_date->getTimestamp());
				$end_dt_tmp->add('m', -1 * $end_ts_minutes);
				
				$ts_end_data = build_timeslot_data($contact, $end_dt_tmp, $end_ts_minutes, $end_member_ids, $remove_from_mem_ids);
				$all_timeslots[] = $ts_end_data;
			}
		}
		return $all_timeslots;
	}
	
}


/**
 * Splits the worked hours timeslot into daily and night timeslots
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param integer $total_timeslot_minutes
 * @param array $member_ids
 */
function split_normal_and_night_hours(Contact $contact, $timeslot_start_date, $total_timeslot_minutes, $member_ids=array()) {
	
	// user timezone offset
	$tz_diff = $contact->getUserTimezoneValue();
	
	$original_ts_start = new DateTimeValue($timeslot_start_date->getTimestamp());
	$original_ts_start->add('s', $tz_diff);
	
	// calculate the current timeslot end time
	$timeslot_end_date = new DateTimeValue($timeslot_start_date->getTimestamp());
	$timeslot_end_date->add('m', $total_timeslot_minutes);
	
	// get night hours settings
	$night_hours_config = get_night_hours_config();
	
	$start_mins = $night_hours_config['start_hour']*60 + $night_hours_config['start_minute'];
	$end_mins = $night_hours_config['end_hour']*60 + $night_hours_config['end_minute'];

	// calculate night hours start and apply user timezone
	$night_start = new DateTimeValue($original_ts_start->getTimestamp());
	$night_start->setHour($night_hours_config['start_hour']);
	$night_start->setMinute($night_hours_config['start_minute']);
	//$night_start->add('s', -1 * $tz_diff);
	if ($night_start->getDay() > $timeslot_end_date->getDay()) {
		$night_start->add('d', -1);
	}
	
	// calculate night hours end and apply user timezone
	$night_end = new DateTimeValue($night_start->getTimestamp());
	$night_end->setHour($night_hours_config['end_hour']);
	$night_end->setMinute($night_hours_config['end_minute']);
	if ($night_end->getTimestamp() < $night_start->getTimestamp()) {
		$night_end->add('d', 1);
	}
	//$night_end->add('s', -1 * $tz_diff);
	
	$night_mins = 0;
	$normal_mins_after = 0;
	
	if ($timeslot_end_date->getTimestamp() > $night_start->getTimestamp()) {
		// normal minutes after night hours end
		if ($timeslot_end_date->getTimestamp() > $night_end->getTimestamp()) {
			$normal_mins_after = floor(($timeslot_end_date->getTimestamp() - $night_end->getTimestamp()) / 60);
		}
		$time_start = $timeslot_start_date->getTimestamp() > $night_start->getTimestamp() ? $timeslot_start_date : $night_start;
		// night minutes
		$night_mins = floor(($timeslot_end_date->getTimestamp() - $time_start->getTimestamp()) / 60) - $normal_mins_after;
	}
	// normal minutes
	$normal_mins = $total_timeslot_minutes - $night_mins - $normal_mins_after;
	
	$splitted_timeslots = array();
	
	// accumulate all night minutes registered in this day
	$accumulated_night_mins = $night_mins;
	$accumulated_night_hours_data = get_accumulated_night_hours_data($contact, $timeslot_start_date, $night_start, $night_end, $night_hours_config['hour_type_id']);
	
	$accumulated_night_mins += $accumulated_night_hours_data['total_night_minutes'];
	
	
	// check if total time worked accomplishes the minimum amount to classify in night hours.
	$min_hours_accomplished = true;
	$min_percentage_to_mark_night_hours = config_option('min_percentage_to_mark_night_hours');
	if ($min_percentage_to_mark_night_hours > 0) {
		// total worked minutes to check against the config option of min % of worked hours 
		$accumulated_total_mins = $total_timeslot_minutes;
		$accumulated_total_mins += $accumulated_night_hours_data['total_minutes'];
		
		// total work shift minutes
		$total_shift_minutes = get_work_shift_total_minutes($contact, $timeslot_start_date);
		
		// minimum amount of minutes to work to enable night hours classification
		$min_total_minutes_to_mark_night_hours = $total_shift_minutes * $min_percentage_to_mark_night_hours / 100;
		
		// total worked time must be greater than minimum total minutes to classify in night hours
		$min_hours_accomplished = $min_total_minutes_to_mark_night_hours <= $accumulated_total_mins;
	}
	
	
	// split only if the all timeslots of current day night hours are greater than the minimum required 
	// or if no minimum is defined
	$min_night_hours_exp = explode(":", config_option('min_night_hours'));
	$min_night_minutes = array_var($min_night_hours_exp, 0, 0) * 60 + array_var($min_night_hours_exp, 1, 0);
	
	if ( ($min_night_minutes == 0 || $night_mins > 0 && $accumulated_night_mins > $min_night_minutes) && $min_hours_accomplished) {
	
		// build timeslots data for normal hours
		if ($normal_mins > 0) {
			$ts_normal_data = build_timeslot_data($contact, $timeslot_start_date, $normal_mins, $member_ids);
		
			$splitted_timeslots[] = $ts_normal_data;
		}
		
		// build timeslots data for night hours
		if ($night_mins > 0) {
			$night_service_mem_id = config_option('night_service_member');
			$night_member_ids = array_merge($member_ids, array($night_hours_config['hour_type_id'], $night_service_mem_id));
			
			$ts_night_data = build_timeslot_data($contact, $time_start, $night_mins, $night_member_ids);
		
			$splitted_timeslots[] = $ts_night_data;
		}
		
		// build timeslots data for normal hours after the night hours
		if ($normal_mins_after > 0) {
			$ts_noraml_after_data = build_timeslot_data($contact, $night_end, $normal_mins_after, $member_ids);
		
			$splitted_timeslots[] = $ts_noraml_after_data;
		}
		
		if (count($accumulated_night_hours_data['timeslots_classified_as_normal']) > 0) {
			split_other_timeslots_to_night_hours($accumulated_night_hours_data['timeslots_classified_as_normal'], $night_hours_config['hour_type_id']);
		}
	
	} else {
		
		$ts_normal_data = build_timeslot_data($contact, $timeslot_start_date, $total_timeslot_minutes, $member_ids);
		$splitted_timeslots[] = $ts_normal_data;
		
	}
	
	return $splitted_timeslots;
	
}


/**
 * Splits a worked timeslot into normal and extra timeslots depending on the contact's shift duration and the total worked hours
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param integer $current_timeslot_minutes
 * @param integer $processed_normal_minutes
 * @param array $member_ids
 */
function split_normal_and_extra_hours($contact, $timeslot_start_date, $current_timeslot_minutes, $processed_normal_minutes, $member_ids=array()) {
	
	// total normal minutes are taken from the contact's shifts
	$max_normal_minutes = get_work_shift_total_minutes($contact, $timeslot_start_date);
	
	// calculate the remaining normal minutes checking previous entered timeslots
	$previous_worked_minutes = get_contact_worked_time($contact->getId(), $timeslot_start_date);
	
	// add the other splitted timeslots normal time to the previous worked minutes
	$previous_worked_minutes += $processed_normal_minutes;
	
	
	// discount the already worked minutes to get the max_normal_minutes remaining
	$max_normal_minutes = $max_normal_minutes - $previous_worked_minutes;
	if ($max_normal_minutes < 0) $max_normal_minutes = 0;
	
	// calculate normal and extra minutes using current_timeslot_minutes 
	$normal_minutes = $current_timeslot_minutes;
	$extra_minutes = 0;
	if ($current_timeslot_minutes > $max_normal_minutes) {
		$normal_minutes = $max_normal_minutes;
		$extra_minutes = $current_timeslot_minutes - $max_normal_minutes;
	}
	
	
	$splitted_timeslots = array();
	
	// build timeslots data for normal hours
	if ($normal_minutes > 0) {
		$ts_normal_data = build_timeslot_data($contact, $timeslot_start_date, $normal_minutes, $member_ids);
	
		$splitted_timeslots[] = $ts_normal_data;
	}
	
	// build timeslots data for extra hours
	if ($extra_minutes > 0) {
		$extra_member_ids = array_merge($member_ids, array(config_option('extra_hours_member')));
		$timeslot_start_date_extra = new DateTimeValue($timeslot_start_date->getTimestamp());
		$timeslot_start_date_extra->add('m', $normal_minutes);
		$ts_extra_data = build_timeslot_data($contact, $timeslot_start_date_extra, $extra_minutes, $extra_member_ids);
	
		$splitted_timeslots[] = $ts_extra_data;
	}
	
	return $splitted_timeslots;
}


/**
 * Builds an associative array with the timeslot information to pass to the controller and create it.
 * @param Contact $contact
 * @param DateTimeValue $date
 * @param integer $total_minutes
 * @param array $member_ids
 * @param array $remove_from_member_ids
 */
function build_timeslot_data($contact, $date, $total_minutes, $member_ids, $remove_from_member_ids = array()) {
	
	$ts_date = new DateTimeValue($date->getTimestamp());
	$minutes = $total_minutes % 60;
	$hours = floor($total_minutes / 60);
	
	$data = array (
			'hours' => $hours,
			'minutes' => $minutes,
			'date' => $ts_date,
			'start_time' => $ts_date->format('H:i'),
			'description' => '',
			'contact_id' => $contact->getId(),
			'additional_member_ids' => array_unique(array_filter($member_ids)),
	);
	if (count($remove_from_member_ids) > 0) {
		$data['ignore_member_ids'] = $remove_from_member_ids;
	}
	
	return $data;
}




/**
 * Returns the total amount of minutes of the contact's work shift
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 */
function get_work_shift_total_minutes($contact, $timeslot_start_date) {
	$work_shifts_data = ContactShifts::getContactShiftsData($contact);
	$max_normal_minutes = 0;
	
	if (count($work_shifts_data) > 0) {
		$work_shift_ids = array_keys($work_shifts_data);
		$work_shifts = WorkShifts::findAll(array("conditions" => "id IN (".implode(',', $work_shift_ids).")"));
		
		$logged_user_tz_offset = logged_user()->getUserTimezoneValue();
		$date_with_tz = new DateTimeValue($timeslot_start_date->getTimestamp());
		$date_with_tz->add('s', $logged_user_tz_offset);
		
		foreach ($work_shifts as $shift) {
			$shift_day = WorkShiftDays::getByShiftAndWeekday($shift->getId(), $date_with_tz->format('w'));
			if (!$shift_day instanceof WorkShiftDay) {
				continue;
			}
				
			$h_diff = $shift_day->getEndHour() - $shift_day->getStartHour();
			if ($shift_day->getStartHour() > $shift_day->getEndHour()) $h_diff += 24;
	
			$m_diff = $shift_day->getEndMinute() - $shift_day->getStartMinute();
			if ($shift_day->getStartMinute() > $shift_day->getEndMinute()) $m_diff += 60;
	
			$max_normal_minutes += $h_diff * 60 + $m_diff;
		}
	}
	return $max_normal_minutes;
}



/**
 * Returns the sql conditions to filter timestlots by worked hour types and ignoring break timeslots.
 * @param string $timeslot_table_alias
 */
function get_contact_worked_time_classification_sql($timeslot_table_alias = 't') {
	
	$worked_hour_type_members = config_option("worked_hour_type_members");
	$break_not_taken_hours_member_member_id = config_option('break_not_taken_hours_member');
	$break_hours_member_id = config_option('break_hours_member');
	
	// only count timeslots classified in worked hour type members
	$clasification_sql = "";
	if (count($worked_hour_type_members) > 0) {
		$clasification_sql = "
			AND EXISTS (
				SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
				WHERE om.object_id = $timeslot_table_alias.object_id 
					AND om.member_id IN (".implode(',', $worked_hour_type_members).")
			)
		";
	} else {
		$clasification_sql = " AND FALSE ";
	}
	
	// don't include break timeslots
	$possible_break_member_ids = array_filter(array($break_not_taken_hours_member_member_id, $break_hours_member_id));
	if (count($possible_break_member_ids) > 0) {
		$clasification_sql .= "
			AND NOT EXISTS (
				SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
				WHERE om.object_id = $timeslot_table_alias.object_id 
					AND om.member_id IN (".implode(',', $possible_break_member_ids).")
			)
		";
	}
	
	return $clasification_sql;
}

/**
 * Returns the value of the option 'time_between_workdays' in minutes
 */
function get_minutes_between_workdays() {
	// amount of hours between workdays, 6 by default
	$time_between_workdays = explode(":", config_option("time_between_workdays"));
	$tmp_hours = array_var($time_between_workdays, 0, 6);
	$tmp_mins = array_var($time_between_workdays, 1, 0);
	$minutes_between_workdays = $tmp_hours * 60 + $tmp_mins;
	
	return $minutes_between_workdays;
}

/**
 * Goes back in time to find the first timeslot date of the day, using the option 'time_between_workdays' also stops in $limit_start_date if no valid date is found
 * @param integer $contact_id
 * @param DateTimeValue $date
 * @param DateTimeValue $limit_start_date
 */
function find_start_time_for_contact_worked_time($contact_id, $date, $limit_start_date) {
	// amount of hours between workdays, 6 by default
	$minutes_between_workdays = get_minutes_between_workdays();
	
	// get sql conditions to filter by worked hour types and not in break hours.
	$clasification_sql = get_contact_worked_time_classification_sql();
	
	// go back in time until there is no timeslot in the last 6 hours or the day changes
	$continue = true;
	
	$logged_user_tz_offset = logged_user()->getUserTimezoneValue();
	$d = new DateTimeValue($date->getTimestamp());
	$d->add('s', -1 * $logged_user_tz_offset);
	
	$tmp_date1 = new DateTimeValue($d->getTimestamp());
	$tmp_date1->add('m', -1 * $minutes_between_workdays);
	$tmp_date2 = new DateTimeValue($d->getTimestamp());
	
	while ($continue) {
		$temp_sql = "SELECT t.object_id, t.start_time FROM `".TABLE_PREFIX."timeslots` t
			WHERE `contact_id` = ". $contact_id ."
				$clasification_sql
				AND `end_time` > ".DB::escape($tmp_date1->toMySQL())."
				AND `end_time` <= ".DB::escape($tmp_date2->toMySQL()).";";
	
		$r = DB::executeOne($temp_sql);

		$continue = $r != null && $r['object_id'] > 0;
		if ($continue) {
			$tmp_date2 = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $r['start_time']);
			$tmp_date1 = new DateTimeValue($tmp_date2->getTimestamp());
			$tmp_date1->add('m', -1 * $minutes_between_workdays);
				
			if ($tmp_date1->getTimestamp() < $limit_start_date->getTimestamp()) {
				$tmp_date1 = new DateTimeValue($limit_start_date->getTimestamp());
				$continue = false;
			}
		}
	}
	$d = new DateTimeValue($tmp_date1->getTimestamp());
	return $d;
}


/**
 * Returns the amount of worked time (in minutes) in the workday of the timeslot
 * @param integer $contact_id
 * @param DateTimeValue $date
 * @return integer
 */
function get_contact_worked_time($contact_id, DateTimeValue $date) {
	$limit_start_date = new DateTimeValue($date->getTimestamp());
	$limit_start_date->add('d', -1);
	
	// amount of hours between workdays, 6 by default
	//$minutes_between_workdays = get_minutes_between_workdays();
	
	// get sql conditions to filter by worked hour types and not in break hours.
	$clasification_sql = get_contact_worked_time_classification_sql();
		
	// start date to filter the query
	$d = find_start_time_for_contact_worked_time($contact_id, $date, $limit_start_date);
	
	// end date to filter the query
	$end_day_time = new DateTimeValue($date->getTimestamp());
	$end_day_time->endOfDay();
	
	
	// build sql
	$sql = "SELECT SUM(GREATEST(TIMESTAMPDIFF(MINUTE,start_time,end_time),0)) - SUM(subtract/60) as total
			FROM `".TABLE_PREFIX."timeslots` t
			WHERE `contact_id` = ". $contact_id ." 
					$clasification_sql
					AND `end_time` > ".DB::escape($d->toMySQL())." 
					AND `end_time` <= ".DB::escape($end_day_time->toMySQL()).";";
	$row = DB::executeOne($sql);
	
	return array_var($row, "total", 0);
}



/**
 * Calculates the amount of night time worked between two dates
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param DateTimeValue $night_start
 * @param DateTimeValue $night_end
 * @param integer $night_member_id
 */
function get_accumulated_night_hours_data($contact, $timeslot_start_date, $night_start, $night_end, $night_member_id) {
	$sd = new DateTimeValue($night_start->getTimestamp());
	$sd = $sd->beginningOfDay();
	
	// exclude timeslots of trashed tasks
	$trashed_task_cond = " AND IF (rel_object_id>0, 0=(SELECT oo.trashed_by_id FROM ".TABLE_PREFIX."objects oo WHERE oo.id=rel_object_id), true) ";
	
	$timeslots = Timeslots::findAll(array(
		"conditions" => "contact_id=". $contact->getId() ."
			AND start_time >= ".DB::escape($sd->toMySQL()) ."
			AND start_time < ".DB::escape($night_end->toMySQL()) ."
			$trashed_task_cond"
	));
	
	$timeslots_classified_as_normal = array();
	$total_night_minutes = 0;
	$total_minutes = 0;
	
	foreach ($timeslots as $ts) { /* @var $ts Timeslot */
		if ($ts->getEndTime()->getTimestamp() > $night_start->getTimestamp()) {
			// end date to calculate night time
			$end_date = $ts->getEndTime()->getTimestamp() > $night_end->getTimestamp() ? $night_end : $ts->getEndTime();
			$start_date = $ts->getStartTime()->getTimestamp() > $night_start->getTimestamp() ? $ts->getStartTime() : $night_start;
			
			// calculate night minutes of this timeslot
			$night_seconds = $end_date->getTimestamp() - $start_date->getTimestamp();
			$night_minutes = round($night_seconds / 60);
			$total_night_minutes += $night_minutes;
			
			// check if this timeslot is classified in night hours member
			$member_ids = $ts->getMemberIds();
			$classified_in_night_hours = in_array($night_member_id, $member_ids);
			
			if (!$classified_in_night_hours) {
				$timeslots_classified_as_normal[] = array(
					'timeslot' => $ts,
					'night_minutes' => $night_minutes,
				);
			}
		}
		$total_minutes += $ts->getColumnValue('worked_time');
	}
	
	return array(
		'total_minutes' => $total_minutes,
		'total_night_minutes' => $total_night_minutes,
		'timeslots_classified_as_normal' => $timeslots_classified_as_normal,
	);
}


/**
 * Returns the total worked time between two dates, also fixes the start date if necesary (using the option time_between_workdays)
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param DateTimeValue $timeslot_end_date
 * @param string $extra_conditions
 */
function get_accumulated_total_hours($contact, $timeslot_start_date, $timeslot_end_date, $extra_conditions="") {
	$limit_sd = new DateTimeValue($timeslot_end_date->getTimestamp());
	$limit_sd = $limit_sd->add('d', -1);
	
	$sd = find_start_time_for_contact_worked_time($contact->getId(), $timeslot_start_date, $limit_sd);
	$ed = new DateTimeValue($timeslot_end_date->getTimestamp());
	
	// exclude timeslots of trashed tasks
	$trashed_task_cond = " AND IF (rel_object_id>0, 0=(SELECT oo.trashed_by_id FROM ".TABLE_PREFIX."objects oo WHERE oo.id=rel_object_id), true) ";
	
	// only include timeslots classified in worked hour types
	$worked_hour_types_cond = get_contact_worked_time_classification_sql('e');
	
	$timeslots = Timeslots::findAll(array(
		"conditions" => "contact_id=". $contact->getId() ."
			AND start_time >= ".DB::escape($sd->toMySQL()) ."
			AND start_time < ".DB::escape($ed->toMySQL()) ."
			$worked_hour_types_cond
			$trashed_task_cond
			$extra_conditions"
	));
	
	$total_minutes = 0;
	foreach ($timeslots as $ts) {
		$total_minutes += $ts->getColumnValue('worked_time');
	}
	return $total_minutes;
}


/**
 * Splits the remaining worked hours timeslot (after the first split) into daily and night timeslots
 * @param array $timeslots_to_split
 * @param integer $night_member_id
 */
function split_other_timeslots_to_night_hours($timeslots_to_split, $night_member_id) {
	$day_hour_member_id =  config_option('day_hours_member');
	
	foreach($timeslots_to_split as $ts_to_split) { 
		/* @var $ts Timeslot */
		/* @var $new_ts Timeslot */
		
		$ts = $ts_to_split['timeslot'];
		$night_minutes = $ts_to_split['night_minutes'];
		
		// create new timeslot
		$new_ts = $ts->copy(false);
		
		// remove night minutes from original timeslot
		$orig_ts_new_end_date = new DateTimeValue($ts->getEndTime()->getTimestamp());
		$orig_ts_new_end_date->add('m', -1 * $night_minutes);
		$ts->setEndTime($orig_ts_new_end_date);
		$ts->save();
		
		// remove normal minutes from new timeslot
		$new_ts_new_start_date = new DateTimeValue($new_ts->getEndTime()->getTimestamp());
		$new_ts_new_start_date->add('m', -1 * $night_minutes);
		$new_ts->setStartTime($new_ts_new_start_date);
		$new_ts->save();
		
		// remove normal hour type member from new timeslot and add the night hours hour type member
		$member_ids = $ts->getMemberIds();
		$new_member_ids = array($night_member_id);
		foreach ($member_ids as $mid) {
			if ($mid != $day_hour_member_id) {
				$new_member_ids[] = $mid;
			}
		}
		$new_member_ids = array_filter($new_member_ids);
		if (count($new_member_ids) > 0) {
			$members_array = Members::findAll(array("conditions" => "id IN (".implode(',', $new_member_ids).")"));
			$new_ts->addToMembers($members_array);
		}
		$new_ts->addToSharingTable();
		

		// if original timeslot has no minutes then delete it
		if ($ts->getEndTime()->getTimestamp() <= $ts->getStartTime()->getTimestamp()) {
			$ts->delete();
		}
	}
}


/**
 * Generates the break timeslots using the current worked timeslot start date and the total timeslot minutes
 * @param Contact $contact
 * @param DateTimeValue $timeslot_start_date
 * @param integer $total_timeslot_minutes
 * @param array $original_member_ids
 * @param ProjectTask $task
 * @param array $all_timeslots
 */
function create_break_hours_timeslots($contact, $timeslot_start_date, $total_timeslot_minutes, $original_member_ids, $task, $all_timeslots) {
	
	$generated_timeslots = array();
	
	$contact_shift = ContactShifts::findOne(array("conditions" => "contact_id=".$contact->getId()));
	if (!$contact_shift instanceof ContactShift) return;
	
	$work_shift = WorkShifts::findById($contact_shift->getShiftId());
	if (!$work_shift instanceof WorkShift) return;
	
	$min_minutes_to_generate = $work_shift->getBreakMinMinutes();
	$generate_every_x_minutes = $work_shift->getBreakEveryXMinutes();
	$break_minutes = $work_shift->getBreakMinutes();
	
	/* @var $timeslot_start_date DateTimeValue */
	$timeslot_end_date = new DateTimeValue($timeslot_start_date->getTimestamp());
	$timeslot_end_date = $timeslot_end_date->add('m', $total_timeslot_minutes);
	
	$worked_minutes = $total_timeslot_minutes;
	$worked_minutes += get_accumulated_total_hours($contact, $timeslot_start_date, $timeslot_end_date);
	
	// calculate the number of break timeslots to create
	$break_timeslots_to_generate = 0;
	
	if ($worked_minutes >= $min_minutes_to_generate && $generate_every_x_minutes > 0) {
		// remoeve first amout of minutes to generate break
		$worked_minutes -= $generate_every_x_minutes;
		$break_timeslots_to_generate++;
		
		// the next ones are generated every $generate_every_x_minutes hours 
		while ($worked_minutes > 0 && $generate_every_x_minutes) {
			$worked_minutes -= $generate_every_x_minutes;
			if ($worked_minutes > 0) {
				$break_timeslots_to_generate++;
			}
		}
	}
	
	// break member ids
	$break_not_taken_hours_member_member_id = config_option('break_not_taken_hours_member');
	$break_hours_member_id = config_option('break_hours_member');
	$possible_break_member_ids = array($break_not_taken_hours_member_member_id, $break_hours_member_id);	
	
	// check if user has taken any breaks
	$break_cond_sql = " AND EXISTS (SELECT om.member_id FROM ".TABLE_PREFIX."object_members om WHERE om.object_id=o.id AND om.member_id='$break_hours_member_id')";
	$already_generated_break_minutes += get_accumulated_total_hours($contact, $timeslot_start_date, $timeslot_end_date, $break_cond_sql);
	
	// remove already break taken minutes from the total amount to generate
	while ($break_minutes > 0 && $already_generated_break_minutes > 0) {
		$already_generated_break_minutes -= $break_minutes;
		$break_timeslots_to_generate--;
	}
	
	if ($break_minutes > 0 && $break_timeslots_to_generate > 0) {
		$first = true;
		$ts_date = new DateTimeValue($timeslot_start_date->getTimestamp());
		
		$option_key = null;
		$cp = CustomProperties::getCustomPropertyByCode($task->getObjectTypeId(), 'takes_break');
		if ($cp instanceof CustomProperty) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), $cp->getId());
			if ($cpv instanceof CustomPropertyValue) {
				$option_key = $cpv->getValue() > 0 ? 'break_hours_member' : 'break_not_taken_hours_member';
			}
		}
		if (is_null($option_key)) {
			$option_key = $contact_shift->getTakesBreak() ? 'break_hours_member' : 'break_not_taken_hours_member';
		}
		
		$ht_dim_id = Dimensions::findByCode('hour_types')->getId();
		$st_dim_id = Dimensions::findByCode('service_types')->getId();
		$htn_dim_id = Dimensions::findByCode('hour_types_night')->getId();
		$hte_dim_id = Dimensions::findByCode('hour_types_extra')->getId();
		$htw_dim_id = Dimensions::findByCode('hour_types_weapon')->getId();
		
		$timesots_classification_dimension_ids = array($ht_dim_id, $st_dim_id, $htn_dim_id, $hte_dim_id, $htw_dim_id);
		
		// if takes break then remove all service type and hour type members else only remove hour types members
		$remove_from_member_ids = array();
		$member_ids = array();
		foreach ($original_member_ids as $mid) {
			$member_ids[] = $mid;
		}
		
		$task_members = $task->getMembers();
		foreach ($task_members as $mem) {
			if ($option_key == 'break_hours_member') {
				$mem_is_in_any_timesots_dimension = in_array($mem->getDimensionId(), $timesots_classification_dimension_ids);
				if ($mem_is_in_any_timesots_dimension) {
					$remove_from_member_ids[] = $mem->getId();
				}
			}
		}
		
		$member_ids[] = config_option($option_key);
		
		// build timeslots data
		while ($break_timeslots_to_generate > 0) {
			
			$d = new DateTimeValue($ts_date->getTimestamp());
			
			// calculate how many hours to advance for next timeslot
			if ($first && $min_minutes_to_generate > 0) {
				$h_amount = $min_minutes_to_generate / 60;
			} else {
				$h_amount = $generate_every_x_minutes / 60;
			}
			
			$ts_date->add('h', $h_amount); // advance date for next timeslot
			$d->add('h', $h_amount / 2); // put the break timeslot in the middle
			
			$break_member_ids = array();
			
			// If it is a worked break, copy all the members from the corresponding worked timeslot and add the break member
			if ($option_key == 'break_not_taken_hours_member') {
				
				// get the timeslot origin (the one from the break has been created)
				$ts_origin = null;
				foreach ($all_timeslots as $timeslot) {
					$ts_ed = new DateTimeValue($timeslot['date']->getTimestamp());
					$ts_ed->add('h', $timeslot['hours']);
					$ts_ed->add('m', $timeslot['minutes']);
					$d_ed = new DateTimeValue($d->getTimestamp());
					$d_ed->add('m', $break_minutes);
					
					if ($timeslot['date']->getTimestamp() <= $d->getTimestamp() && $ts_ed->getTimestamp() >= $d_ed->getTimestamp()) {
						$ts_origin = $timeslot;
						break;
					}
				}
				// copy timeslot origin's members
				if ($ts_origin) {
					$break_member_ids = $timeslot['additional_member_ids'];
				}
				// add break not taken member
				$break_member_ids[] = $break_not_taken_hours_member_member_id;
					
			} else if ($option_key == 'break_hours_member') {
				// only classify the timeslot in the break hours member, don't classify in the other hour types
				foreach ($member_ids as $mid) {
					if (in_array($mid, $possible_break_member_ids)) {
						$break_member_ids[] = $mid;
					}
				}
			}
			
			$ts_data = build_timeslot_data($contact, $d, $break_minutes, $break_member_ids, $remove_from_member_ids);
			$generated_timeslots[] = $ts_data;
			
			
			$first = false;
			$break_timeslots_to_generate--;
		}
	}
	
	return $generated_timeslots;
}





function check_and_classify_task_in_holiday_service_member(ProjectTask $new_task) {
	
	if (Plugins::instance()->isActivePlugin('crpm')) {
		$is_non_workin_start = false;
		$is_non_workin_due = false;
		
		$st = $new_task->getStartDate();
		if ($st instanceof DateTimeValue) {
			$is_non_workin_start = NonWorkingDays::instance()->isNonWorkingDay($st);
		}
		
		$dt = $new_task->getDueDate();
		if ($dt instanceof DateTimeValue) {
			$is_non_workin_due = NonWorkingDays::instance()->isNonWorkingDay($dt);
		}
		
		$holiday_service_member_id = config_option('holiday_service_member');
		if ($holiday_service_member_id > 0) {
			if ($is_non_workin_start || $is_non_workin_due) {
				$m = Members::getMemberById($holiday_service_member_id);
				if ($m instanceof Member) {
					$new_task->addToMembers(array($m));
				}
			} else {
				ObjectMembers::removeObjectFromMembers($new_task, logged_user(), null, array($holiday_service_member_id));
			}
		}
	}
}





function get_unavailable_users_to_assign($parameters, $user_ids) {
	$task_id = array_var($parameters, 'task_id');

	$time_format = (user_config_option('time_format_use_24') ? 'G:i' : 'g:i A');
	$datetime_format = user_config_option('date_format') . " $time_format";

	// parse start date
	$start_date = null;
	if (array_var($parameters, 'start_date')) {
		$start_date = getDateValue(array_var($parameters, 'start_date'));
		$s_time = getTimeValue(array_var($parameters, 'start_time'));
		$start_date->add('h', $s_time['hours']);
		$start_date->add('m', $s_time['mins']);
	}

	// parse due date
	$due_date = null;
	if (array_var($parameters, 'due_date')) {
		$due_date = getDateValue(array_var($parameters, 'due_date'));
		$d_time = getTimeValue(array_var($parameters, 'due_time'));
		$due_date->add('h', $d_time['hours']);
		$due_date->add('m', $d_time['mins']);
	}

	// if duration is specified and one of the dates is missing then calculate the missing date
	$duration_mins = array_var($parameters, 'estimated_mins');
	if ($duration_mins) {
		if ($due_date instanceof DateTimeValue && !$start_date) {
			$start_date = new DateTimeValue($due_date->getTimestamp());
			$start_date = $start_date->add('m', -1 * $duration_mins);

		} else if ($start_date instanceof DateTimeValue && !$due_date) {
			$due_date = new DateTimeValue($start_date->getTimestamp());
			$due_date = $due_date->add('m', $duration_mins);
		}
	}

	// if we have both dates then check if the users are available
	if ($due_date instanceof DateTimeValue && $start_date instanceof DateTimeValue) {
		$logged_user_tz_offset = logged_user()->getUserTimezoneValue();
			
		// put dates in GMT 0 and build date strings
		$start_date_str = '';
		$due_date_str = '';
		if ($start_date) {
			$start_date->add('s', -1 * $logged_user_tz_offset);
			$start_date_str = $start_date->toMysql();
		}
		if ($due_date) {
			$due_date->add('s', -1 * $logged_user_tz_offset);
			$due_date_str = $due_date->toMysql();
		}
			
		// query to check non available users between requiered start and due datetimes
		if ($start_date_str != '' && $due_date_str != '') {
			$task_id_sql = is_numeric($task_id) && $task_id > 0 ? " AND t.object_id != '$task_id'" : "";
			$sql = "
					SELECT DISTINCT(assigned_to_contact_id) as user_id
					FROM ".TABLE_PREFIX."project_tasks t
					INNER JOIN ".TABLE_PREFIX."objects o ON t.object_id=o.id
					WHERE o.trashed_by_id=0 $task_id_sql
					AND assigned_to_contact_id IN (". implode(',', $user_ids) .")
					AND (
					due_date > '$start_date_str' AND start_date < '$start_date_str'
					OR due_date > '$due_date_str' AND start_date < '$due_date_str'
					);
					";

			// execute query
			$rows = DB::executeAll($sql);

			return $rows;
		}
			
	}
}