<?php
Hook::register("advanced_services");


function advanced_services_total_task_timeslots_group_by_criterias($args, &$ret) {
	$wdimension = Dimensions::findByCode ( 'service_types' );
	$tdimension = Dimensions::findByCode ( 'hour_types' );
	
	$enabled_dimension_ids = config_option('enabled_dimensions');
	
	if (in_array($wdimension->getId(), $enabled_dimension_ids)) {
		$ret[] = array('val' => 'dim_'.$wdimension->getId(), 'name' => $wdimension->getName());
	}
	
	if (in_array($tdimension->getId(), $enabled_dimension_ids)) {
		$ret[] = array('val' => 'dim_'.$tdimension->getId(), 'name' => $tdimension->getName());
	}
}


function advanced_services_custom_reports_additional_columns($args, &$ret) {
	// the dim columns are added by this plugin too
	if (Plugins::instance()->isActivePlugin('advanced_reports')) return;

	$ot = array_var($args, 'object_type');
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('dimension_object','dimension_group'))) return;

	$dimensions = Dimensions::findAll ( array("conditions" => "code IN ('service_types','hour_types')") );
	foreach ($dimensions as $dimension) {
		if (in_array($dimension->getId(), config_option('enabled_dimensions'))) {
				
			$name = $dimension->getName();
				
			$ret[] =  array('id' => 'dim_'.$dimension->getId(), 'name' => $name, 'type' => DATA_TYPE_STRING);
		}
	}
}


function advanced_services_after_task_complete($args, &$ret) {
	$task = array_var($args, 'task');
	if (!$task instanceof ProjectTask) return;
	
	if (!$task->getAssignedTo() instanceof Contact) return;
	
	$logged_user_tz_offset = logged_user()->getUserTimezoneValue();
	
	$ot_id = ProjectTasks::instance()->getObjectTypeId();
	$cp = CustomProperties::getCustomPropertyByCode($ot_id, "add_worked_hours_on_complete");
	
	if ($cp instanceof CustomProperty) {
		$cp_val = CustomPropertyValues::getCustomPropertyValue($task->getId(), $cp->getId());
		if ($cp_val instanceof CustomPropertyValue && $cp_val->getValue()) {
			
			$start_date = $task->getStartDate();
			$due_date = $task->getDueDate();
			$duration_min = null;
			
			if ($start_date instanceof DateTimeValue && $due_date instanceof DateTimeValue) {
				$duration_min = floor(($due_date->getTimestamp() - $start_date->getTimestamp()) / 60);
			} else if ($task->getTimeEstimate() > 0) {
				$duration_min = $task->getTimeEstimate();
			}
			
			if (!is_null($duration_min)) {
				$minutes_to_add = $duration_min - $task->getTotalWorkedTime();
				if ($minutes_to_add > 0) {
					$time_controller = new TimeController();
					
					if ($start_date instanceof DateTimeValue) {
						$t_date = new DateTimeValue($start_date->getTimestamp());
					} else if ($due_date instanceof DateTimeValue) {
						$t_date = new DateTimeValue($due_date->getTimestamp());
						$t_date->add('m', $duration_min * -1);
					} else {
						$t_date = DateTimeValueLib::now();
					}
					
					$t_date->add('s', $logged_user_tz_offset);
					
					$hours = floor($minutes_to_add / 60);
					$minutes = floor($minutes_to_add % 60);
					
					// orginal calculated timeslot data
					$original_ts_data = array (
						'hours' => $hours,
						'minutes' => $minutes,
						'date' => $t_date,
						'start_time' => $t_date->format('H:i'),
						'description' => '',
						'contact_id' => $task->getAssignedToContactId(),
					);
					
					// split original timeslot calculating which hours are normal, extra, etc.
					Env::useHelper('functions', 'advanced_services');
					$generated_timeslots_data = split_timeslots_by_hour_type($task, $original_ts_data);
					
					foreach ($generated_timeslots_data as $timeslot_data) {
						$parameters = array();
						$parameters['object_id'] = $task->getId();
						$parameters['use_current_time'] = false;
						$parameters['timeslot'] = $timeslot_data;
						
						$time_controller->add_timeslot($parameters, false);
					}
				}
			}
		}
	}
}




// if new cloned task date is in a non-working day then classify it in the holiday service member of service types dimension
function advanced_services_after_task_cloned($params, &$new_task) {

	if ($new_task instanceof ProjectTask) {
		Env::useHelper('functions', 'advanced_services');
		check_and_classify_task_in_holiday_service_member($new_task);
	}
}


// if new cloned task date is in a non-working day then classify it in the holiday service member of service types dimension
function advanced_services_after_task_repetition_edited($params, &$task) {

	if ($task instanceof ProjectTask) {
		Env::useHelper('functions', 'advanced_services');
		check_and_classify_task_in_holiday_service_member($task);
	}
}



function advanced_services_grouped_timeslots_report_preset_config($preset_parameters, &$config) {

	$end_date = DateTimeValueLib::now();
	$start_date = DateTimeValueLib::now();
	$start_date->add('M', -1);
	
	$crpm_dim = Dimensions::findByCode('customer_project');
	$st_dim = Dimensions::findByCode('service_types');
	$ht_dim = Dimensions::findByCode('hour_types');
	
	switch ($preset_parameters) {
		
		case 'billing_and_cost_by_client':
			$config = array(
				'report_name' => lang('billing and cost by client report'),
				'report_desc' => lang('billing and cost by client report description'),
				'date_type' => 6,
				'start_value' => format_date($start_date),
				'end_value' => format_date($end_date),
				'show_details' => 0,
				'group_by_type' => array(
					'dim', 'member_ids', '0'
				),
				'group_by' => array(
					array('dim_'.$crpm_dim->getId()),
					array('dim_'.$st_dim->getId(), 'dim_'.$ht_dim->getId()),
				),
				'columns' => array(
					'name' => 1,
					'fixed_cost' => 2,
					'fixed_billing' => 3,
				),
			);
			break;
			
		case 'worked_time_by_person_and_hour_type':
			$config = array(
				'report_name' => lang('worked time by person and hour type report'),
				'report_desc' => lang('worked time by person and hour type report description'),
				'date_type' => 6,
				'start_value' => format_date($start_date),
				'end_value' => format_date($end_date),
				'show_details' => 1,
				'group_by_type' => array(
					'contact_id', 'member_ids', '0'
				),
				'group_by' => array(
					'',
					array('dim_'.$st_dim->getId(), 'dim_'.$ht_dim->getId()),
				),
				'columns' => array(
					'name' => 1,
					'start_time' => 2,
					'end_time' => 3,
					'worked_time' => 4,
				),
			);
			break;
			
		default :
			break;
	}
	
	return $config;
}







function advanced_services_before_save_custom_property_value($params, &$value) {

	$object = array_var($params, 'object');
	$cp = array_var($params, 'custom_prop');
	
	if ($object instanceof Timeslot && $cp->getCode() == 'workday' && $value == '') {
		$workday = get_timeslot_workday_value($object);
		$value = format_date($workday, 'Y-m-d', 0);
	}
}


function advanced_services_after_object_save($object, &$ret) {
	
	if ($object instanceof Timeslot) {
		$cp = CustomProperties::getCustomPropertyByCode($object->getObjectTypeId(), 'workday');
		$workday = get_timeslot_workday_value($object);

		if (!is_null($workday)) {
			$workday_str = $workday->format('Y-m-d');
			
			$cpval = CustomPropertyValues::getCustomPropertyValue($object->getId(), $cp->getId());
			if (!$cpval instanceof CustomPropertyValue) {
				$cpval = new CustomPropertyValue();
				$cpval->setObjectId($object->getId());
				$cpval->setCustomPropertyId($cp->getId());
			}
			$cpval->setValue($workday_str);
			$cpval->save();
		}
		
	} else if ($object instanceof ProjectTask) {
		
		$cp = CustomProperties::getCustomPropertyByCode(Timeslots::instance()->getObjectTypeId(), 'workday');
		$timeslots = $object->getTimeslots();
		
		if (is_array($timeslots)) {
			foreach ($timeslots as $timeslot) {
				
				$workday = get_timeslot_workday_value($timeslot, $object);
				
				if (!is_null($workday)) {
					$workday_str = $workday->format('Y-m-d');
						
					$cpval = CustomPropertyValues::getCustomPropertyValue($timeslot->getId(), $cp->getId());
					if (!$cpval instanceof CustomPropertyValue) {
						$cpval = new CustomPropertyValue();
						$cpval->setObjectId($timeslot->getId());
						$cpval->setCustomPropertyId($cp->getId());
					}
					$cpval->setValue($workday_str);
					$cpval->save();
				}
			}
		}
	}
}


function get_timeslot_workday_value(Timeslot $object, $task = null) {
	$workday = null;
	$tz_offset = null;
	
	if ($object->getRelObjectId() > 0) {
		if (is_null($task)) {
			$task = $object->getRelObject();
		}
		if ($task instanceof ProjectTask) {
			$tz_offset = $task->getTimezoneValue();
			if ($task->getStartDate() instanceof DateTimeValue) {
				$workday = new DateTimeValue($task->getStartDate()->getTimestamp());
			}
			
			if (is_null($workday)) {
				if ($task->getDueDate() instanceof DateTimeValue) {
					$workday = new DateTimeValue($task->getDueDate()->getTimestamp());
				}
			}
		}
	}
	if (is_null($workday)) {
		$workday = new DateTimeValue($object->getStartTime()->getTimestamp());
		$tz_offset = $object->getTimezoneValue();
	}
	if ($workday instanceof DateTimeValue) {
		$workday->add('s', $tz_offset);
	}
	
	return $workday;
}

