<?php

//use Sabre\VObject\Property\Time;

function generate_absence_timeslot($args) {
    
    $absence = array_var($args, 'absence');
    if (!$absence instanceof Absence) return;
    
    $ot_absence = ObjectTypes::findByName('absence');
    $absence_ot_id = $ot_absence->getId();
    $total_hours_per_day = null;
    $start_date = null;
    $due_date = null;
    $contact_id = null;
    $description = null;
    
    $hours_per_day_cp = CustomProperties::instance()->getCustomPropertyByCode($absence_ot_id, 'total_hours_per_day');
    if ($hours_per_day_cp instanceof CustomProperty){
        $hpd_cpv = CustomPropertyValues::getCustomPropertyValue($absence->getId(), $hours_per_day_cp->getId());
        if ($hpd_cpv instanceof CustomPropertyValue){
            $total_hours_per_day = $hpd_cpv->getValue();
        }
    }
    
    $sd_cp = CustomProperties::instance()->getCustomPropertyByCode($absence_ot_id, 'start_date');
    if ($sd_cp instanceof CustomProperty){
        $sd_cpv = CustomPropertyValues::getCustomPropertyValue($absence->getId(), $sd_cp->getId());
        if ($sd_cpv instanceof CustomPropertyValue){
            $start_date = DateTimeValueLib::makeFromString($sd_cpv->getValue());
        }        
    }
    
    $dd_cp = CustomProperties::instance()->getCustomPropertyByCode($absence_ot_id, 'due_date');
    if ($dd_cp instanceof CustomProperty){
        $dd_cpv = CustomPropertyValues::getCustomPropertyValue($absence->getId(), $dd_cp->getId());
        if ($dd_cpv instanceof CustomPropertyValue){
            $due_date = DateTimeValueLib::makeFromString($dd_cpv->getValue());
        }
    }
    
    $cid_cp = CustomProperties::instance()->getCustomPropertyByCode($absence_ot_id, 'contact_id');
    if ($cid_cp instanceof CustomProperty){
        $cid_cpv = CustomPropertyValues::getCustomPropertyValue($absence->getId(), $cid_cp->getId());
        if ($cid_cpv instanceof CustomPropertyValue){
            $contact_id = $cid_cpv->getValue();
        }
    }
    
    $desc_cp = CustomProperties::instance()->getCustomPropertyByCode($absence_ot_id, 'description');
    if ($desc_cp instanceof CustomProperty){
        $desc_cpv = CustomPropertyValues::getCustomPropertyValue($absence->getId(), $desc_cp->getId());
        if ($desc_cpv instanceof CustomPropertyValue){
            $description = $desc_cpv->getValue();
        }
    }
    
   
    // retornar expeciones en caso de que no existan estas CP, manejar en controlador
    
    if (!$total_hours_per_day) return;
    
    if (!$contact_id || !Contacts::instance()->findById($contact_id) instanceof Contact) return;
    
    //chequear si es mismo dia (teniendo en cuenta TZ del objeto no en GMT 0)
    if (!$start_date || !$start_date instanceof DateTimeValue || !$due_date || !$due_date instanceof DateTimeValue) {
        return;
    }
    
    $time_controller = new TimeController();    
    
    $current_date = $start_date;
    $current_date_aux = New DateTimeValue($start_date->getTimestamp());
    
    $calculated_due_time_obj = new DatetimeValue($current_date->getTimestamp());
    $calculated_due_time_obj->add('h', $total_hours_per_day);
    
    $due_date_aux = New DateTimeValue($due_date->getTimestamp());
    
    $first_loop_iteration = true;
    
    try {    
        //create one Timeslot per day between start_date and due_date
        while( $current_date_aux->beginningOfDay()->getTimestamp() <= $due_date_aux->beginningOfDay()->getTimestamp()){
            
            if ($start_date == EMPTY_DATETIME) {
                $start_date = DateTimeValueLib::now();
            }
            
            if ($due_date == EMPTY_DATETIME) {
                $due_date = DateTimeValueLib::now();
            }
            
            if ($first_loop_iteration) {
	            $shift_start_time = $start_date->format("H:i");
            	$first_loop_iteration = false;
            } else {
            	$shift_start_time = '09:00';//TODO: turno o sistema
            }
            
            $date_aux = new DateTimeValue($current_date->getTimestamp());
            $date_aux->setHour($calculated_due_time_obj->getHour());
        
            $shift_due_time = $date_aux->format('H:i');
            
            //calculate begin of the current and due days for compare to check if it the last timeslot
            $begin_of_current_day = new DateTimeValue($current_date->getTimestamp());
            $begin_of_current_day = $begin_of_current_day->beginningOfDay();
            
            $begin_of_due_day = new DateTimeValue($due_date->getTimestamp());
            $begin_of_due_day = $begin_of_due_day->beginningOfDay();
            
            //check due time if its the last timeslot
            if ($begin_of_current_day->getTimestamp() == $begin_of_due_day->getTimestamp()){


                $calculated_due_time_same_day = new DateTimeValue($calculated_due_time_obj->getTimestamp());
                $calculated_due_time_same_day->setDay($current_date->getDay());
                
                //if the end_time setted in the form is lower than the end_time of calculated, set the end time setted
                if ( $due_date->getTimestamp() < $calculated_due_time_same_day->getTimestamp()){
                    $date_aux = new DateTimeValue($current_date->getTimestamp());
                    $date_aux->setHour($due_date->getHour());
                    $date_aux->setMinute($due_date->getMinute());
                    $date_aux->setSecond(0);
                    $shift_due_time = $date_aux->format('H:i');                    
                }              
            }
            
            // orginal calculated timeslot data
            $timeslot_data = array (
                'date' => $current_date,
                'start_time' => $shift_start_time,
                'end_date' => $date_aux,
                'end_time' => $shift_due_time,
                'description' => '',
                'contact_id' => $contact_id,
                'specify_end_time' => 1
            );
            
            $parameters = array();
            $parameters['object_id'] = $absence->getId();
            $parameters['use_current_time'] = false;
            $parameters['timeslot'] = $timeslot_data;
            
            
            $time_controller->add_timeslot($parameters, false);
            
            $current_date->add('d', 1);
            $current_date_aux->add('d', 1);
        }
        
    } catch(Exception $e) {
    	throw $e;
        //flash_error($e->getMessage());
    } // try
    
}

function build_timeslot_data($contact, $date, $total_minutes, $member_ids, $remove_from_member_ids = array()) {
    
    $ts_date = new DateTimeValue($date->getTimestamp());
    $minutes = $total_minutes % 60;
    $hours = floor($total_minutes / 60);
    
    $data = array (
        'hours' => $hours,
        'minutes' => $minutes,
        'date' => $ts_date,
        'description' => '',
        'contact_id' => $contact->getId(),
        'additional_member_ids' => array_unique(array_filter($member_ids)),
    );
    if (count($remove_from_member_ids) > 0) {
        $data['ignore_member_ids'] = $remove_from_member_ids;
    }
    
    return $data;
}
?>