<?php
Hook::register('billing_and_cost_permissions');

function billing_and_cost_permissions_object_definition($type, &$ret) {
	if ($type == 'SystemPermission' || $type == 'MaxSystemPermission') {
		$ret['can_see_billing_information'] = DATA_TYPE_BOOLEAN;
	}
	if ($type == 'CustomProperty' || $type == 'MemberCustomProperty') {
		$ret['information_type'] = DATA_TYPE_STRING;
	}
	if ($type == 'SystemPermission' || $type == 'MaxSystemPermission') {
		$ret['can_see_cost_information'] = DATA_TYPE_BOOLEAN;
	}
	/*
	* Commented out to use later
	if ($type == 'Report') {
		$ret['has_billing_info'] = DATA_TYPE_BOOLEAN;
	}
	*/
}

/********
 * 
 * BILLING INFO FUNCTIONS
 */
function billing_and_cost_permissions_get_can_see_billing_information($params, &$result){
	$result = SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information');
}

function billing_and_cost_permissions_add_custom_property_condition($params, &$conditions){
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$conditions .= " AND information_type NOT IN ('billing', 'billing_and_cost') ";
	}
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information')) {
		$conditions .= " AND information_type NOT IN ('cost', 'billing_and_cost') ";
	}
}

function billing_and_cost_permissions_add_custom_property_arguments($params, &$arguments){
	$extra_conditions = '';
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$extra_conditions .= " AND information_type NOT IN ('billing', 'billing_and_cost') ";
	}
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information')) {
		$extra_conditions .= " AND information_type NOT IN ('cost', 'billing_and_cost') ";
	}
	if($extra_conditions != ''){
		if(array_var($arguments, 'conditions')){
			if (is_array($arguments['conditions'])){
				$arguments['conditions'][0] .= $extra_conditions;
			} else {
				$arguments['conditions'] .= $extra_conditions;
			}
		} else if ($arguments == null) {
			$arguments = array('conditions' => $extra_conditions);
		} else {
			$arguments['conditions'] = $extra_conditions;
		}
	}
}

function billing_and_cost_permissions_check_custom_property_info_type($params, &$res){
	$cp = $params['cp'];
	$information_type = $cp instanceof CustomProperty || $cp instanceof MemberCustomProperty ? $cp->getColumnValue('information_type') : '';
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$billing_array = array('billing', 'billing_and_cost');
		if(in_array($information_type, $billing_array)){
			$res = false;
		}
	}
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information')) {
		$cost_array = array('cost', 'billing_and_cost');
		if(in_array($information_type, $cost_array)){
			$res = false;
		}
	}
}

function billing_and_cost_permissions_custom_properties_form_modify_headers($params, &$headers) {
	$last = array_pop($headers);
	$headers[] = "information_type";
	$headers[] = $last;
}

function billing_and_cost_permissions_additional_custom_property_fields($params, &$prop) {
	$cp = array_var($params, 'cp');
	$prop['information_type'] = $cp['information_type'];
}

function billing_and_cost_permissions_custom_properties_form_additional_columns($params, &$additional_columns) {

	$options = array('standard', 'billing', 'cost', 'billing_and_cost');
	$options_html = array();
	foreach($options as $opt){
		$options_html[] = option_tag(lang($opt), $opt, $attr);
	}
	$html = select_box("custom_properties[{number}][information_type]", $options_html, array("id" => "information_type"));

	$additional_columns[] = array('html' => $html);
	
}

/*
Commented out to use later

function billing_and_cost_permissions_additional_report_data_for_view($params, &$report_data){
	$report = $params['report'];
	$report_data['has_billing_info'] = $report->getColumnValue('has_billing_info');
}

function billing_and_cost_permissions_set_additional_report_values($params, &$report){
	$report_data = $params['report_data'];
	$report->setColumnValue('has_billing_info', array_var($report_data, 'has_billing_info') == 'checked');
}

function billing_and_cost_permissions_has_permissions_to_see_report($params, &$show_report){
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$report = $params['report'];
		if($report->getColumnValue('has_billing_info')){
			$show_report = false;
		}
	}
}
*/

function billing_and_cost_permissions_remove_billing_and_cost_columns_from_list($params, &$allowed_columns){
	$object_type = $params['object_type'];
	$task_ot = ObjectTypes::findByName('task')->getId();
	$time_ot = ObjectTypes::findByName('timeslot')->getId();
	$expense_item_ot = ObjectTypes::findByName('expense_item')->getId();
	$payment_receipt_ot = ObjectTypes::findByName('payment_receipt')->getId();

	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$columns_to_remove = array();
		if ($object_type == $task_ot ){
			$columns_to_remove = array('bill_total_price', 'bill_billing_rate', 'bill_adv_billing_id', 'bill_profit_margin', 'bill_profit_margin_percentage', 'earned_value', 'estimated_price');
		} elseif ($object_type == $time_ot ){
			$columns_to_remove = array('billing_category_status', 'adv_billing_id', 'hourly_billing', 'fixed_billing',  'bill_total_price', 'bill_billing_rate', 'bill_adv_billing_id', 'bill_profit_margin', 'bill_profit_margin_percentage');
		} elseif ($object_type == $expense_item_ot ){
			$columns_to_remove = array('price_multiplier', 'total_price', 'unit_price');
		} elseif ($object_type == $payment_receipt_ot ){
			$columns_to_remove = array('unit_price', 'total_price', 'price_multiplier');
		}
		if(!empty($columns_to_remove)){
			foreach($allowed_columns as $key => $column){
				if(in_array($column['id'], $columns_to_remove)){
					unset($allowed_columns[$key]);
				}
			}
		}
	}

	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information')) {
		$columns_to_remove = array();
		if ($object_type == $task_ot ){
			$columns_to_remove = array('bill_total_cost', 'bill_cost_rate', 'bill_adv_cost_id', 'estimated_cost', 'executed_cost');
		} elseif ($object_type == $time_ot ){
			$columns_to_remove = array('hourly_cost', 'fixed_cost', 'bill_total_cost', 'bill_cost_rate', 'bill_adv_cost_id');
		} elseif ($object_type == $expense_item_ot ){
			$columns_to_remove = array('total_cost', 'unit_cost');
		} elseif ($object_type == $payment_receipt_ot ){
			$columns_to_remove = array('amount', 'unit_cost');
		}
		if(!empty($columns_to_remove)){
			foreach($allowed_columns as $key => $column){
				if(in_array($column['id'], $columns_to_remove)){
					unset($allowed_columns[$key]);
				}
			}
		}
	}

}

function billing_and_cost_permissions_remove_billing_columns_from_view($params, &$view_columns){
	$object_type = $params['object_type']->getId();
	$task_ot = ObjectTypes::findByName('task')->getId();
	$time_ot = ObjectTypes::findByName('timeslot')->getId();
	$expense_item_ot = ObjectTypes::findByName('expense_item')->getId();
	$payment_receipt_ot = ObjectTypes::findByName('payment_receipt')->getId();

	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_billing_information')) {
		$columns_to_remove = array();
		if($object_type == $task_ot ){
			$columns_to_remove = array('bill_total_price', 'bill_billing_rate', 'bill_adv_billing_id', 'bill_profit_margin', 'bill_profit_margin_percentage');
		} elseif($object_type == $time_ot ){
			$columns_to_remove = array('billing_category_status', 'adv_billing_id', 'hourly_billing', 'fixed_billing',  'bill_total_price', 'bill_billing_rate', 'bill_adv_billing_id', 'bill_profit_margin', 'bill_profit_margin_percentage');
		} elseif($object_type == $expense_item_ot ){
			$columns_to_remove = array('price_multiplier', 'total_price', 'unit_price');
		} elseif ($object_type == $payment_receipt_ot ){
			$columns_to_remove = array('unit_price', 'total_price', 'price_multiplier');
		}
		if(!empty($columns_to_remove)){
			foreach($view_columns as $key => $column){
				if($column instanceof ReportColumn){
					if(in_array($column->getFieldName(), $columns_to_remove)){
						unset($view_columns[$key]);
					}
				}
			}
		}
	}

	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information')) {
		$columns_to_remove = array();
		if ($object_type == $task_ot ){
			$columns_to_remove = array('bill_total_cost', 'bill_cost_rate', 'bill_adv_cost_id');
		} elseif ($object_type == $time_ot ){
			$columns_to_remove = array('hourly_cost', 'fixed_cost', 'bill_total_cost', 'bill_cost_rate', 'bill_adv_cost_id');
		} elseif ($object_type == $expense_item_ot ){
			$columns_to_remove = array('total_cost', 'unit_cost');
		} elseif ($object_type == $payment_receipt_ot ){
			$columns_to_remove = array('amount', 'unit_cost');
		}
		if(!empty($columns_to_remove)){
			foreach($view_columns as $key => $column){
				if($column instanceof ReportColumn){
					if(in_array($column->getFieldName(), $columns_to_remove)){
						unset($view_columns[$key]);
					}
				}
			}
		}
	}

}


/***********
 * 
 * COST INFO FUNCTIONS
 */

function billing_and_cost_permissions_get_can_see_cost_information($params, &$result){
	$result = SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_cost_information');
}