<?php

/**
 * Member class
 *
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TaskAfterActionAction extends BaseTaskAfterActionAction {


	function execute($task, $group_id) {
		if (!$task instanceof ContentDataObject) return false;
		
		switch ($this->getAction()) {
			case "instantiate":
				$this->instantiateTasks($task);
				break;
			case "change_assoc_dim":
				$this->changeAssociatedDimensionMember($task);
				break;
			case "classify":
				$this->classify($task);
				break;
			default:
				return;
		}
		
		// generate execution log, to prevent multiple execution of this action for the same task
		$log = new TaskAfterActionLog();
		$log->setTaskId($task->getId());
		$log->setGroupId($group_id);
		$log->setAction($this->getAction());
		$log->setLogData($this->getActionData());
		$log->setCreatedById(logged_user()->getId());
		$log->setCreatedOn(DateTimeValueLib::now());
		$log->save();
	}
	
	private function getInstantiatedTemplateParameters(ProjectTask $task) {
		$instantiation_id = $task->getInstantiationId();
		$template_id = $task->getFromTemplateId();
		
		$instantiated_values = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."template_instantiated_parameters WHERE template_id='$template_id' AND instantiation_id='$instantiation_id'");
		$values = array();
		foreach ($instantiated_values as $ivalue) {
			$values[$ivalue['parameter_name']] = $ivalue['value'];
		}
		
		return $values;
	}
	
	private function instantiateTasks($task) {
		
		$task_ids = explode(',', $this->getActionData());
		$parameterValues = $this->getInstantiatedTemplateParameters($task);
		$tasks_info = array();
		
		if (count($task_ids) > 0) {
			// if tasks to create have subtasks then create them too, use parents_path column to create all levels.
			foreach ($task_ids as $tid) {
				$sql = "SELECT object_id FROM ".TABLE_PREFIX."template_tasks 
						WHERE (parents_path like '$tid' OR parents_path like '%,$tid' OR parents_path like '$tid,%' OR parents_path like '%,$tid,%')";
				$subtask_ids = array_flat(DB::executeAll($sql));
				$task_ids = array_filter(array_unique(array_merge($task_ids, $subtask_ids)));
			}
		}
		
		// create copies
		$need_find_parent = array();
		foreach ($task_ids as $id) {
			if (!is_numeric($id)) continue;
			
			$task_to_copy = Objects::findObject($id);
			
			if ($task_to_copy instanceof TemplateTask) {
				$copy = $task_to_copy->copyToProjectTask($task->getInstantiationId());
				$need_find_parent[] = $copy;
			} elseif($task_to_copy instanceof ProjectTask) {
				$copy = $task_to_copy->copy();
			}else{
			    return;
            }
			
			// create workflow conditions
			$cond_sql = "INSERT INTO ".TABLE_PREFIX."task_after_action_conditions (task_id,group_id,condition_field,condition_comparison,condition_value) 
					SELECT ".$copy->getId().", x.group_id, x.condition_field, x.condition_comparison, x.condition_value 
					FROM ".TABLE_PREFIX."task_after_action_conditions x  WHERE x.task_id=".$task_to_copy->getId()."  
					ON DUPLICATE KEY UPDATE group_id=x.group_id;";
			DB::execute($cond_sql);
			
			// create workflow actions
			$cond_sql = "INSERT INTO ".TABLE_PREFIX."task_after_action_actions (task_id,group_id,action,action_data)
					SELECT ".$copy->getId().", x.group_id, x.action, x.action_data
					FROM ".TABLE_PREFIX."task_after_action_actions x  WHERE x.task_id=".$task_to_copy->getId()."
					ON DUPLICATE KEY UPDATE group_id=x.group_id;";
			DB::execute($cond_sql);
			
			// create dependency $task -> $copy
			$dep_sql = "INSERT INTO ".TABLE_PREFIX."project_task_dependencies (previous_task_id, task_id, created_on, created_by_id) 
					VALUES (".$task->getId().", ".$copy->getId().", NOW(), ".(logged_user() instanceof Contact ? logged_user()->getId() : '0').");";
			DB::execute($dep_sql);
			

			$template = COTemplates::findById($task_to_copy->getTemplateId());
			if ($template instanceof COTemplate) {
				$ret = null;
				Hook::fire('after_template_object_instantiation', array('template' => $template, 'original' => $task_to_copy, 'copy' => $copy), $ret);
			}
			
			// copy linked objects
			$original_linked_objects = $task->getLinkedObjects();
			foreach ($original_linked_objects as $lo) {
				$copy->linkObject($lo);
			}
			
			// copy only the members defined in the config option
			$dimensions_to_copy = config_option('workflow_dims_to_copy_next_task');
			$original_members = $task->getMembers();
			
			$members = array();
			foreach ($original_members as $m) {
				if (in_array($m->getDimensionId(), $dimensions_to_copy)) {
					$members[] = $m;
				}
			}
			if (count($members) > 0) {
				$copy->addToMembers($members);
			}
			$copy->addToSharingTable();
			
			instantiate_template_task_parameters($task_to_copy, $copy, $parameterValues);
			
			// collect task data to add to view
			$tasks_info[] = $copy->getArrayInfo();
		}
		
		// recalculate parents for tasks created from template tasks
		foreach ($need_find_parent as $t) {
			if($t->getParentId() > 0){
				foreach ($need_find_parent as $c) {
					if($c instanceof ProjectTask){
						if($c->getFromTemplateObjectId() == $t->getParentId()){
							$t->setParentId($c->getId());
							$t->save();
						}
					}
				}
			}
		}
		
		if (count($tasks_info) > 0) {
			// add new tasks to view 
			evt_add("add tasks info to tasks list", array('tasks' => $tasks_info));
		}
		
	}
	
	
	private function changeAssociatedDimensionMember(ProjectTask $task) {
		$action_data = json_decode($this->getActionData(), true);
		
		$association_id = null;
		$new_member_id = null;
		foreach ($action_data as $assoc_id => $assoc_member_id) {
			if (!is_numeric($assoc_member_id)) {
				$assoc_member_id = json_decode($assoc_member_id, true);
				if (is_array($assoc_member_id) && count($assoc_member_id)>0) {
					$assoc_member_id = $assoc_member_id[0];
				}
			}
			$new_member_id = $assoc_member_id;
			$association_id = $assoc_id;
		}
		
		$association = DimensionMemberAssociations::findById($association_id);
		if (!$association instanceof DimensionMemberAssociation) return;
		
		$keep_old_value = (bool)DimensionAssociationsConfigs::getConfigValue($association_id, 'keeps_old_value');
		
		$task_members = $task->getMembers();
		
		foreach ($task_members as $tmem) {
			
			if ($tmem->getDimensionId() == $association->getDimensionId() && $tmem->getObjectTypeId() == $association->getObjectTypeId()) {
				// get old associated member
				$old_assoc_member_id = 0;
				$old_assoc_member = MemberPropertyMembers::findOne(array("conditions" => "association_id=$association_id AND member_id=".$tmem->getId()));
				if ($old_assoc_member instanceof MemberPropertyMember) {
					$old_assoc_member_id = $old_assoc_member->getPropertyMemberId();
				}
				
				// association info
				$associated_dimension_id = $association->getAssociatedDimensionMemberAssociationId();
				$associated_ot_id = $association->getAssociatedObjectType();
				$associated_dimension = Dimensions::findById($associated_dimension_id);
				$associated_ot = ObjectTypes::findById($associated_ot_id);
				
				// remove old association if needed
				if (!$keep_old_value) {
					MemberPropertyMembers::delete("association_id=$association_id AND member_id=".$tmem->getId());
				}
				
				// asociate member to new member 
				if (is_numeric($new_member_id) && $new_member_id > 0) {
					// collect previous data
					$previous_data = $tmem->getDataForHistory();
					
					associate_member_to_status_member($tmem, $old_assoc_member_id, $new_member_id, $associated_dimension, $associated_ot, !$keep_old_value, $association->getCode());
					
					// save member history
					MemberHistories::instance()->saveMemberHistory($tmem, $previous_data, false);
				}
				
				// fire events after changing association
				$null = null;
				Hook::fire('after_member_association_changed', array('member' => $tmem), $null);
			}
		}
		
	}
	
	
	
	
	private function classify($task) {
	
		$member_ids = explode(',', $this->getActionData());
		foreach ($member_ids as $k => &$mid) {
			if (!is_numeric($mid)) unset($member_ids[$k]);
		}
		
		if (count($member_ids) > 0) {
			$members = Members::findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
			if (is_array($members) && count($members) > 0) {
				$task->addToMembers($members);
				$task->addToSharingTable();
			}
		}
		
	}
	
}
