<?php 
class NonWorkingDaysController extends ApplicationController {
	
	var $plugin_name = "crpm";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	}
		
	function index() {

		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		$non_working_days = NonWorkingDays::findAll(array('order' => 'year, month, day'));
		
		tpl_assign('non_working_days', $non_working_days);
	}
	
	function add() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		$data = array_var($_POST, 'non_working_day');
		if (!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('nw_day', new NonWorkingDay());
			
		} else {
			try {
				DB::beginWork();
				$nw_day = new NonWorkingDay();
		
				$this->save_non_working_day($data, $nw_day, true);
				 
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("reload");
				}
		
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		$nw_day = NonWorkingDays::findById(get_id());
		if (!$nw_day instanceof NonWorkingDay) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		$data = array_var($_POST, 'non_working_day');
		if (!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('nw_day', $nw_day);
			
		} else {
			try {
				DB::beginWork();
		
				$this->save_non_working_day($data, $nw_day, false);
					
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("reload");
				}
		
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
		}
	}
	
	function delete() {
		ajx_current("empty");
		$obj = NonWorkingDays::findById(get_id());
		if ($obj instanceof NonWorkingDay) {
			try {
				DB::beginWork();
				
				$obj->delete();
				flash_success(lang('success delete non working day'));
					
				ajx_current("reload");
				
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
			
		} else {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
	}
	
	private function save_non_working_day($data, $nw_day, $is_new) {
		$nw_day->setName(array_var($data, 'name'));
		$nw_day->setDay(array_var($data, 'day'));
		$nw_day->setMonth(array_var($data, 'month'));
		$nw_day->setYear(array_var($data, 'year'));
		$nw_day->save();
		 
		if ($is_new) {
			flash_success(lang('success add non working day'));
		} else {
			flash_success(lang('success edit non working day'));
		}
	}
	
}