<?php

/**
 * Customer controller
 *
 * @version 1.0
 * @author Marcos Saiz <marcos.saiz@fengoffice.com>
 */
class ProjectController extends ApplicationController{

	var $plugin_name = "crpm";
	var $dimension = null ;
	var $dimension_code = "customer_project";
	var $base_conditions = array( "conditions" => "trashed_by_id = 0 AND archived_by_id = 0 " )  ;
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		$this->dimension = Dimensions::instance()->findOne(array("conditions"=>" code = 'customer_project' " ));
		$this->addHelper('chart');
		Env::useHelper('project_additional_data', 'crpm');
	} 
	
	
	function get_dimension_info() {
		$info = array(
			'dimension_id' => Dimensions::findByCode($this->dimension_code)->getId(),
			'object_type_id' => Projects::instance()->getObjectTypeId(),
		);
		ajx_current("empty");
		ajx_extra_data(array('dim_info' => $info));
	}
	

	
	
	function add() { 
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		$project = new Project();
		$notAllowedMember = '';
		if(!$project->canAdd(logged_user(), active_context(), $notAllowedMember  )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else trim($notAllowedMember) == "" ? flash_error(lang('you must select where to keep', lang('the project'))) : flash_error(lang('no context permissions to add', lang("projects"), $notAllowedMember));
			ajx_current("empty");
			return;
		}
		

		$data = array_var($_POST, 'project');
		
		if(!is_array($data)) {
			$project_data = array('status' => 6);
			
			$cust_id = get_id('customer');
			$project_data = array();
			if ( $name = array_var($_REQUEST, 'name') ) {
				$project_data['name'] = $name ;
			}
			$project_data['automatic_status'] = 1;
			if ($parent = array_var($_REQUEST, 'parent') ){
				tpl_assign("parent_member", $parent);
			}else{
				tpl_assign("parent_member", 0);
			}
			if (isset($_REQUEST['type'])) {
				tpl_assign('obj_type_sel', $_GET['type']);
			}
			tpl_assign("current_dimension", $this->dimension);
			$ret = array();
			Hook::fire('check_additional_member_permissions', array('action' => 'add', 'parent_member_id' => $parent, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
			if (count($ret) > 0 && !array_var($ret, 'ok')) {
				flash_error(array_var($ret, 'message'));
				ajx_current("empty");
				return;
			}
			
			$this->populateComboboxes();
			tpl_assign('project_data', $project_data);
			
			$member = new Member();
			$member->setObjectTypeId($project->getObjectTypeId());
			$member->setDimensionId($this->dimension->getId());
			tpl_assign("member", $member);
			
			// Permissions
			$permission_parameters = permission_member_form_parameters(null, $this->dimension->getId());
			$logged_user_pg = array();
			foreach ($permission_parameters['allowed_object_types'] as $ot){
				$logged_user_pg[] = array('o' => $ot->getId(), 'w' => 1, 'd' => can_manage_dimension_members(logged_user()) ? 1 : 0, 'r' => 1);
			}
			$permission_parameters['member_permissions'][logged_user()->getPermissionGroupId()] = $logged_user_pg;
			
			$permission_parameters = get_default_member_permission($parent,$permission_parameters);
			
			tpl_assign('permission_parameters', $permission_parameters);
			
			Hook::fire('before_add_member', array('member'=>$member, 'parent'=>$parent), $ret);
		}else{
			
			$this->handleSubmit($data, $project);
			
		}
		
		tpl_assign("project", $project);
	}
	
	
	
	function edit() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$project = Projects::findById(get_id());
		if (!$project instanceof Project) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$project->canEdit(logged_user())) {
			flash_error(lang('no context permissions to add', lang("projects")));
			ajx_current("empty");
			return;
		}
		
		$data = array_var($_POST, 'project');
		
		if(!is_array($data)) {
			$project_data = $project->getArrayInfo();
			$this->populateComboboxes();
			tpl_assign('project_data', $project_data);
		}else{
			$action = ApplicationLogs::ACTION_EDIT;
			$this->handleSubmit($data, $project, $action);
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
		
		tpl_assign("project", $project);
		$project_member = Members::findOneByObjectId($project->getId(), Dimensions::instance()->findByCode("customer_project")->getId());
		tpl_assign('member', $project_member);
		
		$ret = array();
		Hook::fire('check_additional_member_permissions', array('action' => 'edit', 'member' => $project_member, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
		if (count($ret) > 0 && !array_var($ret, 'ok')) {
			flash_error(array_var($ret, 'message'));
			ajx_current("empty");
			return;
		}
		
		$permission_parameters = permission_member_form_parameters($project_member);
		tpl_assign('permission_parameters', $permission_parameters);
		
		if ($parent = $project_member->getParentMemberId() ){
			tpl_assign("parent_member", $parent);
		}else{
			tpl_assign("parent_member", 0);
		}
		$this->setTemplate('add');
	}
	
	
	
	function handleSubmit($data, Project $project, $action = null) {
		if($action == null){
			$action = ApplicationLogs::ACTION_ADD;
		}
		$isNew = (boolean) $project->isNew();
		$old_parent = -1;
		if (!$isNew) {
			$old_member = Members::findOneByObjectId($project->getId(), $this->dimension->getId());
			$old_parent = $old_member->getParentMemberId();
		}
		
		// Check if some custom properties are duplicated
		$project_custom_properties = $_REQUEST['object_custom_properties'];
		$result = array(
			'is_unique' => true,
			'error_message' => ''
		);
		Hook::fire('check_unique_custom_properties', array('custom_properties' => $project_custom_properties, 'project' => $project, 'is_new'=> $isNew) , $result);
		if(!$result['is_unique']){
			flash_error($result['error_message']);
			ajx_current('empty');
			return;
		}

		$object_types_with_prefix = config_option("use_autonumeric_prefixes");
		if (in_array($project->getObjectTypeId(), $object_types_with_prefix) && !$isNew) {
			$prefix_obj = get_prefix_for_object($project);
			if ($prefix_obj instanceof ObjectPrefix) {
				$data['name'] = $prefix_obj->getPrefix() . OBJECT_PREFIX_SEPARATOR . $data['name'];
			}
		}
		
		
		try {
			DB::beginWork();
			$member_controller = new MemberController();
			
			$project->setUseOnDuplicateKeyWhenInsert(true);
			$project->getObject()->setUseOnDuplicateKeyWhenInsert(true);
			
			$parent_member_id = array_var($data, 'parent');
			if (!$parent_member_id) {
				$parent_members = json_decode(array_var($_REQUEST, 'members', ''), true);
				$parent_member_id = array_var($parent_members, 0);
			}
			
			$member_data = array(
				'name' => trim(array_var($data, 'name')),
				'dimension_id' => $this->dimension->getId(),
				'object_type_id' => Projects::instance()->getObjectTypeId(),
				'parent_member_id' => $parent_member_id,
				'color' => array_var($data, 'color'),
				'template_id' => array_var($data, 'template_id'),
				'from_mail_id' => array_var($data, 'from_mail_id'),
				'depth' => 1
			);
			
			if(config_option("wysiwyg_projects")){
				$data['type_content'] = "html";
				$desc = preg_replace("/[\n|\r|\n\r]/", '', array_var($data, 'description'));
			}else{
				$desc = array_var($data, 'description');
				$data['type_content'] = "text";
			}
			$member_data['description'] = $desc;
			$dim_obj_data = array(
				'description' => $desc,
				'status' => array_var($data, 'status'),
				'automatic_status' => array_var($data, 'automatic_status'),
				'type_content' => array_var($data, 'type_content'),
				'manager_id' => array_var($data, 'manager_id'),
			);

			if (!Plugins::instance()->isActivePlugin('member_custom_properties')) {
				$dim_obj_data['start_date'] = getDateValue(array_var($data, 'start_date'));
				$dim_obj_data['end_date'] = getDateValue(array_var($data, 'end_date'));
			}
			
			$_POST['dim_obj'] = $dim_obj_data;
			
			if ($project->isNew()) {
				$member = new Member();
			} else {
				$member = Members::findOneByObjectId($project->getId(), $this->dimension->getId());
			}
			
			// save member
			$member = $member_controller->saveMember($member_data, $member, $project->isNew());
			
			// refresh project object
			$project->setId($member->getObjectId());
			$project->setNew(false);
			
			// save associated members
			save_associated_dimension_members(array('member' => $member, 'request' => $_REQUEST, 'is_new' => $isNew));
			
			Hook::fire("after_project_save", array('project_data' => $data, 'member' => $member), $project);
			
			$project->setObjectName($member->getName());
				
			$status_dimension = Dimensions::findByCode("project_statuses");
			$status_ot = ObjectTypes::findByName("project_status");
			$use_project_status_dimension = $status_dimension instanceof Dimension && in_array($status_dimension->getId(), config_option('enabled_dimensions'));
			if ($use_project_status_dimension) {
				$project->setColumnValue('status', get_associated_status_member_id($member, $status_dimension, $status_ot, false, 'project_project_status'));
				$project->save();
			}
			
			
			if ($isNew) {
				$ret = null;
				Hook::fire('after_add_member', $member, $ret);
			}
			
			if ($action == ApplicationLogs::ACTION_EDIT || $action == ApplicationLogs::ACTION_ADD) {
				$select_node = false;
				if ($action == ApplicationLogs::ACTION_ADD) {
					$select_node = intval(DimensionObjectTypeOptions::getOptionValue($member->getDimensionId(), $member->getObjectTypeId(), 'select_after_creation'));
				}
				if ($select_node) {
					evt_add("external dimension member click", array('dim_id' => $member->getDimensionId(),'member_id' => $member->getId()));
				} else {
					evt_add("update dimension tree node", array('dim_id' => $member->getDimensionId(), 'member_id' => $member->getId(), 'select_node' => $select_node));
				}
			}else{
				evt_add("external dimension member click", array('dim_id' => $member->getDimensionId(),'member_id' => $member->getId()));
			}
			
			//calculate_projects_additional_data(array('project_id' => $project->getId()));
			
			DB::commit();
			// Save permissions
			Env::useHelper('permissions');
			save_member_permissions_background(logged_user(), $member, array_var($_REQUEST, 'permissions'), $old_parent);

			ApplicationLogs::createLog($member, $action);
			
			return $member;
			
		}catch (Exception $e) {
			DB::rollback();
			Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}
	
	
	/**
	 * @deprecated
	 */
	function delete() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		$project = Projects::findById(get_id());
		if (!$project instanceof Project) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if(!$project->canDelete(logged_user())) {
			flash_error(lang('no context permissions to delete', lang("projects")));
			ajx_current("empty");
			return;
		}

		$members = Members::findByObjectId($project->getId(), $this->dimension->getId());
		if (count($members) > 0) $member = $members[0];
		
		if ($member instanceof Member) {
			$this->redirectTo('member', 'delete', array('id' => $member->getId()));
		} else {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
	}
	
	
	function view() {
		
		ajx_current("empty");
		return;
		
		$id = get_id();
		if (!$id){
			if ( $memberId = array_var($_GET, "memberId")){
				$id = Members::instance()->findById($memberId)->getObjectId(); 			
			}else{
				flash_error(lang("not found"));
			}
		}

		ajx_set_no_back(true);
		
		$project = Projects::instance()->findById($id);
		/* @var $project Project */
		$context = active_context();
		
		//$tasks_res = ProjectTasks::getContentObjects($context, ObjectTypes::findById(ProjectTasks::instance()->getObjectTypeId()), 'due_date', 'ASC');  // FIXME: performance Killer
		//$tasks_res = ProjectTasks::findByContext();

		$task_assignment_conditions = "";
		if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
			$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
		}
		
		$tasks_res = ProjectTasks::instance()->listing(array(
			"order" => "due_date",
			"order_dir" => "ASC",
			"extra_conditions" => " AND is_template = 0 $task_assignment_conditions"
		));
		
		//FIXME: Last line duration: 3-5s
		$tasks = $tasks_res->objects;
		
		//$milestones_res = ProjectMilestones::getContentObjects($context, ObjectTypes::findById(ProjectMilestones::instance()->getObjectTypeId()), 'due_date', 'ASC');
		$milestones_res = ProjectMilestones::instance()->listing(array(
			"order" => "due_date",
			"order_dir" => "ASC",
			"extra_conditions" => " AND is_template = 0"
		));
		
		$milestones = $milestones_res->objects;
		
		$all_has_due_date = true;
		$all_completed_has_work = true;
		$all_has_estimated_time = true;

		$user_condition = "";
		if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
			$user_condition = " AND contact_id = " . logged_user()->getId();
		}
		
		foreach ($tasks as $task) {
			/* @var ProjectTask $task */
			if ($all_has_due_date && !$task->getDueDate() instanceof DateTimeValue) $all_has_due_date = false;
			
			if ($all_completed_has_work && $task->isCompleted()) {
				//$ts_res = Timeslots::getContentObjects($context, ObjectTypes::findById(Timeslots::instance()->getObjectTypeId()), null, null, "AND `rel_object_id`=".$task->getId() );
				$ts_res = Timeslots::instance()->listing(array(
					"extra_conditions" => " AND rel_object_id =".$task->getId() . $user_condition
				));
				if ($ts_res->total == 0) $all_completed_has_work = false;
			}
			
			if ($all_has_estimated_time && $task->getTimeEstimate() == 0) $all_has_estimated_time = false;
			
		}
		
		if ($all_has_due_date) {
			$hours = $this->sumTasksHoursByDueDate($tasks);
		} else {
			$hours = $this->sumTasksHoursByType($tasks);
		}
		
		
		$overdue_upcoming_objects = ProjectTasks::getOverdueAndUpcomingObjects(20);
		
		// build statistics
		$milestone_stats = array('upcoming' => 0, 'late' => 0, 'complete' => 0, 'today' => 0, 'total' => $milestones_res->total);
		$task_stats = array('upcoming' => 0, 'late' => 0, 'complete' => 0, 'today' => 0, 'total' => $tasks_res->total);
		$today = DateTimeValueLib::now();
		$today->beginningOfDay();
		foreach ($tasks as $obj) {
			if ($obj->isCompleted()) {
				$task_stats['complete'] = $task_stats['complete'] + 1;
			} else {
				if ($obj->getDueDate() instanceof DateTimeValue && $obj->getDueDate()->isToday()) {
					$task_stats['today'] = $task_stats['today'] + 1;
				} else if ($obj->getDueDate() instanceof DateTimeValue && $obj->getDueDate()->getTimestamp() < $today->getTimestamp()) { 
					$task_stats['late'] = $task_stats['late'] + 1;
				} else {
					$task_stats['upcoming'] = $task_stats['upcoming'] + 1;
				}
			}
		}
		
		foreach ($milestones as $obj) {
			if ($obj->isCompleted()) {
				$milestone_stats['complete'] = $milestone_stats['complete'] + 1;
			} else {
				if ($obj->getDueDate() instanceof DateTimeValue && $obj->getDueDate()->isToday()) {
					$milestone_stats['today'] = $milestone_stats['today'] + 1;
				} else if ($obj->getDueDate() instanceof DateTimeValue && $obj->getDueDate()->getTimestamp() < $today->getTimestamp()) { 
					$milestone_stats['late'] = $milestone_stats['late'] + 1;
				} else {
					$milestone_stats['upcoming'] = $milestone_stats['upcoming'] + 1;
				}
			}
		}
		
		if (Plugins::instance()->isActivePlugin('gantt')) {
			$task_assignment_conditions = "";
			if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
				$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
			}
			$gantt_tasks_res = ProjectTasks::instance()->listing(array(
				"order" => "start_date",
				"order_dir" => "ASC",
				"extra_conditions" => "$task_assignment_conditions",
			));
						
			$gantt_tasks = array();
			foreach ($gantt_tasks_res->objects as $gt) {
				$gantt_tasks[$gt->getId()] = $gt;
			}
			
			$dependencies = array();
			if (count($gantt_tasks) > 0) {
				$ids_csv = implode(",", array_keys($gantt_tasks));
				$dependencies_res = ProjectTaskDependencies::findAll(array('conditions' => "previous_task_id IN ($ids_csv) || task_id IN ($ids_csv)"));
				// Solo se toma en cuenta 1 tarea previa
				foreach ($dependencies_res as $dep) {
					if(!isset($dependencies[$dep->getTaskId()])) $dependencies[$dep->getTaskId()] = $dep->getPreviousTaskId();
				}
			}
			
			tpl_assign('gantt_tasks', $gantt_tasks);
			tpl_assign('gantt_tasks_dependencies', $dependencies);
		}
 
		if(isset($customer))tpl_assign("customer", $customer);
		$contacts = $project->getContacts();
		tpl_assign("contacts", $contacts);
		tpl_assign("project", $project);
		tpl_assign("hours", $hours);
		tpl_assign("all_has_due_date", $all_has_due_date);
		tpl_assign("all_completed_has_work", $all_completed_has_work);
		tpl_assign("all_has_estimated_time", $all_has_estimated_time);
		tpl_assign("overdue_upcoming_objects", $overdue_upcoming_objects);
		tpl_assign("project_milestones_stats", $milestone_stats);
		tpl_assign("project_tasks_stats", $task_stats);
		tpl_assign("unassigned_contacts",$this->getUnassignedContacts($project));
	}
	
	


	private function sumTasksHoursByDueDate($tasks) {
		if (count($tasks) == 0) return array();
		
		$result = array();
		$d = $tasks[0]->getDueDate()->beginningOfDay();

		$user_condition = "";
		if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
			$user_condition = " AND contact_id = " . logged_user()->getId();
		}
		
		foreach ($tasks as $task) {
			$task_tstamp = $task->getDueDate()->beginningOfDay()->getTimestamp();
			$tstamp = $d->getTimestamp();
			if (!isset($result[$tstamp])) $result[$tstamp] = array('estimated' => 0, 'worked' => 0);

			if ($tstamp != $task_tstamp) {
				
				while ($task_tstamp != $tstamp) {
					$d = $d->add('d', 1);
					$tstamp = $d->getTimestamp();
					$result[$d->getTimestamp()] = array('estimated' => 0, 'worked' => 0);
				}
			}
			$context = active_context();
			$result[$d->getTimestamp()]['estimated'] += $task->getTimeEstimate();
			//$ts_res = Timeslots::getContentObjects($context, ObjectTypes::findById(Timeslots::instance()->getObjectTypeId()), null, null, "AND `rel_object_id`=".$task->getId() );
			$ts_res = Timeslots::instance()->listing(array(
				"extra_conditions" => " AND rel_object_id =".$task->getId() . $user_condition
			));
			foreach ($ts_res->objects as $timeslot) {
				if ($timeslot->getEndTime() instanceof DateTimeValue) {
					$worked_sec = $timeslot->getEndTime()->getTimestamp() - $timeslot->getStartTime()->getTimestamp() - $timeslot->getSubtract();
					$result[$d->getTimestamp()]['worked'] += round($worked_sec / 60);
				}
			}
		}
		
		return $result; 
	}
	
	private function sumTasksHoursByType($tasks) {
		if (count($tasks) == 0) return array();
		
		$result = array('estimated' => 0, 'worked' => 0);
		$context=active_context();
		$user_condition = "";
		if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
			$user_condition = " AND contact_id = " . logged_user()->getId();
		}
		foreach ($tasks as $task) {
			
			$result['estimated'] += $task->getTimeEstimate();
			//$ts_res = Timeslots::getContentObjects($context, ObjectTypes::findById(Timeslots::instance()->getObjectTypeId()), null, null, "AND `rel_object_id`=".$task->getId() );
			$ts_res = Timeslots::instance()->listing(array(
				"extra_conditions" => " AND rel_object_id =".$task->getId() . $user_condition
			));
			
			foreach ($ts_res->objects as $timeslot) {
				if ($timeslot->getEndTime() instanceof DateTimeValue) {
					$worked_mili = $timeslot->getEndTime()->getTimestamp() - $timeslot->getStartTime()->getTimestamp() - $timeslot->getSubtract();
					$result['worked'] += round($worked_mili / 60);
				}
			}
		}
		
		return $result; 
	}
	
	
	
	
	private function getUnassignedContacts(Project $project) {
		$project_contacts = $project->getContactIds();
		$conditions = $this->base_conditions ;
		$conditions['conditions'] .= "  AND  is_company = 0 ";
		$conditions['order'] = "name"; 
		$contacts = array();
		foreach (Contacts::instance()->findAll( $conditions ) as $contact) {
			/* @var $contact Contact */
			if (!in_array($contact->getId(), $project_contacts) ) {
				$text  = $contact->getObjectName() ;
				$value = $contact->getId() ;
				$contacts[] =  array ($value, $text) ;
			}
		} 
		return $contacts;
	}
		
	private function populateComboboxes() {
		$contacts =  array(array(0,"-")) ;
		$conditions = $this->base_conditions ;
		$conditions['conditions'] .= "  AND  is_company = 0 ";
		$conditions['order'] = "name"; 
		
		foreach (Contacts::instance()->findAll( $conditions ) as $contact) {
			/* @var $contact Contact */
			$text  = $contact->getObjectName() ;
			$value = $contact->getId() ;
			$contacts[] =  array ($value, $text) ;
		} 
		$status = array() ;
		for ($i=1; $i<=7; $i++) $status[] = array($i, lang('project_field_status'.$i));
		
		tpl_assign('customers', Customers::findAll(array('conditions' => '`trashed_by_id`=0 AND `archived_by_id`=0')));
		tpl_assign('status', $status);
		tpl_assign("contacts", $contacts);
	}
	
	
	

	
	
} 