<?php 


class CrpmAPI {

	/**
	 * Customer - PErson associations
	 */
	static function getCustomerAssociation(){
		return DimensionMemberAssociations::findOne(array("conditions"=>"
		  dimension_id IN (SELECT dimension_id FROM ".TABLE_PREFIX."dimensions WHERE code ='customer_project') AND
		  associated_dimension_id IN (SELECT id FROM ".TABLE_PREFIX."dimensions WHERE code ='feng_persons') AND
		  object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE name = 'customer') AND
		  associated_object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE name = 'person')
		"))->getId();
	}
	
	/**
	 * Project-Person 
	 */
	static function getProjectAssociation(){
		return DimensionMemberAssociations::findOne(array("conditions"=>"
		  dimension_id IN (SELECT dimension_id FROM ".TABLE_PREFIX."dimensions WHERE code ='customer_project') AND
		  associated_dimension_id IN (SELECT id FROM ".TABLE_PREFIX."dimensions WHERE code ='feng_persons') AND
		  object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE name = 'project') AND
		  associated_object_type_id IN (SELECT id FROM ".TABLE_PREFIX."object_types WHERE name = 'person')
		"))->getId();
	}
	
	
	static function getCustomers($contact) {
		$customers = array();
		// projects customers
		$projects = self::getProjects($contact);
		foreach ($projects as $pr) {
			if ($pr->getCustomerId() && !isset($customers[$pr->getCustomerId()])) {
				$customers[$pr->getCustomerId()] = $pr->getCustomer();
			}
		}
		// direct associated customers
		$associationId = self::getCustomerAssociation();
		$contactMembers = Members::findByObjectId($contact->getId(), Dimensions::findByCode('feng_persons')->getId());
		if(count($contactMembers) != 1) {
			return $customers;
		}
		$contactMemberId = $contactMembers[0]->getId();
		$mpms = MemberPropertyMembers::findAll(array("conditions"=>array(
			"association_id = $associationId AND property_member_id = $contactMemberId "
		)));
		
		$pg_ids = implode(',', logged_user()->getPermissionGroupIds());
		
		foreach ($mpms as $mpm) {
			$member = Members::findById($mpm->getMemberId());
			if ($member) {
				
				$dimension = $member->getDimension();
				if ($dimension->deniesAllForContact($pg_ids)) continue;
				if (!$dimension->hasAllowAllForContact($pg_ids) && !$member->canBeReadByContact($pg_ids, logged_user())) continue;
				
				$customer = Customers::findById($member->getObjectId());
				if ($customer) {
					if (!isset($customers[$customer->getId()])) {
						$customers[$customer->getId()] = $customer;
					}
				}			
			}
		}
		return feng_sort($customers);
	}

	static function getProjects($contact) {
		$associationId = self::getProjectAssociation();
		$contactMembers = Members::findByObjectId($contact->getId(), Dimensions::findByCode('feng_persons')->getId());
		if(count($contactMembers) != 1) {
			return null ;
		}
		$contactMemberId = $contactMembers[0]->getId();
		$mpms = MemberPropertyMembers::findAll(array("conditions"=>array(
			"association_id = $associationId AND property_member_id = $contactMemberId "
		)));
		
		$pg_ids = implode(',', logged_user()->getPermissionGroupIds());
		
		$projects = array();		
		foreach ($mpms as $mpm) {
			$member = Members::findById($mpm->getMemberId());
			if (!$member instanceof Member) continue;
			
			$dimension = $member->getDimension();
			if ($dimension->deniesAllForContact($pg_ids)) continue;
			if (!$dimension->hasAllowAllForContact($pg_ids) && !$member->canBeReadByContact($pg_ids, logged_user())) continue;
			
			$project = Projects::findById($member->getObjectId());
			if (!$project instanceof Project) continue;
			$projects[strtolower($project->getName())] = $project ;
		}
		ksort($projects);
		return $projects ;
	}		
	
	static function sumTasksHoursByDueDate($tasks) {
		if (count($tasks) == 0) return array();
		
		$result = array();
		$d = $tasks[0]->getDueDate()->beginningOfDay();
		$user_condition = "";
		if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
			$user_condition = " AND contact_id = " . logged_user()->getId();
		}
		foreach ($tasks as $task) {
			$task_tstamp = $task->getDueDate()->beginningOfDay()->getTimestamp();
			$tstamp = $d->getTimestamp();
			if (!isset($result[$tstamp])) $result[$tstamp] = array('estimated' => 0, 'worked' => 0);

			if ($tstamp != $task_tstamp) {
				
				while ($task_tstamp != $tstamp) {
					$d = $d->add('d', 1);
					$tstamp = $d->getTimestamp();
					$result[$d->getTimestamp()] = array('estimated' => 0, 'worked' => 0);
				}
			}
			$context = active_context();
			$result[$d->getTimestamp()]['estimated'] += $task->getTimeEstimate();
			
			$ts_res = Timeslots::instance()->listing(array(
				"extra_conditions" => " AND rel_object_id =".$task->getId() . $user_condition
			));
			foreach ($ts_res->objects as $timeslot) {
				if ($timeslot->getEndTime() instanceof DateTimeValue) {
					$worked_sec = $timeslot->getEndTime()->getTimestamp() - $timeslot->getStartTime()->getTimestamp() - $timeslot->getSubtract();
					$result[$d->getTimestamp()]['worked'] += round($worked_sec / 60);
				}
			}
		}
		
		return $result; 
	}
	
	static function sumTasksHoursByType($tasks, $raw_timeslots_by_task) {
		if (count($tasks) == 0) return array();
		
		$result = array('estimated' => 0, 'worked' => 0);
		foreach ($tasks as $task_data) {
			$result['estimated'] += $task_data['time_estimate'];
			
			$task_timeslots = array_var($raw_timeslots_by_task, $task_data['object_id']);
			
			if (is_array($task_timeslots) && count($task_timeslots) > 0) {
				foreach ($task_timeslots as $timeslot) {
					$end_time = DateTimeValueLib::makeFromString($timeslot['end_time']);
					$start_time = DateTimeValueLib::makeFromString($timeslot['start_time']);
					if ($end_time instanceof DateTimeValue) {
						$worked_mili = $end_time->getTimestamp() - $start_time->getTimestamp() - $timeslot['subtract'];
						if ($worked_mili > 0) $result['worked'] += round($worked_mili / 60);
					}
				}
			}
		}
		
		return $result; 
	}
	
	
	static function sumTasksHoursByExecutionTime($tasks, $raw_timeslots_by_task) { 
		if (count($tasks) == 0) return array();
		
		$result = array();
		
		foreach ($tasks as $task_data) {	
			$all_dates = array();
			$task_timeslots = array_var($raw_timeslots_by_task, $task_data['object_id']);
			if (!is_array($task_timeslots)) $task_timeslots = array();
			
			foreach ($task_timeslots as $timeslot) {
				$end_time = DateTimeValueLib::makeFromString($timeslot['end_time']);
				$start_time = DateTimeValueLib::makeFromString($timeslot['start_time']);
				if (!$end_time instanceof DateTimeValue) continue;
				$ts_end_date = new DateTimeValue($end_time->getTimestamp());
				$ts_timestamp = $ts_end_date->beginningOfDay()->getTimestamp();
				if (!isset($all_dates[$ts_timestamp])) {
					$all_dates[$ts_timestamp] = array('estimated' => 0, 'worked' => 0);
				}
				
				$worked_sec = $end_time->getTimestamp() - $start_time->getTimestamp() - $timeslot['subtract'];
				if ($worked_sec > 0) $all_dates[$ts_timestamp]['worked'] += round($worked_sec / 60);
			}
			$start_date = null;
			if ($task_data['start_date'] != EMPTY_DATETIME) {
				$start_date = DateTimeValueLib::makeFromString($task_data['start_date']);
			}
			$due_date = null;
			if ($task_data['due_date'] != EMPTY_DATETIME) {
				$due_date = DateTimeValueLib::makeFromString($task_data['due_date']);
			}
			// Estimated time
			if ($start_date instanceof DateTimeValue) {
				$task_start_date = new DateTimeValue($start_date->getTimestamp());
				$et_timestamp = $task_start_date->beginningOfDay()->getTimestamp();
			} else if ($due_date instanceof DateTimeValue) {
				$task_due_date = new DateTimeValue($due_date->getTimestamp() - $task_data['time_estimate']*60);
				$et_timestamp = $task_due_date->beginningOfDay()->getTimestamp();
			} else {
				continue;
			}
			if (!isset($all_dates[$et_timestamp])) {
				$all_dates[$et_timestamp] = array('estimated' => 0, 'worked' => 0);
			}
			$all_dates[$et_timestamp]['estimated'] += $task_data['time_estimate'];
			
			// Join this task's times with all results
			foreach ($all_dates as $time => $values) {
				if (!isset($result[$time])) {
					$result[$time] = $values;
				} else {
					$result[$time]['estimated'] += $values['estimated'];
					if ($values['worked'] > 0) $result[$time]['worked'] += $values['worked'];
				}
			}
			
		}
		
		ksort($result, SORT_NUMERIC);
		
		foreach ($result as $k => &$value) {
			if ($value['estimated'] == 0 && $value['worked'] == 0) {
				unset($result[$k]);
			} else {
				break;
			}
		}
		return $result; 
	}
	
}