<?php

/**
 * Calculates the additional data to show in clients listing
 */
function calculate_customers_additional_data($params) {
	Env::useHelper('functions', 'crpm');
	$clients_dim = get_customers_dimension();
	$customer_ot = ObjectTypes::findByName('customer');
	$customers = array();
	
	if (array_var($params, 'customer_id')) {
		
		$customer = Customers::findById(array_var($params, 'customer_id'));
		if ($customer instanceof Customer) $customers[] = $customer;
	
	} else if (array_var($params, 'member_id')) {
		
		$member = Members::findById(array_var($params, 'member_id'));
		if ($member instanceof Member) {
			$customer = Customers::findById($member->getObjectId());
			if ($customer instanceof Customer) $customers[] = $customer;
		}
		
	} else if (array_var($params, 'task_id')) {
		
		$task = ProjectTasks::findById(array_var($params, 'task_id'));
		if ($task instanceof ProjectTask) {

			$all_members = $task->getMembers();
			$task_members = array();
			foreach ($all_members as $mem) {
				if ($mem->getDimensionId() == $clients_dim->getId()) {
					$task_members = array_merge($task_members, $mem->getAllParentMembersInHierarchy(true));
				}
			}
			
			foreach ($task_members as $tm) {
				if ($tm->getObjectTypeId() == $customer_ot->getId()) {
					$customer = Customers::findById($tm->getObjectId());
					if ($customer instanceof Customer) $customers[] = $customer;
				}
			}
		}
		
	} else if (array_var($params, 'multiple_task_ids')) {
		$ids = array_var($params, 'multiple_task_ids');
		if (count($ids) > 0) {
			$tasks = ProjectTasks::findAll(array("conditions" => "id IN (".implode(',', $ids).")"));
			foreach ($tasks as $task) {
				
				$all_members = $task->getMembers();
				$task_members = array();
				foreach ($all_members as $mem) {
					if ($mem->getDimensionId() == $clients_dim->getId()) {
						$task_members = array_merge($task_members, $mem->getAllParentMembersInHierarchy(true));
					}
				}
				
				foreach ($task_members as $tm) {
					if ($tm->getObjectTypeId() == $customer_ot->getId() && !isset($customers[$tm->getObjectId()])) {
						$customer = Customers::findById($tm->getObjectId());
						if ($customer instanceof Customer) $customers[$tm->getObjectId()] = $customer;
					}
				}
			}
		}
		
	}
	
	
	foreach ($customers as $customer) {
		calculate_single_customer_additional_data($customer);
	}
	
}

/**
 * Calculates the additional for a single project
 */
function calculate_single_customer_additional_data(Customer $customer) {
	Env::useHelper('functions', 'crpm');
	$dimension = get_customers_dimension();
	$customer_member = Members::findOneByObjectId($customer->getId(), $dimension->getId());
	if (!$customer_member instanceof Member) return;
	
	DB::execute("DELETE FROM ".TABLE_PREFIX."customer_additional_data WHERE customer_id=".$customer->getId());
	
	$all_tasks = ProjectTasks::findAll(array(
		'join' => array('table' => TABLE_PREFIX.'object_members', 'e_field' => 'object_id', 'jt_field' => 'object_id'),
		'conditions' => "jt.member_id = ".$customer_member->getId()." AND trashed_by_id=0 ",
	));
	
	$completed_num = 0;
	$total_estimated_time = 0;
	$total_worked_time = 0;
	foreach ($all_tasks as $t) {
		/* @var $t ProjectTask */
		if ($t->getCompletedById() > 0) $completed_num++;
		$total_estimated_time += $t->getTimeEstimate();
		$total_worked_time += $t->getTotalMinutes();
	}
	
	if ($total_estimated_time > 0) {
		$time_worked_p = $total_worked_time * 100 / $total_estimated_time;
	} else {
		$time_worked_p = 0;
	}
	
	$data = array(
		'customer_id' => $customer->getId(),
		'total_tasks' => count($all_tasks),
		'completed_tasks' => $completed_num,
		'task_completion_p' => count($all_tasks) > 0 ? $completed_num * 100 / count($all_tasks) : 0,
		'total_estimated_time' => $total_estimated_time,
		'total_worked_time' => $total_worked_time,
		'time_worked_p' => $time_worked_p,
	);
	
	$columns = array_keys($data);
	$sql = "INSERT INTO ".TABLE_PREFIX."customer_additional_data (". implode(',', $columns) .") VALUES ('". implode("','", $data) ."');";
	DB::execute($sql);
}
