<?php

/**
 * Customers class
 *
 * 
 */
class Customers extends BaseCustomers {
	
	public function __construct() {
		parent::__construct ();
		$this->object_type_name = 'customer';
	}
	
	static function getCustomerFieldNames() {
		return array('customer[name]' => lang('name'),
			'customer[system_usage]' => lang('system usage'), 
			'customer[status]' => lang('status'),
			'customer[CMRR]' => lang('CMRR')
		);
	}
	
	
	static function memberize(Customer $customer, $dimension_id , $parent = null, $color = 0, $template_id=0, $from_mail_id=0) {
		$member = Members::instance()->findOne(array(
			"conditions" => "dimension_id = $dimension_id AND object_id = ". $customer->getId()
		));
		$is_new = false;
		if (!$member) {
			$member = new Member();
			$member->setObjectTypeId(Customers::instance()->getObjectTypeId());
			$member->setDimensionId($dimension_id);
			$member->setObjectId($customer->getId());
			$member->setDepth(1);
			
			$is_new = true;
		}
		
		$member->setColor($color);
		if (!is_null($parent)) {
			$member->setParentMemberId($parent);
			$p = Members::findById($parent);
			if ($p instanceof Member) $member->setDepth($p->getDepth() + 1);
			else $member->setDepth(1);
		}
		if ($template_id > 0) {
			$member->setColumnValue('template_id', $template_id);
		}
		if ($from_mail_id > 0) {
			$member->setColumnValue('from_mail_id', $from_mail_id);
		}
		
		$member->setName($customer->getName());
		$member->save();
		
		
		if ($is_new && !array_var($_REQUEST, 'permissions')) {
			// set all permissions for the creator
			$dimension = $member->getDimension();
			$object_type = ObjectTypes::findByName('customer');

			$changed_pgs = array();

			$allowed_object_types = array();
			$dim_obj_types = $dimension->getAllowedObjectTypeContents();
			foreach ($dim_obj_types as $dim_obj_type) {
				// To draw a row for each object type of the dimension
				if (!in_array($dim_obj_type->getContentObjectTypeId(), $allowed_object_types) && $dim_obj_type->getDimensionObjectTypeId() == $member->getObjectTypeId()) {
					$allowed_object_types[] = $dim_obj_type->getContentObjectTypeId();
				}
			}
			$allowed_object_types[]=$object_type->getId();
			foreach ($allowed_object_types as $ot) {
				$cmp = ContactMemberPermissions::findOne(array('conditions' => 'permission_group_id = '.logged_user()->getPermissionGroupId().' AND member_id = '.$member->getId().' AND object_type_id = '.$ot));
				if (!$cmp instanceof ContactMemberPermission) {
					$cmp = new ContactMemberPermission();
					$cmp->setPermissionGroupId(logged_user()->getPermissionGroupId());
					$cmp->setMemberId($member->getId());
					$cmp->setObjectTypeId($ot);
				}
				$cmp->setCanWrite(1);
				$cmp->setCanDelete(1);
				$cmp->save();

				$changed_pgs[$cmp->getPermissionGroupId()] = $cmp->getPermissionGroupId();
			}
		
			// set all permissions for permission groups that has allow all in the dimension
			$permission_groups = ContactDimensionPermissions::findAll(array("conditions" => array("`dimension_id` = ? AND `permission_type` = 'allow all'", $dimension->getId())));
			if (is_array($permission_groups)) {
				foreach ($permission_groups as $pg) {
					foreach ($allowed_object_types as $ot) {
						$cmp = ContactMemberPermissions::findById(array('permission_group_id' => $pg->getPermissionGroupId(), 'member_id' => $member->getId(), 'object_type_id' => $ot));
						if (!$cmp instanceof ContactMemberPermission) {
							$cmp = new ContactMemberPermission();
							$cmp->setPermissionGroupId($pg->getPermissionGroupId());
							$cmp->setMemberId($member->getId());
							$cmp->setObjectTypeId($ot);
						}
						$cmp->setCanWrite(1);
						$cmp->setCanDelete(1);
						$cmp->save();
					}

					$changed_pgs[$pg->getPermissionGroupId()] = $pg->getPermissionGroupId();
				}
			}
		
			// Inherit permissions from parent node, if they are not already set
			if ( $member->getDepth() && $member->getParentMember() ) {
				$parentNodeId = $member->getParentMember()->getId();
				$condition = "member_id = $parentNodeId" ;
				foreach ( ContactMemberPermissions::instance()->findAll(array("conditions"=>$condition)) as $parentPermission ){
					/* @var $parentPermission ContactMemberPermission */
					$g = $parentPermission->getPermissionGroupId() ;
					$t = $parentPermission->getObjectTypeId() ;
					$w = $parentPermission->getCanWrite() ;
					$d = $parentPermission->getCanDelete() ;
					$existsCondition = "member_id = ".$member->getId()." AND permission_group_id= $g AND object_type_id = $t";
					if (!ContactMemberPermissions::instance()->count(array("conditions"=>$existsCondition))){
						$newPermission = new ContactMemberPermission();
						$newPermission->setPermissionGroupId($g);
						$newPermission->setObjectTypeId($t);
						$newPermission->setCanWrite($w);
						$newPermission->setCanDelete($d);
						$newPermission->setMemberId($member->getId());
						$newPermission->save();

						$changed_pgs[$parentPermission->getPermissionGroupId()] = $parentPermission->getPermissionGroupId();
					}
				}
			}

			//Update Contact Member cache
			$contactMemberCacheController = new ContactMemberCacheController();
			$contactMemberCacheController->afterMemberPermissionChanged(array('changed_pgs' => $changed_pgs, 'member' => $member));
		}
		
		
		$customer->addToMembers(array($member));
		$customer->addToSharingTable();
	}
	
	static function findByInstallation($installation_id){
		return self::findOne(array("conditions" => "installation_id = '". $installation_id."'"));
	}
	
	




	function getAdditionalCustomProperties() {
		$contact_ot = ObjectTypes::findByName('contact');
		return CustomProperties::instance()->getAllCustomPropertiesByObjectType($contact_ot->getId());
	}
	
	
	function getAssociatedObjectsFixedColumns() {
		$columns = array(
			'contact_id' => array(
				array('col' => 'email', 'external' => true, 'type' => 'email', 'label' => lang("email")),
				array('col' => 'phone', 'external' => true, 'type' => 'phone', 'label' => lang("phone")),
				array('col' => 'address', 'external' => true, 'type' => 'address', 'label' => lang("address")),
				array('col' => 'webpage', 'external' => true, 'type' => 'webpage', 'label' => lang("webpage")),
				array('col' => 'picture_file', 'external' => true, 'type' => 'image', 'label' => lang("picture")),
				array('col' => 'comments', 'type' => DATA_TYPE_STRING, 'label' => lang("notes"), 'large' => true),
			//	array('col' => 'timezone_id', 'type' => DATA_TYPE_TIMEZONE, 'label' => lang("timezone")),
				array('col' => 'company_id', 'type' => 'company', 'label' => lang("company")),
				array('col' => 'birthday', 'type' => DATA_TYPE_DATE, 'label' => lang("birthday")),
				array('col' => 'department', 'type' => DATA_TYPE_STRING, 'label' => lang("department")),
			)
		);
		
		return $columns;
	}
	
	
	
} 

