<?php
  /**
  * Customers class
  *
  * 
  */
  abstract class BaseCustomers extends ContentDataObjects {  
    /**
    * Column name => Column type map
    *
    * @var array
    * @static
    */
    static private $columns = array(
		'object_id' => DATA_TYPE_INTEGER,
		'contact_id' => DATA_TYPE_INTEGER,
		'status_id' => DATA_TYPE_INTEGER,
    	'installation_id' => DATA_TYPE_STRING, 
		'system_usage' => DATA_TYPE_FLOAT,
		'users_limit' => DATA_TYPE_INTEGER,
		'diskspace_limit' => DATA_TYPE_INTEGER,
		'collaborators_limit' => DATA_TYPE_INTEGER,
		'active_users' => DATA_TYPE_INTEGER,
		'cmrr' => DATA_TYPE_FLOAT,
		'payment_cycle_id' => DATA_TYPE_FLOAT,
		'service_type_id' => DATA_TYPE_FLOAT,
		'feedback_id' => DATA_TYPE_INTEGER,//
    	'code' => DATA_TYPE_INTEGER,
    	'start_date' => DATA_TYPE_DATE,
    	'end_date' => DATA_TYPE_DATE,
    	'industry' => DATA_TYPE_STRING,
    	'internal_users' => DATA_TYPE_INTEGER,
    	'external_users' => DATA_TYPE_INTEGER,
    	'subscription_channel_id' => DATA_TYPE_INTEGER,
    	'payment_method_id' => DATA_TYPE_INTEGER,
        'customer_name' => DATA_TYPE_STRING,
        'customer_address' => DATA_TYPE_STRING,
        'customer_rut' => DATA_TYPE_STRING,
        'customer_number' => DATA_TYPE_INTEGER,
        'automatic_status' => DATA_TYPE_INTEGER,
        'contact_object_type_id' => DATA_TYPE_INTEGER
     
    );
  
    /**
    * Construct
    *
    * @return BaseCustomers 
    */
    function __construct() {
      Hook::fire('object_definition', 'Customer', self::$columns);
      parent::__construct('Customer', 'customers', true);
    } 
    
    /**
    * Return array of object columns
    *
    * @access public
    * @param void
    * @return array
    */
    function getColumns() {
      return array_keys(self::$columns);
    }
    
  	/**
	* Return column type
	*
	* @access public
	* @param string $column_name
	* @return string
	*/
	function getColumnType($column_name) {
		return parent::getCOColumnType($column_name, self::$columns);
	}
    
    /**
    * Return array of PK columns. If only one column is PK returns its name as string
    *
    * @access public
    * @param void
    * @return array or string
    */
    function getPkColumns() {  
      return 'object_id';
    }
    
    /**
    * Return name of first auto_incremenent column if it exists
    *
    * @access public
    * @param void
    * @return string
    */
    function getAutoIncrementColumn() {
      return 'object_id';      
    }
    
    function getSystemColumns() {
    	return array_merge(parent::getSystemColumns(), array('contact_id','active_users','automatic_status','code','collaborators_limit','contact_object_type_id','customer_address','customer_name',
    			'customer_number','customer_rut','diskspace_limit','diskspace_limit','industry','installation_id','internal_users','payment_method_id','service_type_id','subscription_channel_id',
    			'system_usage','users_limit','external_users','feedback_id', 'updated_on', 'updated_by_id', 'created_on', 'created_by_id', 'archived_on', 'archived_by_id'));
    }
    
    /**
    * Return external columns
    *
    * @access public
    * @param void
    * @return array
    */
    function getExternalColumns() {
      $external_columns = array_merge(parent::getExternalColumns(), array('payment_cycle_id', 'status_id'));
      Hook::fire('additional_external_columns', $this, $external_columns);
      
      return $external_columns;
    } 
    
    /**
    * Return report object title columns
    *
    * @access public
    * @param void
    * @return array
    */
    function getReportObjectTitleColumns() {
      return array('name');
    } 
    
    /**
    * Return report object title
    *
    * @access public
    * @param void
    * @return string
    */
    function getReportObjectTitle($values) {
    	return null ;
    }
    
    /**
    * Do a SELECT query over database with specified arguments
    *
    * @access public
    * @param array $arguments Array of query arguments. Fields:
    * 
    *  - one - select first row
    *  - conditions - additional conditions
    *  - order - order by string
    *  - offset - limit offset, valid only if limit is present
    *  - limit
    * 
    * @return one or Customers objects
    * @throws DBQueryError
    */
    function find($arguments = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::find($arguments);
      } else {
        return Customers::instance()->find($arguments);
      } 
    } 
    
    /**
    * Find all records
    *
    * @access public
    * @param array $arguments
    * @return one or Customers objects
    */
    function findAll($arguments = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::findAll($arguments);
      } else {
        return Customers::instance()->findAll($arguments);
      } // if
    } // findAll
    
    /**
    * Find one specific record
    *
    * @access public
    * @param array $arguments
    * @return Customer 
    */
    function findOne($arguments = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::findOne($arguments);
      } else {
        return Customers::instance()->findOne($arguments);
      } // if
    } // findOne
    
    /**
    * Return object by its PK value
    *
    * @access public
    * @param mixed $id
    * @param boolean $force_reload If true cache will be skipped and data will be loaded from database
    * @return Customer 
    */
    function findById($id, $force_reload = false) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::findById($id, $force_reload);
      } else {
        return Customers::instance()->findById($id, $force_reload);
      } // if
    } // findById
    
    /**
    * Return number of rows in this table
    *
    * @access public
    * @param string $conditions Query conditions
    * @return integer
    */
    function count($condition = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::count($condition);
      } else {
        return Customers::instance()->count($condition);
      } // if
    } // count
    
    /**
    * Delete rows that match specific conditions. If $conditions is NULL all rows from table will be deleted
    *
    * @access public
    * @param string $conditions Query conditions
    * @return boolean
    */
    function delete($condition = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::delete($condition);
      } else {
        return Customers::instance()->delete($condition);
      } // if
    } // delete
    
    /**
    * This function will return paginated result. Result is an array where first element is 
    * array of returned object and second populated pagination object that can be used for 
    * obtaining and rendering pagination data using various helpers.
    * 
    * Items and pagination array vars are indexed with 0 for items and 1 for pagination
    * because you can't use associative indexing with list() construct
    *
    * @access public
    * @param array $arguments Query argumens (@see find()) Limit and offset are ignored!
    * @param integer $items_per_page Number of items per page
    * @param integer $current_page Current page number
    * @return array
    */
    function paginate($arguments = null, $items_per_page = 10, $current_page = 1, $count = null) {
      if(isset($this) && instance_of($this, 'Customers')) {
        return parent::paginate($arguments, $items_per_page, $current_page);
      } else {
        return Customers::instance()->paginate($arguments, $items_per_page, $current_page);
      } // if
    } // paginate
    
    
    /**
    * Return manager instance
    *
    * @return Customers 
    */
    function instance() {
      static $instance;
      if(!instance_of($instance, 'Customers')) {
        $instance = new Customers();
      } 
      return $instance;
    } 
  
  } 