<?php
Hook::register("member_custom_reports");


function member_reports_build_group_by_sql($ot, $group_by_criterias, &$select_columns, &$joins_sql, &$order_by_sql) {
	
	$order_sql = "";
	
	foreach ($group_by_criterias as $criteria) {
		$id = $criteria['id'];
		switch ($criteria['type']) {
			case 'parent_member':
				
				$select_columns .= ", COALESCE(mx_$id.id, 0) as _group_id_pm_$id, mx_$id.name as _group_name_pm_$id";
				$joins_sql .= "
					left join ".TABLE_PREFIX."members mx_$id on mx_$id.id=m.parent_member_id
				";
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "_group_name_pm_$id";
				
				break;
			case 'association':
				
				$select_columns .= ", COALESCE(mz_$id.id, 0) as _group_id_a_$id, mz_$id.name as _group_name_a_$id";
				$joins_sql .= "
					left join ".TABLE_PREFIX."member_property_members z_$id on z_$id.association_id=$id and z_$id.member_id=m.id
					left join ".TABLE_PREFIX."members mz_$id on mz_$id.id=z_$id.property_member_id
				";
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "_group_name_a_$id";
				
				break;
			case 'custom_property':
				
				if ($ot->getType() == 'dimension_object') {
					$cp = CustomProperties::findById($id);
					$cp_values_table = "custom_property_values";
					$cp_values_field = "object_id";
					$cp_values_join_field = "m.object_id";
				} else {
					$cp = MemberCustomProperties::findById($id);
					$cp_values_table = "member_custom_property_values";
					$cp_values_field = "member_id";
					$cp_values_join_field = "m.id";
				}
				
				$select_columns .= ", COALESCE(y_$id.value, 0) as _group_id_cp_$id";
				$joins_sql .= "
					left join ".TABLE_PREFIX."$cp_values_table y_$id on y_$id.custom_property_id=$id and y_$id.$cp_values_field=$cp_values_join_field
				";
				$order_sql .= ($order_sql == "" ? "ORDER BY " : ", ") . "_group_name_cp_$id";
				
				if (in_array($cp->getType(), array('contact', 'user'))) {
					$select_columns .= ", oy_$id.name as _group_name_cp_$id";
					$joins_sql .= "
						left join ".TABLE_PREFIX."objects oy_$id on oy_$id.id=y_$id.value
					";
				} else {
					$select_columns .= ", y_$id.value as _group_name_cp_$id";
				}
				
				break;
			default:
				break;
		}
	}
	
	if (count($group_by_criterias) > 0) {
		$last_order_by = "";
		if (trim($order_by_sql) != "") {
			$last_order_by = str_replace("ORDER BY ", "", $order_by_sql);
		}
		$order_by_sql = "$order_sql, $last_order_by";
	}
	
}





function member_custom_reports_replace_execute_report_function($hook_parameters, &$results) {
	$results = null;
	
	$report = array_var($hook_parameters, 'report');
	$params = array_var($hook_parameters, 'params');
	$order_by_col = array_var($hook_parameters, 'order_by_col');
	$order_by_asc = array_var($hook_parameters, 'order_by_asc');
	$offset = array_var($hook_parameters, 'offset');
	$limit = array_var($hook_parameters, 'limit');
	$to_print = array_var($hook_parameters, 'to_print');
	
	$group_by = explode(',', $report->getColumnValue('group_by'));
	Hook::fire('override_custom_report_group_by', $report, $group_by);
	
	$group_by_options = null;
	$gb_options = trim($report->getColumnValue('group_by_options'));
	if ($gb_options) {
		$group_by_options = json_decode($gb_options, true);
	}
	$group_by_criterias = parse_custom_report_group_by($group_by, $group_by_options);
	
	if ($report instanceof Report) {
		$id = $report->getId();
		$ot = ObjectTypes::findById($report->getReportObjectTypeId());
		
		// this function is only for member reports
		if (!in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
			return;
		}
		
		$dimension = null;
		$all_dimensions = Dimensions::findAll();
		foreach ($all_dimensions as $dim) {
			$dim_ots = DimensionObjectTypes::getObjectTypeIdsByDimension($dim->getId());
			foreach ($dim_ots as $dim_ot_id) {
				if ($ot->getId() == $dim_ot_id) {
					$dimension = $dim;
					break;
				}
			}
		}
		if (!$dimension instanceof Dimension) {
			return;
		}
		
		$order_by = '';
		if (is_object($params)) {
			$params = get_object_vars($params);
		}
		
		$report_columns = ReportColumns::getAllReportColumns($id);
	
		$join_str = "";
		$model = $ot->getHandlerClass();
		$model_instance = null;
		if ($model) {
			$model_instance = new $model();
			$join_str = " INNER JOIN ".TABLE_PREFIX."objects o ON o.id=m.object_id 
			              INNER JOIN ".TABLE_PREFIX . $ot->getTableName() ." e ON o.id=e.object_id ";
			
			if ($ot->getName() == 'customer') {
				$join_str .= "
					INNER JOIN ".TABLE_PREFIX."contacts c ON e.contact_id = c.object_id
				";
			}
		}
		
		
		$cond_sql_obj = build_report_conditions_sql(array('report' => $report, 'params' => $params));
		$allConditions = array_var($cond_sql_obj, 'all_conditions');
		
		Hook::fire('override_custom_report_order', $report, $report);
		
		if ($order_by_col == '') {
			$order_by_col = $report->getOrderBy();
		}
		if ($order_by_asc == null) $order_by_asc = $report->getIsOrderByAsc();
		$order_by_dir = $order_by_asc ? "ASC" : "DESC";
		
		if (is_numeric($order_by_col)) {
			if ($ot->getType() == 'dimension_object') {
				$mcp = CustomProperties::getCustomProperty($order_by_col);
				$cp_values_table = "custom_property_values";
				$cp_values_field = "object_id";
				$cp_values_join_field = "o.id";
			} else {
				$mcp = MemberCustomProperties::getCustomProperty($order_by_col);
				$cp_values_table = "member_custom_property_values";
				$cp_values_field = "member_id";
				$cp_values_join_field = "m.id";
			}
			if ($mcp instanceof MemberCustomProperty || $mcp instanceof CustomProperty) {
				
				$order_join = "LEFT JOIN ".TABLE_PREFIX."$cp_values_table ordcpv on ordcpv.$cp_values_field = $cp_values_join_field AND ordcpv.custom_property_id=$order_by_col";
				if ($mcp->getType() == 'numeric') {
					$order_sql_string = "ORDER BY CONVERT(ordcpv.value, SIGNED INTEGER) $order_by_dir";
				} else {
					$order_sql_string = "ORDER BY ordcpv.value $order_by_dir";
				}
			} else {
				// default order if custom property is not valid
				$order_sql_string = "ORDER BY m.name $order_by_dir";
			}
			
		} else {
			$order_join = "";
			$order_sql_string = "";
			if ($order_by_col != '') {
				if (in_array($order_by_col, Members::getColumns())) {
					$order_by_col = 'm.'.$order_by_col;
				
				} else if (in_array($order_by_col, Objects::getColumns())) {
					$order_by_col = 'o.'.$order_by_col;
				} else {
					// default order if order column is not valid
					$order_by_col = 'm.name';
				}
				$order_sql_string = "ORDER BY $order_by_col $order_by_dir";
			}
		}
		
		$parent_member = null;
		$context = active_context();
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getDimensionId() == $dimension->getId()) {
				$parent_member = $selection;
			}
		}
		
		// filter by parent member hierarchy
		$parent_member_condition = "";
		if ($parent_member instanceof Member) {
			$all_child_ids = $parent_member->getAllChildrenIds(true);
			if (!is_array($all_child_ids) || count($all_child_ids)==0) $all_child_ids[] = 0;
			$parent_member_condition = " AND m.id IN (".implode(',', $all_child_ids).")";
		}
		
		// associated dimension conditions
		$mem_controller = new MemberController();
		$assoc_params = $mem_controller->build_listing_associated_dimensions_parameters($dimension, $report->getReportObjectTypeId(), 'm');
		$member_association_cond = array_var($assoc_params, 'member_assoc_cond');
		$allConditions .= $member_association_cond;
		
		
		if (logged_user()->isAdministrator()) {
			$permissions_condition = "";
		} else {
			$logged_users_pg_ids = implode(',', logged_user()->getPermissionGroupIds());
			$permissions_condition = " AND EXISTS (SELECT cmp.member_id FROM ".TABLE_PREFIX."contact_member_permissions cmp WHERE cmp.member_id=m.id AND permission_group_id IN ($logged_users_pg_ids)) ";
		}
		
		$limit_str = "";
		if ($offset>=0 && $limit > 0) {
			$limit_str = "LIMIT $offset, $limit";
		}
		
		$select_columns = "DISTINCT(m.id) as mid";
		if ($ot->getType() == 'customer') {
			$select_columns .= ", c.object_id as contact_id ";
		}
		
		member_reports_build_group_by_sql($ot, $group_by_criterias, $more_select_columns, $join_str, $order_sql_string);
		
		$sql = "
			SELECT $select_columns $more_select_columns
			FROM ".TABLE_PREFIX."members m 
			$join_str 
			$order_join 
			WHERE m.object_type_id=".$ot->getId()."
				$allConditions
				$parent_member_condition
				$permissions_condition
			$order_sql_string
			$limit_str
		";
		
		$time = microtime(true);
		$rows = DB::executeAll($sql);
		
		$grouped_rows = group_custom_report_results($rows, $group_by_criterias, $ot);
		
		$group_totals = array();
		$params_array = array(
				'report' => $report,
				'conditions' => "m.object_type_id=".$ot->getId()." $allConditions $parent_member_condition $permissions_condition",
				'columns' => "$select_columns $more_select_columns",
				'from' => "".TABLE_PREFIX."members m $join_str $order_join",
				'order' => $order_sql_string,
				'group_by_criterias' => $group_by_criterias
		);
		Hook::fire('custom_report_group_totals', $params_array, $group_totals);
		
		
		$member_ids = array();
		foreach ($rows as $row) {
			$member_ids[] = $row['mid'];
		}
		
		
		$count_sql = "SELECT count(distinct(m.id)) as totalrows $more_select_columns FROM ".TABLE_PREFIX."members m $join_str $order_join 
			WHERE m.object_type_id=".$ot->getId()." $allConditions $parent_member_condition $permissions_condition $order_sql_string";
		
		$time = microtime(true);
		$totalrows = DB::executeAll($count_sql);
		$totalResults = $totalrows[0]['totalrows'];
		
		
		// if hiding group details, build the result with the totals and return
		if ($report->getColumnValue("hide_group_details")) {
				
			$tmp = _accumulate_group_totals_hidden_details($report, $ot, $group_totals);
			$grouped_rows = group_custom_report_results($tmp, $group_by_criterias, $ot);
				
			// put the unclassified group in the last place
			if (isset($grouped_rows[0])) {
				$grouped_rows[PHP_INT_MAX] = $grouped_rows[0];
				unset($grouped_rows[0]);
			}
				
			$results = array(
					'group_totals' => $group_totals,
					'grouped_rows' => $grouped_rows,
					'hide_group_details' => true,
					'total' => $totalResults,
					'group_by_criterias' => $group_by_criterias,
					'pagination' => '',
					'objects' => array(),
			);
				
			return $results;
		}
		
		$time = microtime(true);
		$all_members = array();
		if (count($member_ids) > 0) {
			$members = Members::findAll(array('conditions' => 'id IN ('.implode(',', $member_ids).')'));
			// follow the order resultant of the first query
			foreach ($member_ids as $mid) {
				foreach ($members as $member) {
					if ($mid == $member->getId()) {
						$all_members[] = $member;
						break;
					}
				}
			}
		}
		
		$results = array();
		
		$results['pagination'] = Reports::getReportPagination($id, $params, $report->getOrderBy(), $order_by_asc, $offset, $limit, $totalResults);
		
		$time = microtime(true);
		// additional data over result
		$results['totalCount'] = $totalResults;
		$from_sql = "FROM ".TABLE_PREFIX."members mem";
		$all_conditions_sql = str_replace(array("m.","`m`."), "mem.", "$allConditions $parent_member_condition $permissions_condition AND m.object_type_id=".$ot->getId());
		$join_sql = str_replace(array("m.","`m`."), "mem.", "$join_str");
		$params = array('type_id' => $report->getReportObjectTypeId(), 'from_sql' => $from_sql, 'joins_sql' => $join_sql, 'conditions_sql' => $all_conditions_sql,
				'group_by' => '', 'order' => $order_by_col, 'order_dir' => $order_by_dir, 'start' => $offset, 'limit' => $limit);
		
		Hook::fire('member_listing_additional_data', $params, $results);
		
		// ---------------------------
		
		$results['columns'] = array('names' => array(), 'order' => array());
		
		$dim_member_associations = array();
		
		// header
		foreach($report_columns as $column){
			if ($column->getCustomPropertyId() == 0) {
				$field = $column->getFieldName();
				if ($model_instance && $model_instance->columnExists($field) || Members::instance()->columnExists($field)) {
					$column_name = Localization::instance()->lang('field '.$ot->getHandlerClass().' '.$field);
					if (is_null($column_name)) $column_name = lang('field Objects '.$field);
					
					$results['columns']['names'][$field] = $column_name;
					$results['columns']['order'][] = $field;
					
				} else if (str_starts_with($field, "dimassoc_")) {
					
					$aid = str_replace("dimassoc_", "", $field);
					$a = DimensionMemberAssociations::findById($aid);
					
					if ($a instanceof DimensionMemberAssociation) {
						$did = $dimension->getId() == $a->getDimensionId() ? $a->getAssociatedDimensionMemberAssociationId() : $a->getDimensionId();
						$assocdim = Dimensions::getDimensionById($did);
						$column_name = $assocdim->getName();
						
						$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($a->getId(), 'custom_association_name');
						if ($custom_assoc_name) {
							$column_name = $custom_assoc_name;
						}
						
						$results['columns']['names'][$field] = $column_name;
						$results['columns']['order'][] = $field;
						
						$dim_member_associations[] = $a;
					}
					
				} else if(str_starts_with($field, "contact_")){

                    $langs = Contacts::getContactFieldNames();
                    $results['columns']['names'][$field] = $langs['contact['.substr($field,8).']'];
                    $results['columns']['order'][] = $field;
                } else if($field == 'budgeted_price'){
					$results['columns']['names'][$field] = lang('budgeted price');
                    $results['columns']['order'][] = $field;
				} else if ($field == 'budgeted_cost'){
					$results['columns']['names'][$field] = lang('budgeted cost');
                    $results['columns']['order'][] = $field;
				}
			} else {
				if (Plugins::instance()->isActivePlugin('member_custom_properties')) {

                    $field = $column->getFieldName();
                    if (str_starts_with($field,'group_cp_')){

                        $list_of_terms = explode('_',$field);
                        $cp = CustomProperties::getCustomProperty($list_of_terms[2]);
                        $field_name = '';
                        $cp_name = '';
                        if ($cp instanceof CustomProperty){
                            $cp_name = $cp->getName();
                        }
                        $value = $list_of_terms[3];
                        if (isset($list_of_terms[4])){
                            $value .= '_'.$list_of_terms[4];
                        }
                        if (isset($list_of_terms[5])){ // to support zip_code
                        	$value .= '_'.$list_of_terms[5];
                        }
                        $langs = Contacts::getContactFieldNames();
                        $field_name = $langs['contact['.$value.']'].' ('.$cp_name.')';

                        $results['columns']['names'][$field] = $field_name;
                        $results['columns']['order'][] = $field;
                    }else{


                        if ($ot->getType() == 'dimension_object') {
                            $cp = CustomProperties::getCustomProperty($column->getCustomPropertyId());
                        } else {
                            $cp = MemberCustomProperties::findById($column->getCustomPropertyId());
                        }
                        if ($cp instanceof MemberCustomProperty || $cp instanceof CustomProperty) {

                            $results['columns']['names'][$column->getCustomPropertyId()] = $cp->getName();
                            $results['columns']['order'][] = $column->getCustomPropertyId();
                        }
                    }
				}
			}
		}
		$results['columns']['names']['located_under'] = lang('located under');
		$results['columns']['order'][] = 'located_under';
		
		
		// dim member associations
		$associated_members = array();
		if (count($dim_member_associations) > 0 && count($member_ids) > 0) {
			$assoc_cols_sql = "";
			$assoc_joins_sql = "";
			
			foreach ($dim_member_associations as $a) {
				$aid = $a->getId();
				$assoc_cols_sql .= ($assoc_cols_sql==""?"":", ") . "amember".$aid.".name as dimassoc_".$aid;
				$assoc_joins_sql .= "
					LEFT JOIN ".TABLE_PREFIX."member_property_members mpm".$aid." ON mpm".$aid.".member_id=m.id AND mpm".$aid.".association_id=$aid
					LEFT JOIN ".TABLE_PREFIX."members amember".$aid." ON amember".$aid.".id = mpm".$aid.".property_member_id
				";
			}
			
			// get all associated members for all member_ids in result
			$assoc_sql = "SELECT m.id as mid, $assoc_cols_sql FROM ".TABLE_PREFIX."members m $assoc_joins_sql WHERE m.id IN (".implode(',', $member_ids).")";
			$assoc_rows = DB::executeAll($assoc_sql);
			foreach ($assoc_rows as $arow) {
				if (!isset($associated_members[$arow["mid"]])) {
					$associated_members[$arow["mid"]] = $arow;
				} else {
					// merge results
					$r = $associated_members[$arow["mid"]];
					foreach ($dim_member_associations as $a) {
						$f = "dimassoc_".$a->getId();
						$f_val = trim(array_var($r, $f));
						$r[$f] .= ($f_val == '' ? '' : "\n") . $arow[$f];
					}
					$associated_members[$arow["mid"]] = $r;
				}
			}
		}
		
		
		$custom_properties = array();
		$report_rows = array();
		foreach($all_members as &$member){/* @var $member Member */
			
			$obj_name = $member->getName();
			$icon_class = $member->getIconClass();
	
			$row_values = array('object_type_id' => $member->getObjectTypeId());
			$all_parent_members = array_reverse($member->getAllParentMembersInHierarchy());
			$parent_str = "";
			$color = "";
			foreach ($all_parent_members as $pm) {
				$parent_str .= ($parent_str == "" ? "" : " / ") . $pm->getName();
				$color = $pm->getColor();
			}
			if ($color != "") $parent_str = "<span class='member-path real-breadcrumb og-wsname-color-$color'>$parent_str</span>";
			$row_values['located_under'] = $parent_str;
			
			foreach($report_columns as $column){
				$value = null;
				
				if ($column->getCustomPropertyId() == 0) {

					$field = $column->getFieldName();
					if (in_array($field, Members::instance()->getColumns())) {
						
						$value = $member->getColumnValue($field);
						
					} else if ($model_instance && in_array($field, $model_instance->getColumns())) {
						
						$rel_object = Objects::findObject($member->getObjectId(), $dimension->getId());
						if ($rel_object instanceof ContentDataObject) {
							$value = $rel_object->getColumnValue($field);
						}
						
					} else if (str_starts_with($field, "dimassoc_")) {
						
						$assoc_members = array_var($associated_members, $member->getId());
						$value = array_var($assoc_members, $field);
						
					} else if ($field == 'budgeted_price'){
						if(Plugins::instance()->isActivePlugin('advanced_billing')){
							Env::useHelper('member_financials_functions', 'advanced_billing');
							$member_id = $member->getId();
							$member_financials = get_member_budgeted_financials($member_id);
							$csymbol = null;
							if ($currency instanceof Currency) {
								$csymbol = $currency->getSymbol();
							}else{
								$csymbol = config_option('currency_code');
							}
							$value  = format_money_amount($member_financials['budgeted_price'], $csymbol);
						}
					} else if ($field == 'budgeted_cost'){
						if(Plugins::instance()->isActivePlugin('advanced_billing')){
							Env::useHelper('member_financials_functions', 'advanced_billing');
							$member_id = $member->getId();
							$member_financials = get_member_budgeted_financials($member_id);
							$csymbol = null;
							if ($currency instanceof Currency) {
								$csymbol = $currency->getSymbol();
							}else{
								$csymbol = config_option('currency_code');
							}
							$value  = format_money_amount($member_financials['budgeted_cost'], $csymbol);
						}
					} else if (str_starts_with($field, "contact_")) {

                        $customer = Customers::findById($member->getObjectId());
                        if ($customer instanceof Customer){
                            $contact = $customer->getContact();
                            if ($contact instanceof Contact){
                                $value = $contact->getColumnValue(substr($field, 8));
                            }

                            if (substr($field, 8) == "all_phones"){
                                $phone_types = TelephoneTypes::getAllTelephoneTypesInfo(true);
                                $all_phones = ContactTelephones::findAll(array('conditions' => 'contact_id = '.$contact->getId()));
                                if (!empty($all_phones)){
                                    $values = array();            
                                        foreach ($all_phones as $phone){
                                            if (user_config_option('show_type_phone')) {
                                                $values[] = $phone_types[$phone->getTelephoneTypeId()]['name'].': '.$phone->getNumber();
                                            } else {
                                                $values[] = $phone->getNumber();
                                            }    
                                        }
                                        $value = implode($values,',');
                                    }    
                            }
                            
                            $field_key = substr($field, 8);
                            if (str_starts_with($field_key, "all_addresses")) {

                            	// full address
	                            if ($field_key == "all_addresses"){
	                                $address_types = AddressTypes::getAllAddressTypesInfo(true);
	                                $all_addresses = ContactAddresses::findAll(array('conditions' => 'contact_id = '.$contact->getId()));
	                                if (!empty($all_addresses)){
	                                    $values = array();
	                                    foreach ($all_addresses as $address){
	                                    	if (user_config_option('show_type_phone')) {
	                                        	$values[] = $address_types[$address->getAddressTypeId()]['name'].': '.$address->toString();
	                                    	} else {
	                                    		$values[] = $address->toString();
	                                    	}
	                                    }
	                                    $value = implode($values,',');
	                                }
	                            } else {
	                            	
	                            	// only one column of the address (example: key format = all_address|street)
	                            	$exploded_key = explode('|', $field_key);
	                            	if (count($exploded_key) == 2) {
	                            		$address_column = $exploded_key[1]; // can be street, city, state, zip_code or country
	                            		$all_addresses = ContactAddresses::findAll(array('conditions' => 'contact_id = '.$contact->getId()));
	                            		if (!empty($all_addresses)){
	                            			$values = array();
	                            			foreach ($all_addresses as $address){
	                            				$address_column_value = $address->getColumnValue($address_column);
	                            				if ($address_column == 'country') {
	                            					$address_column_value = Countries::getCountryNameByCode(strtoupper($address_column_value));
	                            				}
	                            				$values[] = $address_column_value;
	                            			}
	                            			$value = implode($values,',');
	                            		}
	                            	}
	                            }
                            }

                            if (substr($field, 8) == "all_webpages"){
                                $web_types = WebpageTypes::getAllWebpageTypesInfo(true);
                                $all_webpages = ContactWebpages::findAll(array('conditions' => 'contact_id = '.$contact->getId()));
                                if (!empty($all_webpages)){
                                    $values = array();
                                    foreach ($all_webpages as $webpage){
                                    	if (user_config_option('show_type_phone')) {
                                        	$values[] = $web_types[$webpage->getWebTypeId()]['name'].': '.$webpage->getUrl();
                                    	} else {
                                    		$values[] = $webpage->getUrl();
                                    	}
                                    }
                                    $value = implode($values,',');
                                }
                            }

                            if (substr($field, 8) == "all_emails"){
                                $all_emails = $contact->getNonMainEmails();
                                if (!empty($all_emails)){
                                    $values = array();
                                    $types = EmailTypes::getAllEmailTypesInfo(true);
                                    foreach ($all_emails as $email){
                                    	if (user_config_option('show_type_phone')) {
                                        	$values[] = $types[$email->getEmailTypeId()]['name'].': '.$email->getEmailAddress();
                                    	} else {
                                    		$values[] = $email->getEmailAddress();
                                    	}
                                    }
                                    $value = implode($values,',');
                                }
                            }

                        }

                    }

					if ($value instanceof DateTimeValue) {
						if ($model_instance) {
							$field_type = $model_instance->columnExists($field) ? $model_instance->getColumnType($field) : Objects::instance()->getColumnType($field);
							$value = format_value_to_print($field, $value->toMySQL(), $field_type, $report->getReportObjectTypeId());
						} else {
							$value = format_value_to_print($field, $value->toMySQL(), DATA_TYPE_DATETIME, $report->getReportObjectTypeId());
						}
					}
						
					if ($model_instance && in_array($field, $model_instance->getExternalColumns())){
						
						$value = Reports::instance()->getExternalColumnValue($field, $value, $model_instance, $member);
						
					} else {
						
						if ($member->getColumnType($field) == DATA_TYPE_STRING) {
							// change html block end tags and brs to \n, then remove all other html tags, then replace \n with <br>, to remove all styles and keep the enters
							$value = str_replace(array("</div>", "</p>", "<br>", "<br />", "<br/>"), "\n", $value);
							$value = nl2br(strip_tags($value));
						}
					}
					
					if(Reports::instance()->isReportColumnEmail($value)) {
						if(logged_user()->hasMailAccounts()){
							$value = '<a class="internalLink" href="'.get_url('mail', 'add_mail', array('to' => clean($value))).'">'.clean($value).'</a></div>';
						}else{
							$value = '<a class="internalLink" target="_self" href="mailto:'.clean($value).'">'.clean($value).'</a></div>';
						}
					}
					$row_values[$field] = $value;
					
				}
				else {
					if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
                        $field = $column->getFieldName();
                        if (str_starts_with($field,'group_cp_')){

                            $list_of_terms = explode('_',$field);
	                        if (isset($list_of_terms[5])){ // to support zip_code
	                        	$list_of_terms[4] .= '_'.$list_of_terms[5];
	                        }
	                        
                            $cpv = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(),$list_of_terms[2]);
                            if ($cpv instanceof CustomPropertyValue){
                                $contact_assc_id = $cpv->getValue();
                                $contact_assc = Contacts::findById($contact_assc_id);
                                if ($contact_assc instanceof Contact){
                                    $value = $contact_assc->getColumnValue($list_of_terms[3]);

                                    if ($list_of_terms[3] == "departament"){
                                        $all_addresses = ContactAddresses::findAll(array('conditions' => 'contact_id = '.$contact_assc->getId()));
                                        if (!empty($all_addresses)){
                                            $values = array();
                                            foreach ($all_addresses as $address){
                                                $values[] = $address->getCountry();
                                            }
                                            $value = implode($values,',');
                                        }
                                    }
                                    if ($list_of_terms[3] == "all"){

                                        if ($list_of_terms[4] == "emails"){
                                            $values = array();
                                            $main_email = $contact_assc->getMainEmails();
                                            if (!empty($main_email)){
                                                $types = EmailTypes::getAllEmailTypesInfo(true);
                                                foreach ($main_email as $email){
                                                	if (user_config_option('show_type_phone')) {
                                                    	$values[] = $types[$email->getEmailTypeId()]['name'].': '.$email->getEmailAddress();
                                                	} else {
                                                		$values[] = $email->getEmailAddress();
                                                	}
                                                }
                                                $value = implode($values,',');
                                            }
                                            $all_emails = $contact_assc->getNonMainEmails();
                                            if (!empty($all_emails)){
                                                $types = EmailTypes::getAllEmailTypesInfo(true);
                                                foreach ($all_emails as $email){
                                                	if (user_config_option('show_type_phone')) {
                                                    	$values[] = $types[$email->getEmailTypeId()]['name'].': '.$email->getEmailAddress();
                                                	} else {
                                                		$values[] = $email->getEmailAddress();
                                                	}
                                                }
                                                $value = implode($values,',');
                                            }
                                        }

                                        if ($list_of_terms[4] == "phones"){
                                            $phone_types = TelephoneTypes::getAllTelephoneTypesInfo(true);
                                            $all_phones = ContactTelephones::findAll(array('conditions' => 'contact_id = '.$contact_assc->getId()));
                                            if (!empty($all_phones)){
                                                $values = array();
                                                
                                                    foreach ($all_phones as $phone){
                                                        if (user_config_option('show_type_phone')) {
                                                            $values[] = $phone_types[$phone->getTelephoneTypeId()]['name'].': '.$phone->getNumber();
                                                        } else {
                                                            $values[] = $phone->getNumber();
                                                        }               
	                                                }
	                                            
                                                $value = implode($values,',');
                                            }
                                        }
                                        
                                        
                                        if (str_starts_with($list_of_terms[4], "addresses")) {
                                        
                                        	// full address
	                                        if ($list_of_terms[4] == "addresses"){
	                                            $address_types = AddressTypes::getAllAddressTypesInfo(true);
	                                            $all_addresses = ContactAddresses::findAll(array('conditions' => 'contact_id = '.$contact_assc->getId()));
	                                            if (!empty($all_addresses)){
	                                                $values = array();
	                                                foreach ($all_addresses as $address){
	                                                	if (user_config_option('show_type_phone')) {
	                                                    	$values[] = $address_types[$address->getAddressTypeId()]['name'].': '.$address->toString();
	                                                	} else {
	                                                		$values[] = $address->toString();
	                                                	}
	                                                }
	                                                $value = implode($values,',');
	                                            }
	                                        } else {
	                                        	
	                                        	// only one column of the address (example: key format = all_address|street)
	                                        	$exploded_key = explode('|', $list_of_terms[4]);
	                                        	if (count($exploded_key) == 2) {
	                                        		$address_column = $exploded_key[1]; // can be street, city, state, zip_code or country
	                                        		$all_addresses = ContactAddresses::findAll(array('conditions' => 'contact_id = '.$contact_assc->getId()));
	                                        		if (!empty($all_addresses)){
	                                        			$values = array();
	                                        			foreach ($all_addresses as $address){
	                                        				$address_column_value = $address->getColumnValue($address_column);
	                                        				if ($address_column == 'country') {
	                                        					$address_column_value = Countries::getCountryNameByCode(strtoupper($address_column_value));
	                                        				}
	                                        				$values[] = $address_column_value;
	                                        			}
	                                        			$value = implode($values,',');
	                                        		}
	                                        	}
	                                        }
                                        }
                                        
                                        if ($list_of_terms[4] == "webpages"){
                                            $web_types = WebpageTypes::getAllWebpageTypesInfo(true);
                                            $all_webpages = ContactWebpages::findAll(array('conditions' => 'contact_id = '.$contact_assc->getId()));
                                            if (!empty($all_webpages)){
                                                $values = array();
                                                foreach ($all_webpages as $webpage){
                                                	if (user_config_option('show_type_phone')) {
                                                    	$values[] = $web_types[$webpage->getWebTypeId()]['name'].': '.$webpage->getUrl();
                                                	} else {
                                                		$values[] = $webpage->getUrl();
                                                	}
                                                }
                                                $value = implode($values,',');
                                            }
                                        }

                                    }



                                }
                            }

                            $row_values[$field] = $value;
                        } else {

                            $cp = array_var($custom_properties, $column->getCustomPropertyId());
                            if (!$cp) {
                                if ($ot->getType() == 'dimension_object') {
                                    $cp = CustomProperties::getCustomProperty($column->getCustomPropertyId());
                                } else {
                                    $cp = MemberCustomProperties::getCustomProperty($column->getCustomPropertyId());
                                }
                                $custom_properties[$cp->getId()] = $cp;
                            }

                            if ($cp instanceof MemberCustomProperty) {
                                $row_values[$cp->getId()] = get_member_custom_property_value_for_listing($cp, $member->getId());
                            } else if ($cp instanceof CustomProperty) {

                                $value = get_custom_property_value_for_listing($cp, $member->getObjectId());
                                if ($value == "") {

                                    $customer = Customers::findById($member->getObjectId());
                                    if ($customer instanceof Customer) {
                                        $contact = $customer->getContact();
                                        $value = get_custom_property_value_for_listing($cp, $contact->getObjectId());
                                    }
                                }
                                $row_values[$cp->getId()] = $value;
                            }
                        }
					}
				}
			}
			$row_values['link'] = '<span class="link-ico '.$icon_class.'"></span>';
			
			$report_rows[$member->getId()] = $row_values;
		}
		
		// Remove this when print and export is re-implemented for member reports
		tpl_assign('allow_export', true);
		tpl_assign('disable_print', false);
		// ------------------
		
		if (!$to_print) {
			if (is_array($results['columns'])) {
				$results['columns']['names']['link'] = '';
				array_unshift($results['columns']['order'], 'link');
			} else {
				$results['columns']['names'] = array('');
				$results['columns']['order'] = array('link');
			}
			Hook::fire("report_header", $ot, $results['columns']);
		} else {
			if (is_array($results['columns'])) {
				$results['columns']['names']['link'] = '';
				array_unshift($results['columns']['order'], 'link');
			}
		}
		$results['rows'] = $report_rows;
		
		$results['group_by_criterias'] = $group_by_criterias;
		$results['grouped_rows'] = $grouped_rows;
		$results['group_totals'] = $group_totals;

	}
	
	return $results;
	
}


function member_custom_reports_additional_report_print_css($ignored, &$css_html) {
	
	$css_html .= stylesheet_tag(ROOT_URL."/plugins/crpm/public/assets/css/crpm.css");
	
}