<?php
Hook::register("tasks_repetition");

function tasks_repetition_object_definition($type, &$ret) {
	if ($type == 'ProjectTask' || $type == 'TemplateTask') {
		$ret['repeat_generate_instances'] = DATA_TYPE_BOOLEAN;
		$ret['repeat_fixed_days'] = DATA_TYPE_STRING;
	}
}


function tasks_repetition_additional_system_columns($manager, &$columns) {
	if ($manager instanceof ProjectTasks || $manager instanceof TemplateTasks) {
		$columns[] = 'repeat_generate_instances';
		$columns[] = 'repeat_fixed_days';
	}
}


function tasks_repetition_task_clone_more_attributes($parameters, &$ignored) {
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');

	if (!is_null($copy) && !is_null($original)) {
		$copy->setColumnValue('repeat_generate_instances', $original->getColumnValue('repeat_generate_instances'));
		$copy->setColumnValue('repeat_fixed_days', $original->getColumnValue('repeat_fixed_days'));
	}
}

function tasks_repetition_template_task_to_task_columns($parameters, &$new_task) {

	$temp_task = array_var($parameters, 'template_task');
	if ($temp_task instanceof TemplateTask && $new_task instanceof ProjectTask) {

		$new_task->setColumnValue('repeat_generate_instances', $temp_task->getColumnValue('repeat_generate_instances'));
		$new_task->setColumnValue('repeat_fixed_days', $temp_task->getColumnValue('repeat_fixed_days'));
	}
}


function tasks_repetition_additional_cron_function_files($ignored, &$files) {
	$files[] = ROOT . "/plugins/crpm/application/cron_functions.php";
}




function tasks_repetition_check_working_days_to_correct_repetition($params, &$correct_the_days) {
	// Ignore the config option of working days if task defines the set of fixed days to repeat.
	$task = array_var($params, 'task');
	if ($task->isRepetitive()) {
		$rep_fixed_days_str = trim($task->getColumnValue('repeat_fixed_days'));
		if ($rep_fixed_days_str != '') {
			$correct_the_days = false;
		}
	}
}


function tasks_repetition_render_page_actions($task, &$ret) {
	if ($task instanceof ProjectTask && !$task->isTrashed() && $task->isRepetitive()) {
		
		$label = lang('generate next x days repetitions short', config_option('days range to replicate tasks'));
		
		add_page_action($label, get_url("crpm", "generate_first_repetitions_for_task", array("id" => $task->getId())), 'ico-recurrent', null, null, true);
		
	}
}
// generate first batch of repetitions
function tasks_repetition_after_task_controller_add_task($params, &$ignored) {
	Env::useHelper('functions', 'crpm');

	$task = array_var($params, 'task');
	if ($task instanceof ProjectTask && $task->isRepetitive()) {
		// generate first batch of repetitions
		try {
			DB::beginWork();
			$count = generate_first_repetitions_for_task($task);
			if($count > 0) {
				evt_add('reload tab panel', 'tasks-panel');
			}

			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			Logger::log("ERROR: Generating task repetitions for task: ".$task->getId()." message:".$e->getMessage());
			Logger::log($e->getTraceAsString());
		}
	}

	if (Plugins::instance()->isActivePlugin("advanced_services")) {
		// if task date is in a non-working day then classify it in the holiday service member of service types dimension
		if ($task instanceof ProjectTask) {
			Env::useHelper('functions', 'advanced_services');
			check_and_classify_task_in_holiday_service_member($task);
		}
	}
}

function tasks_repetition_after_template_object_instantiation_and_commit($parameters, &$ignored) {
	$object = array_var($parameters, 'object');

	if ($object instanceof ProjectTask && $object->isRepetitive()) {
		Env::useHelper('functions', 'crpm');
		// generate first batch of repetitions
		$count = generate_first_repetitions_for_task($object);
	}
}




function tasks_repetition_check_valid_repetition_date_days_add($params, &$days) {
	$task = array_var($params, 'task'); /* @var $task ProjectTask */
	$new_st_date = array_var($params, 'new_st_date');
	$new_due_date = array_var($params, 'new_due_date');
	$move_direction = array_var($params, 'move_direction');

	// This condition caused bugs: if ($task->isRepetitive()) {
		// Returns which days of the week the start date or due date can be created in
		$rep_fixed_days_str = trim($task->getColumnValue('repeat_fixed_days'));

		if ($rep_fixed_days_str != '') {
			$rep_fixed_days = explode(',', $rep_fixed_days_str);
			$dtmp = $task->getRepeatBy() == 'due_date' ? $new_due_date : $new_st_date;
				
			if ($dtmp instanceof DateTimeValue && count($rep_fixed_days)>0) {
				$d = new DateTimeValue($dtmp->getTimestamp());
				
				// This line was the culprit: 
				$d->add('s', $task->getTimezoneValue());
				$dw = $d->format('w');
				
				// Instantiate Non working days
				$non_working_days = new NonWorkingDays();
				
				// Check if day of the week falls in desirable day of the week
				// OR
				// if the day is falling on the holiday day, 
				// then add or subtract (depends on $move_direction) one day if any of the condition is true

				if(isset($move_direction)){
					if($move_direction == "move_back"){
						while (!in_array($dw, $rep_fixed_days) || $non_working_days->isNonWorkingDay($d)) {
							$days--;
							$d->add('d', -1);
							$dw = $d->format('w');
						}
					} else {
						while (!in_array($dw, $rep_fixed_days) || $non_working_days->isNonWorkingDay($d)) {
							$days++;
							$d->add('d', 1);
							$dw = $d->format('w');
						}
					}

				} else {
					while (!in_array($dw, $rep_fixed_days) || $non_working_days->isNonWorkingDay($d)) {
						$days++;
						$d->add('d', 1);
						$dw = $d->format('w');
					}
				}
			}
		}
	// End of condition that caused bug: }
}


function tasks_repetition_form_repeat_by_more_checkboxes($args, &$html) {
	$object = array_var($args, 'object');
	if ($object instanceof ProjectTask || $object instanceof TemplateTask) {
		$can_manage_repetitive_properties_of_tasks = SystemPermissions::userHasSystemPermission(logged_user(), 'can_manage_repetitive_properties_of_tasks');

		if ($can_manage_repetitive_properties_of_tasks){

			if(!$object->isNew()){
				$last_related_task_id = ProjectTasks::getLastRepetitiveTaskId($object->getId());

				if ($last_related_task_id > 0 && $last_related_task_id != $object->getId() ){
					$last_repetitive_task = ProjectTasks::instance()->findById($last_related_task_id);
					$object = $last_repetitive_task;
				}
			}
		}


/*
		// instance generation checkbox
		$checked = '';
		if ($object->isNew() || $object->getColumnValue('repeat_generate_instances')) {
			$checked = 'checked="checked"';
		}
		$label = lang('generate next x days repetitions', config_option('days range to replicate tasks'));
		$html .= '<tr><td><input class="checkbox" type="checkbox" '.$checked.' name="task[repeat_generate_instances]" /> '. $label .'</td></tr>';
*/

		// repeat only in fixed days selector
		$days = array('sunday','monday','tuesday','wednesday','thursday','friday','saturday');
		$sel_days = array();
		$sel_days_str = $object->isNew() ? config_option('working_days') : $object->getColumnValue('repeat_fixed_days');
		if (trim($sel_days_str) != '') {
			$sel_days = explode(',', $sel_days_str);
		}

		$genid = gen_id();
		$fixed_days_checked = count($sel_days) > 0 ? 'checked="checked"' : '';
		$label = lang("repeat only in fixed days");

		$html .= '<tr><td><input class="checkbox" type="checkbox" '.$fixed_days_checked.' name="task[repeat_fixed_days_enabled]"';
		$html .= ' onchange="og.crpm.onRepeatFixedDaysEnable(\''.$genid.'rep-days-selector\', this);" /> ' . $label;
		$html .= '<div class="rep-days-selector" id="'.$genid.'rep-days-selector" style="'.($fixed_days_checked ? '' : 'display:none;').'">';

		foreach ($days as $k => $d) {
			$checked_day = in_array($k, $sel_days) ? 'checked="checked"' : '';
			$html .= '<span class="rep-day">';
			$html .= '<input class="checkbox" type="checkbox" '.$checked_day.' name="task[repeat_fixed_day]['.$k.']" /> ';
			$html .= lang($d);
			$html .= '</span>';
		}
		$html .= '</td></tr>';
	}
}


function tasks_repetition_before_object_save($object, &$ret) {
	if ($object instanceof ProjectTask || $object instanceof TemplateTask) {
		$can_manage_repetitive_properties_of_tasks = SystemPermissions::userHasSystemPermission(logged_user(), 'can_manage_repetitive_properties_of_tasks');

		if ($can_manage_repetitive_properties_of_tasks){
			$last_related_task_id = ProjectTasks::getLastRepetitiveTaskId($object->getId());

			if ($last_related_task_id > 0 && $last_related_task_id != $object->getId() ){
				$last_repetitive_task = ProjectTasks::instance()->findById($last_related_task_id);
				$object = $last_repetitive_task;
			}
		}
		$data = array_var($_REQUEST, 'task');
		if ($can_manage_repetitive_properties_of_tasks && is_array($data)) {
			if (array_var($data, 'repeat_fixed_days_enabled')) {
					
				$repeat_fixed_days = array_var($data, 'repeat_fixed_day', array());
				$repeat_fixed_days_str = '';

				$days_array = array();
				if (is_array($repeat_fixed_days)) {
					foreach ($repeat_fixed_days as $fday => $selected) {
						if ($selected) $days_array[] = $fday;
					}
				}
				if (count($days_array) > 0) {
					$repeat_fixed_days_str = implode(',', $days_array);
				}

				$object->setColumnValue('repeat_fixed_days', $repeat_fixed_days_str);
					
			} else {
				$object->setColumnValue('repeat_fixed_days', '');
			}
			
			$object->setColumnValue('repeat_generate_instances', config_option('generate repetitive instances'));
		}
	}
}

