<?php
/**
 * Event Ticket Types controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class EventTicketTypeController extends ApplicationController {

	var $plugin_name = "event_tickets";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('format');
	}
	
	
	function add() {

		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$event_ticket_type_data = array_var($_POST, 'event_ticket_type');
		
		if(!is_array($event_ticket_type_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new EventTicketType();
			$obj->setEventId(get_id('event_id'));
			
			// init some of the ticket types with member custom property values
			$event = ManagedEvents::findById(get_id('event_id'));
			if ($event instanceof ManagedEvent) {
				$ev_dim = Dimensions::findByCode('managed_events');
				$ev_ot = ObjectTypes::findByName('managed_event');
				$ev_member = Members::findOneByObjectId($event->getId(), $ev_dim->getId());
				
				if ($ev_member instanceof Member) {
					$codes = array('available_from', 'available_to');
					foreach ($codes as $code) {
						$cp = CustomProperties::getCustomPropertyByCode($ev_ot->getId(), $code);
						if ($cp instanceof CustomProperty) {
							$cp_val = CustomPropertyValues::getCustomPropertyValue($ev_member->getObjectId(), $cp->getId());
							if ($cp_val instanceof CustomPropertyValue && $cp_val->getValue()) {
								$obj->setColumnValue($code, $cp_val->getValue());
							}
						}
					}
				}
			}
			tpl_assign('event_ticket_type', $obj);
			tpl_assign('event', $event);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
		
				$event_ticket_type = $this->save_event_ticket_type($event_ticket_type_data);
		
				DB::commit();
				
				flash_success(lang('success add event_ticket_type', $event_ticket_type->getName()));
		
				ajx_current("back");
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
		
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
		
	}
	
	
	
	function edit() {
		$this->setTemplate('add');
		
		$event_ticket_type = EventTicketTypes::getEventTicketById(get_id());
		if(!($event_ticket_type instanceof EventTicketType)) {
			flash_error(lang('event_ticket_type dnx'));
			ajx_current("empty");
			return;
		}
		
		$managed_event = ManagedEvents::findById($event_ticket_type->getEventId());
		
		if (logged_user()->isGuest() || !$managed_event->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$event_ticket_type_data = array_var($_POST, 'event_ticket_type');
		
		if(!is_array($event_ticket_type_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('event_ticket_type', $event_ticket_type);
			tpl_assign('event', $managed_event);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$event_ticket_type = $this->save_event_ticket_type($event_ticket_type_data, $event_ticket_type);
				
				DB::commit();

				flash_success(lang('success edit event_ticket_type', $event_ticket_type->getName()));
				
				ajx_current("back");
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	
	function save_event_ticket_type($event_ticket_type_data, $event_ticket_type = null) {
		// Save event_ticket_type object
		$is_new = false;
		if (is_null($event_ticket_type) || !$event_ticket_type instanceof EventTicketType) {
			$event_ticket_type = new EventTicketType();
			$is_new = true;
		}
		
		$event_ticket_type_data['available_from'] = getDateValue($event_ticket_type_data['available_from']);
		$event_ticket_type_data['available_to'] = getDateValue($event_ticket_type_data['available_to']);
		
		if (!$event_ticket_type_data['currency_id']) {
			$default_currency = Currencies::getDefaultCurrencyInfo();
			if ($default_currency) {
				$event_ticket_type_data['currency_id'] = $default_currency['id'];
			}
		}
	
		$event_ticket_type->setFromAttributes($event_ticket_type_data);
		$event_ticket_type->save();
	
		return $event_ticket_type;
	}

    function save_event_ticket_cp_bulk($params, $member_ids)
    {

        if (isset($params) && is_array($params) && count(array_var($params, 'event_tickets_cp', array())) > 0) {
            $response_data = array();
            try {
                // filter non numeric member ids
                $member_ids = array_filter($member_ids, "is_numeric");

                foreach ($params['event_tickets_cp'] as $attendee) {
                    $event_ticket_data = array(
                        "event_ticket_id" => array_var($attendee, 'event_ticket_id', 0),
                        "contact_email_address" => array_var($attendee, 'contact_email_address', ''),
                        "contact_first_name" => array_var($attendee, 'contact_first_name', ''),
                        "contact_surname" => array_var($attendee, 'contact_surname', ''),
                        "contact_id" => array_var($attendee, 'contact_id', null),
                        "from_web" => array_var($attendee, 'from_web', null)
                    );
                    $attendee_info = $this->save_event_attendee($event_ticket_data, $member_ids);
                    if ($attendee_info instanceof EventAttendee) {
                        $response_data[] = $attendee_info->getArrayInfo();
                    }

                }

                return $response_data;

            } catch (Exception $e) {
                Logger::log("Error saving event attendee: " . $e->getMessage() . "\n" . $e->getTraceAsString());
                throw $e;
            }
        }

        return array();
    }
	
	
}