<?php


function render_event_ticket_status_selector($input_name, $selected, $attributes=null) {

	$all_status = array("pending", "paid", "canceled");
	
	$options = array();
	foreach ($all_status as $status) {
		$attr = $status == $selected ? array("selected" => "selected") : null;
		
		$status_name = lang("event ticket status $status");
		$options[] = option_tag($status_name, $status, $attr);
	}
	
	return select_box($input_name, $options, $attributes);
}


function recalculate_ticket_types_sold_quantity($ticket_types) {
	
	foreach ($ticket_types as $tt) {/* @var $tt EventTicketType */
		if ($tt instanceof EventTicketType) {
			$sold_amount = EventTickets::instance()->count(array("trashed_by_id=0 AND ticket_type_id = ?", $tt->getId()));
			
			$tt->setSoldQuantity($sold_amount);
			$tt->save();
		}
	}
}


function get_member_ticket_types_tab_html($genid, $member) {
	ob_start();
	render_member_ticket_types_tab($genid, $member);
	return ob_get_clean();
}


function render_member_ticket_types_tab($genid, $member){

	if ($member instanceof Member) {
		
		if (!$member->isNew()) {
			$ticket_types = EventTicketTypes::findAllByEventId($member->getObjectId());
		} else {
			$ticket_types = array();
		}

		tpl_assign('member', $member);
		tpl_assign('ticket_types', $ticket_types);
		
		echo '<div id="'. $genid .'tickets_types" class="form-tab tickets-tab-content">';

		include get_template_path("tickets_types_list", "event_ticket_type", "event_tickets");

		echo '</div>';
	}

}

function get_member_event_tickets_tab_html($genid, $member) {
    ob_start();
	render_member_event_tickets_tab($genid, $member);
    return ob_get_clean();
}


function render_member_event_tickets_tab($genid, $member){

	if ($member instanceof Member) {

		if (!$member->isNew()) {
			$event_tickets = EventTickets::findAllByEventId($member->getObjectId());
		} else {
			$event_tickets = array();
		}

		tpl_assign('member', $member);
		tpl_assign('event_tickets', $event_tickets);
		
		echo '<div id="'. $genid .'event_tickets" class="form-tab">';

		include get_template_path("event_tickets_list", "event_ticket", "event_tickets");

		echo '</div>';
	}

}

/**
 * Creates the empty attendees for the recently saved EventTicket
 * @param EventTicket $event_ticket
 * @param integer $amount_of_attendees_to_create
 * @throws Exception
 * @return array with the new attendee objects
 */
function create_empty_attendees_for_ticket(EventTicket $event_ticket) {
	// get the current attendee count for this ticket
	$current_attendees_count = EventAttendees::instance()->count(array(
			"event_ticket_id = ?", $event_ticket->getId()
	));
	
	// get the amount of attendees per ticket
	$tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());
	$attendees_per_ticket = $tt instanceof EventTicketType ? $tt->getAttendeesPerTicket() : 1;
	
	$total_attendees_count = $attendees_per_ticket;
	
	// calculate the amount of tickets to create
	$attendees_to_create = $total_attendees_count - $current_attendees_count;
	
	$new_attendees = array();
	
	try {
		DB::beginWork();
		
		// create the attendees
		$controller = new EventAttendeeController();
		while ($attendees_to_create > 0) {
			$data = array("event_ticket_id" => $event_ticket->getId());
			
			$new_attendees[] = $controller->save_event_attendee($data, $event_ticket->getMemberIds());
			
			$attendees_to_create--;
		}
		
		DB::commit();
		
	} catch (Exception $e) {
		DB::rollback();
		Logger::log_r("ERROR creating event ticket attendees\n".$e->getMessage()."\n".$e->getTraceAsString());
		throw $e;
	}
	
	return $new_attendees;
}



function create_contact_from_ticket_data($data, $member_ids = array()) {
	$attendee_data = array(
			'first_name' => trim($data['contact_first_name']),
			'surname' => trim($data['contact_surname']),
			'email' => trim($data['contact_email_address']),
	);
	
	if (trim($attendee_data['first_name']." ".$attendee_data['surname']) == "") {
		return;
	}
	
	$members_array = array();
	$tt = EventTicketTypes::getEventTicketById($data['ticket_type_id']);
	if ($tt instanceof EventTicketType) {
		$event = ManagedEvents::findById($tt->getEventId());
		if ($event instanceof ManagedEvent) {
			$event_member = Members::findOneByObjectId($event->getId());
			if ($event_member instanceof Member) {
				$members_array[] = $event_member;
			}
		}
	}
	
	if (count($members_array) > 0) {
		foreach ($members_array as $m) {
			if ($m instanceof Member) $members_ids[] = $m->getId();
		}
	}
	
	return create_contact_from_data($attendee_data, $members_ids);
	
}


function tickets_attendee_report_result_to_arrays($result) {
    $array_rows = array();
    $ts_ot = ObjectTypes::findByName('event_ticket');
    $grouped_event_attendee = array_var($result, 'grouped_event_attendee');
    $report_data = array_var($result, 'report_data');

    $table_columns = array();
    $columns_type = $report_data['columns_type'];
    foreach ($report_data['columns'] as $col => $order) {
        if ($order > 0) {
            $col_lang = lang('field Objects '.$col);
            $col_data = array('id' => $col, 'name' => $col_lang);
            $table_columns[$order] = $col_data;
        }
    }
    ksort($table_columns, SORT_NUMERIC);

    $header_row = array();
    foreach ($table_columns as $col) {
        $header_row[] = $col['name'];
    }
    $array_rows[] = $header_row;

    // first level groups
    foreach ($grouped_event_attendee as $first_group_id => $second_level_groups) {

        $first_total = array();

        // second level groups
        foreach ($second_level_groups as $second_group_id => $tickets) {

            $row = array();
            foreach ($table_columns as $col) {
                $field = $col['id'];
                if (isset($tickets[$field])){
                    $type = $columns_type[$field];
                    //$row[] = format_value_to_print($field, $tickets[$field], $type, $ts_ot->getId());
                    $row[] = $tickets[$field];
                }else{
                    $row[] = ' ';
                }
            }
            $array_rows[] = $row;

        }


    }


    return $array_rows;
}

function build_event_report_conditions_html_main($parameters=array()) {
    $conditionHtml = "";
    if (!empty($parameters)){


        $dim = Dimensions::findByCode('managed_events');
        $dim_name = $dim->getName();

        $paramValue = array_var($parameters, 0, '');
        $paramValue = str_replace("[","",$paramValue);
        $paramValue = str_replace("]","",$paramValue);
            if ($paramValue){
                $member = Members::findById($paramValue);
                if ($member instanceof Member){
                    $memberName = $member->getName();
                }
            }
            $cond_html = null;

            $already_rendered = false;

            $conditionHtml .= '<div class="custom-report-info-block">
	                            <div class="bold">'.lang('condition').':</div>
	                            <ul><li>'. $dim_name .': '.lang('classified in') .' '.$memberName.'</li></ul>
                               </div>';


    }
    return $conditionHtml;
}

function build_event_report_conditions_plain_main($parameters=array()) {

    $conditionHtml = "";
    if (!empty($parameters)){

        $dim = Dimensions::findByCode('managed_events');
        $dim_name = $dim->getName();

        $paramValue = array_var($parameters, 0, '');
        $paramValue = str_replace("[","",$paramValue);
        $paramValue = str_replace("]","",$paramValue);
        if ($paramValue){
            $member = Members::findById($paramValue);
            if ($member instanceof Member){
                $memberName = $member->getName();
            }
        }
        $cond_html = null;

        $conditionHtml .= lang('condition').': '. $dim_name .': '.lang('classified in') .' '.$memberName;

    }
    return $conditionHtml;
}

