<?php
	class EventTicket extends BaseEventTicket {
		
		
		function save() {
			$ret = parent::save();
			
			$tt = EventTicketTypes::getEventTicketById($this->getTicketTypeId());
			if ($tt instanceof EventTicketType) {
				Env::useHelper('functions', 'event_tickets');
				recalculate_ticket_types_sold_quantity(array($tt));
			}
			
			return $ret;
		}
		
		
		function getArrayInfo($include_attendees_data=false, $include_ticket_type_data=false) {
			$info = array();
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_datetime($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			$currency = Currencies::getCurrency($this->getCurrencyId());
			$info['currency_sym'] = $currency instanceof Currency ? $currency->getSymbol() : '$';
			
			if ($include_attendees_data) {
				$attendees_data = array();
				$attendee_contacts = Contacts::findAll(array("conditions" => "id IN (
					SELECT a.contact_id FROM ".TABLE_PREFIX."event_attendees a
					INNER JOIN ".TABLE_PREFIX."objects ao ON ao.id=a.object_id
					WHERE a.event_ticket_id=".$this->getId()." AND ao.trashed_by_id=0
				)"));
                $custom_properties_contact= CustomProperties::getAllCustomPropertiesByObjectType(Contacts::instance()->getObjectTypeId());
				foreach ($attendee_contacts as $c) {/* @var $c Contact */
					$attendees_data[] = array(
                        'name' => $c->getObjectId(),
						'name' => $c->getObjectName(),
						'email' => $c->getEmailAddress(),
					);

                    foreach ($custom_properties_contact as $cp) {
                        $attendees_data[0]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $c);
                    }
				}
				$info['attendees'] = $attendees_data;

			}
			
			if ($include_ticket_type_data) {
				$tt = EventTicketTypes::getEventTicketById($this->getTicketTypeId());
				if ($tt instanceof EventTicketType) {
					$info['ticket_type_object'] = $tt->getArrayInfo();
				}
			}
			
			return $info;
		}
		
		
		function getRequestedByName() {
			$contact = Contacts::findById($this->getRequestedById());
			return $contact instanceof Contact ? $contact->getObjectName() : "";
		}
		
		
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, EventTickets::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
		
		
		function validate($errors) {
			if(!$this->validatePresenceOf('ticket_type_id')) {
				$errors[] = lang('ticket type required');
			} // if
		}
	}