<?php

Hook::register("event_tickets");

function event_tickets_object_edit_categories_extra_data($params, &$categories) {
    $object = $params['object'];
    if ($object instanceof SalesPurchaseOrder) {
        Env::useHelper('functions', 'event_tickets');
        $event_member =  Members::findById($object->getRelatedObjectId());
        $categories[] = array(
            'id' => 'event_tickets_info',
            'name' => lang('event tickets'),
            'required' => false,
            'visible' => true,
            'content' => get_member_event_tickets_tab_html($params['genid'],$event_member),
        );
    }
}



function event_tickets_after_object_save($object, &$ret) {

	if ($object instanceof IncomeInvoice) {
		if (in_array($object->getStatus(), array('pending','canceled','confirmed'))) {
			$ev_ticket_status = null;
			switch ($object->getStatus()) {
				case 'pending': $ev_ticket_status = 'pending'; break;
				case 'canceled': $ev_ticket_status = 'canceled'; break;
				case 'confirmed': $ev_ticket_status = 'paid'; break;
			}
			if ($ev_ticket_status) {
				DB::execute("UPDATE ".TABLE_PREFIX."event_tickets SET `status`='$ev_ticket_status' WHERE invoice_id=".$object->getId());
			}
		}
	}
}

function event_tickets_after_member_save($params, &$ret) {
	$member = array_var($params, 'member');
	$previous_data = array_var($params, 'previous_data');
	$is_new = array_var($params, 'is_new');
	
	// ticket types
	
	$del_ticket_type_ids_csv = array_var($_REQUEST, 'del_ticket_types');
	$del_ticket_type_ids = array_filter(explode(',', $del_ticket_type_ids_csv));
	if (count($del_ticket_type_ids) > 0) {
		EventTicketTypes::delete("id IN (".implode(',', $del_ticket_type_ids).")");
	}
	
	$ticket_types_data = array_var($_REQUEST, 'ticket_types');
	
	if (is_array($ticket_types_data) && count($ticket_types_data) > 0) {
		$controller = new EventTicketTypeController();
		
		foreach ($ticket_types_data as $idx => $data) {
			if (is_numeric($idx)) {
				$data['event_id'] = $member->getObjectId();
				$object = EventTicketTypes::getEventTicketById(array_var($data, 'id'));
				
				$controller->save_event_ticket_type($data, $object);
			}
		}
	}
}



function event_tickets_member_edit_categories($params, &$categories) {
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$ot_id = ObjectTypes::findByName('managed_event')->getId();
	
	if ($member instanceof Member && $member->getObjectTypeId() == $ot_id) {
		Env::useHelper('functions', 'event_tickets');
		
		$categories[] = array(
			'id' => $genid . 'tickets_types',
			'link_id' => $genid . 'tickets_types_tab',
			'name' => lang('ticket types'),
			'content' => get_member_ticket_types_tab_html($genid, $member),
			'class' => 'ticket-types-container',
		);
	}
}




function event_tickets_additional_member_definition_columns($ot_id, &$ext_cols) {
	$ot = ObjectTypes::findById($ot_id);

	if ($ot instanceof ObjectType && $ot->getName() == 'managed_event') {
		$ext_cols['ticket_types'] = array(
				'id' => 'ticket_types',
				'type' => 'array',
				'label' => lang('ticket types'),
		);
	}
}



function event_tickets_additional_member_column_values($params, &$info) {
	
	$definition = array_var($params, 'definition');
	$member = array_var($params, 'member');
	
	$ot = ObjectTypes::findById($member->getObjectTypeId());
	
	if ($ot instanceof ObjectType && $ot->getName() == 'managed_event') {
		
		$info['ticket_types'] = array();
		$ticket_types = EventTicketTypes::findAllByEventId($member->getObjectId());
		
		foreach ($ticket_types as $ticket_type) {
			$tt_data = $ticket_type->getArrayInfo(true);
			$info['ticket_types'][] = $tt_data;
		}
	}
	
}


/**
 * When cloning a managed event -> also clone the associated event ticket types
 * @param array $params
 * @param ContentDataObject $copy
 */
function event_tickets_after_content_data_object_copy($params, &$copy) {
	$object = array_var($params, 'object');

	if ($object instanceof ManagedEvent && $copy instanceof ManagedEvent) {
		$sql = "
			INSERT INTO ".TABLE_PREFIX."event_ticket_types (event_id,name,description,show_description_in_website,currency_id,amount,available_from,available_to,total_quantity,sold_quantity,attendees_per_ticket)
			SELECT ".$copy->getId().",name,description,show_description_in_website,currency_id,amount,available_from,available_to,total_quantity,0,attendees_per_ticket
			FROM ".TABLE_PREFIX."event_ticket_types tt
			WHERE tt.event_id=".$object->getId()."
		";
		DB::execute($sql);
	}
}



/**
 * Add the copy member button to the active context widget
 * @param Member $member
 * @param string $edit_onclick
 */
function event_tickets_more_active_context_widget_buttons($params, &$buttons_html) {
	$member = array_var($params, 'member');
	if (!$member instanceof Member) return;

	$object_type = ObjectTypes::findById($member->getObjectTypeId());

	if ($object_type->getName() == 'managed_event') {
		
		// disable classes registration, use this feature only for events workflow
		if (config_option('use_classes_workflow')) {
			return;
		}
		
		$onclick = "og.render_modal_form('', {c:'event_ticket', a:'add_multiple'});";
		$title = lang('new event registration');
		$label = lang('new event registration');

		$buttons_html .= '
			<button class="btn" style="margin-left:10px;" onclick="'. $onclick .'" title="'. $title .'">
                <img alt="" style="height: 15px" src="public/assets/themes/default/images/icons-feng-3/16x16/new_color.png">
                '. $label .'
            </button>
		';
	}
}




function event_tickets_override_autogenerated_invoice_name($params, &$invoice_name) {
	$invoice_data = array_var($params, 'invoice_data');
	$customer_name = array_var($params, 'customer_name');
	$member_ids = array_var($params, 'member_ids');

	$event_dim = Dimensions::findByCode('managed_events');
	$event_object_type = ObjectTypes::findByName('managed_event');

	if ($event_dim instanceof Dimension && $event_object_type instanceof ObjectType) {

		$event_count = 0;
		$event_name = "";
		foreach ($member_ids as $member_id) {
			$member = Members::findById($member_id);
			if ($member instanceof Member && $member->getObjectTypeId() == $event_object_type->getId()) {
				$event_name = $member->getName();
				$event_count++;
			}
		}

		if ($event_count == 1) {
			$invoice_name = lang('event invoice for', $event_name, $customer_name, format_date($invoice_data['date'], null, 0));
		} else if ($event_count > 1) {
			$multiple_events_lang = config_option('use_classes_workflow') ? lang('classes') : lang('events');
			$invoice_name = lang('event invoice for', $multiple_events_lang, $customer_name, format_date($invoice_data['date'], null, 0));
		}
	}


}

function event_tickets_additional_search_for_invoice_customer_member($params, &$customer_member) {

	$invoice_data = array_var($params, 'invoice_data');
	$ordered_by = array_var($params, 'ordered_by');

	// if no client found and config option is set to create the client then create it from the ordered by contact
	if (!$customer_member instanceof Member
			&& array_var($invoice_data, 'from_web')
			&& $ordered_by instanceof Contact
			&& config_option('create_member_in_web_registration')) {

				try {
						
					Env::useHelper('functions', 'crpm');
					$customer_member = create_client_member_by_email_and_name(array(
							'customer_name' => $ordered_by->getObjectName(),
							'customer_email_address' => $ordered_by->getEmailAddress(),
					));
						
					// associate new customer to the web stage configured
					if ($customer_member instanceof Member) {
						$web_classes_stage_member_id = config_option('web_classes_stage_member');
						if ($web_classes_stage_member_id > 0) {
							$stages_dim = Dimensions::findByCode("customer_stages");
							$stages_ot = ObjectTypes::findByName("customer_stage");
							$a = DimensionMemberAssociations::instance()->getAssociationObject($customer_member->getDimensionId(), $customer_member->getObjectTypeId(), $stages_dim->getId(), $stages_ot->getId());
							if ($a instanceof DimensionMemberAssociation) {
								$mpm = new MemberPropertyMember();
								$mpm->setAssociationId($a->getId());
								$mpm->setMemberId($customer_member->getId());
								$mpm->setPropertyMemberId($web_classes_stage_member_id);
								$mpm->setIsActive(true);
								$mpm->setCreatedById(logged_user()->getId());
								$mpm->setCreatedOn(DateTimeValueLib::now());
								$mpm->save();
							}
								
							// classify ordered by in web stage member
							$ordered_by_mem_ids = array($web_classes_stage_member_id);
							foreach ($ordered_by->getMembers() as $member){
								$ordered_by_mem_ids[] = $member->getId();
							}
							$object_controller = new ObjectController();
							$object_controller->add_to_members($ordered_by, $ordered_by_mem_ids);
						}
					}
				}catch (Exception $e) {
					Logger::log_r($e->getMessage());
					Logger::log_r($e->getTraceAsString());
				}
			}
}

/**
 * Classify event invoices in account member
 */
function event_tickets_after_add_to_members($object, &$validMembers) {
	
	if (Plugins::instance()->isActivePlugin('income') && $object instanceof IncomeInvoice) {

		// Check if invoice is classified in a managed_event and classify in the account member
		$managed_event_ot = ObjectTypes::findByName('managed_event');
		$event_member = null;
		$is_in_event = false;
		foreach ($validMembers as $mem) {
			if ($mem instanceof Member && $mem->getObjectTypeId() == $managed_event_ot->getId()) {
				$event_member = $mem;
				$is_in_event = true;
				break;
			}
		}
		
		// if is an event invoice, then inherit the account from the event
		// if event doesn't have any accounts then use the one defined in the config option "event_invoice_account_id"
		if ($is_in_event) {
			$account_member = null;
			
			// get the event's account member
			$financial_accounts_dim = Dimensions::findByCode('financial_accounts');
			$financial_accounts_ot = ObjectTypes::findByName('financial_account');
			if ($financial_accounts_dim instanceof Dimension && $financial_accounts_ot instanceof ObjectType) {
				// get the association with accounts
				$a = DimensionMemberAssociations::instance()->getAssociationObject($event_member->getDimensionId(), $event_member->getObjectTypeId(), $financial_accounts_dim->getId(), $financial_accounts_ot->getId());
				if ($a instanceof DimensionMemberAssociation) {
					// get the associated member
					$mem_ids_csv = MemberPropertyMembers::getAllPropertyMemberIds($a->getId(), $event_member->getId());
					$mem_ids = array_filter(explode(',', $mem_ids_csv), 'is_numeric');
					if (count($mem_ids) > 0) {
						$account_member = Members::findOne(array("conditions" => "id IN (".implode(',', $mem_ids).")"));
					}
				}
			}
			
			// event doesn't have an account, use the one defined in the config option "event_invoice_account_id"
			if (!$account_member instanceof Member) {
				Env::useHelper('functions', 'income');
				$account_member_id = income_config_option('event_invoice_account_id');
				if ($account_member_id) {
					$account_member = Members::findById($account_member_id);
				}
			}
			
			// classify the invoice in the account member
			if ($account_member instanceof Member) {
				$object->addToMembers(array($account_member));
			}
		}
		
		
	}
}


function event_tickets_modify_invoice_type_for_sending_mail($params, &$hook_result) {
	$invoice = array_var($params, 'invoice');
	if ($invoice instanceof IncomeInvoice) {
		
		$members = $invoice->getMembers();
		$event_member = null;
		$client_member = null;
		
		foreach ($members as $member) {
			$member_ot = ObjectTypes::findById($member->getObjectTypeId());
			if ($member_ot instanceof ObjectType) {
				if ($member_ot->getName() == 'managed_event') {
					$is_event_invoice = true;
					$event_member = $member;
				} else if ($member_ot->getName() == 'customer') {
					$client_member = $member;
				}
			}
		}
		
		if ($is_event_invoice) {
			// templates for event invoices
			if ($invoice->getStatus() == 'confirmed') {
				$hook_result['invoice_type'] = 'event_registration_paid';
			} else {
				$hook_result['invoice_type'] = 'event_registration_pending';
			}

			if (!isset($hook_result['more_variables'])) $hook_result['more_variables'] = array();
			// add the event member to the template variables
			if ($event_member instanceof Member) {
				$hook_result['more_variables']['event_member'] = $event_member;
			}
			// add the client member to the template variables
			if ($client_member instanceof Member) {
				$hook_result['more_variables']['client_member'] = $client_member;
			}
		}
	}
}


function event_tickets_parent_subtype_id($object, &$parent_subtype_id) {

	if ($object instanceof EventTicket) {
		$tt = EventTicketTypes::findById($object->getTicketTypeId());
		if ($tt instanceof EventTicketType) {
			$event = ManagedEvents::findById($tt->getEventId());
			if ($event instanceof ManagedEvent) {
				$parent_subtype_id = $event->getColumnValue('object_subtype_id');
			}
		}
	}
}


