<?php

$widgets_data_cache = array();

function collect_tasks_and_time_amounts_data_for_widget() {
	
	global $widgets_data_cache;
	
	Env::useHelper('functions', 'advanced_billing');
	
	// This is a patch to prevent reaching memory limit when no filters are applied and there are too much timeslots in the system
	// @TODO: Improve memory usage of the listing function for timeslots and tasks and remove this condition
	$this_year_cond = "";
	$this_year_cond_time = "";
	$active_context_members = active_context_members(false);
	$include_subtitle = false;
	if (count($active_context_members) == 0) {
		$beginning_of_year = DateTimeValueLib::now();
		if( $beginning_of_year->getMonth() < 6){
			$previous_year = $beginning_of_year->getYear() - 1;
			$beginning_of_year->setYear($previous_year);
		} 
		$beginning_of_year->setDay(1);
		$beginning_of_year->setMonth(1);
		$beginning_of_year->setHour(0);
		$beginning_of_year->setMinute(0);
		$beginning_of_year->setSecond(0);
		$this_year_cond = " AND (start_date >= '".$beginning_of_year->toMySQL()."' OR due_date >= '".$beginning_of_year->toMySQL()."')";
		$this_year_cond_time = " AND (start_time >= '".$beginning_of_year->toMySQL()."' OR end_time >= '".$beginning_of_year->toMySQL()."')";
		$include_subtitle = true;
		$subtitle_date_start = $beginning_of_year;
	}
	// --
	
	$task_assignment_conditions = "";
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
		$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
	}
	
	if (isset($widgets_data_cache['financial_tasks'])) {
		$tasks = $widgets_data_cache['financial_tasks'];
	} else {
		$tasks_res = ProjectTasks::instance()->listing(array(
				"raw_data" => true,
				"fire_additional_data_hook" => false,
				"order" => "due_date",
				"order_dir" => "ASC",
				"extra_conditions" => " AND is_template=0 AND depth=0 $task_assignment_conditions $this_year_cond"
		));
		$tasks = $tasks_res->objects;
		
		$widgets_data_cache['financial_tasks'] = $tasks;
	}

	$extra_conditions = "";
	if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
		$extra_conditions .= " AND contact_id = " . logged_user()->getId();
	}
	
	if (isset($widgets_data_cache['financial_timeslots'])) {
		$timeslots = $widgets_data_cache['financial_timeslots'];
	} else {
		$timeslots_res = Timeslots::instance()->listing(array(
				"raw_data" => true,
				"fire_additional_data_hook" => false,
				"order" => "o.id",
				"order_dir" => "ASC",
				"extra_conditions" => $extra_conditions . $this_year_cond_time
		));
		$timeslots = $timeslots_res->objects;
		
		$widgets_data_cache['financial_timeslots'] = $timeslots;
	}
	
	$amounts_by_day = array();
	
	
	// Collect tasks information and build the estimated billing from billing category and estimated time
	$tasks_without_due_date = array();
	$total_amount_tasks_without_due_date = 0;
	$tasks_without_billing_cat = array();
	$total_amount_tasks_without_billing_cat = 0;
	$tasks_without_estimated_time = array();
	$total_amount_tasks_without_estimated_time = 0;
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	
	foreach ($tasks as $task) {
		/* @var $task ProjectTask */
		if (count($allowed_dimensions) == 0) continue;

		if ($task['due_date'] == '0000-00-00 00:00:00' && $task['start_date'] == '0000-00-00 00:00:00') {
			$tasks_without_due_date[] = $task;
			$total_amount_tasks_without_due_date += $task['estimated_price'];
			continue;
		}
		
		if ($task['time_estimate'] == 0) {
			$tasks_without_estimated_time[] = $task;
			$total_amount_tasks_without_estimated_time += $task['estimated_price'];
			continue;
		}
		
		$object_id = $task['id'];

		if($task['due_date'] != '0000-00-00 00:00:00'){
			$due_date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $task['due_date']);
		} else if($task['start_date'] != '0000-00-00 00:00:00') {
			$due_date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $task['start_date']);
		} else if($task['due_date'] == '0000-00-00 00:00:00' && $task['start_date'] == '0000-00-00 00:00:00'){
			continue;
		}
		$date_key = $due_date->format('Y-m-d');
		$due_date_str = $due_date->format('m/d/Y');

		if (!isset($amounts_by_day[$date_key])) {
			$amounts_by_day[$date_key] = array(
					'date' => $due_date_str, 
					'estimated' => 0,
					'executed' => 0,
			);
		}

		$amounts_by_day[$date_key]['estimated'] += $task['estimated_price'];
		
		/*
		* Previous solution, slow in performance too many queries to calculate task estimated price
		*
		$sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
			WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$object_id'";
	
		$dim_member_rows = DB::executeAll($sql);
		$dim_members = array();
		if (is_array($dim_member_rows)) {
			foreach ($dim_member_rows as $dmr) {
				if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
				$dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
			}
		}
		$cant_decide_between_billing_definitions = null;
		
		$billing_definition = find_billing_def_by_user_and_members($task['assigned_to_contact_id'], $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);
		if($billing_definition instanceof BillingDefinition){
			$task_billing = $billing_definition->getHourlyRate() * ($task['time_estimate'] / 60);	$amounts_by_day[$date_key]['estimated'] += $task_billing;
		} else {
			$tasks_without_billing_cat[] = $task;
			continue;
		}*/
	
	}
	
	
	// Collect timeslots information and build the worked hours values
	foreach ($timeslots as $timeslot) {
		/* @var $timeslot Timeslot */
		
		$timeslot_billing = $timeslot['fixed_billing']; // ->getFixedBilling();
		
		$date_key = substr($timeslot['start_time'], 0, 10); // exclude the time part
		$start_time_str = date('m/d/Y', strtotime($date_key));
		
		if (!isset($amounts_by_day[$date_key])) {
			$amounts_by_day[$date_key] = array(
					'date' => $start_time_str, //format_date($timeslot->getStartTime()),
					'estimated' => 0,
					'executed' => 0,
			);
		}
		
		$amounts_by_day[$date_key]['executed'] += $timeslot_billing;
	}
	
	// sort the amounts by date
	ksort($amounts_by_day);
	
	$result = array(
		'amounts_by_day' => $amounts_by_day,
		'number_of_tasks' => count($tasks),
		'tasks_without_billing_cat' => $tasks_without_billing_cat,
		'tasks_without_estimated_time' => $tasks_without_estimated_time,
		'tasks_without_due_date' => $tasks_without_due_date,
		'total_amount_tasks_without_billing_cat' => $total_amount_tasks_without_billing_cat,
		'total_amount_tasks_without_estimated_time' => $total_amount_tasks_without_estimated_time,
		'total_amount_tasks_without_due_date' => $total_amount_tasks_without_due_date,
		'include_subtitle' => $include_subtitle,
		'subtitle_date_start' => $subtitle_date_start
	);

	return $result;
}
