<?php 
Env::useHelper('chart');

$crpmDimension = Dimensions::instance()->findByCode('customer_project')->getId();
$project_ot = ObjectTypes::instance()->findByName('project');

$folder_ot = ObjectTypes::instance()->findByName('folder');
$cfolder_ot = ObjectTypes::instance()->findByName('customer_folder');
$pfolder_ot = ObjectTypes::instance()->findByName('project_folder');

$draw = true;

$pdot = DimensionObjectTypes::findOne(array('conditions' => 'dimension_id='.$crpmDimension.' AND object_type_id='.$project_ot->getId()));
if (!$pdot instanceof DimensionObjectType || !$pdot->getEnabled()) {
	$draw = false;
}


$parent = null;
$context = active_context();
if (is_array($context)) {
	foreach ($context as $selection) {
		if ($selection instanceof Member) {
			if ($selection->getDimensionId() != $crpmDimension) {
				$draw = false;
				break;
			} else {
				if (!in_array($selection->getObjectTypeId(), array($folder_ot->getId(), $cfolder_ot->getId(), $pfolder_ot->getId()))) {
					//$draw = false;
				}
				$parent = $selection;
				if ($selection->getObjectTypeId() == $project_ot->getId()) {
					$draw = false;
				}
				break;
			}
		}
	}
}

if ($draw) {
	// Render only when ( Client, Project or Folder was selected ) or when no context is selected
	$projects = array();
	$limit = 5;
	$start = 0;
	$order = 'name';
	$total = 0;
	
	$projects_object_type_id = Projects::instance()->getObjectTypeId();
	$pg_array = logged_user()->getPermissionGroupIds();
	
	$ids = array();
	$current_member_cond = $parent instanceof Member ? "AND parent_member_id=".$parent->getId() : "";
	
	if (Plugins::instance()->isActivePlugin('object_subtypes')) {
		$subtype_filter = $this->getContactOptionValue('subtype_filter');
		$subtype = null;
		if ($subtype_filter > 0) {
			$subtype = ObjectSubtypes::findById($subtype_filter);
		}
		if ($subtype) {
			$current_member_cond .= " AND (SELECT o.object_subtype_id FROM ".TABLE_PREFIX."objects o WHERE o.id=".TABLE_PREFIX."members.object_id)=".$subtype->getId();
		}
	}
	
	$members = Members::findAll(array(
		'limit' => $limit,
		'order' => "depth, name",
		'conditions' => "object_type_id=$projects_object_type_id $current_member_cond AND archived_by_id=0 AND EXISTS (
			SELECT cmp.member_id FROM ".TABLE_PREFIX."contact_member_cache cmp 
			WHERE cmp.member_id=".TABLE_PREFIX."members.id AND cmp.contact_id='".logged_user()->getId()."'
		)"
	));
	
	if ($parent instanceof Member && count($members) < $limit) {
		$tmp_ids = array();
		foreach ($members as $m) {
			$tmp_ids[] = $m->getId();
		}
		
		$extra_conds = "AND archived_by_id=0 AND EXISTS (
			SELECT cmp.member_id FROM ".TABLE_PREFIX."contact_member_cache cmp 
			WHERE cmp.member_id=".TABLE_PREFIX."members.id AND cmp.contact_id='".logged_user()->getId()."'
		)";

		$childs = $parent->getAllChildren(true, 'name', $extra_conds);
		foreach ($childs as $ch) {
			if (in_array($ch->getId(), $tmp_ids)) continue;
			if ($ch->getObjectTypeId() == $projects_object_type_id && count($members) <= $limit) {
				$members[] = $ch;
			}
			if (count($members) >= $limit) break;
		}
	}
	
	if (count($members) > 0){
		foreach ($members as $m){
			$ids[] = $m->getObjectId();
		}		
		if (count($ids)){
			$args = array(
				'conditions' => " trashed_on = '0000-00-00 00:00:00' AND archived_on = '0000-00-00 00:00:00' AND `object_id` IN (".implode(",",$ids).")",
				'limit' => $limit,
				'order' => $order,
			);
			$projects = Projects::findAll($args);
			$total = count($projects);
		}
	}
	
	if ($parent instanceof Member && $parent->getObjectTypeId() == $project_ot->getId()) {
		$tmp_p = Projects::findById($parent->getObjectId());
		array_unshift($projects, $tmp_p);
	}
	
	
	// tasks data foreach project
	$completed_task_summary = array();
	foreach ($projects as $project) {
		
		$project_member = Members::findOneByObjectId($project->getId(), $crpmDimension);
		
		$task_assignment_conditions = "";
		if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
			$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
		}
		
		$tasks = ProjectTasks::instance()->listing(array(
				"raw_data" => true,
				"order" => "due_date",
				"order_dir" => "ASC",
				"fire_additional_data_hook" => false,
				"ignore_context" => true,
				"member_ids" => array($project_member->getId()),
				"extra_conditions" => " AND is_template = 0 $task_assignment_conditions"
		))->objects;
		
		$completed_task_summary[$project->getId()] = array(
				"overdue" 	=> 0,
				"pending" 	=> 0,
				"today" 	=> 0,
				"completed" => 0,
		);
		
		$now = DateTimeValueLib::now();
		if (is_array($tasks)) {
			foreach ($tasks as $task){
				$dd = $task['due_date'] != EMPTY_DATETIME ? DateTimeValueLib::makeFromString($task['due_date']) : null;
				if ($task['completed_by_id'] > 0) {
					$completed_task_summary[$project->getId()]['completed']++;
				} elseif ($task['completed_by_id'] == 0 && $dd !== null && $dd->getTimestamp() < $now->getTimestamp()) {
					$completed_task_summary[$project->getId()]['overdue']++;
				} else {
					if ($dd instanceof DateTimeValue && $dd->isToday()) $completed_task_summary[$project->getId()]['today']++;
					else $completed_task_summary[$project->getId()]['pending']++;
				}
			}
		}
	}
	
	
	$active_members = array();
	if (is_array($context)) {
		foreach ($context as $selection) {
			if ($selection instanceof Member) $active_members[] = $selection;
		}
	}
	if (count($active_members) > 0) {
		$mnames = array();
		$allowed_contact_ids = array();
		foreach ($active_members as $member) {
			$allowed_contact_ids[] = $member->getAllowedContactIds();
			$mnames[] = clean($member->getName());
		}
		$widget_title = lang('projects');
	} else {
		$widget_title = lang('projects');
	}
	
	// Specific subtype widget
	if ($subtype) {
		$widget_title = $subtype->getName();
	}
	// ---
	
	
	$render_add = can_manage_dimension_members(logged_user());
	$render_add_templates = false;
	
	if ($render_add) {
		if (Plugins::instance()->isActivePlugin('member_templates')) {
			$member_templates = MemberTemplates::findAll(array("conditions" => "object_type_id=".$project_ot->getId(), "order" => "name"));
			if (count($member_templates) > 0) {
				$render_add = false;
				$render_add_templates = true;
			}
		}
	}
	
	if ($total > 0 || $render_add || $render_add_templates) {
		include 'template.php';	
	}
}
