<?php
/**
 * Expense controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExpenseController extends ApplicationController {

	var $plugin_name = "expenses";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	function init() {
		require_javascript("ExpenseManager.js", $this->plugin_name);
		ajx_current("panel", "expenses-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		
		ajx_add("overview-panel", "reload");
		
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
		
			switch ($order) {
				case 'date':
					$order = '`jt`.`date`';
					$join_params = array(
						'table' => Payments::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'expense_id',
						'e_field' => 'object_id',
					);
					$extra_conditions .= " AND (jt.payment_num IS NULL OR jt.payment_num = (SELECT MIN(p2.payment_num) FROM ".TABLE_PREFIX."payments p2 WHERE p2.expense_id=jt.expense_id)) ";
					break;
				case 'status':
					$order = '`jt`.`order`';
					$join_params = array(
						'table' => ExpensesStatuses::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'id',
						'e_field' => 'status',
					);
					$select_columns = array("e.*, o.*, jt.`order`");
					break;
				default:
					if (str_starts_with($order, 'total_')) {
						$status_code = str_replace("total_", "", $order);
						
						$select_columns = array("*, (SELECT sum(amount) FROM ".TABLE_PREFIX."payment_history p 
								WHERE
								p.modification_date = (
									SELECT max(x.modification_date) FROM ".TABLE_PREFIX."payment_history x 
									WHERE x.expense_id=p.expense_id AND x.payment_num=p.payment_num  AND p.status_id=x.status_id
								)
								AND p.expense_id=e.object_id and p.status_id=(
									SELECT st.id FROM ".TABLE_PREFIX."expenses_status st WHERE st.code='$status_code')
								) as order_col");
						
						$order = "order_col";
						
					} else {
						$order = "name";
					}
					break;
			}
		}
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$filters = array();
		if (array_var($_GET,'status_filter')) $filters['status'] = array_var($_GET,'status_filter');

		foreach ($filters as $col => $value) {
			if (!$value || $value == '0') continue;
			$extra_conditions .= " AND `$col` = " . DB::escape($value);
		}
		
		if (array_var($_GET, 'from_filter')) {
			$from_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'from_filter'));
			if ($from_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."payments p WHERE p.expense_id=o.id AND p.date >= '".$from_date->toMySQL()."')";
			}
		}
		if (array_var($_GET, 'to_filter')) {
			$to_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'to_filter'));
			if ($to_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."payments p WHERE p.expense_id=o.id AND p.date <= '".$to_date->toMySQL()."')";
			}
		}
		
		$grid_message = "";
		$grid_message_cls = "";
		
		// conditions to filter the totals and payments by the active context
		$active_member_ids = active_context_members(false);
		if (count($active_member_ids) > 0) {
			
			$context_member_names_str = "";
			$active_context_members = Members::findAll(array("conditions" => "id IN (".implode(',', $active_member_ids).")"));
			$i=1;
			foreach ($active_context_members as $m) {
				$sep = ($i == count($active_context_members)) ? " ".lang('and')." " : ", ";
				$context_member_names_str .= ($context_member_names_str == "" ? "" : $sep) . $m->getName();
				$i++;
			}
			$grid_message = lang('payments and expenses totals are being filtered by their classification in', $context_member_names_str);
			$grid_message_cls = "warning";
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		$result = Expenses::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"select_columns" => $select_columns,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"join_params" => $join_params,
			"start" => $start,
			"limit" => $limit,
		));
		$expenses = $result->objects;
		$object = $this->prepareObject($expenses, $start, $limit, $result);
		
		$object['filters'] = array(
			'status' => ExpensesStatuses::instance()->getAllStatusInfo(),
		);
		
		if ($grid_message) {
			$object['message'] = $grid_message;
			$object['message_cls'] = $grid_message_cls;
		}
		
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	private function prepareObject($expenses, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"expenses" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Expenses::instance()->getObjectTypeId());
		$count = min(array($limit, count($expenses)));
		for ($i = 0; $i < $count; $i++){
			if (isset($expenses[$i])){
				$expense = $expenses[$i];
				if ($expense instanceof Expense){
					$info = $expense->getArrayInfo(true, true);
					$info['memPath'] = json_encode($expense->getMembersIdsToDisplayPath(true));
					$object["expenses"][$i] = $info;
					
					foreach ($custom_properties as $cp) {
						$object["expenses"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $expense);
					}
    			}
			}
		}
		return $object;
	}
	
	function get_totals_for_listing() {
		ajx_current("empty");
		
		$extra_conditions = "";
		
		$st_filter = array_var($_REQUEST, 'st');
		if (is_numeric($st_filter) && $st_filter > 0) {
			$extra_conditions .= " AND e.status = ".DB::escape($st_filter);
		}
		
		if (array_var($_REQUEST, 'fromd')) {
			$from_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'fromd'));
			if ($from_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p2.expense_id FROM ".TABLE_PREFIX."payments p2 WHERE p2.expense_id=e.object_id AND p2.date >= '".$from_date->toMySQL()."')";
				//$extra_conditions .= " AND p.date >= '".$from_date->toMySQL()."'";
			}
		}
		if (array_var($_REQUEST, 'tod')) {
			$to_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'tod'));
			if ($to_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p2.expense_id FROM ".TABLE_PREFIX."payments p2 WHERE p2.expense_id=e.object_id AND p2.date <= '".$to_date->toMySQL()."')";
				//$extra_conditions .= " AND p.date <= '".$to_date->toMySQL()."'";
			}
		}
		if (array_var($_REQUEST, 'expense_id')) {
			$extra_conditions .= " AND p.expense_id = '".array_var($_REQUEST, 'expense_id')."'";
		}
		
		if (isset($_SESSION['last_expense_text_filter'])) {
			$extra_conditions .= " AND (o.name LIKE '%".$_SESSION['last_expense_text_filter']."%')";
			unset($_SESSION['last_expense_text_filter']);
		}
		
		
		$members_cond = "true";
		$member_ids = array();
		$context = active_context();
		foreach ($context as $selection) {
			if ($selection instanceof Member) {
				$member_ids[] = $selection->getId();
			}
		}
		foreach ($member_ids as $mid) {
			$members_cond .= " AND EXISTS( SELECT om.object_id FROM ".TABLE_PREFIX."object_members om where om.member_id=$mid AND om.object_id=o.id )";
		}
		
		$logged_user_pgs = logged_user()->getPermissionGroupIds();
		if (logged_user()->isAdministrator()) {
			$permissions_cond = "AND TRUE";
		} else {
			if (count($logged_user_pgs) > 0) {
				$permissions_cond = "AND EXISTS (SELECT sh.object_id FROM ".TABLE_PREFIX."sharing_table sh WHERE sh.group_id IN (".implode(',',$logged_user_pgs).") AND sh.object_id=e.object_id)";
			} else {
				$permissions_cond = "AND FALSE";
			}
		}
		
		$extra_member_conds = "";
		$active_member_ids = active_context_members(false);
		if (count($active_member_ids) > 0) {
			$mem_str = implode(',', $active_member_ids);
			$extra_member_conds = "
				OR EXISTS (
					SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp INNER JOIN ".TABLE_PREFIX."object_members pm ON pm.object_id=pp.object_id 
					WHERE pm.member_id IN ($mem_str) AND pp.expense_id=p.expense_id AND pp.payment_num=p.payment_num
					GROUP BY pm.object_id HAVING count(pm.member_id)=".count($active_member_ids)."
				)";
		}
		
		$sql = "SELECT sum(p.amount) as total, p.currency, p.status_id FROM ".TABLE_PREFIX."payment_history p
			INNER JOIN ".TABLE_PREFIX."expenses e ON e.object_id = p.expense_id
			INNER JOIN ".TABLE_PREFIX."objects o ON o.id = e.object_id
			WHERE o.trashed_by_id=0 AND o.archived_by_id=0
			$extra_conditions
			AND ($members_cond $extra_member_conds)
			$permissions_cond
			AND p.modification_date = (
				SELECT max(x.modification_date) FROM ".TABLE_PREFIX."payment_history x 
				WHERE x.expense_id=p.expense_id AND x.payment_num=p.payment_num  AND p.status_id=x.status_id
			)
			AND EXISTS (SELECT pp.object_id FROM ".TABLE_PREFIX."payments pp INNER JOIN ".TABLE_PREFIX."objects oo ON oo.id=pp.object_id 
					WHERE pp.expense_id=p.expense_id AND pp.payment_num=p.payment_num AND oo.trashed_by_id=0)
			AND p.status_id <= (SELECT pp.status_id FROM ".TABLE_PREFIX."payments pp WHERE pp.expense_id=p.expense_id AND pp.payment_num=p.payment_num LIMIT 1)
						
			GROUP BY status_id, currency;";
		$rows = DB::executeAll($sql);
		
		
		$totals = array();
		if (is_array($rows)) {
		  foreach ($rows as $row) {
			$st = ExpensesStatuses::instance()->findById($row['status_id']);
			if (!$st instanceof ExpensesStatus) continue;
			
			$currency = Currencies::findById($row['currency']);
			
			$totals[$st->getCode()] = format_money_amount($row['total'], $currency->getSymbol());
		  }
		}
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		
		if (!isset($currency) || !$currency instanceof Currency) {
			$currency = Currencies::findOne();
		}
		
		foreach ($all_status as $st) {
			if ($st['only_for_expenses']) continue;
			if (!isset($totals[$st['code']])) $totals[$st['code']] = format_money_amount(0, $currency instanceof Currency ? $currency->getSymbol() : '');
		}
		
		$totals_info = array('id' => -1, 'name' => lang('total'));
		foreach ($totals as $k => $t) {
			$totals_info['total_' . $k] = $t;
		}
		
		ajx_extra_data(array('totals' => $totals_info));
	}
	
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		/*
		$notAllowedMember = '';
		if(!Expense::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("expenses"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		*/
		$expense_data = array_var($_POST, 'expense');
		
		if(!is_array($expense_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('expense', new Expense());
			tpl_assign('max_pnum', 0);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$expense = $this->save_expense($expense_data, $member_ids);
				
				DB::commit();
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
			
			ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_ADD);
			flash_success(lang('success add expense', $expense->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('expense dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$expense->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$expense_data = array_var($_POST, 'expense');
		
		if(!is_array($expense_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('expense', $expense);
			$rows = DB::executeAll("SELECT max(payment_num) as num FROM ".TABLE_PREFIX."payment_history WHERE expense_id=".$expense->getId());
			$max_pnum = 0;
			if (count($rows) > 0) {
				$max_pnum = $rows['0']['num'];
			}
			tpl_assign('max_pnum', $max_pnum);
			
			$payments = Payments::findAll(array('conditions' => array('trashed_by_id=0 AND expense_id = ?', $expense->getId())));
			tpl_assign('payments', $payments);
			
			$payment_attributes = array();
			foreach ($payments as $p) {
				if ($p->getTypeId() > 0) {
					$p_attributes = $p->getTypeAttributes();
					$payment_attributes[$p->getPaymentNum()] = $p_attributes;
				}
			}
			tpl_assign('payment_attributes', $payment_attributes);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$expense = $this->save_expense($expense_data, $member_ids, $expense);
				
				DB::commit();
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
				
			ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_EDIT);
			flash_success(lang('success edit expense', $expense->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function save_expense($expense_data, $member_ids, $expense = null) {
		// Save expense object
		$is_new = false;
		if (is_null($expense) || !$expense instanceof Expense) {
			$expense = new Expense();
			$is_new = true;
		}
		
		$object_controller = new ObjectController();
		
		$expense->setFromAttributes($expense_data);
		
		$payments = array_var($expense_data, 'payments');
		/*
		if (!$is_new) {
			// delete deleted payments (those that were not submitted)
			$submitted = array();
			if (is_array($payments)) {
				foreach ($payments as $p) {
					$submitted[] = $p['num'];
				}
			}
			$to_delete = array();
			if (!$is_new) {
				$to_del_cond = "";
				$submitted = array_filter($submitted);
				if (count($submitted) > 0) $to_del_cond = ' AND payment_num NOT IN ('.implode(',',$submitted).')';
				$to_delete = Payments::findAll(array('conditions' => 'expense_id = '.$expense->getId() . $to_del_cond));
			}
			foreach ($to_delete as $td) {
				$td->addToHistory(logged_user(), 'delete', false, false);
				$td->delete();
				PaymentAttributeValues::delete("expense_id=".$expense->getId()." AND payment_num=".$td->getPaymentNum());
			}
		}*/
		
		if (config_option("use_date_in_expense_level")) {
			$ex_date = getDateValue(array_var($expense_data, 'date'));
			if ($ex_date instanceof DateTimeValue) {
				$expense->setDate($ex_date);
			}
		}
		
		if ($is_new) {
			$all_payment_status_ids = array_flat(DB::executeAll("SELECT id FROM ".TABLE_PREFIX."expenses_status WHERE only_for_expenses=0 AND enabled=1 ORDER BY `order`"));
			$first_status_id = array_var($all_payment_status_ids, 0);
			$expense->setStatus($first_status_id);
		}
		
		$expense->save(false);
		
		$expense_payments = array();
		$status_count = array();
		/*
		// Save payments
		if (is_array($payments)) {
		  foreach ($payments as $p_data) {
		  	
		  	$submitted_pdate = array_var($p_data, 'date');
		  	if (config_option("use_date_in_expense_level")) {
		  		$submitted_pdate = array_var($expense_data, 'date');
		  		if ($submitted_pdate == date_format_tip(user_config_option('date_format'))) $submitted_pdate = '';
		  	}
		  	
		  	$p_amount = floatval(preg_replace('/[^\d.]/', '', array_var($p_data, 'amount')));
		  	
		  	// ignore payments with no data
		  	if ($submitted_pdate == '' && $p_amount == 0 && array_var($p_data, 'type_id', '0') == '0' && array_var($p_data, 'description') == '') continue;

		  	// sanitize payment amount, if negative amounts are allowed then add the "-" sign to the regular expression that sanitizes the amounts
		  	$reg_exp = config_option('allow_negative_amounts') ? '/[^\d.-]/' : '/[^\d.]/';
		  	$p_data['amount'] = preg_replace($reg_exp, '', array_var($p_data, 'amount'));
		  	
			$num = array_var($p_data, 'num');
			$payment = null;
			if (!$is_new) {
				$payment = Payments::findOne(array('conditions' => 'expense_id='.$expense->getId()." AND payment_num='".$num."'"));
			}
			$is_new_p = false;
			if (!$payment instanceof Payment) {
				$payment = new Payment();
				$payment->setExpenseId($expense->getId());
				$payment->setPaymentNum($num);
				$is_new_p = true;
			}
			$p_data['amount'] = str_replace(',', '', array_var($p_data, 'amount', ''));
			if (config_option("use_date_in_expense_level")) {
				$p_data['date'] = $ex_date;
			} else {
				$p_data['date'] = getDateValue(array_var($p_data, 'date'));
			}
			
			$p_data['name'] = $p_data['description'];
			
			$payment->setFromAttributes($p_data);
			$payment->save();
			
			if (isset($p_data['member_ids']) && is_array($p_data['member_ids'])) {
				$mem_ids = array_filter($p_data['member_ids']);
				if (count($mem_ids) > 0) {
					$members = Members::findAll(array("conditions" => "id IN (".implode(',', $mem_ids).")"));
					if (count($members) > 0) {
						$payment->addToMembers($members);
					}
				}
			}
			
			$cp_data = null;
			if (isset($p_data['object_custom_properties'])) {
				$cp_data = $p_data['object_custom_properties'];
				$object_controller->add_custom_properties($payment, $cp_data);
			}
			
			$payment_type = array_var($p_data, 'type_id');
			if ($payment_type > 0) {
				$type_attributes = array_var($p_data, $payment_type);
				
				if (count($type_attributes) > 0) {
					DB::execute("DELETE FROM ".TABLE_PREFIX."payment_attribute_values WHERE expense_id=".$expense->getId().
						" AND payment_num=".$payment->getPaymentNum());
					$type_attr_ids = array_keys($type_attributes);
					
					foreach ($type_attributes as $attr_id => $attr_val) {
						
						$attr_def = PaymentTypeAttributes::findOne(array('conditions' => "payment_type_id='$payment_type' AND attribute_id='$attr_id'"));
						if (!$attr_def instanceof PaymentTypeAttribute) continue;
						if ($attr_def->getType() == 'date') {
							$date_val = getDateValue($attr_val);
							if ($date_val instanceof DateTimeValue) {
								$attr_val = $date_val->toMySQL();
							} else {
								$attr_val = "";
							}
						}
						if ($attr_def->getType() == 'contact' || $attr_def->getType() == 'user') {
							if ($attr_val == '0') {
								$attr_val = "";
							}
						}
						
						if ($attr_def->getIsRequired() && trim($attr_val) == '' && $attr_def->getType() != 'boolean') {
							throw new Exception(lang('custom property value required', escape_character($attr_def->getName())));
						}
						
						DB::execute("INSERT INTO ".TABLE_PREFIX."payment_attribute_values (expense_id, payment_num, type_id, attribute_id, value)
								VALUES ('".$expense->getId()."', '".$payment->getPaymentNum()."', '".$payment->getTypeId()."', '$attr_id', '$attr_val')
								ON DUPLICATE KEY UPDATE `value`='$attr_val'");
					}
				}
			}
			
			// if edit, save only if there are changes, don't recalculate the expense totals in each payment (do it after saving the expense)
			$payment->addToHistory(logged_user(), ($is_new_p ? 'add' : 'edit'), true, false);
			$expense_payments[] = $payment;
			
			if (!isset($status_count[$payment->getStatusId()])) $status_count[$payment->getStatusId()] = 0;
			$status_count[$payment->getStatusId()] = $status_count[$payment->getStatusId()]+1;
		  }
		}
		
		if (count($status_count) > 1) {
			$part_app = ExpensesStatuses::instance()->getStatusByCode('part_approved');
			$st_app = ExpensesStatuses::instance()->getStatusByCode('approved');
			$part_exe = ExpensesStatuses::instance()->getStatusByCode('part_executed');
			$st_exe = ExpensesStatuses::instance()->getStatusByCode('executed');
			
			if (array_var($status_count, $st_exe->getId()) > 0) {
				$expense->setStatus($part_exe->getId());
			} else if (array_var($status_count, $st_app->getId()) > 0) {
				$expense->setStatus($part_app->getId());
			}
			$expense->save();
			
		} else if (count($status_count) == 1) {
			$keys = array_keys($status_count);
			$expense->setStatus($keys[0]);
			$expense->save(false);
		} else {
			$expense->setStatus($first_status_id);
			$expense->save(false);
		}
		*/
		
		
		// if expense is a copy of an existing expense then also copy the payments
		if (array_var($expense_data, 'original_expense_id') > 0) {
				
			$exp_id = array_var($expense_data, 'original_expense_id');
			$original_payments = Payments::findAll(array("conditions" => "expense_id='$exp_id'"));
				
			$payment_controller = new PaymentController();
			$log_data = "";
			foreach ($original_payments as $op) {
				/* @var $op Payment */
				$payment_data = array(
						'name' => $op->getName(),
						'description' => $op->getDescription(),
						'expense_id' => $expense->getId(),
						'payment_num' => $op->getPaymentNum(),
						'date' => $op->getDate() instanceof DateTimeValue ? format_date($op->getDate(), null, 0) : '',
						'currency' => $op->getCurrency(),
						'amount' => $op->getAmount(),
						'status_id' => $op->getStatusId(),
				);
		
				$new_payment = new Payment();
				$new_payment->setExpenseId($expense->getId());
				$payment_controller->save_payment($payment_data, $op->getMemberIds(), $new_payment, $log_data);
			}
		}
		
		// classify expense
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($expense, $member_ids);
			
			// classify new payments in the same places of the expense
			if (config_option('copy_exp_classiffication_to_payments')) {
				$exp_members = $expense->getMembers();
				
				$exp_payments = Payments::instance()->findAll(array("conditions" => "expense_id=".$expense->getId()));
				if (count($exp_payments) > 0) {
					$dims_to_keep = array(); // this are the dimensions where exepenses are autoclassified when classfiying a payment, this can't be inherited by payments.
					
					$ot_hierarchy = ObjectTypeHierarchies::findOne(array(
						"conditions" => "parent_object_type_id=".$expense->getObjectTypeId()." AND child_object_type_id=".Payments::instance()->getObjectTypeId()
					));
					if ($ot_hierarchy instanceof ObjectTypeHierarchy) {
						$oth_options = ObjectTypeHierarchyOptions::findAll(array("conditions" => "hierarchy_id='".$ot_hierarchy->getId()."' AND `option`='autoclassify_in_parent_members'"));
						foreach ($oth_options as $opt) {
							$dims_to_keep[] = $opt->getDimensionId();
						}
					}
					
					// collect the members to be copied to the payments
					$members_to_inherit = array();
					foreach ($exp_members as $mem) {
						if (!in_array($mem->getDimensionId(), $dims_to_keep)) {
							$members_to_inherit[] = $mem;
						}
					}
					
					foreach ($exp_payments as $payment) {
						$members_to_keep = array();
						$p_members = $payment->getMembers();
						// collect the payment members to be kept in the payment
						foreach ($p_members as $mem) {
							if (in_array($mem->getDimensionId(), $dims_to_keep)) {
								$members_to_keep[] = $mem;
							}
						}
						
						$new_payment_members = array_merge($members_to_inherit, $members_to_keep);
						
						$payment->removeFromMembers(logged_user());
						$payment->addToMembers($new_payment_members);
						$payment->addToSharingTable();
					}
				}
			}
		}
		
		$object_controller->add_subscribers($expense);
		
		$object_controller->link_to_new_object($expense);
		$object_controller->add_custom_properties($expense);
		
		// guardar en el historico de expenses los cambios realizados
		$expense->addToHistory(logged_user(), ($is_new ? 'add' : 'edit'));
		
		
		
		return $expense;
	}
	
	function view() {
		ajx_set_no_toolbar();
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$expense->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_extra_data(array("title" => $expense->getObjectName(), 'icon' => $expense->getIconClass()."s"));
		tpl_assign('expense', $expense);
	}
	
	function get_payments_table_html() {
		ajx_current("empty");
		
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		if (!$expense->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('expense', $expense);
		tpl_assign('order', array_var($_REQUEST, 'order', 'date'));
		tpl_assign('order_dir', array_var($_REQUEST, 'order_dir', 'ASC'));
		
		$html = tpl_fetch(get_template_path('view_payments_table', 'expense', 'expenses'));
		
		ajx_extra_data(array('html' => $html));
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $message
	 */
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$expense = Expenses::findById($id);
					if ($expense instanceof Expense && $expense->canDelete(logged_user())){
						try{
							DB::beginWork();
							$expense->trash();
							DB::commit();
							ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed expense', $expense instanceof Expense ? clean($expense->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$expense = Expenses::findById($id);
					if ($expense instanceof Expense && $expense->canEdit(logged_user())){
						try{
							DB::beginWork();
							$expense->archive();
							DB::commit();
							ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function get_payments() {
		ajx_current("empty");
		$id = get_id();
		
		$payments = get_payments_for_expense($id, null, true);
		
		ajx_extra_data(array('payments' => $payments));
		
	}
	
	/**
	 * Change status from budgeted to executed and saves history
	 */
	function change_payment_status() {
		ajx_current("reload");
		
		if (isset($_REQUEST['payment_id'])) {
			
			$p = Payments::instance()->findById($_REQUEST['payment_id']);
			$expense = null;
			if ($p instanceof Payment) {
				$expense = Expenses::findById($p->getExpenseId());
			}
			
		} else {
			$expense_id = get_id();
			$payment_num = get_id('num');
			
			$p = null;
			$expense = Expenses::findById($expense_id);
			if ($expense instanceof Expense) {
				$p = Payments::instance()->findOne(array('conditions' => "expense_id = $expense_id AND payment_num = $payment_num"));
			}
		}
		
		$new_status = get_id('st');
		
		// use can write instead of canEdit to allow common users to change a bufgeted payment when the expense status is final
		if (logged_user()->isGuest() || $expense instanceof Expense && !can_write(logged_user(), $expense->getMembers(), $expense->getObjectTypeId())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		if (!$p instanceof Payment) {
			ajx_current("empty");
			flash_error("payment dnx");
			return;
		}
		
		// dont allow common users to edit finalized payments, check for special permission
		if ($p->getStatusId() == ExpensesStatuses::getFinalStatus()->getId()) {
			if (!can_edit_executed_payments_pg_array(logged_user()->getPermissionGroupIds())) {
				flash_error(lang('no access permissions'));
				ajx_current('empty');
				return;
			}
		}
		
		$status_obj = ExpensesStatuses::instance()->findById($new_status);
		if (!$status_obj instanceof ExpensesStatus) {
			ajx_current("empty");
			flash_error("status dnx");
			return;
		}
		if ($status_obj->getOnlyForExpenses()) {
			ajx_current("empty");
			flash_error("invalid status for payments");
			return;
		}
		
		$old_data = $p->getArrayInfo();
		
		try {
			DB::beginWork();
			// change status
			$p->setStatusId($new_status);
			$p->save();
			// add to history
			$p->addToHistory(logged_user(), 'edit');
			Hook::fire('after_save_payment', array('object' => $p, 'data' => $p->getArrayInfo(), 'old_data' => $old_data), $null);
			
			// refresh expense status
			if ($expense instanceof Expense) $expense->refreshStatus();
			
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}
	
	
	
	
	function get_expense_status() {
		ajx_current('empty');
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		$status_by_code = array();
		$first_final_status = null;
		foreach ($all_status as $st) {
			if ($st['is_final']) {
				$first_final_status = $st;
			}
			$status_by_code[$st['code']] = $st;
		}
		
		$next_status_array = array();
		foreach ($all_status as $status) {
		
			$next_status = null;
			$x = 0;
			while ($x < count($all_status) && $all_status[$x]['id'] != $status['id']) {
				$x++;
			}
			$x = $x + 1;
			while ($x < count($all_status) && isset($all_status[$x]) && $all_status[$x]['only_for_expenses']) $x = $x + 1;
			if ($x < count($all_status) && isset($all_status[$x])) {
				$next_status = $all_status[$x];
			}
			if ($next_status) {
				$next_status_array[$status['code']] = $next_status;
			}
		}
		
		ajx_extra_data(array('status' => $all_status, 'next_status_config' => $next_status_array));
		
	}
	
	
	
	function get_expense_data() {
		ajx_current("empty");
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		$data = $expense->getArrayInfo();
		$data['memPath'] = $expense->getMembersIdsToDisplayPath();
		
		ajx_extra_data(array('expense' => $data));
	}
	
	
	function check_expense_members() {
		ajx_current("empty");
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			ajx_current("empty");
			return;
		}
		
		$member_ids = array();
		$members_str = array_var($_REQUEST, 'member_ids');
		if ($members_str) {
			$member_ids = json_decode($members_str, true);
		}
		
		$expense_member_ids = $expense->getMemberIds();
				
		$ok = true;

		//@ToDo: do not check members that do not aply to an Expense Object
		
// 		foreach ($member_ids as $member_id) {
// 			if (!in_array($member_id, $expense_member_ids)) {
// 				$ok = false;
// 				break;
// 			}
// 		}
		
		ajx_extra_data(array('ok' => $ok));
		
	}
	
	
	function administration() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$expense_administration_links = array();
		$expense_administration_links[] = array(
			'url' => get_url('expense', 'general_config_options'),
			'name' => lang("expenses general configuration"),
			'desc' => lang('expenses general configuration description'),
		);
		/* NOW THESE PAYMENT TYPES SHOULD BE MIGRATED TO OBJECT SUBTYPES
		$expense_administration_links[] = array(
			'url' => get_url('payment_types', 'list_all'),
			'name' => lang('payment types'),
			'desc' => lang('payment types description'),
		);*/
		
		tpl_assign('expense_administration_links', $expense_administration_links);
	}
	
	function general_config_options() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$options = ConfigOptions::findAll(array('conditions' => "category_name='expenses'"));
		tpl_assign('options', $options);
	}
	
	function save_general_config_options() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		try {
			DB::beginWork();
			
			$submited_values = array_var($_POST, 'options');
			if (is_array($submited_values)) {
				$options = ConfigOptions::findAll(array('conditions' => "category_name='expenses'"));
				foreach ($options as $option) {
					$new_value = array_var($submited_values, $option->getName());
					
					if (is_null($new_value) || ($new_value == $option->getValue())) {
						continue;
					}
					
					$option->setValue($new_value);
					$option->save();
				}
				flash_success(lang('success update config category', lang('expenses')));
				ajx_current("back");
			}
			
			DB::commit();
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
			return;
		}
	}
	
	
	
	
	
	
	function copy_expense() {

		// set the "add expense" form
		$this->setTemplate('add');
		
		// get the expense to copy
		$expense = Expenses::findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('expense dnx'));
			ajx_current("empty");
			return;
		}
		
		// check write permissions
		if (logged_user()->isGuest() || !$expense->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		if (array_var($_REQUEST, 'modal')) {
			$this->setLayout("json");
			tpl_assign('modal', true);
		}
		
		// find max payment num
		$rows = DB::executeAll("SELECT max(payment_num) as num FROM ".TABLE_PREFIX."payment_history WHERE expense_id=".$expense->getId());
		$max_pnum = 0;
		if (count($rows) > 0) {
			$max_pnum = $rows['0']['num'];
		}
		// get the payments to copy
		$payments = Payments::findAll(array('conditions' => array('expense_id = ?', $expense->getId())));
		
		// build the new expense name, if already exists keep trying until finding an unused name
		$copy_name_orig = lang('copy of', $expense->getName());
		$copy_name = $copy_name_orig;
		$copy_name_suffix = 1;
		$copy_exists = Expenses::findOne(array("conditions" => "name='$copy_name_orig'"));
		while ($copy_exists instanceof Expense) {
			$copy_name_suffix++;
			$copy_name = "$copy_name_orig $copy_name_suffix";
			$copy_exists = Expenses::findOne(array("conditions" => "name='$copy_name'"));
		}
		$expense->setObjectName($copy_name);
		
		// payment attributes
		$payment_attributes = array();
		foreach ($payments as $p) {
			if ($p->getTypeId() > 0) {
				$p_attributes = $p->getTypeAttributes();
				$payment_attributes[$p->getPaymentNum()] = $p_attributes;
			}
		}
		
		// assign all the variables to the form
		tpl_assign('expense', $expense);
		tpl_assign('original_expense_id', $expense->getId());
		tpl_assign('max_pnum', $max_pnum);
		tpl_assign('payments', $payments);
		tpl_assign('payment_attributes', $payment_attributes);
		tpl_assign('override_form_action', get_url('expense', 'add'));
		
	}
	
	
	
	
	
	
}