<?php 

	class Payments extends BasePayments {
		function __construct() {
			parent::__construct();
			$this->object_type_name = 'payment';
		}
		
		
		static function getTypeAttributes($payment) {
			$attributes = array();
				
			if ($payment->getTypeId() > 0) {
				$attributes = PaymentAttributeValues::findAll(array(
					'conditions' => array('expense_id=? AND payment_num=? AND type_id=?', $payment->getExpenseId(), $payment->getPaymentNum(), $payment->getTypeId())
				));
			}
				
			return $attributes;
		}
		
		static function getDescriptionWithAttributes($payment, $separator=", ", $include_payment_description = true, $include_type_name=true, $type_str='html') {
			$description = $payment->getDescription();
			$description_additional = array();
				
			$attributes = $payment->getTypeAttributes();
			if ($payment->getTypeId() > 0) {
				$type = PaymentTypes::findById($payment->getTypeId());
		
				if ($type instanceof PaymentType) {
						
					$definitions = PaymentTypeAttributes::getAttributesForType($payment->getTypeId());
					foreach ($attributes as $attr) {
						$val = trim($attr->getValue());
						if ($val != '') {
							$def = array_var($definitions, $attr->getAttributeId());
							if ($def instanceof PaymentTypeAttribute) {
								if ($def->getType() == 'contact' || $def->getType() == 'user') {
									$c = Contacts::findById($val);
									if ($c instanceof Contact) $val = $c->getObjectName();
									else $val = "";
								} else if ($def->getType() == 'boolean') {
									$val = ($val > 0 ? lang('yes') : lang('no'));
								} else if ($def->getType() == 'date') {
									$dt = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $val);
									if ($dt instanceof DateTimeValue) {
										$val = format_date($dt);
									} else {
										$val = "";
									}
								}
								$description_additional[] = $def->getName().": ".$val;
							}
						}
					}
					
					$description = self::getDescriptionHtml($description, implode($separator, $description_additional), $type, $include_payment_description, $include_type_name, $type_str);
				}
			}
				
			return $description;
		}
		
		static function getDescriptionHtml($description, $description_additional, $type, $include_payment_description = true, $include_type_name=true, $type_str='html') {
			if ($type instanceof PaymentType) {
				
				if ($type_str == 'html') {
					$description_attr = '<div class="payment-type-attributes"><span class="bold">'. ($include_type_name ? $type->getName() : '') .($description_additional!="" && $include_type_name?' - ':'').'</span>'. $description_additional . '</div>';
				} else {
					$description_attr = ($include_type_name ? $type->getName() : '') . ($description_additional=="" ? "" : ($include_type_name?' - ':''). $description_additional);
				}
				if ($include_payment_description) {
					$description = $description_attr . $description;
				} else {
					$description = $description_attr;
				}
			}
			return $description;
		}
		
		
		
		function getColumnsToAggregateInTotals() {
			return array('amount' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency', 'group_by' => 'currency'));
		}
		
	}
