<?php
Hook::register("expenses");

function expenses_object_definition($type, &$ret) {
	if ($type == 'SystemPermission' || $type == 'MaxSystemPermission') {
		$ret['can_edit_executed_payments'] = DATA_TYPE_BOOLEAN;
	}
}

function expenses_add_user_permissions($permission_group_id, &$permissions) {
	//$permissions['can_edit_executed_payments'] = can_edit_executed_payments($permission_group_id);
}


function expenses_on_delete_tables_to_clean($params, &$result) {
	$object_type_name = array_var($params, 'type_name');
	if ($object_type_name == 'expense') {
		$result[] = array('table' => TABLE_PREFIX."payments", 'column' => 'expense_id');
	}
}


function expenses_render_administration_icons($ignored, &$icons){
	$user = logged_user();

	if ($user instanceof Contact && can_manage_configuration($user)) {
		$icons[] = array(
				'ico' => 'ico-large-expense',
				'name' => lang('expenses'),
				'url' => get_url('expense', 'administration'),
		);
	}

}

// PERMISSION FUNCTIONS
function can_edit_executed_payments($permission_group_id) {
	$sys_per = SystemPermissions::findById($permission_group_id);
	if ($sys_per instanceof SystemPermission) {
		return $sys_per->getColumnValue('can_edit_executed_payments');
	}
	return false;
}

function can_edit_executed_payments_pg_array($permission_group_ids) {
	if (!is_array($permission_group_ids) || count($permission_group_ids) == 0) {
		return false;
	}
	$rows = DB::executeAll("SELECT can_edit_executed_payments FROM ".TABLE_PREFIX."system_permissions WHERE permission_group_id IN (".implode(',',$permission_group_ids).")");
	foreach ($rows as $r) {
		if ($r['can_edit_executed_payments']) return true;
	}
	return false;
}




function expenses_custom_reports_get_external_column_value($parameters, &$value) {
	$external_id = array_var($parameters, 'external_id');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');

	if ($manager instanceof Expenses) {
		if ($field == 'status') {
			$st = ExpensesStatuses::findById($external_id);
			if ($st instanceof ExpensesStatus) {
				if (trim($st->getCode()) != '') {
					$value = clean(lang($st->getCode()));
				} else {
					$value = clean($st->getName());
				}
			} else {
				$value = "";
			}
		}
	}
}

function expenses_custom_reports_get_possible_external_column_values($parameters, &$values) {
	$ot_id = array_var($parameters, 'ot_id');
	$field = array_var($parameters, 'field');
	$ot = ObjectTypes::findById($ot_id);

	if ($ot->getName() == 'expense') {

		if ($field == 'status') {
			$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
			foreach ($all_status as $status) {
				$values[] = array('id' => $status['id'], 'name' => $status['name']);
			}
		}
	}
}



function expenses_list_totals_accumulate_total_rows($params, &$info) {
	$ot = array_var($params, 'object_type');
	if ($ot instanceof ObjectType && $ot->getName() == 'expense') {
		if (!is_array($info)) $info = array();
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $st) {
			if ($st['only_for_expenses']) continue;
			
			$status_id = $st['id'];
			$status = 'total_'.$st['code'];
			
			$info[$status] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency', 'group_by' => 'currency');
		}
		
	} else if ($ot instanceof ObjectType && $ot->getName() == 'payment') {
		if (!is_array($info)) $info = array();
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $st) {
			if ($st['only_for_expenses']) continue;
				
			$status_id = $st['id'];
			$status = $st['code'];
				
			$info[$status] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency', 'group_by' => 'currency');
		}
	}
}


function expenses_more_list_total_columns($parameters, &$total_rows) {
	$ot = array_var($parameters, 'object_type');
	$params = array_var($parameters, 'original_params');

	if ($ot instanceof ObjectType && $ot->getName() == 'expense') {
		// get the totals from the payments
		$result = Payments::instance()->listing(array("raw_data" => true));
		
		// for each status set the total calculated from the payments
		if (isset($result->totals)) {
			$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
			foreach ($all_status as $st) {
				if ($st['only_for_expenses']) continue;
				
				if (isset($result->totals[$st['code']])) {
					$total_rows[0]["total_".$st['code']] = $result->totals[$st['code']];
				}
			}
			// avoid further process of the totals, they are already processed
			$total_rows['totals_already_formatted'] = true;
		}
		
	} else if ($ot instanceof ObjectType && $ot->getName() == 'payment') {
		
		$from_sql = array_var($params, 'from_sql');
		$joins_sql = array_var($params, 'joins_sql');
		$conditions_sql = array_var($params, 'conditions_sql');
		
		$group_by_sql = "GROUP BY o.id, pt.status_id, pt.currency_id";
		$joins_sql .= "
				LEFT JOIN ".TABLE_PREFIX."payment_totals pt ON pt.payment_id=o.id
		";
		
		if (is_array($params['member_ids'])) {
			foreach ($params['member_ids'] as $mid) {
				if (is_numeric($mid) && $mid > 0) {
					$conditions_sql .= "
					and exists (select object_id from ".TABLE_PREFIX."object_members where object_id=o.id and member_id=$mid) ";
				}
			}
		}
		
		$columns = "SUM(pt.amount)/count(*) as total_amount, pt.status_id, pt.currency_id";
		
		$totals_sql = "
			SELECT $columns
				$from_sql
				$joins_sql
			WHERE
				$conditions_sql
				$group_by_sql
		";
		
		if (!is_array($total_rows)) $total_rows = array();
		
		$rows = DB::executeAll($totals_sql);
		
		$data = array();
		if (is_array($rows)) {
			foreach ($rows as $r) {
				$st = ExpensesStatuses::instance()->getStatus($r['status_id']);
				if ($st instanceof ExpensesStatus) {
					$status = $st->getCode();
					$total_rows[] = array('currency' => $r['currency_id'], $status => $r['total_amount']);
				}
			}
		}
		
	}
	
	Hook::fire('more_list_total_columns_expenses', $parameters, $total_rows);
}



function expenses_custom_reports_additional_columns($args, &$ret) {
	$ot = array_var($args, 'object_type');

	if ($ot instanceof ObjectType && $ot->getName()=='expense') {

		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;
			$ret[] =  array('id' => 'total_'.$status['code'], 'name' => $status['name'], 'type' => DATA_TYPE_STRING);
		}
	} else if ($ot instanceof ObjectType && $ot->getName()=='payment') {
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;
			$ret[] =  array('id' => $status['code'], 'name' => $status['name'], 'type' => DATA_TYPE_STRING);
		}
	}
}

function expenses_custom_reports_fixed_additional_columns_def($args, &$results) {
	$ot = array_var($args, 'object_type');
	$field = array_var($args, 'field');

	if ($ot instanceof ObjectType && ($ot->getName()=='expense' || $ot->getName()=='payment')) {

		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($ot->getName()=='payment') {
				$f = $status['code'];
			} else {
				$f = 'total_'.$status['code'];
			}
			if ($f == $field) {
				$results['columns']['names'][$field] = $status['name'];
				$results['columns']['order'][] = $field;
			}
		}
		
	}
}

function expenses_custom_reports_override_column_format($parameters, &$value) {
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($manager instanceof Expenses || $manager instanceof Payments) {
		
		$status_by_code = array();
		$amount_columns = array();
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;
			
			if ($manager instanceof Payments) {
				$amount_columns[] = $status['code'];
			} else {
				$amount_columns[] = 'total_'.$status['code'];
			}
			$status_by_code[$status['code']] = $status;
		}
		
		if (in_array($field, $amount_columns)) {
			
			$st_code = str_replace('total_', '', $field);
			$st = $status_by_code[$st_code];
			
			$amount_html = "";
			if ($manager instanceof Payments) {
				$amount_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."payment_totals WHERE payment_id=".$object->getId()." AND status_id=".$st['id']);
				
				if (is_array($amount_rows)) {
				  foreach ($amount_rows as $row) {
					$currency = Currencies::getCurrency($row['currency_id']);
					$csymbol = "";
					if ($currency instanceof Currency) {
						$csymbol = $currency->getSymbol();
					}
					$formatted = format_money_amount($row['amount'], $csymbol);
					
					$amount_html .= ($amount_html=="" ? "" : "<br/>") . $formatted;
				  }
				}
				$value = '<div class="right">'.$amount_html.'</div>';
			
			} else {
				$array_info = $object->getArrayInfo(false, true);
				$amounts = array_var($array_info, $field, array());
				foreach ($amounts as $k => $v) {
					$value .= is_array($v) ? implode("<br/>", $v) : $v;
				}
				$value = '<div class="right">'.$value.'</div>';
			}
			
		}
	}
}


function expenses_modify_group_totals_format($params, &$results) {
	$ot = array_var($params, 'object_type');
	
	if ($ot instanceof ObjectType && $ot->getName()=='expense') {
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
			
		foreach ($results as &$row) {
			foreach ($all_status as $status) {
				if ($status['only_for_expenses']) continue;
				
				if (isset($row["total_".$status['code']])) {
					$cid = $row["currency_id_".$status['code']];
					$row["currency"] = $cid;
				}
			}
		}
	}
}


/**
 * Calculate the group totals for each expense status after the other field totals have been calculated
 */
function expenses_more_report_group_totals($params, &$level_gr_totals) {
	$report_ot = array_var($params, 'object_type');
	
	if ($report_ot && $report_ot->getName() == 'expense') {
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;
			
			$payment_context_cond = "";
			
			$active_member_ids = active_context_members(false);
			if (count($active_member_ids) > 0) {
				$payment_context_cond = "
					AND EXISTS (
						SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
						WHERE om.member_id IN (".implode(',', $active_member_ids).") AND et".$status['id'].".payment_id = om.object_id
						GROUP BY object_id
						HAVING count(member_id) = ".count($active_member_ids)."
					)";
			}
			
			// make the group query for each status separately, to avoid duplicate rows from joins 
			// also group the totals by currency
			$group_sql = "
				SELECT ".$params['sql_cols'].", 
						sum(et".$status['id'].".amount) as total_".$status['code'].", 
						et".$status['id'].".currency_id as currency_id_".$status['code']."
				FROM ".$params['sql_from']."
				
				LEFT JOIN ".TABLE_PREFIX."payment_totals et".$status['id']."
					ON et".$status['id'].".payment_id IN (SELECT pp.object_id FROM ".TABLE_PREFIX."payments pp WHERE pp.expense_id=e.object_id)
					AND et".$status['id'].".status_id=".$status['id']."
					$payment_context_cond
					
				WHERE ".$params['sql_where']."
				".$params['sql_gb'].", currency_id_".$status['code']."
			";
			
			$more_totals = DB::executeAll($group_sql);
			
			$new_total_rows = array();
			
			// foreach total row, append the current status' total currency and amount
			foreach ($level_gr_totals as $k => &$lgt) {
				
				// get this row's group keys
				$keys = array();
				$i = 0;
				foreach ($lgt as $key => $val) {
					if (str_starts_with($key, "_group_id")) {
						$keys[] = $key;
						$i++;
						if ($i > $params['level']) break;
					}
				}
				
				foreach ($more_totals as $more_total) {
					// process only the rows of $more_totals that correspond to the current level total row ($lgt)
					$keys_match = true;
					foreach ($keys as $key) {
						if ($more_total[$key] != $lgt[$key]) $keys_match = false;
					}
					
					if ($keys_match && $more_total["currency_id_".$status['code']]) {
						
						// if there is more than one total for the same group (more than one currency) 
						// => clone row and only put the new currency totals
						if (isset($lgt["total_".$status['code']])) {
							
							$clone_total_row = array();
							foreach ($keys as $key) {
								$clone_total_row[$key] = $lgt[$key];
							}
							
							$clone_total_row["total_".$status['code']] = $more_total["total_".$status['code']];
							$clone_total_row["currency_id_".$status['code']] = $more_total["currency_id_".$status['code']];
							
							$new_total_rows[] = $clone_total_row;
							
						} else {
							// append the totals to the existing total row
							$lgt["total_".$status['code']] = $more_total["total_".$status['code']];
							$lgt["currency_id_".$status['code']] = $more_total["currency_id_".$status['code']];
						}
						
					}
				}
				
			}
			
			// append the new total rows for all currencies
			foreach ($new_total_rows as $new_total_row) {
				$level_gr_totals[] = $new_total_row;
			}
			
		}
	}

}



function expenses_modify_group_totals_sql($params, &$results) {
	
	$ot = array_var($params, 'object_type');
	
	if ($ot instanceof ObjectType && $ot->getName()=='payment') {
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;

			if (!isset($results['cols_sql'])) $results['cols_sql'] = array();
			if (!isset($results['join_sql'])) $results['join_sql'] = array();
			if (!isset($results['gb_sql'])) $results['gb_sql'] = array();
			
			$results['cols_sql'][] = " SUM(et".$status['id'].".amount) AS ".$status['code'].", et".$status['id'].".currency_id AS currency_id_".$status['code'];
			
			$results['join_sql'][] = "
				LEFT JOIN ".TABLE_PREFIX."payment_totals et".$status['id']." 
				ON et".$status['id'].".payment_id=e.object_id AND et".$status['id'].".status_id=".$status['id'];
			
			$results['gb_sql'][] = " currency_id_".$status['code'];
		}
	}
	
}


function expenses_custom_report_override_order_column($params, &$new_order_params) {
	$report = array_var($params, 'report');
	$ot = array_var($params, 'ot');
	$order_by_col = array_var($params, 'order_by_col');
	
	if ($ot instanceof ObjectType && ($ot->getName() == 'expense' || $ot->getName() == 'payment')) {
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status as $status) {
			if ($status['only_for_expenses']) continue;
			
			if ($ot->getName() == 'expense') {
				$join_table = TABLE_PREFIX."expense_totals";
				$join_col = 'expense_id';
				$col_id = 'total_'.$status['code'];
			} else {
				$join_table = TABLE_PREFIX."payment_totals";
				$join_col = 'payment_id';
				$col_id = $status['code'];
			}
			
			if ($order_by_col == $col_id) {
				$new_order_params = array(
					'order_by_col' => "jt.amount",
					'join_params' => array(
						'table' => $join_table,
						'jt_field' => $join_col,
						'e_field' => "object_id",
						'join_type' => 'left',
						'on_extra' => " AND jt.status_id=".$status['id']
					)
				);
				break;
			}
		}
	}
}




function expenses_more_report_header_info($params, &$ret) {
	
	$report_id = array_var($params, 'report_id');
	$results = array_var($params, 'results');
	$group_by = array_var($results, 'group_by_criterias');
	
	$report = Reports::findById($report_id);
	if (!$report instanceof Report) return;
	$ot = ObjectTypes::findById($report->getReportObjectTypeId());
	
	if ($ot->getName() == 'expense') {
		$active_member_ids = active_context_members(false);
		
		if (count($active_member_ids) > 0) {
			$active_context_members = Members::findAll(array("conditions" => "id IN (".implode(',', $active_member_ids).")"));
			$i=1;
			$context_member_names_str = "";
			
			foreach ($active_context_members as $m) {
				$sep = ($i == count($active_context_members)) ? " ".lang('and')." " : ", ";
				$context_member_names_str .= ($context_member_names_str == "" ? "" : $sep) . $m->getName();
				$i++;
			}
			$context_message = lang('payments and expenses totals are being filtered by their classification in', $context_member_names_str);
		}
		?>
		<div class="custom-report-info-block"><div class="context-warning"><?php echo $context_message ?></div></div>
		<?php 
	}
}


