
og.ExpenseManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.actual_from_filter = null;
	this.actual_to_filter = null;
	this.actual_status_filter = 0;
	
	this.is_loading = false;
	
	this.fields = ['object_id', 'name', 'description', 'status', 'status_code', 'date', 'memPath'];
	
  	var cps = og.custom_properties_by_type['expense'] ? og.custom_properties_by_type['expense'] : [];
  	var cp_names = [];
  	for (i=0; i<cps.length; i++) {
  		cp_names.push('cp_' + cps[i].id);
  	}
  	this.fields = this.fields.concat(cp_names);
  	
  	var exp_cols = [];
  	for (i=0; i<og.expenses.all_status.length; i++) {
  		exp_cols.push('total_' + og.expenses.all_status[i].code);
  	}
  	this.fields = this.fields.concat(exp_cols);
   	
   	var dim_names = [];
   	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		dim_names.push('dim_' + did);
	}
   	this.fields = this.fields.concat(dim_names);
	
	if (og.extra_expense_list_columns) {
		for (i=0; i<og.extra_expense_list_columns.length; i++) {
			this.fields.push(og.extra_expense_list_columns[i].dataIndex);
		}
	}
	
	if (!og.ExpenseManager.store) {
		og.ExpenseManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('expense', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'expenses',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					
					var d = this.reader.jsonData;
					this.fireEvent('after list load', "");
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("expenses"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("expenses")));
						}
					} else {
						if (d.message) {
							this.fireEvent('messageToShow', d.message, d.message_cls);
						} else {
							this.fireEvent('messageToShow', "");
						}
					}
					
					var man = Ext.getCmp("expense-manager");
					
					if (d.filters.status) {
						var items = [['0', '-- ' + lang('All') + ' --']];
						for (i=0; i<d.filters.status.length; i++) {
							items[items.length] = [d.filters.status[i].id, d.filters.status[i].name];
						}
						var status_filter = Ext.getCmp('ogExpensesStatusFilterCombo');
						status_filter.reset();
						status_filter.store.removeAll();
						status_filter.store.loadData(items);
						
						status_filter.setValue(man.actual_status_filter);
						status_filter.collapse();
					}
					
					var sm = Ext.getCmp('expense-manager').getSelectionModel();
					sm.clearSelections();
					
					og.eventManager.fireEvent('after grid panel load', {man:man, data:d});
					
					if (d && d.totals) {
						man.updateGridPagingToolbar({totalCount: d.totals.total_rows});
					} else {
						man.reloadGridPagingToolbar('expense','list_all','expense-manager');
					}
					
					Ext.getCmp('expense-manager').is_loading = false;
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
					
					Ext.getCmp('expense-manager').fireEvent('resize');
				}
			}
		});
		og.ExpenseManager.store.setDefaultSort('status', 'asc');
	}
	this.store = og.ExpenseManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderDragHandle(value, p, r, ix) {
		if (isNaN(r.data.object_id) || r.data.object_id == -1) {
			return '';
		}
		return '<div class="img-grid-drag" title="' + lang('click to drag') + '" onmousedown="var sm = Ext.getCmp(\'expense-manager\').getSelectionModel();if (!sm.isSelected('+ix+')) sm.clearSelections();sm.selectRow('+ix+', true);"></div>';
	}
	
	function renderExpandHandle(value, p, r, ix) {
		if (isNaN(r.data.object_id) || r.data.object_id == -1) {
			return '';
		}
		return '<div id="expand-handle-'+ix+'" class="db-ico ico-expand-lens cursor-pointer" title="' + lang('click to expand') + '" onclick="og.expenses.requestPayments(this, '+r.data.object_id+');"></div>';
	}
	
	var readClass = 'read-unread-' + Ext.id();
	function renderName(value, p, r) {
		
		if (isNaN(r.data.object_id) || r.data.object_id == -1) {
			return '<span class="bold" id="__total_row__">'+ og.clean(value) +'</span>';
		}
		
		var text = '<span class="bold">'+ og.clean(value) +'</span>';
		if (r.data.description && r.data.description.length > 0) {
			text += '<span class="desc"> - ' + r.data.description + '</span>';
		}
		
		var onclick = 'og.openLink(og.getUrl(\'expense\', \'view\', {id: '+r.data.object_id+'})); return false;';
		
		return String.format('<a style="font-size:120%;" href="{1}" onclick="{3}" title="{2}">{0}</a>',
				text, og.getUrl('expense', 'view', {id: r.data.object_id}), og.clean(value), onclick);
		
	}
	
	function renderDate(value, p, r) {
		return og.clean(value);
	}

	function renderIcon(value, p, r) {
		if (isNaN(r.data.object_id) || r.data.object_id == -1) {
			return '';
		}
		return '<div class="link-ico ico-expense" style="margin-top:4px;"></div>';
	}
	
	function renderAmount(value, p, r) {
		if (typeof(value) == 'undefined') {
			return '';
		}
		
		var str_val = value;
		if (typeof(value) == 'object') {
			str_val = "<div class='amounts-container'>";
			for (curr_id in value) {
				var amount = value[curr_id];
				str_val += "<div class='amount-container'>" + amount + "</div>";
			}
			str_val += "</div>";
		}
		return str_val;
	}
	
	function renderStatus(value, p, r) {
		if (r.data.status) {
			return '<span class="expense-row-status-'+r.data.status_code+'">' + value + '</span>';
		}else{
			return '';
		}
	}

	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.object_id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			return selections[0].data.object_id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange', function() {
		if (sm.getCount() <= 0) {
			actions.edit.setDisabled(true);
			actions.del.setDisabled(true);
			actions.archive.setDisabled(true);
		} else {
			actions.edit.setDisabled(false);
			actions.del.setDisabled(false);
			actions.archive.setDisabled(false);
		}
	});
	
	var cm_info = [
		sm,{
			/*id: 'draghandle',
			header: '&nbsp;',
			dataIndex: 'draghandle',
			width: 18,
        	renderer: renderDragHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{*/
			id: 'expand',
			header: '&nbsp;',
			dataIndex: 'expand',
			width: 30,
        	renderer: renderExpandHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'icon',
			header: '&nbsp;',
			dataIndex: 'type',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("expense"),
			dataIndex: 'name',
			width: 120,
			renderer: renderName,
			sortable:true
        },{
			id: 'status',
			header: lang("status"),
			dataIndex: 'status',
			width: 100,
			renderer: renderStatus,
			sortable:true
        },{
			id: 'date',
			header: lang("dates"),
			dataIndex: 'date',
			width: 60,
			align: 'center',
			renderer: renderDate,
			sortable:true
        }];
	
	// expense columns
	for (i=0; i<og.expenses.all_status.length; i++) {
		if (og.expenses.all_status[i].only_for_expenses) continue;
		cm_info.push({
			id: 'total_' + og.expenses.all_status[i].code,
			header: og.expenses.all_status[i].name,
			dataIndex: 'total_' + og.expenses.all_status[i].code,
			sortable: true,
			align: 'right',
			renderer: renderAmount
		});
	}
	
	if (og.extra_expense_list_columns) {
		for (i=0; i<og.extra_expense_list_columns.length; i++) {
			cm_info.push(og.extra_expense_list_columns[i]);
		}
	}
        
    // custom property columns
	var cps = og.custom_properties_by_type['expense'] ? og.custom_properties_by_type['expense'] : [];
	this.addCustomPropertyColumns(cps, cm_info, 'expense-manager');
	
	// dimension columns
	// Commented because if these columns are before the expense columns, when rendering the payments table cells are in the wrong order
	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		var key = 'lp_dim_' + did + '_show_as_column';
		if (og.preferences['listing_preferences'][key]) {
			cm_info.push({
				id: 'dim_' + did,
				header: og.dimensions_info[did].name,
				dataIndex: 'dim_' + did,
				sortable: true,
				renderer: og.renderDimCol
			});
			og.breadcrumbs_skipped_dimensions[did] = did;
		}
	}
	
	// create column model
    var cm = new Ext.grid.ColumnModel(cm_info);
    cm.on('hiddenchange', this.afterColumnShowHide, this);

    
	cm.defaultSortable = false;
		
	actions = {
		newCO: new Ext.Action({
			id: 'new_button_expense',
			text: lang('add new expense'),
            tooltip: lang('add new expense'),
            iconCls: 'ico-new new_button',
            hidden: og.replace_list_new_action && og.replace_list_new_action.expense,
            handler: function() {
				og.render_modal_form('', {c:'expense', a:'add', position:[60]});
			}
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				og.render_modal_form('', {c:'expense', a:'edit', params: {id:getFirstSelectedId()}, position:[60]});
			},
			scope: this
		}),
		del: new Ext.Action({
			text: lang('move to trash'),
            tooltip: lang('move selected objects to trash'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				var confirm_trash_config = parseInt(og.preferences['enableTrashConfirmation']);
				
				if (og.confirmNorification(lang('confirm move to trash'), confirm_trash_config)) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		}),
		archive: new Ext.Action({
			text: lang('archive'),
            tooltip: lang('archive selected object'),
            iconCls: 'ico-archive-obj',
			disabled: true,
			handler: function() {
				var confirm_archive_config = parseInt(og.preferences['enableArchiveConfirmation']);
				
				if (og.confirmNorification(lang('confirm archive selected objects'), confirm_archive_config)) {
					this.load({
						action: 'archive',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		}),
		expand_all: new Ext.Action({
			id: 'expand-all-btn',
			text: lang('expand all payments'),
            tooltip: lang('expand all payments'),
            iconCls: 'ico-expand-lens',
			handler: function() {
				$("#"+this.id+" .x-grid3-row .db-ico.ico-expand-lens").click();
				this.getSelectionModel().clearSelections();
				Ext.getCmp('collapse-all-btn').show();
				Ext.getCmp('expand-all-btn').hide();
			},
			scope: this
		}),
		collapse_all: new Ext.Action({
			id: 'collapse-all-btn',
			text: lang('collapse all payments'),
            tooltip: lang('collapse all payments'),
            iconCls: 'ico-collapse-lens',
            hidden: true,
			handler: function() {
				$("#"+this.id+" .x-grid3-row .db-ico.ico-collapse-lens").click();
				this.getSelectionModel().clearSelections();
				Ext.getCmp('expand-all-btn').show();
				Ext.getCmp('collapse-all-btn').hide();
			},
			scope: this
		}),
		switch_to_payments: new Ext.Action({
			text: lang('show payments list'),
			tooltip: lang('show plain payments list'),
			iconCls: 'ico-properties',
			handler: function() {
				this.hide();
				
				var pman = Ext.getCmp("payment-manager");
				pman.show();
				pman.load();
			},
			scope: this
		})
    };
	
	var uid = Ext.id();
	filters = {
		status_filter: new Ext.form.ComboBox({
	    	id: 'ogExpensesStatusFilterCombo',
	    	store: new Ext.data.SimpleStore({
		        fields: ['value', 'text'],
		        data : []
		    }),
		    displayField:'text',
	        //typeAhead: true,
	        mode: 'local',
	        triggerAction: 'all',
	        selectOnFocus:true,
	        width:160,
	        valueField: 'value',
	        valueNotFoundText: '',
	        listeners: {
	        	'select' : function(combo, record) {
					var man = Ext.getCmp("expense-manager");
					man.actual_status_filter = combo.getValue();
					man.reloadFiltering();
	        	}
	        }
		}),
		filter_from: new Ext.Action({
			id: uid + 'filter_date_from',
			text: lang('select a date'),
	        tooltip: lang('select a date'),
	        menu: new og.drawDateMenuPicker({
	        	items:[new Ext.menu.Item({
	        		id: uid + 'filter_from_remove',
	        		text: lang('remove filter'),
	        		iconCls: 'ico-delete',
	        		hidden: true,
	        		handler: function() {
		        		var man = Ext.getCmp('expense-manager');
	        			man.actual_from_filter = null;
	        			man.reloadFiltering();
	        			this.hide();
	        			Ext.getCmp(uid + 'filter_date_from').setText(lang('select a date'));
	        		}
	        	})],
	        	listeners: {
	        		'select': function(dp, date) {
			        	var man = Ext.getCmp('expense-manager');
		    			man.actual_from_filter = date.format(og.preferences['date_format']);
			        	Ext.getCmp(uid + 'filter_date_from').setText(date.format(og.preferences['date_format']));
			        	man.reloadFiltering();
			        	Ext.getCmp(uid + 'filter_from_remove').show();
	        		}
		        }
	        })
		}),
		filter_to: new Ext.Action({
			id: uid + 'filter_date_to',
			text: lang('select a date'),
	        tooltip: lang('select a date'),
	        menu: new og.drawDateMenuPicker({
	        	items:[new Ext.menu.Item({
	        		id: uid + 'filter_to_remove',
	        		text: lang('remove filter'),
	        		iconCls: 'ico-delete',
	        		hidden: true,
	        		handler: function() {
		        		var man = Ext.getCmp('expense-manager');
	        			man.actual_to_filter = null;
	        			man.reloadFiltering();
	        			this.hide();
	        			Ext.getCmp(uid + 'filter_date_to').setText(lang('select a date'));
	        		}
	        	})],
	        	listeners: {
	        		'select': function(dp, date) {
	        			var man = Ext.getCmp('expense-manager');
	        			man.actual_to_filter = date.format(og.preferences['date_format']);
			        	Ext.getCmp(uid + 'filter_date_to').setText(date.format(og.preferences['date_format']));
			        	man.reloadFiltering();
			        	Ext.getCmp(uid + 'filter_to_remove').show();
	        		}
		        }
	        })
		})
	}
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		if (og.replace_list_new_action && og.replace_list_new_action.expense) {
			tbar.push(og.replace_list_new_action.expense);
		}
		
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.archive);
		tbar.push(actions.del);
		tbar.push('-');
		tbar.push(actions.expand_all);
		tbar.push(actions.collapse_all);
		tbar.push('-');
		
	}
//	tbar.push(actions.switch_to_payments);
	
	this.topTbar1 = new Ext.Toolbar({
		style: 'border:0px none;',
		items: tbar
	});
	
	var tbar2 = [];
	tbar2.push('&nbsp;'+lang('filters')+": ");
	tbar2.push(' ');
	tbar2.push('&nbsp;'+lang('status') + ": ");
	tbar2.push(filters.status_filter);
	tbar2.push(' ');
	tbar2.push('&nbsp;'+lang('from date') + ": ");
	tbar2.push(filters.filter_from);
	tbar2.push(' ');
	tbar2.push('&nbsp;'+lang('to date') + ": ");
	tbar2.push(filters.filter_to);
	
	if (og.additional_list_actions && og.additional_list_actions.expense) {
		tbar2.push(' ');
		for (var i=0; i<og.additional_list_actions.expense.length; i++) {
			tbar2.push(og.additional_list_actions.expense[i]);
		}
	}
	
	og.ExpenseManager.superclass.constructor.call(this, {
		renderTo: 'exp-tab-expenses',
		store: this.store,
		layout: 'fit',
		cm: cm,
		enableDrag: true,
		ddGroup: 'MemberDD',
		stateful: og.preferences['rememberGUIState'],
		id: 'expense-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),/*
		viewConfig: {
			forceFit: true
		},*/
		sm: sm,
		tbar: this.topTbar1,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});
					
					// add the second toolbar
					this.topTbar2 = new Ext.Toolbar({
					    renderTo: this.tbar,
					    items: tbar2
					});
				},
				scope: this
			},
			'resize' : {
				fn: function() {
					setTimeout(function(){
						// fit columns in the view
						var v = Ext.getCmp("expense-manager").getView();
						if (v && v.forceFit) {
							v.fitColumns();
						}

						// calculate the hieght
						var h = $("#expense-manager").parent().height();
						var tb_h = $("#expense-manager .x-panel-tbar.x-panel-tbar-noheader").height();
						var tabs_h = $("#expenses-tabs-panel .x-tab-panel-header").height();
						var new_h = h - tb_h - tabs_h + 6;
						
						// adjust containers width
						$("#expense-manager .x-grid3").css('height', new_h+'px');
						$("#expense-manager .x-grid3 .x-grid3-scroller").css('height', (new_h - 26)+'px');
					}, 200);
				}
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.ExpenseManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}

		this.store.baseParams.context = og.contextManager.plainContext();
		this.store.baseParams.status_filter = params.status_filter ? params.status_filter : this.actual_status_filter;

		
		if (params.actual_from_filter) this.store.baseParams.from_filter = params.actual_from_filter;
		if (params.actual_to_filter) this.store.baseParams.to_filter = params.actual_to_filter;
		
		// don't make the request if panel is hidden
		if (!this.hidden) {
			this.store.removeAll();
			
			this.updateColumnModelHiddenColumns();
			
			this.store.load({
				params: Ext.apply(params, {
					start: start,
					limit: og.config['files_per_page']
				}),
				callback: function(){
					var el = Ext.getCmp('expand-all-btn');
					if (el) el.show();
					var el2 = Ext.getCmp('collapse-all-btn');
					if (el2) el2.hide();
				}
			});
		}
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.reset();
		}
	},
	
	reset: function() {
		var params = {start:0};
		if (this.actual_from_filter) params.from_filter = this.actual_from_filter;
		if (this.actual_to_filter) params.to_filter = this.actual_to_filter;
		
		this.load(params);
	},
	
	showMessage: function(text, msg_cls) {
		if (msg_cls) {
			this.innerMessage.className += " " + msg_cls;
		} else {
			this.innerMessage.className = "inner-message";
		}
		this.innerMessage.innerHTML = text;
	},
	
	reloadFiltering: function() {
		if (this.isLoading()) return;
		this.is_loading = true;
		
		var config = {
			start:0, 
			status_filter: this.actual_status_filter
		};
		if (this.actual_from_filter) config.from_filter = this.actual_from_filter;
		if (this.actual_to_filter) config.to_filter = this.actual_to_filter;
		
		this.load(config);
	},
	
	isLoading: function() {
		return this.is_loading;
	},
	
	trashObjects: function() {
		if (confirm(lang('confirm move to trash'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	},
	
	archiveObjects: function() {
		if (confirm(lang('confirm archive selected objects'))) {
			this.load({
				action: 'archive',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
});


Ext.reg("expenses", og.ExpenseManager);

/************************************************
Container for ExpenseManager,
*************************************************/
og.ExpenseManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	var expenses_subtabs = [
		{title:lang("expenses"), id: 'subtab-expense', html:'<div id="exp-tab-expenses" style="height:100%;"></div>'},
		{title:lang("payments"), id: 'subtab-payment', html:'<div id="exp-tab-payments" style="height:100%;"></div>'}
	];
	
	if (og.expenses.modify_expenses_subtabs) {
		for (var i=0; i<og.expenses.modify_expenses_subtabs.length; i++) {
			var fn = og.expenses.modify_expenses_subtabs[i];
			if (typeof(fn) == 'function') {
				fn.call(null, expenses_subtabs);
			}
		}
	}
	
	og.ExpenseManagerPanel.superclass.constructor.call(this, {
		id: 'expense-manager-container',
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
	        new Ext.TabPanel({
				id: 'expenses-tabs-panel',
				cls: 'subtab',
				activeTab: 0,
				enableTabScroll: false,
				items: expenses_subtabs,
				listeners: {
					resize: function(tabp, adjWidth, adjHeight, rawWidth, rawHeight) {
						if (!tabp) return;
						var t = tabp.activeTab;
						if (!t) return;
						if (t.id == 'subtab-expense') {
							if (og.ExpenseManagerPanel.manager) og.ExpenseManagerPanel.manager.fireEvent("resize");
						}
						if (t.id == 'subtab-payment') {
							if (og.ExpenseManagerPanel.payments_panel) og.ExpenseManagerPanel.payments_panel.fireEvent("resize");
						}
						if (og.ExpenseManagerPanel.other_panels) {
							for (var i=0; i<og.ExpenseManagerPanel.other_panels.length; i++) {
								og.ExpenseManagerPanel.other_panels[i].fireEvent("resize");
							}
						}
					},
					tabchange: function(tabp, t) {
						if (t.id == 'subtab-expense') {
							
							if (!og.ExpenseManagerPanel.manager) {
								og.ExpenseManagerPanel.manager = new og.ExpenseManager();
								og.ExpenseManagerPanel.manager.load();
							}
							og.ExpenseManagerPanel.manager.fireEvent('resize');
						
						} else if (t.id == 'subtab-payment') {
							
							if (!og.ExpenseManagerPanel.payments_panel) {
								og.ExpenseManagerPanel.payments_panel = new og.ObjectGrid({
									renderTo: 'exp-tab-payments',
									url: og.getUrl('payment', 'list_all'),
									type_name: 'payment',
									response_objects_root: 'payments',
									grid_id: 'payment-manager',
									nameRenderer: og.gridObjectNameRenderer,
									fixed_height: true,
									forceFit: false,
									store_params: {
										url_controller: 'payment',
										url_action: 'list_all'
									},
									//hidden: true, // initializate this panel hidden
									checkbox_sel_model: true,
									columns: og.expenses.payment_list_grid_config.columns,
									add_default_actions_column: false,
									tbar_items: og.expenses.payment_list_grid_config.tbar_items,
									filters: og.expenses.payment_list_grid_config.filters
								});
								og.ExpenseManagerPanel.payments_panel.load();
							}
							og.ExpenseManagerPanel.payments_panel.fireEvent('resize');
							
						} else {
							if (t.panel_id) {
								var subpanel = Ext.getCmp(t.panel_id);
								if (!subpanel) {
									og.expenses.loadSubtab(t);
								} else {
									subpanel.fireEvent('resize');
								}
							}
						}
					}
					
				},
				scope: this
			})
		],
		scope: this
	});
}

Ext.extend(og.ExpenseManagerPanel, Ext.Panel, {
	load: function(params) {
		if (og.ExpenseManagerPanel.manager) og.ExpenseManagerPanel.manager.load(params);
		if (og.ExpenseManagerPanel.payments_panel) og.ExpenseManagerPanel.payments_panel.load(params);
		
		if (og.ExpenseManagerPanel.other_panels) {
			for (var i=0; i<og.ExpenseManagerPanel.other_panels.length; i++) {
				og.ExpenseManagerPanel.other_panels[i].load(params);
			}
		}
	},
	activate: function() {
		if (og.ExpenseManagerPanel.manager) og.ExpenseManagerPanel.manager.activate();
		if (og.ExpenseManagerPanel.payments_panel) og.ExpenseManagerPanel.payments_panel.activate();
		
		if (og.ExpenseManagerPanel.other_panels) {
			for (var i=0; i<og.ExpenseManagerPanel.other_panels.length; i++) {
				og.ExpenseManagerPanel.other_panels[i].activate();
			}
		}
	},	
	reset: function() {
		if (og.ExpenseManagerPanel.manager) og.ExpenseManagerPanel.manager.reset();
		if (og.ExpenseManagerPanel.payments_panel) og.ExpenseManagerPanel.payments_panel.reset();
		
		if (og.ExpenseManagerPanel.other_panels) {
			for (var i=0; i<og.ExpenseManagerPanel.other_panels.length; i++) {
				og.ExpenseManagerPanel.other_panels[i].reset();
			}
		}
	},	
	showMessage: function(text) {
		if (this.manager) this.manager.showMessage(text);
	}
});

Ext.reg("expenses-containerpanel", og.ExpenseManagerPanel);


Ext.grid.GridView.override({
	getRowClass: function (record, index, rowParams, store) {
		return "";
		if (record.data.status) {
			return "expense-row-status-"+record.data.status;
		}else{
			return "";
		}
	},
	focusCell : function(row, col, hscroll){
		this.syncFocusEl(this.ensureVisible(row, col, hscroll));
		this.focusEl.focus.defer(1, this.focusEl);
	},

    syncFocusEl : function(row, col, hscroll){
        var xy = row;
        if(!Ext.isArray(xy)){
            row = Math.min(row, Math.max(0, this.getRows().length-1));
            //xy = this.getResolvedXY(this.resolveCell(row, col, hscroll));
        }
        this.focusEl.setXY(xy||this.scroller.getXY());
    }
});