<?php

/**
 * ActualExpenseTax controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ActualExpenseTaxController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	/**
	 * Construct the ActualExpenseTaxController
	 *
	 * @access public
	 * @param void
	 * @return ActualExpenseTaxController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
			Env::useHelper('functions', 'multiple_currencies');
		}
		Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("ActualExpenseTaxesManager.js", $this->plugin_name);
		ajx_current("panel", "actual-expense-taxes-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		
		// Get all variables from request
		$start = array_var($_GET, 'start', 0);
		$limit = array_var($_GET, 'limit', config_option('files_per_page'));
		$order = array_var($_GET, 'sort');
		$order_dir = strtoupper(array_var($_GET, 'dir'));
		
		if (!is_numeric($start)) $start = 0;
		if (!is_numeric($limit)) $limit = config_option('files_per_page');
		
		$action = array_var($_GET, 'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET, 'ids')),
		);
		
		//Resolve actions to perform
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}

		$joins = array();
		$select_cols = array();
		
		if (!in_array($order_dir, array('ASC', 'DESC'))) {
			$order_dir = 'ASC';
		}
		
		switch ($order) {
			case 'name':
			case 'value':
			case 'valid_from':
			case 'valid_to':
			case 'select_by_default':
			case 'is_percentage':
				$order = "d.$order";
				break;
			default:
				$order = 'd.name';
		}
		
		
		$select_cols_sql = (count($select_cols) > 0 ? ",".implode(',',$select_cols) : "");
		$joins_sql = (count($joins) > 0 ? implode(' ', $joins) : "");
		$order_sql = "ORDER BY $order $order_dir";
		$limit_sql = "LIMIT $start, $limit";
		
		$conditions_sql = "true";
		
		$sql = "SELECT d.* $select_cols_sql FROM ".TABLE_PREFIX."actual_expense_taxes d $joins_sql WHERE $conditions_sql $order_sql $limit_sql";
		
		$taxes = DB::executeAll($sql);
		
		$count_sql = "SELECT count(d.id) as total FROM ".TABLE_PREFIX."actual_expense_taxes d $joins_sql WHERE $conditions_sql";
		$total_rows = DB::executeAll($count_sql);
		
		// Prepare response object
		$object = $this->prepareObject($taxes, $start, $limit, $total_rows[0]['total']);
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	

	private function prepareObject($taxes, $start, $limit, $total) {
		$object = array(
			"totalCount" => $total,
			"start" => $start,
			"tax" => array(),
		);
		
		for ($i = 0; $i < $limit; $i++){
			$tax_data = array_var($taxes, $i);
			
			if (!is_null($tax_data)) {
				
				if ($tax_data['is_percentage']) {
					$tax_data['value'] = $tax_data['value'] . ' %';
				} else {
					$currency = Currencies::findById($tax_data['currency_id']);
					$c_symbol = $currency instanceof Currency ? $currency->getSymbol() : "$";
					$tax_data['value'] = format_money_amount($tax_data['value'], $c_symbol, config_option('product_type_decimals'));
				}
				
				$tax_data['is_percentage'] = $tax_data['is_percentage'] ? lang('yes') : lang('no');
				$tax_data['select_by_default'] = $tax_data['select_by_default'] ? lang('yes') : lang('no');
				
				if ($tax_data['valid_from'] != EMPTY_DATETIME) {
					$valid_from_dt = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $tax_data['valid_from']);
					$tax_data['valid_from'] = $valid_from_dt ? format_date($valid_from_dt) : '--';
				} else {
					$tax_data['valid_from'] = '--';
				}
				
				if ($tax_data['valid_to'] != EMPTY_DATETIME) {
					$valid_to_dt = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $tax_data['valid_to']);
					$tax_data['valid_to'] = $valid_to_dt ? format_date($valid_to_dt) : '--';
				} else {
					$tax_data['valid_to'] = '--';
				}
				$object["tax"][$i] = $tax_data;
			}
		}
		
		return $object;
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 */
	private function resolveAction($action, $attributes){
		 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$tax = ActualExpenseTaxes::findById($id);
					if (can_manage_billing(logged_user()) && $tax instanceof ActualExpenseTax){
						try{
							DB::beginWork();
							
							$tax->delete();
							
							DB::commit();
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
		
	}
	
	
	

	/**
	 * View single category
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$object = ActualExpenseTaxes::findById(get_id());
		if(!($object instanceof ActualExpenseTax)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		tpl_assign('object', $object);
		
		ajx_extra_data(array("title" => $object->getName()));
		ajx_set_no_toolbar(true);
		
	} // view
	

	/**
	 * Add billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$tax = new ActualExpenseTax();
		tpl_assign('tax', $tax);

		$data = array_var($_POST, 'tax');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_actual_expense_tax($data, $tax);
				
				DB::commit();
				
				flash_success(lang('success add tax', $tax->getName()));
				
	          	if (array_var($_REQUEST, 'modal')) {
	          		evt_add("reload current panel");
          		} else {
          			ajx_current("back");
          		}

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		$tax = ActualExpenseTaxes::findById(get_id());
		if (!$tax instanceof ActualExpenseTax) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('tax', $tax);
		
		$data = array_var($_POST, 'tax');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_actual_expense_tax($data, $tax);
				
				DB::commit();
				
				flash_success(lang('success edit tax', $tax->getName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	private function save_actual_expense_tax($data, ActualExpenseTax $tax) {
		
		$tax->setFromAttributes($data);
		$tax->save();
		
		return $tax;
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$tax = ActualExpenseTaxes::findById(get_id());
		if (!$tax instanceof ActualExpenseTax) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			$name = $tax->getName();
			DB::beginWork();
			
			$tax->delete();
		
			DB::commit();
		
			flash_success(lang('success delete tax', $name));
		
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			} else {
				ajx_current("back");
			}
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete

}
