<?php

/**
 * ProductType controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ProductTypeController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	/**
	 * Construct the ProductTypeController
	 *
	 * @access public
	 * @param void
	 * @return ProductTypeController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
			Env::useHelper('functions', 'multiple_currencies');
		}
	//	Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("ProductTypesManager.js", $this->plugin_name);
		ajx_current("panel", "product-types-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function get_allowed_dimensions() {
		ajx_current("empty");
		ajx_extra_data(array('allowed_dimensions' => array_filter(config_option('product_type_dimensions'))));
	}
	
	function enable_disable_dimension() {
		ajx_current("empty");
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$dimension = Dimensions::getDimensionById(array_var($_GET, 'dim_id'));
		if (!$dimension instanceof Dimension) {
			flash_error(lang('dimension dnx'));
			ajx_current('empty');
			return;
		}
		
		$enable = array_var($_GET, 'enable');
		$allowed_dimensions = array_filter(config_option('product_type_dimensions'));
		
		if ($enable) {
			if (!in_array($dimension->getId(), $allowed_dimensions)) {
				$allowed_dimensions[] = $dimension->getId();
				set_config_option('product_type_dimensions', $allowed_dimensions);
			}
		} else {
			if (in_array($dimension->getId(), $allowed_dimensions)) {
				foreach ($allowed_dimensions as $k => &$d) {
					if ($d == $dimension->getId()) unset($allowed_dimensions[$k]);
				}
				set_config_option('product_type_dimensions', $allowed_dimensions);
			}
		}
		
	}
	
	function enable_disable_filter_up_dimension() {
		ajx_current("empty");
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$dimension = Dimensions::getDimensionById(array_var($_GET, 'dim_id'));
		if (!$dimension instanceof Dimension) {
			flash_error(lang('dimension dnx'));
			ajx_current('empty');
			return;
		}
		
		$enable = array_var($_GET, 'enable');
		$allowed_dimensions = array_filter(config_option('product_type_filter_up_dimensions'));
		
		if ($enable) {
			if (!in_array($dimension->getId(), $allowed_dimensions)) {
				$allowed_dimensions[] = $dimension->getId();
				set_config_option('product_type_filter_up_dimensions', $allowed_dimensions);
			}
		} else {
			if (in_array($dimension->getId(), $allowed_dimensions)) {
				foreach ($allowed_dimensions as $k => &$d) {
					if ($d == $dimension->getId()) unset($allowed_dimensions[$k]);
				}
				set_config_option('product_type_filter_up_dimensions', $allowed_dimensions);
			}
		}
		
	}
	
	function list_all() {
		ajx_current("empty");
		
		// Get all variables from request
		$start = array_var($_GET, 'start', 0);
		$limit = array_var($_GET, 'limit', config_option('files_per_page'));
		$order = array_var($_GET, 'sort');
		$order_dir = strtoupper(array_var($_GET, 'dir'));
		
		if (!is_numeric($start)) $start = 0;
		if (!is_numeric($limit)) $limit = config_option('files_per_page');
		
		$action = array_var($_GET, 'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET, 'ids')),
		);
		
		//Resolve actions to perform
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$context_conditions_array = array();
		$context = active_context();
		foreach ($context as $selection) {
			if ($selection instanceof Member) {
				$all_mem_ids = array($selection->getId());
				$all_mem_ids = array_merge($all_mem_ids, $selection->getAllChildrenIds(true));
				$context_conditions_array[] = "EXISTS (SELECT bdm.member_id FROM ".TABLE_PREFIX."product_type_members bdm 
						WHERE bdm.member_id IN (".implode(',',$all_mem_ids).") AND bdm.dimension_id='".$selection->getDimensionId()."' AND bdm.product_type_id=d.id)";
			}
		}
		$context_conditions_sql = "";
		if (count($context_conditions_array) > 0) {
			$context_conditions_sql = " AND ".implode(" AND ", $context_conditions_array);
		}


		$joins = array();
		$select_cols = array();
		
		
		if (!in_array($order_dir, array('ASC', 'DESC'))) {
			$order_dir = 'ASC';
		}
		switch ($order) {
			case 'hourly_rate':
				$order = "d.enable_rate $order_dir, d.$order";
				break;
			case 'hourly_cost':
				$order = "d.enable_cost $order_dir, d.$order";
				break;
			case 'name':
			case 'updated_on':
			case 'updated_by_id':
				$order = "d.$order";
				break;
			case 'user_id':
				$order = "user_name";
				break;
			case 'price_multiplier':
				$order = "price_multiplier";
				break;
			default:
				if (str_starts_with($order, 'dim_')) {
					$joins[] = " LEFT JOIN ".TABLE_PREFIX."members mem ON dm".$did.".member_id=mem.id";
					$select_cols[] = "mem.name as order_member_name";
					$order = "order_member_name";
				}
				if (in_array($order, ProductTypes::instance()->getColumns())) {
					$order = 'd.'.$order;
				} else {
					$order = 'd.name';
				}
		}
		
		
		// join for category user
		$joins[] = " LEFT JOIN ".TABLE_PREFIX."objects o ON o.id=d.user_id ";
		$select_cols[] = "o.name as user_name";
		
		$select_cols_sql = (count($select_cols) > 0 ? ",".implode(',',$select_cols) : "");
		$joins_sql = (count($joins) > 0 ? implode(' ', $joins) : "");
		$order_sql = "ORDER BY $order $order_dir";
		$limit_sql = "LIMIT $start, $limit";
		
		$conditions_sql = "true $context_conditions_sql";
		
		$sql = "SELECT d.* $select_cols_sql FROM ".TABLE_PREFIX."product_types d $joins_sql WHERE $conditions_sql $order_sql $limit_sql";
		
		$product_types = DB::executeAll($sql);
		
		$count_sql = "SELECT count(d.id) as total FROM ".TABLE_PREFIX."product_types d $joins_sql WHERE $conditions_sql";
		$total_rows = DB::executeAll($count_sql);
		
		// Prepare response object
		$object = $this->prepareObject($product_types, $start, $limit, $total_rows[0]['total']);
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	

	private function prepareObject($categories, $start, $limit, $total) {
		$object = array(
			"totalCount" => $total,
			"start" => $start,
			"product_types" => array(),
			"allowed_dimensions" => array_filter(config_option('product_type_dimensions')),
		);
		
		$amount_decimals = config_option('product_type_decimals');
		
		for ($i = 0; $i < $limit; $i++){
			$product_type_data = array_var($categories, $i);
			
			if (!is_null($product_type_data)) {
				
				$def_mem_sql = "SELECT member_id FROM ".TABLE_PREFIX."product_type_members WHERE product_type_id=".$product_type_data['id'];
				$def_member_ids = array_unique(array_filter(array_flat(DB::executeAll($def_mem_sql))));
				
				$mem_path = array();
				foreach ($def_member_ids as $member_id) {
					$mem = Members::getMemberById($member_id);
					if (!$mem instanceof Member) continue;
					
					$member_type_id = $mem->getObjectTypeId();
					$dim = $mem->getDimensionId();
					if (!isset($mem_path[$dim])) $mem_path[$dim] = array();
					if (!isset($mem_path[$dim][$member_type_id])) $mem_path[$dim][$member_type_id] = array();
					$mem_path[$dim][$member_type_id][] = $member_id;
				}
				$product_type_data['memPath'] = json_encode($mem_path);

				$rate_currency = Currencies::findById($product_type_data['rate_currency_id']);
				$rate_c_symbol = $rate_currency instanceof Currency ? $rate_currency->getSymbol() : "";
				
				$cost_currency = Currencies::findById($product_type_data['cost_currency_id']);
				$cost_c_symbol = $cost_currency instanceof Currency ? $cost_currency->getSymbol() : "";
				
				$product_type_data['user_name'] = escape_character($product_type_data['user_name']);
				if ($product_type_data['enable_rate']) {
					$product_type_data['hourly_rate'] = $rate_c_symbol .' '. number_format($product_type_data['hourly_rate'], $amount_decimals);
				} else {
					$product_type_data['hourly_rate'] = '';
				}
				if ($product_type_data['uses_price_multiplier']) {
					$price = $product_type_data['hourly_cost'] * $product_type_data['price_multiplier'];
					$product_type_data['hourly_rate'] = $rate_c_symbol .' '. number_format($price, $amount_decimals);
					
					$product_type_data['price_multiplier'] = number_format($product_type_data['price_multiplier'], $amount_decimals);
				} else {
					$product_type_data['price_multiplier'] = '';
				}
				if ($product_type_data['enable_cost']) {
					$product_type_data['hourly_cost'] = $cost_c_symbol .' '. number_format($product_type_data['hourly_cost'], $amount_decimals);
				} else {
					$product_type_data['hourly_cost'] = '';
				}
				
				$updated_on = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $product_type_data['updated_on']);
				$product_type_data['updated_on'] = format_datetime($updated_on);
				
				$product_type_data['updated_by_name'] = lang('n/a');
				$updated_by = Contacts::findById($product_type_data['updated_by_id']);
				if ($updated_by instanceof Contact) {
					$product_type_data['updated_by_name'] = clean($updated_by->getObjectName());
				}
				
				$object["product_type"][$i] = $product_type_data;
				
			}
		}
		
		return $object;
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 */
	private function resolveAction($action, $attributes){
		 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$product_type = ProductTypes::findById($id);
					if (can_manage_billing(logged_user()) && $product_type instanceof ProductType){
						try{
							DB::beginWork();
							
							ProductTypeMembers::instance()->delete("product_type_id=".$product_type->getId());
							$product_type->delete();
							
							DB::commit();
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
		
	}
	
	
	

	/**
	 * View single category
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$object = ProductTypes::findById(get_id());
		if(!($object instanceof ProductType)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		tpl_assign('object', $object);
		
		ajx_extra_data(array("title" => $object->getName()));
		ajx_set_no_toolbar(true);
		
	} // view
	

	/**
	 * Add billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$product_type = new ProductType();
		tpl_assign('product_type', $product_type);

		$data = array_var($_POST, 'product_type');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('selected_members', array());
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_product_type($data, $product_type);
				
				DB::commit();
				
				flash_success(lang('success add product type', $product_type->getName()));
				
	          	if (array_var($_REQUEST, 'modal')) {
	          		evt_add("reload current panel");
          		} else {
          			ajx_current("back");
          		}
          		
	          	ajx_add("overview-panel", "reload");

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		$product_type = ProductTypes::findById(get_id());
		if (!$product_type instanceof ProductType) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('product_type', $product_type);
		
		$data = array_var($_POST, 'product_type');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$selected_members = array_flat(DB::executeAll("SELECT member_id FROM ".TABLE_PREFIX."product_type_members 
					WHERE product_type_id=".$product_type->getId()));
			tpl_assign('selected_members', $selected_members);
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_product_type($data, $product_type);
				
				DB::commit();
				
				flash_success(lang('success edit product type', $product_type->getName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	private function save_product_type($data, $product_type) {
		
		$product_type->setFromAttributes($data);
		$product_type->save();
		
		$values_sql = array();
		foreach ($data as $key => $member_data) {
			
			if (str_starts_with($key, 'dim_')) {
				
				$member_id = array_var($member_data, 'member_id', 0);
				if (!is_numeric($member_id)) {
					if (is_string($member_id)) $member_id = json_decode($member_id);
				} else {
					$member_id = array($member_id);
				}
				
				$dim_id = substr($key, 4);
				if (is_numeric($dim_id)) {
					$member_id = array_filter(array_unique($member_id));
					foreach ($member_id as $mid) {
						$values_sql[] = "('".$product_type->getId()."','$dim_id','$mid')";
					}
				}
			}
		}
		if (count($values_sql) > 0) {
			DB::execute("DELETE FROM ".TABLE_PREFIX."product_type_members WHERE product_type_id=".$product_type->getId());
			DB::execute("INSERT INTO ".TABLE_PREFIX."product_type_members (product_type_id, dimension_id, member_id) 
					VALUES ".implode(',', $values_sql)." ON DUPLICATE KEY UPDATE product_type_id=product_type_id;");
		}
		
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$product_type = ProductTypes::findById(get_id());
		if (!$product_type instanceof ProductType) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			
			$name = $product_type->getName();
			ProductTypeMembers::instance()->delete("product_type_id=".$product_type->getId());
			$product_type->delete();
		
			DB::commit();
		
			flash_success(lang('success delete product type', $name));
		
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			} else {
				ajx_current("back");
			}
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete


	
	
	function get_filtered_product_types() {
		
		ajx_current("empty");
		
		$product_types = null;
		
		$member_ids = array();
		$member_ids_json = trim(array_var($_REQUEST, 'member_ids', ""));
		if ($member_ids_json != "") {
			$member_ids = json_decode($member_ids_json, true);
			if ($member_ids) {
				$member_ids = array_filter($member_ids);
			}
			if (count($member_ids) > 0) {
				$members = Members::findAll(array('conditions' => "id IN (".implode(',', $member_ids).")"));
				
				$product_types = ProductTypes::getFilteredProductTypesData($members);
			}
		}
		
		if (is_null($product_types)) {
			$product_types = ProductTypes::getAllProductTypesData();
		}
		
		ajx_extra_data(array('product_types' => $product_types));
	}

	function get_filtered_product_types_for_payment_receipt() {
		
		ajx_current("empty");
		
		$product_types = null;
		$expense_id = array_var($_REQUEST, 'expense_id');
		if($expense_id > 0){
			$expense_items = ExpenseItems::findAll(array('conditions' => 'expense_id ='.$expense_id));
			$product_types = array();
			foreach($expense_items as $item){
				if($item->getProductTypeId() > 0){
					/*$product_type = ProductTypes::findById($item->getProductTypeId())->getArrayInfo();
					$product_type['hourly_cost'] = $item->getUnitCost();
					$product_type['hourly_rate'] = $item->getUnitPrice();
					$product_type['uses_price_multiplier'] = 0;
					$product_types[] = $product_type;*/
					$product_types[] = ProductTypes::findById($item->getProductTypeId())->getArrayInfo();
				}
			}
		} else {
			$member_ids = array();
			$member_ids_json = trim(array_var($_REQUEST, 'member_ids', ""));
			if ($member_ids_json != "") {
				$member_ids = json_decode($member_ids_json, true);
				if ($member_ids) {
					$member_ids = array_filter($member_ids);
				}
				if (count($member_ids) > 0) {
					$members = Members::findAll(array('conditions' => "id IN (".implode(',', $member_ids).")"));
					
					$product_types = ProductTypes::getFilteredProductTypesData($members);
				}
			}
			if (is_null($product_types)) {
				$product_types = ProductTypes::getAllProductTypesData();
			}
		}

		ajx_extra_data(array('product_types' => $product_types));
	}

} // ProductTypeController

?>