<?php 

/**
 * Get estimated expenses info
 *
 */

function update_estimated_expenses_info_for_member($object) {
    Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_estimated_expenses_info_for_member_using_member_id($project_member_id);
    }
}

function update_estimated_expenses_info_for_member_using_member_id($project_member_id) {
    $expenses_info = get_estimated_expense_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $expenses_info);
    update_estimated_overall_totals_for_member($project_member_id);
    $project_member = Members::findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0){
            update_estimated_expenses_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_estimated_expense_info_for_member($project_member_id) {
    $expenses_info = array(
        'estimated_expense_billable_price' => 0,
        'estimated_expense_non_billable_price' => 0,
        'estimated_expense_total_price' => 0,
        'estimated_expense_billable_cost' => 0,
        'estimated_expense_non_billable_cost' => 0,
        'estimated_expense_total_cost' => 0
    );
    
    Env::useHelper('member_financials_functions', 'advanced_billing');
    $billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=1 ');
    $non_billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=0 ');

    calculate_estimated_expenses_values($expenses_info, $billable_expenses_info, $non_billable_expenses_info);
    
    return $expenses_info;
}

function calculate_estimated_expenses_values(&$expenses_info, $billable_expenses,$non_billable_expenses) {
     // Calculate estimated expense billable price
     $expenses_info['estimated_expense_billable_price'] += $billable_expenses['budgeted_price'] ? $billable_expenses['budgeted_price'] : 0;
     // Calculate estimated expense non-billable price
     $expenses_info['estimated_expense_non_billable_price'] += $non_billable_expenses['budgeted_price'] ? $non_billable_expenses['budgeted_price'] : 0;
     // Calculate estimated total price
     $expenses_info['estimated_expense_total_price'] += $billable_expenses['budgeted_price'] ? $billable_expenses['budgeted_price'] : 0;
     $expenses_info['estimated_expense_total_price'] += $non_billable_expenses['budgeted_price'] ? $non_billable_expenses['budgeted_price'] : 0;
 
      // Calculate estimated expense billable cost
      $expenses_info['estimated_expense_billable_cost'] += $billable_expenses['budgeted_cost'] ? $billable_expenses['budgeted_cost'] : 0;
      // Calculate estimated expense non-billable cost
      $expenses_info['estimated_expense_non_billable_cost'] += $non_billable_expenses['budgeted_cost'] ? $non_billable_expenses['budgeted_cost'] : 0;
      // Calculate estimated total cost
      $expenses_info['estimated_expense_total_cost'] += $billable_expenses['budgeted_cost'] ? $billable_expenses['budgeted_cost'] : 0;
      $expenses_info['estimated_expense_total_cost'] += $non_billable_expenses['budgeted_cost'] ? $non_billable_expenses['budgeted_cost'] : 0;
} 

/**
 * Get executed expenses info
 *
 */

function update_executed_expenses_info_for_member($object) {
    Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_executed_expenses_info_for_member_by_using_member_id($project_member_id);
    }
}

function update_executed_expenses_info_for_member_by_using_member_id($project_member_id) {
    $expenses_info = get_executed_expense_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $expenses_info);
    update_executed_overall_totals_for_member($project_member_id);
    $project_member = Members::findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0){
            update_executed_expenses_info_for_member_by_using_member_id($parent_id);
        }
    }
}

function get_executed_expense_info_for_member($project_member_id) {
    $expenses_info = array(
        'executed_expense_billable_price' => 0,
        'executed_expense_non_billable_price' => 0,
        'executed_expense_total_price' => 0,
        'executed_expense_billable_cost' => 0,
        'executed_expense_non_billable_cost' => 0,
        'executed_expense_total_cost' => 0
    );
    
    $billable_expenses_info = get_executed_expenses_totals_for_member($project_member_id, ' AND `is_billable`=1 ');
    $non_billable_expenses_info = get_executed_expenses_totals_for_member($project_member_id, ' AND `is_billable`=0 ');

    calculate_executed_expenses_values($expenses_info, $billable_expenses_info[0], $non_billable_expenses_info[0]);
    
    return $expenses_info;
}

function calculate_executed_expenses_values(&$expenses_info, $billable_expenses,$non_billable_expenses) {
     // Calculate executed expense billable price
     $expenses_info['executed_expense_billable_price'] += $billable_expenses['total_executed_price'] ? $billable_expenses['total_executed_price'] : 0;
     // Calculate executed expense non-billable price
     $expenses_info['executed_expense_non_billable_price'] += $non_billable_expenses['total_executed_price'] ? $non_billable_expenses['total_executed_price'] : 0;
     // Calculate executed total price
     $expenses_info['executed_expense_total_price'] += $billable_expenses['total_executed_price'] ? $billable_expenses['total_executed_price'] : 0;
     $expenses_info['executed_expense_total_price'] += $non_billable_expenses['total_executed_price'] ? $non_billable_expenses['total_executed_price'] : 0;
 
      // Calculate executed expense billable cost
      $expenses_info['executed_expense_billable_cost'] += $billable_expenses['total_executed_cost'] ? $billable_expenses['total_executed_cost'] : 0;
      // Calculate executed expense non-billable cost
      $expenses_info['executed_expense_non_billable_cost'] += $non_billable_expenses['total_executed_cost'] ? $non_billable_expenses['total_executed_cost'] : 0;
      // Calculate executed total cost
      $expenses_info['executed_expense_total_cost'] += $billable_expenses['total_executed_cost'] ? $billable_expenses['total_executed_cost'] : 0;
      $expenses_info['executed_expense_total_cost'] += $non_billable_expenses['total_executed_cost'] ? $non_billable_expenses['total_executed_cost'] : 0;
} 

function get_executed_expenses_totals_for_member($member_id, $extra_conditions = ''){
    $executed_cost = "SUM(amount) AS total_executed_cost ";
    $executed_price = "SUM(total_price) AS total_executed_price ";

    $expenses_info = PaymentReceipts::instance()->listing(array(
        "select_columns" => array("is_billable", "amount", "total_price"),
        "query_wraper_start" => "SELECT $executed_cost, $executed_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "member_ids" => array($member_id),
        "extra_conditions" => $extra_conditions
    ))->objects; 

    return $expenses_info;
}
