<?php
	class ExpenseItem extends BaseExpenseItem {
		
		function getArrayInfo($formatted_amounts = false) {
			$info = array();
			Env::useHelper('format');
				
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if (in_array($this->manager()->getColumnType($col), array(DATA_TYPE_DATETIME, DATA_TYPE_DATE))) {
					$info[$col] = format_date($this->getColumnValue($col), null, 0);
				}
			}
			$info['name'] = $this->getObjectName();
			$info['object_id'] = $this->getId();
			if (!$info['name']) {
				$info['name'] = clean($this->getDescription());
			}

			$pt = ProductTypes::getById($this->getProductTypeId());
			if ($pt instanceof ProductType) {
				$info['product_type_name'] = $pt->getName();
			}
			
			$currency = Currencies::getCurrency($this->getCurrencyId());
			$symbol = $currency ? $currency->getSymbol() : '$';
			$info['currency_sym'] = $symbol;
			
			$amount_decimals = config_option('product_type_decimals');
			$info['amount_decimals'] = $amount_decimals;
			
			if ($formatted_amounts) {
				$amount_cols = array('unit_cost', 'total_cost', 'unit_price', 'total_price');
				foreach ($amount_cols as $amount_col) {
					$info[$amount_col] = format_money_amount($info[$amount_col], $symbol, $amount_decimals);
				}
			}
			
			return $info;
		}
		
/*		
		function getArrayInfo($include_dates = false, $filter_payments_by_context = false, $calculate_amounts = false) {
			$info = array();
			Env::useHelper('format');
			
			$context_conditions = "";
			if ($filter_payments_by_context) {
				$active_member_ids = active_context_members(false);
				if (count($active_member_ids) > 0) {
					$mem_str = implode(',', $active_member_ids);
					$context_conditions = "
						AND EXISTS (
							SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp
							INNER JOIN ".TABLE_PREFIX."object_members pm ON pm.object_id=pp.object_id
							WHERE pm.member_id IN ($mem_str)
								AND pp.expense_id=".TABLE_PREFIX."payment_history.expense_id AND pp.payment_num=".TABLE_PREFIX."payment_history.payment_num
							GROUP BY pm.object_id HAVING count(pm.member_id) = ".count($active_member_ids)."
						)";
				}
			}
			
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
			}
			$info['name'] = $this->getObjectName();
			$info['object_id'] = $this->getId();
			
			$status = ExpensesStatuses::instance()->getStatus($this->getStatusId());
			$info['status'] = $status instanceof ExpensesStatus ? ($status->getName() == '' ? lang($status->getCode()) : $status->getName()) : '';
			$info['status_code'] = $status instanceof ExpensesStatus ? $status->getCode() : '';
			
			$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
			
			// calculate amounts from payment history
			if ($calculate_amounts) {
				$info['amounts'] = array();
				foreach ($all_status as $st) {
					if ($st['only_for_expenses']) continue;
					$symbol = null;
					$totals = array();
					
					//$ps = Payments::findAll(array('conditions' => array('expense_id = ? AND status_id = ?', $this->getId(), $st['id'])));
					$ps = PaymentHistories::findAll(array('conditions' => array('expense_id = ? AND status_id >= ? 
							'. $context_conditions .'
							AND 0=(SELECT oo.trashed_by_id FROM '.TABLE_PREFIX.'payments pp INNER JOIN '.TABLE_PREFIX.'objects oo ON oo.id=pp.object_id 
									WHERE pp.expense_id='.TABLE_PREFIX.'payment_history.expense_id AND pp.payment_num='.TABLE_PREFIX.'payment_history.payment_num AND oo.trashed_by_id=0)
							AND status_id <= (SELECT p.status_id FROM '.TABLE_PREFIX.'payments p WHERE p.expense_id='.TABLE_PREFIX.'payment_history.expense_id AND p.payment_num='.TABLE_PREFIX.'payment_history.payment_num LIMIT 1)
							AND modification_log IN (\'add\', \'edit\')
							AND modification_date = (
								SELECT max(x.modification_date) FROM '.TABLE_PREFIX.'payment_history x 
								WHERE x.expense_id='.$this->getId().' AND x.status_id='.$st['id'].' AND x.payment_num='.TABLE_PREFIX.'payment_history.payment_num
							)', $this->getId(), $st['id'])));
					
					foreach ($ps as $p) {
						$currency = Currencies::getCurrency($p->getCurrency());
						if (!$currency instanceof Currency) continue;
						
						if (!isset($totals[$currency->getId()])) $totals[$currency->getId()] = 0;
						$totals[$currency->getId()] += $p->getAmount();
					}
					
					foreach ($totals as $cid => $total) {
						$currency = Currencies::getCurrency($cid);
						$symbol = $currency->getSymbol();
						
						$info['total_' . $st['code']][$cid] = format_money_amount($total, $symbol);
						
						if (!isset($info['amounts'][$st['id']])) $info['amounts'][$st['id']] = array();
						$info['amounts'][$st['id']][$cid] = $total;
					}
				}
				
			} else {
				// get amounts from cache
				$info['amounts'] = array();
				
				$active_context_cond = "";
				if ($filter_payments_by_context) {
					$active_member_ids = active_context_members(false);
					if (count($active_member_ids) > 0) {
						$active_context_cond = "
							AND EXISTS (
								SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
								WHERE om.member_id IN (".implode(',', $active_member_ids).") AND pt.payment_id = om.object_id 
								GROUP BY object_id
								HAVING count(member_id) = ".count($active_member_ids)."
							)
						";
					}
				}
				
				// check permissions over payments
				$perm_conds_sql = "";
				if (!logged_user()->isAdministrator()) {
					$logged_user_pg_ids = implode(',', logged_user()->getPermissionGroupIds());
					$perm_conds_sql = "
						AND payment_id IN (
							SELECT sh.object_id FROM ".TABLE_PREFIX."sharing_table sh
							WHERE payment_id = sh.object_id AND sh.group_id  IN ($logged_user_pg_ids)
						)
					";
				}
				
				$total_rows = DB::executeAll("
					SELECT pt.currency_id, pt.status_id, SUM(pt.amount) as total FROM ".TABLE_PREFIX."payment_totals pt
					WHERE payment_id IN (
						SELECT p.object_id FROM ".TABLE_PREFIX."payments p WHERE p.expense_id='".$this->getId()."'
					)
					$active_context_cond
					$perm_conds_sql
					GROUP BY pt.currency_id, pt.status_id;
				");
				
				if (!is_array($total_rows)) $total_rows = array();
				foreach ($total_rows as $trow) {
					$cid = $trow['currency_id'];
					$currency = Currencies::getCurrency($cid);
					if (!$currency instanceof Currency) continue;
					
					$st = null;
					foreach ($all_status as $stat) {
						if ($stat['id'] == $trow['status_id']) {
							$st = $stat;
							break;
						}
					}
					if (!isset($info['amounts'][$st['id']])) $info['amounts'][$st['id']] = array();
					
					$info['amounts'][$st['id']][$cid] = $trow['total'];
					$info['total_' . $st['code']][$cid] = format_money_amount($trow['total'], $currency->getSymbol());
				}
			}
			
			if ($include_dates) {
				$info['date'] = '';
				$rows = DB::executeAll("select min(`date`) as mindate, max(`date`) as maxdate from ".TABLE_PREFIX."payments 
						where expense_id=".$this->getId()." AND `date`!='".EMPTY_DATETIME."'");
				
				if (count($rows) > 0) {
					$dates = $rows[0];
					$mind = DateTimeValueLib::makeFromString($dates['mindate']);
					$maxd = DateTimeValueLib::makeFromString($dates['maxdate']);
					if ($dates['mindate'] == $dates['maxdate']) {
						if ($mind instanceof DateTimeValue) {
							$info['date'] = $mind->format(user_config_option('date_format'));
						}
					} else {
						if ($mind instanceof DateTimeValue && $maxd instanceof DateTimeValue) {
							$info['date'] = $mind->format(user_config_option('date_format')) .' '.lang('date_to').' '. $maxd->format(user_config_option('date_format'));
						}
					}
				}
			}
			
			Hook::fire('modify_object_get_array_info', array('object' => $this), $info);
			
			return $info;
		}

*/
/*
		function updateCalculatedTotals() {
			// TODO: fix this function
			return;
			$values = array();
			$total_rows = array();
			
			if ($this->getTrashedById() == 0) {
				$total_rows = DB::executeAll("
					SELECT pt.status_id, pt.currency_id, SUM(pt.amount) as total
					FROM ".TABLE_PREFIX."expense_items_totals pt
					INNER JOIN ".TABLE_PREFIX."expense_items p ON p.object_id=pt.payment_id
					WHERE p.expense_id=".$this->getId()."
					GROUP BY pt.status_id, pt.currency_id
				");
			}
			
			foreach ($total_rows as $tr) {
				$st_id = $tr['status_id'];
				$c_id = $tr['currency_id'];
				$total = $tr['total'];
				
				$values[] = "(".$this->getId().", $st_id, $c_id, $total)";
			}
			
			DB::execute("DELETE FROM ".TABLE_PREFIX."expense_totals WHERE expense_id=".$this->getId());
			if (count($values) > 0) {
				DB::execute("INSERT INTO ".TABLE_PREFIX."expense_totals VALUES ".implode(',',$values)." ON DUPLICATE KEY UPDATE expense_id=expense_id;");
			}
		
		}*/
		
		
		
		function delete() {
			if ($this->canDelete(logged_user())) {
				$this->addToHistory(logged_user(), 'delete');
				parent::delete();
			}
		}
		
		function trash($trashDate = null, $fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				$this->addToHistory(logged_user(), 'delete');
				parent::trash();
			}
		}
		
		function untrash($fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				$this->addToHistory(logged_user(), 'untrash');
				parent::untrash();
			}
		}
		
		
		function getPaymentsFromHistory() {
			
			$current_status = null;
			$current_status_tmp = ExpensesStatuses::findById($this->getStatusId());
			if (str_starts_with($current_status_tmp->getCode(), "part_")) {
				$st_code = str_replace("part_", "", $current_status_tmp->getCode());
				$current_status = ExpensesStatuses::findOne(array('conditions' => "code='$st_code'"));
			} else {
				$current_status = $current_status_tmp;
			}
			if (!$current_status instanceof ExpensesStatus) {
				return array();
			}
			
			$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
			$all_rows = array();
			foreach ($statuses as $st) {
				if ($st['only_for_expenses']) continue;
				
				$exec_cond = "";
				if ($st['code'] != 'executed') $exec_cond = "AND ph.status_id=x.status_id";
				
				// payment history status must not be greater than the expense's status
				$exp_status_condition = "AND (SELECT pst.order FROM ".TABLE_PREFIX."expenses_status pst WHERE pst.id=ph.status_id) 
						<= (SELECT est.order FROM ".TABLE_PREFIX."expenses_status est WHERE est.id=".$current_status->getId().")";
				
				$sql = "
					SELECT * FROM ".TABLE_PREFIX."payment_history ph
					WHERE
						ph.expense_id=".$this->getId()."
						$exp_status_condition
						AND ph.status_id = ".$st['id']."
						AND ph.modification_log in ('add', 'edit')
						AND EXISTS (SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp 
								INNER JOIN ".TABLE_PREFIX."objects po ON po.id=pp.object_id
								WHERE pp.expense_id=ph.expense_id AND pp.payment_num=ph.payment_num AND po.trashed_by_id=0
								LIMIT 1)
						
						AND ph.modification_date = (
							SELECT max(x.modification_date) FROM ".TABLE_PREFIX."payment_history x 
							WHERE x.expense_id=ph.expense_id AND x.payment_num=ph.payment_num 
							$exec_cond
						);
				";
				
				$rows = DB::executeAll($sql);
				
				if (is_array($rows)) {
					$all_rows = array_merge($all_rows, $rows);
				}
			}
			
			$phs = array();
			foreach ($all_rows as $row) {
				$phs[] = PaymentHistories::instance()->loadFromRow($row);
			}
			
			return $phs;
		}
		
		
		function addToHistory($user, $log_data) {
			
			$history = new ExpenseItemsHistory();
			
			$history->setObjectId($this->getId());
			$history->setName($this->getName());
			$history->setDescription($this->getDescription());
			$history->setDate($this->getDate());
			$history->setQuantity($this->getQuantity());
			$history->setProductTypeId($this->getProductTypeId());
			$history->setCurrencyId($this->getCurrencyId());
			$history->setUnitCost($this->getUnitCost());
			$history->setTotalCost($this->getTotalCost());
			$history->setPriceMultiplier($this->getPriceMultiplier());
			$history->setUnitPrice($this->getUnitPrice());
			$history->setTotalPrice($this->getTotalPrice());
			
			$history->setUserId($user->getId());
			$history->setModificationDate(DateTimeValueLib::now());
			$history->setModificationLog($log_data);
			
			$history->save();
			
			//$this->updateCalculatedTotals();
		}
		
		/*
		function save($update_totals=true) {
			parent::save();
			if ($update_totals) {
				$this->updateCalculatedTotals();
			}
		}
		*/
		
		function refreshStatus() {
			/*
			$status_count = array();
			$all_payments = Payments::findAll(array('conditions' => 'expense_id = '.$this->getId()));
			
			foreach ($all_payments as $p) {
				if (!isset($status_count[$p->getStatusId()])) $status_count[$p->getStatusId()] = 0;
				$status_count[$p->getStatusId()] = $status_count[$p->getStatusId()] + 1;
			}
			
			if (count($status_count) > 1) {
				$part_app = ExpensesStatuses::instance()->getStatusByCode('part_approved');
				$st_app = ExpensesStatuses::instance()->getStatusByCode('approved');
				$part_exe = ExpensesStatuses::instance()->getStatusByCode('part_executed');
				$st_exe = ExpensesStatuses::instance()->getStatusByCode('executed');
				
				if (array_var($status_count, $st_exe->getId()) > 0) {
					$this->setStatusId($part_exe->getId());
				} else if (array_var($status_count, $st_app->getId()) > 0) {
					$this->setStatusId($part_app->getId());
				}
				
				$this->save();
				$this->addToHistory(logged_user(), 'edit');
				
			} else if (count($status_count) == 1) {
				$keys = array_keys($status_count);
				$this->setStatusId($keys[0]);
				$this->save();
				$this->addToHistory(logged_user(), 'edit');
			}
			*/
		}
		
		
		
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Expenses::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			/*$status = ExpensesStatuses::findById($this->getStatusId());
			$final_st = ExpensesStatuses::getFinalStatus();
			$final_status_code = $final_st->getCode();
			
			$has_completed_payments = Payments::instance()->count("expense_id=".$this->getId()." AND status_id=".$final_st->getId()) > 0;
			if (!$status instanceof ExpensesStatus || $status->getCode() == $final_status_code || $status->getCode() == 'part_'.$final_status_code || $has_completed_payments) {
				return can_edit_executed_payments_pg_array($user->getPermissionGroupIds());
			}*/
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			/*$status = ExpensesStatuses::findById($this->getStatusId());
			if (!$status instanceof ExpensesStatus || $status->getCode() == 'executed' || $status->getCode() == 'part_executed') {
				return false;
			}*/
			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}