<?php
	class PaymentReceipt extends BasePaymentReceipt {
		
		
		function getPaidByName()
		{
			if($this->getPaidById() <= 0 )
			{
				return '';
			}

			$contact = Contacts::findById($this->getPaidById());
			
			if(!($contact instanceof Contact))
			{
				return '';
			}

			return $contact->getObjectName();
		}

		function getArrayInfo($format_the_data = false, $get_file_content = false) {
			$info = array();
			Env::useHelper('format');
				
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if (in_array($this->manager()->getColumnType($col), array(DATA_TYPE_DATETIME, DATA_TYPE_DATE)) && $this->getColumnValue($col) instanceof DateTimeValue) {
					if ($this->getColumnValue($col)->getTimestamp() > 0) {
						$info[$col] = format_date($this->getColumnValue($col), null, 0);
						$info[$col.'Timestamp'] = $this->getColumnValue($col)->getTimestamp();
					} else {
						$info[$col] = '';
						$info[$col.'Timestamp'] = 0;
					}
				}
			}
			$info['name'] = $this->getObjectName();
			$info['object_id'] = $this->getId();
			
			$info['document_url'] = get_url('files', 'get_public_file', array('id' => $this->getDocumentId()));
			$currency = Currencies::getCurrency($this->getCurrencyId());
			$symbol = $currency ? $currency->getSymbol() : '$';
			$info['currency_sym'] = $symbol;

			if($this->getPaidById() > 0 ){
				$contact = Contacts::findById($this->getPaidById());
				if ($contact instanceof Contact) {
					$info['paid_by_name'] = $contact->getObjectName();
				} else {
					$info['paid_by_name'] = '';
				}
			} else {
				$info['paid_by_name'] = '';
			}

			$info['payment_term_name'] = actual_expense_get_payment_term_name($this->getPaymentTermId());

			if ($this->getExpenseId() > 0) {
				$expense = Expenses::findById($this->getExpenseId());
				if ($expense instanceof Expense) {
					$info['expense_name'] = $expense->getObjectName();
				}
				
				//Previously: WHERE expense_id=".$this->getExpenseId()." AND status_id=(SELECT id FROM ".TABLE_PREFIX."expenses_status WHERE code='budgeted')"
				$expense_cost_row = DB::executeOne("
					SELECT * FROM ".TABLE_PREFIX."expense_totals 
					WHERE expense_id=".$this->getExpenseId()
				);
				if ($expense_cost_row && count($expense_cost_row) > 0) {
					$expense_cost = $expense_cost_row['total_cost'];
					if ($this->getAmount() > $expense_cost) {
						$info['add_cls'] = 'warning';
					}
				}
			}
			if($get_file_content) {
				if ($this->getDocumentId() != '' && FileRepository::isInRepository($this->getDocumentId())) {
					$info['file_content'] = base64_encode(FileRepository::getFileContent($this->getDocumentId()));
				}
			}

			if ($this->manager()->columnExists('invoicing_status') && !$this->getIsBillable()) {
				$info['invoicing_status'] = 'non_billable';
			}

			if ($format_the_data) {
				$amount_cols = array('amount', 'unit_cost', 'unit_price', 'total_price', 'total_cost_without_taxes');
				foreach ($amount_cols as $amount_col) {
					$info[$amount_col] = format_money_amount($info[$amount_col], $symbol, config_option('product_type_decimals'));
				}
				
			}

			$info['expense_type_code'] = $info['expense_type'];
			$info['expense_type'] = lang($info['expense_type']);

			Hook::fire('additional_actual_expense_array_info_data', $this, $info);
			
			return $info;
		}
		
		
		function recalculateBudgetedExpenseInvoicingStatus() {
			$expense = Expenses::findById($this->getExpenseId());
			if ($expense instanceof Expense) {
				$expense->recalculateInvoicingStatus();
			}
		}
		
		
		function changeInvoicingStatus($status) {
			if (PaymentReceipts::instance()->columnExists('invoicing_status')) {
				$old_status = $this->getColumnValue('invoicing_status');
				
				$this->setColumnValue('invoicing_status', $status);
				if($status == 'pending') {
					$this->setColumnValue('invoice_id', 0);
				}
				$this->save();
				
				$ret = null;
				Hook::fire("after_change_object_inv_status", array('object' => $this, 'old_status' => $old_status), $ret);
			}
		}
	
		
		function save() {
			$ok = parent::save();
			
			$this->recalculateBudgetedExpenseInvoicingStatus();
			
			return $ok;
		}
		
		
		function delete() {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'delete');
				parent::delete();
			}
		}
		
		function trash($trashDate = NULL, $fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'delete');
				parent::trash();
			}
		}
		
		function untrash($fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'untrash');
				parent::untrash();
			}
		}
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, PaymentReceipts::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			$can_read = can_read($user, $this->getMembers(), $this->getObjectTypeId());
			if ($can_read && !SystemPermissions::userHasSystemPermission($user, 'can_see_expenses_of_others')) {
				$can_read = $this->getPaidById() == $user->getId();
			}
			return $can_read;
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			//$status = ExpensesStatuses::findById($this->getStatusId());
			//$final_st = ExpensesStatuses::getFinalStatus();
			//$final_status_code = $final_st->getCode();
			/*
			$has_completed_payments = Payments::instance()->count("expense_id=".$this->getId()." AND status_id=".$final_st->getId()) > 0;
			if (!$status instanceof ExpensesStatus || $status->getCode() == $final_status_code || $status->getCode() == 'part_'.$final_status_code || $has_completed_payments) {
				return can_edit_executed_payments_pg_array($user->getPermissionGroupIds());
			}*/
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			/*$status = ExpensesStatuses::findById($this->getStatusId());
			if (!$status instanceof ExpensesStatus || $status->getCode() == 'executed' || $status->getCode() == 'part_executed') {
				return false;
			}*/
			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}