<?php 
 

  abstract class BasePaymentReceipts extends ContentDataObjects {
  
    /**
    * Column name => Column type map
    *
    * @var array
    * @static
    */
    static private $columns = array(
    	'object_id' => DATA_TYPE_INTEGER,
    	'expense_id' => DATA_TYPE_INTEGER,
    	//'status_id' => DATA_TYPE_INTEGER,
    	'date' => DATA_TYPE_DATE,
    	'description' => DATA_TYPE_STRING,
    	'product_type_id' => DATA_TYPE_INTEGER,
    	'currency_id' => DATA_TYPE_INTEGER,
    	'amount' => DATA_TYPE_FLOAT,
    	'document_id' => DATA_TYPE_STRING,
    	'is_billable' => DATA_TYPE_BOOLEAN,
		'paid_by_id' => DATA_TYPE_INTEGER,
    	'quantity' => DATA_TYPE_FLOAT,
    	'unit_cost' => DATA_TYPE_FLOAT,
    	'unit_price' => DATA_TYPE_FLOAT,
    	'total_price' => DATA_TYPE_FLOAT,
    	'price_multiplier' => DATA_TYPE_FLOAT,
    	'total_cost_without_taxes' => DATA_TYPE_FLOAT,
		'expense_type' => DATA_TYPE_STRING,
		'due_date' => DATA_TYPE_DATE,
		'payment_term_id' => DATA_TYPE_INTEGER,
    );
  

    function __construct() {
		Hook::fire('object_definition', 'PaymentReceipt', self::$columns);
    	parent::__construct('PaymentReceipt', 'payment_receipts', true);
    } 
    
   
    
    /**
    * Return array of object columns
    *
    * @access public
    * @param void
    * @return array
    */
    function getColumns() {
      return array_keys(self::$columns);
    }
    
    /**
    * Return column type
    *
    * @access public
    * @param string $column_name
    * @return string
    */
    function getColumnType($column_name) {
    	return parent::getCOColumnType($column_name, self::$columns);
    } // getColumnType
    
    /**
    * Return array of PK columns. If only one column is PK returns its name as string
    *
    * @access public
    * @param void
    * @return array or string
    */
    function getPkColumns() {
      return 'object_id';
    } // getPkColumns
    
    /**
    * Return name of first auto_incremenent column if it exists
    *
    * @access public
    * @param void
    * @return string
    */
    function getAutoIncrementColumn() {
      return 'object_id';
    } // getAutoIncrementColumn

    
    /**
     * Return external columns
     *
     * @access public
     * @param void
     * @return array
     */
    function getExternalColumns() {
    	//Unnecessary. We are moving status to a dimension 
    	//return array_merge(parent::getExternalColumns(), array('status'));
    	return array_merge(parent::getExternalColumns(), array('product_type_id', 'expense_id', 'paid_by_id', 'expense_type'));
    } // getExternalColumns
    
    // -------------------------------------------------------
    //  Finders
    // -------------------------------------------------------
    
    /**
    * Do a SELECT query over database with specified arguments
    *
    * @access public
    * @param array $arguments Array of query arguments. Fields:
    * 
    *  - one - select first row
    *  - conditions - additional conditions
    *  - order - order by string
    *  - offset - limit offset, valid only if limit is present
    *  - limit
    * 
    * @return mixed one or more objects
    * @throws DBQueryError
    */
    function find($arguments = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::find($arguments);
      } else {
        return PaymentReceipts::instance()->find($arguments);
      }
    }
    

    function findAll($arguments = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::findAll($arguments);
      } else {
        return PaymentReceipts::instance()->findAll($arguments);
      }
    } 
    

    function findOne($arguments = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::findOne($arguments);
      } else {
        return PaymentReceipts::instance()->findOne($arguments);
      } // if
    } // findOne
    
    /**
    * Return object by its PK value
    *
    * @access public
    * @param mixed $id
    * @param boolean $force_reload If true cache will be skipped and data will be loaded from database
    * @return PaymentReceipt 
    */
    function findById($id, $force_reload = false) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::findById($id, $force_reload);
      } else {
        return PaymentReceipts::instance()->findById($id, $force_reload);
      } // if
    } // findById
    
    /**
    * Return number of rows in this table
    *
    * @access public
    * @param string $conditions Query conditions
    * @return integer
    */
    function count($condition = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::count($condition);
      } else {
        return PaymentReceipts::instance()->count($condition);
      } // if
    } // count
    
    /**
    * Delete rows that match specific conditions. If $conditions is NULL all rows from table will be deleted
    *
    * @access public
    * @param string $conditions Query conditions
    * @return boolean
    */
    function delete($condition = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::delete($condition);
      } else {
        return PaymentReceipts::instance()->delete($condition);
      } // if
    } // delete
    
    /**
    * This function will return paginated result. Result is an array where first element is 
    * array of returned object and second populated pagination object that can be used for 
    * obtaining and rendering pagination data using various helpers.
    * 
    * Items and pagination array vars are indexed with 0 for items and 1 for pagination
    * because you can't use associative indexing with list() construct
    *
    * @access public
    * @param array $arguments Query argumens (@see find()) Limit and offset are ignored!
    * @param integer $items_per_page Number of items per page
    * @param integer $current_page Current page number
    * @return array
    */
    function paginate($arguments = null, $items_per_page = 10, $current_page = 1, $count = null) {
      if(isset($this) && instance_of($this, 'PaymentReceipts')) {
        return parent::paginate($arguments, $items_per_page, $current_page);
      } else {
        return PaymentReceipts::instance()->paginate($arguments, $items_per_page, $current_page);
      }
    } 
    

    function instance() {
      static $instance;
      if(!instance_of($instance, 'PaymentReceipts')) {
        $instance = new PaymentReceipts();
      } // if
      return $instance;
    } // instance
  
  }
