/**
 *  ActualExpenseTaxesManager
 *
 */
og.ActualExpenseTaxesManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	//og.eventManager.addListener('member changed', this.reset, this);
	
	this.fields = [
		'id', 'name', 'valid_from', 'valid_to', 'value', 'is_percentage', 'select_by_default', 'currency_id'
   	];
   	
	if (!og.ActualExpenseTaxesManager.store) {
		og.ActualExpenseTaxesManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('actual_expense_tax', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'tax',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					var d = this.reader.jsonData;
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("taxes"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("taxes")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					var cmp = Ext.getCmp('actual-expense-taxes-manager');
					if (cmp) {
						var sm = cmp.getSelectionModel();
						sm.clearSelections();
					}
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.ActualExpenseTaxesManager.store.setDefaultSort('name', 'asc');
	}
	this.store = og.ActualExpenseTaxesManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderName(value, p, r) {
		
		var name = String.format('<a style="font-size:120%;" href="#" onclick="return false;">{0}</a>', og.clean(value));
		
		return name;
	}
	
	function renderIcon(value, p, r) {
		return '<div class="link-ico ico-billing"></div>';
	}

	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		if (sm.hasSelection()) {
			return sm.getSelected().data.id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange',
		function() {
			var allUnread = true, allRead = true;
			var selections = sm.getSelections()
			for (var i=0; i < selections.length; i++) {
				if (selections[i].data.isRead){
					allUnread = false;
				} else {
					allRead = false;
				}
			}
			if (sm.getCount() <= 0) {
				
				actions.del.setDisabled(true);
				actions.edit.setDisabled(true);
				
			} else {
				
				actions.del.setDisabled(false);
				actions.edit.setDisabled(false);
				
			}
		});
	
	var cm_info = [
		sm,{
			id: 'icon',
			header: '&nbsp;',
			//dataIndex: 'type',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("name"),
			dataIndex: 'name',
			width: 250,
			renderer: renderName,
			sortable:true
		},{
			id: 'value',
			header: lang("value"),
			dataIndex: 'value',
			sortable:true
		},{
			id: 'is_percentage',
			header: lang("is_percentage"),
			dataIndex: 'is_percentage',
			hidden: true,
			sortable:true
		},{
			id: 'select_by_default',
			header: lang("select_by_default"),
			dataIndex: 'select_by_default',
			hidden: true,
			sortable:true
		},{
			id: 'valid_from',
			header: lang("valid_from"),
			dataIndex: 'valid_from',
			width: 50,
			sortable: true
		},{
			id: 'valid_to',
			header: lang("valid_to"),
			dataIndex: 'valid_to',
			width: 50,
			sortable: true
		},
    ];
	
	// create column model
	var cm = new Ext.grid.ColumnModel(cm_info);
	cm.defaultSortable = false;
	

	actions = {
		back: new Ext.Action({
			text: lang('back'),
            tooltip: lang('back'),
            iconCls: 'ico-back',
            handler: function() {
            	og.goback(this.el.dom);
			}
		}),
		newCO: new Ext.Action({
			text: lang('new tax'),
            tooltip: lang('add new tax'),
            iconCls: 'ico-new new_button',
            handler: function() {
				og.render_modal_form('', {c:'actual_expense_tax', a:'add', params: {reload:1}});
			}
		}),
		del: new Ext.Action({
			text: lang('delete'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				if (confirm(lang('delete selected objects permanently'))) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				og.render_modal_form('', {c:'actual_expense_tax', a:'edit', params: {id:getFirstSelectedId(), reload:1}});
			},
			scope: this
		})
    };
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		tbar.push(actions.back);
		tbar.push('-');
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.del);
	}
	
	
	og.ActualExpenseTaxesManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		stateful: og.preferences['rememberGUIState'],
		id: 'actual-expense-taxes-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: true
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});					
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.ActualExpenseTaxesManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}
		
		this.store.baseParams = {
			context: og.contextManager.plainContext()
		};
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
	},
	resetVars: function(){
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.load({start:0});
		}
	},
	
	reset: function() {
		this.load({start:0});
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	trashObjects: function() {
		if (confirm(lang('delete selected objects permanently'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
	
	
});


Ext.reg("actual-expense-taxes", og.ActualExpenseTaxesManager);

/************************************************
Container for ActualExpenseTaxesManager,
*************************************************/
og.ActualExpenseTaxesManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.ActualExpenseTaxesManager();
	
	og.ActualExpenseTaxesManagerPanel.superclass.constructor.call(this, {
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.manager
		],
		closable: true
	});
}

Ext.extend(og.ActualExpenseTaxesManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	}
});

Ext.reg("actual-expense-taxes-containerpanel", og.ActualExpenseTaxesManagerPanel);