<?php
class Factura
{
    public $destino = '';
    public $fechaVencimiento = null;
    public $fechaComprobante = null;
    public $formaPago = 1;
    public $nroCompra = null;
    public $tasaBasicaIva = null;
    public $tasaMinimaIva = null;
    public $textoAdicional = '';
    public $adenda = '';

    public $tipo = 111;
    public $tipoCambio = 1;
    public $tipoMoneda = null;

    public $totalMontoExportacion = 0;
    public $totalMontoImpuestoPercibido =0;
    public $totalMontoIvaEnSuspenso = 0;
    public $totalMontoIvaOtraTasa = 0;
    public $totalMontoIvaTasaBasica = 0;
    public $totalMontoIvaTasaMinima = 0;
    public $totalMontoNoGravado = 0;

    public $montoBruto = 0;
    public $montoNoFacturable = 0;

    public $receptor = null;

    public $productos = array();

    public $infoDescuentosRecargos = array();

    public function __construct(IncomeInvoice $invoice) {
        Env::useHelper('functions', 'facturalista');

        //Invoice type
        $external_type_id = FacturalistaController::get_einvoice_external_type($invoice->getColumnValue('object_subtype_id'), $invoice->getFinalConsumer());
        if(is_null($external_type_id)){
            //error no hay definido un tipo de factura externo para este tipo de factura
        }
        $this->tipo = $external_type_id;

        //Taxes
        $basic_iva = IncomeTaxes::findOne(array('condition' => array("code = 'basic_iva'")));
        $min_iva = IncomeTaxes::findOne(array('condition' => array("code = 'min_iva'")));
        $this->tasaBasicaIva = $basic_iva->getRecord()->getValue();
        $this->tasaMinimaIva = $min_iva->getRecord()->getValue();

        //Currency
        $this->tipoMoneda = 'UYU';
        $currency = Currencies::getCurrency($invoice->getCurrencyId());
        if($currency instanceof Currency){
            $currency_short_name = $currency->getShortName();
            $this->tipoMoneda = $currency_short_name;

            //si el tipo de moneda es diferente a uyu hay que poner el tipo de cambio $tipoCambio, tomado desde la factura
            $invoice_ex_rate = $invoice->getColumnValue("einvoice_destiny_rate");
            if ($invoice_ex_rate == 0) $invoice_ex_rate = 1;
            
            if ($this->tipoMoneda != 'UYU') {
            	if ($invoice_ex_rate == 1) {
            		$all_currencies = Currencies::getAllCurrencies();
            		$uyu_currency = null;
            		foreach ($all_currencies as $c) {
            			if ($c->getShortName() == 'UYU') {
            				$uyu_currency = $c;
            				break;
            			}
            		}
            		if ($uyu_currency instanceof Currency) {
            			$invoice_ex_rate = ExchangeRates::getLastExchangeRate($currency->getId(), $uyu_currency->getId());
            		}
            	}
            	if (strpos($invoice_ex_rate, ".") !== false) {
            		$invoice_ex_rate = str_replace(".", ",", $invoice_ex_rate);
            	}
            	$this->tipoCambio = $invoice_ex_rate;
            }
        }

        //Payment Period
        $cp_invoice_payment_period = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
        if ($cp_invoice_payment_period instanceof CustomProperty) {
            $cpv = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $cp_invoice_payment_period->getId());
            if ($cpv instanceof CustomPropertyValue) {
                if($cpv->getValue() == 'Due on receipt'){
                    //Contado
                    $this->formaPago = 1;
                }elseif ($cpv->getValue() == 'Net D'){
                    //Crédito
                    $this->formaPago = 2;
                }
            }
        }

        //Dates
        $this->fechaComprobante = format_date($invoice->getDate(),config_option('FACTURALISTA_DATEFORMAT'),0);
        $this->fechaVencimiento = format_date($invoice->getExpirationDate(),config_option('FACTURALISTA_DATEFORMAT'),0);

        //Owner company data
        $owner_company_id_conf = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='owner_company_id'"));
        if($owner_company_id_conf instanceof IncomeConfigOption){
            $owner_company_id = $owner_company_id_conf->getValue();

        }
        
        // Set client number in "destino" property
        $this->destino = trim($invoice->getCustomerNumber());

        //Client Data
        $customer_rut = trim(str_replace(' ', '', $invoice->getCustomerRut()));
        if(empty($customer_rut) && $invoice->getFinalConsumer()){
           // $customer_rut = 'Consumidor final'; //tenemos hace el campo obligatorio
        }

		// cusomter document type
		$customer_doc_type = $invoice->getColumnValue('customer_einvoice_doc_type');

		// old invoices compatibility (just to ensure that a value is sent in this field)
		if (!$customer_doc_type) {
			$customer_doc_type = $invoice->getFinalConsumer() ? FACTURALISTA_DOC_TYPE_CI : FACTURALISTA_DOC_TYPE_RUT; // rut 2, cedula 3, otros 4, pasaporte 5, documento extranjero 6,
		}

        $invoice_client = array(
            'ciudad' => $invoice->getCustomerCity(),
            'cp' => '',
            'departamento' => '',
            'direccion' => $invoice->getCustomerAddress(),
            'documento' => array(
                'documento' => $customer_rut,
            	'pais' => $invoice->getCustomerCountry() == '' ? 'UY' : strtoupper($invoice->getCustomerCountry()),// 'UY',
                'tipo' => $customer_doc_type,
            ),
            'nombre' => $invoice->getCustomerName(),
            'pais' => '',
        );
        $this->receptor = $invoice_client;

        $indicadorFacturacion_values = array(
            'exento_iva' => 1,
            'gravado_iva_minimo' => 2,
            'gravado_iva_basico' => 3
            );
        $indicadorFacturacion = $indicadorFacturacion_values['gravado_iva_basico'];

        //Totals
        $invoice_taxes = $invoice->getTaxes();
        if(count($invoice_taxes) == 0){
            $this->totalMontoNoGravado = $invoice->getSubtotal();
            $indicadorFacturacion = $indicadorFacturacion_values['exento_iva'];
        }else{
            foreach ($invoice_taxes as $tax_data) {
                $tax = IncomeTaxes::findById($tax_data->getTaxId());
                if($tax->getcode() == "basic_iva"){
                    $this->totalMontoIvaTasaBasica = $invoice->getSubtotal();
                    $indicadorFacturacion = $indicadorFacturacion_values['gravado_iva_basico'];
                }elseif($tax->getcode() == "min_iva"){
                    $this->totalMontoIvaTasaMinima = $invoice->getSubtotal();
                    $indicadorFacturacion = $indicadorFacturacion_values['gravado_iva_minimo'];
                }
            }
        }

        //Invoice Lines and discounts
        $invoice_lines = array();
        $invoice_line_number = 0;
        $invoice_discount_lines = array();
        $invoice_discount_line_number = 0;
        foreach ($invoice->getInvoiceLines() as $invoice_line) {
            if($invoice_line instanceof IncomeInvoiceLine){
                if($invoice_line->getTotal() >= 0){
                	
                	$line_name = $invoice_line->getDetail();
                	if (strlen($line_name) >= 80) {
                		$line_name = substr($line_name, 0, 79);
                	}
                	$line_desc = $invoice_line->getDetail();
                	
                    $invoice_line_number++;
                    $invoice_lines[] = array(
                        'indicadorFacturacion' => $indicadorFacturacion,
                    	'nombre' => $line_name,
                        'cantidad' => $invoice_line->getAmount(),
                        'precioUnitario' => $invoice_line->getUnitValue(),
                        'unidadDeMedida' => 'hs',
                    	'descripcion' => $line_desc,
                        'monto' => $invoice_line->getTotal(),
                        'montoTotalAPagar' => $invoice_line->getTotal(),
                        'nroLinea' => $invoice_line_number,
                        'porcentajeDescuento' => '0',
                        'porcentajeRecargo' => '0',
                    );
                }else{
                	$glosa = $invoice_line->getDetail();
                	if (strlen($glosa) >= 50) {
                		$glosa = substr($glosa, 0, 49);
                	}
                    $invoice_discount_line_number++;
                    $invoice_discount_lines[] = array(
                        'indicadorFacturacion' => $indicadorFacturacion,
                    	'glosa' => $glosa,
                        'tipoDescuentoRecargo' => '$',
                        'nroLinea' => $invoice_discount_line_number,
                        'tipoMovimiento' => 'D', //Descuento
                        'valor' => abs($invoice_line->getTotal()),
                        'codigo'=> '',
                    );
                }

            }
        }
        $this->productos = $invoice_lines;
        $this->infoDescuentosRecargos = $invoice_discount_lines;

        //Invoice description
        $this->adenda = $invoice->getAdditionalInformation();


    }
}