<?php

class FacturalistaService
{
    function __construct() {
        Env::useHelper('facturalista_logger', 'facturalista');
    }

    private function service_connect()
    {
        /* Initialize webservice */
        $client = new SoapClient(config_option('FACTURALISTA_SOAP_URL'), array("trace" => 1, "exceptions" => 0));

        return $client;
    }

    function generarFactura($factura, $invoice_id){
        try{
            $client = $this->service_connect();
            $response = $client->generarFactura($factura);//,'PDFCreator','pdf'

            facturalista_save_log($invoice_id,'generarFactura last request',$client->__getLastRequest());

            facturalista_save_log($invoice_id,'generarFactura response',print_r($response, true));

            //check response state
            //codigo = 0 OK
            if($response->codigo == 0){
                Env::useHelper('functions', 'facturalista');
                $factura_data = array();
                $factura_data['invoice_id'] = $invoice_id;
                $factura_data['id_externa'] = $response->documento->id;
                $factura_data['numero'] = $response->documento->numero;
                $factura_data['serie'] = $response->documento->serie;
                $factura_data['tipoCFE'] = $response->documento->tipoCFE;
                save_facturalista_factura($factura_data);

                flash_success("Factura generada con exito");
            }else{
            	// generate user friendly error message
            	$error_message = $this->buildErrorMessage($response);
            	throw new Exception($error_message);
            }
        } catch(Exception $e) {
            throw $e;
        }
    }

    function getImprimible($invoice_id,$members){
        try{
            $client = $this->service_connect();

            Env::useHelper('functions', 'facturalista');
            $data = get_facturalista_factura($invoice_id);

            $response = $client->getImprimible($data['serie'],$data['numero'],$data['tipoCFE'],'pdf');

            facturalista_save_log($invoice_id,'getImprimible last request',$client->__getLastRequest());

            facturalista_save_log($invoice_id,'getImprimible response',print_r($response, true));

            Env::useHelper('functions', 'advanced_core');
            $file = upload_generated_document($data['serie'].$data['numero'].".pdf", $members, $response->imprimible, 'application/pdf');

            flash_success("Estado de descarga del documento: ".$response->descripcion);

            return $file;
        } catch(Exception $e) {
            Logger::log($e->getMessage());
        }
    }
    
    
    private function buildErrorMessage($response) {
    	$error_message = "";
    	if ($response && $response->codigo > 0) {
    		switch ($response->codigo) {
    			// add a case block for each special error message, otherwise use the default message.
    			default:
		    		$error_message = "Error al generar la factura: " . $response->descripcion . " (código de error: ".$response->codigo.").";
    		}
    	}
    	
    	return $error_message;
    }
}