<?php
Hook::register('fidelis_reports');


function fidelis_reports_dont_show_group_totals($params, &$dont_show_group_totals) {

	$report = array_var($params, 'report');/* @var $report Report */
	if (!$report instanceof Report) return;
	
	$report_code = $report->getColumnValue('code');
	
	if ($report_code == 'op_horas_guardia_confirmar' || $report_code == 'op_horas_guardia_detallado_serv_tipohora') {
		// for this report, dont show each group totals, only the last total
		$dont_show_group_totals = true;
	}
}

function fidelis_reports_override_excel_custom_report_sheet_options($params, &$excel_report_options) {
	$report = array_var($params, 'report');
	if (!$report instanceof Report) return;
	
	$excel_report_options['fixed_cells_width'] = 20;
}

function fidelis_reports_list_totals_accumulate_total_rows($params, &$info) {
	$ot = array_var($params, 'object_type');
	if ($ot instanceof ObjectType && $ot->getName() == 'timeslot') {
		if (!is_array($info)) $info = array();
		
		$info['hora_entrada'] = array();
		$info['hora_salida'] = array();
	}
}


function fidelis_reports_modify_custom_report_results($params, &$results) {
	ini_set('memory_limit','2G');
	$report = array_var($params, 'report');/* @var $report Report */
	if (!$report instanceof Report) return;
	
	$report_code = $report->getColumnValue('code');
	
	if ($report_code == 'op_horas_guardia_confirmar') {
		
		$col_key = 'hora_entrada';
		if (!in_array($col_key, $results['columns']['order'])) {
			$results['columns']['order'][] = $col_key;
			$results['columns']['names'][$col_key] = 'Hora de Entrada';
			$results['columns']['types'][$col_key] = DATA_TYPE_FLOAT;
		}
		
		$col_key = 'hora_salida';
		if (!in_array($col_key, $results['columns']['order'])) {
			$results['columns']['order'][] = $col_key;
			$results['columns']['names'][$col_key] = 'Hora de Salida';
			$results['columns']['types'][$col_key] = DATA_TYPE_FLOAT;
		}
		
		$workday_cp = CustomProperties::getCustomPropertyByCode(Timeslots::instance()->getObjectTypeId(), 'workday');
		$date_format = user_config_option('date_format');
		
		foreach ($results['group_totals'][1] as &$group_totals) {
			
			$user_id = $group_totals['_group_id_fp_contact_id'];
			$date_str = $group_totals['_group_name_cp_'.$workday_cp->getId()];
			$date =  DateTimeValueLib::dateFromFormatAndString($date_format, $date_str);
			
			$sql = "SELECT ts.start_time 
					FROM ".TABLE_PREFIX."timeslots ts 
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=ts.object_id
					INNER JOIN ".TABLE_PREFIX."custom_property_values cpv ON cpv.object_id=ts.object_id
					WHERE ts.contact_id=$user_id AND cpv.value='".$date->format('Y-m-d')."' AND o.trashed_by_id=0
					ORDER BY start_time
					LIMIT 1;";
			$row = DB::executeOne($sql);
			
			$tmp_date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $row['start_time']);
			$hora_entrada = format_datetime($tmp_date, 'H:i'); // use this function to apply timezone offset
			
			$group_totals['hora_entrada'] = $hora_entrada;
			
			$sql = "SELECT ts.end_time
					FROM ".TABLE_PREFIX."timeslots ts
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=ts.object_id
					INNER JOIN ".TABLE_PREFIX."custom_property_values cpv ON cpv.object_id=ts.object_id
					WHERE ts.contact_id=$user_id AND cpv.value='".$date->format('Y-m-d')."' AND o.trashed_by_id=0
					ORDER BY end_time DESC
					LIMIT 1;";
			$row = DB::executeOne($sql);
			
			$tmp_date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $row['end_time']);
			$hora_salida = format_datetime($tmp_date, 'H:i'); // use this function to apply timezone offset
			
			$group_totals['hora_salida'] = $hora_salida;
		}
		
		
		// add the 'observaciones' column at the end of the column set, this column will be emtpy for each report row
		$col_key = 'observaciones';
		if (!in_array($col_key, $results['columns']['order'])) {
			$results['columns']['order'][] = $col_key;
			$results['columns']['names'][$col_key] = 'Observaciones';
			$results['columns']['types'][$col_key] = DATA_TYPE_STRING;
		}
	}
	
}

function fidelis_reports_override_report_group_ext_col_order_sql($params, &$hook_return_vals) {
	
	$ot = array_var($params, 'ot');
	$id = array_var($params, 'col');
	$t_prefix = array_var($params, 't_prefix');
	$ext_col_info = array_var($params, 'ext_col_info');
	
	// if this order sql comes from a timeslots report grouped by task
	if ($ot->getName() == 'timeslot' && $id == 'rel_object_id') {
		
		$hook_return_vals['joins_sql'] = "
			left join ".TABLE_PREFIX."project_tasks tasks_$id ON tasks_$id.object_id = $t_prefix.$id
		";
		
		$hook_return_vals['ext_col_select_columns'] = ", CONCAT(tasks_$id.start_date, namestable_$id.".$ext_col_info['name_col'].") as _group_order_col_fp_$id";
		
		$hook_return_vals['order_column'] = "_group_order_col_fp_$id";
		
	}
	
}

