<?php
Hook::register('hour_types');

function hour_types_object_definition($type, &$ret) {
	if ($type == 'Contact') {
		$ret['default_hour_type_id'] = DATA_TYPE_INTEGER;
	}
}


function hour_types_additional_system_columns($manager, &$columns) {
	if ($manager instanceof Contacts) {
		$columns[] = 'default_hour_type_id';
	}
}


function hour_types_before_render_main_custom_properties($params, &$categories) {

	$object = array_var($params, 'object');
	//$genid = array_var($params, 'genid');
	$genid = gen_id();

	if ($object instanceof Contact) {
		
		if ($object->getUserType() > 0 || ($object->isNew() && array_var($_REQUEST, 'is_user') > 0)) {
			
			$dimension = Dimensions::findByCode('hour_types');
			$default_member_id = $object->getColumnValue('default_hour_type_id');
			
			echo '<div class="dataBlock">';
			render_single_member_selector($dimension, $genid, array($default_member_id), array(
					'hidden_field_name' => 'contact[default_hour_type_id]',
					'label' => lang('default hour type'),
					'is_multiple' => false,
			));
			echo '</div>';
		}
		
	}
}

function hour_types_before_object_save($object, &$ret) {
	if ($object instanceof Contact) {
		
		$default_hour_type_id_json = array_var($_REQUEST['contact'], 'default_hour_type_id');
		$decoded = json_decode($default_hour_type_id_json, true);
		$default_hour_type_id = count($decoded) > 0 ? $decoded[0] : 0;
		
		$object->setColumnValue('default_hour_type_id', $default_hour_type_id);
	}
}


function hour_types_after_contact_view($contact, &$ret) {
	
	if ($contact->getUserType() > 0 && $contact->getColumnValue('default_hour_type_id') > 0) {
		
		$mem = Members::findById($contact->getColumnValue('default_hour_type_id'));
		if ($mem instanceof Member) {
			
			$member_breadcrumb .= '<span class="member-path real-breadcrumb og-wsname-color-'.$mem->getColor().'">'.$mem->getName().'</span>';
			
			$html = '<div class="basic-info">
						<div class="info-type">'. lang('default hour type') .'</div>
						<div class="info-content">
							<div class="info-content-item">'.$member_breadcrumb.'</div>
						</div>
						<div class="clear"></div>
					</div>';
			echo $html;
		}
	}
	
}


function hour_types_preselected_time_form_member_ids($params, &$pre_selected_member_ids) {
	$timeslot = array_var($params, 'object');
	
	if ($timeslot instanceof Timeslot) {
		
		$ts_contact = Contacts::findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_hour_type_id') > 0) {
			
			if (!is_array($pre_selected_member_ids)) $pre_selected_member_ids = array();
			
			$hour_type_set = false;
			if (count($pre_selected_member_ids) > 0) {
				$ht_dimension = Dimensions::findByCode('hour_types');
				foreach ($pre_selected_member_ids as $mid) {
					$m = Members::findById($mid);
					if ($m instanceof Member && $m->getDimensionId() == $ht_dimension->getId()) {
						$hour_type_set = true;
						break;
					}
				}
			}
			if (!$hour_type_set) {
				$pre_selected_member_ids[] = $ts_contact->getColumnValue('default_hour_type_id');
			}
		}
	}
}

function hour_types_after_api_add_timeslot($params, &$modified) {
	$timeslot = array_var($params, 'timeslot');
	
	if ($timeslot instanceof Timeslot) {
	
		$ts_contact = Contacts::findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_hour_type_id') > 0) {
			$hour_type_set = false;
			$ht_dimension = Dimensions::findByCode('hour_types');
			
			$ts_members = $timeslot->getMembers();
			foreach ($ts_members as $m) {
				if ($m instanceof Member && $m->getDimensionId() == $ht_dimension->getId()) {
					$hour_type_set = true;
					break;
				}
			}
			if (!$hour_type_set) {
				$hour_type_member = Members::getMemberById($ts_contact->getColumnValue('default_hour_type_id'));
				if ($hour_type_member instanceof Member) {
					ObjectMembers::addObjectToMembers($timeslot->getId(), array($hour_type_member));
					$modified = true;
				}
			}
		}
	}
}


/**
 * This function is used when grouping timeslots by person for invoicing
 * hour types plugin is used then the hour type (labor category) member id will be appended to the grouping key
 */
function hour_types_override_timeslot_person_key_for_grouping($t, &$key) {
	$dim = Dimensions::findByCode('hour_types');
	$members = $t->getMembers();
	foreach ($members as $m) {
		if ($m instanceof Member && $m->getDimensionId() == $dim->getId()) {
			$key .= "-".$m->getId();
			break;
		}
	}
}

