<?php

/**
 * CopyController
 * Generado el 18/5/2012
 * 
 */
class CopyController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the CopyController
     *
     * @access public
     * @param void
     * @return CopyController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
    }

    /**
     * Show copies
     *
     * @param void
     * @return null
     */
    function index() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $copies = IncomeCopies::findAll(array('conditions' => 'removed_by_id=0'));
        tpl_assign('copies', $copies);
    }
    
    
    function add() {
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    	 
    	$copy_post = array_var($_POST, 'copy');
    	if (!is_array($copy_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		tpl_assign('copy_edit', array());
    		tpl_assign('copy', new IncomeCopy());
    	} else {
    		try {
    			DB::beginWork();
    			$copy_data = new IncomeCopy();
    			 
    			$this->save_copy($copy_post, $copy_data, true);
    	
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    			 
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function edit() {
    	$this->setTemplate("add");
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    	 
    	$id = get_id();
    	$copy = IncomeCopies::findById($id);
    	 
    	$copy_post = array_var($_POST, 'copy');
    	if (!is_array($copy_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    
    		tpl_assign('copy', $copy);
    		$copy_edit = array(
                'id' => $copy->getId(),
                'name' => $copy->getName(),
                'description' => $copy->getDescription(),
                'color' => $copy->getColor(),
                'select_default' => $copy->getSelectDefault()
            );
    		tpl_assign('copy_edit', $copy_edit);
    
    	} else {
    		try {
    			DB::beginWork();
    
    			$this->save_copy($copy_post, $copy, false);
    			 
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
	function save_copy($copy_post, $copy_data, $is_new=true) {
    	
    	$copy_data->setName($copy_post['name']);
    	$copy_data->setDescription($copy_post['description']);
    	$copy_data->setColor($copy_post['color']);
    	$copy_data->setSelectDefault($copy_post['select_default']);
    	$copy_data->save();
    	
    	if ($is_new) {
    		flash_success(lang('success add copy'));
    	} else {
    		flash_success(lang('success edit copy'));
    	}
    	
    }

    function delete() {
        ajx_current("empty");
        $copy = IncomeCopies::findById(get_id("cop_id"));
        $copy->delete();

        flash_success(lang('success delete copy'));
    }

}

// CopyController
?>