<?php

/**
 * IncomeController
 * Generado el 18/5/2012
 * 
 */
class IncomeController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the IncomeController
     *
     * @access public
     * @param void
     * @return IncomeController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
        Env::useHelper('functions', 'income');
    }

    /**
     * Show configuration index page
     *
     * @param void
     * @return null
     */
    function administration() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');
    }

    function update_config() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        $options = IncomeConfigOptions::findAll(array("condition" => array("is_system = 1")));
        tpl_assign('options', $options);

        $submited_values = array_var($_POST, 'options');
        if (is_array($submited_values)) {
            foreach ($options as $option) {
                $new_value = array_var($submited_values, $option->getName());
                if (is_null($new_value) || ($new_value == $option->getValue()))
                    continue;

                $option->setValue($new_value);
                $option->save();
            }
            flash_success(lang('success update config'));
            ajx_current("back");
        } // if
    }
    
    
    function get_disabled_subtabs() {
    	
    	ajx_current("empty");
    	
    	$disabled_subtabs = array();
    	
    	if (income_config_option('disable_general_invoices_tab')) {
    		$disabled_subtabs[] = 'subtab-invoices';
    	}
    	
    	ajx_extra_data(array('disabled_subtabs' => $disabled_subtabs));
    	
    }

    function edit_logo() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));
        tpl_assign('option', $option);

        $logo = array_var($_FILES, 'new_logo');
        if (is_array($logo)) {
            try {
                if (!isset($logo['name']) || !isset($logo['type']) || !isset($logo['size']) || !isset($logo['tmp_name']) || !is_readable($logo['tmp_name'])) {
                    throw new InvalidUploadError($logo, lang('error upload file'));
                } // if

                $valid_types = array('image/jpg', 'image/jpeg', 'image/pjpeg', 'image/gif', 'image/png', 'image/x-png');
                $max_width = 400;
                $max_height = 128;

                if (!in_array($logo['type'], $valid_types) || !($image = getimagesize($logo['tmp_name']))) {
                    throw new InvalidUploadError($logo, lang('invalid upload type', 'JPG, GIF, PNG'));
                } // if

                $old_file = $option->getLogoPath();

                DB::beginWork();

                if (!$option->setLogo($logo['tmp_name'], $logo['type'], $max_width, $max_height, true)) {
                    throw new InvalidUploadError($avatar, lang('error edit invoice logo'));
                } // if

                DB::commit();

                if (is_file($old_file)) {
                    @unlink($old_file);
                } // uf

                flash_success(lang('success edit invoice logo'));
                ajx_current("back");
            } catch (Exception $e) {
                ajx_current("empty");
                DB::rollback();
                flash_error($e->getMessage());
            } // try
        } // if
    }

    function delete_logo() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));

        try {
            DB::beginWork();
            FileRepository::deleteFile($option->getValue());
            $option->setValue('');
            $option->save();
            DB::commit();

            flash_success(lang('success delete invoice logo'));
            ajx_current("back");
        } catch (Exception $e) {
            DB::rollback();
            flash_error(lang('error delete invoice logo'));
            ajx_current("empty");
        } // try
    }
    
    
    function get_project_billing_address() {
    	ajx_current("empty");
    	
    	$address_data = array();
    	$address = null;
    	
    	$client_member_id = array_var($_REQUEST, 'client_id');
    	$billing_contact_id = array_var($_REQUEST, 'bc_id');
    	$genid = array_var($_REQUEST, 'genid');
    	
    	$address = $this->find_billing_address_by_client_and_contact($client_member_id, $billing_contact_id);
    	
    	if ($address instanceof ContactAddress) {
    		$address_data = $address->getArrayInfo();
    	}
    	
    	ajx_extra_data(array('billing_address' => $address_data, 'genid' => $genid));
    }


    function find_billing_address_by_client_and_contact($client_member_id, $billing_contact_id = null) {
		$address = null;

		if ($billing_contact_id) {
    		$billing_contact = Contacts::findById($billing_contact_id);
    		
    		if ($billing_contact) {
    			$address = $billing_contact->getAddress('work');
    			if (!$address) $address = $billing_contact->getAddress('home');
    			if (!$address) $address = $billing_contact->getAddress('postal');
    			if (!$address) $address = $billing_contact->getAddress('other');
    		}
    		if ($address && trim($address->getStreet()) == '') {
    			$address = null;
    		}
    	}
    	if (!$address) {
    		$client_member = Members::getMemberById($client_member_id);
    		if ($client_member instanceof Member) {
    			$client_obj = Customers::findbyId($client_member->getObjectId());
    			if ($client_obj instanceof Customer) {
    				$billing_contact = $client_obj->getContact();
    				
    				if ($billing_contact) {
    					$address = $billing_contact->getAddress('work');
    					if (!$address) $address = $billing_contact->getAddress('home');
    					if (!$address) $address = $billing_contact->getAddress('postal');
    					if (!$address) $address = $billing_contact->getAddress('other');
    				}
    				if ($address && trim($address->getStreet()) == '') {
    					$address = null;
    				}
    			}
    		}
    	}

		return $address;
	}


    function find_billing_email_by_client_and_contact($client_member_id, $billing_contact_id = null) {
		$email_address = null;

		if ($billing_contact_id) {
    		$billing_contact = Contacts::findById($billing_contact_id);
    		
    		if ($billing_contact) {
    			$email_address = $billing_contact->getEmailAddress();
    		}
    	}
    	if (!$email_address) {
    		$client_member = Members::getMemberById($client_member_id);
    		if ($client_member instanceof Member) {
    			$client_obj = Customers::findbyId($client_member->getObjectId());
    			if ($client_obj instanceof Customer) {
    				$billing_contact = $client_obj->getContact();
    				
    				if ($billing_contact) {
    					$email_address = $billing_contact->getEmailAddress();
					}
    			}
    		}
    	}

		return $email_address;
	}
    

	function edit_timestlot_status_tab() {
		
	}
	
	function tasks_toolbar_actions() {
		
	}



	// function to mark time/expenses as billable, non billable or invoiced
	private function mark_time_as($new_status, $ids) {
		try {
			DB::beginWork();
			
			$this->change_time_status($new_status, $ids);
			
			DB::commit();
			ajx_current("reload");
			flash_success('Timeslots marked as '.$new_status.' successfully');

		} catch (Exception $e) {
			DB::rollback();
			ajx_current("empty");
			flash_error($e->getMessage());
		}		
	}

	private function change_time_status($new_status, $ids) {
		if (!is_array($ids)) {
			$ids = explode(',', $ids);
		}
		$ids = array_filter($ids, 'is_numeric');
		
		
		if (count($ids) > 0 && in_array($new_status, array('pending', 'invoiced', 'non_billable'))) {

			$sql = "
				UPDATE ".TABLE_PREFIX."timeslots
				SET invoicing_status = '$new_status'
				WHERE object_id IN (".implode(',', $ids).")
				AND (invoice_id = 0 OR (SELECT COUNT(o.id) FROM ".TABLE_PREFIX."objects o WHERE o.id=invoice_id AND o.trashed_by_id=0)=0)
			";
			DB::execute($sql);
		}
	}

	function mark_time_as_invoiced() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_time_as('invoiced', $ids);
	}

	function mark_time_as_unbilled() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_time_as('pending', $ids);
	}

	function mark_time_as_non_billable() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_time_as('non_billable', $ids);
	}

	
	private function mark_expense_as($new_status, $ids) {
		try {
			DB::beginWork();
			
			$this->change_expense_status($new_status, $ids);
			
			DB::commit();
			ajx_current("reload");
			flash_success('Expenses marked as '.$new_status.' successfully');

		} catch (Exception $e) {
			DB::rollback();
			ajx_current("empty");
			flash_error($e->getMessage());
		}		
	}

	private function change_expense_status($new_status, $ids) {
		if (!is_array($ids)) {
			$ids = explode(',', $ids);
		}
		$ids = array_filter($ids, 'is_numeric');
		
		if (count($ids) > 0 && in_array($new_status, array('pending', 'invoiced', 'non_billable'))) {

			$is_billable = $new_status == 'non_billable' ? '0' : '1';

			$sql = "
				UPDATE ".TABLE_PREFIX."payment_receipts
				SET invoicing_status = '$new_status', is_billable = $is_billable
				WHERE object_id IN (".implode(',', $ids).")
				AND (invoice_id = 0 OR (SELECT COUNT(o.id) FROM ".TABLE_PREFIX."objects o WHERE o.id=invoice_id AND o.trashed_by_id=0)=0)
			";
			DB::execute($sql);
		}
	}

	function mark_expense_as_invoiced() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_expense_as('invoiced', $ids);
	}

	function mark_expense_as_unbilled() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_expense_as('pending', $ids);
	}

	function mark_expense_as_non_billable() {
		$ids = array_var($_REQUEST, 'ids');
		$this->mark_expense_as('non_billable', $ids);
	}
}

// InvoiceController
?>