<?php

/**
 * InvoiceNotebookController
 * Generado el 20/8/2012
 * 
 */
class InvoiceNotebookController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the InvoiceNotebookController
     *
     * @access public
     * @param void
     * @return InvoiceNotebookController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
    }

    /**
     * Show invoice_notebooks
     *
     * @param void
     * @return null
     */
    function index() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $invoice_notebooks = IncomeInvoiceNotebooks::findAll(array('condition' => array('removed_by_id = 0')));
        tpl_assign('invoice_notebooks', $invoice_notebooks);
    }
    
    function add() {
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    
    	$invoice_notebook_post = array_var($_POST, 'invoice_notebook');
    	if (!is_array($invoice_notebook_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		
    		$ot = ObjectTypes::findByName('invoice');
    		$inv_types = ObjectSubtypes::findAll(array("conditions" => "object_type_id=".$ot->getId()));
    		
    		$array_type = array();
    		foreach ($inv_types as $inv_type) {
    			$array_type[] = array($inv_type->getId(),$inv_type->getName());
    		}
    		tpl_assign('inv_types', $array_type);
    		tpl_assign('invoice_notebook_edit', array());
    		tpl_assign('invoice_notebook', new IncomeInvoiceNotebook());
    	} else {
    		try {
    			DB::beginWork();
    			$invoice_notebook = new IncomeInvoiceNotebook();
    
    			$this->save_invoice_notebook($invoice_notebook_post, $invoice_notebook, true);
    
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function edit() {
    	$this->setTemplate("add");
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    
    	$id = get_id();
    	$invoice_notebook = IncomeInvoiceNotebooks::findById($id);
    
    	$invoice_notebook_post = array_var($_POST, 'invoice_notebook');
    	if (!is_array($invoice_notebook_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    
    		$ot = ObjectTypes::findByName('invoice');
    		$inv_types = ObjectSubtypes::findAll(array("conditions" => "object_type_id=".$ot->getId()));
    		
    		$array_type = array();
    		foreach ($inv_types as $inv_type) {
    			$array_type[] = array($inv_type->getId(),$inv_type->getName());
    		}
    		tpl_assign('inv_types', $array_type);
    		tpl_assign('invoice_notebook', $invoice_notebook);
    		tpl_assign('invoice_notebook_edit', $invoice_notebook->getArrayInfo());
    
    	} else {
    		try {
    			DB::beginWork();
    
    			$this->save_invoice_notebook($invoice_notebook_post, $invoice_notebook, false);
    
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("back");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function save_invoice_notebook($invoice_notebook_post, $invoice_notebook_data, $is_new=true) {
    
    	$invoice_notebook_data->setInvoiceTypeId($invoice_notebook_post['invoice_type_id']);
    	$invoice_notebook_data->setRut($invoice_notebook_post['rut']);
    	$invoice_notebook_data->setExpirationDate(getDateValue($invoice_notebook_post['expiration_date']));
    	$invoice_notebook_data->setSeries($invoice_notebook_post['series']);
    	$invoice_notebook_data->setInitialNumber($invoice_notebook_post['initial_number']);
    	$invoice_notebook_data->setFinalNumber($invoice_notebook_post['final_number']);
    	$invoice_notebook_data->setAddress($invoice_notebook_post['address']);
    	$invoice_notebook_data->setExtraInformation($invoice_notebook_post['extra_information']);
    	$invoice_notebook_data->setPressInformation($invoice_notebook_post['press_information']);
    	$invoice_notebook_data->save();
    
    	if ($is_new) {
    		flash_success(lang('success add invoice notebook'));
    	} else {
    		flash_success(lang('success edit invoice notebook'));
    	}
    
    }
    
    
    function delete() {
        ajx_current("empty");
        $invoice_notebook = IncomeInvoiceNotebooks::findById(get_id("invoice_notebook_id"));
        if ($invoice_notebook instanceof IncomeInvoiceNotebook) {
        	$invoice_notebook->delete();
        }

        flash_success(lang('success delete invoice notebook'));
    }

}

// InvoiceNotebookController
?>