<?php

/**
 * TaxController
 * Generado el 18/5/2012
 * 
 */
class TaxController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the TaxController
     *
     * @access public
     * @param void
     * @return TaxController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
    }

    /**
     * Show taxes
     *
     * @param void
     * @return null
     */
    function index() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $taxes = IncomeTaxes::findAll(array('condition' => array('removed_by_id = 0')));
        tpl_assign('taxes', $taxes);

    }
    
    
    function add() {
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    	
    	$tax_post = array_var($_POST, 'tax');
    	if (!is_array($tax_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		tpl_assign('tax_edit', array());
    		tpl_assign('tax', new IncomeTax());
    	} else {
	    	try {
	    		DB::beginWork();
	    		$tax_data = new IncomeTax();
	    		
				$this->save_tax($tax_post, $tax_data, true);
				
	    		if (array_var($_REQUEST, 'modal')) {
	    			evt_add("reload current panel");
		    		ajx_current("empty");
	    		} else {
	    			ajx_current("reload");
	    		}
	    		
	    		DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    function edit() {
    	$this->setTemplate("add");
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    	
    	$id = get_id();
    	$tax = IncomeTaxes::findById($id);
    	
    	$tax_post = array_var($_POST, 'tax');
    	if (!is_array($tax_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		
    		tpl_assign('tax', $tax);
    		$tax_record = IncomeTaxRecords::LastByTaxId($id);
            $tax_edit = array(
                'id' => $tax->getId(),
                'name' => $tax->getName(),
                'select_default' => $tax->getSelectDefault(),
                'type' => $tax_record->getType(),
                'value' => $tax_record->getValue()
            );
    		tpl_assign('tax_edit', $tax_edit);
    		
    	} else {
    		try {
    			DB::beginWork();
    			
    			$this->save_tax($tax_post, $tax, false);
    	
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    			 
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function save_tax($tax_post, $tax_data, $is_new=true) {
    	
    	$tax_data->setFromAttributes($tax_post);
    	$tax_data->save();
   		
   		$tax_record = new IncomeTaxRecord();
   		$tax_record->setTaxId($tax_data->getId());
   		$tax_record->setType($tax_post['type']);
   		$tax_record->setValue($tax_post['value']);
   		$tax_record->setDate(date("Y-m-d"));
   		$tax_record->save();
    	
    	if ($is_new) {
    		flash_success(lang('success add tax'));
    	} else {
    		flash_success(lang('success edit tax'));
    	}
    	
    }
    

    function delete() {
        ajx_current("empty");
        $tax = IncomeTaxes::findById(get_id("tax_id"));
        $tax->delete();

        flash_success(lang('success delete tax'));
    }

    /**
     * Show tax records
     *
     * @param void
     * @return null
     */
    function record() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $tax_records = IncomeTaxRecords::findAll(array(
                    "condition" => array("tax_id = ? AND removed_on = ? AND removed_by_id = 0", get_id("tax_id"), EMPTY_DATETIME),
                    'order' => '`date` DESC, `id` DESC'
                ));
        $tax_data = IncomeTaxes::findById(get_id("tax_id"));
        tpl_assign('tax_name', $tax_data->getName());
        tpl_assign('tax_records', $tax_records);
        tpl_assign('tax_id', get_id("tax_id"));

        $type_tax['percentage'] = array("percentage", lang('percentage'));
        $type_tax['fixed'] = array("fixed", lang('fixed'));
        tpl_assign('type_tax', $type_tax);

        $edit = false;
        if (get_id("tax_record_id")) {
            $tax_record = IncomeTaxRecords::findById(get_id("tax_record_id"));
            $info = array(
                'id' => $tax_record->getId(),
                'tax_id' => $tax_record->getTaxId(),
                'type' => $tax_record->getType(),
                'value' => $tax_record->getValue()
            );
            tpl_assign('tax_edit', $info);
            $edit = true;
        } else {
            $tax_record = new IncomeTaxRecord();
            $tax_record->setDate(date("Y-m-d"));
        }

        if (is_array(array_var($_POST, 'tax_record'))) {
            try {
                $tax_post = array_var($_POST, 'tax_record');

                DB::beginWork();
                $tax_record->setTaxId(get_id("tax_id"));
                $tax_record->setType($tax_post['type']);
                $tax_record->setValue($tax_post['value']);
                $tax_record->save();
                DB::commit();

                if ($edit) {
                    flash_success(lang('success edit tax record'));
                    ajx_current("back");
                } else {
                    flash_success(lang('success add tax record'));
                    ajx_current("reload");
                }
            } catch (Exception $e) {
                DB::rollback();
                flash_error($e->getMessage());
                ajx_current("empty");
            } // try
        } // if
    }

    function delete_record() {
        ajx_current("empty");
        $tax_record = IncomeTaxRecords::findById(get_id("tax_record_id"));
        $tax_record->delete();

        flash_success(lang('success delete tax'));
    }

}

// TaxController
?>