<?php

function income_sent_status_selector($input_name, $selected_value='', $attributes=null) {
	
	$combo_values = array('no', 'invoice_sent', 'receipt_sent', 'void_sent');
	foreach ($combo_values as $value) {
		$attr = array();
		if ($value == $selected_value) $attr['selected'] = 'selected';
		$options[] = option_tag(lang($value), $value, $attr);
	}
	
	return select_box($input_name, $options, $attributes);
}

function income_config_option($option_name) {
	$option = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='$option_name'"));
	$value = "";
	if ($option instanceof IncomeConfigOption) {
		$handler = $option->getConfigHandler();
		$value = $handler->getValue();
	}
	return $value;
}

function update_income_config_option($option_name, $value) {
	$option = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='$option_name'"));
	
	if ($option instanceof IncomeConfigOption) {			
		$option->setValue($value);
		$option->save();
		
	}
	return $option;
}

function is_invoice_status_available($status) {

	if (in_array($status, array("printed"))) {
		return in_array($status, income_config_option("enabled_invoices_status"));
	}
	return true;
}

function create_ordered_by_contact_from_invoice_data($data, $member_ids) {
	if (trim(trim($data['ordered_by_name']) ." ". trim($data['ordered_by_lastname'])) == "") return;

	$contact_data = array(
			'first_name' => trim($data['ordered_by_name']),
			'surname' => trim($data['ordered_by_lastname']),
			'email' => trim($data['ordered_by_email']),
	);

	return create_contact_from_data($contact_data, $member_ids);
}

function invoice_status_selector($input_name, $selected, $attributes=null) {

	$all_status = array("pending");
	if (is_invoice_status_available("printed")) {
		$all_status[] = "printed";
	}
	$all_status[] = "part_confirmed";
	$all_status[] = "confirmed";
	$all_status[] = "canceled";

	$options = array();
	foreach ($all_status as $status) {
		$attr = $status == $selected ? array("selected" => "selected") : null;
		$options[] = option_tag(lang($status), $status, $attr);
	}

	return select_box($input_name, $options, $attributes);
}


function create_client_member_by_email_and_name_and_set_billing_contact($client_data) {
	Env::useHelper('functions', 'crpm');
	// create the client
	$member = create_client_member_by_email_and_name($client_data);

	// set the billing contact
	$billing_contact_cp = CustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), "billing_contact");
	if ($billing_contact_cp instanceof CustomProperty && $member instanceof Member) {
		$customer = Customers::findById($member->getObjectId());

		$cp_val = new CustomPropertyValue();
		$cp_val->setObjectId($member->getObjectId());
		$cp_val->setCustomPropertyId($billing_contact_cp->getId());
		$cp_val->setValue($customer->getContactId());
		$cp_val->save();
	}

	return $member;
}


function fill_invoice_data_serie_and_number($invoice, &$invoice_data) {		
	if (!income_config_option("assign_number_when_printing") && $invoice->isNew()) {
		// get last used notebook
		$last_invoice_notebook = user_config_option('last_notebook_used');
		$use_invoice_notebooks = income_config_option('use_invoice_notebooks');
		$invoice_notebook = null;

		if($use_invoice_notebooks){
			$invoice_notebook = IncomeInvoiceNotebooks::findById($last_invoice_notebook);
			// if the notebook cannot be used for this invoice type -> find one usable
			if (!$invoice_notebook || $invoice_notebook->getColumnValue('invoice_type_id') != $invoice_data['object_subtype_id']) {
				$invoice_notebook = IncomeInvoiceNotebooks::findOne(array(
					"conditions" => array("invoice_type_id=? AND initial_number<final_number", $invoice_data['object_subtype_id']),
				));
			}
		}
		
		$number = null;
		if ($use_invoice_notebooks && $invoice_notebook && !$invoice_notebook->getExhausted()) {
			$invoice_numb = IncomeInvoice::findInvoiceNotebook($invoice_notebook->getId());
			if ($invoice_numb) {
				$num_last_inv = intval($invoice_numb->getInvoiceNumber());
				$number = $num_last_inv + 1;
			} else {
				$number = $invoice_notebook->getInitialNumber();
			}
		} else {
			// get the last invoice number, no matter the notebook
			$last_invoice = IncomeInvoices::getInvoiceWithGreatestNumber();
			if ($last_invoice instanceof IncomeInvoice) {
				$num_last_inv = intval($last_invoice->getInvoiceNumber());
				$number = $num_last_inv + 1;
			}
		}

		if ($number) {
			if($use_invoice_notebooks){
				$invoice_data["invoice_notebook_id"] = $invoice_notebook->getId();
				$invoice_data["invoice_number"] = $number;
				$invoice_data["series"] = $invoice_notebook->getSeries();

				if ($invoice_data['invoice_number'] == $invoice_notebook->getFinalNumber()) {
					$invoice_notebook->setExhausted(1);
					$invoice_notebook->save();
				}
			}else{
				$invoice_data["invoice_number"] = $number;
			}

		}
	}
}


function apply_batch_properties_to_invoice_data(&$invoice_generated_data, $member_id, $properties) {
	
	$invoice_properties = array_var($properties, $member_id);
	if ($invoice_properties) {
		
		$add_info = array_var($invoice_properties, 'additional_information');
		if ($add_info) {
			$invoice_generated_data['invoice']['additional_information'] = $add_info;
		}
		
		$currency_id = $invoice_generated_data['invoice_lines'][0]['currency_id'];
		
		// create a new line with the fee
		$add_fee = array_var($invoice_properties, 'additional_fee');
		if ($add_fee && is_numeric($add_fee)) {
			$fee_text = income_config_option('automatic_adm_fee_line_text');
			if (!$fee_text) $fee_text = lang('administration fee');
			$new_line = array(
					'currency_id' => $currency_id,
					'amount' => 1,
					'unit_value' => $add_fee,
					'total' => $add_fee,
					'detail' => $fee_text,
			);
			$invoice_generated_data['invoice_lines'][] = $new_line;
		}
		
		// create a new line with the discount
		$discount = array_var($invoice_properties, 'discount');
		if ($discount && is_numeric($discount)) {
			$new_line = array(
					'currency_id' => $currency_id,
					'amount' => 1,
					'unit_value' => -1 * $discount,
					'total' => -1 * $discount,
					'detail' => lang('discount'),
			);
			$invoice_generated_data['invoice_lines'][] = $new_line;
		}
	}
	
}


function find_objects_to_generate_invoice(DateTimeValue $billing_period_from, DateTimeValue $billing_period_to, $member = null) {
	
	$from_str = $billing_period_from->format(DATE_MYSQL);
	$to_str = $billing_period_to->format(DATE_MYSQL);
	
	$member_ids = $member instanceof Member ? array($member->getId()) : array();
	
	$classified_in_member_ids = $member_ids;
	Hook::fire("timeslots_for_invoices_modify_classification", array(), $classified_in_member_ids);

	$res = Timeslots::instance()->listing(array(
			"ignore_context" => true,
			"member_ids" => $classified_in_member_ids,
			"extra_conditions" => " AND fixed_billing>0 AND invoicing_status='pending' AND start_time >= '$from_str' AND start_time <= '$to_str'",
	));
	
	$timeslots = $res->objects;
	if (!$timeslots) $timeslots = array();
	
	if (Plugins::instance()->isActivePlugin('expenses2')) {

		// dates had the user timezone applied to get the gmt 0 time, actual expenses (payemnt receitps) don't use hours so we have to remove the tz from the date limits
		$billing_period_from->advance(logged_user()->getUserTimezoneValue(), true);
    	$billing_period_to->advance(logged_user()->getUserTimezoneValue(), true);
		// build the date strings again
		$from_str = $billing_period_from->format(DATE_MYSQL);
		$to_str = $billing_period_to->endOfDay()->format(DATE_MYSQL);

		// add more members to the query if any plugin requrires some specific classification
		if ($member instanceof Member) {
			$classified_in_member_ids = $member_ids;
			Hook::fire("expenses_for_invoices_modify_classification", array(), $classified_in_member_ids);
		}
		
		$res = PaymentReceipts::instance()->listing(array(
				"ignore_context" => true,
				"member_ids" => $classified_in_member_ids,
				"extra_conditions" => " 
					AND (`total_price` > 0 OR `amount` > 0)
					AND `is_billable`=1 AND invoicing_status='pending' AND `date` >= '$from_str' AND `date` <= '$to_str'",
		));
		$payment_receipts = $res->objects;
	}
	if (!$payment_receipts) $payment_receipts = array();
	
	return array(
			'timeslots' => $timeslots,
			'payment_receipts' => $payment_receipts,
	);
}


function group_objects_to_generate_invoice($timeslots, $payment_receipts) {
	
	$grouped_by_project = array();
	$project_ot = ObjectTypes::findByName('project');
	$time_ot = ObjectTypes::findByName('timeslot');
	$pr_ot = ObjectTypes::findByName('payment_receipt');
	
	$timeslots_indexed = array();
	foreach ($timeslots as $object) {
		$timeslots_indexed[$object->getId()] = $object;
	}
	$timeslots_ids = array_keys($timeslots_indexed);
	
	$payment_receipts_indexed = array();
	foreach ($payment_receipts as $object) {
		$payment_receipts_indexed[$object->getId()] = $object;
	}
	$payment_receipts_ids = array_keys($payment_receipts_indexed);
	
	$all_object_ids = array_merge($timeslots_ids, $payment_receipts_ids);
	if (count($all_object_ids) > 0) {
		
		$sql = "SELECT o.id, m.id as member_id, o.object_type_id
				FROM ".TABLE_PREFIX."objects o 
				INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
				INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
				WHERE o.id IN (".implode(',', $all_object_ids).")
				AND om.is_optimization=0
				AND m.object_type_id=".$project_ot->getId()."
				ORDER BY m.id, o.id;
		";
		$ordered_rows = DB::executeAll($sql);
		
		foreach ($ordered_rows as $row) {
			$mem = $row['member_id'];
			$obj = $row['id'];
			$ot = $row['object_type_id'];
			
			if ($mem == 0) continue; // exclude objects without project
			
			if (!isset($grouped_by_project[$mem])) $grouped_by_project[$mem] = array('timeslots' => array(), 'payment_receipts' => array());
			if ($ot == $time_ot->getId()) {
				$grouped_by_project[$mem]['timeslots'][] = $timeslots_indexed[$obj];
			} else if ($ot == $pr_ot->getId()) {
				$grouped_by_project[$mem]['payment_receipts'][] = $payment_receipts_indexed[$obj];
			}
		}
	}
	
	return $grouped_by_project;
}


function get_client_billing_address_from_cp($client_member) {
	$return_address = null;

	$billing_address_cp = CustomProperties::getCustomPropertyByCode($client_member->getObjectTypeId(), 'billing_address');
	if ($billing_address_cp) {
		$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $billing_address_cp->getId());
		if ($cp_val) {
			$address_string = trim($cp_val->getValue());
			if ($address_string != '') {

				$exploded = explode("|", $address_string);
					
				if (count($exploded) > 0) {
					$return_address = new ContactAddress();

					$return_address->setAddressTypeId(array_var($exploded, 0, ''));
					$return_address->setStreet(array_var($exploded, 1, ''));
					$return_address->setCity(array_var($exploded, 2, ''));
					$return_address->setState(array_var($exploded, 3, ''));
					$return_address->setCountry(array_var($exploded, 4, ''));
					$return_address->setZipCode(array_var($exploded, 5, ''));

				}
			}
		}
	}

	return $return_address;
}


function getPeriodFilter($period_filter) {
	$now = DateTimeValueLib::now();
	$now->advance(logged_user()->getUserTimezoneValue(), true);

	$from_date = null;
	$to_date = null;

	switch ($period_filter) {
		case 0:
			break;
		case 1: // today
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), $now->getDay(), $now->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), $now->getDay(), $now->getYear());
			break;
		case 2: // this week
			$sunday = $now->getMondayOfWeek()->add('d', -1);
			$nextSunday = $now->getMondayOfWeek()->add('w', 1)->add('d', -2);
			$from_date = DateTimeValueLib::make(0, 0, 0, $sunday->getMonth(), $sunday->getDay(), $sunday->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $nextSunday->getMonth(), $nextSunday->getDay(), $nextSunday->getYear());
			break;
		case 3: // last week
			$sunday = $now->getMondayOfWeek()->add('w', -1)->add('d', -1);
			$nextSunday = $now->getMondayOfWeek()->add('d', -2);
			$from_date = DateTimeValueLib::make(0, 0, 0, $sunday->getMonth(), $sunday->getDay(), $sunday->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $nextSunday->getMonth(), $nextSunday->getDay(), $nextSunday->getYear());
			break;
		case 4: // this month
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 1, $now->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 1, $now->getYear())->add('M', 1)->add('d', -1);
			break;
		case 5: // last month
			$now->add('M', -1);
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 1, $now->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 1, $now->getYear())->add('M', 1)->add('d', -1);
			break;
		case 6: //Date interval
			break;
		case 7: //first half of month
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 1, $now->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 15, $now->getYear());
			break;
		case 8: //second half of month
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 16, $now->getYear());
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 1, $now->getYear())->add('M', 1)->add('d', -1);
			break;
		case 9: //First half of last month
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 1, $now->getYear())->add('M', -1);
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 15, $now->getYear())->add('M', -1);
			break;
		case 10: //Last half of last month
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), 16, $now->getYear())->add('M', -1);
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), 1, $now->getYear())->add('d', -1);
			break;
		case 11: //Yesterday
			$from_date = DateTimeValueLib::make(0, 0, 0, $now->getMonth(), $now->getDay(), $now->getYear())->add('d', -1);
			$to_date = DateTimeValueLib::make(23, 59, 59, $now->getMonth(), $now->getDay(), $now->getYear())->add('d', -1);
			break;
		default:
			break;
	}

	return array(
		'from' => $from_date,
		'to' => $to_date
	);
}