<?php

/**
 * IncomeConfigOption
 * Generado el 21/5/2012
 * 
 */
class IncomeConfigOption extends BaseIncomeConfigOption {

    /**
     * Config handler instance
     *
     * @var ConfigHandler
     */
    private $config_handler;

    /**
     * In DB we store uniqe name. This function will convert that name to the catetory display name in propert language
     *
     * @param void
     * @return string
     */
    function getDisplayName() {
        return lang('config income name ' . $this->getName());
    }

    /**
     * Return config handler instance
     *
     * @param void
     * @return ConfigHandler
     */
    function getConfigHandler() {
        if ($this->config_handler instanceof ConfigHandler)
            return $this->config_handler;

        $handler_class = trim($this->getConfigHandlerClass());
        if (!$handler_class)
            throw new Error('Handler class is not set for "' . $this->getName() . '" config option');

        $handler = new $handler_class();
        if (!($handler instanceof ConfigHandler))
            throw new Error('Handler class for "' . $this->getName() . '" config option is not valid');

        $handler->setConfigOption($this);
        $handler->setRawValue(parent::getValue());
        $this->config_handler = $handler;
        return $this->config_handler;
    }

// getConfigHandler

    /**
     * Render this control
     *
     * @param string $control_name
     * @return string
     */
    function render($control_name) {
        $handler = $this->getConfigHandler();
        return $handler->render($control_name);
    }
    
    function setValue($value) {
    	$handler = $this->getConfigHandler();
    	$handler->setValue($value);
    	return parent::setValue($handler->getRawValue());
    }

    // ---------------------------------------------------
    //  Logo
    // ---------------------------------------------------

    /**
     * Set logo value
     *
     * @param string $source Source file
     * @param integer $max_width
     * @param integer $max_height
     * @param boolean $save Save object when done
     * @return null
     */
    function setLogo($source, $fileType, $max_width = 50, $max_height = 50, $save = true) {
        if (!is_readable($source))
            return false;

        do {
            $temp_file = CACHE_DIR . '/' . sha1(uniqid(rand(), true));
        } while (is_file($temp_file));

        try {
            Env::useLibrary('simplegd');

            $image = new SimpleGdImage($source);
            if ($image->getImageType() == IMAGETYPE_PNG) {
                if ($image->getHeight() > 128 || $image->getWidth() > 128) {
                    //	resize images if are png bigger than 128 px
                    /*$thumb = $image->scale($max_width, $max_height, SimpleGdImage::BOUNDARY_DECREASE_ONLY, false);
                    $thumb->saveAs($temp_file, IMAGETYPE_PNG);
                    $public_fileid = FileRepository::addFile($temp_file, array('type' => 'image/png', 'public' => true));*/
                    $public_fileid = FileRepository::addFile($source, array('type' => 'image/png', 'public' => true));
                } else {
                    // keep the png as it is.
                    $public_fileid = FileRepository::addFile($source, array('type' => 'image/png', 'public' => true));
                }
            } else {
                $thumb = $image->scale($max_width, $max_height, SimpleGdImage::BOUNDARY_DECREASE_ONLY, false);
                $thumb->saveAs($temp_file, IMAGETYPE_PNG);
                $public_fileid = FileRepository::addFile($temp_file, array('type' => 'image/png', 'public' => true));
            }
            if ($public_fileid) {
                $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));
                $option->setValue($public_fileid);
                if ($save) {
                    $option->save();
                } // if
            } // if

            $result = true;
        } catch (Exception $e) {
            $result = false;
        } // try
        // Cleanup
        if (!$result && $public_fileid) {
            FileRepository::deleteFile($public_fileid);
        } // if
        @unlink($temp_file);

        return $result;
    }

    /**
     * Returns path of company logo. This function will not check if file really exists
     *
     * @access public
     * @param void
     * @return string
     */
    function getLogoPath() {
        $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));
        return PublicFiles::getFilePath($option->getValue());
    }

    /**
     * description
     *
     * @access public
     * @param void
     * @return string
     */
    function getLogoUrl() {
        $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));
        return $this->hasLogo() ? get_url('files', 'get_public_file', array('id' => $option->getValue())) : get_image_url('default-avatar.png');
    }

    /**
     * Returns true if this company have logo file value and logo file exists
     *
     * @access public
     * @param void
     * @return boolean
     */
    function hasLogo() {
        $option = IncomeConfigOptions::findOne(array("condition" => array("name = 'invoice_logo'")));
        return trim($option->getValue()) && FileRepository::isInRepository($option->getValue());
    }

    /**
     * Return update avatar URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getEditLogoUrl() {
        return get_url('income', 'edit_logo');
    }

    /**
     * Return delete logo URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getDeleteLogoUrl() {
        return get_url('income', 'delete_logo');
    }

}

?>