<?php


abstract class BaseIncomeInvoiceLineTaxes extends DataManager {

    /**
     * Column name => Column type map
     *
     * @var array
     * @static
     */
    static private $columns = array(
        'invoice_line_id' => DATA_TYPE_INTEGER,
    	'tax_id' => DATA_TYPE_INTEGER,
    	'value' => DATA_TYPE_FLOAT,
    	'applied_value' => DATA_TYPE_FLOAT,
    );

    /**
     * Construct
     *
     * @return BaseIncomeInvoiceLineTaxes
     */
    function __construct() {
        Hook::fire('object_definition', 'IncomeInvoiceLineTax', self::$columns);
        parent::__construct('IncomeInvoiceLineTax', 'income_invoice_line_taxes', true);
    }

    // -------------------------------------------------------
    //  Description methods
    // -------------------------------------------------------

    /**
     * Return array of object columns
     *
     * @access public
     * @param void
     * @return array
     */
    function getColumns() {
        return array_keys(self::$columns);
    }

    /**
     * Return column type
     *
     * @access public
     * @param string $column_name
     * @return string
     */
    function getColumnType($column_name) {
        if (isset(self::$columns[$column_name])) {
            return self::$columns[$column_name];
        } else {
            return DATA_TYPE_STRING;
        } // if
    }

    /**
     * Return array of PK columns. If only one column is PK returns its name as string
     *
     * @access public
     * @param void
     * @return array or string
     */
    function getPkColumns() {
        return array('invoice_line_id', 'tax_id');
    }

    /**
     * Return name of first auto_incremenent column if it exists
     *
     * @access public
     * @param void
     * @return string
     */
    function getAutoIncrementColumn() {
        return null;
    }

    // -------------------------------------------------------
    //  Finders
    // -------------------------------------------------------

    /**
     * Do a SELECT query over database with specified arguments
     *
     * @access public
     * @param array $arguments Array of query arguments. Fields:
     *
     *  - one - select first row
     *  - conditions - additional conditions
     *  - order - order by string
     *  - offset - limit offset, valid only if limit is present
     *  - limit
     *
     * @return array
     * @throws DBQueryError
     */
    function find($arguments = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::find($arguments);
        } else {
            return IncomeInvoiceLineTaxes::instance()->find($arguments);
        } // if
    }

    /**
     * Find all records
     *
     * @access public
     * @param array $arguments
     * @return array
     */
    function findAll($arguments = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::findAll($arguments);
        } else {
            return IncomeInvoiceLineTaxes::instance()->findAll($arguments);
        } // if
    }

    /**
     * Find one specific record
     *
     * @access public
     * @param array $arguments
     * @return IncomeInvoiceLineTax
     */
    function findOne($arguments = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::findOne($arguments);
        } else {
            return IncomeInvoiceLineTaxes::instance()->findOne($arguments);
        } // if
    }

    /**
     * Return object by its PK value
     *
     * @access public
     * @param mixed $id
     * @param boolean $force_reload If true cache will be skipped and data will be loaded from database
     * @return MailAccount
     */
    function findById($id, $force_reload = false) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::findById($id, $force_reload);
        } else {
            return IncomeInvoiceLineTaxes::instance()->findById($id, $force_reload);
        } // if
    }

    /**
     * Return number of rows in this table
     *
     * @access public
     * @param string $conditions Query conditions
     * @return integer
     */
    function count($condition = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::count($condition);
        } else {
            return IncomeInvoiceLineTaxes::instance()->count($condition);
        } // if
    }

    /**
     * Delete rows that match specific conditions. If $conditions is NULL all rows from table will be deleted
     *
     * @access public
     * @param string $conditions Query conditions
     * @return boolean
     */
    function delete($condition = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::delete($condition);
        } else {
            return IncomeInvoiceLineTaxes::instance()->delete($condition);
        } // if
    }

    /**
     * This function will return paginated result. Result is an array where first element is
     * array of returned object and second populated pagination object that can be used for
     * obtaining and rendering pagination data using various helpers.
     *
     * Items and pagination array vars are indexed with 0 for items and 1 for pagination
     * because you can't use associative indexing with list() construct
     *
     * @access public
     * @param array $arguments Query argumens (@see find()) Limit and offset are ignored!
     * @param integer $items_per_page Number of items per page
     * @param integer $current_page Current page number
     * @return array
     */
    function paginate($arguments = null, $items_per_page = 10, $current_page = 1, $count = null) {
        if (isset($this) && instance_of($this, 'IncomeInvoiceLineTaxes')) {
            return parent::paginate($arguments, $items_per_page, $current_page);
        } else {
            return IncomeInvoiceLineTaxes::instance()->paginate($arguments, $items_per_page, $current_page);
        } // if
    }

    /**
     * Return manager instance
     *
     * @return IncomeInvoiceLineTaxes
     */
    function instance() {
        static $instance;
        if (!instance_of($instance, 'IncomeInvoiceLineTaxes')) {
            $instance = new IncomeInvoiceLineTaxes();
        }
        return $instance;
    }

}

?>