<?php

/**
 * IncomeInvoices
 * Generado el 10/5/2012
 * 
 */
class IncomeInvoices extends BaseIncomeInvoices {

    function __construct() {
        parent::__construct();
        $this->object_type_name = 'invoice';
    }


    function getColumnsToAggregateInTotals() {
    	return array_merge(parent::getColumnsToAggregateInTotals(), array(
    		'subtotal' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id'),
    		'total' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id')
    	));
    }
    
    
    function getInvoicesInMembers($member_ids, $extra_conditions = "") {
    	
    	$member_invoices = $this->listing(array(
    			"extra_conditions" => "AND status != 'canceled' AND trashed_by_id=0 $extra_conditions",
    			"ignore_context" => true,
    			"member_ids" => $member_ids,
    	))->objects;
    	
    	return $member_invoices;
    }
    
    function getInvoicedTotalInMembers($member_ids, $extra_conditions = "") {
    	
    	$total_billed = 0;
    	$member_invoices = $this->getInvoicesInMembers($member_ids, $extra_conditions);
    	foreach ($member_invoices as $invoice) {
    		$total_billed += $invoice->getTotal();
    	}
    	
    	return $total_billed;
    }

	static function getInvoiceWithGreatestNumber() {
		return IncomeInvoices::findOne(array(
			'conditions' => array('trashed_on = ? AND trashed_by_id = 0', EMPTY_DATETIME),
			'order' => 'CAST(invoice_number as UNSIGNED) DESC',
		));
	}

	static function findInvoiceByNumber($invoice_number) {
		return IncomeInvoices::findOne(array(
			'conditions' => array('invoice_number = ? AND trashed_by_id=0', $invoice_number)
		));
	}

	static function findInvoiceContainNumber($invoice_number) {
		return IncomeInvoices::findOne(array(
			'conditions' => array('invoice_number LIKE "'.$invoice_number.'%"'),
			'order' => 'invoice_number DESC'
		));
	}
    
    static function getInvoiceTemplateFromMembers($members) {
    	
    	$project_ot = ObjectTypes::findByName('project');
    	$client_ot = ObjectTypes::findByName('customer');
    	
    	$project_member = null;
    	$client_member = null;
    	
    	foreach ($members as $m) {
    		if ($m instanceof Member) {
	    		if ($m->getObjectTypeId() == $project_ot->getId()) $project_member = $m;
	    		if ($m->getObjectTypeId() == $client_ot->getId()) $client_member = $m;
    		}
    	}
    	
    	$invoice_template = null;
    	
    	if ($project_member instanceof Member) {
    		
    		$cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'invoice_template');
    		$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $cp->getId());
    		if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    		
    		if (!$invoice_template) {
    			if (!$client_member instanceof Member) {
    				Env::useHelper('functions', 'crpm');
    				$clients_dimension = get_customers_dimension();
    				if ($clients_dimension->getId() != $project_member->getDimensionId()) {
    					$client_mem_id = get_associated_status_member_id($project_member, $clients_dimension, $client_ot, false, 'project_customer');
    					$client_member = Members::getMemberById($client_mem_id);
    				} else {
    					$project_parents = $project_member->getAllParentMembersInHierarchy(false, false);
    					foreach ($project_parents as $p) {
    						if ($p->getObjectTypeId() == $client_ot->getId()) {
    							$client_member = $p;
    							break;
    						}
    					}
    				}
    			}
    			
    			if ($client_member instanceof Member) {
    				$cp = CustomProperties::getCustomPropertyByCode($client_ot->getId(), 'invoice_template');
    				$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $cp->getId());
    				if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    			}
    		}
    		
    	} else if ($client_member instanceof Member) {
    		
    		$cp = CustomProperties::getCustomPropertyByCode($client_ot->getId(), 'invoice_template');
    		$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $cp->getId());
    		if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    	}
    	
    	return $invoice_template;
    }
}

?>