<?php
	$invoice_currency = Currencies::findById($invoice->getCurrencyId());
	if (!$invoice_currency instanceof Currency) {
		$invoice_currency_symbol = config_option('currency_code');
		$invoice_currency_name = $invoice_currency_symbol;
	} else {
		$invoice_currency_symbol = $invoice_currency->getSymbol();
		$invoice_currency_name = $invoice_currency->getName() . " ($invoice_currency_symbol)";
	}
	
	$invoice_payment_url = null;
	if ($invoice->getStatus() == 'pending') {
		$invoice_payment_base_url = income_config_option('invoice_payment_url');
		if ($invoice_payment_base_url) {
			$invoice_payment_url = $invoice_payment_base_url . "?invoice_id=".$invoice->getId() . "&contact_id=".logged_user()->getId();
			if (!is_null($invoice->getMemberInvoice())) {
				$customer_id = $invoice->getMemberInvoice()->getId();
				$invoice_payment_url .= "&customer_id=$customer_id";
			}
		}
	}

	$ordered_by = Contacts::findById($invoice->getOrderedById());
?>
<div class="commentsTitle"><?php echo $invoice->getInvoiceType() . " " . $invoice->getSeries() . " " . ($invoice->getInvoiceNumber() != '' ? $invoice->getInvoiceNumber() : "") ?>
	&nbsp;<div style="float:right;"><span class="invoice-row-status-<?php echo $invoice->getStatus()?>"><?php echo lang($invoice->getStatus())?></span></div>
</div>
<div style="margin-left:5px;float:left;">
    <table class="blank" cellspacing="2">
        <tr>
            <td>
                <?php echo lang('date') ?>
            </td>
            <td>
                <?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?>
            </td>
        </tr>
        <tr>
            <td>
                <?php echo lang('currency') ?>
            </td>
            <td>
                <?php echo $invoice_currency_name ?>
            </td>
        </tr>
        <tr>
            <td>
                <?php echo lang('expiration date') ?>
            </td>
            <td>
                <?php
                    if($invoice->getExpirationDate() instanceof DateTimeValue){
                        echo format_date($invoice->getExpirationDate()->advance(logged_user()->getUserTimezoneValue() * -1, false));
                    }
                ?>
            </td>
        </tr>
        <?php if ($invoice->getCustomerName() != "") { ?>
        <tr>
            <td>
                <?php echo lang('bill to') ?>
            </td>
            <td>
                <?php echo nl2br($invoice->getCustomerName()) ?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($ordered_by instanceof Contact && trim($ordered_by->getObjectName()) != trim($invoice->getCustomerName())) { ?>
        <tr>
            <td>
                <?php echo lang('ordered by') ?>
            </td>
            <td>
				<a class="internalLink underline" href="<?php echo $ordered_by->getViewUrl() ?>"><?php echo $ordered_by->getObjectName() ?></a>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getCustomerAddress()) { ?>
        <tr>
            <td>
                <?php echo lang('customer address') ?>
            </td>
            <td>
                <?php echo $invoice->getCustomerAddress() ?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getCustomerCity()) { ?>
        <tr>
            <td>
                <?php echo lang('city') ?>
            </td>
            <td>
                <?php echo $invoice->getCustomerCity() ?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getCustomerCountry()) { ?>
        <tr>
            <td>
                <?php echo lang('country') ?>
            </td>
            <td>
                <?php echo CountryCodes::getCountryNameByCode($invoice->getCustomerCountry()) ?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getCustomerNumber()) { ?>
        <tr>
            <td>
                <?php echo lang('customer number') ?>
            </td>
            <td>
                <?php echo $invoice->getCustomerNumber() ?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getCustomerRut()) { ?>
        <tr>
            <td>
                <?php echo lang('customer rut') ?>
            </td>
            <td>
                <?php 
					$customer_rut_html = $invoice->getCustomerRut();
					Hook::fire('override_invoice_view_customer_rut_html', array('invoice'=>$invoice), $customer_rut_html);
					echo $customer_rut_html;
				?>
            </td>
        </tr>
        <?php } ?>
        <?php if ($invoice->getAdditionalInformation()) { ?>
        <tr>
            <td>
                <?php echo lang('additional information') ?>
            </td>
            <td>
                <?php echo nl2br($invoice->getAdditionalInformation()) ?>
            </td>
        </tr>
        <?php } ?>
        <tr>
            <td>
                <?php echo lang('sent') ?>
            </td>
            <td>
                <?php echo $invoice->getSent() ? lang($invoice->getSent()) : lang('no') ?>
            </td>
        </tr>
    </table>
    
</div>


<?php if ($invoice_payment_url) { ?>
<div style="float:right;position:sticky;top:0;">
	<button class="submit blue" onclick="location.href='<?php echo $invoice_payment_url?>'"><?php echo lang("pay this invoice"); ?></button>
</div>
<?php } ?>
<div class="clear"></div>


<?php if(count($invoice->getInvoiceReferences()) > 0){?>
<table class="invoice-lines">
    <tr>
        <th colspan="3">
            <?php echo lang('invoice references') ?>
        </th>
    </tr>
    <tr class="header">
        <td>
            <b><?php echo lang('series') ?></b>
        </td>
        <td>
            <b><?php echo lang('invoice number') ?></b>
        </td>
        <td>
            <b><?php echo lang('description') ?></b>
        </td>
    </tr>
    <?php foreach ($invoice->getInvoiceReferences() as $invoice_ref){?>
    <tr>
        <td><?php echo $invoice_ref->getSerie()?></td>
        <td><?php echo $invoice_ref->getInvoiceNumber()?></td>
        <td><?php echo $invoice_ref->getDetail()?></td>
    </tr>
    <?php }?>
</table>
<?php }?>
<table class="invoice-lines">
    <tr>
        <th>
            <?php echo lang('details') ?>
        </th>
    </tr>
    <tr>
        <td>
            <table class="lines-container">
                <tr class="header">
                    <td style="min-width:200px;">&nbsp;</td>
                    <td align="center">
                        <b><?php echo lang('quantity') ?></b>
                    </td>
                    <td align="right">
                        <b><?php echo lang('unit value') ?></b>
                    </td>
                    <td align="right">
                        <b><?php echo lang('total') ?></b>
                    </td>
                </tr>
                <?php foreach ($invoice->getInvoiceLines() as $invoice_line){

	                	$line_currency = Currencies::findById($invoice_line->getCurrencyId());
	                	if (!$line_currency instanceof Currency) {
	                		$line_currency_symbol = config_option('currency_code');
	                	} else {
	                		$line_currency_symbol = $line_currency->getSymbol();
	                	}

                        $unitValueConverted=$invoice_line->getUnitValue();

                        if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
                            //convert to main currency
                            if($invoice_line->getCurrencyId()!=$invoice->getCurrencyId())
                            {
                                $unitValueConverted=$invoice_line->getUnitValue()*$invoice_line->getExchangeRate();
                            }
                        }
                ?>
                <tr>
                    <td>
                        <?php echo str_replace("    ","&nbsp;&nbsp;&nbsp;&nbsp;", nl2br($invoice_line->getDetail())) ?>
                    </td>
                    <td align="center">
                        <?php echo $invoice_line->getAmount()?>
                    </td>
                    <td align="right">
                        <?php 
						if ($unitValueConverted < 0) {
							echo '('. $invoice_currency_symbol. ' ' . number_format(abs($unitValueConverted), 2) . ')';
						} else {
							echo $invoice_currency_symbol. ' ' . number_format($unitValueConverted, 2);
						}
						?>
                    </td>
                    <td align="right">
                        <?php 

						if ($invoice_line->getSubTotal() < 0) {
							echo '('. $invoice_currency_symbol. ' ' . number_format(abs($invoice_line->getSubTotal()), 2) . ')';
						} else {
							echo $invoice_currency_symbol. ' ' . number_format($invoice_line->getSubTotal(), 2);
						}
						?>
                    </td>
                </tr>
                <?php }?>

                <tr class="separator">
                    <td>
                        <div class="bold"><?php echo lang('subtotal') ?></div>
                    </td>
                    <td></td>
                    <td></td>
                    <td align="right">
                        <div class="bold"><?php echo $invoice_currency_symbol. ' ' . number_format($invoice->getSubtotal(), 2);?></div>
                    </td>
                </tr>
                <?php foreach ($invoice->getTaxes() as $invoice_tax) {
                		$tax_def = $invoice_tax->getType() == 'percentage' ? $invoice_tax->getValue() . " %" : array_var($def_currency, 'symbol', '$'). ' ' .$invoice_tax->getValue();
                		$tax_instance = IncomeTaxes::findById($invoice_tax->getTaxId());
                		if ($tax_instance instanceof IncomeTax) {
                			$tax_name = $tax_instance->getName();
                		} else {
							$tax_name = lang('n/a');
						}
                ?>
                <tr>
                    <td><?php echo $tax_name ?></td>
                    <td></td>
                    <td align="right"><?php echo $tax_def ?></td>
                    <td align="right"><?php echo $invoice_currency_symbol. ' ' . number_format($invoice_tax->getFixedValue(), 2)?></td>
                </tr>
                <?php }?>

                <tr class="separator">
                	<td>
                        <div class="bold"><?php echo lang('total') ?></div>
                    </td>
                    <td></td>
                    <td></td>
                    <td align="right">
                        <div class="bold"><?php echo $invoice_currency_symbol. ' ' . number_format($invoice->getTotal(), 2)?></div>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>


<?php if (count($related_invoices) > 0) { ?>

<div class="commentsTitle"><?php echo lang("related invoices"); ?></div>

<div style="margin-left:5px;">
	<table class="blank">
<?php
	foreach ($related_invoices as $related_invoice) {
		?><tr><td><?php
			echo $related_invoice['label'];
		?></td><td><?php
			echo format_date($related_invoice['object']->getDate(), null, 0);
		?></td><td><?php
			$link = $related_invoice['object']->getViewUrl();
			$name = $related_invoice['object']->getObjectName();
		?>
			<a href="<?php echo $link?>" class="internalLink underline link-ico ico-open-link"><?php echo $name?></a>

		</td></tr><?php
	}
?>
	</table>
</div>

<?php } ?>


<?php $null=null; Hook::fire('view_invoice_additional_sections', array('invoice'=>$invoice), $null); ?>


