<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$terms_text = "";
if (class_exists('IncomePaymentTerms')) {

    $payment_term = IncomePaymentTerms::findById($invoice->getPaymentTermId());
    if (!$payment_term) {
        $payment_term = IncomePaymentTerms::getDefaultPaymentTerm();
    }
    if ($payment_term) {
        $terms_text = $payment_term->getName();
    }
} else {
    // use old custom property
    $payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
    if ($payment_period_cp instanceof CustomProperty) {
        $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
        if ($payment_period_cp_val instanceof CustomPropertyValue) {
            $terms_text = $payment_period_cp_val->getValue();
        }
    }
}

$project_and_po_number_text = "";
$inv_po_number_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), "po_number");
if ($inv_po_number_cp instanceof CustomProperty) {

    $cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $inv_po_number_cp->getId());
    $verify = $cpval->getValue();
    if ($cpval instanceof CustomPropertyValue && $verify != '') {
        $project_and_po_number_text = trim($cpval->getValue());
    } else {
        if ($project_member instanceof Member) {
            $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "po_number_for_project");
            if ($project_id_cp instanceof CustomProperty) {
                $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
                if ($cpval instanceof CustomPropertyValue) {
                    $project_and_po_number_text = trim($cpval->getValue());
                }
            }
        }
    }
}

/**
 * get the "description project" from the custom property "description_special"
 */
$project_description = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "description_special");

if ($project_member instanceof Member) {

    if ($project_description instanceof CustomProperty) {
        $project_descriptionval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_description->getId());
        if ($project_descriptionval instanceof CustomPropertyValue) {

            $project_description_value =  $project_descriptionval->getValue();
        }
    }
}

$due_date_text = '';
if ($invoice->getExpirationDate() instanceof DateTimeValue) {
    $due_date_text = format_date($invoice->getExpirationDate(), null, 0);
}

$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
$billing_from_date = '';
$billing_to_date = '';
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_from_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_from_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_to_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_to_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}

$project_member_complete_name = "";
if ($project_member) {
    $tmp_array = array($project_member);
    build_member_list_text_to_show_in_trees($tmp_array);
    $project_member_complete_name = $project_member->getName();
}

$project_manager_name = "";
if ($project_member) {
    $project_manager_id = get_associated_status_member_id($project_member, Dimensions::findByCode('project_managers'));
    $project_manager = Members::getMemberById($project_manager_id);
    if ($project_manager) $project_manager_name = $project_manager->getName();
}

$project_type_name = "";
if ($project_member) {
    $project_type_id = get_associated_status_member_id($project_member, Dimensions::findByCode('countries'));
    $project_type = Members::getMemberById($project_type_id);
    if ($project_type) {
        $project_type_value = $project_type->getName();
        $project_type_value = preg_replace('/[^A-Za-z\-]/', '', $project_type_value);
    }
}

$allocation_percentage = 100;
$whole_invoice_total = $invoice->getTotal();

$summary = $invoice->getProjectSummary(array('ignore_discount' => true));
$allocation_per_client = array_var($summary, 'allocation_per_client', array());

?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    @media print {
        .wrapperAditionalANDPayment {
            page-break-inside: avoid;
        }
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 900px;
        margin: 10px auto;
        padding: 10px 0;
    }

    #items {
        width: 100%;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
    }

    #items th {
        background: #eee;
    }

    #items td.description {
        /* width: 300px;*/
    }

    .TotalWidht {
        width: 112px;
    }

    #items .left {
        text-align: left;
    }

    #items .right {
        text-align: right;
    }

    #items .center {
        text-align: center;
    }

    #items .short {
        width: 90px;
    }

    #items td.total-line {
        border-right: 0;
        text-align: right;
        font-weight: bold
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 20px;
        background: none;
        text-align: right;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    div.footer-info {
        margin-top: 10px;
        font-size: 14px;
        font-weight: bold;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    th,
    td {
        border-color: #1a4567 !important;
    }

    table {
        border-spacing: 0;
    }

    #bill_to table {
        border: .1px solid #1a4567;
        border-radius: 15px;
        padding-left: 5px;
        padding-right: 20px;
        padding-top: 5px;
        padding-bottom: 30px;
        min-width: 250px;
    }

    #bill_to table td {
        border: none !important;
        padding: 2px 5px;
    }

    .additionalInfo td {
        word-break: break-all;
        white-space: pre-wrap;
    }

    .paymentOptions table tbody tr td {
        border-bottom: none;
        text-align: center;
        padding: 2px;
        color: red;
    }

    .paymentOptions table tr:last-child td {
        border-bottom: .1px solid #1a4567;
    }

    .paymentOptions table tr:first-child td {
        padding-top: 15px;
    }

    .paymentOptions table tr:last-child td {
        padding-bottom: 15px;
    }

    .footerBottomData {
        display: flex;
        justify-content: center;
        margin-top: 20px;
    }

    .billingQuestion,
    .financeCharge {
        text-align: center;
    }

    .billingQuestion {
        border: .1px solid #1a4567;
        padding: 10px 45px;
        display: flex;
        align-items: center;
        margin-top: 20px;
    }

    .financeCharge {
        padding: 10px;
        display: flex;
        align-items: center;
    }

    table.customData th,
    table tr th {
        background-color: #1a4567 !important;
        -webkit-print-color-adjust: exact !important;
        color: #ffffff !important;
        font-size: 10px;
        padding: 5px 10px;
    }

    table.customData td,
    table tr td {
        border-left: .1px solid #1a4567;
        border-bottom: .1px solid #1a4567;
        border-collapse: collapse;
        padding: 7px;
    }


    table.customData td:last-child,
    table tr td:last-child {
        border-right: .1px solid #1a4567;
    }

    table.customData tbody:last-child th:first-child,
    table.customData thead th:first-child {
        border-top-left-radius: 10px;
    }

    table.customData tbody:last-child th:last-child,
    table.customData thead th:last-child {
        border-top-right-radius: 10px;
    }

    table.customData tbody:last-child tr:last-child td:last-child {
        border-bottom-right-radius: 10px;
    }

    table.customData tbody:last-child tr:last-child td:first-child {
        border-bottom-left-radius: 10px;
    }


    .addressTable tr td {
        padding: 2px 10px;
    }

    .aditionalANDPayment {
        display: flex;
        justify-content: space-between;
    }

    table.summary-information tbody>tr:first-child>td {
        border-top-left-radius: 10px;
        border-bottom-left-radius: 10px;
        border-top: .1px solid #1a4567;
    }

    table.summary-information tbody:last-child tr:last-child td:first-child {
        border-bottom-left-radius: 0;
    }

    .additionalInfo table {
        width: 100%;
    }


    #items td {
        border-bottom: none;
        padding: 2px 5px;
    }

    #items tr:last-child td {
        border-bottom: .1px solid #1a4567;
    }

    #items.summary-information tr:last-child td {
        border-bottom: .1px solid #1a4567;
    }

    #items.summary-information tr:first-child td {
        border-bottom: .1px solid #1a4567;
    }

    #items.summary-information td {
        padding: 10px;
    }

    #items tbody .item-row:nth-last-child(-n+2) td {
        border-bottom: .1px solid #1a4567;
    }

    #items tbody:last-child th:first-child,
    #items thead:first-child th:first-child {
        border-top-left-radius: 10px;
    }

    #items tbody:last-child th:last-child,
    #items thead:first-child th:last-child {
        border-top-right-radius: 10px;
    }

    #items tbody:last-child tr:last-child td:last-child {
        border-bottom-right-radius: 10px;
    }

    #items tbody:last-child tr:last-child td:first-child {
        border-bottom-left-radius: 10px;
    }

    #items.summary-information tbody:last-child tr:last-child td:first-child {
        border-bottom-left-radius: 0;
    }



    #items>thead:first-child th:first-child {
        border-top-left-radius: 10px;
    }

    #items>thead:first-child th:last-child {
        border-top-right-radius: 10px;
    }

    /* #items tr:last-child td:first-child {
        border-bottom-left-radius: 10px;
    } */

    .item-row.short:nth-last-of-type(-n+3) td:first-child {
        border-bottom-left-radius: 10px;
    }

    .item-row.short:nth-last-of-type(-n+3) td {
        border-bottom: .1px solid #1a4567 !important;
    }

    /* .item-row.short td:first-child {
        border-bottom-left-radius: 10px;
    } */

    .item-row.short~tr:last-child td {
        border-top: .1px solid #1a4567 !important;
    }

    .dateWidht {
        max-width: 60px;
        width: 60px;
    }
</style>

<div id="page-wrap">
    <div id="customer" style="display: flex; align-items: center; justify-content: space-between;">
        <div style="">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img width="140" height="80" src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>
        </div>

        <div style="width: 100%;">
            <div>
                <h2 style="text-align:center;font-size:30px;font-weight:bold;color: #1a4567;">
                    <?php echo $to_pay_information['company_name']; ?>
                </h2>
                <span style="text-align: center; display: inline-block; width:100%;">Environmental Services & Government Affairs</span>
            </div>
        </div>
    </div>

    <div id="adress" style="width: 100%; display: flex; justify-content: center;">
        <div style="">
            <table class="addressTable" style="text-align: center;">
                <?php
                if ($invoice_options['view_address']) :
                ?>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px; font-weight: 600;">Corporate Office:</td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            7202 E. 87th Street, Suite 110
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            Indianapolis, IN 46256
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            (P): (317) 578-4233
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 12px;">
                            (F): (317) 578-4250
                        </td>
                    </tr>
                <?php
                endif;
                ?>
            </table>
        </div>
        <div style="">
            <table class="addressTable" style="text-align: center;">
                <?php
                if ($invoice_options['view_address']) :
                ?>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px; font-weight: 600;">Other locations:</td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            <?php // echo $invoice_notebook['address'] 
                            ?><?php //echo $to_pay_information['address']; 
                                ?>
                            Goshen, IN (574) 238-4998
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            <?php // echo $to_pay_information['address_location']; 
                            ?>
                            Louisville, KY (502) 890-7702
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2" style="border: 0px; text-align: center; font-weight:bold; font-size: 11px;">
                            <?php // echo $to_pay_information['phone']; 
                            ?>
                            Guaynabo, PR (787) 789-1889
                        </td>
                    </tr>
                <?php
                endif;
                ?>
            </table>
        </div>
    </div>
    <div class="invoiceBillandNumber" style="display: flex; justify-content: space-between; margin-top: 5px; margin-bottom: 10px;">
        <div id="bill_to">
            <div style="display: flex; align-items: center; margin-bottom: 5px; margin-left: 15px;">
                <h3><?php echo lang('bill to') ?>:</h3>
            </div>
            <table id="items" class="customData">
                <tr>
                    <td colspan="1"><?php echo nl2br($bill_to_information['company_name']); ?></td>
                </tr>

                <?php $attnValue = $invoice->getAttn(); //show attn only if it is not empty 
                if ($attnValue != '') { ?>
                    <tr>
                        <td colspan="1">Attn:&nbsp;<?php echo nl2br($attnValue); ?></td>
                    </tr>
                <?php } ?>

                <tr>
                    <td colspan="1"><?php echo nl2br($bill_to_information['address']); ?></td>
                </tr>
                <tr>
                    <!-- <td colspan="1">
                        </td>-->
                    <td colspan="1"><?php echo $bill_to_information['address_location']; ?></td>
                </tr>
                <!-- <tr> -->
                <!--<td colspan="1" colspan="2" style="width: 15%;"><b><?php echo lang('phone') ?></b></td>-->
                <!-- <td colspan="1"><?php echo $bill_to_information['phone']; ?></td> -->
                <!-- </tr> -->
                <!-- <tr> -->
                <!--<td colspan="1" colspan="2" style="width: 15%;"><b><?php echo lang('email') ?></b></td>-->
                <!-- <td colspan="1"><?php echo $bill_to_information['email']; ?></td> -->
                <!-- </tr> -->
            </table>
        </div>
        <div>
            <span style="margin-left: 10px;">FID#35-1970390</span>
        </div>
        <div id="invoice_number" style="margin-top: -10px;">
            <div style="text-align:center; font-size: 25px; font-weight: bold; margin-bottom: 10px; color: #1a4567;">
                <?php echo $invoice->getInvoiceType() ?>
            </div>
            <div style="">
                <table id="meta" class="customData" width="200px">
                    <?php // if ($invoice_options['view_rut']) : 
                    ?>
                    <!-- <tr>
                            <td style="border: 0px;" colspan="2">
                                <b><?php //echo lang('rut') . ": " 
                                    ?></b><?php //echo $invoice_notebook['rut'] 
                                            ?>
                            </td>
                        </tr> -->
                    <?php // endif; 
                    ?>
                    <?php if ($invoice_options['view_invoice_number']) : ?>
                        <tr>
                            <th class="meta-head" colspan="2" style="text-align: center;"><?php echo lang('date') ?></th>
                            <th class="meta-head" colspan="4" style="text-align: center;"><?php echo $invoice->getInvoiceType() ?> #</th>
                        </tr>
                        <tr>
                            <td colspan="2" style="text-align: center;"><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                            <td colspan="4" style="text-align: center;"><?php echo $invoice->getSeries() . " " . $invoice->getInvoiceNumber() ?></td>
                        </tr>
                    <?php endif; ?>
                    <?php //if ($invoice_options['view_date']): 
                    ?>
                    <!-- <tr>
                              <td class="meta-head" style="text-align: center;"><?php //echo lang('expiration date') 
                                                                                ?></td>
                              <td class="meta-head" style="text-align: center;"><?php //echo lang('status') 
                                                                                ?></td>
                          </tr>
                          <tr>
                              <td style="text-align: center;"><?php //echo format_date($invoice_notebook['expiration_date']) 
                                                                ?></td>
                              <td style="text-align: center;" class="print-invoice-status <?php //echo $invoice->getStatus()
                                                                                            ?>"><?php //echo $inovice_status 
                                                                                                ?></td>
                          </tr> -->
                    <?php //endif; 
                    ?>
                    <?php if ($invoice_options['view_customer_number'] && $invoice->getCustomerNumber()) : ?>
                        <tr>
                            <td class="meta-head"><?php echo lang('customer number') ?></td>
                            <td style="text-align: center;"><?php echo $invoice->getCustomerNumber() ?></td>
                        </tr>
                    <?php endif; ?>
                    <?php if ($show_inovice_status === true) : ?>

                    <?php endif; ?>

                </table>
            </div>
        </div>
    </div>

    <div id="project_resume" style="margin-top: 10px; margin-bottom: 10px;">
        <table style="width:100%" class="customData">
            <tr>
                <th class="meta-head"> Terms </th>
                <th class="meta-head"> <?php echo lang('expiration date') ?> </th>
                <th class="meta-head"> <?php echo lang('project') ?> </th>
                <th class="meta-head"> Office </th>
                <th class="meta-head"> P.O Number </th>
            </tr>
            <tr>
                <td style="text-align: center;"><?php echo $terms_text; ?></td>
                <td style="text-align: center;"><?php echo $due_date_text; ?></td>
                <td width="250px"><?php echo $project_member_complete_name; ?></td>
                <td style="text-align: center;"><?php echo $project_type_value; ?></td>
                <td style="text-align: center;"><?php echo $project_and_po_number_text; ?></td>
            </tr>
        </table>
    </div>

    <!-- ADITIONAL INFORMATION -->
    <?php if ($invoice->getAdditionalInformation() != '') : ?>
        <div class="additionalInfo">
            <table class="customData">
                <thead>
                    <tr>
                        <th>Description</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td><?= $invoice->getAdditionalInformation(); ?></td>
                    </tr>
                </tbody>
            </table>
        </div>
    <?php endif; ?>

    <?php
    if ($invoice_template instanceof IncomeInvoiceTemplate) {
        echo $invoice_template->getHtmlDetailsTableForInvoice($invoice);
    } else {
    ?>

        <table id="items" class="customData" style="margin-bottom: 10px;">
            <tr>
                <th><?php echo lang('detail') ?></th>
                <th><?php echo lang('quantity') ?></th>
                <th><?php echo lang('unit value') ?></th>
                <th>Total due for this invoice</th>
                <!-- <th><?php echo lang('total') ?></th> -->

            </tr>
            <?php foreach ($invoice->getInvoiceLines() as $invoice_line) :
                $line_currency = Currencies::getCurrency($invoice_line->getCurrencyId());
                $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
            ?>
                <tr class="item-row short">
                    <td class="description"><?php echo nl2br($invoice_line->getDetail()); ?></td>
                    <td><?php echo $invoice_line->getAmount() ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                </tr>
            <?php endforeach; ?>
            <?php
            if ($invoice_options['view_taxes']) :
                foreach ($taxes as $tax) :
            ?>
                    <tr>
                        <td colspan="2" class="blank"></td>
                        <td class="total-line-tax"><?php echo $tax['name'] ?></td>
                        <td class="total-value">
                            <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($tax['tax_value'], 2) ?></div>
                        </td>
                    </tr>
            <?php
                endforeach;
            endif;
            ?>
            <tr>
                <td colspan="2" rowspan="2" class="blank description"><?php
                                                                        if (trim($invoice->getAdditionalInformation()) != '') {
                                                                            echo "<b>" . lang('additional information') . ":</b><br />" . $invoice->getAdditionalInformation();
                                                                        }
                                                                        ?></td>
                <td class="total-line">Subtotal</td>
                <td class="total-value">
                    <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($invoice->getSubtotal(), 2) ?></div>
                </td>
            </tr>
            <tr>

                <td class="total-line">Total due for this invoice</td>
                <td class="total-value">
                    <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                </td>
            </tr>
        </table>
    <?php } ?>

    <?php
    // if this invoice will be paid by more than one billing client then show this summary box specifying each payor's amount
    if (count($allocation_per_client) > 1) { ?>
        <table id="items" class="summary-information customData">
            <tr>
                <td rowspan="<?php echo (count($allocation_per_client) + 1) ?>" class="left" style="width:220px;"><?php
                                                                                                                    if ($billing_from_date != '' && $billing_to_date != '') {
                                                                                                                        echo "Billing period: " . $billing_from_date . " through " . $billing_to_date;
                                                                                                                    }
                                                                                                                    ?></td>
                <th class="left">Payor</th>
                <th class="right short">To be paid (%)</th>
                <th class="right short">To be paid ($)</th>
            </tr>
            <?php foreach ($allocation_per_client as $client_allocation) { ?>
                <tr>
                    <td class="medium"><?php echo $client_allocation['name'] ?></td>
                    <td class="right short"><?php echo number_format($client_allocation['allocation'], 2) ?> %</td>
                    <td class="right short"><?php echo format_money_amount($client_allocation['amount'], $currency_symbol)  ?></td>
                </tr>
            <?php } ?>
        </table>
    <?php } ?>

    <div class="footer-info"><?php echo income_config_option('invoice_footer_disclaimer') ?></div>

    <div style="margin-top: 10px">
        <div>
            <table style="text-align: left; margin-right:20px;" class="customData">
                <?php
                if ($invoice_options['view_press_information']) :
                ?>
                    <tr>
                        <td colspan="2" style="border: 0px;">
                            <?php echo nl2br($invoice_notebook['press_information']) ?>
                        </td>
                    </tr>
                <?php
                endif;
                ?>
            </table>
        </div>
        <!--            <div style="float: left; margin-right:20px;">-->
        <!--                --><?php
                                //                if ($invoice_options['view_expiration_date']):
                                //                    
                                ?>
        <!--                    <div style="text-align: center; padding: 3px;">-->
        <!--                        --><?php //echo lang('expiration date') 
                                        ?>
        <!--                    </div>-->
        <!--                    <div style="text-align: center; padding: 3px;">-->
        <!--                        --><?php //echo format_date($invoice_notebook['expiration_date']) 
                                        ?>
        <!--                    </div>-->
        <!--                --><?php
                                //                endif;
                                //                
                                ?>
        <!--            </div>-->
        <div>
            <table style="text-align: right;">
                <tr>
                    <td colspan="2" style="border: 0px;">
                        <?php echo $copy['name'] ?>
                    </td>
                </tr>
            </table>
        </div>
        <div style="clear:both"></div>
    </div>

    <!-- Fixed Footer -->
    <div class="wrapperAditionalANDPayment">
        <div class="aditionalANDPayment">
            <div>
                <div class="paymentOptions">
                    <table class="customData" width="315px">
                        <thead>
                            <tr>
                                <th>Payment Options</th>
                            </tr>
                        </thead>
                        <tbody>
                            <tr>
                                <td>Pay ONLINE at www.bcaconsultants.com</td>
                            </tr>
                            <tr>
                                <td>Go to Contact Us, then Pay Bills</td>
                            </tr>
                            <tr>
                                <td>OR Mail to 7202 E 87th Street, Suite 110</td>
                            </tr>
                            <tr>
                                <td>Indianapolis, IN 46256</td>
                            </tr>
                        </tbody>
                    </table>
                </div>
                <div class="billingQuestion">
                    <h3>Billing Questions: 1-317-749-0060</h3>
                </div>
            </div>
        </div>
        <div class="footerBottomData">
            <div class="financeCharge">
                <h4>A finance charge of 1.5% per month may be added to past-due accounts.</h4>
            </div>
        </div>
    </div>
</div>


<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>