<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

// => PO NUMBER ::
$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$project_and_po_number_text = "";

$inv_po_number_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), "po_number");

if ($inv_po_number_cp instanceof CustomProperty) {
    $cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $inv_po_number_cp->getId());
    if ($cpval instanceof CustomPropertyValue) {
        $project_and_po_number_text = trim($cpval->getValue());
    }
}
if ($project_and_po_number_text == '' && $project_member instanceof Member) {
    $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "po_number");
    $po_number_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $po_number_cp_val = $cpval->getValue();
        }
    }

    $project_and_po_number_text = $project_member->getName();
    if ($po_number_cp_val != "") {
        $project_and_po_number_text .= " $po_number_cp_val";
    }
}

//=> DATE DUE ::
$due_date_text = '';
if ($invoice->getExpirationDate() instanceof DateTimeValue) {
    $due_date_text = format_date($invoice->getExpirationDate(), null, 0);
}

$client_project_number = '';
if ($project_member instanceof Member) {
    $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_id");

    $client_project = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_contact");

    if ($client_project instanceof CustomProperty) {
        $pjctval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $client_project->getId());

        if ($pjctval instanceof CustomPropertyValue) {
            $client_project_number = $pjctval->getValue();
        }
    }

    $project_id_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $tmp_project_obj_id = $project_member->getObjectId();

        $tmp_mem = $project_member;
        while ($project_id_cp_val == "" && $tmp_project_obj_id > 0) {
            $cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $project_id_cp->getId());
            if ($cpval instanceof CustomPropertyValue) {
                $project_id_cp_val = $cpval->getValue();
            }
            if ($project_id_cp_val == "") {
                $parent_member = $tmp_mem->getParentMember();
                if ($parent_member instanceof Member) $tmp_project_obj_id = $parent_member->getObjectId();
                else $tmp_project_obj_id = 0;

                $tmp_mem = $parent_member;
            }
        }
    }

    // description
    $project_description = "";
    $po_desc_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "description_special");
    if ($po_desc_cp instanceof CustomProperty) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $po_desc_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $project_description = $cpval->getValue();
        }
    }

    // manager
    $project_member_manager = '';
    $manager_member = null;
    $managers_dim = Dimensions::findByCode('project_managers');
    // if invoice is classified in a manager then use that member
    foreach ($invoice->getMembers() as $inv_mem) {
        if ($inv_mem->getDimensionId() == $managers_dim->getId()) {
            $manager_member = $inv_mem;
            break;
        }
    }
    // if not classified in any manager then use the manager associated to the project
    if (!$manager_member) {
        $manager_member_id = get_associated_status_member_id($project_member, $managers_dim);
        $manager_member = Members::getMemberById($manager_member_id);
    }
    if ($manager_member instanceof Member) {
        $project_member_manager = $manager_member->getName();
    }
}

$terms_text = "";
if (class_exists('IncomePaymentTerms')) {

    $payment_term = IncomePaymentTerms::findById($invoice->getPaymentTermId());
    if (!$payment_term) {
        $payment_term = IncomePaymentTerms::getDefaultPaymentTerm();
    }
    if ($payment_term) {
        $terms_text = $payment_term->getName();
    }
} else {
    // use old custom property
    $payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
    if ($payment_period_cp instanceof CustomProperty) {
        $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
        if ($payment_period_cp_val instanceof CustomPropertyValue) {
            $terms_text = $payment_period_cp_val->getValue();
        }
    }
}


//=> BILLING CONTACT ::
function get_custom_property_value($ot_id, $object, $code)
{
    $cp = CustomProperties::getCustomPropertyByCode($ot_id, $code);
    if ($cp instanceof CustomProperty) {

        if ($object instanceof Member) :
            $value = CustomPropertyValues::getCustomPropertyValue($object->getObjectId(), $cp->getId());
        else :
            $value = CustomPropertyValues::getCustomPropertyValue($object->getId(), $cp->getId());
        endif;

        return $value instanceof CustomPropertyValue ? $value->getValue() : '';
    } else {
        return '';
    }
}

// Get client info
$client_id = get_associated_status_member_id($project_member, Dimensions::findByCode('customers'));
$client_member = Members::getMemberById($client_id);
$client_name = $client_member->getName();
$client_ot = ObjectTypes::findByName('customer');
$client_ot_id = $client_ot->getId();

// Get main contact info
$main_contact_name = "";
$main_contact_id = get_custom_property_value($client_ot_id, $client_member, 'main_contact');
if ($main_contact_id) {
    $main_contact = Contacts::findById($main_contact_id);
    $main_contact_name = $main_contact instanceof Contact ? $main_contact->getName() : '';
}

// Get billing contact info
$billing_contact_name = "";
$billing_contact_phone = "";
$billing_contact_email = "";
// $billing_contact_id = get_custom_property_value($client_ot_id, $client_member, 'billing_contact');
$billing_contact_id = $invoice->getOrderedById();

if ($billing_contact_id) {
    $billing_contact = Contacts::findById($billing_contact_id);
    $billing_contact_phone = $invoice->getCustomerPhone();
    if ($billing_contact instanceof Contact) {
        // Get name
        $billing_contact_name = $billing_contact->getName();
        // Get telephone
        if ($billing_contact_phone == '') :
            $contact_telephone = ContactTelephones::findOne(array('conditions' => "contact_id=" . $billing_contact_id));
            if ($contact_telephone instanceof ContactTelephone) {
                $billing_contact_phone = $contact_telephone->getNumber();
            }
        endif;
        //Get email
        $contact_email = $billing_contact->getEmail();
        $billing_contact_email = $contact_email instanceof ContactEmail ? $contact_email->getEmailAddress() : '';
    }
}

?>
<style>
    @import url('https://fonts.googleapis.com/css2?family=Commissioner:wght@500;600;700&family=Josefin+Sans:wght@300&family=Lexend+Deca:wght@100;200;300;400;500&family=Poppins:wght@300;500;600;700&display=swap');

    * {
        margin: 0;
        padding: 0;
    }

    ul {
        list-style: none;
    }

    #page-wrap {
        width: 900px;
        margin: 10px auto;
        padding: 10px 0;
    }

    .headerInvoice {
        display: flex;
        justify-content: space-between;
        align-items: center;
    }

    .invoiceLogo {
        max-width: 310px;
    }

    .titleInvoice {
        display: flex;
        justify-content: center;
        width: 100%;
        font-family: 'Josefin Sans', sans-serif;
        color: #4e5a5f;
        font-size: 1.8em;
    }

    .headerSlogan .slogan {
        text-align: center;
        margin: 0 auto 20px;
        font-family: 'Poppins', sans-serif;
    }

    .companyData {
        display: flex;
        justify-content: center;
        margin-bottom: 30px;
    }

    .flexItems {
        display: flex;
        justify-content: space-between;
        text-transform: uppercase;
    }

    .companyData>.flexContainer:first-child {
        width: 60%;
    }

    .companyData>.flexContainer:last-child {
        width: 40%;
    }

    .companyData>.flexContainer .limitedColumn:first-child {
        margin-top: 20px;
    }

    .companyData>.flexContainer .limitedColumn:last-child {
        margin-top: 70px;
    }

    .listItem {
        font-weight: normal;
        display: block;
    }

    table#items tbody tr:nth-child(even),
    .tableData tbody tr:nth-child(even) {
        background: #fefffe;
        -webkit-print-color-adjust: exact !important;
    }

    table#items tbody tr:nth-child(odd),
    .tableData tbody tr:nth-child(odd) {
        background: #dadfe1;
        -webkit-print-color-adjust: exact !important;
    }

    .address li,
    .projectContact li,
    .toAddress li,
    .projectContact h4 {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    .main-header.object-type-header tr th {
        text-align: center !important;
    }

    .projectContact,
    .toAddress {
        margin-top: 30px;
    }

    .projectContact h4 {
        margin: 0 0 5px;
        text-decoration: underline;
    }

    .toAddress h4 {
        font-family: 'Commissioner', sans-serif;
        font-size: 17px;
        line-height: 1.5;
    }

    .toAddress li {
        margin-left: 80px;
    }

    .limitedColumn h5 {
        font-family: 'Commissioner', sans-serif;
        font-size: 17px;
        line-height: 1.5;
    }

    .groupItems {
        display: flex;
        align-items: center;
    }

    .groupItems h5,
    .groupItems span {
        flex: 1 1 0px
    }

    .groupItems span {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    table#items,
    .tableData {
        border-spacing: 0;
        width: 100%;
        margin-bottom: 20px;
    }

    table#items tr th,
    .tableData tr th {
        border-top: 1px solid #4e5a5f;
        border-bottom: 1px solid #4e5a5f;
        text-transform: uppercase;
    }

    table#items:last-child,
    .tableData:last-child {
        margin-bottom: 0;
    }

    table#items tr th,
    .tableData tr th {
        color: #3a4346;
        font-family: 'Commissioner', sans-serif;
        font-size: 14px;
        padding: 3px;
        padding: 2px 5px;
    }

    table#items tr td,
    .tableData tr td {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 400;
        color: black;
        line-height: 1.5;
        font-size: 14px;
        padding: 2px 5px;
    }

    table#items tr th,
    .tableData tr th {
        text-align: left;
    }

    table#items tbody:last-child tr:last-child td,
    .tableData tbody:last-child tr:last-child td {
        /* border-bottom: 1px solid #4e5a5f; */
    }

    .footerContent {
        margin-bottom: 30px;
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    .footerContent strong font {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
        font-size: 1rem;
        margin-top: 15px;

    }

    .footerContent img {
        max-height: 60px;
        margin-left: 20px;
    }

    .footerList {
        margin-top: -20px;
    }

    .footerList h4 {
        margin: 0 0 5px;
        text-decoration: underline;
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    .footerList ul {
        display: flex;
        width: 75%;
        justify-content: space-between;
    }

    .footerList ul li {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    .items-group-name {
        height: 22px;
        display: flex;
        align-items: center;
        background: #dadfe1;
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 500;
        color: black;
        line-height: 1.5;
        font-size: 14px;
        padding: 2px 5px;
    }

    .tableContainer {
        border-top: 1px solid;
        border-bottom: 1px solid;
    }

    .projectContact div {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
    }

    .totalLine {
        font-family: 'Lexend Deca', sans-serif;
        font-weight: 300;
        color: #4e5a5f;
        line-height: 1.5;
        border-top: 1px solid;
        padding: 0 !important;
    }

    tr .totalLine::last-child {
        border-bottom: 1px solid;
    }

    .projectContactDescription {
        word-break: break-all;
        white-space: pre-wrap;
    }
</style>

<div id="page-wrap">

    <div class="headerInvoice">
        <div id="customer">
            <img class="invoiceLogo" src="<?= $invoice_options['invoice_logo'] ?>" alt="logo" />
        </div>
        <div class="titleInvoice">
            <h2><?php echo strtoupper($invoice->getInvoiceType()) ?></h2>
        </div>
    </div>

    <div class="headerSlogan">
        <h3 class="slogan">Sensible solutions for project success</h3>
    </div>

    <div class="companyData">
        <div class="flexContainer">
            <ul class="address">
                <li class="listItem"><?= $to_pay_information['company_name']; ?></li>
                <li class="listItem"><?= $to_pay_information['address']; ?></li>
                <li class="listItem"><?= $to_pay_information['address_location']; ?></li>
            </ul>
            <?php if ($client_project != '') : ?>
                <div class="projectContact">
                    <h4>Project Contact:</h4>
                    <div class="listItem projectContactDescription"><?= $client_project_number; ?></div>
                </div>
            <?php endif; ?>
            <ul class="toAddress">
                <h4>TO</h4>
                <li><span><?php echo $invoice->getCustomerLegalName(); ?></span></li>
                <li><span><?php echo $invoice->getAttn(); ?></span></li>
                <li class="listItem"><span><?php echo $invoice->getCustomerAddress(); ?></span></li>
                <li>
                    <?php
                    $city_state_str =  $invoice->getCustomerCity();
                    $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($invoice->getCustomerState() . " " .  $invoice->getCustomerZipcode());
                    echo $city_state_str;
                    ?>
                </li>
                <!-- <li class="listItem"><?= $bill_to_information['city']; ?></li> -->
                <li class="listItem"><span><?= $invoice->getCustomerPhone(); ?></span></li>
                <li class="listItem"><?= $bill_to_information['email']; ?></li>
            </ul>
        </div>
        <div class="flexContainer">
            <div class="limitedColumn">
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('invoice no'); ?></h5>
                    <span class="listItem"><?= $invoice->getSeries() . " " . $invoice->getInvoiceNumber() ?></span>
                </div>
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('date submitted'); ?></h5>
                    <span class="listItem"><?= format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></span>
                </div>
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('customer id'); ?></h5>
                    <span class="listItem"><?= $invoice->getCustomerId(); ?></span>
                </div>
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('expiration date'); ?></h5>
                    <span class="listItem"><?= $due_date_text; ?></span>
                </div>
            </div>
            <div class="limitedColumn">
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('po no'); ?></h5>
                    <span class="listItem"><?= $project_and_po_number_text; ?></span>
                </div>
                <!-- <div class="groupItems">
                    <h5 class="flexItems">Revision</h5>
                    <span class="listItem">NONE</span>
                </div> -->
                <div class="groupItems">
                    <h5 class="flexItems"><?= lang('buyer'); ?></h5>
                    <span class="listItem"><?= $billing_contact_name ?></span>
                </div>
                <div class="groupItems">
                    <h5 class="flexItems"></h5>
                    <span class="listItem"><?= $billing_contact_phone ?></span>
                </div>
            </div>
        </div>
    </div>
    <table class="tableData">
        <thead>
            <tr>
                <th>JOB</th>
                <th>PAYMENT TERMS</th>
            </tr>
        </thead>
        <tbody>
            <tr>
                <td><?= $project_member->getName(); ?></td>
                <td><?= $terms_text; ?></td>
            </tr>
        </tbody>
    </table>
    <div class="tableContainer">
        <?php
        if ($invoice_template instanceof IncomeInvoiceTemplate) {
            echo $invoice_template->getHtmlDetailsTableForInvoice($invoice);
        } else {
        ?>

            <table id="items">
                <tr>
                    <th><?php echo lang('detail') ?></th>
                    <th><?php echo lang('quantity') ?></th>
                    <th><?php echo lang('unit value') ?></th>
                    <th><?php echo lang('total') ?></th>
                </tr>
                <?php foreach ($invoice->getInvoiceLines() as $invoice_line) :
                    $line_currency = Currencies::getCurrency($invoice_line->getCurrencyId());
                    $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
                ?>
                    <tr class="item-row">
                        <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                        <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                        <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                        <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                    </tr>
                <?php endforeach; ?>
                <?php
                if ($invoice_options['view_taxes']) :
                    foreach ($taxes as $tax) :
                ?>
                        <tr>
                            <td colspan="2" class="blank"></td>
                            <td class="total-line-tax"><?php echo $tax['name'] ?></td>
                            <td class="total-value">
                                <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($tax['tax_value'], 2) ?></div>
                            </td>
                        </tr>
                <?php
                    endforeach;
                endif;
                ?>
                <tr>
                    <td colspan="2" rowspan="2" class="blank description"><?php
                                                                            if (trim($invoice->getAdditionalInformation()) != '') {
                                                                                echo "<b>" . lang('additional information') . ":</b><br />" . $invoice->getAdditionalInformation();
                                                                            }
                                                                            ?></td>
                    <td class="total-line">Subtotal</td>
                    <td class="total-value">
                        <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($invoice->getSubtotal(), 2) ?></div>
                    </td>
                </tr>
                <tr>

                    <td class="total-line">Total</td>
                    <td class="total-value">
                        <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                    </td>
                </tr>
            </table>
        <?php } ?>
    </div>

    <div class="footer-info"><?php echo income_config_option('invoice_footer_disclaimer') ?></div>

    <div style="margin: 10px 0 0 0;">
        <div style="float: left;">
            <table style="text-align: left; margin-right:20px;">
                <?php
                if ($invoice_options['view_press_information']) :
                ?>
                    <tr>
                        <td colspan="2" style="border: 0px;">
                            <?php echo nl2br($invoice_notebook['press_information']) ?>
                        </td>
                    </tr>
                <?php
                endif;
                ?>
            </table>
        </div>
        <div style="float: right;">
            <table style="text-align: right;">
                <tr>
                    <td colspan="2" style="border: 0px;">
                        <?php echo $copy['name'] ?>
                    </td>
                </tr>
            </table>
        </div>
        <div style="clear:both"></div>
    </div>

    <div class="footerWrapper">
        <div class="footerContent">
            <?php
            $footer_disclaimer = income_config_option('invoice_footer_disclaimer');
            $footer_disclaimer = $invoice_template->getIncludeFooterText();
            echo $footer_disclaimer;
            ?>
        </div>
    </div>
</div>

<?php if (!isset($dont_print) && !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>