<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$terms_text = "";
$payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
if ($payment_period_cp instanceof CustomProperty) {
    $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
    if ($payment_period_cp_val instanceof CustomPropertyValue) {
        $terms_text = $payment_period_cp_val->getValue();
    }
}

$project_and_po_number_text = "";
if ($project_member instanceof Member) {
    $po_number_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_id");
    $po_number_cp_val = "";
    if ($po_number_cp instanceof CustomProperty) {
        $tmp_project_obj_id = $project_member->getObjectId();

        $tmp_mem = $project_member;
        while ($po_number_cp_val == "" && $tmp_project_obj_id > 0) {
            $cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $po_number_cp->getId());
            if ($cpval instanceof CustomPropertyValue) {
                $po_number_cp_val = $cpval->getValue();
            }
            if ($po_number_cp_val == "") {
                $parent_member = $tmp_mem->getParentMember();
                if ($parent_member instanceof Member) $tmp_project_obj_id = $parent_member->getObjectId();
                else $tmp_project_obj_id = 0;

                $tmp_mem = $parent_member;
            }
        }
    }
    /*
	$project_and_po_number_text = $project_member->getName();
	if ($po_number_cp_val != "") {
		$project_and_po_number_text .= " # $po_number_cp_val";
	}
	*/
}

$expiration_date = format_date($invoice->getExpirationDate()->advance(logged_user()->getUserTimezoneValue() * -1, false));

?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 700px;
        margin: 10px auto;
        padding: 10px 0;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        border: 1px solid #555;
        padding: 3px;
        font-size: 14px;
    }

    table th {
        font-size: 12px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
        border: 1px solid #555;
    }

    #items th {
        background: #eee;
        -webkit-print-color-adjust: exact !important;
    }

    #items tr.item-row td {
        border: 0;
        vertical-align: top;
        border-right: 1px solid #555;
    }

    #items td.description {
        /* width: 300px;*/
    }

    #items td.total-line {
        border-right: 0;
        text-align: center;
        font-weight: bold
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 15px;
        background: none;
        text-align: right;
        font-weight: bold;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #items td.due {
        font-size: 17px;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    /*div.bill-to {
    	margin: 20px 5px;
    }
    div.bill-to div {
    	margin: 4px 0px;
    }
    */
    div.owner-company-info {
        margin: 0 5px 5px 5px;
    }

    div.project-table {
        float: right;
        width: 300px;
        margin-bottom: 5px;
    }

    div.project-table table {
        width: 100%;
        text-align: center;
    }

    div.project-table .red {
        color: red;
    }

    div.bill-to {
        float: left;
        width: 300px;
        margin-bottom: 5px;
    }

    div.bill-to table {
        width: 100%;
    }

    table.company-contact {
        width: 100%;
        text-align: center;
    }

    div.footer-info {
        margin-top: 5px;
        font: "Times New Roman";
        text-align: center;
    }
</style>



<div id="page-wrap">
    <div id="customer">
        <div style="float: left; width: 299px;">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>

            <div class="owner-company-info">
                <div><?php echo $to_pay_information['company_name']; ?></div>
                <div><?php echo $to_pay_information['address']; ?></div>
                <div><?php echo $to_pay_information['address_location']; ?></div>
                <div><?php echo lang('phone') . ": " . $to_pay_information['phone']; ?></div>
            </div>

        </div>
        <div style="float: right; margin-top:25px;">
            <div style="text-align:center; font-weight:bold; font-size:150%; margin-bottom:20px;">
                <?php echo strtoupper($invoice->getInvoiceType()) ?>
            </div>
            <table>
                <tr>
                    <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('date')) ?></td>
                    <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('invoice')) . " #" ?></td>
                </tr>
                <tr>
                    <td style="text-align:center; font-size:12px; "><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                    <td style="text-align:center; font-size:12px; "><?php echo trim($invoice->getSeries() . " " . $invoice->getInvoiceNumber()) ?></td>
                </tr>
            </table>
        </div>
    </div>

    <div class="bill-to">
        <table>
            <tr>
                <th><?php echo strtoupper(lang('bill to')) ?></th>
            </tr>
            <tr>
                <td style="height: 60px;">
                    <div><?php echo $bill_to_information['company_name']; ?></div>
                    <div><?php echo $bill_to_information['ordered_by_name']; ?></div>
                    <div><?php echo $bill_to_information['address']; ?></div>
                    <div><?php
                            $city_state_str = $bill_to_information['city'];
                            $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($bill_to_information['state'] . " " . $bill_to_information['zip_code']);
                            echo $city_state_str;
                            ?></div>
                </td>
            </tr>
        </table>
    </div>

    <div class="project-table">
        <table>
            <tr>
                <th colspan="3">Project Name</th>
            </tr>
            <tr>
                <td colspan="3"><?php echo $project_member->getName() ?></td>
            </tr>
            <tr>
                <th>Terms</th>
                <th>Project Number</th>
                <th class="red">Due Date</th>
            </tr>
            <tr>
                <td><?php echo $terms_text ?></td>
                <td><?php echo $po_number_cp_val ?></td>
                <td class="red"><?php echo $expiration_date ?></td>
            </tr>
        </table>
    </div>

    <div style="clear:both"></div>

    <table id="items">

        <tr>
            <th style="width:335px;"><?php echo strtoupper(lang('description')) ?></th>
            <th style="width:70px;"><?php echo strtoupper('quantity') ?></th>
            <th style="width:90px;"><?php echo strtoupper('rate') ?></th>
            <th style="width:90px;"><?php echo strtoupper(lang('amount')) ?></th>
        </tr>
        <?php
        $lines_left = 20;
        foreach ($invoice->getInvoiceLines() as $invoice_line) :
            $line_currency = Currencies::getCurrency($invoice_line->getCurrencyId());
            $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
            $lines_left--;
        ?>
            <tr class="item-row">
                <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
            </tr>
        <?php endforeach; ?>
        <?php while ($lines_left > 0) :
            $lines_left--;
        ?>
            <tr class="item-row">
                <td>&nbsp;</td>
                <td class="description">&nbsp;</td>
                <td></td>
                <td></td>
            </tr>
        <?php endwhile; ?>
        <tr>
            <td colspan="2"></td>
            <td class="total-line">Total</td>
            <td class="total-value">
                <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
            </td>
        </tr>
        <tr>
            <td colspan="2"></td>
            <td class="total-line">Payments/Credits</td>
            <td class="total-value">
                <div id="total"><?php echo $currency_symbol . ' ' . "0" ?></div>
            </td>
        </tr>
        <tr>
            <td colspan="2" style="padding:0;">
                <table class="company-contact">
                    <tr>
                        <td style="border-top:0px none;border-left:0px none;">Phone #</td>
                        <td style="border-top:0px none;">Fax #</td>
                        <td style="border-top:0px none;border-right:0px none;">E-mail</td>
                    </tr>
                    <tr>
                        <td style="border-bottom:0px none;border-left:0px none;">973-527-7451</td>
                        <td style="border-bottom:0px none;">973-858-0280</td>
                        <td style="border-bottom:0px none;border-right:0px none;">annmarie.b@equityenvonmental.com</td>
                    </tr>
                </table>
            </td>
            <td class="total-line due">Balance Due</td>
            <td class="total-value due">
                <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
            </td>
        </tr>
    </table>

    <div class="footer-info">*** EQUITY NOW ACCEPTS CREDIT CARD PAYMENTS. FOR MORE INFORMATION PLEASE CALL 973/527.7451 OR VISIT WWW.EQUITYENVRONMENTAL.COM FOR AN AUTHORIZATION FORM. ***</div>


</div>


<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>