/**
 *  InvoiceManager
 *
 */
og.InvoiceManager = function() {

    var actions, markactions;
    this.doNotRemove = true;
    this.needRefresh = false;

	this.actual_status_filter = '';
	this.actual_sent_filter = '';
    this.period_filter  = '';
    this.from_filter = '';
    this.to_filter = '';

	this.is_loading = false;

    og.eventManager.fireEvent('invoice_start_manager');

	this.fields = [
       'object_id', 'name', 'invoice_number', 'date','created_on', 'expiration_date', 'subtotal', 'total', 'currency_id', 'client', 'client_id',
       'updatedById', 'updatedBy', 'dateUpdated', 'dateUpdated_today', 'isRead', 'memPath', 'status', 'sent', 'is_repetitive'
    ];
  	
  	if (og.invoices.additional_list_columns) {
  		for (var i=0; i<og.invoices.additional_list_columns.length; i++) {
  			this.fields.push(og.invoices.additional_list_columns[i].name);
  		}
  	}
  	
  	var cps = og.custom_properties_by_type['invoice'] ? og.custom_properties_by_type['invoice'] : [];
  	var cp_names = [];
  	for (i=0; i<cps.length; i++) {
  		cp_names.push('cp_' + cps[i].id);
  	}
  	this.fields = this.fields.concat(cp_names);

    var dim_names = [];
    for (did in og.dimensions_info) {
   		if (isNaN(did)) continue;
   		dim_names.push('dim_' + did);
   	}
    this.fields = this.fields.concat(dim_names);

    if (!og.InvoiceManager.store) {
        og.InvoiceManager.store = new Ext.data.Store({
            proxy: new og.GooProxy({
                url: og.getUrl('invoice', 'list_all')
            }),
            reader: new Ext.data.JsonReader({
                root: 'invoices',
                totalProperty: 'totalCount',
                id: 'id',
                fields: this.fields
            }),
            remoteSort: true,
            listeners: {
                'load': function() {
                    var d = this.reader.jsonData;
                    if (d.totalCount === 0) {
                        this.fireEvent('messageToShow', lang("no objects invoice", lang('context')));
                    } else if (d.invoices.length == 0) {
                        this.fireEvent('messageToShow', lang("no more objects invoice"));
                    } else {
                        this.fireEvent('messageToShow', "");
                    }
                    var cmp = Ext.getCmp('invoice-manager')
                    if (cmp) {
                    	cmp.getView().focusRow(og.lastSelectedRow.invoices+1);
                    	var sm = cmp.getSelectionModel();
                    	sm.clearSelections();

						og.eventManager.fireEvent('after grid panel load', {man:cmp, data:d});
                    }
                    
                    if (d && d.totals) {
						if (cmp) cmp.updateGridPagingToolbar({totalCount: d.totals.total_rows});
					} else {
						if (cmp) cmp.reloadGridPagingToolbar('invoice','list_all','invoice-manager');
					}
					
					if (d.filters.status) {
						var items = [['', '-- ' + lang('All') + ' --']];
						for (i=0; i<d.filters.status.length; i++) {
							items[items.length] = [d.filters.status[i].id, d.filters.status[i].name];
						}
						var status_filter = Ext.getCmp('ogInvoiceStatusFilterCombo');
						status_filter.reset();
						status_filter.store.removeAll();
						status_filter.store.loadData(items);

						status_filter.setValue(cmp.actual_status_filter);
						status_filter.collapse();
					}

					Ext.getCmp('invoice-manager').is_loading = false;

					og.eventManager.fireEvent('replace all empty breadcrumb', null);
                }
            }
        });
        og.InvoiceManager.store.setDefaultSort('date', 'desc');
    }
    this.store = og.InvoiceManager.store;
    this.store.addListener({
        messageToShow: {
            fn: this.showMessage,
            scope: this
        }
    });

    function renderDragHandle(value, p, r, ix) {
        return '<div class="img-grid-drag" title="' + lang('click to drag') + '" onmousedown="var sm = Ext.getCmp(\'invoice-manager\').getSelectionModel();if (!sm.isSelected('+ix+')) sm.clearSelections();sm.selectRow('+ix+', true);"></div>';
    }

	function renderExpandHandle(value, p, r, ix) {
		if (isNaN(r.data.object_id)) return;

		return '<div id="'+Ext.id()+'-expand-handle-'+ix+'" class="db-ico ico-expand cursor-pointer" title="' + lang('click to expand') + '" onclick="og.invoices.requestInvoiceDetails(this, '+r.data.object_id+');"></div>';
	}

    var readClass = 'read-unread-' + Ext.id();

    function renderIcon(value, p, r) {
    	var html = '<div class="link-ico ico-invoice"></div>';
    	if (r.data.is_repetitive > 0) {
    		var html = '<div class="link-ico ico-recurrent" title="'+ lang('this invoice is repetitive')+'"></div>';
    	}
        return html;
    }

    function renderIsRead(value, p, r){
		if (isNaN(r.data.object_id)) return;

		var idr = Ext.id();
		var idu = Ext.id();
		var jsr = 'og.InvoiceManager.store.getById(\'' + r.id + '\').data.isRead = true; Ext.select(\'.' + readClass + r.id + '\').removeClass(\'bold\'); Ext.get(\'' + idu + '\').setDisplayed(true); Ext.get(\'' + idr + '\').setDisplayed(false); og.openLink(og.getUrl(\'object\', \'mark_as_read\', {ids:' + r.data.object_id + '}));';
		var jsu = 'og.InvoiceManager.store.getById(\'' + r.id + '\').data.isRead = false; Ext.select(\'.' + readClass + r.id + '\').addClass(\'bold\'); Ext.get(\'' + idr + '\').setDisplayed(true); Ext.get(\'' + idu + '\').setDisplayed(false); og.openLink(og.getUrl(\'object\', \'mark_as_unread\', {ids:' + r.data.object_id + '}));';
		return String.format(
			'<div id="{0}" title="{1}" class="db-ico ico-read" style="display:{2}" onclick="{3}"></div>' +
			'<div id="{4}" title="{5}" class="db-ico ico-unread" style="display:{6}" onclick="{7}"></div>',
			idu, lang('mark as unread'), value ? 'block' : 'none', jsu, idr, lang('mark as read'), value ? 'none' : 'block', jsr
		);
    }

    function renderTitle(value, p, r){
		if (isNaN(r.data.object_id)) {
			return '<span class="bold" id="'+r.data.object_id+'">'+ (value ? og.clean(value) : '') +'</span>';
		}
        var classes = readClass + r.id;
        if (!r.data.isRead) classes += " bold";


        name = String.format(
            '<a style="font-size:120%;" classes="{3}" href="{1}" onclick="og.openLink(\'{1}\');return false;" title="{2}">{0}</a>',
            og.clean(value), og.getUrl('invoice', 'view', {id: r.data.object_id}), og.clean(r.data.name), classes);

        return name;
    }

    function renderInvoiceNumber(value, p, r){
		if (isNaN(r.data.object_id)) return;

        var classes = readClass + r.id;
        if (!r.data.isRead) classes += " bold";

        name = String.format(
            '<a style="font-size:120%;" classes="{3}" href="{1}" onclick="og.openLink(\'{1}\');return false;" title="{2}">{0}</a>',
            og.clean(value), og.getUrl('invoice', 'view', { id: r.data.object_id }), og.clean(r.data.name), classes);

        return name;
    }

    function renderClient(value, p, r){
        var classes = readClass + r.id;
        if (!r.data.isRead) classes += " bold";

        name = String.format(
            '<a style="font-size:120%;" classes="{3}" href="{1}" onclick="og.openLink(\'{1}\');return false;" title="{2}">{0}</a>',
            og.clean(value), og.getUrl('invoice', 'view', { id: r.data.object_id }), og.clean(r.data.name), classes);

        return name;
    }

    function renderDate(value, p, r){
		if (isNaN(r.data.object_id)) return;

        var classes = readClass + r.id;
        if (!r.data.isRead) classes += " bold";

        name = String.format(
            '<a style="font-size:120%;" classes="{3}" href="{1}" onclick="og.openLink(\'{1}\');return false;" title="{2}">{0}</a>',
            og.clean(value), og.getUrl('invoice', 'view', { id: r.data.object_id }), og.clean(r.data.name), classes);

        return name;
    }

    function renderAmount(value, p, r){
    	var name = "";
		if (isNaN(r.data.object_id)) {
			name = value;
		} else {
	        var classes = readClass + r.id;
	        if (!r.data.isRead) classes += " bold";

	        name = String.format(
	            '<a style="font-size:120%;" classes="{3}" href="{1}" onclick="og.openLink(\'{1}\');return false;" title="{2}">{0}</a>',
	            og.clean(value), og.getUrl('invoice', 'view', { id: r.data.object_id }), og.clean(r.data.name), classes);
		}

        return name;
    }

    function renderStatus(value, p, r){
    	if (r.data.status) {
			return '<span class="invoice-row-status-'+value+'">' + lang(value) + '</span>';
		}else{
			return '';
		}
    }

    function renderSent(value, p, r){
    	if (typeof(r.data.sent) != 'undefined') {
    		if (r.data.sent != '' && r.data.sent != '0') return lang(r.data.sent);
    		else return lang('no');
		}else{
			return '';
		}
    }

    function renderUpdated(value, p, r) {
        if (!value) {
            return "";
        }
        var userString = String.format('<a href="{1}" onclick="og.openLink(\'{1}\');return false;">{0}</a>', og.clean(r.data.updatedBy), og.getUrl('contact', 'card', { id: r.data.updatedById }));

        if (!r.data.dateUpdated_today) {
            return lang('last updated by on', userString, value);
        } else {
            return userString +", "+ value;
        }
    }

    function getSelectedIds() {
        var selections = sm.getSelections();
        if (selections.length <= 0) {
            return '';
        } else {
            var ret = '';
            for (var i=0; i < selections.length; i++) {
                ret += "," + selections[i].data.object_id;
            }
            og.lastSelectedRow.invoices = selections[selections.length-1].data.ix;
            return ret.substring(1);
        }
    }
    this.getSelectedIds = getSelectedIds;

    function getSelectedTypes() {
        var selections = sm.getSelections();
        if (selections.length <= 0) {
            return '';
        } else {
            var ret = '';
            for (var i=0; i < selections.length; i++) {
                ret += "," + selections[i].data.type;
            }
            return ret.substring(1);
        }
    }
    this.getSelectedTypes = getSelectedTypes;

    function getFirstSelectedId() {
        if (sm.hasSelection()) {
            return sm.getSelected().data.object_id;
        }
        return '';
    }

    var sm = new Ext.grid.CheckboxSelectionModel();
    sm.on('selectionchange',
        function() {
            var allUnread = true, allRead = true;
            var selections = sm.getSelections()
            for (var i=0; i < selections.length; i++) {
                if (selections[i].data.isRead){
                    allUnread = false;
                } else {
                    allRead = false;
                }
            }
            if (sm.getCount() <= 0) {
                actions.del.setDisabled(true);
                actions.edit.setDisabled(true);
                markactions.markAsRead.setDisabled(true);
                markactions.markAsUnread.setDisabled(true);
                markactions.markAsVoid.setDisabled(true);
                markactions.markAsPaid.setDisabled(true);

                og.eventManager.fireEvent('invoice_selectionchange_count_off', actions);
            } else {
                actions.del.setDisabled(false);
                actions.edit.setDisabled(false);
                
                og.eventManager.fireEvent('invoice_selectionchange_count_on', actions);

                if (allUnread) {
                    markactions.markAsUnread.setDisabled(true);
                } else {
                    markactions.markAsUnread.setDisabled(false);
                }
                if (allRead) {
                    markactions.markAsRead.setDisabled(true);
                } else {
                    markactions.markAsRead.setDisabled(false);
                }
                actions.archive.setDisabled(false);
                markactions.markAsVoid.setDisabled(false);
                markactions.markAsPaid.setDisabled(false);
            }
        }
    );


    var cm_info = [
        sm, {
            /*id: 'draghandle',
            header: '&nbsp;',
            width: 18,
            renderer: renderDragHandle,
            fixed:true,
            resizable: false,
            hideable:false,
            menuDisabled: true
        },{*/
			id: 'expand',
			header: '&nbsp;',
			dataIndex: 'expand',
			width: 30,
        	renderer: renderExpandHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
            id: 'icon',
            header: '&nbsp;',
            dataIndex: 'type',
            width: 28,
            renderer: renderIcon,
            fixed:true,
            resizable: false,
            hideable:false,
            menuDisabled: true
        },{
            id: 'isRead',
            header: '&nbsp;',
            dataIndex: 'isRead',
            width: 16,
            renderer: renderIsRead,
            fixed:true,
            resizable: false,
            hideable:false,
            menuDisabled: true
        },{
            id: 'name',
            header: lang("title"),
            dataIndex: 'name',
            width: 300,
            renderer: renderTitle,
            sortable:true
        },{
            id: 'invoice_number',
            header: lang("invoice number"),
            dataIndex: 'invoice_number',
            width: 80,
            renderer: renderInvoiceNumber,
            sortable:true
        },{
        	id: 'status',
        	header: lang("status"),
        	dataIndex: 'status',
        	width: 80,
        	renderer: renderStatus,
        	sortable:true
        },{
        	id: 'sent',
        	header: lang("sent"),
        	dataIndex: 'sent',
        	width: 80,
        	renderer: renderSent,
        	sortable:true
        },{
            id: 'date',
            header: lang("date"),
            dataIndex: 'date',
            width: 80,
            renderer: renderDate,
            sortable:true
        },{
            id: 'created_on',
            header: lang("created_on"),
            dataIndex: 'created_on',
            width: 80,
            renderer: renderDate,
            sortable:true
        },{
            id: 'expiration_date',
            header: lang("expiration date"),
            dataIndex: 'expiration_date',
            width: 80,
            renderer: renderDate,
            sortable:true
        },{
            id: 'subtotal',
            header: lang("subtotal"),
            dataIndex: 'subtotal',
            width: 80,
            align: 'right',
            renderer: renderAmount,
            sortable:true
        },{
            id: 'total',
            header: lang("total"),
            dataIndex: 'total',
            width: 80,
            align: 'right',
            renderer: renderAmount,
            sortable:true
        },{
            id: 'dateUpdated',
            header: lang("last updated by"),
            dataIndex: 'dateUpdated',
            width: 110,
            align: 'right',
            sortable: true,
            renderer: renderUpdated
        }
    ];
    
    // additional fields
    if (og.invoices.additional_list_columns) {
    	for (var i=0; i<og.invoices.additional_list_columns.length; i++) {
    		var add_col = og.invoices.additional_list_columns[i];
    		var fname = add_col.name;
    		cm_info.push({
				id: fname,
				header: add_col.text ? add_col.text : lang(fname),
				dataIndex: fname,
	            align: add_col.align ? add_col.align : 'left',
                sortable: add_col.sortable ? add_col.sortable : false,
				renderer: og.invoices.additional_list_columns[i].renderer
			});
    	}
    }

    // custom property columns
	var cps = og.custom_properties_by_type['invoice'] ? og.custom_properties_by_type['invoice'] : [];
	this.addCustomPropertyColumns(cps, cm_info, 'invoice-manager');

	// dimension columns
	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		var key = 'lp_dim_' + did + '_show_as_column';
		if (og.preferences['listing_preferences'][key]) {
			cm_info.push({
				id: 'dim_' + did,
				header: og.dimensions_info[did].name,
				dataIndex: 'dim_' + did,
				sortable: true,
				renderer: og.renderDimCol
			});
			og.breadcrumbs_skipped_dimensions[did] = did;
		}
	}
	// create column model
	var cm = new Ext.grid.ColumnModel(cm_info);
    cm.defaultSortable = false;
    cm.on('hiddenchange', this.afterColumnShowHide, this);

    markactions = {
        markAsRead: new Ext.Action({
            text: lang('mark as read'),
            tooltip: lang('mark as read desc'),
            iconCls: 'ico-mark-as-read',
            disabled: true,
            handler: function() {
                this.load({
                    action: 'markasread',
                    ids: getSelectedIds(),
                    types: getSelectedTypes()
                });
                this.getSelectionModel().clearSelections();
            },
            scope: this
        }),

        markAsUnread: new Ext.Action({
            text: lang('mark as unread'),
            tooltip: lang('mark as unread desc'),
            iconCls: 'ico-mark-as-unread',
            disabled: true,
            handler: function() {
                this.load({
                    action: 'markasunread',
                    ids: getSelectedIds(),
                    types: getSelectedTypes()
                });
                this.getSelectionModel().clearSelections();
            },
            scope: this
        }),

        markAsVoid: new Ext.Action({
            text: lang('mark as void'),
            iconCls: 'ico-delete',
            disabled: true,
            handler: function() {
            	if (!og.loggedUser.isGuest) {
                    og.openLink(og.getUrl('invoice', 'check_invoices_integration_status', {ids: getSelectedIds()}), {
                        hideLoading: true,
                        callback: function(success, data) {
                            if(data.result){
                                this.load({
                                    action: 'markasvoid',
                                    ids: getSelectedIds(),
                                    types: getSelectedTypes()
                                });
                                this.getSelectionModel().clearSelections();
                            }
                        }
                    });
            	}
            },
            scope: this
        }),

        markAsPaid: new Ext.Action({
            text: "Mark as paid",
            iconCls: 'ico-delete',
            disabled: true,
            handler: function() {
            	if (!og.loggedUser.isGuest) {
                    og.openLink(og.getUrl('invoice', 'mark_as_paid', {ids: getSelectedIds()}), {
                        callback: function(success, data) {
                        }
                    });
            	}
            },
            scope: this
        }) 
    };

    var menu_items = [
        markactions.markAsRead,
        markactions.markAsUnread,
        markactions.markAsVoid
        // markactions.markAsPaid
    ];

    if(typeof og.invoice_payment_receipts === 'undefined'){
        menu_items.push(markactions.markAsPaid);
    }

    actions = {
        newCO: new Ext.Action({
			id: 'new_button_invoice',
            text: lang('new'),
            tooltip: lang('add new invoice'),
            iconCls: 'ico-new',
            hidden: og.replace_list_new_action && og.replace_list_new_action.invoice,
            handler: function() {
            	og.render_modal_form('', {c:'invoice', a:'add'});
            }
        }),
        del: new Ext.Action({
            text: lang('move to trash'),
            tooltip: lang('move selected objects to trash'),
            iconCls: 'ico-trash',
            disabled: true,
            handler: function() {
                og.openLink(og.getUrl('invoice', 'check_invoices_integration_status', {ids: getSelectedIds()}), {
                    hideLoading: true,
                    callback: function(success, data) {
                        if(data.result){
                            var confirm_trash_config = parseInt(og.preferences['enableTrashConfirmation']);
							var trash_msg = lang('confirm move to trash');
							if (data && data.additional_data && data.additional_data.additional_trash_msg) {
								trash_msg += "\n" + data.additional_data.additional_trash_msg;
							}
                            if (og.confirmNorification(trash_msg, confirm_trash_config)) {
                                var cmp = Ext.getCmp('invoice-manager');
                                cmp.load({
                                    action: 'delete',
                                    ids: getSelectedIds(),
                                    types: getSelectedTypes()
                                });
                                cmp.getSelectionModel().clearSelections();
                                og.eventManager.fireEvent('reload current panel');
                            }
                        }
                    }
                });
            },
            scope: this
        }),
        edit: new Ext.Action({
            text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
            disabled: true,
            handler: function() {
                og.render_modal_form('', {c:'invoice', a:'edit', params: {id:getFirstSelectedId()}});
            },
            scope: this
        }),
        markAs: new Ext.Action({
            text: lang('mark as'),
            tooltip: lang('mark as desc'),
            menu: menu_items
        }),
        archive: new Ext.Action({
            text: lang('archive'),
            tooltip: lang('archive selected object'),
            iconCls: 'ico-archive-obj',
            disabled: true,
            handler: function() {
                var confirm_archive_config = parseInt(og.preferences['enableArchiveConfirmation']);

                if (og.confirmNorification(lang('confirm archive selected objects'), confirm_archive_config)) {
                    this.load({
                        action: 'archive',
                        ids: getSelectedIds(),
                        types: getSelectedTypes()
                    });
                    this.getSelectionModel().clearSelections();
                }
            },
            scope: this
        }),
        generateProjectInvoice: new Ext.Action({
        	id: 'generate_project_invoice_btn',
            text: lang('generate project invoice'),
            tooltip: '',
            iconCls: 'ico-invoice',
            //disabled: true,
            handler: function() {
            	
            	var project_member_id = og.income.get_context_selected_project_id();
            	if (project_member_id) {
            		og.income.generate_project_invoice_modal_form(project_member_id);
            	} else {
            		og.err(lang('you must select a project or client to generate the invoice'));
            	}
            	
            },
            scope: this
        }),
        generateBatchInvoices: new Ext.Action({
        	id: 'generate_batch_invoices_btn',
            text: lang('batch invoicing'),
            tooltip: '',
            iconCls: 'ico-invoice',
            //disabled: true,
            handler: function() {
            	og.income.batch_project_invoices_generation_tab();
            },
            scope: this
        })
    };
    
    actions.CONTEXT = this;
    og.eventManager.fireEvent('invoice_append_actions', actions);

	var filters_genid = Ext.id();

	var period_options = [
		[0, lang('no filter')],
		[1, lang('today')],
		[11, lang('yesterday+')],
		[2, lang('this week')],
		[3, lang('last week+')],
		[7, lang('First half month+')],
		[8, lang('Second half month+')],
		[9, lang('First half of last month+')],
		[10, lang('Second half of last month+')],
		[4, lang('this month')],
		[5, lang('last month+')],
		[6, lang('select dates...')]
	];
	var period_ini_val = 0;

	if (period_ini_val == 6) {
		var hidden_from = false;
		var hidden_to = false;
	} else {
		var hidden_from = true;
		var hidden_to = true;
	}

	var period_filter_config = {
		genid: filters_genid,
		type: "period",
		label: {
			id: filters_genid + "label_period_filter",
			text: "&nbsp;" + lang("Time period") + ": ",
		},
		options: period_options,
		value: period_ini_val,
		initial_val: period_ini_val,
		secondToolbar: true,
	};

	var from_filter_config = {
		genid: filters_genid,
		type: "date",
		label: {
			id: filters_genid + "label_from_filter",
			hidden: hidden_from,
			text: "&nbsp;" + lang("from date") + ": ",
		},
		value: "",
		hidden: hidden_from,
		secondToolbar: true,
	};

	var to_filter_config = {
		genid: filters_genid,
		type: "date",
		label: {
			id: filters_genid + "label_to_filter",
			hidden: hidden_to,
			text: "&nbsp;" + lang("to date") + ": ",
		},
		value: "",
		hidden: hidden_to,
		secondToolbar: true,
	};

	filters = {
		
		period_filter: og.buildToolbarFilterAction('period_filter', period_filter_config, 'invoice-manager'),        

		from_filter: og.buildToolbarFilterAction('from_filter', from_filter_config, 'invoice-manager'),

		to_filter: og.buildToolbarFilterAction('to_filter', to_filter_config, 'invoice-manager'),

		status_filter: new Ext.form.ComboBox({
			id: "ogInvoiceStatusFilterCombo",
			store: new Ext.data.SimpleStore({
				fields: ["value", "text"],
				data: [],
			}),
			displayField: "text",
			mode: "local",
			triggerAction: "all",
			selectOnFocus: true,
			width: 100,
			valueField: "value",
			valueNotFoundText: "",
			listeners: {
				select: function (combo, record) {
					var man = Ext.getCmp("invoice-manager");
					man.actual_status_filter = combo.getValue();
					man.reloadFiltering();            
				},
			},
		}),

		sent_filter: new Ext.form.ComboBox({
			id: "ogInvoiceSentFilterCombo",
			store: new Ext.data.SimpleStore({
				fields: ["value", "text"],
				data: [
					["", "-- " + lang("All") + " --"],
					["no", lang("not sent")],
					["invoice_sent", lang("invoice_sent")],
					["receipt_sent", lang("receipt_sent")],
					["void_sent", lang("void_sent")],
				],
			}),
			value: "",
			displayField: "text",
			mode: "local",
			triggerAction: "all",
			selectOnFocus: true,
			width: 80,
			valueField: "value",
			valueNotFoundText: "",
			listeners: {
				select: function (combo, record) {
					var man = Ext.getCmp("invoice-manager");
					man.actual_sent_filter = combo.getValue();
					man.reloadFiltering();
				},
			},
		}),

    };

    var invoice_ot = 0;
    for (x in og.objectTypes) {
    	if (og.objectTypes[x].name == 'invoice') {
    		invoice_ot = x;
    		break;
    	}
    }
    var can_add = !og.loggedUser.isGuest && og.maxRoleObjectTypePermissions[og.loggedUser.type][invoice_ot].can_write;

    var tbar = [];
	this.tbar2 = [];

    if (can_add) {
    	if (og.replace_list_new_action && og.replace_list_new_action.invoice) {
			tbar.push(og.replace_list_new_action.invoice);
		}

    	tbar.push(actions.newCO);
        tbar.push('-');
        tbar.push(actions.edit);
        tbar.push(actions.archive);
        tbar.push(actions.del);
        tbar.push('-');
        tbar.push(actions.markAs);
        tbar.push('-');
	    tbar.push(actions.generateProjectInvoice);
	    tbar.push(actions.generateBatchInvoices);

        og.eventManager.fireEvent('invoice_append_tbar', [tbar, actions]);
    }
    tbar.push('-');
	
	this.tbar2.push(lang('status')+'&nbsp;');
	this.tbar2.push(filters.status_filter);

	for (var x=0; x<filters.period_filter.length; x++) {
		this.tbar2.push(filters.period_filter[x]);
	}   

	for (var x=0; x<filters.from_filter.length; x++) {
		this.tbar2.push(filters.from_filter[x]);
	}   
	
	for (var x=0; x<filters.to_filter.length; x++) {
		this.tbar2.push(filters.to_filter[x]);
	}
    
    this.tbar2.push(' ');
	this.tbar2.push(filters.sent_filter);

	if (og.additional_list_actions && og.additional_list_actions.invoice) {
		tbar.push(' ');
		for (var i=0; i<og.additional_list_actions.invoice.length; i++) {
			if (og.additional_list_actions.invoice[i] && og.additional_list_actions.invoice[i].id == 'invoice-manager_text_filter_input') {
				this.tbar2.push(og.additional_list_actions.invoice[i]);
			} else {
				tbar.push(og.additional_list_actions.invoice[i]);
			}
			
		}
	}


  

	this.toptbar = new Ext.Toolbar({
		id: 'invoices-list-toptbar',
		items: tbar,
	});

	var render_to = null;
	if (og.income.additional_panels && og.income.additional_panels.length > 0) {
		render_to = 'inv-tab-invoices';
	}

    og.InvoiceManager.superclass.constructor.call(this, {
        renderTo: render_to,
    	store: this.store,
        layout: 'fit',
        cm: cm,
        enableDrag: true,
        ddGroup: 'MemberDD',
        stateful: og.preferences['rememberGUIState'],
        id: 'invoice-manager',
        stripeRows: true,
        closable: true,
        loadMask: true,
        bbar: new og.CurrentPagingToolbar({
            pageSize: og.config['files_per_page'],
            store: og.InvoiceManager.store,
            displayInfo: true,
            displayMsg: lang('displaying objects of'),
            emptyMsg: lang("no objects to display")
        }),
        viewConfig: {
            forceFit: false
        },
        sm: sm,
        tbar: this.toptbar,
        listeners: {
            'render': {
                fn: function() {
                    this.innerMessage = document.createElement('div');
                    this.innerMessage.className = 'inner-message';
                    var msg = this.innerMessage;
                    var elem = Ext.get(this.getEl());
                    var scroller = elem.select('.x-grid3-scroller');
                    scroller.each(function() {
                        this.dom.appendChild(msg);
                    });
					
					// add the second toolbar
					this.toptbar2 = new Ext.Toolbar({
						renderTo: 'invoices-list-toptbar',
						items: this.tbar2,
					});
                },
                scope: this
            },
			'resize' : {
				fn: function() {
					setTimeout(function(){
						// fit columns in the view
						var v = Ext.getCmp("invoice-manager").getView();
						if (v && v.forceFit) {
							v.fitColumns();
						}

						// calculate the hieght
						var h = $("#invoice-manager").parent().height();
						var tb_h = $("#invoice-manager .x-panel-tbar.x-panel-tbar-noheader").height();
						var tabs_h = $("#invoices-tabs-panel .x-tab-panel-header").height();
						var new_h = h - tb_h - tabs_h + 6;
						
						// adjust containers width
						$("#invoice-manager .x-grid3").css('height', new_h+'px');
						$("#invoice-manager .x-grid3 .x-grid3-scroller").css('height', (new_h - 26)+'px');
					}, 200);
				}
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
        }
    });
};

Ext.extend(og.InvoiceManager, Ext.grid.GridPanel, {
    load: function(params) {
        if (!params) params = {};
        var start;
        if (typeof params.start == 'undefined') {
            start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
        } else {
            start = 0;
        }

        this.store.baseParams.context = og.contextManager.plainContext();
        if (typeof(this.actual_status_filter) != "undefined") {
        	this.store.baseParams.status_filter = this.actual_status_filter;
        }
        if (typeof(this.actual_sent_filter) != "undefined") {
        	this.store.baseParams.sent_filter = this.actual_sent_filter;
        }
        if (typeof(this.period_filter ) != "undefined") {
        	this.store.baseParams.period_filter = this.period_filter ;
        }

        if (typeof(this.from_filter) != "undefined") {
        	this.store.baseParams.from_filter = this.from_filter;
        }

        if (typeof(this.to_filter) != "undefined") {
        	this.store.baseParams.to_filter = this.to_filter;
        }

        this.store.removeAll();

		this.updateColumnModelHiddenColumns();

		this.store.load({
            params: Ext.apply(params, {
                start: start,
                limit: og.config['files_per_page']
            })
        });
    },
    resetVars: function(){
    },

    activate: function() {
        if (this.needRefresh) {
            this.load({
                start:0
            });
        }
    },

    reset: function() {
        this.load({
            start:0
        });
    },

    showMessage: function(text) {
        this.innerMessage.innerHTML = text;
    },

    reloadFiltering: function() {
		if (this.isLoading()) return;
		this.is_loading = true;

		var config = {
			start:0,
			status_filter: this.actual_status_filter,
			sent_filter: this.actual_sent_filter,
			period_filter: this.period_filter,
			from_filter: this.from_filter,
			to_filter: this.to_filter,
		};

		this.load(config);
	},

	isLoading: function() {
		return this.is_loading;
	},

    trashObjects: function() {
        if (confirm(lang('confirm move to trash'))) {
            this.load({
                action: 'delete',
                ids: this.getSelectedIds(),
                types: this.getSelectedTypes()
            });
            this.getSelectionModel().clearSelections();
        }
    },

    archiveObjects: function() {
        if (confirm(lang('confirm archive selected objects'))) {
            this.load({
                action: 'archive',
                ids: this.getSelectedIds(),
                types: this.getSelectedTypes()
            });
            this.getSelectionModel().clearSelections();
        }
    }
});


Ext.reg("invoices", og.InvoiceManager);

/************************************************
Container for InvoiceManager,
*************************************************/
og.InvoiceManagerPanel = function() {
    this.doNotRemove = true;
    this.needRefresh = false;
	
    this.panels = [];
    
    if (og.income.additional_panels && og.income.additional_panels.length > 0) {

    	var invoices_subtabs = [];
    	
    	if (!og.income.disabled_subtabs) og.income.disabled_subtabs = [];
    	
    	if (og.income.disabled_subtabs.indexOf('subtab-invoices') < 0) {
    		invoices_subtabs.push({title:lang("invoices"), id: 'subtab-invoices', html:'<div id="inv-tab-invoices" style="height:100%;"></div>'});
    	}
    	
    	for (var i=0; i<og.income.additional_panels.length; i++) {
    		var p = og.income.additional_panels[i];
    		invoices_subtabs.push({title: p.title, id: p.id, html:'<div id="inv-tab-'+p.id+'" style="height:100%;"></div>'});
		}
    	
    	og.InvoiceManagerPanel.superclass.constructor.call(this, {
    		id: 'invoices-manager-container',
    		layout: 'fit',
    		border: false,
    		bodyBorder: false,
    		items: [
    	        new Ext.TabPanel({
    				id: 'invoices-tabs-panel',
    				cls: 'subtab',
    				activeTab: 0,
    				enableTabScroll: false,
    				items: invoices_subtabs,
    				listeners: {
    					resize: function(tabp, adjWidth, adjHeight, rawWidth, rawHeight) {
    						
    						if (this.ownerCt.invoice_manager) {
    							this.ownerCt.invoice_manager.fireEvent('resize');
    						}
    						
    						if (this.ownerCt.panels) {
    							for (var i=0; i<this.ownerCt.panels.length; i++) {
    								this.ownerCt.panels[i].fireEvent("resize");
    							}
    						}
    					},
    					tabchange: function(tabp, t) {
    						if (!t.id) return;
    						
    						if (t.id == 'subtab-invoices') {
									
								if (!this.ownerCt.invoice_manager) {
									this.ownerCt.invoice_manager = new og.InvoiceManager();
									this.ownerCt.invoice_manager.load();
								}
								this.ownerCt.invoice_manager.fireEvent('resize');
							
							} else {
								
								for (var i=0; i<og.income.additional_panels.length; i++) {
									
									var p = og.income.additional_panels[i];                                    
						    		if (p.id == t.id) {
						    			if (!p.grid_id) p.grid_id = p.id + '-grid';
						    			
						    			var grid_panel = Ext.getCmp(p.grid_id);
						    			if (!grid_panel) {
						    				
						    				grid_panel = new og.ObjectGrid({
						    					no_icon_col: true,
						    					renderTo: 'inv-tab-'+p.id,
						    					url: og.getUrl(p.controller, p.action),
						    					type_name: p.type_name,
						    					response_objects_root: p.response_objects_root,
						    					grid_id: p.grid_id,
						    					nameRenderer: og.gridObjectNameRenderer,
						    					fixed_height: true,
						    					forceFit: false,
						    					store_params: {
						    						url_controller: p.controller,
						    						url_action: p.action
						    					},
						    					checkbox_sel_model: true,
						    					allow_drag_drop: true,
						    					columns: p.columns,
						    					tbar_items: p.tbar_items,
						    					filters: p.filters,
						    					add_default_actions_column: false,
						    				});
						    				grid_panel.load();
						    				this.ownerCt.panels.push(grid_panel);
						    			}
					    	        	
					    	        	grid_panel.fireEvent('resize');
					    	        	break;
					    	        }
								}
							}
    					}
    				},
    				scope: this
    			})
    		],
    		scope: this
        });
    	
    } else {
    	
    	this.invoice_manager = new og.InvoiceManager();
    	
    	og.InvoiceManagerPanel.superclass.constructor.call(this, {
            layout: 'fit',
            border: false,
            bodyBorder: false,
            items: [
            	this.invoice_manager
            ],
            closable: true
        });
    	
    }

}

Ext.extend(og.InvoiceManagerPanel, Ext.Panel, {
    load: function(params) {
    	if (this.invoice_manager) this.invoice_manager.load(params);
    	
        if (this.panels) {
        	for (var i=0; i<this.panels.length; i++) {
        		this.panels[i].load(params);
        	}
        }
    },
    activate: function() {
    	if (this.invoice_manager) this.invoice_manager.activate();

    	if (this.panels) {
        	for (var i=0; i<this.panels.length; i++) {
        		this.panels[i].activate();
        	}
        }
    },
    reset: function() {
    	if (this.invoice_manager) this.invoice_manager.reset();

    	if (this.panels) {
        	for (var i=0; i<this.panels.length; i++) {
        		this.panels[i].reset();
        	}
        }
    },
    showMessage: function(text) {
    	if (this.invoice_manager) this.invoice_manager.showMessage(text);
    }
});

Ext.reg("invoices-containerpanel", og.InvoiceManagerPanel);

