<?php
/**
 * Indicator controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class IndicatorController extends ApplicationController {

	var $plugin_name = "indicators";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("IndicatorManager.js", $this->plugin_name);
		ajx_current("panel", "indicators-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		}
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		$result = Indicators::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$indicators = $result->objects;
		$object = $this->prepareObject($indicators, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($indicators, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"indicators" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Indicators::instance()->getObjectTypeId());
		$count = min(array($limit, count($indicators)));
		for ($i = 0; $i < $count; $i++){
			if (isset($indicators[$i])){
				$obj = $indicators[$i];
				if ($obj instanceof Indicator){
					$info = $obj->getArrayInfo();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["indicators"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["indicators"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$indicator_data = array_var($_POST, 'indicator');
		
		if(!is_array($indicator_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Indicator();
			$obj->setObjectTypeId(Indicators::instance()->getObjectTypeId());
			tpl_assign('indicator', $obj);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$indicator = $this->save_indicator($indicator_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($indicator, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add indicator', $indicator->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$indicator = Indicators::findById(get_id());
		if(!($indicator instanceof Indicator)) {
			flash_error(lang('indicator dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$indicator->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$indicator_data = array_var($_POST, 'indicator');
		
		if(!is_array($indicator_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('indicator', $indicator);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$indicator = $this->save_indicator($indicator_data, $member_ids, $indicator);
				
				DB::commit();

				ApplicationLogs::createLog($indicator, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit indicator', $indicator->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_indicator($indicator_data, $member_ids, $indicator = null) {
		// Save Indicator object
		$is_new = false;
		if (is_null($indicator) || !$indicator instanceof Indicator) {
			$indicator = new Indicator();
			$is_new = true;
		}
		
		$indicator->setFromAttributes($indicator_data);
		
		$indicator->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($indicator, $member_ids);
		}
		$object_controller->add_subscribers($indicator);
		$object_controller->link_to_new_object($indicator);
		$object_controller->add_custom_properties($indicator);
		
		return $indicator;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$indicator = Indicators::findById(get_id());
		if(!($indicator instanceof Indicator)) {
			flash_error(lang('indicator dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$indicator->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('indicator', $indicator);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$indicator = Indicators::findById($id);
					if (isset($indicator) && $indicator->canDelete(logged_user())){
						try{
							DB::beginWork();
							$indicator->trash();
							DB::commit();
							ApplicationLogs::createLog($indicator, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete indicator', $indicator instanceof Indicator ? clean($indicator->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$indicator = Indicators::findById($id);
					if (isset($indicator) && $indicator->canEdit(logged_user())){
						try{
							DB::beginWork();
							$indicator->archive();
							DB::commit();
							ApplicationLogs::createLog($indicator, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}