<?php
	class IndicatorValue extends BaseIndicatorValue {
		
		function getValue() {
			
			$cp = CustomProperties::findOne(array("conditions" => array("object_type_id=? AND object_subtype_id=? AND code LIKE '%_value'",
					$this->getObjectTypeId(), $this->getColumnValue('object_subtype_id')
			)));
			
			if ($cp instanceof CustomProperty) {
				return get_custom_property_value_for_listing($cp, $this);
			}
			
			return "";
		}
		
		function getDate() {
			
			$cp = CustomProperties::findOne(array("conditions" => array("object_type_id=? AND code LIKE 'indicator_value_date'", $this->getObjectTypeId())));
			
			if ($cp instanceof CustomProperty) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
				if ($cp_val instanceof CustomPropertyValue) {
					return DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $cp_val->getValue());
				}
			}
			
			return null;
		}
		
		function getArrayInfo() {
			$info = array();
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			return $info;
		}
		
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, IndicatorValues::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}