<?php
	class Indicator extends BaseIndicator {
		
		
		function getArrayInfo() {
			$info = array();
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			$current_value = $this->getCurrentValue();
			$info['current_value'] = "";
			if ($current_value instanceof IndicatorValue) {
				$info['current_value'] = $current_value->getValue();
			}
			
			$info['date'] = "";
			if ($current_value instanceof IndicatorValue) {
				$d = $current_value->getDate();
				$info['date'] = $d instanceof DateTimeValue ? format_date($d, null, 0) : '';
			}
			
			return $info;
		}
		
		
		function getUnit() {
			
			$cp = CustomProperties::findOne(array("conditions" => array("object_type_id=? AND code ='indicator_unit'", $this->getObjectTypeId())));
				
			if ($cp instanceof CustomProperty) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
				return $cp_val instanceof CustomPropertyValue ? $cp_val->getValue() : '';
			}
				
			return "";
		}
		
		
		function getCurrentValue() {
			
			$last_indicator_value = IndicatorValues::findOne(array(
					'conditions' => 'indicator_id = '.$this->getId(),
					'order' => 'updated_on DESC',
			));
			
			return $last_indicator_value;
		}
		
		function getAllValues() {
			return IndicatorValues::getAllIndicatorValues($this);
		}
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Indicators::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}