<?php
Hook::register('invoice_payment_receipts');


function invoice_payment_receipts_object_definition($type, &$ret) {
	if ($type == 'IncomeInvoice') {
		$ret['paid_amount'] = DATA_TYPE_FLOAT;
		$ret['due_amount'] = DATA_TYPE_FLOAT;
	}
}


function invoice_payment_receipts_more_columns_to_aggregate_in_totals($manager, &$columns) {
	if ($manager instanceof IncomeInvoices) {
		$columns['paid_amount'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id');
		$columns['due_amount'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id');
	}
}

function invoice_payment_receipts_invoices_list_additional_column_values($parameters, &$info) {
	$invoice = array_var($parameters, 'invoice');
	if ($invoice instanceof IncomeInvoice) {
		$info['paid_amount'] = $invoice->getColumnValue('paid_amount');
		$info['due_amount'] = $invoice->getColumnValue('due_amount');
	}
}

function invoice_payment_receipts_view_invoice_additional_sections($parameters, &$info) {
	$invoice = array_var($parameters, 'invoice');
	
	tpl_assign('invoice', $invoice);
	tpl_display(get_template_path('invoice_payments_table', 'invoice_payment_receipt', 'invoice_payment_receipts'));
}


function invoice_payment_receipts_render_page_actions($invoice, &$ret) {
	if ($invoice instanceof IncomeInvoice && !$invoice->isTrashed()){
		// remove mark as paid action
		if($invoice->getTotal() && Plugins::instance()->isActivePlugin('invoice_payment_receipts')){
			PageActions::instance()->removeAction('ico-complete');
		}

		// add action to pay this invoice
		$action_url = "javascript:og.render_modal_form('', {c:'invoice_payment_receipt', a:'add', params: {invoice_id:".$invoice->getId()."}});";
		add_page_action(lang('add payment'), $action_url, 'ico-invoice-payment-receipt', null, null, true);
	}
}


function invoice_payment_receipts_custom_reports_override_column_format($parameters, &$value) {
	$report = array_var($parameters, 'report');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($manager instanceof InvoicePaymentReceipts && in_array($field, array('amount','balance'))) {
		$currency = Currencies::getCurrency($object->getCurrencyId());
		$csymbol = null;
		if ($currency instanceof Currency) {
			$csymbol = $currency->getSymbol();
		}
		$value = format_money_amount($value, $csymbol);
	}
}

function invoice_payment_receipts_after_invoice_save($params, &$ignored) {
	$invoice = array_var($params, 'invoice');
	/* @var $invoice IncomeInvoice */
	
	if ($invoice instanceof IncomeInvoice) {
		DB::execute("update ".TABLE_PREFIX."income_invoices set due_amount = total - paid_amount where object_id=".$invoice->getId());
	
		Env::useHelper('persist_member_invoice_totals', 'income');
		update_invoice_total_info_for_member($invoice);
	}
}

function invoice_payment_receipts_add_allowed_object_types_to_import($parameters, &$allowed_object_types_to_import){
	$allowed_object_types_to_import[] = 'invoice_payment_receipt';
}
