<?php

/**
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */

class KjolhaugReportsController extends ApplicationController {

	/**
	 * Construct the ReportingController
	 *
	 * @access public
	 * @param void
	 * @return ReportingController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('grouping');
		Env::useHelper('reporting');
		Env::useHelper('advanced_reports', 'advanced_reports');
		Env::useHelper('functions', 'advanced_billing');
		Env::useHelper('functions', 'expenses2');
		Env::useHelper('functions', 'project_reports');
		Env::useHelper('invoice_report_excel_functions', 'project_reports');
		Env::useHelper('invoice_report_csv_functions', 'project_reports');
		Env::useHelper('kjolhaug_invoice_report_functions', 'kjolhaug');
		

	} // __construct


	function kjolhaug_invoice_report_p(){
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_data = array_var($_SESSION, 'invoice_report_data');
		if (!$report_data) {
			$report_data = array();
			
			$_SESSION['invoice_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'kjolhaug_invoice_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function kjolhaug_invoice_report($report_data = null, $csv = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		// dates config
		Env::useHelper('advanced_reports', 'advanced_reports');
		$report_dates = parse_report_param_date_type($report_data, false); 
		$start_date = array_var($report_dates, 'st');
		$end_date = array_var($report_dates, 'et');
		$report_data['start_date'] = $start_date;
		$report_data['end_date'] = $end_date;

		$_SESSION['invoice_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        $project_member = null;
        $project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
        if (is_array($project_member_ids) && count($project_member_ids) > 0) {
        	$project_member = Members::getMemberById($project_member_ids[0]);
        }
        if($project_member instanceof Member){
			$data = kjolhaug_get_main_invoice_report_data($project_member, $report_data);
		} else {
			$group_report_by = array_var($report_data, 'group_report_by');
			if($group_report_by == 1){
				$data = kjolhaug_get_without_grouping_invoice_report_data($report_data);
			} elseif($group_report_by == 2){
				$data = kjolhaug_get_grouped_by_project_invoice_report_data($report_data);
			} elseif($group_report_by == 3){
				$data = kjolhaug_get_grouped_by_clients_invoice_report_data($report_data);
			} elseif($group_report_by == 4){
				$data = kjolhaug_get_grouped_by_clients_and_projects_invoice_report_data($report_data);
			} else {
				flash_error('A project has to be selected to run the report');
				ajx_current('empty');
				return;
			}
		
		}

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'invoice_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($csv) {
            return array(
                'data' => $data,
            );
        }
		
		tpl_assign('data', $data);
		tpl_assign('project_member', $project_member);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());
		tpl_assign('hide_print_report_header', 'true');
		tpl_assign('print_action_url', get_url('kjolhaug_reports', 'kjolhaug_invoice_report_print'));
		tpl_assign('pdf_export_function', 'og.kjolhaug.export_kjolhaug_invoice_report_pdf');
		tpl_assign('excel_export_action', 'og.kjolhaug.export_kjolhaug_invoice_report_excel');

		tpl_assign('report_template_path', get_template_path('kjolhaug_invoice_report', 'kjolhaug_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}
	

	function kjolhaug_invoice_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->kjolhaug_invoice_report($_POST, true);
			
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('invoicing history report').".csv"));
			
			$rows_csv = build_invoice_report_csv($report_result['data']);
	
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
	
		} else if($export_pdf) {
			
			ob_start();
			$this->setTemplate('kjolhaug_invoice_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			
			$this->kjolhaug_invoice_report($_POST);
			tpl_display(get_template_path('kjolhaug_invoice_report', 'kjolhaug_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "invoicing_history", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function kjolhaug_invoice_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->kjolhaug_invoice_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('invoicing history report').".xlsx"));
		
		$excel_data = build_excel_data_for_invoice_report($report_result['data'], $_POST);

		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('invoicing history report'), 'fixed_cells_width' => 30), 'headers' => array(),'data'=>array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}
	
	
}
