<?php

function kjolhaug_get_without_grouping_invoice_report_data($report_data = array()) {
    $report_data['group_columns_by'] = 0;
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $project_member_ids = kjolhaug_get_project_member_ids_for_invoice_report(array(), $report_data);

    $whole_system = array(
        'total_budgeted' => 0,
        'invoice_data' => array(
            'invoices' => array(),
            'total_invoices' => 0
        ),
        'total_difference' => 0,
        'total_percentage' => 0
    );
    $budgeted_total = 0;
    $invoices_total = 0;
    
    foreach($project_member_ids as $project_member_id){
        $project_member = Members::findById($project_member_id);
        if(!($project_member instanceof Member)) continue;
        $project_data = kjolhaug_get_main_invoice_report_data($project_member, $report_data);
        $whole_system['total_budgeted'] += $project_data['total_budgeted'];
        if ($group_by_month){
            foreach($project_data['invoice_data']['invoices'] as $month_name => $month_info){
                if (!isset($whole_system['invoice_data']['invoices'][$month_name])){
                    $whole_system['invoice_data']['invoices'][$month_name]['total'] = 0;
                    $whole_system['invoice_data']['invoices'][$month_name]['month'] = $month_name;
                }
                $whole_system['invoice_data']['invoices'][$month_name]['total'] += $month_info['total'];
            }
        } else {
            $whole_system['invoice_data']['invoices'] = array_merge($whole_system['invoice_data']['invoices'], $project_data['invoice_data']['invoices']);
        }
        $whole_system['invoice_data']['total_invoices'] += $project_data['invoice_data']['total_invoices']; 
    }
    if (!$group_by_month){
        usort($whole_system['invoice_data']['invoices'], function($a, $b) {
            $date_format = user_config_option('date_format');
            $adtime = DateTime::createFromFormat($date_format, $a['date']);
            $atimestamp = $adtime->getTimestamp();
            $bdtime = DateTime::createFromFormat($date_format, $b['date']);
            $btimestamp = $bdtime->getTimestamp();
            return ($atimestamp < $btimestamp) ? -1 : (($atimestamp > $btimestamp) ? 1 : 0);//$atimestamp <=> $btimestamp;
        });
    } else {
        usort($whole_system['invoice_data']['invoices'], function($a, $b) {
            $adtime = DateTime::createFromFormat('M Y', $a['month']);
            $atimestamp = $adtime->getTimestamp();
            $bdtime = DateTime::createFromFormat('M Y', $b['month']);
            $btimestamp = $bdtime->getTimestamp();
            return ($atimestamp < $btimestamp) ? -1 : (($atimestamp > $btimestamp) ? 1 : 0);//$atimestamp <=> $btimestamp;
        });
    }
    $client_budgeted = $whole_system['total_budgeted'];
    $client_invoiced = $whole_system['invoice_data']['total_invoices'];
    $whole_system['total_difference'] = $client_budgeted - $client_invoiced;
    $whole_system['total_percentage'] = $client_budgeted != 0 ? round(($client_invoiced / $client_budgeted) * 100) : 0;
    $budgeted_total += $client_budgeted;
    $invoices_total += $client_invoiced;
    

    $result = array(
        'whole_system' => $whole_system,
        'project_phases' => array(),
        'report_total_budgeted' => $budgeted_total,
        'report_total_invoiced' => $invoices_total,
        'type_of_report' => 'total_without_grouping'
    );

    return $result;
}

function kjolhaug_get_grouped_by_clients_invoice_report_data($report_data= array()){
    $report_data['group_columns_by'] = 0;
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $client_ot = ObjectTypes::findByName('customer');
    $client_members = Members::findAll(array('conditions' => '`object_type_id`='.$client_ot->getId().' AND `depth`=1'));
    $clients = array();
    $budgeted_total = 0;
    $invoices_total = 0;

    $project_client_assoc_id = kjolhaug_get_project_client_assoc_id();

    foreach($client_members as $cm){
        $project_member_ids = kjolhaug_get_project_member_ids_for_invoice_report(array($project_client_assoc_id => $cm->getId()), $report_data);
        $clients[$cm->getName()] = array(
            'total_budgeted' => 0,
            'invoice_data' => array(
                'invoices' => array(),
                'total_invoices' => 0
            ),
            'total_difference' => 0,
            'total_percentage' => 0
        );
        foreach($project_member_ids as $project_id){
            $project_member = Members::getMemberById($project_id);
            $project_data = kjolhaug_get_main_invoice_report_data($project_member, $report_data);
            $clients[$cm->getName()]['total_budgeted'] += $project_data['total_budgeted'];
            if ($group_by_month){
                foreach($project_data['invoice_data']['invoices'] as $month_name => $month_info){
                    if (!isset($clients[$cm->getName()]['invoice_data']['invoices'][$month_name])){
                        $clients[$cm->getName()]['invoice_data']['invoices'][$month_name]['total'] = 0;
                        $clients[$cm->getName()]['invoice_data']['invoices'][$month_name]['month'] = $month_name;
                    }
                    $clients[$cm->getName()]['invoice_data']['invoices'][$month_name]['total'] += $month_info['total'];
                }
            } else {
                $clients[$cm->getName()]['invoice_data']['invoices'] = array_merge($clients[$cm->getName()]['invoice_data']['invoices'], $project_data['invoice_data']['invoices']);
            }
            $clients[$cm->getName()]['invoice_data']['total_invoices'] += $project_data['invoice_data']['total_invoices']; 
        }
        if (!$group_by_month){
            usort($clients[$cm->getName()]['invoice_data']['invoices'], function($a, $b) {
                $date_format = user_config_option('date_format');
                $adtime = DateTime::createFromFormat($date_format, $a['date']);
                $atimestamp = $adtime->getTimestamp();
                $bdtime = DateTime::createFromFormat($date_format, $b['date']);
                $btimestamp = $bdtime->getTimestamp();
                return ($atimestamp < $btimestamp) ? -1 : (($atimestamp > $btimestamp) ? 1 : 0);//$atimestamp <=> $btimestamp;
            });
        } else {
            usort($clients[$cm->getName()]['invoice_data']['invoices'], function($a, $b) {
                $adtime = DateTime::createFromFormat('M Y', $a['month']);
                $atimestamp = $adtime->getTimestamp();
                $bdtime = DateTime::createFromFormat('M Y', $b['month']);
                $btimestamp = $bdtime->getTimestamp();
                return ($atimestamp < $btimestamp) ? -1 : (($atimestamp > $btimestamp) ? 1 : 0);//$atimestamp <=> $btimestamp;
            });
        }
        if ($clients[$cm->getName()]['total_budgeted'] == 0 && $clients[$cm->getName()]['invoice_data']['total_invoices'] == 0){
            unset($clients[$cm->getName()]);
        } else {
            $client_budgeted = $clients[$cm->getName()]['total_budgeted'];
            $client_invoiced = $clients[$cm->getName()]['invoice_data']['total_invoices'];
            $clients[$cm->getName()]['total_difference'] = $client_budgeted - $client_invoiced;
            $clients[$cm->getName()]['total_percentage'] = $client_budgeted != 0 ? round(($client_invoiced / $client_budgeted) * 100) : 0;
            $budgeted_total += $client_budgeted;
            $invoices_total += $client_invoiced;
        }
    }

    $result = array(
        'clients' => $clients,
        'project_phases' => array(),
        'report_total_budgeted' => $budgeted_total,
        'report_total_invoiced' => $invoices_total,
        'type_of_report' => 'total_grouped_by_clients'
    );

    return $result;
}

function kjolhaug_get_grouped_by_clients_and_projects_invoice_report_data($report_data = array()){
    $report_data['group_columns_by'] = 0;
    $client_ot = ObjectTypes::findByName('customer');
    $client_members = Members::findAll(array('conditions' => '`object_type_id`='.$client_ot->getId().' AND `depth`=1'));
    $clients = array();
    $budgeted_total = 0;
    $invoices_total = 0;

    $project_client_assoc_id = kjolhaug_get_project_client_assoc_id();

    foreach($client_members as $cm){
        $project_member_ids = kjolhaug_get_project_member_ids_for_invoice_report(array($project_client_assoc_id => $cm->getId()),$report_data);
        $clients[$cm->getName()] = array(
            'projects' => array(),
            'client_budget_total' => 0,
            'client_invoices_total' => 0
        );
        foreach($project_member_ids as $project_id){
            $project_member = Members::getMemberById($project_id);
            $project_data = kjolhaug_get_main_invoice_report_data($project_member, $report_data);
            // Don't include projects that doesn't have budgeted and invoices info
            if (!($project_data['total_budgeted'] == 0 && $project_data['invoice_data']['total_invoices'] == 0)){
                $clients[$cm->getName()]['projects'][] = $project_data;
                $clients[$cm->getName()]['client_budget_total'] += $project_data['total_budgeted'];
                $clients[$cm->getName()]['client_invoices_total'] += $project_data['invoice_data']['total_invoices']; 
            }
        }
        $budgeted_total += $clients[$cm->getName()]['client_budget_total'];
        $invoices_total += $clients[$cm->getName()]['client_invoices_total'];
        // Don't include clients that doesn't have budgeted and invoices info
        if ($clients[$cm->getName()]['client_budget_total'] == 0 && $clients[$cm->getName()]['client_invoices_total'] == 0){
            unset($clients[$cm->getName()]);
        }
    }

    $result = array(
        'clients' => $clients,
        'project_phases' => array(),
        'report_total_budgeted' => $budgeted_total,
        'report_total_invoiced' => $invoices_total,
        'type_of_report' => 'total_grouped_by_clients_and_projects'
    );

    return $result;
}

function kjolhaug_get_grouped_by_project_invoice_report_data($report_data = array()){
    $project_member_ids = kjolhaug_get_project_member_ids_for_invoice_report(array(), $report_data);

    $report_data['group_columns_by'] = 0;
    $projects = array();
    $project_phases = array();
    $invoices_total = 0;
    $budgeted_total = 0;
    foreach($project_member_ids as $project_member_id){
        $project_member = Members::findById($project_member_id);
        if(!($project_member instanceof Member)) continue;
        $project_data = kjolhaug_get_main_invoice_report_data($project_member, $report_data);
        if(!($project_data['total_budgeted'] == 0 && $project_data['invoice_data']['total_invoices'] == 0)){
            $projects[] = $project_data;
            $project_phases = array_unique(array_merge($project_phases, array_var($project_data, 'project_phases')));
            $budgeted_total += $project_data['total_budgeted'];
            $invoices_total += $project_data['invoice_data']['total_invoices'];
        }
    }
    asort($project_phases);

    $result = array(
        'projects' => $projects,
        'project_phases' => $project_phases,
        'report_total_budgeted' => $budgeted_total,
        'report_total_invoiced' => $invoices_total,
        'type_of_report' => 'total_grouped_by_projects'
    );

    return $result;
}

function kjolhaug_get_main_invoice_report_data(Member $project_member, $report_data = array()) {
    $group_by_subprojects = array_var($report_data, 'group_columns_by');
    $use_budget_total_custom_property = array_var($report_data, 'use_budget_total_custom_property');

    // Get project name and id
    $project_name_and_number_info = get_project_name_and_number_info($project_member);

    $total_amount_per_phase = array();
    $budget_phases = array();

    // Get custom property value with code 'budget_total'
    if($use_budget_total_custom_property){
        $cp_object_type_id = $project_member->getObjectTypeId();
        $custom_props = CustomProperties::getAllCustomPropertiesByObjectType($cp_object_type_id);
        foreach ($custom_props as $c_prop) {
            if($c_prop->getCode() == 'budget_total'){
                $values = CustomPropertyValues::getCustomPropertyValues($project_member->getObjectId(), $c_prop->getId());
                $custom_value = !empty($values) && $values[0] instanceof CustomPropertyValue ? $values[0]->getValue() : 0;
                $total_budget = floatval(str_replace(',','', $custom_value)) > 0 ? floatval(str_replace(',','', $custom_value)) : 0;  
            }
        }
        if(!isset($total_budget)){
            $total_budget = 0;
        }
    } else { // Get budget data if project doesn't have custom property with 'budget_total' code
        $budget_data = get_budget_report_data($project_member, array('group_columns_by' => $group_by_subprojects));
        $budget_phases = $budget_data['project_phases'];
        $total_amount_per_phase = array();
        foreach($budget_phases as $project_phase){
            $labor_amount = isset($budget_data['labor_data']['total_labor_per_phase_amount'][$project_phase]) ? $budget_data['labor_data']['total_labor_per_phase_amount'][$project_phase] : 0;
            $expense_amount = isset($budget_data['expenses_data']['total_expenses_amount_per_phase'][$project_phase]) ? $budget_data['expenses_data']['total_expenses_amount_per_phase'][$project_phase] : 0;
            $total_amount_per_phase[$project_phase] = $labor_amount + $expense_amount;
        }
        $total_budget = !empty($total_amount_per_phase) ? array_sum($total_amount_per_phase) : 0;
    }

    $invoice_data = kjolhaug_get_invoice_data($project_member, $report_data);
    $invoice_phases = isset($invoice_data['invoices_amount_per_phase']) ? array_keys($invoice_data['invoices_amount_per_phase']) : array();

    $project_phases = array_unique(array_merge($budget_phases, $invoice_phases));
    asort($project_phases);

    $difference_per_phase = array();
    $percentage_per_phase = array();
    foreach($project_phases as $phase){
        $budget_amount = isset($total_amount_per_phase[$phase]) ? $total_amount_per_phase[$phase] : 0;
        $invoice_amount = isset($invoice_data['invoices_amount_per_phase'][$phase]) ? $invoice_data['invoices_amount_per_phase'][$phase] : 0;
        $difference_per_phase[$phase] = $budget_amount - $invoice_amount;
        $percentage_per_phase[$phase] = $budget_amount != 0 ? round(($invoice_amount / $budget_amount) * 100) : 0;

    }

    $total_difference = $total_budget - $invoice_data['total_invoices'];
    $total_percentage = $total_budget != 0 ? round(($invoice_data['total_invoices'] / $total_budget) * 100) : 0;
   
    asort($project_phases); 

    $result = array(
        'type_of_report' => 'individual_project',
        'project_member' => $project_member,
        'project_name' => array_var($project_name_and_number_info, 'project_name'),
        'project_id' => array_var($project_name_and_number_info, 'project_id'),
        'project_id_raw' => array_var($project_name_and_number_info, 'project_id_raw'),
        'project_phases' => $project_phases,
        'budgeted_amounts_per_phase' => $total_amount_per_phase,
        'total_budgeted' => $total_budget,
        'invoice_data' => $invoice_data,
        'difference_per_phase' => $difference_per_phase,
        'percentage_per_phase' => $percentage_per_phase,
        'total_difference' => $total_difference,
        'total_percentage' => $total_percentage
        );

    return $result;
}

function kjolhaug_get_invoice_data(Member $project_member, $report_data = array()){
    $group_by_subprojects = array_var($report_data, 'group_columns_by');
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $invoices = kjolhaug_get_member_invoices_for_invoice_report($project_member, $report_data);
    $invoices_data = array();
    $project_phases_invoices = array();
    $date_format = user_config_option('date_format');
    // We are using 'Job phase' as project phase.
    $job_phase_ot = ObjectTypes::findByName('job_phase');
    if ($job_phase_ot instanceof ObjectType){
        $phase_object_type_name = $job_phase_ot->getObjectTypeName();
    } else {
        $phase_object_type_name = lang('job phase');
    }
    

    foreach($invoices as $invoice){
        if($group_by_month) {
            $date = $invoice['date']->format('M Y');
            if(!isset($invoices_data[$date])){
                $invoices_data[$date] = array();
                $invoices_data[$date]['month'] = $date;
            }
            if($group_by_subprojects){
                $invoice_phase = $invoice['customer_project'][0];
            } else {
                $invoice_phase = isset($invoice['job_phases']) ? $invoice['job_phases'][0] : ucwords(lang('without ').$phase_object_type_name);
            }

            if(!isset($invoices_data[$date][$invoice_phase])){
                $invoices_data[$date][$invoice_phase] = 0;
            }
            $invoices_data[$date][$invoice_phase] += $invoice['total'];

            if(!isset($invoices_data[$date]['total'])){
                $invoices_data[$date]['total'] = 0;
            }
            $invoices_data[$date]['total'] += $invoice['total'];

            if(!isset($project_phases_invoices[$invoice_phase])){
                $project_phases_invoices[$invoice_phase] = 0;
            }
    
            $project_phases_invoices[$invoice_phase] += $invoice['total'];
        } else {
            $inv_data = array();
            $inv_data['date'] = $invoice['date']->format($date_format);
            $inv_data['amount'] = $invoice['total'];
            $inv_data['invoice_number'] = $invoice['invoice_number'];
            $inv_data['project'] = !empty($invoice['customer_project']) ? $invoice['customer_project'][0] : '';
            $inv_data['client'] = !empty($invoice['customers']) ? $invoice['customers'][0] : '';
            if($group_by_subprojects){
                $inv_data['project_phase'] = $invoice['customer_project'][0];
            } else {
                $inv_data['project_phase'] = isset($invoice['job_phases']) ? $invoice['job_phases'][0] : ucwords(lang('without ').$phase_object_type_name);
            }
            $invoices_data[] = $inv_data;
            if(!isset($project_phases_invoices[$inv_data['project_phase']])){
                $project_phases_invoices[$inv_data['project_phase']] = 0;
            }
    
            $project_phases_invoices[$inv_data['project_phase']] += $invoice['total'];
        }


    }
    $total_invoices = !empty($project_phases_invoices) ? array_sum($project_phases_invoices) : 0;
    
    $invoices_result = array(
        'invoices' => $invoices_data,
        'invoices_amount_per_phase' => $project_phases_invoices,
        'total_invoices' => $total_invoices
    );

    return $invoices_result;
}

function kjolhaug_get_member_invoices_for_invoice_report(Member $project_member, $report_data = array()){
    $extra_conditions = " AND `status` != 'canceled' ";

    $res = IncomeInvoices::instance()->listing(array(
        "order" => 'date',
        "order_dir" => 'ASC',
        "start" => 0,
        "limit" => 0,
        "ignore_context" => true,
        "count_results" => false,
        "only_count_results" => false,
        "extra_conditions" => $extra_conditions,
        "member_ids" => array($project_member->getId()),
    ));
    $invoices = $res->objects;

    $invoices_data = array();
	foreach ($invoices as $invoice) {
        /* @var $task ProjectTask */
        $object = Objects::findById($invoice->getObjectId());
        $is_invoice = ObjectSubtypes::findById($object->getObjectSubtypeId())->getCode() == 'invoice';
        
        if($is_invoice) {
            $i_data = $invoice->getArrayInfo();
        
            $members = $invoice->getMembers();
            foreach ($members as $member) {
                /* @var $member Member */
                $mem_dim = $member->getDimension();
                if($mem_dim->getCode() == 'customer_project'){
                    $project_info = get_project_name_and_number_info($member);
                    $memberList = array($member);
                    // build the name shown in trees
                    build_member_list_text_to_show_in_trees($memberList);
                }
                if (!isset($i_data[$mem_dim->getCode()])) $i_data[$mem_dim->getCode()] = array();
                if($mem_dim->getCode() == 'customer_project'){  
                    $i_data[$mem_dim->getCode()][] = $memberList[0]->getName();
                } else {
                    $i_data[$mem_dim->getCode()][] = $member->getName();
                }
            }
            
            $invoices_data[] = $i_data;
        }
	}
	return $invoices_data;
}

function kjolhaug_get_project_member_ids_for_invoice_report($filter_members_id = array(), $report_data){
    // Get project members
    $project_ot = ObjectTypes::findByName('project');
    $project_dimension = Dimensions::findByCode('customer_project');
    $extra_conditions = " AND mem.parent_member_id=0";
    $params = array(
        'dim_id' => $project_dimension->getId(), 
        'type_id' => $project_ot->getId(), 
        'start' => 0, 
        'limit' => 99999, 
        'extra_conditions' => $extra_conditions,
        'filter_members_id' => $filter_members_id
    );
    $memberController = new MemberController();
    $object = $memberController->listing($params);
    $project_members = $object['members'];

    
    $project_member_ids = array();
    if(!empty($project_members)){
        $project_member_ids = array_map(function ($row){
                                            return $row['id'];
                                        }, $project_members);
    }

    // Get projects filtered by date and contract type
    $project_ids_filtered_by_date = kjolhaug_get_project_ids_in_date_range($report_data);
    $contract_type = array_var($report_data, 'contract_type', 'all');
    $project_ids_filtered_by_contract_type = kjolhaug_get_project_ids_with_contract_type($contract_type);
    $filtered_project_ids = array();
    foreach($project_member_ids as $project_id){
        if($contract_type == 'all'){
            if(in_array($project_id, $project_ids_filtered_by_date)) $filtered_project_ids[] = $project_id;
        } else {
            if(in_array($project_id, $project_ids_filtered_by_date) && in_array($project_id, $project_ids_filtered_by_contract_type)) $filtered_project_ids[] = $project_id;
        }
    }

    return $filtered_project_ids;
}

function kjolhaug_get_project_client_assoc_id(){
    $project_ot = ObjectTypes::findByName('project');
    $project_dimension = Dimensions::findByCode('customer_project');
    $client_ot = ObjectTypes::findByName('customer');

    $project_client_assoc_id = DimensionMemberAssociations::instance()->findOne(array('conditions' => array('object_type_id=? AND associated_object_type_id=?',
    $project_ot->getId(), $client_ot->getId())))->getId();

    return $project_client_assoc_id;
}

function kjolhaug_get_project_ids_in_date_range($report_data){
    $project_ot = ObjectTypes::findByName('project');

    // Get start and end date in SQL format
    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    // Get project member ids assigned to invoices in the given date range
    $invoice_sql = "SELECT DISTINCT(m.id)
            FROM ".TABLE_PREFIX."income_invoices i INNER JOIN ".TABLE_PREFIX."object_members om ON i.object_id=om.object_id
            INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
            WHERE i.date>='".$sql_st_date."' AND i.date<='".$sql_end_date."'
            AND m.object_type_id=".$project_ot->getId().";";
    $invoice_rows = DB::executeAll($invoice_sql);
    $project_member_ids = array();
    if(!empty($invoice_rows)){
        $project_member_ids = array_map(function ($row){
                                            return $row['id'];
                                        }, $invoice_rows);
    }

    return $project_member_ids;
}

function kjolhaug_get_project_ids_with_contract_type($contract_type){
    if($contract_type == 'all') return array();

    $project_ot_id = ObjectTypes::findByName('project')->getId();
	$contract_type_cp_id = CustomProperties::getCustomPropertyByCode($project_ot_id, 'contract_type')->getId();

    $sql = "SELECT m.id 
            FROM ".TABLE_PREFIX."members m INNER JOIN ".TABLE_PREFIX."custom_property_values cpv 
            ON m.object_id=cpv.object_id
            WHERE cpv.custom_property_id=".$contract_type_cp_id." AND cpv.value='".$contract_type."';";

    $members_rows = DB::executeAll($sql);

    $project_member_ids = array();
    if(!empty($members_rows)){
        $project_member_ids = array_map(function ($row){
                                            return $row['id'];
                                        }, $members_rows);
    }

    return $project_member_ids;
}
