<?php 

Env::useHelper('view_functions', 'project_reports');


$project_phases = array_var($data, 'project_phases');
$report_data = array_var($_SESSION, 'invoice_report_data');


function render_invoice_table($project_data, $report_data, $data, $params = array()){
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $add_indentation = array_var($params, 'add_indentation');
    $use_budget_total_custom_property = array_var($report_data, 'use_budget_total_custom_property');
    $invoices = isset($project_data['invoice_data']['invoices']) ? $project_data['invoice_data']['invoices'] : array();
    $invoices_amount_per_phase = isset($project_data['invoice_data']['invoices_amount_per_phase']) ? $project_data['invoice_data']['invoices_amount_per_phase'] : array();
    if($data['type_of_report'] != 'individual_project') {
        $project_phases = array();
    } else {
        $project_phases = array_var($data, 'project_phases');
    }
    $currency_sym = config_option('currency_code');

    $indent_class = $add_indentation ? 'indent-medium' : '';
    // Build table header
    echo '<thead><tr><th></th>';
    if(!$group_by_month){
        echo '<th>'.lang('invoice number').'</th>';
        if($data['type_of_report'] == 'total_without_grouping') {
            echo '<th>'.lang('client').'</th>';
        }
        if($data['type_of_report'] == 'total_grouped_by_clients' || $data['type_of_report'] == 'total_without_grouping'){
            echo '<th>'.lang('project').'</th>';
        }
    }
    foreach($project_phases as $phase){
        echo '<th class="phase-column">'.$phase.'</th>';
    }
    echo '<th>'.lang('total').'</th></tr></thead>';

    // Build table body.
    // Build budgeted data row
    $budgeted_colspan = $group_by_month ? 1 : 2;
    if($use_budget_total_custom_property){
        $budgeted_colspan += count($project_phases);
    }
    if(!$group_by_month){
        if ($data['type_of_report'] == 'total_grouped_by_clients'){
            $budgeted_colspan += 1;
        } elseif ($data['type_of_report'] == 'total_without_grouping'){
            $budgeted_colspan += 2;
        }
    }
    echo '<tbody>';
    echo '<tr><td colspan="'.$budgeted_colspan.'" class="category-name invoice-budget-cell '.$indent_class.'">'.lang('budgeted').'</td>';
    if(!$use_budget_total_custom_property){
        foreach($project_phases as $phase){
            $budget_amount = isset($project_data['budgeted_amounts_per_phase'][$phase]) ? format_money_amount($project_data['budgeted_amounts_per_phase'][$phase], $currency_sym) : '0';
            echo '<td class="report-number invoice-budget-cell">'.$budget_amount.'</td>';
    
        }
    }
    echo '<td class="report-number invoice-budget-cell">'.format_money_amount($project_data['total_budgeted'], $currency_sym).'</td></tr>';
    echo '</tbody>';

    // Build invoices data
    echo '<tbody class="report-invoices">';
    if(count($invoices) > 0){
        if($group_by_month){
            foreach($invoices as $v){
                echo '<tr><td class="category-name '.$indent_class.'" style="">'.$v['month'].'</td>';
                foreach($project_phases as $phase){
                    if(isset($v[$phase])){
                        echo '<td class="report-number">'.format_money_amount($v[$phase], $currency_sym).'</td>';
                    } else {
                        echo '<td></td>';
                    }
                }
                echo '<td class="report-number">'.format_money_amount($v['total'], $currency_sym).'</td></tr>';
            }
        } else {
            foreach($invoices as $invoice){
                echo '<tr><td class="category-name '.$indent_class.'" style="">'.$invoice['date'].'</td>';
                echo '<td class="category-name" style="">'.$invoice['invoice_number'].'</td>';
                if ($data['type_of_report'] == 'total_without_grouping'){
                    echo '<td class="category-name">'.$invoice['client'].'</td>';
                }
                if($data['type_of_report'] == 'total_grouped_by_clients' || $data['type_of_report'] == 'total_without_grouping'){
                    echo '<td class="category-name">'.$invoice['project'].'</td>';
                }
                foreach($project_phases as $phase){
                    if($invoice['project_phase'] == $phase){
                        echo '<td class="report-number">'.format_money_amount($invoice['amount'], $currency_sym).'</td>';
                    } else {
                        echo '<td></td>';
                    }
                }
                echo '<td class="report-number">'.format_money_amount($invoice['amount'], $currency_sym).'</td></tr>';
            }
        }
    }
    echo '</tbody>';

    // Build total part
    echo '<tbody class="grand-total">';

    // Build 'Billed to date'
    $billed_colspan = $group_by_month ? 1 : 2;
    if(!$group_by_month){
        if ($data['type_of_report'] == 'total_grouped_by_clients'){
            $billed_colspan += 1;
        } elseif ($data['type_of_report'] == 'total_without_grouping') {
            $billed_colspan += 2;
        }
    }
    echo '<tr><td colspan="'.$billed_colspan.'" class="'.$indent_class.'">'.lang('billed to date').'</td>';
    foreach($project_phases as $phase){
        if(isset($invoices_amount_per_phase[$phase])){
            echo '<td class="report-number" style="color: #0BA888; font-weight: 600;">'.format_money_amount($invoices_amount_per_phase[$phase], $currency_sym).'</td>';
        } else {
            echo '<td class="report-number" style="color: #0BA888; font-weight: 600;">'.format_money_amount(0, $currency_sym).'</td>';
        }
    }
    echo '<td class="report-number" style="color: #0BA888; font-weight: 600;">'.format_money_amount($project_data['invoice_data']['total_invoices'], $currency_sym).'</td></tr>';

    // Build 'Remaining budget'
    $remaining_colspan = $group_by_month ? 1 : 2;
    if(!$group_by_month){
        if ($data['type_of_report'] == 'total_grouped_by_clients'){
            $remaining_colspan += 1;
        } elseif ($data['type_of_report'] == 'total_without_grouping') {
            $remaining_colspan += 2;
        }
    }
    if($use_budget_total_custom_property){
        $remaining_colspan += count($project_phases);
    }
    echo '<tr><td colspan="'.$remaining_colspan.'" class="'.$indent_class.'">'.lang('remaining budget').'</td>';
    if(!$use_budget_total_custom_property){
        foreach($project_phases as $phase){
            if(isset($project_data['difference_per_phase'][$phase])){
                $opening_parenthesis = $project_data['difference_per_phase'][$phase] < 0 ? '( ' : '';
                $closing_parenthesis = $project_data['difference_per_phase'][$phase] < 0 ? ' )' : '';
                $difference_per_phase = $project_data['difference_per_phase'][$phase] < 0 ? $project_data['difference_per_phase'][$phase]*(-1) :  $project_data['difference_per_phase'][$phase];
                echo '<td class="report-number">' . $opening_parenthesis . format_money_amount($difference_per_phase, $currency_sym) . $closing_parenthesis . '</td>';
            } else {
                echo '<td class="report-number">'.format_money_amount(0, $currency_sym).'</td>';
            }
        }
    }
    $total_opening_parenthesis = $project_data['total_difference'] < 0 ? '( ' : '';
    $total_closing_parenthesis = $project_data['total_difference'] < 0 ? ' )' : '';
    $total_difference = $project_data['total_difference'] < 0 ? $project_data['total_difference']*(-1) : $project_data['total_difference'];
    echo '<td class="report-number">'.$total_opening_parenthesis.format_money_amount($total_difference, $currency_sym).$total_closing_parenthesis.'</td></tr>';

    // Build 'Billed vs budgeted %'
    $percentage_colspan = $group_by_month ? 1 : 2;
    if(!$group_by_month){
        if ($data['type_of_report'] == 'total_grouped_by_clients'){
            $percentage_colspan += 1;
        } elseif ($data['type_of_report'] == 'total_without_grouping') {
            $percentage_colspan += 2;
        }
    }
    if($use_budget_total_custom_property){
        $percentage_colspan += count($project_phases);
    }
    echo '<tr><td colspan="'.$percentage_colspan.'" class="'.$indent_class.'">'.lang('billed vs budgeted %').'</td>';
    if(!$use_budget_total_custom_property){
        foreach($project_phases as $phase){
            if(isset($project_data['percentage_per_phase'][$phase])){
                echo '<td class="report-number">'.$project_data['percentage_per_phase'][$phase].' %'.'</td>';
            } else {
                echo '<td></td>';
            }
        }
    }
     echo '<td class="report-number">'.$project_data['total_percentage'].' %'.'</td></tr>';
     echo '</tbody>';

}

function render_project_invoice_table_header($project_member, $report_data = array(), $header_params = array()){
    $project_phases = array_var($data, 'project_phases');
    $show_main_information_blocks = isset($report_data['show_main_project_info']);
    // $count_phases = count($project_phases);
    $colspan = 10;
    echo '<thead></thead>';
    if ($show_main_information_blocks) {
        if ($project_member instanceof Member) {
            $properties_array = '';
            $params =  array(
                    'member' => $project_member,
                    'visible_by_default' => true,
                    'return_as_array' => true,
            );
            Hook::fire('override_render_properties_for_view', $params, $properties_array);
            // Logger::log_r($properties_array);
            // function from project_reports/application/helpers/view_funtions.php
            render_project_report_main_information($project_member, $properties_array, $colspan, $header_params);
        }
    } else {
        if ($project_member instanceof Member) {
            $project_prefix_object = ObjectPrefixes::findById($project_member->getObjectId());
            if($project_prefix_object){
                $prefix_number = $project_prefix_object->getPrefix();
                $project_name = str_replace($prefix_number.' - ', '', $project_member->getName());
                $project_id = '<span style="color: #0a85d8; text-transform: capitalize;">'.lang('number').' </span>'.$prefix_number;
            } else {
                $pot = ObjectTypes::findByName('project');
                $project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
                $project_id = '';
                if ($project_id_cp instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $project_number_custom_property_name = CustomProperties::findById($cp_val->getCustomPropertyId())->getName();
                        $project_id = '<span style="color: #0a85d8; text-transform: capitalize;">'.$project_number_custom_property_name.' </span>'.$cp_val->getValue();
                    }
                }
                $project_name = $project_member->getName();
            }
            $client_info = '';
            if (array_var($header_params, 'include_client_name')){
                $client_dimension = Dimensions::findByCode('customers');
                $client_ot = ObjectTypes::findByName('customer');
                $selected_ids = get_all_associated_status_member_ids($project_member, $client_dimension, $client_ot, false, 'project_customer');
                if(!empty($selected_ids)){
                    $client_member = Members::getMemberById($selected_ids[0]);
                    if($client_member instanceof Member){
                        $client_name = $client_member->getName();
                        // $client_object_type_name = Objects::findById($client_member->getObjectId())->getObjectTypeName();
                        $client_info = '<span style="color: #0a85d8; text-transform: capitalize;">'.lang('client').' </span>'. $client_name;
                    }
                }
            }
            $indent_class = array_var($header_params, 'add_indentation') ? 'indent-medium' : '';
            $project_object_type_name = Objects::findById($project_member->getObjectId())->getObjectTypeName();
            echo '<thead class="main-project-name--invoices">'.
            '<tr><td colspan="10" class="'.$indent_class.'"><div style="width: 100%; white-space: normal; line-height: 1.4;">'.
            '<span style="padding-right: 15px; "><span style="color: #0a85d8; margin-right: 20px;">'.lang($project_object_type_name).'</span>'.$project_name.'</span>'.
            '<span style="font-size: 13px; padding-right: 15px; font-weight: bold;">'.$project_id.'</span>'.
            '<span style="font-size: 13px; font-weight: bold;">'.$client_info.'</span>'.
            '</div></td></tr></thead>';
            echo '<thead class="table-separator--small"><tr><td></td></tr></thead>';
        }
    }
}

function render_invoicing_history_total_table($report_data, $params){
    $table_name = array_var($params, 'table_name');
    $total_budgeted = array_var($params, 'total_budgeted');
    $total_invoiced = array_var($params, 'total_invoiced');
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $currency_sym = config_option('currency_code');
    $group_report_by = array_var($report_data, 'group_report_by');

    $colspan = $group_by_month ? 1 : 2;
    if(!$group_by_month){
        if($group_report_by == 3){
            $colspan += 1;
        } elseif ($group_report_by == 1) {
            $colspan += 2;
        }
    }
    echo '<thead class="table-name table-name--totals" style="font-size: 28px;"><tr><th colspan='.$colspan.'>'.$table_name.'</th></tr></thead>';
    echo '<thead class="table-separator--small"><tr><td></td></tr></thead>';
    echo '<tbody>';

    // Total Budgeted info
    echo '<tr class="invoice-report-total-table-row"><td colspan="'.$colspan.'" class="category-name invoice-budget-cell">'.lang('budgeted').'</td>';
    echo '<td class="report-number invoice-budget-cell">'.format_money_amount($total_budgeted, $currency_sym).'</td></tr>';

    // Total 'Billed to date' info
    echo '<tr class="invoice-report-total-table-row"><td colspan="'.$colspan.'" class="category-name invoice-budget-cell">'.lang('billed to date').'</td>';
    echo '<td class="report-number invoice-budget-cell" style="color: #0BA888; font-weight: 600;">'.format_money_amount($total_invoiced, $currency_sym).'</td></tr>';

    // Total 'Remaining balance' info
    echo '<tr class="invoice-report-total-table-row"><td colspan="'.$colspan.'" class="category-name invoice-budget-cell">'.lang('remaining budget').'</td>';
    $total_difference = $total_budgeted - $total_invoiced;
    $total_opening_parenthesis = $total_difference < 0 ? '( ' : '';
    $total_closing_parenthesis = $total_difference < 0 ? ' )' : '';
    $total_difference = $total_difference < 0 ? $total_difference*(-1) : $total_difference;
    echo '<td class="report-number invoice-budget-cell">'.$total_opening_parenthesis.format_money_amount($total_difference, $currency_sym).$total_closing_parenthesis.'</td></tr>';

    // Total 'Billed to date' info
    echo '<tr class="invoice-report-total-table-row"><td colspan="'.$colspan.'" class="category-name invoice-budget-cell">'.lang('billed vs budgeted %').'</td>';
    $total_percentage = $total_budgeted != 0 ? round(($total_invoiced / $total_budgeted)*100) : 0;
    echo '<td class="report-number invoice-budget-cell">'.$total_percentage.' %'.'</td></tr>';
    
    echo '</tbody>';
}



echo '<div class="">';

echo '<div style="font-size: 46px; font-weight: 600px; margin-top: 10px; display: inline-block; margin-bottom: 30px;"><p>'.$title.'</p></div>';

echo '<table id="invoice-report-table">';
if($data['type_of_report'] == 'individual_project'){
    render_project_invoice_table_header($project_member, $report_data);
    render_invoice_table($data, $report_data, $data);
} elseif ($data['type_of_report'] == 'total_grouped_by_projects'){
    $header_params = array(
        'include_client_name' => true
    );
    foreach($data['projects'] as $project_data){
        render_project_invoice_table_header($project_data['project_member'], $report_data, $header_params);
        render_invoice_table($project_data, $report_data, $data);
        echo '<thead class="table-separator"><tr><td></td></tr></thead>';  
    }

    $total_params = array(
        'table_name' => lang('totals'),
        'total_budgeted' => $data['report_total_budgeted'],
        'total_invoiced' => $data['report_total_invoiced']
    );
    render_invoicing_history_total_table($report_data, $total_params);

} elseif ($data['type_of_report'] == 'total_grouped_by_clients_and_projects'){
    foreach($data['clients'] as $client_name => $client_info){
        echo '<thead class="table-name table-name--client" style="font-size: 28px;"><tr><th colspan="10"><span style="color: #888; margin-right: 15px;">'.lang('client').'</span>'.$client_name.'</th></tr></thead>';
        echo '<thead class="table-separator--small"><tr><td></td></tr></thead>';
        $params = array(
            'add_indentation' => true
        );
        foreach($client_info['projects'] as $project_data){
            render_project_invoice_table_header($project_data['project_member'], $report_data, $params);
            render_invoice_table($project_data, $report_data, $data, $params);
            echo '<thead class="table-separator--small"><tr><td></td></tr></thead>'; 
        }
        $client_total_params = array(
            'table_name' => lang('totals').': '.$client_name,
            'total_budgeted' => $client_info['client_budget_total'],
            'total_invoiced' => $client_info['client_invoices_total']
        );
        render_invoicing_history_total_table($report_data, $client_total_params);
        echo '<thead class="table-separator"><tr><td></td></tr></thead>';

    }

    $total_params = array(
        'table_name' => lang('totals'),
        'total_budgeted' => $data['report_total_budgeted'],
        'total_invoiced' => $data['report_total_invoiced']
    );
    render_invoicing_history_total_table($report_data, $total_params);
    
} elseif ($data['type_of_report'] == 'total_grouped_by_clients'){
    foreach($data['clients'] as $client_name => $client_info){
        echo '<thead class="table-name table-name--client" style="font-size: 28px;"><tr><th colspan="10"><span style="color: #888; margin-right: 15px;">'.lang('client').'</span>'.$client_name.'</th></tr></thead>';
        echo '<thead class="table-separator--small"><tr><td></td></tr></thead>'; 
        render_invoice_table($client_info, $report_data, $data);
        echo '<thead class="table-separator"><tr><td></td></tr></thead>';
    }

    $total_params = array(
        'table_name' => lang('totals'),
        'total_budgeted' => $data['report_total_budgeted'],
        'total_invoiced' => $data['report_total_invoiced']
    );
    render_invoicing_history_total_table($report_data, $total_params);
    
} elseif ($data['type_of_report'] == 'total_without_grouping'){
    render_invoice_table($data['whole_system'], $report_data, $data);
    echo '<thead class="table-separator"><tr><td></td></tr></thead>';
    $total_params = array(
        'table_name' => lang('totals'),
        'total_budgeted' => $data['report_total_budgeted'],
        'total_invoiced' => $data['report_total_invoiced']
    );
    render_invoicing_history_total_table($report_data, $total_params);
}
echo '</table>';

/*
$today = DateTimeValueLib::now();
//Logger::log_r($report_data);
$tz_offset = Timezones::getTimezoneOffsetToApply(Objects::findById($project_member->getObjectId()));
$today = $today->add('s', $tz_offset)->beginningOfDay();
$date_format = user_config_option('date_format');
$dateValue = $today->format($date_format);
echo '<p style="margin-top: 20px; text-align: right; color: #999; font-size: 12px;">'.$dateValue.'</date>';*/

echo '</div>';
