<?php
Hook::register("managed_events");


function managed_events_object_definition($type, &$ret) {
	if ($type == 'ProjectEvent') {
		$ret['is_managed_event'] = DATA_TYPE_BOOLEAN;
	}
}


function managed_events_member_listing_additional_joins($params, &$add_joins) {
	$ot = array_var($params, 'ot');
	if ($ot instanceof ObjectType && $ot->getName() == 'managed_event') {
		$add_joins[] = array(
			'join' => "INNER JOIN ".TABLE_PREFIX."project_events events ON events.object_id=obj_type_table.event_id",
			'cols' => "events.*"
		);
		$add_joins[] = array(
			'join' => "INNER JOIN ".TABLE_PREFIX."objects events_obj ON events_obj.id=obj_type_table.event_id and events_obj.trashed_by_id=0",
			'cols' => "events_obj.*"
		);
	}
}


function managed_events_before_action($args, &$ret) {
	$controller = array_var($args, 'controller');
	$action = array_var($args, 'action');
	
	if ($controller instanceof EventController && $action == 'edit') {
		$object = ProjectEvents::findById(get_id());
		if ($object instanceof ProjectEvent && $object->getColumnValue('is_managed_event')) {
			
			$dimension = Dimensions::findByCode('managed_events');
			
			$managed_event = ManagedEvents::instance()->findByEventId($object->getId());
			if (!$managed_event instanceof ManagedEvent) {
				return;
			}
			
			$managed_event_member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
			
			if ($managed_event_member instanceof Member) {
				$ret = false;
				$controller->redirectTo('member', 'edit', 
					array('id' => $managed_event_member->getId(), 'modal' => array_var($_REQUEST, 'modal')));
			}
		}
	} else if ($controller instanceof EventController && $action == 'view') {
		$event = ProjectEvents::findById(get_id());
		if ($event instanceof ProjectEvent && $event->getColumnValue('is_managed_event')) {
			$managed_event = ManagedEvents::findOne(array("conditions" => array("event_id=?",$event->getId())));
			if ($managed_event instanceof ManagedEvent) {
				$dimension = Dimensions::findByCode('managed_events');
				$member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
				if ($member instanceof Member) {
					$ret = false;
					evt_add("external dimension member click", array('dim_id' => $member->getDimensionId(),'member_id' => $member->getId()));
				}
			}
		}
	}
}


function managed_events_override_event_edit_action($object, &$action) {
	if ($object instanceof ProjectEvent && $object->getColumnValue('is_managed_event')) {
		
		$dimension = Dimensions::findByCode('managed_events');
		
		$managed_event = ManagedEvents::instance()->findByEventId($object->getId());
		if (!$managed_event instanceof ManagedEvent) {
			return;
		}
			
		$managed_event_member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
		if ($managed_event_member instanceof Member) {
			$url = get_url('member', 'edit', array('id' => $managed_event_member->getId()));
			$action = "javascript:og.openLink('$url');";
		}
	}
}


function managed_events_override_calendar_views_view_action($params, &$action) {
	$object = array_var($params, 'object');
	$raw_url = array_var($params, 'raw_url');
	
	if ($object instanceof ProjectEvent || $object instanceof ProjectTask) {
		$action = "javascript:og.openLink('$raw_url');";
	}
}


// for managed events replace the edit function with the member edit
function managed_events_modify_object_edit_url($object, &$url) {
	
	if ($object instanceof ProjectEvent) {
		$dbrow = DB::executeOne("SELECT is_managed_event FROM ".TABLE_PREFIX."project_events WHERE object_id=".$object->getId());
		
		if ($dbrow && $dbrow['is_managed_event']) {
	
			$object->setColumnValue('is_managed_event', $dbrow['is_managed_event']);
			$dimension = Dimensions::findByCode('managed_events');
			
			$managed_event = ManagedEvents::instance()->findByEventId($object->getId());
			if (!$managed_event instanceof ManagedEvent) {
				return;
			}
			
			$managed_event_member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
			if ($managed_event_member instanceof Member) {
				$url = get_url('member', 'edit', array('id' => $managed_event_member->getId()));
			}
		}
	}
}


// additional fields in managed event member form, only if not using property groups
function managed_events_additional_member_data_fields($params, &$fields) {
	if (Plugins::instance()->isActivePlugin("advanced_core")) return;
	
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$ot = ObjectTypes::findByName('managed_event');

	if ($member instanceof Member && $ot->getId() == $member->getObjectTypeId()) {
		
		if ($member->isNew()) {
			$start_date = DateTimeValueLib::now();
			$start_date->setMinute(0);
			$start_date->setSecond(0);
			$start_date->add('h', 1);
			$start_date->advance(logged_user()->getUserTimezoneValue());
			
			$end_date = new DateTimeValue($start_date->getTimestamp());
			$end_date->add('h', 1);
			
			$timezone_id = logged_user()->getUserTimezoneId();
			
		} else {
			$obj = Objects::findObject($member->getObjectId());
			if ($obj instanceof ProjectEvent) {
				$start_date = $obj->getStart();
				$start_date->advance($obj->getTimezoneValue());
				
				$end_date = $obj->getDuration();
				$end_date->advance($obj->getTimezoneValue());
				
				$timezone_id = $obj->getTimezoneId();
			}
		}

		$html = '<div class="field"><table><tr><td>';
		$html .= pick_date_widget2("dim_obj[associated][event_id][start_date]", $start_date, $genid);
		$html .= '</td><td>';
		$html .= pick_time_widget2("dim_obj[associated][event_id][start_time]", $start_date, $genid);
		$html .= '</td></tr></table></div>';
		
		$fields[] = array('input_html' => $html, 'label' => lang("field ProjectEvents start"));

		$html = '<div class="field"><table><tr><td>';
 		$html .= pick_date_widget2("dim_obj[associated][event_id][duration_date]", $end_date, $genid);
		$html .= '</td><td>';
		$html .= pick_time_widget2("dim_obj[associated][event_id][duration_time]", $end_date, $genid);
		$html .= '</td></tr></table></div>';
		
		$fields[] = array('input_html' => $html, 'label' => lang("field ProjectEvents duration"));
		
		$html = '<div class="field">';
		$html .= timezone_selector("dim_obj[associated][event_id][timezone_id]", $timezone_id);
		$html .= '</div>';

		$fields[] = array('input_html' => $html, 'label' => lang("timezone"));

	}
}


/**
 * When cloning a managed event -> also clone the associated project event
 * @param array $params
 * @param ContentDataObject $copy
 */
function managed_events_after_content_data_object_copy($params, &$copy) {
	$object = array_var($params, 'object');
	
	if ($object instanceof ManagedEvent) {
		$event = ProjectEvents::findById($object->getEventId());
		if ($event instanceof ProjectEvent) {
			$new_event = $event->copy();
			if ($new_event instanceof ProjectEvent) {
				$copy->setEventId($new_event->getId());
				$copy->save();
			}
		}
	}
}



// adjust dimension object associated to the managed event member after saving it
function managed_events_after_add_dimension_object_member($params, &$ignored) {
	$member = array_var($params, 'member');
	$object = array_var($params, 'dimension_object');
	$dimension_obj_data = array_var($params, 'dimension_obj_data');
	$is_new = array_var($params, 'is_new');
	
	$ot = ObjectTypes::findByName('managed_event');

	if ($member instanceof Member && $ot->getId() == $member->getObjectTypeId() && $object instanceof ManagedEvent) {
		
		$associated_obj_data = array_var($dimension_obj_data, 'associated');
		
		$event = ProjectEvents::findById($object->getEventId());
		if (!$event instanceof ProjectEvent) {
			$event = new ProjectEvent();
		}
		
		$event->setColumnValue('is_managed_event', true);
		
		$tz_id = $associated_obj_data['event_id']['timezone_id'];
		$tz_offset = Timezones::getTimezoneOffset($tz_id);
		
		$start_date = getDateValue($associated_obj_data['event_id']['start_date']);
		if ($start_date instanceof DateTimeValue) {
			$start_time = getTimeValue($associated_obj_data['event_id']['start_time']);
			if ($start_time) {
				$start_date->setHour($start_time['hours']);
				$start_date->setMinute($start_time['mins']);
				$start_date->advance($tz_offset * -1);
			}
		}
		
		$end_date = getDateValue($associated_obj_data['event_id']['duration_date']);
		if ($end_date instanceof DateTimeValue) {
			$end_time = getTimeValue($associated_obj_data['event_id']['duration_time']);
			if ($end_time) {
				$end_date->setHour($end_time['hours']);
				$end_date->setMinute($end_time['mins']);
				$end_date->advance($tz_offset * -1);
			}
		}
		
		$is_all_day_event = false;
		if (!$start_date instanceof DateTimeValue || !$end_date instanceof DateTimeValue) {
			// one of the dates is not defined -> set as all_day_event if the corresponding time is not defined
			if ($start_date instanceof DateTimeValue && !isset($start_time)) {
				$is_all_day_event = true;
			}
			if ($end_date instanceof DateTimeValue && !isset($end_time)) {
				$is_all_day_event = true;
			}
		} else {
			// both dates are defined -> check if times are defined
			if (!isset($start_time) && !isset($end_time)) {
				// no dates defined -> set as all_day_event
				$is_all_day_event = true;
			} else {
				// one of the dates is defined -> set start or end as 00:00, end must always be greater than start
				if (!isset($end_time)) {
					// end time not set -> set end in the next day
					$end_date->beginningOfDay();
					while ($end_date->getTimestamp() < $start_date->getTimestamp()) {
						$end_date->add('d', 1);
					}
				} else if (!isset($start_time)) {
					// start time not set -> set as beginning of day
					$start_date->beginningOfDay();
				}
			}
		}

		$event->setTypeId($is_all_day_event ? 2 : 1);

		$event->setObjectName($object->getObjectName());
		$event->setStart($start_date);
		$event->setDuration($end_date);
		$event->setTimezoneId($tz_id);
		$event->setTimezoneValue($tz_offset);
		$event->setDescription($member->getDescription());
		
		$event->save();
		
		$event->addToMembers(array($member));
		
		$object->setEventId($event->getId());
		$object->save();
		
		if ($is_new) {
			$inv = EventInvitations::instance()->findByEventAndContact($event->getId(), logged_user()->getId());
			if (!$inv instanceof EventInvitation) {
				$inv = new EventInvitation();
				$inv->setEventId($event->getId());
				$inv->setContactId(logged_user()->getId());
				$inv->save();
			}
		}
	}
}


