<?php
/**
 * Member Templates controller
 *
 * @author Feng Office
 */
class MemberTemplatesController extends ApplicationController {

	var $plugin_name = "member_templates";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	}
	

	
	function add() {
		
		if (!logged_user()->isAdminGroup()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$data = array_var($_POST, 'template');
		
		if (!is_array($data)) {
			
			$dimension_id = array_var($_REQUEST, 'dimension_id');
			$object_type = ObjectTypes::findById(array_var($_REQUEST, 'object_type_id'));
			
			$dimension_associations = DimensionMemberAssociations::getAssociatations($dimension_id, $object_type->getId());
			
			tpl_assign('dimension_id', $dimension_id);
			tpl_assign('object_type', $object_type);
			tpl_assign('dimension_associations', $dimension_associations);
			tpl_assign('member_template', new MemberTemplate());
			tpl_assign('selected_member_ids', array());
			tpl_assign('steps_info', array());
			
		} else {
			
			try {
				DB::beginWork();
				
				$member_template = new MemberTemplate();
				$this->save_member_template($data, $member_template);
			
				DB::commit();
				
			} catch (Exception $e) {
				DB::rollback();
				ajx_current("empty");
				flash_error($e->getMessage());
				return;
			}
			
			flash_success(lang('success save template'));
			ajx_current("back");
		}
		
	}
	
	
	function edit() {
		$this->setTemplate('add');
		if (!logged_user()->isAdminGroup()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$member_template = MemberTemplates::findById(get_id());
		if (!$member_template instanceof MemberTemplate) {
			flash_error(lang('object dnx'));
			ajx_current('empty');
			return;
		}
	
		$data = array_var($_POST, 'template');
	
		if (!is_array($data)) {
			
			$dimension_id = $member_template->getDimensionId();
			$object_type = ObjectTypes::findById($member_template->getObjectTypeId());
			
			$dimension_associations = DimensionMemberAssociations::getAssociatations($dimension_id, $object_type->getId());
			
			$selected_member_ids = array();
			$selected_members = MemberTemplatesInitialAssociations::findAll(array('conditions' => 'template_id='.$member_template->getId()));
			foreach ($selected_members as $sm) $selected_member_ids[] = $sm->getAssociatedMemberId();
			
			sort($selected_member_ids, SORT_NUMERIC);
			$task_template_ids = MemberTemplatesTaskTemplates::getTemplateIdsToInstantiate($member_template->getId(), $selected_member_ids);
			
			$task_templates_info = array();
			if (is_array($task_template_ids)) {
				foreach ($task_template_ids as $k => &$id) {
					if (!is_numeric($id)) unset($task_template_ids[$k]);
				}
				if (count($task_template_ids) > 0) {
					$task_templates = COTemplates::findAll(array('conditions' => 'id IN ('.implode(',',$task_template_ids).')'));
					foreach ($task_templates as $tt) {
						$task_templates_info[] = array('id' => $tt->getId(), 'name' => $tt->getObjectName());
					}
				}
			}
			
			$steps_info = MemberTemplatesTaskTemplates::getWorkflowStepsInfo($member_template->getId());
			
			tpl_assign('dimension_id', $dimension_id);
			tpl_assign('object_type', $object_type);
			tpl_assign('member_template', $member_template);
			tpl_assign('dimension_associations', $dimension_associations);
			tpl_assign('task_templates_info', $task_templates_info);
			tpl_assign('selected_member_ids', $selected_member_ids);
			tpl_assign('steps_info', $steps_info);
			
		} else {
				
			try {
				DB::beginWork();	
				
				$this->save_member_template($data, $member_template);
					
				DB::commit();
	
			} catch (Exception $e) {
				DB::rollback();
				ajx_current("empty");
				flash_error($e->getMessage());
				return;
			}
				
			flash_success(lang('success save template'));
			ajx_current("back");
		}
	
	}
	
	private function save_member_template($data, MemberTemplate $member_template) {
		
		$member_template->setFromAttributes($data);
		if (!isset($data['can_instantiate_from_mail'])) $member_template->setCanInstantiateFromMail(false);
		
		if (trim($member_template->getName()) == "") {
			throw new Exception(lang('template name required'));
		}
		$member_template->save();
		
		// save initial member associations
		$initial_associated_members = array_filter(array_var($data, 'initial_associated_members'));
		if (count($initial_associated_members) == 0) {
			MemberTemplatesInitialAssociations::delete("template_id=".$member_template->getId());
		}
		
		$associated_member_ids = array();
		foreach ($initial_associated_members as $assoc_id => $mid_json) {
			$mid = 0;
			$mids = json_decode($mid_json);
			if (is_array($mids) && count($mids) > 0) $mid = $mids[0];
			else continue;
			
			MemberTemplatesInitialAssociations::delete("template_id=".$member_template->getId()." AND dim_association_id='$assoc_id'");

			$mtia = new MemberTemplatesInitialAssociation();
			$mtia->setTemplateId($member_template->getId());
			$mtia->setAssociatedMemberId($mid);
			$mtia->setDimAssociationId($assoc_id);
			$mtia->save();
			
			foreach ($mids as $assoc_mid) {
				if (!in_array($assoc_mid, $associated_member_ids)) $associated_member_ids[] = $assoc_mid;
			}
		}
		
		// save initial task tempaltes
		MemberTemplatesTaskTemplates::delete('template_id = '. $member_template->getId() . ' AND is_initial=1');
		sort($associated_member_ids, SORT_NUMERIC);
		
		// get the task template ids to save
		$task_templates = array_var($data, 'initial_task_templates');
		$task_template_ids = array();
		foreach ($task_templates as $tt_id) {
			if (is_numeric($tt_id)) $task_template_ids[] = $tt_id;
		}
		
		// get the generic template ids to save
		$cobj_templates = array_var($data, 'initial_cobj_templates');
		$cobj_template_ids = array();
		foreach ($cobj_templates as $tt_id) {
			if (is_numeric($tt_id)) $cobj_template_ids[] = $tt_id;
		}

		// save initial record with the associated members, task template ids and the generic template ids
		if (count($task_template_ids) + count($cobj_template_ids)> 0) {
			sort($task_template_ids, SORT_NUMERIC);
			DB::execute("INSERT INTO ".TABLE_PREFIX."member_templates_task_templates (template_id, associated_member_ids, task_template_ids, cobj_template_ids, is_initial) 
				VALUES ('".$member_template->getId()."','".implode(',', $associated_member_ids)."','".implode(',', $task_template_ids)."','".implode(',', $cobj_template_ids)."',1)
				ON DUPLICATE KEY UPDATE template_id=template_id");
			
		}
		
		
		// save workflow steps
		$steps = array_var($data, 'steps');
		if (!is_array($steps)) $steps = array();
		
		MemberTemplatesTaskTemplates::delete('template_id = '. $member_template->getId() . ' AND is_initial=0');
		
		foreach ($steps as $step_info) {
			
			// step member associations
			$members = array_var($step_info, 'members');
			$all_member_ids = array();
			foreach ($members as $assoc_id => $assoc_member_ids) {
				$json = array_var($assoc_member_ids, $assoc_id);
				$member_ids = json_decode($json);
				$all_member_ids = array_merge($all_member_ids, $member_ids);
			}
			sort($all_member_ids, SORT_NUMERIC);
			$all_member_ids_str = implode(',', $all_member_ids);
			
			// templates to instantiate
			$task_template_ids_str = implode(',', array_var($step_info, 'task_templates'));
			
			// save step
			DB::execute("INSERT INTO ".TABLE_PREFIX."member_templates_task_templates (template_id, associated_member_ids, task_template_ids, is_initial)
				VALUES ('".$member_template->getId()."','$all_member_ids_str','$task_template_ids_str',0)
				ON DUPLICATE KEY UPDATE template_id=template_id");

		}
	}
	
	
	function delete() {
		
		if (!logged_user()->isAdminGroup()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$member_template = MemberTemplates::findById(get_id());
		if (!$member_template instanceof MemberTemplate) {
			flash_error(lang('object dnx'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			
			$temp_name = $member_template->getName();
			$member_template->delete();
			
			DB::commit();
			
			flash_success(lang('success delete template', $temp_name));
			ajx_current("reload");
			
		} catch (Exception $e) {
			DB::rollback();
			ajx_current("empty");
			flash_error($e->getMessage());
		}
	}
	
	
	
	function render_single_member_selector_for_template() {
		
		$dim_id = array_var($_REQUEST, 'dim_id');
		$hf_name = array_var($_REQUEST, 'hf_name');
		$dim_assoc_id = array_var($_REQUEST, 'dim_assoc_id');
		$object_type_id = array_var($_REQUEST, 'object_type_id');
		$selected_member_ids = explode(',', array_var($_REQUEST, 'selected_member_ids', ""));
		
		if (is_numeric($dim_id)) {
			$dim = Dimensions::findById($dim_id);
			tpl_assign('dim', $dim);
		} else {
			$dim_id = 0;
		}
		
		tpl_assign('hf_name', $hf_name);
		tpl_assign('dim_assoc_id', $dim_assoc_id);
		tpl_assign('object_type_id', $object_type_id);
		
		foreach ($selected_member_ids as $k => &$v) {
			if (!is_numeric($v)) unset($selected_member_ids[$k]);
		}
		if (count($selected_member_ids) > 0) {
			$sql = "SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.id IN (".implode(',', $selected_member_ids).") AND m.dimension_id=$dim_id";
			$clean_sel_member_ids = array_flat(DB::executeAll($sql));
			tpl_assign('selected_member_ids', $clean_sel_member_ids);
		} else {
			tpl_assign('selected_member_ids', array());
		}
	}

	
	function enable_disable_default_add_action() {
		ajx_current('empty');
		
		if (!logged_user()->isAdminGroup()) {
			flash_error(lang('no access permissions'));
			return;
		}
		
		$disable = array_var($_REQUEST, 'disable');
		$dim_id = array_var($_REQUEST, 'dim_id');
		$ot_id = array_var($_REQUEST, 'ot_id');
		if ($dim_id == 0 || $ot_id == 0) return;
		
		$disabled_ids_csv = config_option('disabled_default_add_dimensions');
		$disabled_ids = explode(',', $disabled_ids_csv);
		foreach ($disabled_ids as $k => &$d) {
			if (!$d) unset($disabled_ids[$k]);
		}
		
		$key = $dim_id."_".$ot_id;
		
		if ($disable) {
			if (!in_array($key, $disabled_ids)) {
				$disabled_ids[] = $key;
				set_config_option('disabled_default_add_dimensions', implode(',', $disabled_ids));
			}
		} else {
			if (in_array($key, $disabled_ids)) {
				foreach ($disabled_ids as $k => &$v) {
					if ($v == $key) unset($disabled_ids[$k]);
				}
				set_config_option('disabled_default_add_dimensions', implode(',', $disabled_ids));
			}
		}
	}
	
	
	
	
	function instantiate_multiple_templates() {

		$parameter_values = array_var($_REQUEST, 'parameter_values');
		if (!$parameter_values) {

			$data = json_decode(array_var($_REQUEST, 'data', ''), true);

			// group parameters by template
			$grouped_by_template = array();

			// member ids
			$member_ids = array_var($data, 'member_ids');

			// get parameters for each task template
			$task_templates = array_var($data, 'task_templates_with_parameters');
			foreach ($task_templates as $template_id) {
				$template = COTemplates::findById($template_id);
				$parameters = TemplateParameters::getParametersByTemplate($template_id);

				$grouped_by_template[] = array(
					'template' => $template,
					'type' => 'task',
					'parameters' => $parameters,
				);
			}

			// get parameters for each generic template
			$cobj_templates = array_var($data, 'generic_templates_with_parameters');
			foreach ($cobj_templates as $template_id) {
				$template = CObjTemplates::findById($template_id);
				$parameters = CObjTemplateParameters::instance()->getTemplateParameters($template_id);

				$grouped_by_template[] = array(
					'template' => $template,
					'type' => 'generic',
					'parameters' => $parameters,
				);
			}
			
			// assign variables to form
			tpl_assign('grouped_by_template', $grouped_by_template);
			tpl_assign('member_ids', $member_ids);

		} else {

			$member_ids = json_decode(array_var($_REQUEST, 'member_ids', '[]'));
			$template_types = array_var($_REQUEST, 'template_types');

			try {
				DB::beginWork();

				// instantiate each template
				foreach ($parameter_values as $template_id => $param_values) {

					$type = $template_types[$template_id];

					if ($type == 'task') {

						$template_controller = new TemplateController();
						$template_controller->instantiate(array('id' => $template_id, 'members' => $member_ids, 'parameterValues' => $param_values));

					} else if ($type == 'generic') {

						$cobj_template_controller = new CObjTemplateController();
						$cobj_template_controller->instantiate($template_id, $param_values, $member_ids);

					}

				}

				DB::commit();

				ajx_current('reload');
				flash_success(lang('templates successfully instantiated'));

			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
			

		}

		
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
}