<?php
Hook::register("member_templates");

function member_templates_object_definition($type, &$ret){
	if ($type == 'Member') {
		$ret['template_id'] = DATA_TYPE_INTEGER;
		$ret['from_mail_id'] = DATA_TYPE_INTEGER;
	}
}

function member_templates_additional_member_hidden_fields ($member, &$fields) {
	$fields['template_id'] = array_var($_REQUEST, 'template_id', $member->getColumnValue('template_id'));
	$fields['from_mail_id'] = array_var($_REQUEST, 'from_mail_id', $member->getColumnValue('from_mail_id'));
}

function member_templates_member_quick_add_urls($parameters, &$urls) {
	
	$dim_object_types = array_var($parameters, 'object_types');
	$dimension = array_var($parameters, 'dimension');
	$parent_member = array_var($parameters, 'parent_member');
	
	// check if "default add" menu option has to be removed
	$disabled_ids_csv = config_option('disabled_default_add_dimensions');
	
	$disabled_ids = explode(',', $disabled_ids_csv);
	$tmp_urls = array();
	foreach ($urls as $k => &$u) {
		$dec = parse_url($u['url']);
		$query_params = explode('&', $dec['query']);
		
		$type_id = "";
		foreach ($query_params as $qp) {
			if (str_starts_with($qp, 'type')) {
				$qp_decoded = explode('=', $qp);
				$type_id = array_var($qp_decoded, 1);
			}
		}
		if ($type_id != "") {
			$key = $dimension->getId() . "_" . $type_id;
			$template_count = MemberTemplates::count("dimension_id=".$dimension->getId()." AND object_type_id=$type_id");
			if (!in_array($key, $disabled_ids) || $template_count==0) {
				$tmp_urls[] = $u;
			}
		}
	}
	$urls = $tmp_urls;
	
	foreach ($dim_object_types as $dim_ot) {
		$ot = ObjectTypes::findById($dim_ot->getObjectTypeId());
		
		// exclude client template links of crpm dimension if separate clients dimension is enabled 
		$customers_dim = null;
		if (Plugins::instance()->isActivePlugin('crpm')) {
			Env::useHelper('functions', 'crpm');
			$customers_dim = get_customers_dimension();
		}
		$crpm_dim = Dimensions::findByCode('customer_project');
		$customer_ot = ObjectTypes::findByName('customer');
		
		if ($crpm_dim instanceof Dimension && $customers_dim instanceof Dimension && $customers_dim->getId() != $crpm_dim->getId()
				&& $dimension->getId() == $crpm_dim->getId() && $ot->getId() == $customer_ot->getId()) continue;
		// ---
		
		$template_urls = array();
		
		$templates = MemberTemplates::findAll(array('conditions' => array('dimension_id=? AND object_type_id=?', $dimension->getId(), $ot->getId())));
		foreach ($templates as $t) {/* @var $t MemberTemplate */
			
			$class_name = ucfirst($t->getName())."Controller";
			if ($ot && controller_exists($ot->getName(), $ot->getPluginId())) {
				$params = array("type" => $ot->getId(), 'template_id' => $t->getId(), "dim_id" => $dimension->getId());
				if ($parent_member instanceof Member) $params['parent'] = $parent_member->getId();
				if ($t->getObjectSubtypeId() > 0) $params['object_subtype_id'] = $t->getObjectSubtypeId();
			
				$url = get_url($ot->getName(), 'add', $params);
			} else {
				$params = array("dim_id" => $dimension->getId(), "type" => $ot->getId(), 'template_id' => $t->getId());
				if ($parent_member instanceof Member) $params['parent'] = $parent_member->getId();
				if ($t->getObjectSubtypeId() > 0) $params['object_subtype_id'] = $t->getObjectSubtypeId();
			
				$url = get_url('member', 'add' , $params);
			}
			
			$template_urls[] = array('link_text' => $t->getName(), 'url' => $url, 'iconcls' => $ot->getIconClass());
		}
		
		// sort the templates of current dimension object type
		$urls_sorted = array();
		foreach ($template_urls as $url) {
			$urls_sorted[$url['link_text']] = $url;
		}
		ksort($urls_sorted);
		$template_urls = array_values($urls_sorted);
		
		// append the sorted template urls to the output variable
		$urls = array_merge($urls, $template_urls);
	}
}


function member_templates_after_member_association_changed($parameters, &$ignored) {

	// for projects this event is triggered twice, but we want to execute this function only once
	if (array_var($_REQUEST, 'template_already_executed')) return;

	$member = array_var($parameters, 'member');
	$is_new = array_var($parameters, 'is_new');
	
    //Lets find if there is a template associated for this member
	$member_template = MemberTemplates::findById($member->getColumnValue('template_id'));
	
	//If there is one...
	if ($member_template instanceof MemberTemplate) {

		// for projects this event is triggered twice, but we want to execute this function only once
		$_REQUEST['template_already_executed'] = true;
	
		$associated_member_ids = array();
		$mpms = MemberPropertyMembers::getAssociatedPropertiesForMember($member->getId());
		foreach ($mpms as $mpm) {
			/* @var $mpm MemberPropertyMember */
			$prop_mem = Members::findById($mpm->getPropertyMemberId());
			if ($prop_mem instanceof Member && $prop_mem->getDimension()->getCode() != 'feng_persons') {
				$associated_member_ids[] = $mpm->getPropertyMemberId();
			}
		}
	
		// get all task templates to instantiate
		$task_template_ids = MemberTemplatesTaskTemplates::getTemplateIdsToInstantiate($member_template->getId(), $associated_member_ids, $is_new);
		if (!$task_template_ids) $task_template_ids = array();
		
		// get generic object templates to instantiate
		$cobj_template_ids = array();
		if ($is_new && Plugins::instance()->isActivePlugin('advanced_core')) {
			$reg = MemberTemplatesTaskTemplates::findOne(array('conditions' => "template_id = '".$member_template->getId()."' AND is_initial = 1 "));
			if ($reg) {
				$cobj_template_ids_csv = $reg->getCobjTemplateIds();
				if (trim($cobj_template_ids_csv) != '') {
					$cobj_template_ids = explode(',', $cobj_template_ids_csv);
				}
			}
		}
	
		if (count($task_template_ids) + count($cobj_template_ids) > 0) {
			
			// Before instantiating any task template, ensure that the permissions are saved, to avoid permission errors when assignint tasks
			Env::useHelper('permissions');
			$permissions_string = array_var($parameters['request'], 'permissions');
			if (!$permissions_string) {
				$permission_parameters = permission_member_form_parameters(null, $member->getDimensionId());
				$logged_user_pg = array();
				foreach ($permission_parameters['allowed_object_types'] as $ot){
					$logged_user_pg[] = array('o' => $ot->getId(), 'w' => 1, 'd' => can_manage_dimension_members(logged_user()) ? 1 : 0, 'r' => 1);
				}
				$permission_parameters['member_permissions'][logged_user()->getPermissionGroupId()] = $logged_user_pg;
				
				$permissions_parameters = get_default_member_permission($member->getParentMemberId(), $permission_parameters);
				
				// reformat the default permissions to simulate the permissions string sent in the request
				$permissions_object = $permissions_parameters['member_permissions'];
				foreach ($permissions_object as $pg_id => &$perm_array) {
					foreach ($perm_array as &$perm) {
						$perm['pg'] = $pg_id;
					}
				}
				$permissions_object_plain = array();
				foreach ($permissions_object as $perm_array) {
					$permissions_object_plain = array_merge($permissions_object_plain, $perm_array);
				}
				if ($permissions_object_plain) {
					$permissions_string = json_encode($permissions_object_plain);
				}
			}
			if ($permissions_string) {
				save_member_permissions($member, $permissions_string);
			}
			// -----------------

			$task_templates_with_parameters = array();
			$generic_templates_with_parameters = array();
	
			$member_ids = array($member->getId());
			$member_ids = array_merge($member_ids, $associated_member_ids);
			
			// Now we can instantiate the task templates
			$template_controller = new TemplateController();
			foreach ($task_template_ids as $template_id) {
	
				if (!is_numeric($template_id) || $template_id == 0) continue;
				
				if (!MemberTaskTemplatesInstantiationLogs::logExists($member->getId(), $template_id)) {
					// instantiate template
					$task_template_parameters = TemplateParameters::getParametersByTemplate($template_id);
					if (count($task_template_parameters) == 0) {
						$template_controller->instantiate(array('id' => $template_id, 'members' => $member_ids, 'parameterValues' => array()));
					} else {
						//evt_add('instantiate_template_parameters', array('id' => $template_id, 'member_id' => $member->getId()));
						$task_templates_with_parameters[] = $template_id;
					}
				}
				
				// to prevent another instantiation if status changes and comes to the same status
				MemberTaskTemplatesInstantiationLogs::createLog($member->getId(), $template_id);
			}


			if (Plugins::instance()->isActivePlugin('advanced_core')) {

				$cobj_template_controller = new CObjTemplateController();

				foreach ($cobj_template_ids as $cobj_template_id) {

					if (!MemberCobjTemplatesInstantiationLogs::logExists($member->getId(), $cobj_template_id)) {

						$cobj_template = CObjTemplates::findById($cobj_template_id);
						$parameters = CObjTemplateParameters::instance()->getTemplateParameters($cobj_template_id);

						if (is_array($parameters) && count($parameters) > 0) {
							
							$generic_templates_with_parameters[] = $cobj_template_id;

						} else {

							$cobj_template_controller->instantiate($cobj_template->getId(), array(), $member_ids);

						}
						
						// to prevent another instantiation if status changes and comes to the same status
						MemberCobjTemplatesInstantiationLogs::createLog($member->getId(), $cobj_template_id);
					}
				}
			}


			// trigger event to create a form with parameters from all templates 
			// and after user fills them all we instantiate all pending templates
			if (count($task_templates_with_parameters) + count($generic_templates_with_parameters) > 0) {
				evt_add("instantiate_multiple_templates", array(
					'member_ids' => $member_ids,
					'task_templates_with_parameters' => $task_templates_with_parameters, 
					'generic_templates_with_parameters' => $generic_templates_with_parameters,
				));
			}
		}
	}
}




function member_templates_render_more_type_templates($parameters, &$ret) {
	$customers_dim = null;
	if (Plugins::instance()->isActivePlugin('crpm')) {
		Env::useHelper('functions', 'crpm');
		$customers_dim = get_customers_dimension();
	}
	$crpm_dim = Dimensions::findByCode('customer_project');
	$customer_ot = ObjectTypes::findByName('customer');
	
	$available_types = MemberTemplatesAvailableTypes::findAll();
	foreach ($available_types as $type) {
		
	    // if separated clients dimension is enabled then exclude client templates of crpm dimension
		if ($crpm_dim instanceof Dimension && $customers_dim instanceof Dimension && $customers_dim->getId() != $crpm_dim->getId()
				&& $type->getDimensionId() == $crpm_dim->getId() && $type->getObjectTypeId() == $customer_ot->getId()) {
			continue;
		}
		
		// do not include templates of disabled dimensions
		if (!in_array($type->getDimensionId(), config_option('enabled_dimensions'))) {
			continue;
		}
		
		$dot = DimensionObjectTypes::findOne(array('conditions' => 'dimension_id='.$type->getDimensionId().' AND object_type_id='.$type->getObjectTypeId()));
		if (!$dot instanceof DimensionObjectType || $dot->getEnabled() == 0) {
			continue;
		}
		
		tpl_assign('dimension_id', $type->getDimensionId());
		tpl_assign('object_type_id', $type->getObjectTypeId());
		tpl_display(get_template_path('list_by_type', 'member_templates', 'member_templates'));
		
	}
}



function member_templates_additional_email_actions($parameters, &$ret) {
	$email = array_var($parameters, 'email');

	// get all member templates marked with can_instantiate_from_mail
	$member_templates = MemberTemplates::findAll(array('conditions' => "can_instantiate_from_mail = 1"));
	foreach ($member_templates as $template) {/* @var $template MemberTemplate */
		
		$ot = ObjectTypes::findById($template->getObjectTypeId());
		
		// get template url controller and params
		if ($ot && controller_exists($ot->getName(), $ot->getPluginId())) {
			$params = array("type" => $ot->getId(), 'template_id' => $template->getId(), "dim_id" => $template->getDimensionId());
			$controller = $ot->getName();
		} else {
			$params = array("dim_id" => $template->getDimensionId(), "type" => $ot->getId(), 'template_id' => $template->getId());
			$controller = 'member';
		}
		
		// add email parameters
		$params['name'] = $email->getFromName();
		$params['email'] = $email->getFrom();
		$params['from_mail_id'] = $email->getId();
		
		// build add template url
		$url = get_url($controller, 'add', $params);
		
		// add action to the panel
		add_page_action($template->getName(), "javascript:og.openLink('$url');", 'ico-template', null, null, true);
	}
	
}

function member_templates_after_add_member($member, &$ret) {
	// When instantiating a member template from an email => classify the email and create an email rule.
	if (Plugins::instance()->isActivePlugin('mail') && $member instanceof Member 
			&& $member->getColumnValue('template_id') > 0 && $member->getColumnValue('from_mail_id') > 0) {
		
		$mail = MailContents::findById($member->getColumnValue('from_mail_id'));
		if ($mail instanceof MailContent) {
			// classify mail in the new member
			$mail->addToMembers(array($member));
			$mail->addToSharingTable();	
		}
	
	}
}

function member_templates_member_edit_categories($params, &$categories) {
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	
	if ($member instanceof Member && !$member->isNew() && $member->getColumnValue('template_id') <= 0) {
		Env::useHelper('functions', 'member_templates');
		
		$templates = MemberTemplates::findAll(array(
			"conditions" => array("dimension_id=? AND object_type_id=?", $member->getDimensionId(), $member->getObjectTypeId())
		));
		
		if (is_array($templates) && count($templates) > 0) {
			$categories[] = array(
				'id' => $genid . 'member_templates',
				'link_id' => $genid . 'member_template_tab',
				'name' => lang("select template"),
				'content' => get_member_template_tab_html($genid, $member, $templates),
				'class' => 'templates-container',
			);
		}
	}

}




