<?php

/**
 * Class of utilities for Microsoft Calendar
 *
 * @author Pablo Silvera <pablo.silvera@gmail.com>
 */
class MicrosoftUtilities {

	/**
	 * Return type pattern if is forever or there is end date
	 *
	 * @param object type ProjectEvent
	 * @return string
	 */
	static function getRangeType(&$object) {

		$type = ($object->getRepeatForever() == 1) ? 'noEnd' : 'endDate';
		return $type;
	}

	/**
	 * Return array with format for the recurrence
	 *
	 * @param object type ProjectEvent
	 * @param object string START-DATE with TimeZone applied
	 * @return array
	 */
	static function buildRange(&$object, &$start_dt) {

		$arrayRange = [
			'type' => self::getRangeType($object),
			'startDate' => $start_dt->format('Y-m-d'),
		];

		if (!empty($object->getRepeatEnd())) {
			$arrayRange['endDate'] = $object->getRepeatEnd()->format('Y-m-d');
		}

		return $arrayRange;
	}

	/**
	 * Return the before minutes because MS-Calendar not accept reminders to after start of event
	 * MS-Calendar accept just one reminder by event
	 * So, is returned the first element of reminders list
	 *
	 * @param object type ProjectEvent
	 * @return mixed boolean or minutes of reminder
	 */
	static function getMinutesOfReminder(&$object) {

		$reminders = ObjectReminders::instance()->getByObject($object);

		return (empty($reminders)) ? false : $reminders[0]->getMinutesBefore();
	}

}