<?php
Hook::register("microsoft_integrations");

Env::useLibrary('microsoft_graph', 'microsoft_integrations');
Env::useHelper('microsoft_functions', 'microsoft_integrations');


function microsoft_integrations_add_feature_to_sub_categories_list($params, &$ret){
	$config_category = array_var($params, 'config_category');
	$config_name = $config_category instanceof ContactConfigCategory ? $config_category->getName() : '';
	if($config_name == 'microsoft_365'){

		$accessToken = get_ms_access_token(logged_user()->getId());
		//$_SESSION['microsoftAccessToken'] = null;

		$token_not_valid = !isset($accessToken) || !is_valid_microsoft_access_token(logged_user()->getId(), $accessToken);

		// if token is valid, try to get the user information, if not then delete the token and tell the user to login again
		if (!$token_not_valid) {

			try {

				$graph = new Microsoft\Graph\Graph();
				$graph->setAccessToken($accessToken->getToken());
	
				// get ms user data
				$user = $graph->createRequest('GET', '/me?$select=displayName,mail,mailboxSettings,userPrincipalName')
					->execute();

			} catch (Exception $e) {
				// if any problem occured when getting user info then delete token and ask the user to login again
				$token_not_valid = true;
				delete_ms_access_token(logged_user()->getId());
			}
		}

		if ($token_not_valid) {
			// link to create the connection to an ms account
			$ret .= '<a style="padding-left: 15px;" href="'.get_url('microsoft', 'connect_to_microsoft', array('controller'=>'microsoft', 'action' => 'reload_current_panel', 'id'=>'dummy')).'">Connect to Microsoft 365</a>';
		} else {
			// here the user is connected -> display some info (email, etc.) and provide a disconnection link
			$user_array = $user->getBody();
			
			$disconnect_link = '<a href="#" onclick="og.openLink(og.getUrl(\'microsoft\',\'disconnect\'));">Disconnect</a>';

			$ret .= '<p style="padding-left: 15px;">'.'<span style="color: rgb(12, 190, 155); font-size: 20px;">&#10003;</span> Microsoft account <span style="font-weight: bold;">'.$user_array['displayName']." (".$user_array['userPrincipalName'].')</span> is connected. ('.$disconnect_link.')</p>';
			
			$ret .= '<br /><a style="padding-left: 15px;" href="'.get_url('microsoft', 'update_webhooks').'">Refresh webhooks</a>';
			$ret .= '<br /><a style="padding-left: 15px;" href="'.get_url('microsoft', 'delete_all_webhooks').'">Delete webhooks</a>';
		}

	}
}

function microsoft_integrations_after_add_to_members($object, &$member_ids) {
	
	if ($object instanceof ProjectEvent) {
		
		save_feng_event_in_ms_calendar($object, logged_user()->getId());
	
	}
}

function microsoft_integrations_after_event_change_duration($params, &$ignored) {
	$object = array_var($params, 'event');
	
	if ($object instanceof ProjectEvent) {
		
		save_feng_event_in_ms_calendar($object, logged_user()->getId());
		
	}
}

function microsoft_integrations_after_event_move($params, &$ignored) {
	$object = array_var($params, 'event');
	
	if ($object instanceof ProjectEvent) {
		
		save_feng_event_in_ms_calendar($object, logged_user()->getId());
		
	}
}

function microsoft_integrations_after_content_object_trash($params, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'object');
	
	if ($object instanceof ProjectEvent) {
		$ms_event_id = MsCalendarObjects::getMsIdByUserAndObject(logged_user()->getId(), $object->getId());
		if (isset($ms_event_id)) {
			delete_feng_event_in_ms_calendar(logged_user()->getId(), $ms_event_id);
		}
	}
	
}

function microsoft_integrations_after_object_controller_trash($ids, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	if (!is_array($ids)) {
		$ids = explode(',', $ids);
	}
	$ids = array_filter($ids, 'is_numeric');
	foreach ($ids as $id) {
		$object = Objects::findObject($id);
		if ($object instanceof ProjectEvent) {
			$ms_event_id = MsCalendarObjects::getMsIdByUserAndObject(logged_user()->getId(), $object->getId());
			if (isset($ms_event_id)) {
				delete_feng_event_in_ms_calendar(logged_user()->getId(), $ms_event_id);
			}
		}
	}
}

function microsoft_integrations_after_content_object_untrash($params, $ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'object');
	
	if ($object instanceof ProjectEvent) {
		save_feng_event_in_ms_calendar($object, logged_user()->getId());
	}
}

function microsoft_integrations_show_reminder_note($params, &$note) {
	$note = "<div>".lang('object reminders note')."<br><br></div>";
}
