<?php

/**
 * ExchangeRate class
 *
 * 
 */
class ExchangeRate extends BaseExchangeRate {
	
	var $source_currency = null;
	var $destiny_currency = null;
	
	function getSourceCurrency() {
		if (!$this->source_currency) {
			$this->source_currency = Currencies::instance()->getCurrency($this->getSourceCurrencyId());
		}
		return $this->source_currency;	
	}
	
	function getDestinyCurrency() {
		if (!$this->destiny_currency) {
			$this->destiny_currency = Currencies::instance()->getCurrency($this->getDestinyCurrencyId());
		}
		return $this->destiny_currency;	
	}
	
	
	function getArrayInfo() {
		return array(
			'id' => $this->getId(),
			'source_currency_id' => $this->getSourceCurrencyId(),
			'source_currency_name' => $this->getSourceCurrency() instanceof Currency ? $this->getSourceCurrency()->getName() : '',
			'destiny_currency_id' => $this->getDestinyCurrencyId(),
			'destiny_currency_name' => $this->getDestinyCurrency() instanceof Currency ? $this->getDestinyCurrency()->getName() : '',
			'date' => format_date($this->getDate(), null, 0),
			'timestamp' => $this->getDate()->getTimestamp(),
			'rate' => $this->getRate()
		);
	}
	
	function getEditUrl() {
		return get_url('exchange_rates', 'edit', array('id' => $this->getId()));
	}
	
	function getDeleteUrl() {
		return get_url('exchange_rates', 'delete', array('id' => $this->getId()));
	}
	
	
	
	
	
	function validate($errors) {
	
		$source_currency_id = $this->getSourceCurrencyId();
		$destiny_currency_id = $this->getDestinyCurrencyId();
	
		if (!is_numeric($this->getSourceCurrencyId())) {
			$errors[] = lang('source currency required');
		}
		if (!is_numeric($this->getDestinyCurrencyId())) {
			$errors[] = lang('destiny currency required');
		}
	
		$date = $this->getDate();
	
		if (!$date instanceof DateTimeValue) {
			
			$errors[] = lang('date required');
				
		} else {
			// Check if date already has exchange rate for the source and destiny currencies
			$date_sql_begin = $date->beginningOfDay()->toMySQL();
			$date_sql_end = $date->endOfDay()->toMySQL();
				
			if (ExchangeRates::count("id<>'".$this->getId()."' AND date >= '$date_sql_begin' AND date <= '$date_sql_end' AND source_currency_id = '$source_currency_id' AND destiny_currency_id = '$destiny_currency_id'")) {
				$errors[] = lang('already exists exchange rate for the selected date');
			}
			
			if (ExchangeRates::count("id<>'".$this->getId()."' AND date >= '$date_sql_begin' AND date <= '$date_sql_end' AND source_currency_id = '$destiny_currency_id' AND destiny_currency_id = '$source_currency_id'")) {
				$errors[] = lang('already exists exchange rate for the selected date');
			}
		}
	
		if (!trim($this->getRate())) {
			$errors[] = lang('amount required');
		} elseif (!is_numeric($this->getRate())) {
			$errors[] = lang('invalid amount');
		} elseif ($this->getRate() <= 0) {
			$errors[] = lang('amount must be greater than zero');
		}
		
	}
} 