<?php
/**
 * Newsletter controller
 *
 * @version 1.0
 */
class NewsletterController extends ApplicationController {

	var $plugin_name = "newsletters";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("NewsletterManager.js", $this->plugin_name);
		ajx_current("panel", "newsletters-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	// put contact_ids selected with ObjectSelector in the newsletter contact list
	function save_newsletter_contacts($newsletter=null, $genid=null) {
		ajx_current("empty");
		Env::useHelper("object_selector");
		
		if (is_null($newsletter)) {
			$newsletter = Newsletters::findById(get_id());
		}
		if (is_null($genid)) {
			$genid = array_var($_REQUEST, 'genid');
		}
		
		if (!$newsletter instanceof Newsletter) {
			Logger::log("Newsletter does not exists: ".get_id());
			return;
		}
		if (logged_user()->isGuest() || !$newsletter->canEdit(logged_user())) {
			Logger::log("User ".logged_user()->getId()." cannot edit newsletter: ".$newsletter->getId());
			return;
		}
		
		try {
			// get selected contacts
			$identifier = get_selector_identifier($genid);
			
			$row = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."object_selector_temp_values 
					WHERE user_id=".logged_user()->getId()." AND identifier='$identifier'");
			
			$sent_contact_ids = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_sent WHERE newsletter_id=".$newsletter->getId());
			$sent_contact_ids = array_flat($sent_contact_ids);
			//$hola prevenir que se le mande a los que no estan subscriptos
			if (is_array($row) && isset($row['value'])) {
				$all_ids = explode(',', $row['value']);
				
				try {
					// delete previous selection
					$delete_sql = "DELETE FROM ".TABLE_PREFIX."newsletter_contacts_pending WHERE newsletter_id=".$newsletter->getId()." AND status='pending'";
					DB::execute($delete_sql);
					
					// remove ids of users that have disabled the newsletter subscription
					$subs_newsletter_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'subs_newsletter');
					if ($subs_newsletter_cp instanceof CustomProperty) {
						$subscribed_rows = DB::executeAll("SELECT cpv.object_id FROM ".TABLE_PREFIX."custom_property_values cpv
								WHERE cpv.custom_property_id=".$subs_newsletter_cp->getId()." AND cpv.value = '1'");
						
						if (is_array($subscribed_rows)) {
							$ids_subscribed = array_flat($subscribed_rows);
							$all_ids = array_intersect($all_ids, $ids_subscribed);
						}
					}
					
					$sql_values = array();
					foreach ($all_ids as $id) {
						if (!is_numeric($id) || in_array($id, $sent_contact_ids)) continue;
						$sql_values[] = "('".$newsletter->getId()."', '$id', 'pending')";
					}
					
					$sql_values_str = "";
					if (count($sql_values) > 0) {
						$sql_values_str = implode(',', $sql_values);
					}
					
					if ($sql_values_str != "") {
						$sql = "INSERT INTO ".TABLE_PREFIX."newsletter_contacts_pending (newsletter_id, contact_id, status)
								VALUES $sql_values_str
								ON DUPLICATE KEY UPDATE newsletter_id=newsletter_id;";
					
					
						DB::beginWork();
				
						// insert current selection
						DB::execute($sql);
						
						DB::commit();
					}
				} catch (Exception $e) {
					DB::rollback();
					Logger::log("Error when saving newsletter contacts - newsletter_id=".$newsletter->getId()."\n".$e->getMessage());
					return;
				}
				
				flash_success(lang("success newsletter contact save"));
				ajx_current("reload");
			}
		} catch (Exception $e) {
			flash_error($e->getMessage());
			Logger::log($e->getMessage());
			Logger::log($e->getTraceAsString());
		}
	}
	
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		}
		
		$extra_conditions = "";
		/*$status_filter = array_var($_GET,'status_filter');
		if ($status_filter != -1) {
			$extra_conditions .= " AND `status`=$status_filter";
		}*/
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		$result = Newsletters::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$newsletters = $result->objects;
		$object = $this->prepareObject($newsletters, $start, $limit, $result);
		
		/*$object['filters'] = array('status' => array(
			array('id' => 0, 'name' => lang('incomplete')),
			array('id' => 1, 'name' => lang('in progress')),
			array('id' => 2, 'name' => lang('complete')),
		));*/
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($newsletters, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"Newsletters" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Newsletters::instance()->getObjectTypeId());
		$count = min(array($limit, count($newsletters)));
		for ($i = 0; $i < $count; $i++){
			if (isset($newsletters[$i])){
				$obj = $newsletters[$i];
				if ($obj instanceof Newsletter){
					$info = $obj->getArrayInfo();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["newsletters"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["newsletters"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$newsletter_data = array_var($_POST, 'newsletter');
		
		if(!is_array($newsletter_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Newsletter();
			$obj->setObjectTypeId(Newsletters::instance()->getObjectTypeId());
			tpl_assign('newsletter', $obj);
			
			$mail_accounts = MailACcounts::findAll(array('conditions' => "for_newsletters=1"));
			tpl_assign('mail_accounts', $mail_accounts);

			$newsletters_default_account_id = config_option('newsletters_default_account');

			$newsletter_data = array();
			$newsletter_data['mail_account_id'] = $newsletters_default_account_id;
			tpl_assign('newsletter_data', $newsletter_data);

		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$newsletter_data['text'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($newsletter_data, 'text'));
				$newsletter_data['status'] = 'pending';

				$newsletter = $this->save_newsletter($newsletter_data, $member_ids);
				
				$this->save_newsletter_contacts($newsletter, array_var($_REQUEST, 'genid'));
				
				DB::commit();

				ApplicationLogs::createLog($newsletter, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add newsletter', $newsletter->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$newsletter = Newsletters::findById(get_id());
		if(!($newsletter instanceof Newsletter)) {
			flash_error(lang('newsletter dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$newsletter->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Newsletter::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("Newsletters"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$newsletter_data = array_var($_POST, 'newsletter');
		
		if(!is_array($newsletter_data)) {

			$start_date = $newsletter->getStartDate() instanceof DateTimeValue ? $newsletter->getStartDate() : null;
			if($start_date instanceof DateTimeValue){
				$start_date->advance(logged_user()->getUserTimezoneValue());
			}

			$newsletter_data = $newsletter->getArrayInfo();
			$newsletter_data['start_date'] = $start_date;
			
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			// contact ids to pre-load the recipients selector
			$newsletter_contact_ids = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_pending WHERE newsletter_id=".$newsletter->getId());
			$newsletter_contact_ids = array_flat($newsletter_contact_ids );
			$sent_newsletter_contact_ids = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_sent WHERE newsletter_id=".$newsletter->getId());
			$sent_newsletter_contact_ids = array_flat($sent_newsletter_contact_ids );
			$newsletter_contact_ids = array_unique(array_merge($newsletter_contact_ids, $sent_newsletter_contact_ids));
			
			$mail_accounts = MailACcounts::findAll(array('conditions' => "for_newsletters=1"));
			
			tpl_assign('newsletter', $newsletter);
			tpl_assign('newsletter_data', $newsletter_data);
			tpl_assign('mail_accounts', $mail_accounts);
			tpl_assign('newsletter_contact_ids', $newsletter_contact_ids);
			
		} else {
			// form submitted
			try {

				try {
					$newsletter_data['start_date'] = getDateValue(array_var($_POST, 'newsletter_start_date'));
				} catch (Exception $e) {
					throw new Exception(lang('date format error', date_format_tip(user_config_option('date_format'))));
				}

				if ($newsletter_data['start_date'] instanceof DateTimeValue) {
					$starttime = getTimeValue(array_var($_POST, 'newsletter_start_time'));
					if (is_array($starttime)) {
						$newsletter_data['start_date']->setHour(array_var($starttime, 'hours'));
						$newsletter_data['start_date']->setMinute(array_var($starttime, 'mins'));
						$newsletter_data['start_date']->advance(logged_user()->getUserTimezoneValue());
					}
				}

				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$newsletter_data['text'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($newsletter_data, 'text'));

				$newsletter = $this->save_newsletter($newsletter_data, $member_ids, $newsletter);
				
				DB::commit();

				ApplicationLogs::createLog($newsletter, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit newsletter', $newsletter->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_newsletter($newsletter_data, $member_ids, $newsletter = null) {
		$newsletter_data['type_content'] = "html";
		// Save Newsletter object
		$is_new = false;
		if (is_null($newsletter) || !$newsletter instanceof Newsletter) {
			$newsletter = new Newsletter();
			$is_new = true;
		}
		
		$newsletter->setFromAttributes($newsletter_data);
		
		$newsletter->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($newsletter, $member_ids);
		}
		$object_controller->add_subscribers($newsletter);
		$object_controller->link_to_new_object($newsletter);
		$object_controller->add_custom_properties($newsletter);
		
		return $newsletter;
	}
	
	
	function view() {
		ajx_set_no_toolbar();
		$newsletter = Newsletters::findById(get_id());
		if(!($newsletter instanceof Newsletter)) {
			flash_error(lang('newsletter dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$newsletter->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		$newsletter_contact_ids = array();
		
		$pending_contacts_data = $this->getPendingContactData($newsletter->getId());
		$pending_ids = array_var($pending_contacts_data,'pending_ids');
		$pending_status = array_var($pending_contacts_data,'pending_status');
		
		$pending_contacts = array();
		if (count($pending_ids) > 0) {
			$pending_contacts = Contacts::findAll(array("limit" => 5, "offset" => 0, "conditions" => "id IN (".implode(',', $pending_ids).")"));
		}
		$pending_contacts_info = array();
		foreach ($pending_contacts as $c) {
			$pending_contacts_info[] = array('id' => $c->getId(), 'name' => $c->getObjectName(), 
					'email' => $c->getEmailAddress(), 'status' => $pending_status[$c->getId()]);
			$newsletter_contact_ids[] = $c->getId();
		}

		// sent list
		$sent_rows = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_sent 
				WHERE newsletter_id=".$newsletter->getId()." LIMIT 5");
		$sent_ids = array_filter(array_flat($sent_rows));
		
		$sent_contacts = array();
		if (count($sent_ids) > 0) {
			$sent_contacts = Contacts::findAll(array("limit" => 5, "offset" => 0, "conditions" => "id IN (".implode(',', $sent_ids).")"));
		}
		$sent_contacts_info = array();
		foreach ($sent_contacts as $c) {
			$sent_contacts_info[] = array('id' => $c->getId(), 'name' => $c->getObjectName(), 'email' => $c->getEmailAddress());
			$newsletter_contact_ids[] = $c->getId();
		}
		
		// put variables in view
		tpl_assign('pending_contacts', $pending_contacts_info);
		tpl_assign('sent_contacts', $sent_contacts_info); 
		
		tpl_assign('newsletter_contact_ids', $newsletter_contact_ids);

		tpl_assign('newsletter', $newsletter);
	}
	
	function send() {

		try {
			Notifier::notifyAction($object, $action, $log_data);
		} catch (Exception $ex) {
			Logger::log($ex->getMessage());
		}

		flash_success(lang('success send mail'));
		ajx_current("empty");
	}


	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$newsletter = Newsletters::findById($id);
					if ($newsletter instanceof Newsletter && $newsletter->canDelete(logged_user())){
						try{
							DB::beginWork();
							$newsletter->trash();
							DB::commit();
							ApplicationLogs::createLog($newsletter, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete newsletter', $newsletter instanceof Newsletter ? clean($newsletter->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$newsletter = Newsletters::findById($id);
					if ($newsletter instanceof Newsletter && $newsletter->canEdit(logged_user())){
						try{
							DB::beginWork();
							$newsletter->archive();
							DB::commit();
							ApplicationLogs::createLog($newsletter, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function get_newsletter_contacts() {
		ajx_current("empty");
		$ocontroller = new ObjectController();
		
		$params = $ocontroller->get_list_objects_params();
		
		$table = array_var($_REQUEST, 'sent') ? "newsletter_contacts_sent" : "newsletter_contacts_pending";
		$newlsetter_id = array_var($_REQUEST, 'newsletter_id');
		
		$params['filters']['object_ids'] = "SELECT contact_id FROM ".TABLE_PREFIX."$table WHERE newsletter_id=".DB::escape($newlsetter_id);
		
		$listing = $ocontroller->get_objects_list($params);
		foreach($listing['objects'] as &$contact) {
		    if (array_var($contact, 'type') == 'contact' && array_var($contact, 'object_id') != ''){
	            $contact['email'] = Contacts::findById($contact['object_id'])->getEmailAddress();
	            $contact['status'] = $this->getContactStatus($newlsetter_id, $contact['object_id']);
	        }
	        
		}
		
		ajx_extra_data($listing);
		tpl_assign("listing", $listing);
	}
	
	
	function get_newsletter_sent_emails() {
		ajx_current("empty");
		
		$newsletter_id = array_var($_REQUEST, 'newsletter_id');
		if (!is_numeric($newsletter_id)) return;
		
		$listing = array('objects' => array());

		$order = array_var($_REQUEST, 'sort');
		$order_dir = array_var($_REQUEST, 'dir');
		if (!in_array(strtoupper($order_dir), array('ASC','DESC'))) $order_dir = 'ASC';
		
		$order_sql = '';
		switch ($order) {
			case 'name':
				$order_sql = "ORDER BY `to` $order_dir, `cc` $order_dir, `bcc` $order_dir";
				break;
				
			case 'sent_date':
			default:
				$order_sql = "ORDER BY `sent_date` $order_dir";
				break;
		}
		
		$rows = DB::executeAll("
				SELECT `id`, `sent_date`, `to`, `cc`, `bcc`, `from` 
				FROM `".TABLE_PREFIX."newsletter_sent_emails` 
				WHERE `newsletter_id`='$newsletter_id'
				$order_sql
		");
		
		foreach ($rows as $row) {
			$to = trim($row['bcc']);
			if (trim($row['cc']) != '')	$to .= ($to == '' ? '' : ', ') . trim($row['cc']);
			if (trim($row['to']) != '')	$to .= ($to == '' ? '' : ', ') . trim($row['to']);
			$to = htmlspecialchars($to);
			
			$from = htmlspecialchars($row['from']);
			
			$date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $row['sent_date']);
			
			$data = array('name' => $to, 'from' => $from, 'sent_date' => format_datetime($date));
			$listing['objects'][] = $data;
		}

		ajx_extra_data($listing);
		tpl_assign("listing", $listing);
	}
	
	
	//get information of pending contacts of determinate newsletter
	private function getPendingContactData($newsletter_id) {
	    $pending_rows = DB::executeAll("SELECT contact_id, status 
				FROM ".TABLE_PREFIX."newsletter_contacts_pending
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=contact_id
				WHERE newsletter_id=".$newsletter_id." AND o.trashed_by_id=0");
	    $pending_status = array();
	    foreach ($pending_rows as $row) {
	        $pending_status[$row['contact_id']] = $row['status'];
	    }
	    $pending_ids = array_filter(array_keys($pending_status));
	    return array("pending_ids"=>$pending_ids, "pending_status"=>$pending_status);
	}
	
	//get status for a contact on newsletter
	private function getContactStatus($newsletter_id, $contact_id) {
	    
	    $pending_data = $this->getPendingContactData($newsletter_id);
	    
	    $pending_ids = array_var($pending_data, "pending_ids");
	    $pending_status = array_var($pending_data, "pending_status");	    

	    return in_array($contact_id,$pending_ids ) ? $pending_status[$contact_id] : lang('sent');
	}
	
	
	
	
}