<?php
	class Newsletter extends BaseNewsletter {
		
		protected $searchable_columns = array('name','text');
	
		/**
		 * @var string
		 */
		var $summary_field = "text";
	
	
		function getArrayInfo() {
			$info = array();
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			$info['name'] = $this->getObjectName();
			$info['updated_on'] = format_datetime($this->getUpdatedOn());
			
			return $info;
		}
		
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Newsletters::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this newsletter
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this newsletter
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this newsletter
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this newsletter
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
		
		
		
		// ---------------------------------------------------
		//  System
		// ---------------------------------------------------
		
		/**
		 * Validate before save
		 *
		 * @param array $error
		 * @return null
		 */
		function validate($errors) {
			if(!$this->validatePresenceOf('subject')) {
				$errors[] = lang('subject required');
			}
			if(!$this->validatePresenceOf('mail_account_id')) {
				$errors[] = lang('mail account required');
			}
		}
	}