<?php

/**
 * ObjectSubtypes controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ObjectSubtypesController extends ApplicationController {

	var $plugin_name = "object_subtypes";
	
	/**
	 * Construct the ObjectSubtypesController
	 *
	 * @access public
	 * @param void
	 * @return ObjectSubtypesController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	
	function get_object_subtypes() {
		
		$subtypes_info = array();
		$subtypes_info_by_id = array();
		
		$subtypes = ObjectSubtypes::findAll();
		foreach ($subtypes as $subtype) {
			$info = $subtype->getArrayInfo();
			
			if (!isset($subtypes_info[$info['ot_name']])) {
				$subtypes_info[$info['ot_name']] = array();
				$subtypes_info_by_id[$info['ot_id']] = array();
			}
			$subtypes_info[$info['ot_name']][] = $info;
			$subtypes_info_by_id[$info['ot_id']][] = $info;
		}
		
		ajx_current("empty");
		ajx_extra_data(array("subtypes" => $subtypes_info, "subtypes_by_otid" => $subtypes_info_by_id));
	}
	
	
	/**
	 * Add object subtype definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$object_subtype = new ObjectSubtype();
		$object_subtype->setGender('m');
		tpl_assign('object_subtype', $object_subtype);
		

		$data = array_var($_POST, 'object_subtype');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$object_type = ObjectTypes::findById(array_var($_GET, 'ot_id'));
			
			tpl_assign('object_type', $object_type);
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_object_subtype($data, $object_subtype);
				
				DB::commit();
				
				$object_type = ObjectTypes::findById($object_subtype->getObjectTypeId());
				flash_success(lang('success add object subtype', strtolower($object_type->getObjectTypeName()), $object_subtype->getName()));
				
				evt_add("reload object subtype menus");
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("reload");
				} else {
					ajx_current("back");
				}
          		
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		$object_subtype = ObjectSubtypes::findById(get_id());
		if (!$object_subtype instanceof ObjectSubtype) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('object_subtype', $object_subtype);
		
		$data = array_var($_POST, 'object_subtype');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$object_type = ObjectTypes::findById($object_subtype->getObjectTypeId());
			tpl_assign('object_type', $object_type);
			
			$parent_ot_id = ObjectTypeDependencies::getMainObjectTypeId($object_type->getId());
			if ($parent_ot_id) {
				$dependant_of_subtype_id = 0;
				$row = DB::executeOne("
						SELECT object_subtype_id FROM ".TABLE_PREFIX."object_subtype_dependencies
						WHERE dependant_object_subtype_id='".$object_subtype->getId()."'
				");
				if (is_array($row)) {
					$dependant_of_subtype_id = array_var($row, 'object_subtype_id');
				}
				tpl_assign('dependant_of_subtype_id', $dependant_of_subtype_id);
			}
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_object_subtype($data, $object_subtype);
				
				DB::commit();
				
				$object_type = ObjectTypes::findById($object_subtype->getObjectTypeId());
				flash_success(lang('success edit object subtype', strtolower($object_type->getObjectTypeName()), $object_subtype->getName()));
				
				evt_add("reload object subtype menus");
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("reload");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	private function save_object_subtype($data, $object_subtype) {
		
		$object_subtype->setFromAttributes($data);
		$object_subtype->save();
		
		$parent_ot_id = ObjectTypeDependencies::getMainObjectTypeId($object_subtype->getObjectTypeId());
		if ($parent_ot_id) {
			
			$main_object_type_id = array_var($data, 'main_object_type_id');
			
			DB::execute("
				DELETE FROM ".TABLE_PREFIX."object_subtype_dependencies
				WHERE dependant_object_subtype_id='".$object_subtype->getId()."';
			");
			
			if (is_numeric($main_object_type_id) && $main_object_type_id > 0) {
				DB::execute("
					INSERT INTO ".TABLE_PREFIX."object_subtype_dependencies (object_subtype_id, dependant_object_subtype_id) VALUES
					('$main_object_type_id', '".$object_subtype->getId()."')
					ON DUPLICATE KEY UPDATE object_subtype_id=object_subtype_id;
				");
			}
		}
		
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$object_subtype = ObjectSubtypes::findById(get_id());
		if (!$object_subtype instanceof ObjectSubtype) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			
			$name = $object_subtype->getName();
			
			$object_subtype->delete();
		
			DB::commit();
		
			flash_success(lang('success delete object subtype', $name));
			ajx_current("reload");
			
			evt_add("reload object subtype menus");
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete


    /**
     * @param $object_id
     */
    function get_object_subtype_dependence($object_id){

        $object = Objects::findById($object_id);
        if($object instanceof Object){
            $id = ObjectSubtypes::getDependantObjectTypeIds($object->getColumnValue('object_subtype_id'));
            if (!empty($id)){
                return $id;
            }
        }
        return null;
    }

} // ObjectSubtypesController

?>