<?php
Hook::register("object_subtypes");

function object_subtypes_object_definition($type, &$ret) {
	if ($type == 'CustomProperty') {
		$ret['object_subtype_id'] = DATA_TYPE_INTEGER;
	}
	if ($type == 'FengObject') {
		$ret['object_subtype_id'] = DATA_TYPE_INTEGER;
	}
}

function object_subtypes_additional_external_columns($object, &$columns) {
	if ($object instanceof ContentDataObject) {
		$columns[] = 'object_subtype_id';
	}
}

function object_subtypes_additional_system_columns($object, &$columns) {
	if ($object instanceof ContentDataObject) {
		$columns[] = 'object_subtype_id';
	}
}

function object_subtypes_general_object_array_info_additional_columns($parameters, &$info) {
	$object = array_var($parameters, 'object');
	if ($object instanceof FengObject) {
		$info['object_subtype_id'] = $object->getColumnValue('object_subtype_id');
	}
}

function object_subtypes_custom_reports_get_possible_external_column_values($parameters, &$values) {
	$ot_id = array_var($parameters, 'ot_id');
	$field = array_var($parameters, 'field');

	if ($field == 'object_subtype_id') {
		
		$subtypes = ObjectSubtypes::findAll(array('conditions' => "object_type_id=$ot_id"));
		$values[] = array('id' => 0, 'name' => lang('general'));
		foreach ($subtypes as $subtype) {
			$values[] = array('id' => $subtype->getId(), 'name' => $subtype->getName());
		}
	}
}


function object_subtypes_custom_reports_get_external_column_value($parameters, &$value) {
	$external_id = array_var($parameters, 'external_id');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($field == 'object_subtype_id') {
		$subtype = ObjectSubtypes::findById($external_id);
		if ($subtype instanceof ObjectSubtype) {
			$value = $subtype->getName();
		} else {
			$value = "";
		}
	}
}

function object_subtypes_get_custom_properties_conditions($parameters, &$extra_conditions) {
	$ot = array_var($parameters, 'ot');
	// if not filtering by object_subtype_id then filter by object_subtype_id=0
	if (strpos($extra_conditions, 'object_subtype_id') === false) {
		$extra_conditions .= " AND object_subtype_id=0";
	}
}


function object_subtypes_object_form_custom_prop_extra_conditions($parameters, &$extra_conditions) {
	$ot = array_var($parameters, 'ot');
	
	$subtype_id = null;
	
	if (isset($_REQUEST['object_subtype_id'])) {
		$subtype_id = $_REQUEST['object_subtype_id'];
	} else {
		$obj = array_var($parameters, 'object');
		if ($obj instanceof ContentDataObject) {
			$subtype_id = $obj->getColumnValue('object_subtype_id');
		}
	}
	if (!$subtype_id) {
		$subtype_id = '0';
	}
	
	if ($subtype_id == '0') {
		$extra_conditions .= " AND object_subtype_id=$subtype_id";
	} else {
		$extra_conditions .= " AND object_subtype_id IN (0,$subtype_id)";
	}
	
	Hook::fire('subtype_cps_extra_conditions', $subtype_id, $extra_conditions);
}

/*
function object_subtypes_custom_property_form_sections($parameters, &$headers) {
	$object_type = array_var($parameters, 'ot');
	
	if ($object_type instanceof ObjectType) {
		$allowed_object_types_ids = ObjectTypes::instance()->findAll(array("id" => true, "conditions" => "`type` IN ('content_object')  
				AND `name` <> 'template_task' AND name <> 'template_milestone' AND `name` <> 'file revision'"));
		
		if (in_array($object_type->getId(), $allowed_object_types_ids)) {
			
			$new_subtype_link = "og.render_modal_form('', {c:'object_subtypes', a:'add', params: {ot_id:".$object_type->getId()."}});";
			add_page_action(lang('new subtype of', strtolower(lang($object_type->getName()))), "javascript:$new_subtype_link", "ico-add",null,null,true);
			
			$subtypes = ObjectSubtypes::findAll(array('conditions' => 'object_type_id='.$object_type->getId(), 'order' => 'name'));
			$count = 0;
			foreach ($subtypes as $subtype) {
				$genid = gen_id();
				$type_name = $subtype->getName();
				
				$additional_hidden_fields = array('object_subtype_id' => $subtype->getId());
				
				tpl_assign('genid', $genid);
				tpl_assign('type_name', $type_name);
				tpl_assign('additional_hidden_fields', $additional_hidden_fields);
				
				tpl_assign('subtype', $subtype);
				tpl_assign('object_type', $object_type);
				
				$count++;
				tpl_assign('id_suffix', "-$count");
				
				tpl_display(get_template_path('cp_sub_table_template', 'object_subtypes', 'object_subtypes'));
				
				
			}
			
		}
		
	}
}
*/
function object_subtypes_custom_property_form_sections($parameters, &$headers) {
	$object_type = array_var($parameters, 'ot');

	if ($object_type instanceof ObjectType) {
		$allowed_object_types_ids = ObjectTypes::instance()->findAll(array("id" => true, "conditions" => "`type` IN ('content_object','dimension_object')
				AND `name` <> 'template_task' AND name <> 'template_milestone' AND `name` <> 'file revision'"));

		if (in_array($object_type->getId(), $allowed_object_types_ids)) {
			$general_type_id = $object_type->getId();
			if ($object_type->getName() == 'template_task') {
				$general_type_id = ObjectTypes::findByName('task')->getId();
			}
			if ($object_type->getName() == 'template_milestone') {
				$general_type_id = ObjectTypes::findByName('milestone')->getId();
			}
				
			$subtypes = ObjectSubtypes::findAll(array('conditions' => 'object_type_id='.$general_type_id, 'order' => 'name'));
			
			tpl_assign('genid', $genid);
			tpl_assign('object_type', $object_type);
			tpl_assign('subtypes', $subtypes);
			
			tpl_display(get_template_path('cp_subtypes_list', 'object_subtypes', 'object_subtypes'));
			
		}

	}
}

function object_subtypes_list_custom_properties_for_type_extra_parameters($parameters, &$extra_params) {
	$object_type = array_var($parameters, 'object_type');
	$request = array_var($parameters, 'request');
	$subtype_id = array_var($request, 'subtype_id');
	
	if ($object_type instanceof ObjectType && $subtype_id > 0) {
		$subtype = ObjectSubtypes::findById($subtype_id);
		if ($subtype instanceof ObjectSubtype) {
			$extra_cond = " AND object_subtype_id = $subtype_id ";
			Hook::fire('subtype_cps_extra_conditions', $subtype_id, $extra_cond);
			
			$extra_params['extra_conditions'] .= $extra_cond;
			$extra_params['add_link_text'] = lang('add new custom property', $subtype->getName());
			$extra_params['form_title'] = lang('custom properties for', $subtype->getName());
			
			if (!isset($extra_params['additional_hidden_fields'])) $extra_params['additional_hidden_fields'] = array();
			$extra_params['additional_hidden_fields']['object_subtype_id'] = $subtype->getId();
		}
	}
}

function object_subtypes_override_add_edit_form_title($params, &$title) {
	$object = array_var($params, 'object');
	$ot = array_var($params, 'ot');
	
	if ($object instanceof ContentDataObject && $ot instanceof ObjectType) {
		if ($object->isNew()) {
			$sub_ot_id = array_var($_REQUEST, 'object_subtype_id', 0);
		} else {
			$sub_ot_id = $object->getColumnValue('object_subtype_id');
		}
		if ($sub_ot_id > 0) {
			$subtype = ObjectSubtypes::findById($sub_ot_id);
			if ($subtype instanceof ObjectSubtype) {
				$lang_key = $object->isNew() ? 'new subtype' : 'edit subtype';
				$lang_key .= " " . strtolower($subtype->getGender());
				$title = lang($lang_key, $subtype->getName());
			}
		}
	}
}

function object_subtypes_override_submit_button_form_title($params, &$title) {
	$object = array_var($params, 'object');
	$ot = array_var($params, 'ot');
	
	if ($object instanceof ContentDataObject && $ot instanceof ObjectType) {
		if ($object->isNew()) {
			$sub_ot_id = array_var($_REQUEST, 'object_subtype_id', 0);
		} else {
			$sub_ot_id = $object->getColumnValue('object_subtype_id');
		}
		if ($sub_ot_id > 0) {
			$subtype = ObjectSubtypes::findById($sub_ot_id);
			if ($subtype instanceof ObjectSubtype) {
				$lang_key = $object->isNew() ? 'add' : 'save changes';
				$title = lang($lang_key);
			}
		}
	}
}

function object_subtypes_override_get_object_type_name($object, &$name) {
	if ($object instanceof ContentDataObject) {
		if ($object->isNew()) {
			$sub_ot_id = array_var($_REQUEST, 'object_subtype_id', 0);
		} else {
			$sub_ot_id = $object->getColumnValue('object_subtype_id');
		}
		if ($sub_ot_id > 0) {
			$subtype = ObjectSubtypes::findById($sub_ot_id);
			if ($subtype instanceof ObjectSubtype) {
				$name = $subtype->getName();
			}
		}
	}
}

/**
 * When the generic object type is not enabled, and the object_subtype_id column is not set
 * then put the first object subtype id in that column
 * @param ContentDataObject $object
 * @param boolean $ret
 */
function object_subtypes_after_object_save($object, &$ret) {
	
	if ($object instanceof ContentDataObject) {
		$allow_generic_type = true;
		Hook::fire('allow_generic_type_object_subtypes', $object, $allow_generic_type);

		if (!$allow_generic_type && $object->getColumnValue('object_subtype_id') == 0) {
			$subtype = ObjectSubtypes::findOne(array("conditions" => array("object_type_id=?", $object->getObjectTypeId())));

			if ($subtype instanceof ObjectSubtype) {
				DB::execute("UPDATE ".TABLE_PREFIX."objects SET object_subtype_id='".$subtype->getId()."' WHERE id='".$object->getId()."'");
			}
		}
	}
}


function object_subtypes_before_render_main_custom_properties($params, &$categories) {
	
	$object = array_var($params, 'object');
	$genid = array_var($params, 'genid');
	
	if ($object instanceof ContentDataObject) {
		
		if ($object->isNew() && array_var($_GET, 'a') == 'copy_task') {
			$original_task = ProjectTasks::findById($_GET['id']);
			if ($original_task instanceof ProjectTask) {
				$_REQUEST['object_subtype_id'] = $original_task->getColumnValue('object_subtype_id');
			}
		}
		
		$sub_type_id = $object->isNew() ? array_var($_REQUEST, 'object_subtype_id', 0) : $object->getColumnValue('object_subtype_id');
		
		$ot = ObjectTypes::instance()->findById($object->getObjectTypeId());
		$all_subtypes = ObjectSubtypes::instance()->getAllObjectSubtypesInfo();
		
		$template_id = null;
		if ($ot->getType() == 'dimension_object') {
			$mem = Members::findOneByObjectId($object->getId());
			$template_id = $object->isNew() ? array_var($_REQUEST, 'template_id', 0) : $mem->getColumnValue('template_id');
		}
		
		$allow_generic_type = true;
		$parent_subtype_id = null;
		$dependant_ids = array();
		Hook::fire('parent_subtype_id', $object, $parent_subtype_id);
		if ($parent_subtype_id) {
			$dependant_rows = DB::executeAll("SELECT dependant_object_subtype_id FROM ".TABLE_PREFIX."object_subtype_dependencies
					WHERE object_subtype_id='$parent_subtype_id'");
			
			$dependant_ids = array_filter(array_flat($dependant_rows));
			$allow_generic_type = count($dependant_ids) == 0;
		}

		Hook::fire('allow_generic_type_object_subtypes', $object, $allow_generic_type);

		$subtypes = array();
		foreach ($all_subtypes as $subtype) {
			if ($subtype['ot_id'] == $ot->getId()) {
				// filter associated subtypes if they are defined
				if (count($dependant_ids) > 0 && !in_array($subtype['id'], $dependant_ids)) {
					continue;
				}
				$subtypes[] = $subtype;
			}
		}

		if (count($subtypes) > 0) {
		
			if ($allow_generic_type) {
				$attr = $sub_type_id == 0 ? array('selected' => 'selected') : null;
				$options = array(option_tag(lang($ot->getName()), 0, $attr));
			} else {
				$options = array();
			}
			foreach ($subtypes as $subtype) {
				$attr = $sub_type_id == $subtype['id'] ? array('selected' => 'selected') : null;
				$options[] = option_tag($subtype['name'], $subtype['id'], $attr);
			}
			
			$base_input_name = $ot->getType() == 'dimension_object' ? "dim_obj" : $object->getObjectTypeName();
			
			
			$is_forced_by_dimension = false;
			$forcing_dimension_id = 0;
			if (Plugins::instance()->isActivePlugin("advanced_core")) {
				$option = DimensionContentObjectOptions::findOne(array('conditions' => "content_object_type_id=".$ot->getId()." AND `option`='forces_object_subtype'"));
				if ($option instanceof DimensionContentObjectOption) {
					$is_forced_by_dimension = $option->getValue() > 0;
					$forcing_dimension_id = $option->getDimensionId();
					
					if ($object->isNew() && $is_forced_by_dimension && $forcing_dimension_id > 0) {
						
						if (array_var($_REQUEST, 'object_subtype_id', 0) > 0) {
							// TODO: set el member associated to the object
							$ost = ObjectSubtypes::findByID($sub_type_id);
							if ($ost instanceof ObjectSubtype && $ost->getMemberId() > 0) {
								echo "<script>
									if (member_selector['$genid'].properties[$forcing_dimension_id]) {
										member_selector.add_relation($forcing_dimension_id, '$genid', ".$ost->getMemberId().", true, true);
										\$('#".$genid."-member-chooser-panel-$forcing_dimension_id-tree-current-selected .empty-text').hide();
									}
								</script>";
							}
						} else {
							// get the subtype member from the active context and set the subtype id
							foreach (active_context() as $selection) {
								if ($selection instanceof Member && $selection->getDimensionId() == $forcing_dimension_id) {
									$forced_subtype = ObjectSubtypes::findOne(array("conditions" => "object_type_id=".$ot->getId()." AND member_id=".$selection->getId()));
									if ($forced_subtype instanceof ObjectSubtype) {
										$sub_type_id = $forced_subtype->getId();
									}
									break;
								}
							}
						}
					}
				}
			}
			
			if (count($options) > 1 && !$is_forced_by_dimension && !$template_id && !($object->isNew() && $ot->getType() == 'dimension_object')) {
				echo '<div class="dataBlock">';
				echo label_tag(lang("x subtype", lang($ot->getName())));
				echo select_box($base_input_name."[object_subtype_id]", $options, array(
						'id' => $base_input_name."_object_subtype_id",
						'onchange' => 'og.object_subtypes.on_object_subtype_changed(this, "'.$object->getId().'", "'.$ot->getId().'", "'.$ot->getName().'");'
				));
				echo '</div>';
			} else {
				$on_change_listener = "";
				if ($is_forced_by_dimension) {
					$on_change_listener = 'onchange="og.object_subtypes.on_object_subtype_changed(this, \''.$object->getId().'\', \''.$ot->getId().'\', \''.$ot->getName().'\');"';
				}
				echo '<input type="hidden" name='.$base_input_name.'[object_subtype_id]" value="'.$sub_type_id.'" 
						id="'.$base_input_name.'_object_subtype_id" '.$on_change_listener.' />';
			}
		}
	}
}


function object_subtypes_after_template_object_instantiation($parameters, &$ignored) {
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');
	
	if (!is_null($copy) && !is_null($original)) {
		DB::execute("UPDATE ".TABLE_PREFIX."objects SET object_subtype_id=" . $original->getColumnValue('object_subtype_id'). " WHERE id=". $copy->getId());
	}
}

function object_subtypes_task_clone_more_attributes($parameters, &$ignored) {
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');
	
	if (!is_null($copy) && !is_null($original)) {
		$copy->getObject()->setColumnValue('object_subtype_id', $original->getColumnValue('object_subtype_id'));
	}
}


function object_subtypes_sum_col_total_query($params, &$sum_col) {
	$list_type = array_var($params, 'list_type');
	$object_type = array_var($params, 'object_type');
	if ($list_type == 'object') {
		$ot_has_subtypes = false;
		if ($object_type) {
			$ot_has_subtypes = ObjectSubtypes::instance()->count("object_type_id = ".$object_type->getId()) > 0;
		}
		if ($ot_has_subtypes) {
			$sum_col = " CASE WHEN os.sum_in_total = 0 THEN -".$sum_col." ELSE ".$sum_col." END ";
		}
	}
}

function object_subtypes_aditional_join_listing_total_query($params, &$joins_sql) {
	$list_type = array_var($params, 'list_type');
	$object_type = array_var($params, 'object_type');
	if ($list_type == 'object') {
		$ot_has_subtypes = false;
		if ($object_type) {
			$ot_has_subtypes = ObjectSubtypes::instance()->count("object_type_id = ".$object_type->getId()) > 0;
		}
		if ($ot_has_subtypes) {
			$joins_sql .= " LEFT JOIN ".TABLE_PREFIX."object_subtypes os ON o.object_subtype_id = os.id ";
		}
	}
}



// adjust dimension object associated to the managed event member after saving it
function object_subtypes_after_add_dimension_object_member($params, &$ignored) {
	$member = array_var($params, 'member');
	$object = array_var($params, 'dimension_object');
	$dimension_obj_data = array_var($params, 'dimension_obj_data');
	$is_new = array_var($params, 'is_new');
	
	$subtype_id = array_var($dimension_obj_data, 'object_subtype_id');
	if (!$subtype_id && isset($_REQUEST['dim_obj']['object_subtype_id'])) {
		$subtype_id = $_REQUEST['dim_obj']['object_subtype_id'];
	}
	
	if ($is_new && $subtype_id > 0) {
		DB::execute("
				UPDATE ".TABLE_PREFIX."objects 
				SET object_subtype_id=".$subtype_id."
				WHERE id=".$object->getId()
		);
	}

}


/**
 * Add member subtype quick add links
 */
function object_subtypes_member_quick_add_urls($parameters, &$urls) {

	$dim_object_types = array_var($parameters, 'object_types');
	$dimension = array_var($parameters, 'dimension');
	$parent_member = array_var($parameters, 'parent_member');

	foreach ($dim_object_types as $dim_ot) {
		$ot = ObjectTypes::findById($dim_ot->getObjectTypeId());

		//$templates = MemberTemplates::findAll(array('conditions' => array('dimension_id=? AND object_type_id=?', $dimension->getId(), $ot->getId())));
		$subtypes = ObjectSubtypes::findAll(array("conditions" => array("object_type_id=?", $dim_ot->getObjectTypeId())));
		//foreach ($templates as $t) {
		foreach ($subtypes as $t) {
				
			$class_name = ucfirst($ot->getName())."Controller";
			if ($ot && controller_exists($ot->getName(), $ot->getPluginId())) {
				$params = array("type" => $ot->getId(), 'object_subtype_id' => $t->getId(), "dim_id" => $dimension->getId());
				if ($parent_member instanceof Member) $params['parent'] = $parent_member->getId();
					
				$url = get_url($ot->getName(), 'add', $params);
			} else {
				$params = array("dim_id" => $dimension->getId(), "type" => $ot->getId(), 'object_subtype_id' => $t->getId());
				if ($parent_member instanceof Member) $params['parent'] = $parent_member->getId();
					
				$url = get_url('member', 'add' , $params);
			}
				
			$urls[] = array('link_text' => $t->getName(), 'url' => $url, 'iconcls' => $ot->getIconClass());
		}
	}
}


function object_subtypes_override_widget_name($widget, &$name) {
	$subtype_id = $widget->getContactOptionValue('subtype_filter');
	if ($subtype_id > 0) {
		$subtype = ObjectSubtypes::findById($subtype_id);
		if ($subtype instanceof ObjectSubtype) {
			$name = $subtype->getName();
		}
	}
}


function object_subtypes_override_active_context_info_widget_header_color($parameters, &$color_class) {
	
	$member = array_var($parameters, 'member');
	if ($member->getObjectId() > 0) {
		$object = Objects::findObject($member->getObjectId());
		if ($object->getColumnValue('object_subtype_id') > 0) {
			$subtype = ObjectSubtypes::findById($object->getColumnValue('object_subtype_id'));
			if ($subtype && $subtype->getColor() > 0) {
				$color_class = "color-".$subtype->getColor();
			}
		}
	}
	
}

