og.object_subtypes = {
	
	init: function() {
		if (!og.additional_on_cp_submit_fn) {
			og.additional_on_cp_submit_fn = [];
		}
		
		og.additional_on_cp_submit_fn.push(og.object_subtypes.on_cp_submit_object_subtype);
		
		og.object_subtypes.load_object_subtypes();
		
	},
	
	execute_add_action: function() {
		// scope = Ext.Action
		var parameters = {c: this.controller, a: this.add_action};
		if (this.add_params) {
			parameters.params = this.add_params;
		} 
		og.render_modal_form('', parameters);
	},
	
	load_object_subtypes: function() {
		og.object_subtype_names = {};
		
		og.openLink(og.getUrl('object_subtypes', 'get_object_subtypes'), {
			callback: function(success, data){
				if (success) {
					if (typeof data.subtypes != 'undefined' && !(data.subtypes instanceof Array )) {
						og.object_subtype_names = data.subtypes;
					}
					if (typeof data.subtypes_by_otid != 'undefined' && !(data.subtypes_by_otid instanceof Array )) {
						og.object_subtypes_by_otid = data.subtypes_by_otid;
					}
					
					og.object_subtypes.load_new_actions_for_listings();
				}
			}
		});
	},
	
	load_new_actions_for_listings: function() {
		
		if (!og.replace_list_new_action) og.replace_list_new_action = {};
		
		og.replace_list_new_handlers = {};
		
		var buttons_under_menu = ['task', 'file', 'milestone', 'contact'];
		
		for (var otname in og.object_subtype_names) {
			var ot_def = null;
			for (var otid in og.objectTypes) {
				if (og.objectTypes[otid].name == otname) {
					ot_def = og.objectTypes[otid];
					break;
				}
			}

			var menuItems = [];

			// only replace original new action when creating a menu
			if (buttons_under_menu.indexOf(otname) < 0 && ot_def && ot_def.allow_generic_type) {

				var general_new = new Ext.Action({
					id: Ext.id(),
					text: lang('new ' + otname),
					tooltip: lang('add new subtype', lang(otname)),
					iconCls: 'ico-'+otname+' new_button',
					controller: ot_def.controller,
					add_action: ot_def.add_action
				});
				var original_btn = Ext.getCmp("new_button_"+otname);
				if (original_btn) {
					general_new.setHandler(original_btn.handler);
				} else {
					general_new.setHandler(og.object_subtypes.execute_add_action);
				}
				menuItems.push(general_new);
			}
			
			for (var i=0; i<og.object_subtype_names[otname].length; i++) {
				
				var sub_ot = og.object_subtype_names[otname][i];
				
				var btn_id = Ext.id();
				var subtype_new = new Ext.Action({
					id: btn_id,
					text: lang('new subtype '+sub_ot.gender, sub_ot.name),
		            tooltip: lang('add new subtype', sub_ot.name),
		            iconCls: 'ico-'+otname+' new_button',
		            controller: sub_ot.controller,
		            add_action: sub_ot.add_action,
		            add_params: {object_subtype_id: sub_ot.id}
				});
				subtype_new.setHandler(og.object_subtypes.execute_add_action);
				menuItems.push(subtype_new);
			}
			
			if (buttons_under_menu.indexOf(otname) < 0) {
				
				// replace current menu
				var current_menu = Ext.getCmp('subtype-new-menu-'+otname);
				if (current_menu) {
					
					current_menu.menu.removeAll();
					for (var j=0; j<menuItems.length; j++) {
						current_menu.menu.add(menuItems[j]);
					}
					
				} else {
					// poner menu en toolbar
				}
			}
			// put menu in actions to replace default "new button"
			var menu = new Ext.Button({
				iconCls: 'ico-new',
				text: lang('new'),
				id: 'subtype-new-menu-'+otname,
				menu: {
					cls:'scrollable-menu',
					items: menuItems
				}
			});
			og.replace_list_new_action[otname] = menu;
			
		}
		
		
		// set visibility of menu and default button 
		for (otid in og.objectTypes) {
			var ot = og.objectTypes[otid];
			if (ot.type != 'dimension_group' && buttons_under_menu.indexOf(ot.name) < 0) {
				
				if (og.replace_list_new_action[ot.name]) {
					// hide button only if menu exists
					if ($("#subtype-new-menu-"+ot.name).length > 0) {
						$("#new_button_"+ot.name).closest('td').addClass("x-hide-display");
						$("#subtype-new-menu-"+ot.name).removeClass("x-hide-display");
					}
				} else {
					
					$("#new_button_"+ot.name).closest('td').removeClass("x-hide-display");
					$("#subtype-new-menu-"+ot.name).addClass("x-hide-display");
				}
				
			}
		}
	},
	
	on_cp_submit_object_subtype: function(prop, pre_id) {
		
		prop.object_subtype_id = $(pre_id + " #object_subtype_id").attr('value');
		
		return prop;
	},
	
	delete_object_subtype: function(id, type_name) {
		if (confirm(lang('confirm delete object subtype', type_name))) {
			og.openLink(og.getUrl('object_subtypes', 'delete', {id:id}));
		}
	},
	
	
	on_object_subtype_changed: function(select, obj_id, ot_id, ot_name) {
		// main custom properties
		var container = $(select).closest('form').find('.main-custom-properties-div'); 
		var visibility = 'visible_by_default';
		og.object_subtypes.reload_custom_properties(select, obj_id, ot_id, ot_name, container, visibility);

		// other custom properties
		var container = $(select).closest('form').find('.other-custom-properties-div'); 
		var visibility = 'other';
		og.object_subtypes.reload_custom_properties(select, obj_id, ot_id, ot_name, container, visibility);
	},

	reload_custom_properties: function(select, obj_id, ot_id, ot_name, container, visibility) {

		$(container).html('<div class="widget-body loading">'+lang('loading')+'</div>');
		var render_cps_params = {id: obj_id, ot_id: ot_id, visibility: visibility, object_subtype_id: $(select).val()};
		if (og.more_params_for_render_cps_params) {
			for (var x=0; x<og.more_params_for_render_cps_params.length; x++) {
				var moreparams = og.more_params_for_render_cps_params[x].call(null, ot_name);
				for (k in moreparams) render_cps_params[k]=moreparams[k];
			}
		}
		
		og.openLink(og.getUrl('object','render_cps', render_cps_params), {
			callback: function(success, data) {
				$(container).html(data.html);
			}
		});
	}
}

og.eventManager.addListener('reload object subtype menus',function(data){
	og.object_subtypes.load_object_subtypes();
});


if (!og.more_params_for_render_cps_params) og.more_params_for_render_cps_params=[];

og.more_params_for_render_cps_params.push(function(typename) {
	var result = {};
	var st = $("#"+typename+"_object_subtype_id").val();
	if (st) {
		result['object_subtype_id'] = st;
	}
	return result;
});
