<?php
/**
 * MailTemplate controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class MailTemplateController extends ApplicationController {

	var $plugin_name = "object_templates";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function index() {
		ajx_set_no_toolbar(true);
	}
	
	function list_all() {
		
		ajx_current("empty");
		
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		
		$ids = array();
		$exploded = explode(',', array_var($_GET,'ids'));
		foreach ($exploded as $exp) {
			if (is_numeric($exp)) $ids[] = $exp;
		}
		$attributes = array("ids" => $ids);
		
		$extra_conditions = "";
		
		Hook::fire('email_templates_list_extra_conditions', array('request' => $_REQUEST), $extra_conditions);
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		/*
		$result = MailTemplates::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		*/
		$result = MailTemplates::instance()->listAll(array(
				"order" => $order,
				"order_dir" => $order_dir,
				"extra_conditions" => $extra_conditions,
				"count_results" => true,
				"start" => $start,
				"limit" => $limit,
		));
		$mail_templates = $result->objects;
		$object = $this->prepareObject($mail_templates, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($mail_templates, $start, $limit, $res_obj) {
		
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"mail_templates" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		//$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(MailTemplates::instance()->getObjectTypeId());
		$count = min(array($limit, count($mail_templates)));
		for ($i = 0; $i < $count; $i++){
			if (isset($mail_templates[$i])){
				$obj = $mail_templates[$i];
				if ($obj instanceof MailTemplate){
					$info = $obj->getArrayInfo();
					$info['object_id'] = $info['id'];
					//$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					
					$object["mail_templates"][] = $info;

					/* enable this when mail temapltes become content objects
					foreach ($custom_properties as $cp) {
						$object["mail_templates"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
					*/
    			}
			}
		}
		return $object;
	}

	
	function add() {
		
		if (!can_manage_mail_templates(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$mail_template_data = array_var($_POST, 'mail_template');
		
		if(!is_array($mail_template_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new MailTemplate();
			$obj->setIsNotificationTemplate(array_var($_REQUEST,'is_notification_template'));
			tpl_assign('mail_template', $obj);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$mail_template = $this->save_mail_template($mail_template_data);
				
				DB::commit();

				//ApplicationLogs::createLog($mail_template, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add mail_template', $mail_template->getName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$mail_template = MailTemplates::findById(get_id());
		if(!($mail_template instanceof MailTemplate)) {
			flash_error(lang('form dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!can_manage_mail_templates(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$mail_template_data = array_var($_POST, 'mail_template');

		
		if(!is_array($mail_template_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('mail_template', $mail_template);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$mail_template = $this->save_mail_template($mail_template_data, $mail_template);
				
				DB::commit();

				//ApplicationLogs::createLog($mail_template, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit mail_template', $mail_template->getName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_mail_template($mail_template_data, $mail_template = null) {
		// Save MailTemplate object
		$is_new = false;
		if (is_null($mail_template) || !$mail_template instanceof MailTemplate) {
			$mail_template = new MailTemplate();
			$is_new = true;
		}
		
		$mail_template_data['body'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($mail_template_data, 'body'));
		if (!isset($mail_template_data['send_datetime'])) {
			$mail_template_data['send_datetime'] = 'now';
		}
		
		$mail_template->setFromAttributes($mail_template_data);
		
		$mail_template->save();
		
		/* enable this code when mail template become content object
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($mail_template, $member_ids);
		}
		$object_controller->add_subscribers($mail_template);
		$object_controller->link_to_new_object($mail_template);
		$object_controller->add_custom_properties($mail_template);
		*/
		return $mail_template;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$mail_template = MailTemplates::findById(get_id());
		if(!($mail_template instanceof MailTemplate)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_mail_templates(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$this->redirectTo('mail_template', 'edit', array('id' => $mail_template->getId()));
		
		tpl_assign('mail_template', $mail_template);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action) {
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$mail_template = MailTemplates::findById($id);
					if ($mail_template instanceof MailTemplate && can_manage_mail_templates(logged_user())){
						try{
							DB::beginWork();
							$mail_template->delete();
							DB::commit();
							//ApplicationLogs::createLog($mail_template, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete form', $mail_template instanceof MailTemplate ? clean($mail_template->getName()) : '');
					}
				};
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");
				$resultCode = 2;	
				break;		
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}

