<?php
function read_csv_file($filename, $delimiter, $only_first_record = false, $first_record_has_names = false) {

    // if encoding=ISO-8859-1 use ut8_encoding function
    $file_content = file_get_contents($filename);
    $file_encoding = detect_encoding($file_content, array('ASCII', 'UTF-8', 'ISO-8859-1'));
    if ($file_encoding == 'ISO-8859-1') {
        $new_filename = ROOT."/tmp/".gen_id()."_utf8.csv";
        $file_content = utf8_encode($file_content);
        file_put_contents($new_filename, $file_content);
        $filename = $new_filename;
    }

    $handle = fopen($filename, 'rb');
    if (!$handle) {
        flash_error(lang('file not exists'));
        ajx_current("empty");
        return;
    }

    if ($only_first_record) {
        $result = fgetcsv($handle, null, $delimiter);
        $aux = array();
        if (function_exists('mb_convert_encoding')) {
            foreach ($result as $title) $aux[] = mb_convert_encoding($title, "UTF-8", detect_encoding($title));
        } else {
            foreach ($result as $title) $aux[] = $title;
        }
        $result = $aux;
    } else {

        $result = array();
        $first_row = true;
        while ($fields = fgetcsv($handle, null, $delimiter)) {
            if($first_record_has_names && $first_row){
                $first_row = false;
                continue;
            }
            $aux = array();
            if (function_exists('mb_convert_encoding')) {
                foreach ($fields as $field) $aux[] = mb_convert_encoding($field, "UTF-8", detect_encoding($field));
            } else {
                foreach ($fields as $field) $aux[] = $field;
            }

            $all_fields_empty = true;
            foreach ($aux as $aux_field){
                if($aux_field != ''){
                    $all_fields_empty = false;
                    break;
                }
            };

            if(!$all_fields_empty){
                $result[] = $aux;
            }

        }


    }

    fclose($handle);
    return $result;
} //read_csv_file


/**
 * Count the number of rows in a CSV file excluding header row.
 *
 * @param string $filename
 *   CSV filename.
 *
 * @return int
 *   Number of rows.
 */
function count_csv_rows($filename, $delimiter) {
    ini_set('auto_detect_line_endings', TRUE);
    $row_count = 0;
    $handle = fopen($filename, "rb");
    if (!$handle) {
        flash_error(lang('file not exists'));
        ajx_current("empty");
        return;
    }
    while (($row_data = fgetcsv($handle, null, $delimiter)) !== FALSE) {
    $row_count++;
    }
    fclose($handle);
    return $row_count;
  }


  /**
 * Load desired_count rows from filename starting at position start.
 *
 * @param string $filename
 *   CSV filename.
 * @param int $start
 *   Starting position in file.
 * @param int $end
 *   Ending position in file.
 *
 * @return array|bool
 *   Array of Objects or FALSE
 */
function slice_csv_file($filename, $start, $end, $delimiter, $first_record_has_names) {
    $row = 0;
    $count = 0;
    $result = array();
    
    $handle = fopen($filename, "rb");
    if (!$handle) {
        flash_error(lang('file not exists'));
        ajx_current("empty");
        return;
    }
    while ($fields = fgetcsv($handle, null, $delimiter)) {
         // Skip headings.
         if ($count == 0 && $first_record_has_names) {
            $count++;
            continue;
        }
        // Not there yet.
        if ($count++ < $start) {
            continue;
        }
        $aux = array();
        if (function_exists('mb_convert_encoding')) {
            foreach ($fields as $field) $aux[] = mb_convert_encoding($field, "UTF-8", detect_encoding($field));
        } else {
            foreach ($fields as $field) $aux[] = $field;
        }

        $all_fields_empty = true;
        foreach ($aux as $aux_field){
            if($aux_field != ''){
                $all_fields_empty = false;
                break;
            }
        };

        if(!$all_fields_empty){
            $result[] = $aux;
        }
        
        if ($count == $end) {
            return $result;
        }

    }
    return $result;
  } // slice_csv_file