<?php
/**
 * Overtime Reports controller
 *
 * @version 1.0
 */
class OvertimeReportsController extends ApplicationController {

	var $plugin_name = "overtime_reports";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('reporting');
		Env::useHelper('format');
	}
	
	function total_overtime_hours_parameters() {
		
		$users = allowed_users_in_context(Timeslots::instance()->getObjectTypeId(), active_context());
		
		tpl_assign('users', $users);
		
	}
	
	
	function total_overtime_hours($return_data = false) {
		
		$csv_export = array_var($_POST, 'csv_export');
		
		if ($return_data) {
			$report_data = $_SESSION['total_overtime_hours_data'];
		} else {
			$report_data = array_var($_POST, 'report', array());
			$_SESSION['total_overtime_hours_data'] = $report_data;
		}
		
		$extra_conditions = '';
		
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$sunday = $now->getMondayOfWeek()->add('d',-1);
				$nextSunday = $now->getMondayOfWeek()->add('w',1)->add('d',-2);
				$st = DateTimeValueLib::make(0,0,0,$sunday->getMonth(),$sunday->getDay(),$sunday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextSunday->getMonth(),$nextSunday->getDay(),$nextSunday->getYear());break;
			case 3: //Last week
				$sunday = $now->getMondayOfWeek()->add('w',-1)->add('d',-1);;
				$nextSunday = $now->getMondayOfWeek()->add('d',-2);
				$st = DateTimeValueLib::make(0,0,0,$sunday->getMonth(),$sunday->getDay(),$sunday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextSunday->getMonth(),$nextSunday->getDay(),$nextSunday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Last month
				$now->add('M',-1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				if (array_var($report_data, 'date_start_value')) {
					$st = getDateValue(array_var($report_data, 'date_start_value'));
					if ($st instanceof DateTimeValue) $st = $st->beginningOfDay();
				}
				if (array_var($report_data, 'date_end_value')) {
					$et = getDateValue(array_var($report_data, 'date_end_value'));
					if ($et instanceof DateTimeValue) $et = $et->endOfDay();
				}
				break;
		}
		if ($et != null && $st != null) {
			tpl_assign('start_time', $st);
			tpl_assign('end_time', $et);
		}
		
		$cp_columns = array('paygroup', 'department', 'key');
		$cps_tmp = CustomProperties::findAll(array("conditions" => "`code` IN ('".implode("','", $cp_columns)."') AND `object_type_id`=".Contacts::instance()->getObjectTypeId().""));
		$cps = array();
		foreach ($cps_tmp as $cp) {
			$cps[$cp->getCode()] = $cp;
		}
		
		$param_user_id = array_var($report_data, 'user');
		
		$totals = array('regular' => 0, 'overtime' => 0, 'double_overtime' => 0, 'miles' => 0, 'total' => 0);
		$all_accumulated_minutes = array();
		$all_timeslots = get_timeslots_grouped_by_user_and_date($st, $et, $param_user_id);
		foreach ($all_timeslots as $user_key => $timeslots) {
			$accumulated_minutes = accumulate_timeslots_worked_time($timeslots);
			
			$exp_key = explode('_|_', $user_key);
			$user_id = $exp_key[1];
			$user = Contacts::findById($user_id);
			
			$accumulated_minutes['user_id'] = $user_id;
			$accumulated_minutes['name'] = $user instanceof Contact ? $user->getObjectName() : "";
			$accumulated_minutes['total'] = $accumulated_minutes['regular'] + $accumulated_minutes['overtime'] + $accumulated_minutes['double_overtime'];
			
			foreach ($cps as $cp) {
				$cpval = CustomPropertyValues::getCustomPropertyValue($user_id, $cp->getId());
				$accumulated_minutes[$cp->getCode()] = $cpval instanceof CustomPropertyValue ? $cpval->getValue() : "";
			}
			
			$all_accumulated_minutes[] = $accumulated_minutes;

			$totals['regular'] += $accumulated_minutes['regular'];
			$totals['overtime'] += $accumulated_minutes['overtime'];
			$totals['double_overtime'] += $accumulated_minutes['double_overtime'];
			$totals['total'] += $accumulated_minutes['total'];
		}
		
		// non worked time
		$non_worked_columns = array();
		$non_worked_time_rows = get_non_worked_time_by_person($st, $et);
		foreach ($non_worked_time_rows as $data) {
			if (!isset($non_worked_columns[$data['member_id']])) $non_worked_columns[$data['member_id']] = $data['member_name'];
			
			$user = Contacts::findById($data['contact_id']);
			if (!$user instanceof Contact) continue;
			
			foreach ($all_accumulated_minutes as &$accumulated_minutes) {
				if ($accumulated_minutes['user_id'] == $data['contact_id']) {
					// add to row
					$accumulated_minutes[$data['member_id']] = $data['total_minutes'];
					// add to row total
					$accumulated_minutes['total'] += $data['total_minutes'];
					// add to general totals
					if (!isset($totals[$data['member_id']])) $totals[$data['member_id']] = 0;
					$totals[$data['member_id']] += $data['total_minutes'];
					$totals['total'] += $data['total_minutes'];
					break;
				}
			}
		}
		
		// miles
		$miles_cp = CustomProperties::getCustomPropertyByCode(Timeslots::instance()->getObjectTypeId(), 'miles');
		$miles_rows = get_miles_by_person($st, $et);
		foreach ($miles_rows as $data) {
				
			$user = Contacts::findById($data['contact_id']);
			if (!$user instanceof Contact) continue;
				
			foreach ($all_accumulated_minutes as &$accumulated_minutes) {
				if ($accumulated_minutes['user_id'] == $data['contact_id']) {
					// add to row
					$accumulated_minutes['miles'] = $data['miles'];
					// add to general totals
					$totals['miles'] += $data['miles'];
					break;
				}
			}
		}
		
		if ($return_data) {
			$headers = array();
			foreach ($cps as $cp) {
				$headers[$cp->getCode()] = $cp->getName();
			}
			$fixed_headers = array(
					'name' => lang('name'), 
					'regular' => lang('regular'), 
					'overtime' => lang('overtime'), 
					'double_overtime' => lang('double overtime')
			);
			$headers = array_merge($headers, $fixed_headers);
			foreach ($non_worked_columns as $col_id => $col_name) {
				$headers[$col_id] = $col_name;
			}
			$headers['miles'] = $miles_cp->getName();
			$headers['total'] = lang('total');
			
			return array('all_accumulated_minutes' => $all_accumulated_minutes, 'totals' => $totals, 'headers' => $headers);
		}

		tpl_assign('non_worked_columns', $non_worked_columns);
		tpl_assign('all_accumulated_minutes', $all_accumulated_minutes);
		tpl_assign('totals', $totals);
		
		
		tpl_assign('post', $report_data);
		tpl_assign('title', lang('total overtime hours report'));
		tpl_assign('allow_export', true);
		
		
		tpl_assign('template_name', 'total_overtime_hours');
		ajx_set_no_toolbar();
		$this->setTemplate('report_wrapper');
	}
	
	
	function total_overtime_hours_print() {
		if(array_var($_POST, 'exportCSV')){
			
			$report_data = $this->total_overtime_hours(true);

			$data = array_var($report_data, 'all_accumulated_minutes');
			$totals = array_var($report_data, 'totals');
			$headers = array_var($report_data, 'headers');
			$filename = 'worked_hours.csv';
			
			$this->generateCSVReport($headers, $data, $totals, $filename);
			
			ajx_current("empty");
			ajx_extra_data(array('filename' => $filename));
		}
	}
	
	
	function generateCSVReport($headers, $data, $totals, $filename){
		$contents = "";
		
		foreach($headers as $col => $colname){
			$contents .= $colname.';';
		}
		$contents .= "\n";
		
		$totals['name'] = lang('total');
		$data[] = $totals;
		
		foreach($data as $row) {
			$i = 0;
			
			foreach($headers as $col => $colname){
				$value = array_var($row, $col, '');

				$cell = '';
				if ($value != '') {
					if (is_numeric($col) || in_array($col, array('regular', 'overtime', 'double_overtime', 'total'))) {
						$cell = number_format($value/60, 2);
					} else {
						$cell = $value;
					}
				}
				
				$contents .= $cell.';';
				$i++;
			}
			$contents .= "\n";
		}
		
		file_put_contents(ROOT."/tmp/$filename", $contents);
	}
	
	
	function download_exported_file() {
		$filename = array_var($_SESSION, 'expense_report_export_tmp_filename', '');
		if ($filename != '') {
			$path = ROOT.'/tmp/'.$filename;
			$size = filesize($path);
			
			$name = array_var($_SESSION, 'expense_report_export_filename', 'report.csv');
			
			unset($_SESSION['expense_report_export_tmp_filename']);
			unset($_SESSION['expense_report_export_filename']);
			
			// download file
			header("Cache-Control: public");
			header("Content-Description: File Transfer");
			header("Content-Disposition: attachment; filename=".$name."");
			header("Content-Transfer-Encoding: binary");
			header("Content-Type: text/csv");
			readfile($path);
			
			// delete tmp file
			//unlink($path);
		}
		die();
	}
	
}