<?php



function get_miles_by_person($date_from, $date_to) {
	$tz_offset = logged_user()->getUserTimezoneHoursOffset();
	
	$date_conditions = "";
	if ($date_from instanceof DateTimeValue) {
		$d_from = new DateTimeValue($date_from->getTimestamp());
		$d_from->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` >= ".DB::escape($d_from->toMySQL());
	}
	if ($date_from instanceof DateTimeValue) {
		$d_to = new DateTimeValue($date_to->getTimestamp());
		$d_to->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` <= ".DB::escape($d_to->toMySQL());
	}
	
	$miles_cp = CustomProperties::getCustomPropertyByCode(Timeslots::instance()->getObjectTypeId(), 'miles');
	if (!$miles_cp instanceof CustomProperty) return array();
	
	$normal_work_member_id = config_option('normal_work_member_id');
	
	$sql = "SELECT t.contact_id, SUM(cpv.value) as miles
			FROM ".TABLE_PREFIX."timeslots t
			INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			INNER JOIN ".TABLE_PREFIX."custom_property_values cpv ON cpv.object_id=o.id
				WHERE
				o.trashed_by_id=0
				AND cpv.custom_property_id = '".$miles_cp->getId()."'
				AND om.member_id = '$normal_work_member_id'
				$date_conditions
			GROUP BY t.contact_id
			";
	
	return DB::executeAll($sql);
}

function get_non_worked_time_by_person($date_from, $date_to) {
	$tz_offset = logged_user()->getUserTimezoneHoursOffset();
	
	$date_conditions = "";
	if ($date_from instanceof DateTimeValue) {
		$d_from = new DateTimeValue($date_from->getTimestamp());
		$d_from->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` >= ".DB::escape($d_from->toMySQL());
	}
	if ($date_from instanceof DateTimeValue) {
		$d_to = new DateTimeValue($date_to->getTimestamp());
		$d_to->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` <= ".DB::escape($d_to->toMySQL());
	}
	
	$overtime_dim_id = Dimensions::findByCode('overtime_hours')->getId();
	$normal_work_member_id = config_option('normal_work_member_id');
	
	$sql = "SELECT t.contact_id, m.id as member_id, m.name as member_name, SUM(worked_time) as total_minutes 
			FROM ".TABLE_PREFIX."timeslots t
			INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
			WHERE 
				o.trashed_by_id=0
				AND m.dimension_id = '$overtime_dim_id'
				AND m.id != '$normal_work_member_id'
				$date_conditions
			GROUP BY t.contact_id, m.id
			";
	
	return DB::executeAll($sql);
}


function accumulate_timeslots_worked_time($timeslots_by_day) {
	// array to accumulate the worked time by type
	$accumulated_minutes = array(
			'regular' => 0,
			'overtime' => 0,
			'double_overtime' => 0,
	);
	//Logger::log_r($timeslots_by_day);
	foreach ($timeslots_by_day as $day => $timeslots) {

		$previous_normal_minutes = 0;
		$previous_extra_minutes = 0;
		
		foreach ($timeslots as $timeslot) {
			$calculated_minutes = calculate_timeslot_worked_time($timeslot, $previous_normal_minutes, $previous_extra_minutes);
	
			$accumulated_minutes['regular'] += $calculated_minutes['regular'];
			$accumulated_minutes['overtime'] += $calculated_minutes['overtime'];
			$accumulated_minutes['double_overtime'] += $calculated_minutes['double_overtime'];

			$previous_normal_minutes += $calculated_minutes['regular'];
			$previous_extra_minutes += $calculated_minutes['overtime'];
		}
	}
	
	return $accumulated_minutes;
}

function calculate_timeslot_worked_time(Timeslot $timeslot, $previous_normal_minutes, $previous_extra_minutes) {
	$tz_offset = logged_user()->getUserTimezoneHoursOffset();
	
	// worked hour members
	$normal_work_member_id = config_option('normal_work_member_id');
	$worked_hours_member_ids = array_filter(array($normal_work_member_id));
	
	// array to accumulate the worked time by type
	$calculated_minutes = array(
			'regular' => 0,
			'overtime' => 0,
			'double_overtime' => 0,
	);
	
	// Ignore this timeslot for calculations if it is classified in a member of non-worked hours (holiday, sick, etc)
	$original_member_ids = $timeslot->getMemberIds();
	$overtime_dimension = Dimensions::findByCode('overtime_hours');
	foreach ($original_member_ids as $original_member_id) {
		$m = Members::getMemberById($original_member_id);
		if ($m->getDimensionId() == $overtime_dimension->getId() && !in_array($m->getId(), array($normal_work_member_id))) {
			return $calculated_minutes;
		}
	}
	
	
	// START THE CALCULATIONS
	//Initialize values from configuration variables
	$max_normal_work_minutes = 60 * config_option('normal_work_hours');
	$max_overtime_work_minutes = 60 * config_option('overtime_work_hours');
	$max_normal_work_minutes_per_week = 60 * config_option('normal_work_hours_per_week');
	
	// Initial date (in GMT-0)
	$timeslot_start_date = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
	
	// Calculate the total amount of minutes of the current timeslot
	$current_timeslot_minutes = floor(($timeslot->getEndTime()->getTimestamp() - $timeslot->getStartTime()->getTimestamp()) / 60) - $timeslot->getSubtract()/60;
	
	// Calculate the remaining normal minutes available for that day, checking other timeslots entered on this day
	$previous_worked_minutes = get_contact_worked_time($timeslot, $timeslot_start_date);
	   
	// calculate the start aof the timeslot's week
	$week_limits = calculate_week_start_and_week_end($timeslot);
	$week_start = $week_limits['start'];
	
	// calculate the beggining of the day
	$day_start = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
	$day_start->add('h', $tz_offset);
	$day_start->beginningOfDay();
	$day_start->add('h', -1 * $tz_offset);
	
	
	// CHECK #1) 7th CONSECUTIVE DAY RULE
	// CheckS if the user has worked 6 consecutive days before this timeslot.
	$has_worked_too_many_consecutive_days_for_overtime = check_consecutive_days($timeslot, $timeslot_start_date);
	
	// If so, then all minutes are overtime. After 10 hours all are double overtime
	if ($has_worked_too_many_consecutive_days_for_overtime) {
		// no normal minutes left if user has worked all the consecutive days for overtime
		$week_remaining_normal_minutes = 0;
	
		// in this case double overtime is after 10 hours of overtime (12-2)
		$max_overtime_work_minutes -= 60 * 2;
	
	} else {
		// CHECK #2) 40 HOURS PER WEEK RULE
		// Check how much regular-time work has been logged so far this week
		// Get the current week's total number of worked hours classified as regular (normal)
		$current_week_normal_worked_minutes = get_normal_worked_minutes($timeslot, $week_start, $timeslot_start_date);
		
		// cap the calculated normal minutes to the max normal minutes per week
		if ($current_week_normal_worked_minutes > $max_normal_work_minutes_per_week) {
			$current_week_normal_worked_minutes = $max_normal_work_minutes_per_week;
		}
	
		// If there are more than 40 hours, $week_remaining_normal_minutes is now 0.
		// (No more work will be under regular hours)
		// Update the caps to calculate normal and overtime hours if necesary (depending on the max hours per week)
		$week_remaining_normal_minutes = $max_normal_work_minutes_per_week - $current_week_normal_worked_minutes;
		if ($week_remaining_normal_minutes < 0) $week_remaining_normal_minutes = 0;
	}
	
	
	// CHECK #3)
	// Check if the space for regular hours in the week (max of 40) is alreay below 10
	// In which case we'll have to fill-up with more overtime today;
	//   * up until 12 hours
	//     * after which everything will be double-overtime
	if ($max_normal_work_minutes > $week_remaining_normal_minutes) {
		// We do this by increasing the max_overtime_hours for today (from 2)
		$max_overtime_work_minutes += ($max_normal_work_minutes - $week_remaining_normal_minutes);
		// Update the normal work minutes variable to make the calculations using the remaining minutes of this week
		$max_normal_work_minutes = $week_remaining_normal_minutes;
	}
	
	
	// CHECK #4) IF ALL GOOD, SIMPLY ADD REGULAR HOURS
	// Check if the sum of (timeslot + previous work) still does not exceed the max amount of normal minutes available
	// If so, return only regular time
	if ($current_timeslot_minutes + $previous_worked_minutes <= $max_normal_work_minutes) {
		
		$calculated_minutes['regular'] = $current_timeslot_minutes;
		return $calculated_minutes;
	
	} else {
		 
		// Calculate regular, overtime and double overtime worked minutes
	
		// get the current week's normal worked time
		$current_day_normal_worked_minutes = $previous_normal_minutes;// get_normal_worked_minutes($timeslot, $day_start, $timeslot_start_date);
		
		// discount the already worked minutes to get the max_normal_minutes remaining
		//$max_normal_minutes = $max_normal_work_minutes - $previous_worked_minutes;
		$max_normal_minutes = $max_normal_work_minutes - $current_day_normal_worked_minutes;
		if ($max_normal_minutes < 0) $max_normal_minutes = 0;
	
		// calculate normal and overtime minutes using current_timeslot_minutes
		$normal_minutes = $current_timeslot_minutes;
		$extra_minutes = 0;
		if ($current_timeslot_minutes > $max_normal_minutes) {
			$normal_minutes = $max_normal_minutes;
			$extra_minutes = $current_timeslot_minutes - $max_normal_minutes;
		}
	
		
		// calculate overtime and double overtime
		if ($extra_minutes > 0) {
			
			// check if there are double overtime minutes
			$double_extra_minutes = 0;
			if ($extra_minutes + $previous_extra_minutes > $max_overtime_work_minutes) {
				$remaining_overtime_minutes = $max_overtime_work_minutes - $previous_extra_minutes;
				if ($remaining_overtime_minutes < 0) $remaining_overtime_minutes = 0;
	
				if ($extra_minutes > $remaining_overtime_minutes) {
					$double_extra_minutes = $extra_minutes - $remaining_overtime_minutes;
					$extra_minutes = $remaining_overtime_minutes;
				}
			}
		}

		$calculated_minutes['regular'] = $normal_minutes;
		$calculated_minutes['overtime'] = $extra_minutes;
		$calculated_minutes['double_overtime'] = $double_extra_minutes;
		
	}
	
	return $calculated_minutes;
}


function get_timeslots_grouped_by_user_and_date($date_from, $date_to, $user_id) {
	
	$tz_offset = logged_user()->getUserTimezoneHoursOffset();

	$date_conditions = "";
	if ($date_from instanceof DateTimeValue) {
		$d_from = new DateTimeValue($date_from->getTimestamp());
		$d_from->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` > ".DB::escape($d_from->toMySQL());
	}
	if ($date_to instanceof DateTimeValue) {
		$d_to = new DateTimeValue($date_to->getTimestamp());
		$d_to->add('h', -1*$tz_offset);
		$date_conditions .= " AND `start_time` <= ".DB::escape($d_to->toMySQL());
	}
	
	$user_condition = "";
	if (is_numeric($user_id) && $user_id > 0) {
		$user_condition .= " AND contact_id=$user_id";
	}
	
	// worked hour members
	$normal_work_member_id = config_option('normal_work_member_id');
	
	// condition to take only the timeslots classified in worked hours
	$worked_time_sql = "AND EXISTS (SELECT om2.object_id FROM ".TABLE_PREFIX."object_members om2
		WHERE om2.member_id IN ('$normal_work_member_id') AND om2.object_id=o.id)";
	
	// get the timeslots
	$timeslots = Timeslots::instance()->findAll(array(
			"conditions" => "o.trashed_by_id = 0
				AND `end_time` > 0
				$worked_time_sql
				$date_conditions
				$user_condition",
			"order" => "contact_id, start_time"
	));
	
	// array to temporary store the timeslot users
	$contacts_cache = array();
	
	$timeslots_by_user_and_date = array();
	foreach ($timeslots as $timeslot) {
		
		/* @var $timeslot Timeslot */
		$ts_date = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
		$ts_date->add('h', $tz_offset);
		$ts_date_str = $ts_date->format('Y-m-d');
		
		// find the user and update cache if necesary
		$user = array_var($contacts_cache, $timeslot->getContactId());
		if (!$user instanceof Contact) {
			$user = Contacts::findById($timeslot->getContactId());
			$contacts_cache[$user->getId()] = $user;
		}
		
		// use name + id as user key to sort them alphabetically
		$user_key = trim(strtolower($user->getObjectName()))."_|_".$user->getId();
		
		// initialize the users array
		if (!isset($timeslots_by_user_and_date[$user_key])) $timeslots_by_user_and_date[$user_key] = array();
		// initialize the date array
		if (!isset($timeslots_by_user_and_date[$user_key][$ts_date_str])) $timeslots_by_user_and_date[$user_key][$ts_date_str] = array();
		
		$timeslots_by_user_and_date[$user_key][$ts_date_str][] = $timeslot;
		
	}
	
	// ensure that all date arrays inside each user are ordered by date key
	foreach ($timeslots_by_user_and_date as $user_k => &$ts_date_array) {
		ksort($ts_date_array);
	}
	// order the array by the users key (alphabetically)
	ksort($timeslots_by_user_and_date);

	// return the grouped timeslots
	return $timeslots_by_user_and_date;
	
}



function get_contact_worked_time(Timeslot $timeslot, DateTimeValue $date, $additional_conditions='', $end_day=null) {

	$tz_offset = logged_user()->getUserTimezoneHoursOffset();

	// start date to filter the query
	$d = new DateTimeValue($date->getTimestamp());
	$d->add('h', $tz_offset);
	$d->beginningOfDay();
	$d->add('h', -1*$tz_offset);

	// end date to filter the query
	if (is_null($end_day)) {
		$end_day_time = new DateTimeValue($date->getTimestamp());
		$end_day_time->add('h', $tz_offset);
		$end_day_time->endOfDay();
	} else {
		$end_day_time = new DateTimeValue($end_day->getTimestamp());
	}
	$end_day_time->add('h', -1*$tz_offset);

	$contact_id = $timeslot->getContactId();
	$ts_id_to_exclude = $timeslot->getId();

	$normal_work_member_id = config_option('normal_work_member_id');

	$worked_time_sql = "AND EXISTS (SELECT om2.object_id FROM ".TABLE_PREFIX."object_members om2
	WHERE om2.member_id IN ('$normal_work_member_id') AND om2.object_id=t.object_id)";

	// build sql
	$sql = "SELECT SUM(GREATEST(TIMESTAMPDIFF(MINUTE,start_time,end_time),0)) - SUM(subtract/60) as total
			FROM `".TABLE_PREFIX."timeslots` t
			INNER JOIN `".TABLE_PREFIX."objects` o ON o.id=t.object_id
			WHERE `contact_id` = ". $contact_id ." AND `object_id` != '$ts_id_to_exclude'
			$worked_time_sql
			AND o.trashed_by_id = 0
			AND `end_time` > 0
			AND `start_time` >= ".DB::escape($d->toMySQL())."
			AND `start_time` <= ".DB::escape($end_day_time->toMySQL())."
			$additional_conditions";

	$row = DB::executeOne($sql);

	return array_var($row, "total", 0);
}


function get_normal_worked_minutes($timeslot, $from_date, $to_date) {
	
	$normal_work_member_id = config_option('normal_work_member_id');
	
	$worked_time_sql = "AND EXISTS (SELECT om2.object_id FROM ".TABLE_PREFIX."object_members om2
	WHERE om2.member_id IN ('$normal_work_member_id') AND om2.object_id=e.object_id)";
	
	
	$timeslots = Timeslots::findAll(array(
		"conditions" => "
			`contact_id` = ". $timeslot->getContactId() ." AND `object_id` != ".$timeslot->getId()."
			$worked_time_sql
			AND o.trashed_by_id = 0
			AND `end_time` > 0
			AND `start_time` >= ".DB::escape($from_date->toMySQL())."
			AND `start_time` <= ".DB::escape($to_date->toMySQL())."
	"));
	
	$max_normal_work_minutes = 60 * config_option('normal_work_hours');
	
	$normal_minutes = 0;
	foreach ($timeslots as $t) {
		/* @var $t Timeslot */
		$worked_minutes = $t->getColumnValue('worked_time');
		if ($worked_minutes > $max_normal_work_minutes) {
			$worked_minutes = $max_normal_work_minutes;
		}
		$normal_minutes += $worked_minutes;
	}
	
	return $normal_minutes;
}



function calculate_week_start_and_week_end(Timeslot $timeslot) {

	// start time in gmt
	$timeslot_start_date = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
	$tz_offset = logged_user()->getUserTimezoneHoursOffset();

	// apply timezone to get the correct day
	$timeslot_start_date->add('h', $tz_offset);
	$day = $timeslot_start_date->format('w');


	$week_start = new DateTimeValue(strtotime('-'.$day.' days', $timeslot_start_date->getTimestamp()));
	$week_start->add('h', $tz_offset);
	$week_start->beginningOfDay();
	$week_start->add('h', -1 * $tz_offset);

	$week_end = new DateTimeValue(strtotime('+'.(6-$day).' days', $timeslot_start_date->getTimestamp()));
	$week_end->add('h', $tz_offset);
	$week_end->endOfDay();
	$week_end->add('h', -1 * $tz_offset);

	$result = array('start' => $week_start, 'end' => $week_end);

	return $result;
}


/**
 * 	This function checks if the user has worked 6 consecutive days before this timeslot.
 *  If so, then all minutes are overtime
 *
 */
function check_consecutive_days($timeslot, $timeslot_start_date) {

	$has_worked_past_max_consecutive_days_for_overtime = false;

	$max_consecutive_days_for_overtime = 6;

	$normal_work_member_id = config_option('normal_work_member_id');

	$tz_offset = logged_user()->getUserTimezoneHoursOffset();

	// start date to filter the query
	$current_day_start = new DateTimeValue($timeslot_start_date->getTimestamp());
	$current_day_start->add('h', $tz_offset);
	$current_day_start->beginningOfDay();
	$current_day_start->add('h', -1*$tz_offset);

	$worked_time_sql = "AND EXISTS (SELECT om2.object_id FROM ".TABLE_PREFIX."object_members om2
	WHERE om2.member_id IN ('$normal_work_member_id') AND om2.object_id=t.object_id)";

	$consecutive_days_sql = "
    		SELECT DATE(start_time) AS day
			FROM ".TABLE_PREFIX."timeslots t
			INNER JOIN `".TABLE_PREFIX."objects` o ON o.id=t.object_id
			WHERE `contact_id` = ". $timeslot->getContactId() ." AND o.trashed_by_id = 0
			$worked_time_sql
			AND end_time > 0
			AND start_time < '".$current_day_start->toMySQL()."'
			AND start_time > ADDDATE('".$current_day_start->toMySQL()."', INTERVAL -".$max_consecutive_days_for_overtime." DAY)
			GROUP BY DATE(day);
	";
				
	$consecutive_days_rows = DB::executeAll($consecutive_days_sql);

	// the amount of rows is the amount of worked days, check if it reached the $max_consecutive_days_for_overtime
	if (count($consecutive_days_rows) >= $max_consecutive_days_for_overtime) {
		$has_worked_past_max_consecutive_days_for_overtime = true;
	}

	return $has_worked_past_max_consecutive_days_for_overtime;
}






