<?php

Hook::register("paemfe");

if(!defined('ALLOW_UPGRADING')) define('ALLOW_UPGRADING', false);

function paemfe_render_page_actions($object, &$ignored) {
	if ($object instanceof ProjectTask && $object->getColumnValue('object_subtype_id') > 0) {
		Env::useHelper('constants', 'paemfe');
		
		add_page_action(lang('print receipt'), get_url('print_receipt', 'print_receipt', array('id' => $object->getId())) , 'ico-print', '_blank', null, true);
		
		// additional actions for service tasks
		$subtype = ObjectSubtypes::findById($object->getColumnValue('object_subtype_id'));
		if ($subtype instanceof ObjectSubtype && $subtype->getCode()=='servicio') {
			
			$cp_value = CustomPropertyValues::getCustomPropertyValue($object->getId(), SERVICE_STARTDATE_CP_ID);
			// if already started don't add the action
			if (!$cp_value instanceof CustomPropertyValue || trim($cp_value->getValue()) == '') {
				$url = get_url("paemfe", "start_service", array("id" => $object->getId()));
				add_page_action(lang('start service'), $url, 'ico-arrowright', null, null, true);
			}
		}
	}
}

function paemfe_render_administration_icons($ignored, &$icons) {
	$icons[] = array(
		'ico' => 'ico-large-image',
		'url' => get_url('print_receipt', 'upload_receipt_image'),
		'name' => lang('receipt images'),
	);
}



function paemfe_subtype_cps_extra_conditions($subtype_id, &$extra_conditions) {
	
	if (is_numeric($subtype_id) && $subtype_id > 0) {
		//$extra_conditions .= " OR EXISTS (SELECT * FROM ".TABLE_PREFIX."custom_properties_by_co_type tt WHERE tt.co_type_id=$subtype_id AND tt.cp_id=id) ";
		$extra_conditions = " AND id IN (SELECT cp_id FROM ".TABLE_PREFIX."custom_properties_by_co_type tt WHERE tt.co_type_id=$subtype_id) AND object_subtype_id>=0";
	}
}

// only administrators can reopen tasks
function paemfe_can_change_task_status($params, &$result) {
	$task = array_var($params, 'task');
	$user = array_var($params, 'user');
	if ($task instanceof ProjectTask && $user instanceof Contact && $user->isUser()) {
		if ($task->isCompleted() && !$user->isAdminGroup()) {
			$result = false;
		}
	}
}


function paemfe_before_object_save($task, $ret) {/* @var $task ProjectTask */
	if (!$task instanceof ProjectTask) return;
	
	Env::useHelper('constants', 'paemfe');
	
	if (!logged_user()->isAdministrator()) {
		
		// actions to take when completing tasks
		if (array_var($_REQUEST, 'a') == 'complete_task') {
			
			// set date custom property equal to completed date, depending on task type
			if ($task->getColumnValue('object_subtype_id') > 0) {
				$subtype = ObjectSubtypes::findById($task->getColumnValue('object_subtype_id'));
				switch ($subtype->getCode()) {
					case 'inst_au':
						CustomPropertyValues::deleteCustomPropertyValues($task->getId(), INSTALLATION_DATE_CP_ID);
						$cpv = new CustomPropertyValue();
						$cpv->setObjectId($task->getId());
						$cpv->setCustomPropertyId(INSTALLATION_DATE_CP_ID);
						$cpv->setValue($task->getCompletedOn()->toMySQL());
						$cpv->save();
						break;
					case 'servicio':
						CustomPropertyValues::deleteCustomPropertyValues($task->getId(), SERVICE_ENDDATE_CP_ID);
						$cpv = new CustomPropertyValue();
						$cpv->setObjectId($task->getId());
						$cpv->setCustomPropertyId(SERVICE_ENDDATE_CP_ID);
						$cpv->setValue($task->getCompletedOn()->toMySQL());
						$cpv->save();
						break;
					default:
						break;
				}
			}
			
			
			// copy tasks custom properties to the linked company
			$company = null;
			$linked = $task->getLinkedObjects();
			foreach($linked as $lo) {
				if ($lo instanceof Contact && $lo->getIsCompany()) {
					$company = $lo;
					break;
				}
			}
			if ($company instanceof Contact && $company->getIsCompany()) {
				$cp_ids_match = array(
					TASKS_INSTALLED_PCS_CP_ID => INSTALLED_PCS_CP_ID,
					TASKS_INSTALLED_PRINTERS_CP_ID => INSTALLED_PRINTERS_CP_ID,
					TASKS_INSTALLED_MOUSES_CP_ID => INSTALLED_MOUSES_CP_ID,
					TASKS_INSTALLED_SWITCHES_CP_ID => INSTALLED_SWITCHES_CP_ID,
					TASKS_INSTALLED_WIRES_CP_ID => INSTALLED_WIRES_CP_ID,
				);
				foreach ($cp_ids_match as $cp => $copycp) {
					$cp_vals = CustomPropertyValues::getCustomPropertyValues($task->getId(), $cp);
					foreach ($cp_vals as $cp_val) {
						$copy = new CustomPropertyValue();
						$copy->setCustomPropertyId($copycp);
						$copy->setObjectId($company->getId());
						$copy->setValue($cp_val->getValue());
						$copy->save();
					}
				}
			}
			
		} else {
			// actions to take when adding/editing tasks
			$old_task = ProjectTasks::findById($task->getId());
			if ($old_task instanceof ProjectTask) {
				// keep old start date and due date values, only administrators can change them
				$task->setStartDate($old_task->getStartDate());
				$task->setDueDate($old_task->getDueDate());
				
				// keep old installation date, start service and end service
			}
			
		}
	}
	
	// when any user saves a task
	if ($task->isNew()) {
		$task_data = array_var($_POST, 'task');
		
		if (array_var($task_data, 'object_subtype_id') > 0) {
			$subtype = ObjectSubtypes::findById(array_var($task_data, 'object_subtype_id'));
			
			switch ($subtype->getCode()) {
				case 'inst_au':
				case 'servicio':
					$_POST['object_custom_properties'][SERVICE_DATE_CP_ID] = format_date(DateTimeValueLib::now());
					break;
				default:
					break;
			}
		}
	
	}
	
	return true;
}



function paemfe_draw_additional_task_html($genid, &$task){
	// dont allow non-admins to edit start and due date
	if (!logged_user()->isAdminGroup()) {
		$html = '<script>';
		$html .= '$("#'.$genid.'due_date").closest(".dataBlock").remove();';
		$html .= '$("#'.$genid.'start_date").closest(".dataBlock").remove();';
		$html .= '</script>';
		echo $html;
	}
}

function __paemfe_build_hide_datecp_js($genid, $cp, $date_format_tip) {
	return '
		var val=$("#'.$genid.'cp'.$cp.'").val();
		if (val=="'.$date_format_tip.'") val="";
		$("#'.$genid.'cp'.$cp.'").closest(".date-picker").html(val);';
}

function paemfe_after_render_custom_properties($params, &$ret) {
	// dont allow non-admins to edit some custom properties
	if (!logged_user()->isAdminGroup()) {
		$date_format_tip = date_format_tip(user_config_option('date_format'));
		Env::useHelper('constants', 'paemfe');
		
		$object = array_var($params, 'object');
		$genid = array_var($params, 'genid');
		
		$html = '<script>';
		$html .= '$(function(){ ';
		
		$html .= __paemfe_build_hide_datecp_js($genid, SERVICE_DATE_CP_ID, $date_format_tip);
		$html .= __paemfe_build_hide_datecp_js($genid, SERVICE_ENDDATE_CP_ID, $date_format_tip);
		$html .= __paemfe_build_hide_datecp_js($genid, SERVICE_STARTDATE_CP_ID, $date_format_tip);
		$html .= __paemfe_build_hide_datecp_js($genid, INSTALLATION_DATE_CP_ID, $date_format_tip);
		
		$html .= '});';
		$html .= '</script>';
		echo $html;
	}
}


if (!defined('DONE_TASKS_REPORT')) define('DONE_TASKS_REPORT', 165999);

function paemfe_get_more_columns_to_header_report($params, &$results) {
	
	//$object_type = array_var ( $params, 'object_type' );
	$report = array_var ( $params, 'report' );
	if ($report->getId() == DONE_TASKS_REPORT) {
		$results ['columns'] ['names'] ['beneficiario'] = 'Beneficiario';
		array_unshift($results ['columns'] ['order'], 'beneficiario');
		$results ['columns'] ['types'] ['beneficiario'] = DATA_TYPE_STRING;
		
		$results ['columns'] ['names'] ['dim_1'] = 'Subsistema';
		
		$results ['columns'] ['names'] ['dim_2'] = 'Departamento';
	}
}


function paemfe_report_results_more_columns_value($params, &$row_values) {
	$report = array_var($params, 'report');
	$object = array_var($params, 'object');
	/* @var $object ContentDataObject */
	
	if ($report->getId() == DONE_TASKS_REPORT) {
		
		$company_name = '';
		$linked_objects = $object->getLinkedObjects();
		foreach ($linked_objects as $lo) {
			if ($lo instanceof Contact) {
				$company_name = $lo->getObjectName();
				break;
			}
		}
		
		$row_values['beneficiario'] = $company_name;
		
		if (strpos($row_values['dim_1'], '/') !== false) {
			$row_values['dim_1'] = substr($row_values['dim_1'], strpos($row_values['dim_1'], '/')+1);
		}
	}
}

