<?php
Hook::register('suppliers');

function suppliers_member_edit_categories($params, &$categories) {
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$sot = ObjectTypes::findByName('supplier');
	
	if ($member instanceof Member && $sot->getId() == $member->getObjectTypeId()) {
		Env::useHelper('contact_render_tab_functions');
		
		$contact = null;
		$supplier = Suppliers::findById($member->getObjectId());
		if ($supplier instanceof Supplier) {
			$contact = Contacts::findById($supplier->getContactId());
		}
		if ($contact instanceof Contact) {
			$associated_contact = $contact;
			$associated_contact_data = $contact->getArrayInfo();
		} else {
			$associated_contact = new Contact();
			$associated_contact_data = array();
		}
		
		$categories[] = array(
			'id' => $genid . 'contact_data',
			'link_id' => $genid . 'contact_data_tab',
			'name' => lang('contact data'),
			'content' => get_contact_data_tab_html($genid, $associated_contact, false, $associated_contact_data),
			'style' => 'display:none;',
			'class' => 'contact-data-container',
		);
		
		$categories[] = array(
			'id' => $genid . 'company_data',
			'link_id' => $genid . 'company_data_tab',
			'name' => lang('company data'),
			'content' => get_company_data_tab_html($genid, $associated_contact, false, $associated_contact_data),
			'style' => 'display:none;',
			'class' => 'contact-data-container',
		);
		
		echo "<script>$(function(){og.renderContactDataFields('$genid', $('#".$genid."supplier_ot_combo').val());});</script>";
	}
}



function suppliers_additional_member_data_fields($params, &$fields) {
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$sot = ObjectTypes::findByName('supplier');
	
	if ($member instanceof Member && $sot->getId() == $member->getObjectTypeId()) {
		
		$contact_ot = ObjectTypes::findByName('contact');
		$company_ot = ObjectTypes::findByName('company');
		
		$contact_types = array(array($company_ot->getId(), lang($company_ot->getName())), array($contact_ot->getId(), lang($contact_ot->getName())));
		
		$sel_type_id = $company_ot->getId();
		if (!$member->isNew()) {
			$obj = Objects::findObject($member->getObjectId());
			if ($obj instanceof Supplier) {
				$sel_type_id = $obj->getContactObjectTypeId();
			}
		}
		
		$html = '<div class="field">';
		$html .= simple_select_box("supplier[contact_data_type]", $contact_types, $sel_type_id,
				array('onchange' => "og.renderContactDataFields('$genid', this.value)", 'id' => $genid."supplier_ot_combo"));
		$html .= '</div>';
		
		$fields[] = array('input_html' => $html, 'label' => lang("this supplier is"));
	}
}


function suppliers_render_additional_member_custom_properties($params, &$ignored) {
	
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$ot = array_var($params, 'ot');
	
	if ($ot->getName() == 'supplier') {
	
		$has_custom_properties_contact = count(CustomProperties::getCustomPropertyIdsByObjectType(Contacts::instance()->getObjectTypeId())) > 0;
		if ($has_custom_properties_contact) {
			$obj = Suppliers::findById($member->getObjectId());
			$contact = null;
			if ($obj instanceof Supplier) {
				$contact = Contacts::findById($obj->getContactId());
			}
			if (is_null($contact)) {
				$contact = new Contact();
			}
			
			echo '<div id="'.$genid.'add_contact_custom_properties_div" class="contact-data-container">';
			echo render_object_custom_properties($contact);
			echo '</div>';
			echo '<script>$("#'.$genid.'_no_cp_msg").hide();</script>';
		}
	}
}



function suppliers_after_add_dimension_object_member($params, &$ignored) {
	$member = array_var($params, 'member');
	$is_new = array_var($params, 'is_new');
	$sot = ObjectTypes::findByName('supplier');

	if ($member instanceof Member && $sot->getId() == $member->getObjectTypeId()) {
		
		$object_controller = new ObjectController();
		$supplier = Objects::findObject($member->getObjectId());
		
		$sup_type_id = array_var(array_var($_REQUEST, 'supplier'), 'contact_data_type');
		$type_obj = ObjectTypes::findById($sup_type_id);
		$name = $member->getName();
		
		
		if ($type_obj->getName() == 'company') {
			
			$company_data = array_var($_POST, 'company');
			
			$contact_email = $company_data['email'];
			
			$existing_contact = null;
			if (!$is_new && $supplier->getContactId() > 0 && $supplier->getContactObjectTypeId() == $type_obj->getId()) {
				$existing_contact = Contacts::findById($supplier->getContactId());
			}
			
			if (!$existing_contact && trim($contact_email) != "") {
				$contact_emails = ContactEmails::findAll(array('conditions' => array("`email_address` = ?", $contact_email)));
				foreach ($contact_emails as $ce) {
					if (can_read_sharing_table(logged_user(), $ce->getContactId())) {
						$existing_contact = Contacts::findById($ce->getContactId());
						break;
					}
				}
			
			}
			
			if ($existing_contact instanceof Contact && $existing_contact->getIsCompany() && can_read_sharing_table(logged_user(), $existing_contact->getId())) {
			
				$supplier->setContactId($existing_contact->getId());
				$company = $existing_contact;
				
			} else {
				$company = new Contact();
			}
			
			if($company_data){
				$company->setFromAttributes($company_data);
			}
			$company->setIsCompany(true);
			$company->setFirstName($name);
			$company->setObjectName();
			
			if (isset($_SESSION['new_contact_picture']) && $_SESSION['new_contact_picture']) {
				$company->setPictureFile($_SESSION['new_contact_picture']);
				$_SESSION['new_contact_picture'] = null;
			}
			
			$company->add_skip_validation('email');
			$company->save();
			
			if($company_data['email'] != "") {
				$c_email = $company->getEmail();
				if ($c_email instanceof ContactEmail) {
					$c_email->delete();
				}
				$company->addEmail($company_data['email'], 'work', true);
			}
			
			if($company_data){
				$contact_controller = new ContactController();
				$contact_controller->save_phones_addresses_webpages($company_data, $company);
				$contact_controller->save_non_main_emails($company_data, $company);
			}
			
			$object_controller->add_custom_properties($company);
			$object_controller->add_to_members($company, array($member->getId()));
			
			$supplier->setContactId($company->getId());
			$supplier->setContactObjectTypeId($type_obj->getId());
			$supplier->save();
			
		} else if ($type_obj->getName() == 'contact') {
			
			$contact_data = array_var($_POST, 'contact');
			$contact_email = $contact_data['email'];
			
			$existing_contact = null;
			if (!$is_new && $supplier->getContactId() > 0 && $supplier->getContactObjectTypeId() == $type_obj->getId()) {
				$existing_contact = Contacts::findById($supplier->getContactId());
			}
			
			if (!$existing_contact && trim($contact_email) != "") {
				$contact_emails = ContactEmails::findAll(array('conditions' => array("`email_address` = ?", $contact_email)));
				foreach ($contact_emails as $ce) {
					if (can_read_sharing_table(logged_user(), $ce->getContactId())) {
						$existing_contact = Contacts::findById($ce->getContactId());
						break;
					}
				}
				
			}
			
			if ($existing_contact instanceof Contact && !$existing_contact->getIsCompany() && can_read_sharing_table(logged_user(), $existing_contact->getId())) {
				
				$supplier->setContactId($existing_contact->getId());
				$contact = $existing_contact;
				
			} else {
				$contact = new Contact();
				$contact->setFirstName($name);
				$contact->setObjectName($name);
				
				// if new company added
				$comp_data = array_var($_POST, 'company');
				if (trim(array_var($comp_data, 'first_name', '')) != '') {
					$comp = new Contact();
					if($comp_data){
						$comp->setFromAttributes($comp_data);
					}
					$comp->setIsCompany(true);
					$comp->setObjectName();
					
					$comp->save();
					
					if(trim(array_var($comp_data, 'email', '')) != "") $comp->addEmail($comp_data['email'], 'personal', true);
					
					$contact_controller = new ContactController();
					$contact_controller->save_phones_addresses_webpages($comp_data, $comp);
					
					$contact_data['company_id'] = $comp->getId();
				}
				// -- end add new company
			}
			
			if($contact_data){
				$contact->setFromAttributes($contact_data);
			}
			$contact->add_skip_validation('email');
			$contact->save();
			
			$contact_data['email'] = $contact_email;
			
			if($contact_email != "") {
				$c_email = $contact->getEmail();
				if ($c_email instanceof ContactEmail && $c_email->getEmailAddress() != $contact_email) {
					$c_email->delete();
				}
				$contact->addEmail($contact_email, 'personal', true);
			}
			
			if($contact_data){
				$contact_controller = new ContactController();
				$contact_controller->save_phones_addresses_webpages($contact_data, $contact);
				$contact_controller->save_non_main_emails($contact_data, $contact);
			}
			$object_controller->add_custom_properties($contact);
			$object_controller->add_to_members($contact, array($member->getId()));
			
			$supplier->setContactId($contact->getId());
			$supplier->setContactObjectTypeId($type_obj->getId());
			$supplier->save();
			
		}
	}
}




function suppliers_types_after_auto_classifying_associated_object_of_member($parameters, &$ignored) {
	$rel_obj = array_var($parameters, 'obj');
	$member = array_var($parameters, 'mem');
	if (!$member instanceof Member) return;

	if ($rel_obj instanceof Supplier) {
		$rel_contact = Objects::findObject($rel_obj->getContactId());
		if ($rel_contact instanceof Contact) {
			ObjectMembers::addObjectToMembers($rel_contact->getId(), array($member));
		}
	}
}


function suppliers_custom_reports_additional_columns($args, &$ret) {
	// the dim columns are added by this plugin too
	if (Plugins::instance()->isActivePlugin('advanced_reports')) return;
	
	$ot = array_var($args, 'object_type');
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('dimension_object', 'dimension_group'))) return;

	$dimensions = Dimensions::findAll ( array("conditions" => "code IN ('suppliers')") );
	foreach ($dimensions as $dimension) {
		if (in_array($dimension->getId(), config_option('enabled_dimensions'))) {

			$name = $dimension->getName();

			$ret[] =  array('id' => 'dim_'.$dimension->getId(), 'name' => $name, 'type' => DATA_TYPE_STRING);
		}
	}
}