<?php

/**
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */

class ProjectReportsController extends ApplicationController {

	/**
	 * Construct the ReportingController
	 *
	 * @access public
	 * @param void
	 * @return ReportingController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('grouping');
		Env::useHelper('reporting');
		Env::useHelper('advanced_reports', 'advanced_reports');
		Env::useHelper('functions', 'advanced_billing');
		Env::useHelper('functions', 'expenses2');
		Env::useHelper('functions', 'project_reports');
		Env::useHelper('aging_invoices_report_functions', 'project_reports');
		Env::useHelper('aging_invoices_report_csv_functions', 'project_reports');
		Env::useHelper('aging_invoices_report_excel_functions', 'project_reports');
		Env::useHelper('invoice_report_functions', 'project_reports');
		Env::useHelper('invoice_report_excel_functions', 'project_reports');
		Env::useHelper('invoice_report_csv_functions', 'project_reports');
		Env::useHelper('budget_report_excel_functions', 'project_reports');
		Env::useHelper('budget_report_csv_functions', 'project_reports');
		Env::useHelper('utilization_report_functions', 'project_reports');
		Env::useHelper('utilization_report_csv_functions', 'project_reports');
		Env::useHelper('utilization_report_excel_functions', 'project_reports');
		Env::useHelper('ar_ledger_report_functions', 'project_reports');
		Env::useHelper('ar_ledger_report_csv_functions', 'project_reports');
		Env::useHelper('ar_ledger_report_excel_functions', 'project_reports');
		Env::useHelper('project_detail_jdt_report_functions', 'project_reports');
		Env::useHelper('project_detail_jdt_report_csv_functions', 'project_reports');
		Env::useHelper('project_detail_jdt_report_excel_functions', 'project_reports');
		Env::useHelper('project_earnings_report_functions', 'project_reports');
		Env::useHelper('project_earnings_report_csv_functions', 'project_reports');
		Env::useHelper('project_earnings_report_excel_functions', 'project_reports');
		Env::useHelper('project_forecast_functions', 'project_reports');
		Env::useHelper('project_forecast_excel_functions', 'project_reports');
		Env::useHelper('project_forecast_csv_functions', 'project_reports');
	} // __construct


	/****
	 *
	 * PROJECT FORECAST REPORT
	 * 
	 ****/
	function project_forecast_report_p(){

		if (!Plugins::instance()->isActivePlugin('income')) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$report_data = array_var($_SESSION, 'project_forecast_report_data');
		if (!$report_data) {
			$report_data = array(
				'project_manager_id' => 0, // all pms
			);
			
			$_SESSION['project_forecast_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_forecast_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function project_forecast_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}
		
		// Get dates
		Env::useHelper('advanced_reports', 'advanced_reports');
		$report_data['date_type'] = 6;
		$report_dates = parse_report_param_date_type($report_data);
		$start_date = array_var($report_dates, 'st');
		$end_date = array_var($report_dates, 'et');
		$report_data['start_date'] = $start_date;
		$report_data['end_date'] = $end_date;

		$_SESSION['project_forecast_report_data'] = $report_data;
		/*$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }*/
        
        // Get report data
		$data = get_main_project_forecast_report_data($report_data);

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_forecast_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('report_data', $report_data);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'project_forecast_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_project_forecast_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_project_forecast_report_excel');

		tpl_assign('report_template_path', get_template_path('project_forecast_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function project_forecast_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->project_forecast_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('project forecast report').".csv"));
			$rows_csv = build_project_forecast_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('project_forecast_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			
			$this->project_forecast_report($_POST);
			tpl_display(get_template_path('project_forecast_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();

			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			$zoom = '--zoom 0.8';
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "project_forecast_report", $page_size, $zoom);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function project_forecast_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->project_forecast_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('project forecast report').".xlsx"));
		
		$excel_data = build_project_forecast_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('project forecast report'), 'fixed_cells_width' => 30), 'headers' => array(), 'data' => array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 *
	 * PROJECT EARNINGS REPORT
	 * 
	 ****/
	function project_earnings_report_p(){

		if (!Plugins::instance()->isActivePlugin('income')) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$report_data = array_var($_SESSION, 'project_earnings_report_data');
		if (!$report_data) {
			$report_data = array(
				'project_manager_id' => 0, // all pms
			);
			
			$_SESSION['project_earnings_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_earnings_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function project_earnings_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['project_earnings_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        // Get report data
		$data = get_project_earnings_report_data($report_data);

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_earnings_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'project_earnings_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_project_earnings_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_project_earnings_report_excel');

		tpl_assign('report_template_path', get_template_path('project_earnings_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function project_earnings_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->project_earnings_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('project earnings report').".csv"));
			$rows_csv = build_project_earnings_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('project_earnings_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_earnings_report_pdf.css" );

			$this->project_earnings_report($_POST);
			tpl_display(get_template_path('project_earnings_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			$zoom = '--zoom 0.8';
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "project_earnings_report", $page_size, $zoom);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function project_earnings_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->project_earnings_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('project earnings report').".xlsx"));
		
		$excel_data = build_project_earnings_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('project earnings report'), 'fixed_cells_width' => 30), 'headers' => array(), 'data' => array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 *
	 * PROJECT JDT DETAIL REPORT
	 * 
	 ****/
	function project_detail_jdt_report_p(){

		if (!Plugins::instance()->isActivePlugin('income')) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$report_data = array_var($_SESSION, 'project_detail_jdt_report_data');
		if (!$report_data) {
			$report_data = array(
				'date_type' => 0, // no date filter by default
			);
			
			$_SESSION['project_detail_jdt_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_detail_jdt_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function project_detail_jdt_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['project_detail_jdt_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        // Get report data
		$data = get_project_detail_jdt_report_data($report_data);

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_detail_jdt_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'project_detail_jdt_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_project_detail_jdt_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_project_detail_jdt_report_excel');

		tpl_assign('report_template_path', get_template_path('project_detail_jdt_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function project_detail_jdt_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->project_detail_jdt_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('project detail jdt report').".csv"));
			$rows_csv = build_project_detail_jdt_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('project_detail_jdt_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_detail_jdt_report_pdf.css" );

			$this->project_detail_jdt_report($_POST);
			tpl_display(get_template_path('project_detail_jdt_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "project_detail_jdt_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function project_detail_jdt_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->project_detail_jdt_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('project detail jdt report').".xlsx"));
		
		$excel_data = build_project_detail_jdt_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('project detail jdt report'), 'fixed_cells_width' => 30), 'headers' => array(), 'data' => array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 *
	 * AR LEDGER REPORT
	 * 
	 ****/
	function ar_ledger_report_p(){

		if (!Plugins::instance()->isActivePlugin('income')) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$report_data = array_var($_SESSION, 'ar_ledger_report_data');
		if (!$report_data) {
			$report_data = array(
				'date_type' => 0, // no date filter by default
			);
			
			$_SESSION['ar_ledger_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'ar_ledger_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function ar_ledger_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['ar_ledger_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        // Get report data
		$data = get_ar_ledger_report_data($report_data);
		

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'ar_ledger_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'ar_ledger_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_ar_ledger_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_ar_ledger_report_excel');

		tpl_assign('report_template_path', get_template_path('ar_ledger_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function ar_ledger_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->ar_ledger_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('ar ledger report').".csv"));
			$rows_csv = build_ar_ledger_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('ar_ledger_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/ar_ledger_report_pdf.css" );

			$this->ar_ledger_report($_POST);
			tpl_display(get_template_path('ar_ledger_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "ar_ledger_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function ar_ledger_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->ar_ledger_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('ar_ledger report').".xlsx"));
		
		$excel_data = build_ar_ledger_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('ar ledger report'), 'fixed_cells_width' => 30), 'headers' => array(), 'data' => array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 *
	 * UTILIZATION REPORT
	 * 
	 ****/
	function utilization_report_p(){
		
		$report_data = array_var($_SESSION, 'utilization_report_data');
		if (!$report_data) {
			$report_data = array(
				'date_type' => 5, // last month by default
			);
			
			$_SESSION['utilization_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'utilization_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function utilization_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['utilization_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        // Get report data
		$data = get_utilization_report_data($report_data);
		

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'utilization_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'utilization_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_utilization_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_utilization_report_excel');

		tpl_assign('report_template_path', get_template_path('utilization_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function utilization_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->utilization_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('utilization report').".csv"));
			$rows_csv = build_utulization_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('utilization_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/utilization_report_pdf.css" );

			$this->utilization_report($_POST);
			tpl_display(get_template_path('utilization_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "utilization_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function utilization_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->utilization_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('utilization report').".xlsx"));
		
		$excel_data = build_utulization_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('options' => array('title' => lang('utilization report'), 'fixed_cells_width' => 30), 'headers' => $excel_data['headers'], 'data' => $excel_data['data']),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 *
	 * AGING INVOICES REPORT
	 * 
	 ****/

	function aging_invoices_report_p(){
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_data = array_var($_SESSION, 'aging_invoices_report_data');
		if (!$report_data) {
			$report_data = array();
			
			$_SESSION['aging_invoices_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'aging_invoices_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function aging_invoices_report($report_data = null, $csv = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['aging_invoices_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        // Get report data
		$data = get_all_aging_invoices_report_data($report_data);
		

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'aging_invoices_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($csv) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'aging_invoices_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_aging_invoices_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_aging_invoices_report_excel');

		tpl_assign('report_template_path', get_template_path('aging_invoices_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function aging_invoices_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->aging_invoices_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('aging invoices report').".csv"));
			$rows_csv = build_aging_invoices_report_csv($report_result);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('aging_invoices_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/aging_invoices_pdf.css" );

			$this->aging_invoices_report($_POST);
			tpl_display(get_template_path('aging_invoices_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "aging_invoices_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function aging_invoices_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->aging_invoices_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('aging invoices report').".xlsx"));
		
		$excel_data = build_aging_invoices_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('aging invoices report'), 'fixed_cells_width' => 30), 'headers' => array(),'data'=>array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}



	/****
	 *
	 * INVOICE REPORT
	 * 
	 ****/

	function invoice_report_p(){
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_data = array_var($_SESSION, 'invoice_report_data');
		if (!$report_data) {
			$report_data = array();
			
			$_SESSION['invoice_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'invoice_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function invoice_report($report_data = null, $csv = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		$_SESSION['invoice_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        $project_member = null;
        $project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
        if (is_array($project_member_ids) && count($project_member_ids) > 0) {
        	$project_member = Members::getMemberById($project_member_ids[0]);
        }
        if($project_member instanceof Member){
			$data = get_main_invoice_report_data($project_member, $report_data);
		} else {
			$group_report_by = array_var($report_data, 'group_report_by');
			if($group_report_by == 1){
				$data = get_without_grouping_invoice_report_data($report_data);
			} elseif($group_report_by == 2){
				$data = get_grouped_by_project_invoice_report_data($report_data);
			} elseif($group_report_by == 3){
				$data = get_grouped_by_clients_invoice_report_data($report_data);
			} elseif($group_report_by == 4){
				$data = get_grouped_by_clients_and_projects_invoice_report_data($report_data);
			} else {
				flash_error('A project has to be selected to run the report');
				ajx_current('empty');
				return;
			}
		
		}

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'invoice_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($csv) {
            return array(
                'data' => $data,
            );
        }
		
		tpl_assign('data', $data);
		tpl_assign('project_member', $project_member);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'invoice_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_invoice_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_invoice_report_excel');

		tpl_assign('report_template_path', get_template_path('invoice_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}

	function invoice_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->invoice_report($_POST, true);
			
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('invoicing history report').".csv"));
			
			$rows_csv = build_invoice_report_csv($report_result['data']);
	
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
	
		} else if($export_pdf) {
			
			ob_start();
			$this->setTemplate('invoice_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			
			$this->invoice_report($_POST);
			tpl_display(get_template_path('invoice_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "invoicing_history", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function invoice_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->invoice_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('invoicing history report').".xlsx"));
		
		$excel_data = build_excel_data_for_invoice_report($report_result['data'], $_POST);

		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('invoicing history report'), 'fixed_cells_width' => 30), 'headers' => array(),'data'=>array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}

	/****
	 *
	 * BUDGET REPORT
	 * 
	 ****/

	function budget_report_p(){
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_data = array_var($_SESSION, 'budget_report_data');
		if (!$report_data) {
			$report_data = array();
			
			$_SESSION['budget_report_data'] = $report_data;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'budget_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}

	function budget_report($report_data = null, $csv = null) {
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}
		
		$_SESSION['budget_report_data'] = $report_data;
		$dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        $project_member = null;
        $project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
        if (is_array($project_member_ids) && count($project_member_ids) > 0) {
        	$project_member = Members::getMemberById($project_member_ids[0]);
        }
        
        if (!$project_member instanceof Member) {
			flash_error('A project has to be selected to run the report');
			ajx_current('empty');
			return;
		}

		// Get the report title
		$report_title = Reports::findOne(array("conditions" => "`code` = 'budget_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		$data = get_main_budget_report_data($project_member, $report_data);

		if ($csv) {
            return array(
                'data' => $data,
            );
        }
		
		tpl_assign('data', $data);
		tpl_assign('project_member', $project_member);
		tpl_assign('title', $title);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		tpl_assign('print_action_url', get_url('project_reports', 'budget_report_print'));

		tpl_assign('pdf_export_function', 'og.project_reports.export_budget_report_pdf');
		tpl_assign('excel_export_action', 'og.project_reports.export_budget_report_excel');

		tpl_assign('report_template_path', get_template_path('budget_report', 'project_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function budget_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->budget_report($_POST, true);
			
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('budget report').".csv"));
			
			$rows_csv = build_budget_report_csv($report_result['data']);
	
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
	
		} else if($export_pdf) {
			
			ob_start();
			$this->setTemplate('budget_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			
			$this->budget_report($_POST);
			tpl_display(get_template_path('budget_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "budget_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	function budget_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->budget_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('budget report').".xlsx"));
		
		$excel_data = build_excel_data_for_budget_report($report_result['data'], $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('budget report'), 'fixed_cells_width' => 30), 'headers' => array(),'data'=>array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}


	/****
	 * 
	 * PROJECT BILLING REPORT
	 * 
	 ****/

	function project_billing_report_p(){
		Logger::log_r('CALLS TO PROJECT BILLING REPORT');
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_billing_report'"))->getObjectName();
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (!$report_data) {
			$report_data = array('project_page_break' => 1, 'date_type' => 5); // last month default
			$_SESSION['project_management_report_data'] = $report_data;
		}
		tpl_assign('title', $report_title);
		
	}

	/****
	 * 
	 * PROJECT BILLING ALTERNATIVE REPORT
	 * 
	 ****/

	function project_billing_alt_report_p(){
		Logger::log_r('CALLS TO PROJECT BILLING ALT REPORT');
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_billing_alt_report'"))->getObjectName();
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (!$report_data) {
			$report_data = array('project_page_break' => 1, 'date_type' => 5); // last month default
			$_SESSION['project_management_report_data'] = $report_data;
		}
		tpl_assign('title', $report_title);
	}


	/****
	 * 
	 * PROJECT MANAGEMENT REPORT
	 * 
	 ****/

	function project_management_report_p(){
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if(!$can_see_billing_info|| !$can_see_cost_info){
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$report_title = Reports::findOne(array("conditions" => "`code` = 'project_management_report'"))->getObjectName();
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (!$report_data) {
			$report_data = array('project_page_break' => 1);
			
			$_SESSION['project_management_report_data'] = $report_data;
		}
		tpl_assign('title', $report_title);
		
	}


	function project_management_report($report_data = null, $csv = null) {
		
		Env::useHelper('advanced_reports', 'advanced_reports');

		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}

		if(array_key_exists('start_value', $report_data)){
      		$report_data['exclude_non_billable'] = true;
			$report_dates = parse_report_param_date_type($report_data, true, 'date_type', 'start_value', 'end_value', 'project_billing_report'); 
			$expense_report_dates = parse_report_param_date_type($report_data, false, 'date_type', 'start_value', 'end_value', 'project_billing_report'); 
    	} else {
			$report_dates = parse_report_param_date_type($report_data);
			$expense_report_dates = parse_report_param_date_type($report_data, false);
		}

		$report_title = array_var($report_data, 'report_title', 'project management report');

		// dates config
		$start_date = array_var($report_dates, 'st');
		$end_date = array_var($report_dates, 'et');
		$report_data['start_date'] = $start_date;
		$report_data['end_date'] = $end_date;
		// dates for expense queries
		$expense_start_date = array_var($expense_report_dates, 'st');
		$expense_end_date = array_var($expense_report_dates, 'et');
		$report_data['expense_start_date'] = $expense_start_date;
		$report_data['expense_end_date'] = $expense_end_date;

		$_SESSION['project_management_report_data'] = $report_data;

        $dateFormat = user_config_option('date_format');
        if (!empty($dateFormat)){
            $dateFormat = 'm/d/Y';
        }
        
        $project_member = null;
        $project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
        if (is_array($project_member_ids) && count($project_member_ids) > 0) {
        	$project_member = Members::getMemberById($project_member_ids[0]);
        }
        
        $data = get_main_project_manager_report_data($project_member, $report_data);
        $show_main_information_blocks = array_var($report_data, 'show_main_project_info');

        if ($csv) {
            return array(
                'data' => $data,
            );
		}
				
        tpl_assign('show_main_information_blocks', $show_main_information_blocks);
        
		tpl_assign('data', $data);
		tpl_assign('title', $report_title);

        tpl_assign('start_date', $report_data['start_date']);
        tpl_assign('end_date', $report_data['end_date']);
        tpl_assign('post', $report_data);
        tpl_assign('allow_export', true);
        tpl_assign('genid', gen_id());
        tpl_assign('hide_print_report_header', 'true');
		
		tpl_assign('template_name', 'project_management_report');
		tpl_assign('print_action_url', get_url('project_reports', 'project_management_report_print'));
		tpl_assign('excel_export_action', 'og.project_reports.export_project_management_report_excel');
		tpl_assign('pdf_export_function', 'og.project_reports.export_project_management_report_pdf');
		tpl_assign('report_template_path', get_template_path('project_management_report', 'project_reports'));
		
		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}
	
	
	
	function project_management_report_excel() {
		ajx_current("empty");
		
		$report_data = $_SESSION['project_management_report_data'];
		if(array_key_exists('start_date', $report_data)){
			$report_title = Reports::findOne(array("conditions" => "`code` = 'project_billing_report'"))->getObjectName();
		} else {
			$report_title = Reports::findOne(array("conditions" => "`code` = 'project_management_report'"))->getObjectName();
		}
		$filename = str_replace(" ", "-", str_replace("/", "-", $report_title.".xlsx"));

		$report_result = $this->project_management_report($_POST, true);
		
		$excel_data = $this->build_excel_data_from_report_data($report_result['data']);

		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('project management report'), 'fixed_cells_width' => 30), 'headers' => array(),'data'=>array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}

	
	private function build_project_information_excel($project_member_id) {
		$report_data = array_var($_SESSION, 'project_management_report_data');
		
		$project_member = Members::getMemberById($project_member_id);
		if (!$project_member instanceof Member) {
			return array();
		}
		
		$excel_data = array();
		
		$properties_array = '';
		$params =  array(
				'member' => $project_member,
				'visible_by_default' => true,
				'return_as_array' => true,
		);
		Hook::fire('override_render_properties_for_view', $params, $properties_array);
		
		$left_properties_array = array();
		$right_properties_array = array();
		foreach ($properties_array as $group_id => $group_data) {
			if (count($group_data['properties']) == 0) continue;
			
			$property_group = PropertyGroups::findById($group_id);
			$group_data['alignment'] = $property_group->getAlignment();
			$group_data['name'] = $property_group->getName();
			
			if ($group_data['alignment'] == 'left') {
				$left_properties_array[] = $group_data;
			} else if ($group_data['alignment'] == 'right') {
				$right_properties_array[] = $group_data;
			}
		}
		
		
		// build the name shown in trees
		$tmp_main_member_array = array($project_member);
		build_member_list_text_to_show_in_trees($tmp_main_member_array);
		
		$main_header_array = array($project_member->getName(), '', '', '', '', '');
		
		if (array_var($report_data, 'include_billing_rate_name')) {
			$main_header_array[] = '';
		}
		if (array_var($report_data, 'include_miles')) {
			$main_header_array[] = '';
		}
		if (array_var($report_data, 'include_estimated_time')) {
			$main_header_array[] = '';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$main_header_array[] = '';
		}
		if (array_var($report_data, 'invoice_status') == 'all') {
			$main_header_array[] = '';
		}
		
		
		$excel_data[] = array('headers' => array(), 'data' => array(), 'main_header' => $main_header_array);
		
		$i = 0;
		$more_groups_left = true;
		while ($more_groups_left) {
			$more_groups_left = false;
			
			$headers = array();
			$data = array();
			
			if (isset($left_properties_array[$i])) {
				$group_data = $left_properties_array[$i];
				$more_groups_left = true;
				
				$headers[] = $group_data['name'];
				$headers[] = '';
				$headers[] = '';
				
				$j = 0;
				foreach ($group_data['properties'] as $property) {
					$data[$j] = array();
					
					$data[$j][0] = $property['label'];
					$data[$j][1] = $property['value'];
					$data[$j][2] = '';
					
					$j++;
				}
			} else {
				$headers[] = '';
				$headers[] = '';
				$headers[] = '';
			}
			
			if (isset($right_properties_array[$i])) {
				$group_data = $right_properties_array[$i];
				$more_groups_left = true;
				
				$headers[] = $group_data['name'];
				$headers[] = '';
				$headers[] = '';
				
				$j = 0;
				foreach ($group_data['properties'] as $property) {
					if (!isset($data[$j])) $data[$j] = array('','','');
					
					$data[$j][3] = $property['label'];
					$data[$j][4] = $property['value'];
					$data[$j][5] = '';
					
					$j++;
				}
			} else {
				$headers[] = '';
				$headers[] = '';
				$headers[] = '';
			}
			
			if (array_var($report_data, 'include_billing_rate_name')) {
				$headers[] = '';
			}
			if (array_var($report_data, 'include_miles')) {
				$headers[] = '';
			}
			if (array_var($report_data, 'include_estimated_time')) {
				$headers[] = '';
			}
			if (array_var($report_data, 'include_pending_time')) {
				$headers[] = '';
			}
			if (array_var($report_data, 'invoice_status') == 'all') {
				$headers[] = '';
			}
			
			$excel_data[] = array('headers' => $headers, 'data' => $data);
			
			$i++;
		}
		
		return $excel_data;
	}
	
	
	
	private function build_excel_data_from_report_data($data) {
		$report_data = array_var($_SESSION, 'project_management_report_data');
		
		$excel_data = array();
		
		$projects_data = array_var($data, 'projects_data', array());
		foreach ($projects_data as $project_data) {
			
			if (empty($project_data['timeslots_summary']) && empty($project_data['expenses_summary']) && (!(array_var($report_data, 'include_estimated_time') || array_var($report_data, 'include_pending_time')) || empty($project_data['timeslots_summary']) )) {
				continue;
			}
			$pi_data = array();
			if (array_var($report_data, 'show_main_project_info')) {
				$pi_data = $this->build_project_information_excel($project_data['project_id']);
			}
			
			
			$excel_data = array_merge($excel_data, $pi_data);
			
			$project_excel_data = $this->build_project_all_data($project_data, $report_data);
			$excel_data = array_merge($excel_data, $project_excel_data);
			
		}
		
		$result = $this->build_project_totals_excel($data, $report_data, 'grand');
		if ($result) {
			$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
		}

		return $excel_data;
	}
	
	
	private function build_project_all_data($project_data, $report_data) {
		$excel_data = array();

		if (!empty($project_data['timeslots_summary']) || !empty($project_data['expenses_summary'])) {
			$main_header_array = array($project_data['project_name'], '', '', '', '', '');
			if (array_var($report_data, 'include_time_description')) $main_header_array[] = '';
			if (array_var($report_data, 'include_billing_rate_name')) $main_header_array[] = '';
			if (array_var($report_data, 'include_miles')) $main_header_array[] = '';
			if (array_var($report_data, 'include_estimated_time')) $main_header_array[] = '';
			if (array_var($report_data, 'include_pending_time')) $main_header_array[] = '';
			if (array_var($report_data, 'include_estimated_billing')) $main_header_array[] = '';
			if (array_var($report_data, 'include_pending_billing')) $main_header_array[] = '';
			if (array_var($report_data, 'include_rate_schedule')) $main_header_array[] = '';
			if (array_var($report_data, 'include_approval_status')) $main_header_array[] = '';
			if (array_var($report_data, 'include_modules_dim')) $main_header_array[] = '';
			if (array_var($report_data, 'invoice_status') == 'all') $main_header_array[] = '';

			$excel_data[] = array('headers' => array(), 'data' => array(), 'main_header' => $main_header_array);
		}

		// subprojects
		$subprojects_data = array_var($project_data, 'subprojects');
		if (count($subprojects_data) > 0) {
			foreach ($subprojects_data as $subproject_data) {
				
				$subp_excel_data = $this->build_project_all_data($subproject_data, $report_data);
				$excel_data = array_merge($excel_data, $subp_excel_data);
				
			}
		}
		
		// timeslots
		$result = $this->build_project_timeslots_table_excel($project_data, $report_data);
		if ($result) {
			$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
		}

		// Timeslots summary
		if (count($subprojects_data) == 0 && count($project_data['timeslots_data']) != 0) {
			$result = $this->build_project_timeslots_summary_excel($project_data, $report_data);
			if ($result) {
				$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
			}
		}
		
		// expenses
		$result = $this->build_project_expenses_table_excel($project_data, $report_data);
		if ($result) {
			$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
		}

		// Expenses Summary
		if (count($subprojects_data) == 0 && count($project_data['expenses_data']) != 0) {
			$result = $this->build_project_expenses_summary_excel($project_data);
			if ($result) {
				$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
			}
		}

		// Project totals summary
		$project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
		if(count($subprojects_data) == 0  && count($project_member_ids) == 0 && (count($project_data['timeslots_data']) != 0 || count($project_data['expenses_data']) != 0)){
			$result = $this->build_project_totals_excel($project_data, $report_data, 'project');
			if ($result) {
				$excel_data[] = array('headers' => array_var($result,'headers'), 'data' => array_var($result,'data'));
			}
		}
		
		return $excel_data;
	}
	
	
	private function build_project_timeslots_table_excel($project_data, $report_data) {
		if (empty($project_data['timeslots_data'])) return;

		$show_invoice_details = get_show_invoicing_detail($report_data);
		
		$headers = array();
		$data = array();
		$columns = array('date', 'person', 'hour_type');
		$billing_columns = array('worked_time', 'totalbilling');
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = 'description';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = 'billing_rate';
		}
		$columns[] = 'hourlybilling';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = '';
		}
		if ($show_invoice_details) {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = 'rate_schedule';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = 'approval_status';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = 'modules_dim';
		}
	
		$columns = array_merge($columns, $billing_columns);
		foreach ($columns as $col) {
			if ($col == 'rate_schedule'){
				$rate_sched_dim = Dimensions::findByCode('rate_schedules');
				$rate_sched_name = $rate_sched_dim->getName();
				$headers[] = $rate_sched_name;
			} else if ($col == 'modules_dim'){
				$modules_dim = Dimensions::findByCode('modules');
				$modules_name = $modules_dim->getName();
				$headers[] = $modules_name;
			} else if ($col == 'approval_status'){
				$approval_status_dim = Dimensions::findByCode('status_timesheet');
				$approval_status_name = $approval_status_dim->getName();
				$headers[] = $approval_status_name;
			} else {
				$headers[] = ($col == '' ? '' : lang($col));
			}
		}
		
		foreach ($project_data['timeslots_data'] as $timeslot) {
			$data_row = array();
			foreach ($columns as $col) {
				switch ($col) {
					case 'date':
						$date = new DateTimeValue($timeslot['start_time_ts']);
						$data_row[] = format_date($date, null, null);
						break;
					case 'person':
						$data_row[] = str_replace(',','', $timeslot['uname']);
						break;
					case 'hour_type':
						$data_row[] = str_replace(',','', $timeslot['hour_types'][0]);
						break;
					case 'billing_rate':
						$data_row[] = str_replace(',','', $timeslot['billing_rates'][0]);
						break;
					case 'worked_time':
						$data_row[] = $timeslot['worked_hours'];
						break;
					case 'miles':
						$data_row[] = $timeslot['miles'];
						break;
					case 'hourlybilling':
						$data_row[] = str_replace(',','', format_money_amount($timeslot['hourlybilling'], $timeslot['rate_currency_sym']));
						break;
					case 'description':
						$data_row[] = str_replace(',','', $timeslot['description']);
						break;
					case 'totalbilling':
						$data_row[] = str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						break;
					case 'rate_schedule':
						$rate_schedule = array_var($timeslot, 'rate_schedules', array(''));
						$data_row[] = str_replace(',','', $rate_schedule[0]);
						break;
					case 'modules_dim':
						$modules = array_var($timeslot, 'modules', array(''));
						$data_row[] = str_replace(',','', $modules[0]);
						break;
					case 'approval_status':
						$approval_status = array_var($timeslot, 'status_timesheet', array(''));
						$data_row[] = str_replace(',','',$approval_status[0]);
						break;
					case 'invoiced':
						if($timeslot['invoicing_status'] == 'invoiced'){
							$data_row[] = str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					case 'not_invoiced':
						if($timeslot['invoicing_status'] == 'pending'){
							$data_row[] = str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					default:
						$data_row[] = '';
						break;
				}
			}
			$data[] = $data_row;
		}
		$data[] = array(' ');
		
		return array('headers' => $headers, 'data' => $data);
	}
	
	
	private function build_project_timeslots_summary_excel($project_data, $report_data) {
		if (empty($project_data['timeslots_summary'])) return;
		
		$show_invoice_details = get_show_invoicing_detail($report_data);

		$headers = array();
		$data = array();
		
		$columns = array('', '', 'labor_category_summary');
		$billing_columns = array('worked_time', 'totalbilling');
		
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = 'billing_rate';
		}
		$columns[] = 'hourlybilling';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = 'estimated_hours';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_billing';
		}
		if ($show_invoice_details) {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = 'pending_hours';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = 'pending_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = '';
		}
		
		$columns = array_merge($columns, $billing_columns);
		
		foreach ($columns as $col) {
			$headers[] = ($col == '' ? '' : lang($col));
		}
		
		foreach ($project_data['timeslots_summary'] as $summary) {
			$data_row = array();
			foreach ($columns as $col) {
				switch ($col) {
					case 'labor_category_summary':
						$data_row[] = $summary['hour_type'];
						break;
					case 'worked_time':
						$data_row[] = $summary['worked_hours'];
						break;
					case 'estimated_hours':
						$data_row[] = $summary['estimated_hours'];
						break;
					case 'pending_hours':
						$data_row[] = $summary['pending_hours'];
						break;
					case 'estimated_billing':
						$estimated_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount($summary['estimated_billing'], $summary['currency_sym']);
						$data_row[] = $estimated_billing;
						break;
					case 'pending_billing':
						$pending_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount($summary['pending_billing'], $summary['currency_sym']);
						$data_row[] = $pending_billing;
						break;
					case 'hourlybilling':
						$data_row[] = (array_var($summary, 'hourlybilling', '') == '' ? '' : str_replace(',','', format_money_amount($summary['hourlybilling'], $summary['currency_sym'])));
						break;
					case 'totalbilling':
						$data_row[] = str_replace(',','', format_money_amount($summary['totalbilling'], $summary['currency_sym']));
						break;
					case 'miles':
						$data_row[] = $summary['miles'];
						break;
					case 'invoiced':
						$data_row[] = str_replace(',','', format_money_amount($summary['invoiced'], $summary['rate_currency_sym']));
						break;
					case 'not_invoiced':
						$data_row[] = str_replace(',','', format_money_amount($summary['not_invoiced'], $summary['rate_currency_sym']));
						break;
					default:
						$data_row[] = '';
						break;
				}
			}
			
			$data[] = $data_row;
		}
		$data[] = array(' ');
		
		return array('headers' => $headers, 'data' => $data);
	}
	
	
	private function build_project_expenses_table_excel($project_data, $report_data) {
		if (empty($project_data['expenses_data'])) return;

		$report_data = array_var($_SESSION, 'project_management_report_data');
		$project_billing_alt_report = Reports::findOne(array("conditions" => "`code` = 'project_billing_alt_report'"));
		$project_billing_alt_report_id = $project_billing_alt_report instanceof Report ? $project_billing_alt_report->getId() : 0;
		if($report_data['report_id']==$project_billing_alt_report_id){
			$project_expenses_alt_table = $this->build_project_expenses_table_for_alt_report($project_data, $report_data);
			return $project_expenses_alt_table;
		}

		
		$show_invoice_details = get_show_invoicing_detail($report_data);

		$headers = array();
		$data = array();
		
		$columns = array('date', 'person', 'expense_category');
		$billing_columns = array('quantity', 'amount');

		if (array_var($report_data, 'include_time_description')) {
			$columns[] = 'description';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = ' ';
		}
		$columns[] = 'unit_price';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = ' ';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = ' ';
		}
		if ($show_invoice_details) {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$enabled_dimensions = config_option('enabled_dimensions');
			$expense_rate_schedule_dim = Dimensions::findByCode('expense_rate_schedules');
	
			if($expense_rate_schedule_dim instanceof Dimension && in_array($expense_rate_schedule_dim->getId(), $enabled_dimensions)){
				$billing_columns[] = 'expense_rate_schedule';
			} else {
				$billing_columns[] = 'rate_schedule';
			}
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = 'approval_status';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = ' ';
		}

		$columns = array_merge($columns, $billing_columns);
	
		foreach ($columns as $col) {
			if($col == 'expense_rate_schedule'){
				$exp_rate_sched_dim = Dimensions::findByCode('expense_rate_schedules');
				$exp_rate_sched_name = $exp_rate_sched_dim->getName();
				$headers[] = $exp_rate_sched_name;
			} else if($col == 'rate_schedule'){
				$rate_sched_dim = Dimensions::findByCode('rate_schedules');
				$rate_sched_name = $rate_sched_dim->getName();
				$headers[] = $rate_sched_name;
			} else if ($col == 'approval_status'){
				$approval_status_dim = Dimensions::findByCode('status_timesheet');
				$approval_status_name = $approval_status_dim->getName();
				$headers[] = $approval_status_name;
			}  else {
				$headers[] = ($col == '' ? '' : lang($col));
			}
		}
		
		foreach ($project_data['expenses_data'] as $expense) {
			$data_row = array();
			foreach ($columns as $col) {
				switch ($col) {
					case 'date':
						$data_row[] = $expense['date'];
						break;
					case 'expense_category':
						$data_row[] = isset($expense['expense_categories']) ? str_replace(',','', $expense['expense_categories'][0]) : '';
						break;
					case 'name':
						$data_row[] = str_replace(',','', $expense['name']);
						break;
					case 'description':
						$data_row[] = str_replace(',','', $expense['description']);
						break;

					case 'person':
						$data_row[] = str_replace(',','', $expense['paid_by_name']);
						break;
					case 'quantity':
						$data_row[] = str_replace(',','', $expense['quantity']);
						break;
					case 'unit_price':
						$data_row[] = str_replace(',','', format_money_amount($expense['unit_price'], $expense['currency_sym']));
						break;
					case 'amount':
						$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['currency_sym']));
						break;
					case 'rate_schedule':
						$rate_schedule = array_var($expense, 'rate_schedules', array(''));
						$data_row[] = str_replace(',','', $rate_schedule[0]);
						break;
					case 'expense_rate_schedule':
						$exp_rate_schedule = array_var($expense, 'expense_rate_schedules', array(''));
						$data_row[] = str_replace(',','', $exp_rate_schedule[0]);
						break;
					case 'approval_status':
						$approval_status = array_var($expense, 'status_timesheet', array(''));
						$data_row[] = str_replace(',','', $approval_status[0]);
						break;
					case 'invoiced':
						if($expense['invoicing_status'] == 'invoiced'){
							$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					case 'not_invoiced':
						if($expense['invoicing_status'] == 'pending'){
							$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					default:
						$data_row[] = '';
						break;
				}
			}
			$data[] = $data_row;
		}
		$data[] = array(' ');
		
		return array('headers' => $headers, 'data' => $data);
	}
	
	private function build_project_expenses_table_for_alt_report($project_data, $report_data) {
		if (empty($project_data['expenses_data'])) return;

		
		$show_invoice_details = get_show_invoicing_detail($report_data);

		$headers = array();
		$data = array();
		
		$columns = array('date','expense_category', 'name', 'description');
		$billing_columns = array(' ', 'amount');

		$report_data = array_var($_SESSION, 'project_management_report_data');

		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = ' ';
		}
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = ' ';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = ' ';
		}
		if ($show_invoice_details) {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$enabled_dimensions = config_option('enabled_dimensions');
			$expense_rate_schedule_dim = Dimensions::findByCode('expense_rate_schedules');
	
			if($expense_rate_schedule_dim instanceof Dimension && in_array($expense_rate_schedule_dim->getId(), $enabled_dimensions)){
				$billing_columns[] = 'expense_rate_schedule';
			} else {
				$billing_columns[] = 'rate_schedule';
			}
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = 'approval_status';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = ' ';
		}

		$columns = array_merge($columns, $billing_columns);
	
		foreach ($columns as $col) {
			if($col == 'expense_rate_schedule'){
				$exp_rate_sched_dim = Dimensions::findByCode('expense_rate_schedules');
				$exp_rate_sched_name = $exp_rate_sched_dim->getName();
				$headers[] = $exp_rate_sched_name;
			} else if($col == 'rate_schedule'){
				$rate_sched_dim = Dimensions::findByCode('rate_schedules');
				$rate_sched_name = $rate_sched_dim->getName();
				$headers[] = $rate_sched_name;
			} else if ($col == 'approval_status'){
				$approval_status_dim = Dimensions::findByCode('status_timesheet');
				$approval_status_name = $approval_status_dim->getName();
				$headers[] = $approval_status_name;
			}  else {
				$headers[] = ($col == '' ? '' : lang($col));
			}
		}
		
		foreach ($project_data['expenses_data'] as $expense) {
			$data_row = array();
			foreach ($columns as $col) {
				switch ($col) {
					case 'date':
						$data_row[] = $expense['date'];
						break;
					case 'expense_category':
						$data_row[] = isset($expense['expense_categories']) ? str_replace(',','', $expense['expense_categories'][0]) : '';
						break;
					case 'name':
						$data_row[] = str_replace(',','', $expense['name']);
						break;
					case 'description':
						$data_row[] = str_replace(',','', $expense['description']);
						break;

					case 'person':
						$data_row[] = str_replace(',','', $expense['paid_by_name']);
						break;
					case 'quantity':
						$data_row[] = str_replace(',','', $expense['quantity']);
						break;
					case 'unit_price':
						$data_row[] = str_replace(',','', format_money_amount($expense['unit_price'], $expense['currency_sym']));
						break;
					case 'amount':
						$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['currency_sym']));
						break;
					case 'rate_schedule':
						$rate_schedule = array_var($expense, 'rate_schedules', array(''));
						$data_row[] = str_replace(',','', $rate_schedule[0]);
						break;
					case 'expense_rate_schedule':
						$exp_rate_schedule = array_var($expense, 'expense_rate_schedules', array(''));
						$data_row[] = str_replace(',','', $exp_rate_schedule[0]);
						break;
					case 'approval_status':
						$approval_status = array_var($expense, 'status_timesheet', array(''));
						$data_row[] = str_replace(',','', $approval_status[0]);
						break;
					case 'invoiced':
						if($expense['invoicing_status'] == 'invoiced'){
							$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					case 'not_invoiced':
						if($expense['invoicing_status'] == 'pending'){
							$data_row[] = str_replace(',','', format_money_amount($expense['amount'], $expense['rate_currency_sym']));
						} else {
							$data_row[] = '';
						}
						break;
					default:
						$data_row[] = '';
						break;
				}
			}
			$data[] = $data_row;
		}
		$data[] = array(' ');
		
		return array('headers' => $headers, 'data' => $data);
	}
	
	private function build_project_expenses_summary_excel($project_data) {
		if (empty($project_data['expenses_summary'])) return;

		$show_invoice_details = get_show_invoicing_detail($report_data);
		
		$headers = array();
		$data = array();
		
		$columns = array('', '', 'expense_category_summary', '');
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_expense_billing';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = '';
		}
		$columns[] = '';
		$columns[] = 'totalbilling';
		if ($show_invoice_details) {
			$columns[] = 'invoiced';
			$columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$columns[] = 'pending_expense_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$columns[] = '';
		}
		foreach ($columns as $col) {
			$headers[] = ($col == '' ? '' : lang($col));
		}
		
		foreach ($project_data['expenses_summary'] as $summary) {
			$data_row = array();
			foreach ($columns as $col) {
				switch ($col) {
					case 'expense_category_summary':
						$data_row[] = str_replace(',','', $summary['expense_category']);
						break;
					case 'estimated_expense_billing':
						$data_row[] = format_money_amount($summary['estimated_expense_billing'], $summary['currency_sym']);
						break;
					case 'pending_expense_billing':
						$data_row[] = format_money_amount($summary['pending_expense_billing'], $summary['currency_sym']);
						break;
					case 'totalbilling':
						$data_row[] = str_replace(',','', format_money_amount($summary['amount'], $summary['currency_sym']));
						break;
					case 'invoiced':
						$data_row[] = str_replace(',','', format_money_amount($summary['invoiced'], $summary['rate_currency_sym']));
						break;
					case 'not_invoiced':
						$data_row[] = str_replace(',','', format_money_amount($summary['not_invoiced'], $summary['rate_currency_sym']));
						break;
					default:
						$data_row[] = '';
						break;
				}
			}
			$data[] = $data_row;
		}
		$data[] = array(' ');
		
		return array('headers' => $headers, 'data' => $data);
	}
	
	
	private function build_project_totals_excel($data, $report_data, $scope) {
		$show_invoice_details = get_show_invoicing_detail($report_data);
		
		$headers = array();
		$excel_data = array(); 
		
		if($scope == 'project'){
			$columns = array('', '', 'totals summary');
		} else if ($scope == 'grand'){
			$columns = array('', '', 'grand totals summary');
		}

		$billing_columns = array('worked_time', 'totalbilling');
		
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = '';
		}
		$columns[] = '';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = 'estimated_hours';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_billing';
		}
		if ($show_invoice_details) {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = 'pending_hours';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = 'pending_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = '';
		}
		
		$columns = array_merge($columns, $billing_columns);
		
		foreach ($columns as $col) {
			$headers[] = ($col == '' ? '' : lang($col));
		}
		// Labor Total Row
		$labor_total = str_replace(',','', format_money_amount($data['totals']['timeslots_total'], $data['totals']['currency_sym']));
		$edata = array('', '', lang('labor total'), '');
		if (array_var($report_data, 'include_time_description')) $edata[] = '';
		if (array_var($report_data, 'include_miles')) $edata[] = $data['totals']['miles'];
		if (array_var($report_data, 'include_billing_rate_name')) $edata[] = '';
		if (array_var($report_data, 'include_estimated_time')) $edata[] = $data['totals']['estimated_hours'];
		if (array_var($report_data, 'include_estimated_billing')) $edata[] = format_money_amount($data['totals']['labor_estimated_billing'], $data['totals']['currency_sym']);
		$edata[] = $data['totals']['worked_hours'];
		$edata[] = $labor_total;
		if ($show_invoice_details) {
			$edata[] = format_money_amount($data['totals']['labor_invoiced'], $data['totals']['currency_sym']);
			$edata[] = format_money_amount($data['totals']['labor_not_invoiced'], $data['totals']['currency_sym']);
		}
		if (array_var($report_data, 'include_pending_time')) $edata[] = $data['totals']['pending_hours'];
		if (array_var($report_data, 'include_pending_billing')) $edata[] = format_money_amount($data['totals']['labor_pending_billing'], $data['totals']['currency_sym']);
		if (array_var($report_data, 'include_approval_status')) $edata[] = '';
		if (array_var($report_data, 'include_modules_dim')) $edata[] = '';
		//$edata = array_merge($edata, $add_columns);
		$excel_data[] = $edata;
		
		// Expenses Total Row
		$exp_total = str_replace(',','', format_money_amount($data['totals']['expenses_total'], $data['totals']['currency_sym']));
		
		$edata = array('', '', lang('expenses total'), '');
		if (array_var($report_data, 'include_time_description')) $edata[] = '';
		if (array_var($report_data, 'include_miles')) $edata[] = '';
		if (array_var($report_data, 'include_billing_rate_name')) $edata[] = '';
		if (array_var($report_data, 'include_estimated_time')) $edata[] = '';
		if (array_var($report_data, 'include_estimated_billing')) $edata[] = format_money_amount($data['totals']['estimated_expense_billing'], $data['totals']['currency_sym']);
		$edata[] = '';
		$edata[] = $exp_total;
		if ($show_invoice_details) {
			$edata[] = format_money_amount($data['totals']['expense_invoiced'], $data['totals']['currency_sym']);
			$edata[] = format_money_amount($data['totals']['expense_not_invoiced'], $data['totals']['currency_sym']);
		}
		if (array_var($report_data, 'include_pending_time')) $edata[] = '';
		if (array_var($report_data, 'include_pending_billing')) $edata[] = format_money_amount($data['totals']['pending_expense_billing'], $data['totals']['currency_sym']);
		if (array_var($report_data, 'include_rate_schedule')) $edata[] = '';
		if (array_var($report_data, 'include_approval_status')) $edata[] = '';
		if (array_var($report_data, 'include_modules_dim')) $edata[] = '';
		//$edata = array_merge($edata, $add_columns);
		$excel_data[] = $edata;
		
		// Labor and Expenses Total Row
		$total = str_replace(',','', format_money_amount($data['totals']['timeslots_total'] + $data['totals']['expenses_total'], $data['totals']['currency_sym']));
		$edata = array('', '', lang('labor and expenses total'), '');
		if (array_var($report_data, 'include_time_description')) $edata[] = '';
		if (array_var($report_data, 'include_miles')) $edata[] = '';
		if (array_var($report_data, 'include_billing_rate_name')) $edata[] = '';
		if (array_var($report_data, 'include_estimated_time')) $edata[] = $data['totals']['estimated_hours'];
		if (array_var($report_data, 'include_estimated_billing')) $edata[] = format_money_amount($data['totals']['total_estimated_billing'], $data['totals']['currency_sym']);
		$edata[] = $data['totals']['worked_hours'];
		$edata[] = $total;
		if ($show_invoice_details) {
			$edata[] = format_money_amount($data['totals']['total_invoiced'], $data['totals']['currency_sym']);
			$edata[] = format_money_amount($data['totals']['total_not_invoiced'], $data['totals']['currency_sym']);
		}
		if (array_var($report_data, 'include_pending_time')) $edata[] = $data['totals']['pending_hours'];
		if (array_var($report_data, 'include_pending_billing')) $edata[] = format_money_amount($data['totals']['total_pending_billing'], $data['totals']['currency_sym']);
		if (array_var($report_data, 'include_rate_schedule')) $edata[] = '';
		if (array_var($report_data, 'include_approval_status')) $edata[] = '';
		if (array_var($report_data, 'include_modules_dim')) $edata[] = '';
		//$edata = array_merge($edata, $add_columns);
		$excel_data[] = $edata;
		if ((!in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) {
			$excel_data[] = array(' ');
		}

		// Project Budget Total Row 
		if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) {
			if(array_var($report_data, 'for_budget_use') == 'tasks_expenses'){
				$row_title = lang('sum of budgeted tasks + expenses'); 
			} else if (array_var($report_data, 'for_budget_use') == 'budget_total_cp'){ 
				$project_object_type = ObjectTypes::findOne(array("conditions" => "`name`='project'"));
				$cp_object_type_id = $project_object_type->getId();
				$budget_custom_property = CustomProperties::findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"'));
				$row_title =  $budget_custom_property->getName();
			} 
			$total = str_replace(',','', format_money_amount($data['totals']['project_total_budget'], $data['totals']['currency_sym']));
			$edata = array('', '', $row_title, '');
			if (array_var($report_data, 'include_time_description')) $edata[] = '';
			if (array_var($report_data, 'include_miles')) $edata[] = '';
			if (array_var($report_data, 'include_billing_rate_name')) $edata[] = '';
			if (array_var($report_data, 'include_estimated_time')) $edata[] = '';
			if (array_var($report_data, 'include_estimated_billing')) $edata[] = '';
			$edata[] = '';
			$edata[] = $total;
			if ($show_invoice_details) {
				$edata[] = '';
				$edata[] = '';
			}
			if (array_var($report_data, 'include_pending_time')) $edata[] = '';
			if (array_var($report_data, 'include_pending_billing')) $edata[] = '';
			if (array_var($report_data, 'include_rate_schedule')) $edata[] = '';
			if (array_var($report_data, 'include_approval_status')) $edata[] = '';
			if (array_var($report_data, 'include_modules_dim')) $edata[] = '';
			//$edata = array_merge($edata, $add_columns);
			$excel_data[] = $edata;
		}

		// Budget Consumption Row
		if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) {
			$budget_difference = $data['totals']['project_total_budget'] == 0 ? 0 :($data['totals']['timeslots_total'] + $data['totals']['expenses_total'])/ $data['totals'] ['project_total_budget'] * 100;
			$edata = array('', '', lang('budget consumption'), '');
			if (array_var($report_data, 'include_time_description')) $edata[] = '';
			if (array_var($report_data, 'include_miles')) $edata[] = '';
			if (array_var($report_data, 'include_billing_rate_name')) $edata[] = '';
			if (array_var($report_data, 'include_estimated_time')) $edata[] = '';
			if (array_var($report_data, 'include_estimated_billing')) $edata[] = '';
			$edata[] = '';
			$edata[] = round($budget_difference) . '%';
			if ($show_invoice_details) {
				$edata[] = '';
				$edata[] = '';
			}
			if (array_var($report_data, 'include_pending_time')) $edata[] = '';
			if (array_var($report_data, 'include_pending_billing')) $edata[] = '';
			if (array_var($report_data, 'include_rate_schedule')) $edata[] = '';
			if (array_var($report_data, 'include_approval_status')) $edata[] = '';
			if (array_var($report_data, 'include_modules_dim')) $edata[] = '';
			//$edata = array_merge($edata, $add_columns);
			$excel_data[] = $edata;
			$excel_data[] = array(' ');
		}
		
		return array('headers' => $headers, 'data' => $excel_data);
	}
	
	



	
	
	function project_management_report_print() {
		ajx_current("empty");
		
		$export_csv = array_var($_POST, 'exportCSV');
		$export_pdf = array_var($_POST, 'exportPDF');
		//$_SESSION['project_management_report_data'] = $report_data;
		$report_data = $_SESSION['project_management_report_data'];
		
		if ($export_csv) {
			
			$report_result = $this->project_management_report($_POST, true);
			
			if(array_key_exists('start_date', $report_data)){
				$report_title = Reports::findOne(array("conditions" => "`code` = 'project_billing_report'"))->getObjectName();
			} else {
				$report_title = Reports::findOne(array("conditions" => "`code` = 'project_management_report'"))->getObjectName();
			}

			$filename = str_replace(" ", "-", str_replace("/", "-", $report_title.".csv"));
			
			$rows_csv = $this->build_csv_from_report_data($report_result['data'], $report_data);
			
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
	
		} else if ($export_pdf) {
			
			ob_start();
			$this->setTemplate('project_management_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			echo stylesheet_tag(ROOT_URL."/plugins/project_reports/public/assets/css/project_reports.css" );
			

			$this->project_management_report($_POST);
			tpl_display(get_template_path('project_management_report', 'project_reports', 'project_reports'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			// Logger::log_r($html);
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);

			// Get correct report title
			if(array_key_exists('start_date', $report_data)){
				$report_title = Reports::findOne(array("conditions" => "`code` = 'project_billing_report'"))->getObjectName();
			} else {
				$report_title = Reports::findOne(array("conditions" => "`code` = 'project_management_report'"))->getObjectName();
			}
			$title = str_replace(" ", "_", str_replace("/", "_", $report_title));

			$pdf_data = convert_to_pdf($html, $orientation, $title, $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}
	
	
	private function build_csv_from_report_data($data, $report_data) {
		$excel_data = $this->build_excel_data_from_report_data($data);
		$csv = build_fixed_report_csv_from_excel_data($data, $excel_data);
		
		return $csv;
	}

	private function build_whole_project_csv($project_data, $report_data){
		if(count($project_data['timeslots_data']) != 0 || count($project_data['expenses_data']) != 0) {
			$csv .= str_replace(',','', $project_data['project_name'])."\n";
	
		}

		// subprojects
		$subprojects_data = array_var($project_data, 'subprojects');
		if (count($subprojects_data) > 0) {
			foreach ($subprojects_data as $subproject_data) {
				$csv .= $this->build_whole_project_csv($subproject_data, $report_data);
			}
		}

		// Timeslots	
		if(count($project_data['timeslots_data']) != 0){
			$csv .= $this->build_project_timeslots_table_csv($project_data, $report_data);
			$csv .= "\n";
		}
		// Timeslots Summary
		if (count($subprojects_data) == 0 && count($project_data['timeslots_data']) != 0) {
			$csv .= $this->build_project_timeslots_summary_csv($project_data, $report_data);
			$csv .= "\n";
		}
		// Expenses
		if(count($project_data['expenses_data']) != 0){
			$csv .= $this->build_project_expenses_table_csv($project_data, $report_data);
			$csv .= "\n";
		}
		// Expenses summary
		if (count($subprojects_data) == 0 && count($project_data['expenses_data']) != 0) {
			$csv .= $this->build_project_expenses_summary_csv($project_data, $report_data);
			$csv .= "\n";
		}

		// Project totals summary
		$project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
		// Check if project doesn't have subprojects, it is not selected, has timeslots or expenses data
		if(count($subprojects_data) == 0  && count($project_member_ids) == 0 && (count($project_data['expenses_data']) != 0 || count($project_data['timeslots_data']) != 0)){
			$csv .= $this->build_project_totals_csv($project_data, $report_data, 'project');
			$csv .= "\n";
		}

		return $csv;
	}
	
	private function build_project_timeslots_table_csv($project_data, $report_data) {
		if (empty($project_data['timeslots_data'])) return ""; 
		$csv = "";
		
		$columns = array('date', 'person', 'hour_type');
		$billing_columns = array('worked_time', 'totalbilling');
		
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = 'description';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = 'billing_rate';
		}
		$columns[] = 'hourlybilling';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = ' ';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = ' ';
		}
		if (array_var($report_data, 'invoice_status') == 'all') {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = ' ';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = 'rate_schedule';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = 'approval_status';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = 'modules_dim';
		}

		$columns = array_merge($columns, $billing_columns);

		$first = true;
		foreach ($columns as $col) {
			if ($col == 'rate_schedule'){
				$rate_sched_dim = Dimensions::findByCode('rate_schedules');
				$rate_sched_name = $rate_sched_dim->getName();
				$csv .= ','.str_replace(',','', $rate_sched_name);
			} else if ($col == 'modules_dim'){
				$modules_dim = Dimensions::findByCode('modules');
				$modules_name = $modules_dim->getName();
				$csv .= ','.str_replace(',','', $modules_name);
			} else if ($col == 'approval_status'){
				$approval_status_dim = Dimensions::findByCode('status_timesheet');
				$approval_status_name = $approval_status_dim->getName();
				$csv .= ','.str_replace(',','', $approval_status_name);
			} else {
				$csv .= ($first ? '' : ',') . lang($col);
				$first = false;
			}
		}
		$csv .= "\n";
		
		foreach ($project_data['timeslots_data'] as $timeslot) {
			
			foreach ($columns as $col) {
				switch ($col) {
					case 'date':
						$date = new DateTimeValue($timeslot['start_time_ts']);
						$csv .= format_date($date, null, null);
						break;
					case 'person':
						$csv .= str_replace(',','', $timeslot['uname']);
						break;
					case 'hour_type':
						$csv .= str_replace(',','', $timeslot['hour_types'][0]);
						break;
					case 'worked_time':
						$csv .= $timeslot['worked_hours'];
						break;
					case 'miles':
						$csv .= $timeslot['miles'];
						break;
					case 'hourlybilling':
						$csv .= str_replace(',','', format_money_amount($timeslot['hourlybilling'], $timeslot['rate_currency_sym']));
						break;
					case 'description':
						$csv .= str_replace(',','', $timeslot['description']);
						break;
					case 'totalbilling':
						$csv .= str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						break;
					case 'rate_schedule':
						$rate_schedule = array_var($timeslot, 'rate_schedules', array(''));
						$csv .= str_replace(',','', $rate_schedule[0]);
						break;
					case 'modules_dim':
						$modules = array_var($timeslot, 'modules', array(''));
						$csv .= str_replace(',','', $modules[0]);
						break;
					case 'approval_status':
						$approval_status = array_var($timeslot, 'status_timesheet', array(''));
						$csv .= str_replace(',','', $approval_status[0]);
						break;
					case 'invoiced':
						if($timeslot['invoicing_status'] == 'invoiced'){
							$csv .= str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						} else {
							$csv .=  '';
						}
						break;
					case 'not_invoiced':
						if($timeslot['invoicing_status'] == 'pending'){
							$csv .= str_replace(',','', format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']));
						} else {
							$csv .=  '';
						}
						break;
					default:
						$csv .= '';
						break;
				}
				$csv .= ",";
			}
			$csv .= "\n";
		}
	
		
		 
		return $csv;
	}
	
	private function build_project_timeslots_summary_csv($project_data, $report_data) {
		if (empty($project_data['timeslots_summary'])) return "";
		$csv = "";
		
		$columns = array('', '', 'labor_category_summary');
		$billing_columns = array('worked_time', 'totalbilling');
		
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = 'billing_rate';
		}
		$columns[] = 'hourlybilling';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = 'estimated_hours';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_billing';
		}
		if (array_var($report_data, 'invoice_status') == 'all') {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = 'pending_hours';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = 'pending_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = '';
		}
		
		$columns = array_merge($columns, $billing_columns);
		
		$first = true;
		foreach ($columns as $col) {
			$csv .= ($first ? '' : ',') . ($col == '' ? '' : lang($col));
			$first = false;
		}
		$csv .= "\n";
		
		foreach ($project_data['timeslots_summary'] as $summary) {
			foreach ($columns as $col) {
				switch ($col) {
					case 'labor_category_summary':
						$csv .= $summary['hour_type'];
						break;
					case 'worked_time':
						$csv .= $summary['worked_hours'];
						break;
					case 'estimated_hours':
						$csv .= $summary['estimated_hours'];
						break;
					case 'pending_hours':
						$csv .= $summary['pending_hours'];
						break;
					case 'estimated_billing':
						$estimated_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : str_replace(',','',format_money_amount($summary['estimated_billing'], $summary['currency_sym']));
						$csv .= $estimated_billing;
						break;
					case 'pending_billing':
						$pending_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : str_replace(',','',format_money_amount($summary['pending_billing'], $summary['currency_sym']));
						$csv .= $pending_billing;
						break;
					case 'miles':
						$csv .= $summary['miles'];
						break;
					case 'hourlybilling':
						$csv .= ($summary['hourlybilling'] == '' ? '' : str_replace(',','', format_money_amount($summary['hourlybilling'], $summary['currency_sym'])));
						break;
					case 'totalbilling':
						$csv .= str_replace(',','', format_money_amount($summary['totalbilling'], $summary['currency_sym']));
						break;
					case 'invoiced':
						$csv .= str_replace(',','', format_money_amount($summary['invoiced'], $summary['currency_sym']));
						break;
					case 'not_invoiced':
						$csv .= str_replace(',','', format_money_amount($summary['not_invoiced'], $summary['currency_sym']));
						break;
					default:
						$csv .= '';
						break;
				}
				$csv .= ",";
			}
			$csv .= "\n";
		}
		
		return $csv;
	}
	
	private function build_project_expenses_table_csv($project_data, $report_data) {
		if (empty($project_data['expenses_data'])) return "";
		$csv = "";
		
		$columns = array('date', 'expense_category', 'name', 'description', '');
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = '';
		}
		$columns[] = 'amount';
		if (array_var($report_data, 'invoice_status') == 'all') {
			$columns[] = 'invoiced';
			$columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$enabled_dimensions = config_option('enabled_dimensions');
			$expense_rate_schedule_dim = Dimensions::findByCode('expense_rate_schedules');
	
			if($expense_rate_schedule_dim instanceof Dimension && in_array($expense_rate_schedule_dim->getId(), $enabled_dimensions)){
				$columns[] = 'expense_rate_schedule';
			} else {
				$columns[] = 'rate_schedule';
			}
		}
		if (array_var($report_data, 'include_approval_status')) {
			$columns[] = 'approval_status';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$columns[] = '';
		}
		
		$first = true;
		foreach ($columns as $col) {
			if($col == 'expense_rate_schedule'){
				$exp_rate_sched_dim = Dimensions::findByCode('expense_rate_schedules');
				$exp_rate_sched_name = $exp_rate_sched_dim->getName();
				$csv .= ','.str_replace(',','', $exp_rate_sched_name);
			} else if($col == 'rate_schedule'){
				$rate_sched_dim = Dimensions::findByCode('rate_schedules');
				$rate_sched_name = $rate_sched_dim->getName();
				$csv .= ','.str_replace(',','', $rate_sched_name);
			} else if ($col == 'approval_status'){
				$approval_status_dim = Dimensions::findByCode('status_timesheet');
				$approval_status_name = $approval_status_dim->getName();
				$csv .= ','.str_replace(',','', $approval_status_name);
			} else {
				$csv .= ($first ? '' : ',') . ($col == '' ? '' : lang($col));
				$first = false;
			}
		}
		$csv .= "\n";
		
		foreach ($project_data['expenses_data'] as $expense) {
			
			foreach ($columns as $col) {
				switch ($col) {
					case 'date':
						$csv .= $expense['date'];
						break;
					case 'expense_category':
						$csv .= str_replace(',','', $expense['expense_categories'][0]);
						break;
					case 'name':
						$csv .= str_replace(',','', $expense['name']);
						break;
					case 'description':
						$csv .= str_replace(',','', $expense['description']);
						break;
					case 'amount':
						$csv .= str_replace(',','', format_money_amount($expense['amount'], $expense['currency_sym']));
						break;
					case 'rate_schedule':
						$rate_schedule = array_var($expense, 'rate_schedules', array(''));
						$csv .= str_replace(',','', $rate_schedule[0]);
						break;
					case 'expense_rate_schedule':
						$exp_rate_schedule = array_var($expense, 'expense_rate_schedules', array(''));
						$csv .= str_replace(',','', $exp_rate_schedule[0]);
						break;
					case 'approval_status':
						$approval_status = array_var($expense, 'status_timesheet', array(''));
						$csv .= str_replace(',','', $approval_status[0]);
						break;
					case 'invoiced':
						if($expense['invoicing_status'] == 'invoiced'){
							$csv .= str_replace(',','', format_money_amount($expense['amount'], $expense['currency_sym']));
						} else {
							$csv .=  '';
						}
						break;
					case 'not_invoiced':
						if($$expense['invoicing_status'] == 'pending'){
							$csv .= str_replace(',','', format_money_amount($expense['amount'], $expense['currency_sym']));
						} else {
							$csv .=  '';
						}
						break;
					default:
						$csv .= '';
						break;
				}
				$csv .= ",";
			}
			$csv .= "\n";
		}
		
		return $csv;
	}
	
	private function build_project_expenses_summary_csv($project_data, $report_data) {
		if (empty($project_data['expenses_summary'])) return "";
		$csv = "";
		
		$columns = array('', '', 'expense_category_summary', '');
		$report_data = array_var($_SESSION, 'project_management_report_data');
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_expense_billing';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = '';
		}
		$columns[] = '';
		$columns[] = 'totalbilling';
		if (array_var($report_data, 'invoice_status') == 'all') {
			$columns[] = 'invoiced';
			$columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$columns[] = 'pending_expense_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$columns[] = '';
		}
		$first = true;
		foreach ($columns as $col) {
			$csv .= ($first ? '' : ',') . ($col == '' ? '' : lang($col));
			$first = false;
		}
		$csv .= "\n";
		
		foreach ($project_data['expenses_summary'] as $summary) {
			
			foreach ($columns as $col) {
				switch ($col) {
					case 'expense_category_summary':
						$csv .= str_replace(',','', $summary['expense_category']);
						break;
					case 'totalbilling':
						$csv .= str_replace(',','', format_money_amount($summary['amount'], $summary['currency_sym']));
						break;
					case 'estimated_expense_billing':
						$csv .= str_replace(',','', format_money_amount($summary['estimated_expense_billing'], $summary['currency_sym']));
						break;
					case 'pending_expense_billing':
						$csv .= str_replace(',','', format_money_amount($summary['pending_expense_billing'], $summary['currency_sym']));
						break;
					case 'invoiced':
						$csv .= str_replace(',','', format_money_amount($summary['invoiced'], $summary['currency_sym']));
						break;
					case 'not_invoiced':
						$csv .= str_replace(',','', format_money_amount($summary['not_invoiced'], $summary['currency_sym']));
						break;
					default:
						$csv .= '';
						break;
				}
				$csv .= ",";
			}
			$csv .= "\n";
		}
		
		return $csv;
	}
	
	private function build_project_totals_csv($data, $report_data, $scope) {
		$csv = "";
		
		// Header row
		if ($scope == 'project'){
			$columns = array('', '', 'totals summary');
		} else if ($scope == 'grand') {
			$columns = array('', '', 'grand totals summary');
		}
		
		$billing_columns = array('worked_time', 'totalbilling');
		
		if (array_var($report_data, 'include_time_description')) {
			$columns[] = '';
		}
		if (array_var($report_data, 'include_miles')) {
			$columns[] = 'miles';
		}
		if (array_var($report_data, 'include_billing_rate_name')) {
			$columns[] = '';
		}
		$columns[] = '';
		if (array_var($report_data, 'include_estimated_time')) {
			$columns[] = 'estimated_hours';
		}
		if (array_var($report_data, 'include_estimated_billing')) {
			$columns[] = 'estimated_billing';
		}
		if (array_var($report_data, 'invoice_status') == 'all') {
			$billing_columns[] = 'invoiced';
			$billing_columns[] = 'not_invoiced';
		}
		if (array_var($report_data, 'include_pending_time')) {
			$billing_columns[] = 'pending_hours';
		}
		if (array_var($report_data, 'include_pending_billing')) {
			$billing_columns[] = 'pending_billing';
		}
		if (array_var($report_data, 'include_rate_schedule')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_approval_status')) {
			$billing_columns[] = '';
		}
		if (array_var($report_data, 'include_modules_dim')) {
			$billing_columns[] = '';
		}
		$columns = array_merge($columns, $billing_columns);
		$first = true;
		foreach ($columns as $col) {
			$csv .= ($first ? '' : ',') . ($col == '' ? '' : lang($col));
			$first = false;
		}
		$csv .= "\n";

		// Labor total row
		$labor_total = str_replace(',','', format_money_amount($data['totals']['timeslots_total'], $data['totals']['currency_sym']));
		$csv .= ",,".lang('labor total').",";
		if (array_var($report_data, 'include_time_description')) $csv .= ",";
		if (array_var($report_data, 'include_billing_rate_name')) $csv .= ",";
		if (array_var($report_data, 'include_miles')) $csv .= ",";
		if (array_var($report_data, 'include_estimated_time')) $csv .= ",".$data['totals']['estimated_hours'];
		if (array_var($report_data, 'include_estimated_billing')){
			$estimated_billing = format_money_amount($data['totals']['labor_estimated_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','', $estimated_billing);
		}
		$csv .= ",".$data['totals']['worked_hours'];
		$csv .= ",".$labor_total;
		if (array_var($report_data, 'invoice_status') == 'all') {
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['labor_invoiced'], $data['totals']['currency_sym']));
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['labor_not_invoiced'], $data['totals']['currency_sym']));
		}
		if (array_var($report_data, 'include_pending_time')) $csv .= ",".$data['totals']['pending_hours'];
		if (array_var($report_data, 'include_pending_billing')) {
			$pending_billing = format_money_amount($data['totals']['labor_pending_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','',$pending_billing);
		}
		if (array_var($report_data, 'include_rate_schedule')) $csv .= ",";
		if (array_var($report_data, 'include_approval_status')) $csv .= ",";
		if (array_var($report_data, 'include_modules_dim')) $csv .= ",";
		$csv .= "\n";
		
		// Expenses total row
		$exp_total = str_replace(',','', format_money_amount($data['totals']['expenses_total'], $data['totals']['currency_sym']));
		$csv .= ",,".lang('expenses total').",";
		if (array_var($report_data, 'include_time_description')) $csv .= ",";
		if (array_var($report_data, 'include_billing_rate_name')) $csv .= ",";
		if (array_var($report_data, 'include_miles')) $csv .= ",";
		if (array_var($report_data, 'include_estimated_time')) $csv .= ",";
		if (array_var($report_data, 'include_estimated_billing')){
			$estimated_billing = format_money_amount($data['totals']['estimated_expense_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','', $estimated_billing);
		}
		$csv .= ",,".$exp_total;
		if (array_var($report_data, 'invoice_status') == 'all') {
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['expense_invoiced'], $data['totals']['currency_sym']));
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['expense_not_invoiced'], $data['totals']['currency_sym']));
		}
		if (array_var($report_data, 'include_pending_time')) $csv .= ",";
		if (array_var($report_data, 'include_pending_billing')){
			$pending_billing = format_money_amount($data['totals']['pending_expense_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','', $pending_billing);
		}
		if (array_var($report_data, 'include_rate_schedule')) $csv .= ",";
		if (array_var($report_data, 'include_approval_status')) $csv .= ",";
		if (array_var($report_data, 'include_modules_dim')) $csv .= ",";
		$csv .= "\n";

		// Overall total row
		$total = str_replace(',','', format_money_amount($data['totals']['timeslots_total'] + $data['totals']['expenses_total'], $data['totals']['currency_sym']));
		$csv .= ",,".lang('labor and expenses total').",";
		if (array_var($report_data, 'include_time_description')) $csv .= ",";
		if (array_var($report_data, 'include_billing_rate_name')) $csv .= ",";
		if (array_var($report_data, 'include_miles')) $csv .= ",";
		if (array_var($report_data, 'include_estimated_time')) $csv .= ",";
		if (array_var($report_data, 'include_estimated_billing')){
			$estimated_billing = format_money_amount($data['totals']['total_estimated_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','', $estimated_billing);
		}
		$csv .= ",,".$total;
		if (array_var($report_data, 'invoice_status') == 'all') {
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['total_invoiced'], $data['totals']['currency_sym']));
			$csv .= ",".str_replace(',','', format_money_amount($data['totals']['total_not_invoiced'], $data['totals']['currency_sym']));
		}
		if (array_var($report_data, 'include_pending_time')) $csv .= ",";
		if (array_var($report_data, 'include_pending_billing')){
			$pending_billing = format_money_amount($data['totals']['total_pending_billing'], $data['totals']['currency_sym']);
			$csv .= ",".str_replace(',','', $pending_billing);
		}
		if (array_var($report_data, 'include_rate_schedule')) $csv .= ",";
		if (array_var($report_data, 'include_approval_status')) $csv .= ",";
		if (array_var($report_data, 'include_modules_dim')) $csv .= ",";
		$csv .= "\n";

		// Project Budget Total Row
		if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) {
			if(array_var($report_data, 'for_budget_use') == 'tasks_expenses'){
				$row_title = lang('sum of budgeted tasks + expenses'); 
			} else if (array_var($report_data, 'for_budget_use') == 'budget_total_cp'){ 
				$project_object_type = ObjectTypes::findOne(array("conditions" => "`name`='project'"));
				$cp_object_type_id = $project_object_type->getId();
				$budget_custom_property = CustomProperties::findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"'));
				$row_title =  $budget_custom_property->getName();
			} 
			$total = str_replace(',','', format_money_amount($data['totals']['project_total_budget'], $data['totals']['currency_sym']));
			$csv .= ",,".$row_title.",";
			if (array_var($report_data, 'include_time_description')) $csv .= ",";
			if (array_var($report_data, 'include_billing_rate_name')) $csv .= ",";
			if (array_var($report_data, 'include_miles')) $csv .= ",";
			if (array_var($report_data, 'include_estimated_time')) $csv .= ",";
			if (array_var($report_data, 'include_estimated_billing')) $csv .= ",";
			$csv .= ",,".$total;
			if (array_var($report_data, 'invoice_status') == 'all') {
				$csv .= ",";
				$csv .= ",";
			}
			if (array_var($report_data, 'include_pending_time')) $csv .= ",";
			if (array_var($report_data, 'include_pending_billing')) $csv .= ",";
			if (array_var($report_data, 'include_rate_schedule')) $csv .= ",";
			if (array_var($report_data, 'include_approval_status')) $csv .= ",";
			if (array_var($report_data, 'include_modules_dim')) $csv .= ",";
			$csv .= "\n";
		}

		// Project Budget Total Row
		if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) {
			$budget_difference = $data['totals']['project_total_budget'] == 0 ? 0 :($data['totals']['timeslots_total'] + $data['totals']['expenses_total'])/ $data['totals'] ['project_total_budget'] * 100;
			$total = str_replace(',','', format_money_amount($data['totals']['project_total_budget'], $data['totals']['currency_sym']));
			$csv .= ",,".lang('budget consumption').",";
			if (array_var($report_data, 'include_time_description')) $csv .= ",";
			if (array_var($report_data, 'include_billing_rate_name')) $csv .= ",";
			if (array_var($report_data, 'include_miles')) $csv .= ",";
			if (array_var($report_data, 'include_estimated_time')) $csv .= ",";
			if (array_var($report_data, 'include_estimated_billing')) $csv .= ",";
			$csv .= ",,".str_replace(',','', round($budget_difference) . '%');
			if (array_var($report_data, 'invoice_status') == 'all') {
				$csv .= ",";
				$csv .= ",";
			}
			if (array_var($report_data, 'include_pending_time')) $csv .= ",";
			if (array_var($report_data, 'include_pending_billing')) $csv .= ",";
			if (array_var($report_data, 'include_rate_schedule')) $csv .= ",";
			if (array_var($report_data, 'include_approval_status')) $csv .= ",";
			if (array_var($report_data, 'include_modules_dim')) $csv .= ",";
			$csv .= "\n";
		}

		return $csv;
	}

}