<?php

function build_aging_invoices_report_csv($data){
    $csv = "";
    $csv .= build_all_aging_invoices_report_csv($data);

    return $csv;
}

function build_all_aging_invoices_report_csv($data){
    $csv = "";
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        $csv .= build_contact_info_csv($contact_info);
    }

    $csv .= build_report_totals_csv($data['totals'], $data['distribution']);

    return $csv;
}


function build_contact_info_csv($contact_info){
    // Get custom property 'project_manager'
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $pm_label = $cp_project_manager instanceof CustomProperty ? $cp_project_manager->getname() : lang('project manager');
    $csv = "";
    // Project Manager name
    $csv .= str_replace(',',' ', $pm_label.': '.$contact_info['contact_name']) . "\n";
    // Header columns
    $header_columns = array(' ', 'invoice', 'date', 'balance', 'current', '31-60', '61-90', '91-120', 'over 120');
    $csv .= implode(',', $header_columns) . "\n";


    // Build projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){
        $csv .= build_project_info_csv($project_info);
    }

    // Render Contact Totals
    $csv .= build_contact_totals_csv($contact_info['contact_name'], $project_and_invoices_info['contact_totals']);
    $csv .= ' ' . "\n";

    return $csv;
}

function build_project_info_csv($project_info){

	// Show client name if exist
	$client_name = (!empty($project_info['client_name'])) ? lang('customer') . ': ' . $project_info['client_name'] . ' - ' : '';

    $csv = "";
     // Project name row
     $project_name = $client_name . lang('project').': '.$project_info['project_info']['full_project_name'];
     $csv .= str_replace(',',' ', $project_name) . "\n";
     // Render invoices
     foreach($project_info['invoices'] as $invoice){
        $csv .= build_invoice_info_csv($invoice);
     }
     // Project total row
     $csv .= build_project_total_csv($project_info);

     return $csv;
}

function build_invoice_info_csv($invoice){
    $csv = "";
    $currency_sym = config_option('currency_code');
    // Build invoice row
    $invoice_row = array('', str_replace(',',' ', $invoice['invoice_number']), str_replace(',',' ', $invoice['date']));
    $invoice_row[] = format_money_amount($invoice['total'], $currency_sym, null, '.', '');
    $invoice_row[] = format_money_amount($invoice['first_month'], $currency_sym, null, '.', '');
    $invoice_row[] = format_money_amount($invoice['second_month'], $currency_sym, null, '.', '');
    $invoice_row[] = format_money_amount($invoice['third_month'], $currency_sym, null, '.', '');
    $invoice_row[] = format_money_amount($invoice['fourth_month'], $currency_sym, null, '.', '');
    $invoice_row[] = format_money_amount($invoice['greater_than_four_month'], $currency_sym, null, '.', '');
    
    $csv .= implode(',', $invoice_row) . "\n";

    return $csv;
}

function build_project_total_csv($project_info){
    $csv = "";
    $project_totals = $project_info['project_totals'];
    $currency_sym = config_option('currency_code');
    // Build project totals row
    $project_total_title = lang('total for').' '.$project_info['project_info']['project_id'];
    $project_totals_row = array(str_replace(',',' ', $project_total_title), '','');
    $project_totals_row[] = format_money_amount($project_totals['total'], $currency_sym, null, '.', '');
    $project_totals_row[] = format_money_amount($project_totals['first_month'], $currency_sym, null, '.', '');
    $project_totals_row[] = format_money_amount($project_totals['second_month'], $currency_sym, null, '.', '');
    $project_totals_row[] = format_money_amount($project_totals['third_month'], $currency_sym, null, '.', '');
    $project_totals_row[] = format_money_amount($project_totals['fourth_month'], $currency_sym, null, '.', '');
    $project_totals_row[] = format_money_amount($project_totals['greater_than_four_month'], $currency_sym, null, '.', '');

    $csv .= implode(',', $project_totals_row) . "\n";

    return $csv;
}

function build_contact_totals_csv($contact_name, $contact_totals){
    $csv = "";
    $currency_sym = config_option('currency_code');
    // Build project totals row
    $contact_total_title = lang('total for').' '.$contact_name;
    $contact_totals_row = array(str_replace(',',' ', $contact_total_title), '','');
    $contact_totals_row[] = format_money_amount($contact_totals['total'], $currency_sym, null, '.', '');
    $contact_totals_row[] = format_money_amount($contact_totals['first_month'], $currency_sym, null, '.', '');
    $contact_totals_row[] = format_money_amount($contact_totals['second_month'], $currency_sym, null, '.', '');
    $contact_totals_row[] = format_money_amount($contact_totals['third_month'], $currency_sym, null, '.', '');
    $contact_totals_row[] = format_money_amount($contact_totals['fourth_month'], $currency_sym, null, '.', '');
    $contact_totals_row[] = format_money_amount($contact_totals['greater_than_four_month'], $currency_sym, null, '.', '');

    $csv .= implode(',', $contact_totals_row) . "\n";

    return $csv;
}


function build_report_totals_csv($totals, $distribution_totals){
    $csv = "";
    $currency_sym = config_option('currency_code');
   
    $csv .= lang('totals') . "\n";
    
    $header_columns = array('', '', '','balance', 'current', '31-60', '61-90', '91-120', 'over 120');
    $csv .= implode(',', $header_columns) . "\n";

     // Build totals row
    $total_title = lang('final totals');
    $totals_row = array(str_replace(',',' ', $total_title), '','');
    $totals_row[] = format_money_amount($totals['total'], $currency_sym, null, '.', '');
    $totals_row[] = format_money_amount($totals['first_month'], $currency_sym, null, '.', '');
    $totals_row[] = format_money_amount($totals['second_month'], $currency_sym, null, '.', '');
    $totals_row[] = format_money_amount($totals['third_month'], $currency_sym, null, '.', '');
    $totals_row[] = format_money_amount($totals['fourth_month'], $currency_sym, null, '.', '');
    $totals_row[] = format_money_amount($totals['greater_than_four_month'], $currency_sym, null, '.', '');
    
    $csv .= implode(',', $totals_row) . "\n";

      // Build distribution row
    $distribution_title = lang('distribution');
    $distribution_row = array(str_replace(',',' ', $distribution_title), '','');
    $distribution_row[] = format_money_amount($distribution_totals['total'], '', 0, '.', '') . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['first_month'], '', 0, '.', '') . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['second_month'], '', 0, '.', '') . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['third_month'], '', 0, '.', '') . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['fourth_month'], '', 0, '.', '') . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['greater_than_four_month'], '', 0, '.', '') . ' %';
    
    $csv .= implode(',', $distribution_row) . "\n";

    return $csv;
}
