<?php

function build_aging_invoices_report_excel($data, $report_data){
    $excel = build_all_aging_invoices_report_excel($data);
    //Logger::log_r($excel);
    return $excel;
}

function build_all_aging_invoices_report_excel($data){
    $excel = array();
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        $excel[] = build_contact_info_excel($contact_info);
        $project_and_invoices_info = $contact_info['project_and_invoices_info'];
        $excel[] = build_contact_totals_excel($contact_info['contact_name'], $project_and_invoices_info['contact_totals']);
    }

    $excel[] = build_report_totals_excel($data['totals'], $data['distribution']);

    


    return $excel;
}


function build_contact_info_excel($contact_info){
    // Get custom property 'project_manager'
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $pm_label = $cp_project_manager instanceof CustomProperty ? $cp_project_manager->getname() : lang('project manager');

    
    // Project Manager name
    $pm_name = $pm_label.': '.$contact_info['contact_name'];
    $header = array($pm_name, '', '', '', '', '', '', '', '');

    // Header columns
    $data = array();
    $data[] = array(' ', lang('invoice'),  lang('date'),  lang('balance'),  lang('current'),  lang('31-60'),  lang('61-90'),  lang('91-120'),  lang('over 120'));

    // Build projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){

		// Show client name if exist
		$client_name = (!empty($project_info['client_name'])) ? lang('customer') . ': ' . $project_info['client_name'] . ' - ' : '';

        $data[] = array($client_name . lang('project').': '.$project_info['project_info']['full_project_name'], '', '', '', '', '', '', '', '');
        // Render invoices
        foreach($project_info['invoices'] as $invoice){
            $data[] = build_invoice_info_excel($invoice);
        }
        // Project total row
        $data[] = build_project_total_excel($project_info);
    }

    return array('headers' => $header, 'data' => $data);
}


function build_invoice_info_excel($invoice){
    $currency_sym = config_option('currency_code');
    // Build invoice row
    $invoice_row = array('', $invoice['invoice_number'], $invoice['date']);
    $invoice_row[] = format_money_amount($invoice['total'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['first_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['second_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['third_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['fourth_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['greater_than_four_month'], $currency_sym);
    

    return $invoice_row;
}

function build_project_total_excel($project_info){
    $project_totals = $project_info['project_totals'];
    $currency_sym = config_option('currency_code');
    // Build project totals row
    $project_total_title = lang('total for').' '.$project_info['project_info']['project_id'];
    $project_totals_row = array($project_total_title, '','');
    $project_totals_row[] = format_money_amount($project_totals['total'], $currency_sym);
    $project_totals_row[] = format_money_amount($project_totals['first_month'], $currency_sym);
    $project_totals_row[] = format_money_amount($project_totals['second_month'], $currency_sym);
    $project_totals_row[] = format_money_amount($project_totals['third_month'], $currency_sym);
    $project_totals_row[] = format_money_amount($project_totals['fourth_month'], $currency_sym);
    $project_totals_row[] = format_money_amount($project_totals['greater_than_four_month'], $currency_sym);

    return $project_totals_row;
}

function build_contact_totals_excel($contact_name, $contact_totals){
    $excel = "";
    $currency_sym = config_option('currency_code');
    // Build project totals row
    $contact_total_title = lang('total for').' '.$contact_name;
    $contact_totals_row = array($contact_total_title, '','');
    $contact_totals_row[] = format_money_amount($contact_totals['total'], $currency_sym);
    $contact_totals_row[] = format_money_amount($contact_totals['first_month'], $currency_sym);
    $contact_totals_row[] = format_money_amount($contact_totals['second_month'], $currency_sym);
    $contact_totals_row[] = format_money_amount($contact_totals['third_month'], $currency_sym);
    $contact_totals_row[] = format_money_amount($contact_totals['fourth_month'], $currency_sym);
    $contact_totals_row[] = format_money_amount($contact_totals['greater_than_four_month'], $currency_sym);

    return array('totals' => $contact_totals_row);
}


function build_report_totals_excel($totals, $distribution_totals){
    $currency_sym = config_option('currency_code');
   
    $header = array(lang('totals'), '', '', '', '', '', '', '', '');
    $totals_excel = array();
    $totals_excel[] = array('', '', '',lang('balance'), lang('current'), lang('31-60'), lang('61-90'), lang('91-120'), lang('over 120'));

     // Build totals row
    $total_title = lang('final totals');
    $totals_row = array($total_title, '','');
    $totals_row[] = format_money_amount($totals['total'], $currency_sym);
    $totals_row[] = format_money_amount($totals['first_month'], $currency_sym);
    $totals_row[] = format_money_amount($totals['second_month'], $currency_sym);
    $totals_row[] = format_money_amount($totals['third_month'], $currency_sym);
    $totals_row[] = format_money_amount($totals['fourth_month'], $currency_sym);
    $totals_row[] = format_money_amount($totals['greater_than_four_month'], $currency_sym);
    
    $totals_excel[] =  $totals_row;
    

      // Build distribution row
    $distribution_title = lang('distribution');
    $distribution_row = array($distribution_title, '','');
    $distribution_row[] = format_money_amount($distribution_totals['total'], '', 0) . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['first_month'], '', 0) . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['second_month'], '', 0) . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['third_month'], '', 0) . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['fourth_month'], '', 0) . ' %';
    $distribution_row[] = format_money_amount($distribution_totals['greater_than_four_month'], '', 0) . ' %';
    
    $totals_excel[] = $distribution_row;

    return array('headers' => $header, 'data' => $totals_excel);
}
