<?php


function get_ar_ledger_report_data($report_data) {

	/**
	 * get invoices by project and date range
	 * get all payment receipts related to the invoices found
	 * 
	 * Group data by project and invoice
	 * 
	 * Project AAAAA
	 * 		Invoice X
	 * 			Invoice X data
	 * 			Payment X1 data
	 * 			Payment X2 data
	 * 		Totals Invoice X
	 * 		Invoice Y
	 * 			Invoice Y data
	 * 			Payment Y1 data
	 * 			Payment Y2 data
	 * 		Totals Invoice Y
	 * Totals AAAAA
	 */

	$projects_dim = Dimensions::findByCode('customer_project');

	// selected project
	$project_member_id_param = json_decode(array_var($report_data, 'project_member_id'), true);
	$project_member_id = array_var($project_member_id_param, 0);

	// dates config
	Env::useHelper('advanced_reports', 'advanced_reports');
	$report_dates = parse_report_param_date_type($report_data, false);
	$start_date = array_var($report_dates, 'st');
	$end_date = array_var($report_dates, 'et');

	// conditions for date filter
	$date_conditions = "";
	if ($start_date instanceof DateTimeValue) {
		$date_conditions .= " AND i.`date` >= '".$start_date->toMySQL()."'";
	}
	if ($end_date instanceof DateTimeValue) {
		$date_conditions .= " AND i.`date` <= '".$end_date->toMySQL()."'";
	}

	// condition for project if selected
	$project_condition = "";
	if (is_numeric($project_member_id) && $project_member_id > 0) {
		$project_condition = " AND om.member_id = $project_member_id";
	}

	// sql to get the invoice ids along with their project member id
	$sql = "
		select i.*, o.name, m.id as project_member_id, os.name as invoice_type_name, os.sum_in_total
		from ".TABLE_PREFIX."income_invoices i
		inner join ".TABLE_PREFIX."objects o on o.id = i.object_id
		inner join ".TABLE_PREFIX."object_subtypes os on os.id=o.object_subtype_id
		inner join ".TABLE_PREFIX."object_members om on om.object_id = i.object_id
		inner join ".TABLE_PREFIX."members m on m.id = om.member_id
		where
		m.dimension_id = ".$projects_dim->getId()."
		$date_conditions $project_condition
	";
	$invoice_rows = DB::executeAll($sql);

	// get all invoice ids to get their payments in one query
	$all_invoice_ids = array();
	foreach ($invoice_rows as $invoice_row) {
		$all_invoice_ids[] = $invoice_row['object_id'];
	}

	// get invoice payments
	$invoice_payments = array();
	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		$invoice_payments = InvoicePaymentReceipts::getPaymentsForInvoiceIds($all_invoice_ids);
	}

	// group each invoice and its payments by project
	$project_totals = array();
	$project_invoices = array();
	foreach ($invoice_rows as &$invoice_row) {

		$invoice_row['taxes'] = $invoice_row['total'] - $invoice_row['subtotal'];

		$project_member_id = $invoice_row['project_member_id'];

		if (!isset($project_invoices[$project_member_id])) $project_invoices[$project_member_id] = array();

		$invoice_key = trim($invoice_row['series']." ".$invoice_row['invoice_number']." ".$invoice_row['object_id']);
		$project_invoices[$project_member_id][$invoice_key] = array(
			'invoice' => $invoice_row,
			'payments' => array_var($invoice_payments, $invoice_row['object_id']),
		);

		if (!isset($project_totals[$project_member_id])) {
			$project_totals[$project_member_id] = array(
				'total' => 0,
				'subtotal' => 0,
				'taxes' => 0,
				'due_amount' => 0,
				'currency_id' => $invoice_row['currency_id'],
			);
		}

		if ($invoice_row['sum_in_total']) {
			// normal invoices
			$project_totals[$project_member_id]['total'] += $invoice_row['total'];
			$project_totals[$project_member_id]['subtotal'] += $invoice_row['subtotal'];
			$project_totals[$project_member_id]['due_amount'] += $invoice_row['due_amount'];
			$project_totals[$project_member_id]['taxes'] += $invoice_row['total'] - $invoice_row['subtotal'];
		} else {
			// credit notes and debit notes
			$project_totals[$project_member_id]['total'] -= $invoice_row['total'];
			$project_totals[$project_member_id]['subtotal'] -= $invoice_row['subtotal'];
			$project_totals[$project_member_id]['due_amount'] -= $invoice_row['due_amount'];
			$project_totals[$project_member_id]['taxes'] -= $invoice_row['total'] - $invoice_row['subtotal'];
		}
		
	}

	// sort invoices inside each project
	foreach ($project_invoices as &$pi) {
		ksort($pi);
	}

	// build all project data (project name, totals, etc)
	$project_groups = array();
	foreach ($project_invoices as $mem_id => $project_data) {
		// get the project member object
		$member = Members::getMemberById($mem_id);
		// get complete name of the project
		$tmp_mem = array($member);
		build_member_list_text_to_show_in_trees($tmp_mem);
		
		$project_groups[$member->getName()] = array(
			'project_id' => $member->getId(),
			'project_name' => $member->getName(),
			'grouped_invoices' => array_var($project_invoices, $member->getId()),
			'totals' => array_var($project_totals, $member->getId()),
		);

	}

	// sort project groups by project name
	ksort($project_groups);

	// build report totals
	$report_totals = null;
	foreach ($project_groups as $group_data) {
		if (!$report_totals) {
			$report_totals = array(
				'total' => 0,
				'subtotal' => 0,
				'taxes' => 0,
				'due_amount' => 0,
				'currency_id' => $group_data['totals']['currency_id'],
			);
		}
		$report_totals['total'] += $group_data['totals']['total'];
		$report_totals['subtotal'] += $group_data['totals']['subtotal'];
		$report_totals['due_amount'] += $group_data['totals']['due_amount'];
		$report_totals['taxes'] += $group_data['totals']['taxes'];
	}


	$columns = array('invoice', 'receipt_detail', 'date', 'total', 'fees', 'taxes');

	$data = array(
		'columns' => $columns,
		'ar_ledger_data' => $project_groups,
		'ar_ledger_totals' => $report_totals,
		'from_date' => $start_date,
		'to_date' => $end_date,
	);

	return $data;
}