<?php

function build_budget_report_csv($report_result){
    $csv = "";
		
    $projects_data = array_var($report_result, 'projects_data', array());
    $project_phases = array_var($report_result, 'project_phases', array());
    $totals =  array_var($report_result, 'totals');

    foreach ($projects_data as $project_data) {
        
        $csv .= strtoupper(str_replace(',','', $project_data['project_name'].' '.$project_data['project_id_raw']))."\n";
        
        $csv .= build_labor_table_budget_report_csv($project_data, $project_phases);
        
        $csv .= build_labor_summary_budget_report_csv($project_data, $project_phases);
        
        $csv .= build_expenses_table_budget_report_csv($project_data, $project_phases);
        
        $csv .= build_expenses_summary_budget_report_csv($project_data, $project_phases);
        
    }
    
    $csv .= build_totals_budget_report_csv($totals, $project_phases);

    return $csv;
}

function build_labor_table_budget_report_csv($project_data, $project_phases){
    $csv = "";
    $labor_data = $project_data['labor_data'];

    $currency_sym = config_option('currency_code');
    $decimals = user_config_option('decimal_digits');

    //$csv .= str_replace(',','', lang('labor'))."\n";
    $header = array(lang('labor'));
    foreach($project_phases as $phase){
        $header[] = str_replace(',','', $phase);
    }
    $header[] = lang('total hours');
    $header[] = lang('rate');
    $header[] = lang('total');

    $csv .= implode(',',$header);
    $csv .= "\n";

    foreach($labor_data['labor_names'] as $labor_id => $labor_name) {
        $row = array(str_replace(',','', $labor_name));
        foreach($project_phases as $phase){
            $row[] = isset($labor_data['project_phases'][$phase][$labor_id]) && $labor_data['project_phases'][$phase][$labor_id] > 0 ? number_format($labor_data['project_phases'][$phase][$labor_id], $decimals, '.', '') : '';
        }
        $row[] = number_format($labor_data['total_per_labor_hours'][$labor_id], $decimals, '.', '');
        $row[] = $currency_sym.' '.number_format($labor_data['hourly_rates'][$labor_id],  $decimals, '.', '');
        $row[] = $currency_sym.' '.number_format($labor_data['total_per_labor_amount'][$labor_id],  $decimals, '.', '');

        $csv .= implode(',', $row);
        $csv .= "\n";
    }
    
    return $csv;
}

function build_labor_summary_budget_report_csv($project_data, $project_phases){
    $csv = "";

    $decimals = user_config_option('decimal_digits');
    $currency_sym = config_option('currency_code');
    $labor_summary = $project_data['labor_summary'];

    $summary_row = array(lang('labor subtotal budget report'));

    foreach($project_phases as $phase){
        $summary_row[] = isset($labor_summary['project_phases_subtotals'][$phase]) ?  number_format($labor_summary['project_phases_subtotals'][$phase],  $decimals, '.', '') : number_format(0,  2, '.', '');
    }

    $summary_row[] = number_format($labor_summary['subtotal_hours'],  $decimals, '.', '');
    $summary_row[] = '';
    $summary_row[] = $currency_sym.' '.number_format($labor_summary['subtotal_amount'], $decimals, '.', '');

    $csv .= implode(',', $summary_row) . "\n";
    $csv .= "\n";

    return $csv;
}

function build_expenses_table_budget_report_csv($project_data, $project_phases){
    $csv = "";

    $report_data = array_var($_SESSION, 'budget_report_data');
    $expenses_data = $project_data['expenses_data'];
    $group_expenses_by = isset($report_data['group_expenses_by']) ? $report_data['group_expenses_by'] : 0;
    $decimals = user_config_option('decimal_digits');
    $currency_sym = config_option('currency_code');

    if($group_expenses_by == 0) { // Group by product type

        // Build expenses table header
        //$csv .= str_replace(',','', lang('expenses'))."\n";
        $header = array(lang('expenses'));
        foreach($project_phases as $phase){
            $header[] = str_replace(',','', $phase);
        }
        $header[] = lang('quantity total');
        $header[] = lang('unit price');
        $header[] = lang('total');

        $csv .= implode(',', $header);
        $csv .= "\n";

        // Build expenses table body
        foreach($expenses_data['expenses_names'] as $expense_id => $expense_name) {
            $row = array(str_replace(',','', $expense_name));
            foreach($project_phases as $phase){
                $row[] = isset($expenses_data['project_phases'][$phase][$expense_id]) ? number_format($expenses_data['project_phases'][$phase][$expense_id], $decimals, '.', '') : '';
            }
            $row[] = number_format($expenses_data['expenses_total_quantity'][$expense_id], $decimals, '.', '');
            $row[] = $currency_sym.' '.number_format($expenses_data['unit_prices'][$expense_id], $decimals, '.', '');
            $row[] = $currency_sym.' '.number_format($expenses_data['amount_per_expense'][$expense_id], $decimals, '.', '');
            
            $csv .= implode(',', $row);
            $csv .= "\n";
        }

    } else if($group_expenses_by == 1){ // Group by expense categories

        // Build expenses table header
        //$csv .= str_replace(',','', lang('expenses'))."\n";
        $header = array(lang('expenses'));
        foreach($project_phases as $phase){
            $header[] = str_replace(',','', $phase);
        }
        $header[] = '';
        $header[] = '';
        $header[] = lang('total');

        $csv .= implode(',', $header);
        $csv .= "\n";

        // Build expenses table body
        foreach($expenses_data['expenses_names'] as $expense_name) {
            $row = array(str_replace(',','', $expense_name));
            foreach($project_phases as $phase){
                $row[] = isset($expenses_data['project_phases'][$phase][$expense_name]) ? $currency_sym.' '.number_format($expenses_data['project_phases'][$phase][$expense_name], $decimals, '.', '') : '';
            }
            $row[] = '';
            $row[] = '';
            $row[] = $currency_sym.' '.number_format($expenses_data['amount_per_expense'][$expense_name], $decimals, '.', '');
            
            $csv .= implode(',', $row);
            $csv .= "\n";
        }

    }


    return $csv;
}

function build_expenses_summary_budget_report_csv($project_data, $project_phases){
    // Build expenses summary
    $csv = "";
    $currency_sym = config_option('currency_code');
    $decimals = user_config_option('decimal_digits');
    $expenses_summary = $project_data['expenses_summary']['subtotal_amount'];

    $summary_row = array(lang('expenses subtotal'));
    
    $footer_columns = count($project_phases) + 2;
    for( $i = 0; $i < $footer_columns; $i++){
        $summary_row[] = '';
    }

    $summary_row[] = $currency_sym.' '.number_format($expenses_summary, $decimals, '.', '');
    $csv .= implode(',', $summary_row) . "\n";
    $csv .= "\n";

    return $csv;

}

function build_totals_budget_report_csv($totals, $project_phases){
    $csv = "";
    $currency_sym = config_option('currency_code');
    $decimals = user_config_option('decimal_digits');
    $overall_total = $totals['labor_total'] + $totals['expenses_total'];
    $empty_columns = array_fill(0, 2 + count($project_phases), '');

    // Build expenses table header
    $csv .= strtoupper(lang('totals')) . "\n";

    $labor_totals = array(lang('labor total'));
    $labor_totals = array_merge($labor_totals, $empty_columns);
    $labor_totals[] = $currency_sym.' '.number_format($totals['labor_total'], $decimals, '.', '');
    $csv .= implode(',', $labor_totals) . "\n";
    
    $expenses_totals = array(lang('expenses total'));
    $expenses_totals = array_merge($expenses_totals, $empty_columns);
    $expenses_totals[] = $currency_sym.' '.number_format($totals['expenses_total'], $decimals, '.', '');
    $csv .= implode(',', $expenses_totals) . "\n";

    $grand_total = array(lang('total'));
    $grand_total = array_merge($grand_total, $empty_columns);
    $grand_total[] = $currency_sym.' '.number_format($overall_total, $decimals, '.', '');
    $csv .= implode(',', $grand_total) . "\n";

    return $csv;
}

