<?php

function build_excel_data_for_budget_report($result_data, $report_data){

    $excel_data = array();
    $project_information_data = array();
    $project_member_id = $result_data['project_member']->getId();

    // Build project information if needed
    if (array_var($report_data, 'show_main_project_info')) {
        $project_information_data = build_budget_project_information_excel($project_member_id, $result_data);
    }

    $excel_data = array_merge($excel_data, $project_information_data);
    $project_phases = $result_data['project_phases'];
    $count_phases = count($project_phases);
    $projects_data = $result_data['projects_data'];
    $totals =  array_var($result_data, 'totals');

    foreach($projects_data as $project_data){

        $header_cols = array($project_data['project_name'].' '.$project_data['project_id_raw'],'','','',);
        foreach($project_phases as $phase){
            $header_cols[] = '';
        }

        $excel_data[] = array('main_header' => $header_cols);

        // Get labor data
        $labor_excel_data = build_budget_labor_excel($project_data, $project_phases);
        $excel_data = array_merge($excel_data, $labor_excel_data);

        // Get expenses data
        $expenses_excel_data = build_budget_expenses_excel($project_data, $project_phases, $report_data);
        $excel_data = array_merge($excel_data, $expenses_excel_data);

        // Get empty row after each project
        $empty_row = array(array('data' => array(array(''), array(''))));
        $excel_data = array_merge($excel_data, $empty_row);
    }
    // Get totals data
    $totals_data = build_budget_totals_excel($totals, $project_phases);
    $excel_data = array_merge($excel_data, $totals_data);
    
    return $excel_data;
}

function build_budget_project_information_excel($project_member_id, $result_data) {
    $report_data = array_var($_SESSION, 'budget_report_data');
    $project_phases = array_var($result_data, 'project_phases');
    $project_member = Members::getMemberById($project_member_id);
    if (!$project_member instanceof Member) {
        return array();
    }
    
    $excel_data = array();
    
    $properties_array = '';
    $params =  array(
            'member' => $project_member,
            'visible_by_default' => true,
            'return_as_array' => true,
    );
    Hook::fire('override_render_properties_for_view', $params, $properties_array);
    
    $left_properties_array = array();
    $right_properties_array = array();
    foreach ($properties_array as $group_id => $group_data) {
        if (count($group_data['properties']) == 0) continue;
        
        $property_group = PropertyGroups::findById($group_id);
        $group_data['alignment'] = $property_group->getAlignment();
        $group_data['name'] = $property_group->getName();
        
        if ($group_data['alignment'] == 'left') {
            $left_properties_array[] = $group_data;
        } else if ($group_data['alignment'] == 'right') {
            $right_properties_array[] = $group_data;
        }
    }
    
    
    // get project name and project id
    $project_prefix_object = ObjectPrefixes::findById($project_member->getObjectId());
	if($project_prefix_object){
		$project_id = $project_prefix_object->getPrefix();
		$project_name = str_replace($project_id.' - ', '', $project_member->getName());
	} else {
		$pot = ObjectTypes::findByName('project');
		$project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
		$project_id = '';
		if ($project_id_cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
			if ($cp_val instanceof CustomPropertyValue) {
				$project_id = $cp_val->getValue();
			}
		}
		$project_name =  $project_member->getName();
	}
    
    $main_header_array = array($project_name.' '.$project_id, '', '', '');

    foreach($project_phases as $phase){
        $main_header_array[] = '';
    }


    $excel_data[] = array('headers' => array(), 'data' => array(), 'main_header' => $main_header_array);
    
    $i = 0;
    $more_groups_left = true;
    while ($more_groups_left) {
        $more_groups_left = false;
        
        $headers = array();
        $data = array();
        
        if (isset($left_properties_array[$i])) {
            $group_data = $left_properties_array[$i];
            $more_groups_left = true;
            
            $headers[] = $group_data['name'];
            $headers[] = '';
            $headers[] = '';
            
            $j = 0;
            foreach ($group_data['properties'] as $property) {
                $data[$j] = array();
                
                $data[$j][0] = $property['label'];
                $data[$j][1] = $property['value'];
                $data[$j][2] = '';
                
                $j++;
            }
        } else {
            $headers[] = '';
            $headers[] = '';
            $headers[] = '';
        }
        
        if (isset($right_properties_array[$i])) {
            $group_data = $right_properties_array[$i];
            $more_groups_left = true;
            
            $headers[] = $group_data['name'];
            $headers[] = '';
            
            $j = 0;
            foreach ($group_data['properties'] as $property) {
                if (!isset($data[$j])) $data[$j] = array('','','');
                
                $data[$j][3] = $property['label'];
                $data[$j][4] = $property['value'];
                
                $j++;
            }
        } else {
            $headers[] = '';
            $headers[] = '';
        }
        
        for($k=0; $k<count($project_phases)-1; $k++){
            $headers[] = '';
        }

        $excel_data[] = array('headers' => $headers, 'data' => $data);
        
        $i++;
    }
    
    return $excel_data;
}

function build_budget_labor_excel($project_data, $project_phases){
    $excel_data = array();
    $labor_data = build_budget_labor_body_excel($project_data, $project_phases);
    if($labor_data){
        $excel_data[] = array('main_header'=>array_var($labor_data, 'main_header'), 'headers' => array_var($labor_data, 'headers'), 'data' => array_var($labor_data, 'data'));
    }
    $labor_summary = build_budget_labor_summary_excel($project_data, $project_phases);
    if($labor_summary){
        $excel_data[] = array('headers' => array_var($labor_summary, 'headers'), 'data' => array_var($labor_summary, 'data'), 'totals' => array_var($labor_summary, 'totals'));
    }

    return $excel_data;
}


function build_budget_labor_body_excel($project_data, $project_phases){
    //$excel_data = array('main_header' = array(), 'headers' => array(), 'data' => array());
    $labor_data = $project_data['labor_data'];

    $currency_sym = config_option('currency_code');

    $header = array(lang('labor'));
    foreach($project_phases as $phase){
        $header[] = $phase;
    }
    $header[] = lang('total hours');
    $header[] = lang('rate');
    $header[] = lang('total');

    $data = array();
    foreach($labor_data['labor_names'] as $labor_id => $labor_name) {
        $row = array($labor_name);
        foreach($project_phases as $phase){
            $row[] = isset($labor_data['project_phases'][$phase][$labor_id]) && $labor_data['project_phases'][$phase][$labor_id] > 0 ? number_format($labor_data['project_phases'][$phase][$labor_id], 2) : '';
        }
        $row[] = number_format($labor_data['total_per_labor_hours'][$labor_id], 2);
        $row[] = format_money_amount($labor_data['hourly_rates'][$labor_id], $currency_sym);
        $row[] = format_money_amount($labor_data['total_per_labor_amount'][$labor_id], $currency_sym);
        $data[] = $row;
    }
    
    return array('headers' => $header, 'data' => $data);
}


function build_budget_labor_summary_excel($project_data, $project_phases){

    $currency_sym = config_option('currency_code');
    $labor_summary = $project_data['labor_summary'];

    $summary_row = array(lang('labor subtotal budget report'));

    foreach($project_phases as $phase){
        $summary_row[] = isset($labor_summary['project_phases_subtotals'][$phase]) ?  number_format($labor_summary['project_phases_subtotals'][$phase], 2) : number_format(0, 2);
    }

    $summary_row[] = number_format($labor_summary['subtotal_hours'], 2);
    $summary_row[] = '';
    $summary_row[] = format_money_amount($labor_summary['subtotal_amount'], $currency_sym);

    return array('totals'=>$summary_row);
}

function build_budget_expenses_excel($project_data = null, $project_phases = array(), $report_data = null){
    $excel_data = array();

    $expenses_data = build_budget_expenses_body_excel($project_data, $project_phases, $report_data);
    if($expenses_data){
        $excel_data[] = array('main_header'=>array_var($expenses_data, 'main_header'), 'headers' => array_var($expenses_data, 'headers'), 'data' => array_var($expenses_data, 'data'));
    }

    $expenses_summary = build_budget_expenses_summary_excel($project_data, $project_phases, $report_data);
    if($expenses_summary){
        $excel_data[] = array('totals' => array_var($expenses_summary, 'totals'));
    }

    return $excel_data;
}

function build_budget_expenses_body_excel($project_data, $project_phases, $report_data){

    $expenses_data = $project_data['expenses_data'];
    $group_expenses_by = isset($report_data['group_expenses_by']) ? $report_data['group_expenses_by'] : 0;

    $currency_sym = config_option('currency_code');

    if($group_expenses_by == 0) { // Group by product type

        // Build expenses table header
        $header = array(lang('expenses'));
        foreach($project_phases as $phase){
            $header[] = $phase;
        }
        $header[] = lang('quantity total');
        $header[] = lang('unit price');
        $header[] = lang('total');


        // Build expenses table body
        $data = array();
        foreach($expenses_data['expenses_names'] as $expense_id => $expense_name) {
            $row = array($expense_name);
            foreach($project_phases as $phase){
                $row[] = isset($expenses_data['project_phases'][$phase][$expense_id]) ? number_format($expenses_data['project_phases'][$phase][$expense_id], 2) : '';
            }
            $row[] = number_format($expenses_data['expenses_total_quantity'][$expense_id], 2);
            $row[] = format_money_amount($expenses_data['unit_prices'][$expense_id], $currency_sym);
            $row[] = format_money_amount($expenses_data['amount_per_expense'][$expense_id], $currency_sym);
            $data[] = $row;
        }

    } else if($group_expenses_by == 1){ // Group by expense categories

        // Build expenses table header
        $header = array(lang('expenses'));
        foreach($project_phases as $phase){
            $header[] = $phase;
        }
        $header[] = '';
        $header[] = '';
        $header[] = lang('total');

        // Build expenses table body
        $data = array();
        foreach($expenses_data['expenses_names'] as $expense_name) {
            $row = array($expense_name);
            foreach($project_phases as $phase){
                $row[] = isset($expenses_data['project_phases'][$phase][$expense_name]) ? format_money_amount($expenses_data['project_phases'][$phase][$expense_name],$currency_sym) : '';
            }
            $row[] = '';
            $row[] = '';
            $row[] = format_money_amount($expenses_data['amount_per_expense'][$expense_name], $currency_sym);
            $data[] = $row;
        }

    }


    return array('headers' => $header, 'data' => $data);
}

function build_budget_expenses_summary_excel($project_data, $project_phases, $report_data){
    // Build expenses summary
    $currency_sym = config_option('currency_code');
    $expenses_summary = $project_data['expenses_summary']['subtotal_amount'];

    $summary_row = array(lang('expenses subtotal'));
    
    $footer_columns = count($project_phases) + 2;
    for( $i = 0; $i < $footer_columns; $i++){
        $summary_row[] = '';
    }

    $summary_row[] = format_money_amount($expenses_summary, $currency_sym);

    return array('totals'=>$summary_row);
}

function build_budget_totals_excel($totals, $project_phases){

    $currency_sym = config_option('currency_code');

    $overall_total = $totals['labor_total'] + $totals['expenses_total'];

    $header_columns = count($project_phases) + 3;
    
    // Build expenses table header
    $header = array(lang('totals'));
    for( $i = 0; $i < $header_columns; $i++){
        $header[] = '';
    }
    
    $empty_columns = array_fill(0, 2 + count($project_phases), '');

    $data = array();

    $labor_totals = array(lang('labor total'));
    $labor_totals = array_merge($labor_totals, $empty_columns);
    $labor_totals[] = format_money_amount($totals['labor_total'], $currency_sym);
    $data[] = $labor_totals;
    
    $expenses_totals = array(lang('expenses total'));
    $expenses_totals = array_merge($expenses_totals, $empty_columns);
    $expenses_totals[] = format_money_amount($totals['expenses_total'], $currency_sym);
    $data[] = $expenses_totals;

    $grand_total = array(lang('total'));
    $grand_total = array_merge($grand_total, $empty_columns);
    $grand_total[] = format_money_amount($overall_total, $currency_sym);
    $data[] = $grand_total;

    return array(array('totals' => $labor_totals), array('totals' => $expenses_totals), array('totals' => $grand_total));
}
