<?php

function build_invoice_report_csv($data){
    $csv = "";

    $type_of_report = $data['type_of_report'];
    if($type_of_report == 'individual_project'){
        $header_params = array(
            'include_client_name' => true,
            'type_of_report' => $type_of_report
        );
        $csv .= build_csv_project_header_for_invoice_report($data, $header_params);
        $csv .= build_csv_project_data_for_invoice_report($data, $type_of_report);
    } elseif ($type_of_report == 'total_grouped_by_projects'){
        $header_params = array(
            'include_client_name' => true,
            'type_of_report' => $type_of_report
        );
        foreach($data['projects'] as $project_data){
            $csv .= build_csv_project_header_for_invoice_report($project_data, $header_params);
            $csv .= build_csv_project_data_for_invoice_report($project_data, $type_of_report);
            $csv .= "\n";
        }
        
        $total_params = array(
            'table_name' => lang('totals'),
            'total_budgeted' => $data['report_total_budgeted'],
            'total_invoiced' => $data['report_total_invoiced']
        );
        $csv .= build_invoicing_history_total_csv_table($total_params);
        
    } elseif ($type_of_report == 'total_grouped_by_clients_and_projects'){
        $header_params = array(
            'include_client_name' => true,
            'type_of_report' => $type_of_report
        );
        foreach($data['clients'] as $client_name => $client_info){
            $csv .= str_replace(',',' ', lang('client').' '.$client_name) . "\n";
            $csv .= "\n";
            foreach($client_info['projects'] as $project_data){
                $csv .= build_csv_project_header_for_invoice_report($project_data, $header_params);
                $csv .= build_csv_project_data_for_invoice_report($project_data, $type_of_report);
                $csv .= "\n";
            }
            $client_total_params = array(
                'table_name' => lang('totals').': '.$client_name,
                'total_budgeted' => $client_info['client_budget_total'],
                'total_invoiced' => $client_info['client_invoices_total']
            );
            $csv .= build_invoicing_history_total_csv_table($client_total_params);
            $csv .= "\n";
            $csv .= "\n";

        }
    
        $total_params = array(
            'table_name' => lang('totals'),
            'total_budgeted' => $data['report_total_budgeted'],
            'total_invoiced' => $data['report_total_invoiced']
        );
        $csv .= build_invoicing_history_total_csv_table($total_params);
        
    } elseif ($data['type_of_report'] == 'total_grouped_by_clients'){

        foreach($data['clients'] as $client_name => $client_info){
            $csv .= str_replace(',',' ', lang('client').' '.$client_name) . "\n";
            $csv .= build_csv_project_data_for_invoice_report($client_info, $type_of_report);
            $csv .= "\n";
        }
    
        $total_params = array(
            'table_name' => lang('totals'),
            'total_budgeted' => $data['report_total_budgeted'],
            'total_invoiced' => $data['report_total_invoiced']
        );
        $csv .= build_invoicing_history_total_csv_table($total_params);

    } elseif ($data['type_of_report'] == 'total_without_grouping'){
        $csv .= build_csv_project_data_for_invoice_report($data['whole_system'], $type_of_report);
        $csv .= "\n";
        $total_params = array(
            'table_name' => lang('totals'),
            'total_budgeted' => $data['report_total_budgeted'],
            'total_invoiced' => $data['report_total_invoiced']
        );
        $csv .= build_invoicing_history_total_csv_table($total_params);
    }

    return $csv;
}

function build_csv_project_header_for_invoice_report($project_data, $header_params = array()){
    $report_data = array_var($_SESSION, 'invoice_report_data');
    $project_member = $project_data['project_member'];
    $type_of_report = array_var($header_params, 'type_of_report');
    $project_phases = $type_of_report == 'individual_project' ?  array_var($data, 'project_phases') : array();
    $main_header = '';
    if ($project_member instanceof Member) {
        $project_prefix_object = ObjectPrefixes::findById($project_member->getObjectId());
        if($project_prefix_object){
            $prefix_number = $project_prefix_object->getPrefix();
            $project_name = str_replace($prefix_number.' - ', '', $project_member->getName());
            $project_id = ' '.lang('number').' '.$prefix_number;
        } else {
            $pot = ObjectTypes::findByName('project');
            $project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
            $project_id = '';
            if ($project_id_cp instanceof CustomProperty) {
                $cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
                if ($cp_val instanceof CustomPropertyValue) {
                    $project_number_custom_property_name = CustomProperties::findById($cp_val->getCustomPropertyId())->getName();
                    $project_id = ' '.$project_number_custom_property_name.' '.$cp_val->getValue();
                }
            }
            $project_name = $project_member->getName();
        }
        $client_info = '';
        if (array_var($header_params, 'include_client_name')){
            $client_dimension = Dimensions::findByCode('customers');
            $client_ot = ObjectTypes::findByName('customer');
            $selected_ids = get_all_associated_status_member_ids($project_member, $client_dimension, $client_ot, false, 'project_customer');
            if(!empty($selected_ids)){
                $client_member = Members::getMemberById($selected_ids[0]);
                if($client_member instanceof Member){
                    $client_name = $client_member->getName();
                    // $client_object_type_name = Objects::findById($client_member->getObjectId())->getObjectTypeName();
                    $client_info = ' '.lang('client').' '. $client_name;
                }
            }
        }
        $indent_class = array_var($header_params, 'add_indentation') ? 'indent-medium' : '';
        $project_object_type_name = Objects::findById($project_member->getObjectId())->getObjectTypeName();
        $main_header = str_replace(',',' ', lang($project_object_type_name).' '.$project_name.$project_id.$client_info ) ."\n" ;
    } else  {
        return '';
    }
    
    return $main_header;
}

function build_csv_project_data_for_invoice_report($data, $type_of_report){
    $report_data = array_var($_SESSION, 'invoice_report_data');
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $use_budget_total_custom_property = array_var($report_data, 'use_budget_total_custom_property');
    $invoices = isset($data['invoice_data']['invoices']) ? $data['invoice_data']['invoices'] : array();
    $invoices_amount_per_phase = isset($data['invoice_data']['invoices_amount_per_phase']) ? $data['invoice_data']['invoices_amount_per_phase'] : array();
    $project_phases = $type_of_report == 'individual_project' ?  array_var($data, 'project_phases') : array();
    $currency_sym = config_option('currency_code');
    $decimals = user_config_option('decimal_digits');

    $csv = "";

    // Build Header
    $header = array('');
    if(!$group_by_month){
        $header[]= lang('invoice number');
        if($type_of_report == 'total_without_grouping') {
            $header[] = lang('client');
        }
        if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
            $header[] = lang('project');
        }
    }
    foreach($project_phases as $phase){
        $header[] = str_replace(',','', $phase);
    }
    $header[] = lang('total');
    $csv .= implode(',', $header) . "\n";

    // Build budgeted row
    $budget_row = array(lang('budgeted'));
    if(!$group_by_month){
        $budget_row[]= '';
        if($type_of_report == 'total_without_grouping') {
            $budget_row[]= '';
        }
        if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
            $budget_row[]= '';
        }
    }
    foreach($project_phases as $phase){
        if ($use_budget_total_custom_property) {
            $budget_amount = '';
        } else {
            $budget_amount = isset($data['budgeted_amounts_per_phase'][$phase]) ? $currency_sym.' '.number_format($data['budgeted_amounts_per_phase'][$phase],  $decimals, '.', '') : '0';
        }
        $budget_row[] = $budget_amount;
    }
    $budget_row[] = $currency_sym.' '.number_format($data['total_budgeted'],  $decimals, '.', '');
    $csv .= implode(',', $budget_row) . "\n";

    // Build invoices data
    if(count($invoices) > 0){
        if($group_by_month){
            foreach($invoices as $k => $v){
                $row = array($v['month']);
                foreach($project_phases as $phase){
                    if(isset($v[$phase])){
                        $row[] = $currency_sym.' '.number_format($v[$phase],  $decimals, '.', '');
                    } else {
                        $row[] = '';
                    }
                }
                $row[] = $currency_sym.' '.number_format($v['total'],  $decimals, '.', '');
                $csv .= implode(',', $row) . "\n";
            }
        } else {
            foreach($invoices as $invoice){
                $row = array($invoice['date']);
                $row[] = $invoice['invoice_number'];
                if ($type_of_report == 'total_without_grouping'){
                    $row[] = str_replace(',',' ', $invoice['client']);
                }
                if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
                    $row[] = str_replace(',', ' ', $invoice['project']);
                }
                foreach($project_phases as $phase){
                    if($invoice['project_phase'] == $phase){
                        $row[] = $currency_sym.' '.number_format($invoice['amount'],  $decimals, '.', '');
                    } else {
                        $row[] = '';
                        
                    }
                }
                $row[] = $currency_sym.' '.number_format($invoice['amount'],  $decimals, '.', '');
                $csv .= implode(',', $row) . "\n";
            }
        }
    }

    // Build 'Billed to date'
    $billed_to_date = array(lang('billed to date'));
    if(!$group_by_month){
        $billed_to_date[] = '';
        if ($type_of_report == 'total_without_grouping'){
            $billed_to_date[] = '';
        }
        if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
            $billed_to_date[] = '';
        }
    }
    foreach($project_phases as $phase){
        if(isset($invoices_amount_per_phase[$phase])){
            $billed_to_date[] = $currency_sym.' '.number_format($invoices_amount_per_phase[$phase],  $decimals, '.', '');
        } else {
            $billed_to_date[] = $currency_sym.' '.number_format(0,  $decimals, '.', '');
        }
    }
    $billed_to_date[] = $currency_sym.' '.number_format($data['invoice_data']['total_invoices'],  $decimals, '.', '');
    $csv .= implode(',', $billed_to_date) . "\n";

    // Build 'Remaining budget'
    $balance_vs_budget = array(lang('remaining budget'));
    if(!$group_by_month){
        $balance_vs_budget[] = '';
        if ($type_of_report == 'total_without_grouping'){
            $balance_vs_budget[] = '';
        }
        if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
            $balance_vs_budget[] = '';
        }
    }
    foreach($project_phases as $phase){
        if ($use_budget_total_custom_property) {
            $balance_vs_budget[] = '';
        } else {
            if(isset($data['difference_per_phase'][$phase])){
                $opening_parenthesis = $data['difference_per_phase'][$phase] < 0 ? '( ' : '';
                $closing_parenthesis = $data['difference_per_phase'][$phase] < 0 ? ' )' : '';
                $difference_per_phase = $data['difference_per_phase'][$phase] < 0 ? $data['difference_per_phase'][$phase]*(-1) :  $data['difference_per_phase'][$phase];
                $balance_vs_budget[] = $opening_parenthesis.$currency_sym.' '.number_format($difference_per_phase,  $decimals, '.', '').$closing_parenthesis;
            } else {
                $balance_vs_budget[] = $currency_sym.' '.number_format(0,  $decimals, '.', '');
            }
        }
    }
    $total_opening_parenthesis = $data['total_difference'] < 0 ? '( ' : '';
    $total_closing_parenthesis = $data['total_difference'] < 0 ? ' )' : '';
    $total_difference = $data['total_difference'] < 0 ? $data['total_difference']*(-1) : $data['total_difference'];
    $balance_vs_budget[] = $total_opening_parenthesis.$currency_sym.' '.number_format($total_difference,  $decimals, '.', '').$total_closing_parenthesis;
    $csv .= implode(',', $balance_vs_budget) . "\n";

    // Build 'Billed vs budgeted %'
    $billed_percentage = array(lang('billed vs budgeted %'));
    if(!$group_by_month){
        $billed_percentage[] = '';
        if ($type_of_report == 'total_without_grouping'){
            $billed_percentage[] = '';
        }
        if($type_of_report == 'total_grouped_by_clients' || $type_of_report == 'total_without_grouping'){
            $billed_percentage[] = '';
        }
    }
    foreach($project_phases as $phase){
        if ($use_budget_total_custom_property) {
            $billed_percentage[] = ''; 
        } else {
            if(isset($data['percentage_per_phase'][$phase])){
            $billed_percentage[] = $data['percentage_per_phase'][$phase].' %';
            } else {
            $billed_percentage[] = '';  
            }
        }
    }
    $billed_percentage[] = $data['total_percentage'].' %';
    $csv .= implode(',', $billed_percentage) . "\n";
    
    return $csv;
}

function build_invoicing_history_total_csv_table($params){
    $report_data = array_var($_SESSION, 'invoice_report_data');
    $table_name = array_var($params, 'table_name');
    $total_budgeted = array_var($params, 'total_budgeted');
    $total_invoiced = array_var($params, 'total_invoiced');
    $group_by_month = array_var($report_data, 'group_invoices_by_month') == 'checked' ? 1 : 0;
    $currency_sym = config_option('currency_code');
    $decimals = user_config_option('decimal_digits');
    $group_report_by = array_var($report_data, 'group_report_by');

    $colspan = $group_by_month ? 1 : 2;
    if(!$group_by_month){
        if($group_report_by == 3){
            $colspan += 1;
        } elseif ($group_report_by == 1) {
            $colspan += 2;
        }
    }

    $csv = "";
    $header = array(str_replace(',',' ',$table_name));
    for($i = 0; $i < $colspan; $i++){
        $header[] = '';
    }
    $csv .= implode(',', $header) . "\n";


    // Total Budgeted info
    $budgeted_data = array(lang('budgeted'));
    for($i = 0; $i < $colspan - 1; $i++){
        $budgeted_data[] = '';
    }
    $budgeted_data[] = $currency_sym.' '.number_format($total_budgeted,  $decimals, '.', '');
    $csv .= implode(',', $budgeted_data) . "\n";

    // Total 'Billed to date' info
    $invoiced_data = array(lang('billed to date'));
    for($i = 0; $i < $colspan - 1; $i++){
        $invoiced_data[] = '';
    }
    $invoiced_data[] = $currency_sym.' '.number_format($total_invoiced,  $decimals, '.', '');
    $csv .= implode(',', $invoiced_data) . "\n";

    // Total 'Remaining balance' info
    $total_difference = $total_budgeted - $total_invoiced;
    $total_opening_parenthesis = $total_difference < 0 ? '( ' : '';
    $total_closing_parenthesis = $total_difference < 0 ? ' )' : '';
    $total_difference = $total_difference < 0 ? $total_difference*(-1) : $total_difference;
    $remaining_data = array(lang('remaining budget'));
    for($i = 0; $i < $colspan - 1; $i++){
        $remaining_data[] = '';
    }
    $remaining_data[] = $total_opening_parenthesis.$currency_sym.' '.number_format($total_difference,  $decimals, '.', '').$total_closing_parenthesis;
    $csv .= implode(',', $remaining_data) . "\n";

    // Total 'Billed to date' info
    $total_percentage = $total_budgeted != 0 ? round(($total_invoiced / $total_budgeted)*100) : 0;
    $percentage_data = array(lang('billed vs budgeted %'));
    for($i = 0; $i < $colspan - 1; $i++){
        $percentage_data[] = '';
    }
    $percentage_data[] = number_format($total_percentage,  $decimals, '.', '').' %'; 
    $csv .= implode(',', $percentage_data) . "\n";

    return $csv;
} 
