<?php

function build_project_detail_jdt_report_excel($data) {

	$excel = array();

	// columns and report data
	$columns = array_var($data, 'columns');
	$report_data = array_var($data, 'report_data');

	$phases_data = $report_data['phases'];
	$totals = $report_data['totals'];
	$project_name = $report_data['project_name'];

	$excel[] = build_project_detail_jdt_report_title($project_name, $columns);

	// render data for each project
	foreach ($phases_data as $phase) {
		$excel = array_merge($excel, build_project_detail_jdt_excel_phase($phase, $columns));
	}
	
	// render totals table
    $excel[] = array('totals' => build_project_detail_jdt_excel_totals($report_data, $columns, 'totals', $project_name));

	return $excel;
}

function build_project_detail_jdt_excel_phase($phase_data, $columns) {
	$excel_tables = array();

	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	// main header = project name
	$excel_table['main_header'][] = $phase_data['group_name'];
	$i=1;
	while ($i < count($columns)) {
		$excel_table['main_header'][] = '';
		$i++;
	}

	// table headers
    foreach($columns as $c_name){
        $excel_table['headers'][] = $c_name == 'item' ? '' : lang($c_name);
    }

	$excel_tables[] = $excel_table;

	
	if (count($phase_data['labor']) > 0) {
		// all data from labor
		$data_rows = array();

		$excel_table = array(
			'main_header' => array(),
			'headers' => array(), 
			'data' => array(),
			'totals' => array(),
			'totals_after_data' => true,
		);

		$excel_table['headers'] = build_project_detail_jdt_excel_title_row(lang('labor'), $columns);

		// labor data
		foreach ($phase_data['labor'] as $tasks_data) {
	
			$data_rows[] = build_project_detail_jdt_excel_title_row($tasks_data['group_name'], $columns);
	
			foreach ($tasks_data['users'] as $users_data) {
	
				$data_rows[] = build_project_detail_jdt_excel_title_row("    " . $users_data['group_name'], $columns);
	
				foreach ($users_data['timeslots'] as $time_data) {
	
					$data_row = array();
					$name_prefix = "        ";
	
					$currency = Currencies::getCurrency($time_data['rate_currency_id']);
					$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';
	
					foreach ($columns as $col) {
						$cell_value = '';
						switch ($col) {
							case 'item':
								$cell_value = $name_prefix . $time_data['description'];
								break;
							case 'date':
								$dt_val = new DateTimeValue(strtotime($time_data['start_time']));
								$cell_value = format_date($dt_val);
								break;
							case 'hours':
								$cell_value = $time_data['hours'];
								break;
							case 'cost':
								$cell_value = format_money_amount($time_data['fixed_cost'], $currency_sym);
								break;
							case 'billing':
								$cell_value = format_money_amount($time_data['fixed_billing'], $currency_sym);
								break;
						}
						$data_row[] = $cell_value;
					}
	
					$data_rows[] = $data_row;
				}

				$data_rows[] = build_project_detail_jdt_excel_totals($users_data, $columns, null, null, "    ");
	
			}

			$data_rows[] = build_project_detail_jdt_excel_totals($tasks_data, $columns);
		}

		$excel_table['data'] = $data_rows;

		$excel_table['totals'] = build_project_detail_jdt_excel_totals($phase_data, $columns, 'labor_totals', lang('labor'));

		$excel_tables[] = $excel_table;
	}
	
	// expenses data
	if (count($phase_data['expenses']) > 0) {
		$data_rows[] = build_project_detail_jdt_excel_title_row(lang('all expenses'), $columns);

		// all data from labor
		$data_rows = array();

		$excel_table = array(
			'main_header' => array(),
			'headers' => array(), 
			'data' => array(),
			'totals' => array(),
			'totals_after_data' => true,
		);

		$excel_table['headers'] = build_project_detail_jdt_excel_title_row(lang('all expenses'), $columns);

		// labor data
		foreach ($phase_data['expenses'] as $exp_cat_data) {
	
			$data_rows[] = build_project_detail_jdt_excel_title_row($exp_cat_data['group_name'], $columns);
	
			foreach ($exp_cat_data['actual_expenses'] as $exp_data) {
	
				$data_row = array();
				$name_prefix = "        ";
	
				$currency = Currencies::getCurrency($time_data['currency_id']);
				$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

				foreach ($columns as $col) {
					$cell_value = '';
					switch ($col) {
						case 'item':
							$cell_value = $name_prefix . $exp_data['description'];
							break;
						case 'date':
							$dt_val = new DateTimeValue(strtotime($exp_data['date']));
							$cell_value = format_date($dt_val);
							break;
						case 'hours':
							break;
						case 'cost':
							$cell_value = format_money_amount($exp_data['total_cost'], $currency_sym);
							break;
						case 'billing':
							$cell_value = format_money_amount($exp_data['total_price'], $currency_sym);
							break;
					}
			
					$data_row[] = $cell_value;
				}
				$data_rows[] = $data_row;
			}

			$data_rows[] = build_project_detail_jdt_excel_totals($exp_cat_data, $columns);

		}

		$excel_table['data'] = $data_rows;

		$excel_table['totals'] = build_project_detail_jdt_excel_totals($phase_data, $columns, 'expense_totals', lang('all expenses'));
		
		$excel_tables[] = $excel_table;
	}

	// phase totals
	$excel_table['totals'] = build_project_detail_jdt_excel_totals($phase_data, $columns);

	return $excel_tables;
}

function build_project_detail_jdt_excel_title_row($group_name, $columns) {

	$row = array($group_name);
	$i=1;
	while ($i < count($columns)) {
		$row[] = '';
		$i++;
	}
	return $row;
}

function build_project_detail_jdt_report_title($project_name, $columns) {
	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	// main header = project name
	$excel_table['main_header'][] = $project_name;
	$i=1;
	while ($i < count($columns)) {
		$excel_table['main_header'][] = '';
		$i++;
	}
	return $excel_table;
}



function build_project_detail_jdt_excel_totals($group_data, $columns, $totals_key=null, $label=null, $prefix='') {
	
	if(!$totals_key) $totals_key = 'totals';

	$totals = $group_data[$totals_key];

	if (!$label) $label = $group_data['group_name'];

	$currency = Currencies::getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	$total_row = array();
	foreach ($columns as $col) {
		$cell_value = '';
		switch ($col) {
			case 'item':
				$cell_value = $prefix . lang('total')." ".$label;
				break;
			case 'date':
				break;
			case 'hours':
				$cell_value = $totals['hours'];
				break;
			case 'cost':
				$cell_value = format_money_amount($totals['cost'], $currency_sym);
				break;
			case 'billing':
				$cell_value = format_money_amount($totals['billing'], $currency_sym);
				break;
		}

		$total_row[] = $cell_value;
	}

	return $total_row;
}

