<?php


function get_project_earnings_report_data($report_data) {

	/**
	 * Project manager X
	 *   Project A1
	 *   Project B1
	 *   ...
	 * Total project manager X
	 * Project manager Y
	 *   Project A2
	 *   Project B2
	 *   ...
	 * Total project manager Y
	 */

	$project_ot = ObjectTypes::findByName('project');
    // Get custom property 'project_manager'
    $cp_project_manager = CustomProperties::findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    if(!($cp_project_manager instanceof CustomProperty)) return array();

    // Get project manager contact ids
    if(array_var($report_data, 'project_manager_id')) {
        $pm_contact_ids = array($report_data['project_manager_id']);
    } else {
        $pm_cp_values = CustomPropertyValues::findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId())));
        $pm_all_ids = array_map('get_cp_value', $pm_cp_values);
        $pm_contact_ids = array_unique($pm_all_ids);
    }

    $result = array(
        'contacts' => array(),
    );

	$grand_total = array();
	$multipliers = array();
    foreach($pm_contact_ids as $pm_contact_id){
        $pm_contact = Contacts::findById($pm_contact_id);
        if(!($pm_contact instanceof Contact)) continue;

        $pm_project_earnings = get_project_manager_project_earnings($pm_contact, $cp_project_manager, $report_data);
        if(!empty($pm_project_earnings['projects'])){

			$result['contacts'][$pm_contact_id] = $pm_project_earnings;

			$grand_total = sum_project_earnings_totals($pm_project_earnings['totals'], $grand_total, $multipliers);
        }
    }

	$grand_total['effective_multiplier'] = count($multipliers) ? array_sum($multipliers)/count($multipliers) : 0; // average effective multiplier

	$result['totals'] = $grand_total;



	$columns = array(
		'name', 'total_budget', 'revenue' ,'percent_complete','total_billed' ,'ar_amount',
		'unbilled','labor_cost','expense_cost','total_spent',
		'added_value','profit','super_profit_loss',
		'potential_provision',
		'effective_multiplier'
	);

	$data = array(
		'columns' => $columns,
		'report_data' => $result,
	);

	return $data;
}


function get_project_manager_project_earnings($pm_contact, $cp_project_manager, $report_data) {

	$member_project_ids = find_projects_for_earnings_reports($pm_contact, $cp_project_manager);
	if(empty($member_project_ids)) return array();

	$manager_totals = array(
		'revenue' => 0,
		'total_billed' => 0,
		'ar_amount' => 0,
		'unbilled' => 0,
		'labor_cost' => 0,
		'expense_cost' => 0,
		'total_spent' => 0,
		'added_value' => 0,
		'profit' => 0,
		'super_profit_loss' => 0,
		'potential_provision' => 0,
		'effective_multiplier' => 0,
		'net_revenue' => 0,
		'total_budget' => 0
	);

	$all_project_earnigns = array();
	$multipliers = array();
	foreach($member_project_ids as $member_id){

		$project_earnings = get_project_earnings($member_id, $report_data);
		if ($project_earnings) {
			$all_project_earnigns[] = $project_earnings;

			$manager_totals = sum_project_earnings_totals($project_earnings, $manager_totals, $multipliers);
		}
	}

	$manager_totals['effective_multiplier'] = count($multipliers) ? array_sum($multipliers)/count($multipliers) : 0; // average effective multiplier

	return array(
		'name' => $pm_contact->getObjectName(),
		'projects' => $all_project_earnigns,
		'totals' => $manager_totals,
	);

}

function sum_project_earnings_totals($data, $totals, &$multipliers) {

	foreach ($data as $k => $v) {
		if (in_array($k, array('name', 'percent_complete'))) continue;
		if ($k == 'effective_multiplier'){
			$multipliers[] = $v;
		} else {
			$totals[$k] += $v;
		}
	}

	return $totals;
}

function get_project_earnings($member_id, $report_data) { 
	$allow_without_budget_total = array_var($report_data, 'include_unset_total_budget') == 'checked' ? true : false;
	$member = Members::getMemberById($member_id);
	if (!$member) return;

	$contract_phase_dim = Dimensions::findByCode('project_phases');
	$contract_phase_mem_id = get_associated_status_member_id($member, $contract_phase_dim);
	$is_active_project = false;
	$contract_phase_mem = Members::getMemberById($contract_phase_mem_id);

	// Check budget total and config option => exclude the project if necessary
	$contract_amount = 0;
	$contract_amount_cp = CustomProperties::instance()->getCustomPropertyByCode($member->getObjectTypeId(), 'budget_total');
	if ($contract_amount_cp) {
		$cp_val = CustomPropertyValues::instance()->getCustomPropertyValue($member->getObjectId(), $contract_amount_cp->getId());
		if ($cp_val) $contract_amount = $cp_val->getValue();
	}
	if ($contract_amount <= 0 && !$allow_without_budget_total) {
		return false;
	}

	$tmp_mem_arr = array($member);
	// get full name
	build_member_list_text_to_show_in_trees($tmp_mem_arr);

	$data = array(
		'name' => $member->getName(),
		'revenue' => 0, // total earned value
		'percent_complete' => 0, // Revenue/ContractAmount
		'total_billed' => 0, //Total amount invoiced (excl. VAT)
		'ar_amount' => 0, // pending invoices including vat
		'unbilled' => 0, // time and expenses not billed yet
		'labor_cost' => 0,
		'expense_cost' => 0,
		'total_spent' => 0, // labor cost + expense cost ***
		'added_value' => 0, // revenue - expense cost ***
		'profit' => 0, // revenue - total spent ***
		'super_profit_loss' => 0, // cp from project ***
		'potential_provision' => 0, // total billed - revenue ***
		'effective_multiplier' => 0, // net revenue / labor cost
		'net_revenue' => 0, // revenue - expense cost
		'total_budget' => 0
	);
	
	// calculate revenue
	$revenue = get_project_earnings_revenue($member);
	$data['revenue'] = $revenue;

	// percent complete
	if ($contract_amount > 0) {
		$data['percent_complete'] = number_format(100 * $revenue / $contract_amount, 2);
	}

	// total billed excluding vat
	$data['total_billed'] = get_project_earnings_total_billed($member);

	// ar amount including vat
	$data['ar_amount'] = get_project_earnings_ar_amount($member);

	// unbilled amount
	$data['unbilled'] = get_project_earnings_unbilled_amount($member);

	// labor cost
	$data['labor_cost'] = get_project_earnings_labor_cost($member);

	// expense cost
	$data['expense_cost'] = get_project_earnings_expense_cost($member);

	// Total budget
	$data['total_budget'] = $contract_amount;

	$data['total_spent'] = $data['labor_cost'] + $data['expense_cost'];
	$data['added_value'] = $data['revenue'] - $data['expense_cost'];
	$data['profit'] = $data['revenue'] - $data['total_spent'];
	$data['potential_provision'] = $data['total_billed'] - $data['revenue'];
	$data['net_revenue'] = $data['revenue'] - $data['expense_cost'];
	$data['effective_multiplier'] = $data['labor_cost'] == 0 ? 0 : $data['net_revenue'] / $data['labor_cost'];

	return $data;
}

function get_project_earnings_labor_cost($member) {
	$amount = 0;

	$ts_row = DB::executeOne("
		SELECT SUM(ts.fixed_cost) as total
		FROM ".TABLE_PREFIX."timeslots ts
		INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
		INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
		WHERE o.trashed_by_id=0
		AND om.member_id=".$member->getId().";
	");

	if ($ts_row) {
		$amount += $ts_row['total'];
	}

	return $amount;
}

function get_project_earnings_expense_cost($member) {
	$amount = 0;

	$exp_row = DB::executeOne("
			SELECT SUM(pr.amount) as total
			FROM ".TABLE_PREFIX."payment_receipts pr
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND om.member_id=".$member->getId().";
		");
		if ($exp_row) {
			$amount += $exp_row['total'];
		}

	return $amount;
}

function get_project_earnings_unbilled_amount($member) {

	$amount = 0;

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(ts.fixed_billing) as total
			FROM ".TABLE_PREFIX."timeslots ts
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND ts.invoicing_status = 'pending'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['total'];

		if (Plugins::instance()->isActivePlugin('expenses2')) {

			$exp_row = DB::executeOne("
				SELECT SUM(pr.total_price) as total
				FROM ".TABLE_PREFIX."payment_receipts pr
				INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
				INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
				WHERE o.trashed_by_id=0
				AND pr.invoicing_status = 'pending'
				AND om.member_id=".$member->getId().";
			");
			if ($exp_row) {
				$amount += $exp_row['total'];
			}
		}
	}
	return $amount;
}


function get_project_earnings_ar_amount($member) {

	$amount = 0;

	$now_str = DateTimeValueLib::now()->format(DATE_MYSQL);

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(i.total) as total
			FROM ".TABLE_PREFIX."income_invoices i
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND i.status != 'canceled' AND i.status != 'confirmed'
			AND i.expiration_date < '".$now_str."'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['total'];
	}

	return $amount;
}

function get_project_earnings_total_billed($member) {

	$amount = 0;

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(i.subtotal) as subtotal
			FROM ".TABLE_PREFIX."income_invoices i
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND i.status != 'canceled'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['subtotal'];
	}

	return $amount;
}

function get_project_earnings_revenue($member) {

	$revenue = 0;

	$approved_member_id = config_option('time_bulk_action_approved_member_id');
	$approved_cond = "AND EXISTS (SELECT om2.object_id FROM ".TABLE_PREFIX."object_members om2 WHERE om2.member_id=$approved_member_id AND om2.object_id=o.id)";

	$ts_row = DB::executeOne("
		SELECT SUM(ts.fixed_billing) as total
		FROM ".TABLE_PREFIX."timeslots ts
		INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
		INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
		WHERE o.trashed_by_id=0
		AND ((ts.invoicing_status='pending'
		$approved_cond) OR ts.invoicing_status='invoiced')
		AND om.member_id=".$member->getId().";
	");

	if ($ts_row) {
		$revenue += $ts_row['total'];
	}

	if (Plugins::instance()->isActivePlugin('expenses2')) {

		$exp_row = DB::executeOne("
			SELECT SUM(pr.total_price) as total
			FROM ".TABLE_PREFIX."payment_receipts pr
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND ((pr.invoicing_status='pending'
			$approved_cond) OR pr.invoicing_status='invoiced')
			AND om.member_id=".$member->getId().";
		");
		if ($exp_row) {
			$revenue += $exp_row['total'];
		}
	}

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(i.subtotal) as total
			FROM ".TABLE_PREFIX."income_invoices i
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND i.status != 'canceled'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $revenue += $row['total'];
	}

	return $revenue;

}

function find_projects_for_earnings_reports($pm_contact, $cp_project_manager) {

	$active_context_project_member_ids = get_project_member_ids_for_project_manager_report(false);

	// Get project ids under project manager
	$project_cp_values = CustomPropertyValues::findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId().' AND `value`='.$pm_contact->getId())));

	if(empty($project_cp_values)) return array();

	$object_project_ids = array_map('get_cp_value_object_id', $project_cp_values);
	$pm_project_ids = array_map('get_object_member_id',  $object_project_ids);

	$pm_member_ids = array_intersect($active_context_project_member_ids, $pm_project_ids);

	return $pm_member_ids;
	
}
		