<?php

function get_main_project_forecast_report_data($report_data){
    $date_periods = get_forecast_report_date_periods($report_data);

    $project_member = null;
    $project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
    if (is_array($project_member_ids) && count($project_member_ids) > 0) {
        $project_member = Members::getMemberById($project_member_ids[0]);
    }
    if($project_member instanceof Member){
        $project_data = get_project_forecast_report_data($project_member, $report_data, $date_periods);
        $result = array(
            'projects_data' => array($project_data)
        );
    } else {
        $result = get_forecast_report_data($report_data, $date_periods);
    }

    // Sort projects
    $sort_project_by = array_var($report_data, 'sort_projects_by', 'name');
    if($sort_project_by == 'project_id'){
        $sort_columns = array_column($result['projects_data'], 'member_project_id');
        array_multisort($columns, SORT_ASC, $result['projects_data']);
    } else if($sort_project_by == 'name'){
        $sort_columns = array_column($result['projects_data'], 'member_name');
    }
    array_multisort($sort_columns, SORT_ASC, $result['projects_data']);

    $result['date_periods'] = $date_periods;

    return $result;
}

function get_forecast_report_data($report_data, $date_periods){
    $result = array('projects_data' => array());
    $grand_totals = array(
        'labor_totals' => array('periods' => array(), 'labor_grand_total' => 0),
        'expenses_totals' => array('periods' => array(), 'expenses_grand_total' => 0),
        'report_totals' => array('periods' => array(), 'report_grand_total' => 0)
    );
    foreach($date_periods as $period){
        $grand_totals['labor_totals']['periods'][$period] = 0;
        $grand_totals['expenses_totals']['periods'][$period] = 0;
        $grand_totals['report_totals']['periods'][$period] = 0;
    }

    // Get filtered project member ids and assemble data
    $project_member_ids = get_project_member_ids_forecast_report($report_data);
    foreach($project_member_ids as $pm_id){
        $project_member = Members::findById($pm_id);
        if($project_member instanceof Member){
            $p_data = get_project_forecast_report_data($project_member, $report_data, $date_periods);
            $result['projects_data'][] = $p_data;
            add_project_totals_to_grand_totals($p_data, $grand_totals, $report_data, $date_periods);
        }
    }
    $result['grand_totals'] = $grand_totals;

    return $result;
}

function add_project_totals_to_grand_totals($p_data, &$grand_totals, $report_data, $date_periods){
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        foreach($date_periods as $period){
            $grand_totals['labor_totals']['periods'][$period] += $p_data['labor_totals']['periods'][$period];
            $grand_totals['expenses_totals']['periods'][$period] += $p_data['expense_totals']['periods'][$period];
            $grand_totals['report_totals']['periods'][$period] += $p_data['project_totals']['periods'][$period];
        }
        $grand_totals['labor_totals']['labor_grand_total'] += $p_data['labor_totals']['labor_total'];
        $grand_totals['expenses_totals']['expenses_grand_total'] += $p_data['expense_totals']['expenses_total'];
        $grand_totals['report_totals']['report_grand_total'] += $p_data['project_totals']['project_total'];
    } elseif($use_labor_estimate_cost){
        foreach($date_periods as $period){
            $grand_totals['labor_totals']['periods'][$period] += $p_data['labor_totals']['periods'][$period];
        }
        $grand_totals['labor_totals']['labor_grand_total'] += $p_data['labor_totals']['labor_total'];
    } elseif($use_expense_estimate_cost){
        foreach($date_periods as $period){
            $grand_totals['expenses_totals']['periods'][$period] += $p_data['expense_totals']['periods'][$period];
        }
        $grand_totals['expenses_totals']['expenses_grand_total'] += $p_data['expense_totals']['expenses_total'];
    }
}

function get_project_forecast_report_data($project_member, $report_data, $date_periods){
    $labor_data = get_project_forecast_labor_data($project_member, $report_data, $date_periods);
    $labor_totals = get_project_forecast_labor_totals($labor_data, $report_data, $date_periods);
    $expenses_data = get_project_forecast_expense_data($project_member, $report_data, $date_periods);
    $expense_totals = get_project_forecast_expense_totals($expenses_data, $report_data, $date_periods);
    $project_totals = get_project_forecast_project_totals($labor_totals, $expense_totals, $report_data, $date_periods);

    // build the name shown in trees
    $member_name = $project_member->getName();
    $member_project_id = get_member_project_id($project_member);
    $tmp_main_member_array = array($project_member);
    build_member_list_text_to_show_in_trees($tmp_main_member_array);

    $project_id = $project_member->getId();
    $project_data = array(
        'member_name' => $member_name,
        'member_project_id' => $member_project_id,
        'name' => $project_member->getName(),
        'project_member_id' => $project_id,
        'labor_data' => $labor_data,
        'labor_totals' => $labor_totals,
        'expenses_data' => $expenses_data,
        'expense_totals' => $expense_totals,
        'project_totals' => $project_totals
    );

    return  $project_data;
}

function get_project_forecast_labor_data($project_member, $report_data, $date_periods){
    $project_tasks = get_forecast_member_project_tasks($project_member, $report_data);

    $labor_data = array();
    $use_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    foreach($project_tasks as $task){

        $start_date = new DateTimeValue($task['startDate']);
        $task_month = $start_date->format('Y').'-'.$start_date->format('m');
        $labor_category_name = lang('without labor category');
        $labor_category_id = 0;
        foreach($task['members_data'] as $m_data){
            if($m_data['object_type_name'] == 'hour_type'){
                $labor_category_name = $m_data['name'];
                $labor_category_id = $m_data['id'];
                break;
            }
        }
        if(!key_exists($labor_category_id, $labor_data)){
            $labor_data[$labor_category_id] = array('name' => $labor_category_name, 
                                                    'periods' => array(),
                                                    'category_total' => 0);
            foreach($date_periods as $period){
                $labor_data[$labor_category_id]['periods'][$period] = 0;
            }
        }
        if($use_estimate_cost){
            if($report_data['type_report'] == 'revenue'){
                $labor_data[$labor_category_id]['periods'][$task_month] += $task['estimated_price'];
            }else{
                $labor_data[$labor_category_id]['periods'][$task_month] += $task['estimated_cost'];
            }
        } else {
            $labor_data[$labor_category_id]['periods'][$task_month] += $task['timeEstimate']/60;
        }
    }
    // Set category totals
    foreach($labor_data as &$category_data){
        $category_total = array_sum($category_data['periods']);
        $category_data['category_total'] = $category_total;
    }
    return $labor_data;
}

function get_project_forecast_labor_totals($labor_data, $report_data, $date_periods){
     
    $labor_totals = array('periods' => array());
    $use_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    if($use_estimate_cost){
        foreach($date_periods as $period){
            $labor_totals['periods'][$period] = array_sum(array_column(array_column($labor_data, 'periods'), $period));
        }
        $labor_totals['labor_total'] = array_sum(array_column($labor_data, 'category_total'));
    }

    return $labor_totals;
}

function get_project_forecast_expense_data($project_member, $report_data, $date_periods){
    $expenses = get_forecast_member_expense_items($project_member, $report_data);
    $expenses_data = array();
    $use_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $date_format = user_config_option('date_format');
    foreach($expenses as $expense){
        $expense_date = DateTimeValueLib::dateFromFormatAndString($date_format, $expense['date']);
        $expense_month = $expense_date->format('Y').'-'.$expense_date->format('m');
        $product_type_name = $expense['name'] == '' ? lang('without product type') : $expense['name'];
        $product_type_id = $expense['product_type_id'];
        if(!key_exists($product_type_id, $expenses_data)){
            $expenses_data[$product_type_id] = array('name' => $product_type_name, 'periods' => array());
            foreach($date_periods as $period){
                $expenses_data[$product_type_id]['periods'][$period] = 0;
            }
        }
        if($use_estimate_cost){
            $expenses_data[$product_type_id]['periods'][$expense_month] += $expense['total_cost'];
        } else {
            $expenses_data[$product_type_id]['periods'][$expense_month] += $expense['quantity'];
        }
    }
    // Set product type totals
    foreach($expenses_data as &$product_type_data){
        $product_type_total = array_sum($product_type_data['periods']);
        $product_type_data['product_type_total'] = $product_type_total;
    }

    return $expenses_data;
}

function get_project_forecast_expense_totals($expenses_data, $report_data, $date_periods) {

    $expenses_totals = array('periods' => array());
    $use_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    if($use_estimate_cost){
        foreach($date_periods as $period){
            $expenses_totals['periods'][$period] = array_sum(array_column(array_column($expenses_data, 'periods'), $period));
        }
        $expenses_totals['expenses_total'] = array_sum(array_column($expenses_data, 'product_type_total'));
    }

    return $expenses_totals;
}

function get_project_forecast_project_totals($labor_totals, $expense_totals, $report_data, $date_periods){
    $project_totals = array('periods' => array(), 'project_total' => 0);

    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        foreach($date_periods as $period){
            $project_totals['periods'][$period] = $labor_totals['periods'][$period] + $expense_totals['periods'][$period];
        }
        $project_totals['project_total'] = $labor_totals['labor_total'] +  $expense_totals['expenses_total'];
    }

    return $project_totals;
}

function get_forecast_report_date_periods($report_data){
    
    // Get dates
    $start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');

    // Set user timezone
    $tz_offset = logged_user()->getUserTimezoneValue();
    $st = $start_date->advance($tz_offset, false);
    $ed = $end_date->advance($tz_offset, false);

    // Create DateTime values to use in DatePeriod
    $begin = new DateTime($st->format('Y-m-d'));
    $end = new DateTime($ed->format('Y-m-d'));

    // Get date periods
    $interval = new DateInterval('P1M');
    $daterange = new DatePeriod($begin, $interval ,$end);
    $periods = array();
    foreach($daterange as $date){
        $periods[] = $date->format('Y').'-'.$date->format('m');
    }

    // Add last month if the end date number is lower than start date number
    if($st->format('d') > $ed->format('d')) $periods[] = $ed->format('Y').'-'.$ed->format('m');
    
    return $periods;
}


function get_forecast_member_project_tasks(Member $project_member, $report_data = array(), $only_current_level = false) { 
	$member_ids = array($project_member->getId());
	$extra_conditions = '';
	if (isset($report_data['start_date'])) {
		$d = $report_data['start_date'];
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `start_date` >= ".DB::escape($d);
		}
	}

	if (isset($report_data['end_date'])) {
		$d = $report_data['end_date'];
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `start_date` <= ".DB::escape($d);
		}
	}
	
	if ($only_current_level) {
		$extra_conditions .= " AND om.is_optimization=0 ";
	}

	$res = ProjectTasks::instance()->listing(array(
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => true,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => false,
			"member_ids" => $member_ids,
	));
	$tasks = $res->objects;

    $tasks_data = array();
	foreach ($tasks as $task) {
		/* @var $task ProjectTask */
		
		$t_data = $task->getArrayInfo(true, true, false);
	
		$tasks_data[] = $t_data;
	}

    return $tasks_data;
}

function get_forecast_member_expense_items($project_member, $report_data, $only_current_level = false){
    $member_ids = array($project_member->getId());
	$extra_conditions = '';
	if (isset($report_data['start_date'])) {
		$d = $report_data['start_date'];
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `date` >= ".DB::escape($d);
		}
	}

	if (isset($report_data['end_date'])) {
		$d = $report_data['end_date'];
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `date` <= ".DB::escape($d);
		}
	}
	
	if ($only_current_level) {
		$extra_conditions .= " AND om.is_optimization=0 ";
	}

	$res = ExpenseItems::instance()->listing(array(
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => true,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => false,
			"member_ids" => $member_ids,
	));
	$ex_items = $res->objects;

    $expenses_data = array();
	foreach ($ex_items as $e_item) {
		/* @var $task ProjectTask */
		$expenses_data[] = $e_item->getArrayInfo();
	}

    return $expenses_data;
}

function get_project_member_ids_forecast_report($report_data){
    $project_ids_filtered_by_active_context = get_project_member_ids_filtered_by_active_context();
    $project_ids_filtered_by_date = get_forecast_project_member_ids_filtered_by_date($report_data);
    $project_member_ids = array_intersect($project_ids_filtered_by_active_context, $project_ids_filtered_by_date);
    return $project_member_ids;

}

function get_project_member_ids_filtered_by_active_context(){
	$project_ot = ObjectTypes::findByName('project');
	$project_dimension = Dimensions::findByCode('customer_project');
	$associated_member_ids = get_project_associated_member_ids_from_active_context($project_dimension->getId(), $project_ot->getId());
	
	$assoc_mem_ids_condition = '';
	if(count($associated_member_ids) > 0){
		$sql_array_assoc_mem_ids = '(' . join(',', $associated_member_ids) . ')';
		$assoc_mem_ids_condition = ' AND mpm.property_member_id IN '.$sql_array_assoc_mem_ids;
	}

	$SQL = "SELECT DISTINCT(m.id)
		FROM ".TABLE_PREFIX."members m INNER JOIN ".TABLE_PREFIX."member_property_members mpm ON m.id=mpm.member_id
		WHERE m.parent_member_id NOT IN (SELECT id FROM ".TABLE_PREFIX."members WHERE object_type_id=".$project_ot->getId().") AND object_type_id=".$project_ot->getId().$assoc_mem_ids_condition;

	$p_rows = DB::executeAll($SQL);
	$project_member_ids = array_map(function ($row){
							return $row['id'];
						}, $p_rows);

	return $project_member_ids;
}

function get_project_associated_member_ids_from_active_context($dimension_id, $object_type_id){
	$persons_dim = Dimensions::findByCode('feng_persons');
	if ($persons_dim instanceof Dimension) $persons_dim_id = $persons_dim->getId();
	else $persons_dim_id = 0;

	// get associations (exclude persons dimension)
	$associated_dimension_ids = array();
	$dimension_associations = DimensionMemberAssociations::findAll(array(
			"conditions" => array("(`dimension_id` = ? AND `associated_dimension_id` != ? AND `object_type_id` = ?)
					OR (`associated_dimension_id` = ? AND `dimension_id` != ? AND `associated_object_type_id` = ?)",
					$dimension_id, $persons_dim_id, $object_type_id, $dimension_id, $persons_dim_id, $object_type_id)
	));
	
	// dimension ids
	$enabled_dimensions = config_option("enabled_dimensions");
	foreach ($dimension_associations as $da) {
		/* @var $da DimensionMemberAssociation */
		if ($da->getDimensionId() == $dimension_id && in_array($da->getAssociatedDimensionMemberAssociationId(), $enabled_dimensions)) {
			$associated_dimension_ids[] = $da->getAssociatedDimensionMemberAssociationId();
		} else if (in_array($da->getDimensionId(), $enabled_dimensions)) {
			$associated_dimension_ids[] = $da->getDimensionId();
		}
	}

	// get current selected members and its children that area associated to this dimension
	$associated_member_ids = array();
	$context = active_context();
	if (is_array($context)) {
		foreach ($context as $sel) {
			if ($sel instanceof Member && in_array($sel->getDimensionId(), $associated_dimension_ids)) {
				$associated_member_ids[] = $sel->getId();
				$mem_ob_type_id = $sel->getObjectTypeId();
				$mem_children = $sel->getAllChildrenIdsByType($mem_ob_type_id);
				if(!empty($mem_children)){
					foreach($mem_children as $m_child){
						$associated_member_ids[] = $m_child;
					}
				}
			}
		}
	}

	return array_unique($associated_member_ids);
}




function get_forecast_project_member_ids_filtered_by_date($report_data){
	$project_ot = ObjectTypes::findByName('project');

	// Get start and end date SQLs
	$start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');
	$sql_st_date = $start_date->toMySQL();
	$sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to timeslots in the given date range
	$task_sql = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."project_tasks pt INNER JOIN ".TABLE_PREFIX."object_members om ON pt.object_id=om.object_id
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
			WHERE pt.start_date>='".$sql_st_date."' AND pt.start_date<='".$sql_end_date."'
			AND m.object_type_id=".$project_ot->getId().";";
	$task_rows = DB::executeAll($task_sql);
	$task_project_member_ids = array();
	if(!empty($task_rows)){
		$task_project_member_ids = array_map(function ($row){
											return $row['id'];
										}, $task_rows);
	}

	// Get project member ids assigned to actual expenses in the given date range									
	$expense_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."expense_items ei INNER JOIN ".TABLE_PREFIX."object_members om ON ei.object_id=om.object_id
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
				WHERE ei.date>='".$sql_st_date."' AND ei.date<='".$sql_end_date."'
				AND m.object_type_id=".$project_ot->getId().";";
	$expense_rows = DB::executeAll($expense_sql);
	$expense_project_member_ids = array();
	if(!empty($expense_rows)){
		$expense_project_member_ids = array_map(function ($row){
										return $row['id'];
									}, $expense_rows);
	}

	// Merge time and expense project member ids										
	$combined_project_member_ids = array_unique(array_merge($task_project_member_ids, $expense_project_member_ids));

	return $combined_project_member_ids;
}

function get_member_project_id($project_member){
    $project_ot_id = ObjectTypes::findByName('project')->getId();
    $proj_id_cp = CustomProperties::findOne(array('conditions' => '`object_type_id`='.$project_ot_id.' AND `code`="project_id"'));
    if($proj_id_cp instanceof CustomProperty){
        $project_object_id = $project_member->getObjectId();
        $custom_prop_id = $proj_id_cp->getId();
        $proj_id_cpv = CustomPropertyValues::findOne(array('conditions' => '`object_id`='.$project_object_id.' AND `custom_property_id`='.$custom_prop_id));
        if($proj_id_cpv instanceof CustomPropertyValue){
            return $proj_id_cpv->getValue();
        } else {
            return '';
        }
    } else {
        return '';
    }
}
