<?php


function get_utilization_report_data($report_data) {

	$labor_cat_dim = Dimensions::findByCode('hour_types');
	$contract_hours_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'contract_hours');
	$utilization_goal_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'billable_utilization_goal');

	$first_level_categories = Members::findAll(array("conditions" => array("dimension_id = ? AND depth = 1", $labor_cat_dim->getId())));

	$billable_labor_categories = find_billable_labor_categories_member_ids();
	$total_hours_type = array_var($report_data, 'total_hours_type', 'contract');
	
	// Dates config
	Env::useHelper('advanced_reports', 'advanced_reports');
	$report_dates = parse_report_param_date_type($report_data, true, 'date_type', 'start_value', 'end_value', 'utilization_report');
	$start_date = array_var($report_dates, 'st');
	$end_date = array_var($report_dates, 'et');

	// Get weekday working days
	if(array_var($report_data, 'working_days_option') == 'general_option'){
		$weekday_working_days = explode(',', config_option('working_days'));
	} else {
		$weekday_working_days = array(0, 1, 2, 3, 4);
	}

	// Get working days in period to calculate contract hours per user
	$working_days_in_period = get_working_days_from_period($start_date, $end_date, $weekday_working_days);


	$start_date_str = $start_date->toMySQL();
	$end_date_str = $end_date->toMySQL();


	// query the sum of worked hours grouped by user and the first level of labor categories
	$sql = "
		select t.contact_id, c.name as contact_name,
		(sum(t.worked_time) / 60) as period_worked_hours,
		m.id as category_id, m.name
		from ".TABLE_PREFIX."timeslots t
		inner join ".TABLE_PREFIX."object_members om on om.object_id=t.object_id
		inner join ".TABLE_PREFIX."members m on m.id=om.member_id
		inner join ".TABLE_PREFIX."objects c on c.id=t.contact_id
		
		where
		m.depth = 1 and m.dimension_id=".$labor_cat_dim->getId()."
		and t.start_time >= '$start_date_str' and t.end_time <= '$end_date_str'
		and (SELECT trashed_by_id FROM ".TABLE_PREFIX."objects oc WHERE oc.id = t.object_id) = 0
		
		group by t.contact_id, om.member_id;
	";

	$raw_rows = DB::executeAll($sql);

	$utilization_data = array();

	// group the data by contact to send to the view
	foreach ($raw_rows as $row) {

		$contact_id = $row['contact_id'];
		$row_cat_id = $row['category_id'];

		if (!isset($utilization_data[$contact_id])) {
			// create the structure for the contact

			$contract_hours_cpval = CustomPropertyValues::getCustomPropertyValue($contact_id, $contract_hours_cp->getId());
			$contract_hours_per_day = $contract_hours_cpval instanceof CustomPropertyValue ? $contract_hours_cpval->getValue() : 8;
			if($contract_hours_per_day == 0) $contract_hours_per_day = 8;
			$utilization_goal_cpval = CustomPropertyValues::getCustomPropertyValue($contact_id, $utilization_goal_cp->getId());
			$utilization_goal = $utilization_goal_cpval ? $utilization_goal_cpval->getValue() : 0;

			// calculate the contract hours using contract hours per day that assigned to user and working days in period
			$contract_hours = $working_days_in_period * $contract_hours_per_day;
			$contract_hours = round($contract_hours, 2);

			$utilization_data[$contact_id] = array(
				'contact_name' => $row['contact_name'],
				'total_hours' => 0,
				'contract_hours' => $contract_hours,
				'utilization_goal' => $utilization_goal,
				'billable_hours' => 0,
				'non_billable_hours' => 0,
			);
			foreach ($first_level_categories as $cat) {
				$utilization_data[$contact_id]['registered_hours_' . $cat->getId()] = 0;
			}
		}

		// set the labor category hours for this contact
		$utilization_data[$contact_id]['registered_hours_' . $row_cat_id] = number_format($row['period_worked_hours'], 2);
		// accumulate total hours
		$utilization_data[$contact_id]['total_hours'] += $row['period_worked_hours'];

		// define if these labor category hours are billable or not
		if (in_array($row_cat_id, $billable_labor_categories)) {
			$utilization_data[$contact_id]['billable_hours'] += $row['period_worked_hours'];
		} else {
			$utilization_data[$contact_id]['non_billable_hours'] += $row['period_worked_hours'];
		}

	}

	if(count($utilization_data) > 0){
		$contact_names = array_column($utilization_data, 'contact_name');
		array_multisort($contact_names, SORT_ASC, $utilization_data);
	}

	
	foreach ($utilization_data as $contact_id => &$contact_data) {
		
		// calculate the billable utilization
		$billable_utilization = 0;
		if($total_hours_type == 'contract'){
			if ($contact_data['contract_hours'] > 0) {
				$billable_utilization = number_format(100 * $contact_data['billable_hours'] / $contact_data['contract_hours'], 2);
			}
		} else {
			$total_hours = $contact_data['billable_hours'] + $contact_data['non_billable_hours'];
			if($total_hours > 0){
				$billable_utilization = number_format(100 * $contact_data['billable_hours'] / $total_hours, 2);
			}
		}
		$contact_data['billable_utilization'] = $billable_utilization;

		// calculate the variation
		$variation = $billable_utilization - $utilization_data[$contact_id]['utilization_goal'];
		$contact_data['variation'] = $variation;

		// format the other totals
		$contact_data['total_hours'] = number_format($contact_data['total_hours'], 2);
		$contact_data['billable_hours'] = number_format($contact_data['billable_hours'], 2);
		$contact_data['non_billable_hours'] = number_format($contact_data['non_billable_hours'], 2);
		$contact_data['variation'] =  number_format($contact_data['variation'], 2);
	}

	// build the array of the report columns
	$columns = array();
	$columns['contact_name'] = lang('name');
	if($total_hours_type == 'contract'){
		$columns['contract_hours'] = $contract_hours_cp->getName();
	}
	$columns['total_hours'] = lang('total');

	foreach ($first_level_categories as $labor_category) {
		$columns['registered_hours_'.$labor_category->getId()] = $labor_category->getName();
	}

	$columns['billable_hours'] = lang('billable');
	$columns['non_billable_hours'] = lang('non_billable');
	$columns['billable_utilization'] = lang('billable_utilization');
	$columns['utilization_goal'] = $utilization_goal_cp->getName();
	$columns['variation'] = lang('variation');

	$data = array(
		'columns' => $columns,
		'utilization_data' => $utilization_data,
		'from_date' => $start_date,
		'to_date' => $end_date,
	);

	return $data;
}

function find_billable_labor_categories_member_ids(){
	$billable_labor_categories = array();
	 
	$labor_cat_ot = ObjectTypes::findByName('hour_type');
	if($labor_cat_ot instanceof ObjectType){
		$labor_cat_ot_id = $labor_cat_ot->getId();
		$billable_labot_cat_cp = MemberCustomProperties::findOne(array('conditions' => '`code`="is_billable" AND `object_type_id`='.$labor_cat_ot_id));
		if($billable_labot_cat_cp instanceof MemberCustomProperty){
			$billable_labot_cat_cp_id = $billable_labot_cat_cp->getId();
			$all_lab_cat_members = Members::findAll(array('conditions' => '`object_type_id`='.$labor_cat_ot_id));
			$non_billable_mcpvs = MemberCustomPropertyValues::findAll(array('conditions' => '`custom_property_id`='.$billable_labot_cat_cp_id. ' AND `value`="-1"'));
			$non_billable_mcpv_mem_ids = array();
			foreach($non_billable_mcpvs as $mcpv){
				$non_billable_mcpv_mem_ids[] = $mcpv->getMemberId();
			}
			foreach($all_lab_cat_members as $member){
				if(!in_array($member->getId(), $non_billable_mcpv_mem_ids)) $billable_labor_categories[] = $member->getId(); 
			}
		}
	}

	if(empty($billable_labor_categories)){
		$billable_labor_categories = config_option('billable_labor_categories');
	}

	return $billable_labor_categories;
}

function get_working_days_from_period($start_date, $end_date, $weekday_working_days){
	$working_days_in_period = 0;

	$start = $start_date->advance(logged_user()->getUserTimezoneValue(), false);
	$end = $end_date->advance(logged_user()->getUserTimezoneValue(), false);

	for($i = $start; $i <= $end; $i->add('d', 1)){
		$weekday = $i->format('N') - 1;
		if(in_array($weekday, $weekday_working_days)){
			$working_days_in_period++;
		}
	}

	return $working_days_in_period;
}
