<?php

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo render_all_aging_invoices_report($data);

function render_all_aging_invoices_report($data){
    // Get custom property 'project_manager'
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $contacts = $data['contacts'];
    if ($cp_project_manager instanceof CustomProperty){
        foreach($contacts as $contact_id => $contact_info){
            echo render_contact_info($contact_info, $cp_project_manager);
        }
    }
    

    echo render_report_totals($data['totals'], $data['distribution']);
}

function render_contact_info($contact_info, $cp_project_manager){
    echo '<table class="aging-invoices-table">';

    // HEADER
    $header_columns = array(' ', 'invoice', 'due_date', 'outstanding amount due', '0-30', '31-60', '61-90', '91-120', 'over 120');
    echo '<thead>';
    echo '<tr class="table-name"><td colspan="9"><span style="color: #0a85d8; padding-right: 10px;">'.$cp_project_manager->getName().'</span> '.$contact_info['contact_name'].'</td></tr>';
    echo '</tr>';
    foreach($header_columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // BODY
    // Project Manager name
    echo '<tbody>';

    // Render projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){
        echo render_project_info($project_info);
    }

    // Render Contact Totals
    echo render_contact_totals($contact_info['contact_name'], $project_and_invoices_info['contact_totals']);


    echo '</tbody></table>';

}

function render_project_info($project_info){
    // Client and Project name row
    $project_name = $project_info['project_info']['full_project_name'];
    echo '<tr>';
	echo '<td class="bold" colspan="9">';

	// Show client name if exist
	if(!empty($project_info['client_name'])) {
		echo lang('customer') . ': ' . $project_info['client_name'] . '<br>';
	}
	
	echo lang('project') . ': ' . $project_name;
	echo '</td>';
	echo '</tr>';

	// Render invoices
    foreach($project_info['invoices'] as $invoice){
        echo render_invoice_info($invoice);
    }
    // Project total row
    echo render_project_total($project_info);
}

function render_invoice_info($invoice){
    $currency = Currencies::getCurrency($invoice['currency_id']);
    $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
    // Build invoice row
    echo '<tr><td></td>';
    echo '<td>'.$invoice['invoice_number'].'</td>';
    echo '<td>'.$invoice['date'].'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['total'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['first_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['second_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['third_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['fourth_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['greater_than_four_month'], $currency_sym).'</td>';
    echo '</tr>';
}

function render_project_total($project_info){
    $project_totals = $project_info['project_totals'];
    $totals = build_totals_using_multicurrencies($project_totals);

    // Build project totals row
    echo '<tr class="bold">';
    echo '<td colspan="3">'.lang('total for').' '.$project_info['project_info']['project_id'].'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';   
}

function render_contact_totals($contact_name, $contact_totals){
    $totals = build_totals_using_multicurrencies($contact_totals);
    // Build project totals row
    echo '<tr class="contact-totals">';
    echo '<td colspan="3">'.lang('total for').' '.$contact_name.'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';
}

function render_report_totals($totals, $distribution_totals){
    $formatted_totals = build_totals_using_multicurrencies($totals);
    $distrib_totals = build_distribution_totals_using_multicurrencies($distribution_totals);
    echo '<table class="aging-invoices-table">';

    // Header
    echo '<thead>';
    echo '<tr class="table-name--totals"><td colspan="9">'.lang('totals').'</td></tr>';
    echo '<tr><th colspan="3"></th>';
    $header_columns = array('outstanding amount due', '0-30', '31-60', '61-90', '91-120', 'over 120');
    foreach($header_columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // Body
    echo '<tbody>';

    // Build totals row
    echo '<tr>';
    echo '<td class="bold" colspan="3">'.lang('final totals').'</td>';
    echo '<td class="amount">'.$formatted_totals['total'].'</td>';
    echo '<td class="amount">'.$formatted_totals['first_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['second_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['third_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['greater_than_four_month'].'</td>';
    echo '</tr>';

      // Build distribution row
    if($distribution_totals['currencies']){
        echo '<tr>';
        echo '<td class="bold" colspan="3">'.lang('distribution').'</td>';
        echo '<td class="amount">'.$distrib_totals['total'].'</td>';
        echo '<td class="amount">'.$distrib_totals['first_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['second_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['third_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['fourth_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['greater_than_four_month'].'</td>';
        echo '</tr>';
        
    }


    echo '</table>';

}

function build_totals_using_multicurrencies($totals) {
     $total = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){
         $currency = Currencies::getCurrency($currency_id);
         $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
         $total .= '<div>' . format_money_amount($currency_totals['total'], $currency_sym) . '</div>';
         $first_month .= '<div>' . format_money_amount($currency_totals['first_month'], $currency_sym) . '</div>';
         $second_month .= '<div>' . format_money_amount($currency_totals['second_month'], $currency_sym) . '</div>';
         $third_month .= '<div>' . format_money_amount($currency_totals['third_month'], $currency_sym) . '</div>';
         $fourth_month .= '<div>' . format_money_amount($currency_totals['fourth_month'], $currency_sym) . '</div>';
         $greater_than_four_month .= '<div>' . format_money_amount($currency_totals['greater_than_four_month'], $currency_sym) . '</div>';
     }

     $result = array(
        'total' => $total,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}

function build_distribution_totals_using_multicurrencies($totals){
    $total = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){  
        $total .= '<div>' . $currency_totals['total'].' %' . '</div>';
        $first_month .= '<div>' . $currency_totals['first_month'].' %' . '</div>';
        $second_month .= '<div>' . $currency_totals['second_month'].' %' . '</div>';
        $third_month .= '<div>' . $currency_totals['third_month'].' %' . '</div>';
        $fourth_month .= '<div>' . $currency_totals['fourth_month'].' %' . '</div>';
        $greater_than_four_month .= '<div>' . $currency_totals['greater_than_four_month'].' %' . '</div>';
     }

     $result = array(
        'total' => $total,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}
