<?php 

Env::useHelper('view_functions', 'project_reports');

$projects_data = array_var($data, 'projects_data');
$project_member = $data['project_member'];
$totals = array_var($data, 'totals');
$project_phases = array_var($data, 'project_phases');
$report_data = array_var($_SESSION, 'budget_report_data');
$show_main_information_blocks = isset($report_data['show_main_project_info']);
$list_subprojects = array_var($report_data, 'list_supbrojects');

function render_labor_table($project_data, $project_phases) {
    if (!empty($project_data['labor_data'])){
        $labor_data = $project_data['labor_data'];
        $report_data = array_var($_SESSION, 'budget_report_data');
        $currency_sym = config_option('currency_code');

        $count_phases = count($project_phases);
        $header_colspan = $count_phases + 4;
        //echo '<thead class="table-separator"><tr><td></td></tr></thead>';
        echo '<thead class="table-name"><tr><th colspan='.$header_colspan.'>'.lang('labor').'</th></tr></thead>';
        // Build table header
        echo '<thead><tr><th></th>';
        foreach($project_phases as $phase){
            echo '<th class="phase-column">'.$phase.'</th>';
        }
        echo '<th>'.lang('total hours').'</th>';
        echo '<th>'.lang('rate').'</th>';
        echo '<th>'.lang('total').'</th></tr></thead>';

        // Build table body
        echo '<tbody>';
        foreach($labor_data['labor_names'] as $labor_id => $labor_name){
            $name = $labor_name;
            echo '<tr><td class="category-name">'.$name.'</td>';
            foreach($project_phases as $phase){     
                $phase_hours = isset($labor_data['project_phases'][$phase][$labor_id]) &&  $labor_data['project_phases'][$phase][$labor_id] > 0 ? number_format($labor_data['project_phases'][$phase][$labor_id], 2) : '';
                echo '<td class="report-number phase-column">'.$phase_hours.'</td>';
            }
            $total_per_labor_hours = number_format($labor_data['total_per_labor_hours'][$labor_id], 2);
            echo '<td class="report-number">'.$total_per_labor_hours.'</td>';
            echo '<td class="report-number">'.format_money_amount($labor_data['hourly_rates'][$labor_id], $currency_sym) .'</td>';
            echo '<td class="report-number report-number-total">'.format_money_amount($labor_data['total_per_labor_amount'][$labor_id], $currency_sym).'</td></tr>';
        }
    }
    if(!empty($project_data['labor_summary'])){
        $labor_summary = $project_data['labor_summary'];
        echo '<tr class="budget-report-summary">';
        echo '<td>'.lang('labor subtotal budget report').'</td>';
        foreach($project_phases as $phase){
            $phase_value = isset($labor_summary['project_phases_subtotals'][$phase]) ?  number_format($labor_summary['project_phases_subtotals'][$phase], 2) : number_format(0, 2);
            echo '<td class="report-number">'.$phase_value.'</td>';
        }
        echo '<td class="report-number">'.number_format($labor_summary['subtotal_hours'], 2).'</td>';
        echo '<td></td>';
        echo '<td class="report-number">'.format_money_amount($labor_summary['subtotal_amount'], $currency_sym).'</td></tr>';
    }
    echo '</tbody>';


}

function render_expenses_table($project_data, $project_phases){
    $report_data = array_var($_SESSION, 'budget_report_data');
    $group_expenses_by = isset($report_data['group_expenses_by']) ? $report_data['group_expenses_by'] : 0;

    if(!empty($project_data['expenses_data'])){
        $expenses_data = $project_data['expenses_data'];
        $currency_sym = config_option('currency_code');

        $count_phases = count($project_phases);
        $header_colspan = $count_phases + 4;
        echo '<thead class="table-separator table-separator--small"><tr><td></td></tr></thead>';
        echo '<thead class="table-name"><tr><th colspan='.$header_colspan.'>'.lang('expenses').'</th></tr></thead>';

        if($group_expenses_by == 0) { // Group by product type

            // Build expenses table header
            echo '<thead><tr><th></th>';
            foreach($project_phases as $phase){
                echo '<th class="phase-column">'.$phase.'</th>';
            }
            echo '<th>'.lang('quantity total').'</th>';
            echo '<th>'.lang('unit price').'</th>';
            echo '<th>'.lang('total').'</th></tr></thead>';

            // Build expenses table body
            echo '<tbody>';
            foreach($expenses_data['expenses_names'] as $expense_id => $expense_name){
                $name = $expense_name;
                echo '<tr><td class="category-name">'.$name.'</td>';
                foreach($project_phases as $phase){
                    $phase_quantity = isset($expenses_data['project_phases'][$phase][$expense_id]) ? $expenses_data['project_phases'][$phase][$expense_id] : '';
                    echo '<td class="report-number phase-column">'.$phase_quantity.'</td>';
                }
                echo '<td class="report-number">'.$expenses_data['expenses_total_quantity'][$expense_id].'</td>';
                echo '<td class="report-number">'.format_money_amount($expenses_data['unit_prices'][$expense_id], $currency_sym) .'</td>';
                echo '<td class="report-number report-number-total">'.format_money_amount($expenses_data['amount_per_expense'][$expense_id], $currency_sym).'</td></tr>';
            }
        } else if($group_expenses_by == 1){ // Group by expense categories
            
            // Build expenses table header
            echo '<thead><tr><th></th>';
            foreach($project_phases as $phase){
                echo '<th class="phase-column">'.$phase.'</th>';
            }
            echo '<th></th>';
            echo '<th></th>';
            echo '<th>'.lang('total').'</th></tr></thead>';

            // Build expenses table body
            echo '<tbody>';
            foreach($expenses_data['expenses_names'] as $expense_name){
                echo '<tr><td class="category-name">'.$expense_name.'</td>';
                foreach($project_phases as $phase){
                    $phase_amount = isset($expenses_data['project_phases'][$phase][$expense_name]) ? format_money_amount($expenses_data['project_phases'][$phase][$expense_name],$currency_sym) : '';
                    echo '<td class="report-number phase-column">'.$phase_amount.'</td>';
                }
                echo '<td class="report-number"></td>';
                echo '<td class="report-number"></td>';
                echo '<td class="report-number report-number-total">'.format_money_amount($expenses_data['amount_per_expense'][$expense_name], $currency_sym).'</td></tr>';
            }
        }

        // Build expenses table summary
        $footer_columns = $count_phases + 2;
        $expenses_summary = $project_data['expenses_summary']['subtotal_amount'];
        echo '<tr class="budget-report-summary">';
        echo '<td>'.lang('expenses subtotal').'</td>';
        for( $i = 0; $i < $footer_columns; $i++){
            echo '<td></td>';
        }
        echo '<td class="report-number">'.format_money_amount($expenses_summary, $currency_sym).'</td></tr>';
        echo '<thead class="table-separator"><tr><td></td></tr></thead>';
        echo '</tbody>';

    }
}

function render_project_totals($totals, $project_phases){
    $count_phases = count($project_phases);
    $colspan = 3 + $count_phases;
    $currency_sym = config_option('currency_code');

    $grand_total = $totals['labor_total'] + $totals['expenses_total'];

    $count_phases = count($project_phases);
    $header_colspan = $count_phases + 4;
    //echo '<thead class="table-separator"><tr><td></td></tr></thead>';
    echo '<thead class="table-name table-name--totals"><tr><th colspan='.$header_colspan.'>'.lang('totals').'</th></tr></thead>';
    echo '<tbody class="grand-total">';
    echo '<tr><td colspan="'.$colspan.'">'.lang('labor total').'</td><td class="report-number">'.format_money_amount($totals['labor_total'], $currency_sym).'</td></tr>';
    echo '<tr><td colspan="'.$colspan.'">'.lang('expenses total').'</td><td class="report-number">'.format_money_amount($totals['expenses_total'], $currency_sym).'</td></tr>';
    echo '<tr><td colspan="'.$colspan.'">'.strtoupper(lang('total')).'</td><td class="report-number">'.format_money_amount($grand_total, $currency_sym).'</td></tr>';
    echo '</tbody>';
    

}

$company = owner_company();
$company_picture_data = $company->getPictureFileContent('large');
if ($company_picture_data) {
	file_put_contents(ROOT . "/tmp/company_picture.png", $company_picture_data);
	$company_picture_url = ROOT_URL . "/tmp/company_picture.png?dummy=".gen_id();
}

echo '<div class="">';
if(isset($company_picture_url)){
    echo '<img style="height: 70px; width: auto; float: right;" src="'.$company_picture_url.'"></br>';
}


echo '<div style="font-size: 46px; font-weight: 600px; margin-top: 10px; display: inline-block; margin-bottom: 30px;"><p>'.$title.'</p></div>';

$count_phases = count($project_phases);
$colspan = 4 + $count_phases;

echo '<table id="budget-report-table">';
echo '<thead></thead>';
if ($show_main_information_blocks) {
    if ($project_member instanceof Member) {
        $properties_array = '';
        $params =  array(
                'member' => $project_member,
                'visible_by_default' => true,
                'return_as_array' => true,
        );
        Hook::fire('override_render_properties_for_view', $params, $properties_array);
        
        render_project_report_main_information($project_member, $properties_array, $colspan);
    }
} else {
    if ($project_member instanceof Member) {
        $project_prefix_object = ObjectPrefixes::findById($project_member->getObjectId());
        if($project_prefix_object){
            $prefix_number = $project_prefix_object->getPrefix();
            $project_name = str_replace($prefix_number.' - ', '', $project_member->getName());
            $project_id = '<span style="color: #0a85d8; text-transform: capitalize;">'.lang('number').' </span>'.$prefix_number;
        } else {
            $pot = ObjectTypes::findByName('project');
            $project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
            $project_id = '';
            if ($project_id_cp instanceof CustomProperty) {
                $cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
                if ($cp_val instanceof CustomPropertyValue) {
                    $project_number_custom_property_name = CustomProperties::findById($cp_val->getCustomPropertyId())->getName();
                    $project_id = '<span style="color: #0a85d8; text-transform: capitalize;">'.$project_number_custom_property_name.' </span>'.$cp_val->getValue();
                }
            }
            $project_name = $project_member->getName();
        }
        $project_object_type_name = Objects::findById($project_member->getObjectId())->getObjectTypeName();
        echo '<div class="main-project-name"><span style="color: #0a85d8; margin-right: 20px;">'.lang($project_object_type_name).'</span>'.$project_name.'<span style="font-size: 13px; padding-left: 15px; font-weight: bold;">'.$project_id.'</span></div>';
    }
}

foreach($projects_data as $project_data){
    if($list_subprojects && !($project_member->getObjectId() == $project_data['project_object_id'])){
        echo '<thead class="report-title"><tr><td colspan="'.$colspan.'"><span style="color: #999; margin-right: 15px;">'.lang($project_data['project_object_type_name']).'</span>'.$project_data['project_name'].'<span style="font-size: 13px; padding-left: 13px;">'.$project_data['project_id'].'</span></td></tr></thead>';
    }
    render_labor_table($project_data, $project_phases);
    render_expenses_table($project_data, $project_phases);
}
render_project_totals($totals, $project_phases);
echo '</table>';


$today = DateTimeValueLib::now();
$tz_offset = Timezones::getTimezoneOffsetToApply(Objects::findById($project_member->getObjectId()));
$today = $today->add('s', $tz_offset)->beginningOfDay();
$date_format = user_config_option('date_format');
$dateValue = $today->format($date_format);
echo '<p style="margin-top: 20px; text-align: right; color: #999; font-size: 12px;">'.$dateValue.'</date>';

echo '</div>';

