<?php
// date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
?>

<div class="project-report-title"><?php echo $title ?></div>
<div class="project-report-info" style="float: right;"><?php echo lang('report date').': '.$today_date?></div>
<div class="clear"></div>

<?php

render_project_detail_jdt_data($data);

function render_project_detail_jdt_data($data) {

	// columns and report data
	$columns = array_var($data, 'columns');
	$jdt_report_data = array_var($data, 'report_data');

	// project name
	$project_name = $jdt_report_data['project_name'];

	// render data for each project
	foreach ($jdt_report_data['phases'] as $phase_data) {
		render_project_detail_jdt_phase($phase_data, $columns, $project_name);
		$project_name = null;
	}
	
	// render totals table
    render_project_detail_jdt_report_totals($jdt_report_data, $columns);
}





function render_project_detail_jdt_phase($phase_data, $columns, $project_name) {
	$table_style = $project_name ? 'page-break-inside: auto;' : '';
	echo '<table class="project-jdt-report-table" style="'.$table_style.'">';

    // HEADER - phase name
    echo '<thead>';
	if ($project_name) {
		echo '<tr class="table-name main"><td colspan="'.(count($columns) + 1).'">'.$project_name.'</td></tr>';
	}
    echo '<tr class="table-name"><td colspan="'.(count($columns) + 1).'">'.$phase_data['group_name'].'</td></tr>';
	// column headers
    echo '<tr>';
    foreach($columns as $c_name){
		$colspan = 1;
		if ($c_name == 'item') $colspan = 2;
        echo '<th colspan="'.$colspan.'">'. ($c_name == 'item' ? '' : lang($c_name)) .'</th>';
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody>';

	// labor section
	if (count($phase_data['labor']) > 0) {
		render_project_detail_jdt_phase_labor($phase_data, $columns);
	}

	// expenses section
	if (count($phase_data['expenses']) > 0) {
		render_project_detail_jdt_phase_expenses($phase_data, $columns);
	}

    // Phase Totals
	render_project_detail_jdt_totals_row($phase_data, $columns, 'table-name');

    echo '</tbody></table>';
}



function render_project_detail_jdt_phase_labor($phase_data, $columns) {
	// row for labor title
	echo '<tr class="table-name-2"><td class="item-name" colspan="'.(count($columns) + 1).'">'.lang('labor').'</td></tr>';

	foreach ($phase_data['labor'] as $task_data) {
		
		render_project_detail_jdt_phase_labor_task($task_data, $columns);

	}

	// row for labor totals
	render_project_detail_jdt_totals_row($phase_data, $columns, 'table-name-2', true, 'labor_totals', lang('labor'));
}

function render_project_detail_jdt_phase_labor_task($task_data, $columns) {

	// row for task title
	echo '<tr class="table-name-3"><td class="item-name" colspan="'.(count($columns) + 1).'">'.$task_data['group_name'].'</td></tr>';

	foreach ($task_data['users'] as $user_data) {

		render_project_detail_jdt_phase_labor_task_user($user_data, $columns);

	}
	// row for task totals
	render_project_detail_jdt_totals_row($task_data, $columns, 'table-name-3', true);
}

function render_project_detail_jdt_phase_labor_task_user($user_data, $columns) {

	// row for user title
	echo '<tr class="table-name-4"><td class="item-name" colspan="'.(count($columns) + 1).'">'.$user_data['group_name'].'</td></tr>';

	foreach ($user_data['timeslots'] as $time_data) {

		$currency = Currencies::getCurrency($time_data['rate_currency_id']);
		$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

		echo '<tr>';
		foreach ($columns as $col) {
			$colspan = 1;
			$cell_class = '';
			$cell_value = '';
			switch ($col) {
				case 'item':
					$cell_class = 'item-name';
					$colspan = 2;
					$cell_value = $time_data['description'];
					break;
				case 'date':
					$dt_val = new DateTimeValue(strtotime($time_data['start_time']));
					$cell_class = 'date';
					$cell_value = format_date($dt_val);
					break;
				case 'hours':
					$cell_class = 'hours';
					$cell_value = $time_data['hours'];
					break;
				case 'cost':
					$cell_class = 'amount';
					$cell_value = format_money_amount($time_data['fixed_cost'], $currency_sym);
					break;
				case 'billing':
					$cell_class = 'amount';
					$cell_value = format_money_amount($time_data['fixed_billing'], $currency_sym);
					break;
			}
	
			echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
		}
		echo '</tr>';

	}
	
	// row for user totals
	render_project_detail_jdt_totals_row($user_data, $columns, 'table-name-4', true);
}




function render_project_detail_jdt_totals_row($group_data, $columns, $row_cls='', $show_hours=true, $totals_key='totals', $label=null) {

	$totals = $group_data[$totals_key];

	$currency = Currencies::getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	if (!$label) $label = $group_data['group_name'];

	echo '<tr class="total '.$row_cls.'">';
	foreach ($columns as $col) {
		$colspan = 1;
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'item':
				$cell_class = 'item-name';
				$colspan = 2;
				$cell_value = lang('total')." ".$label;
				break;
			case 'date':
				break;
			case 'hours':
				$cell_class = 'hours';
				if ($show_hours) $cell_value = $totals['hours'];
				break;
			case 'cost':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['cost'], $currency_sym);
				break;
			case 'billing':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['billing'], $currency_sym);
				break;
		}

		echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
	}
	echo '</tr>';

}




function render_project_detail_jdt_phase_expenses($phase_data, $columns) {

	// row for expenses title
	echo '<tr class="table-name-2"><td class="item-name" colspan="'.(count($columns) + 1).'">'.lang('all expenses').'</td></tr>';

	foreach ($phase_data['expenses'] as $exp_cat_data) {
		
		render_project_detail_jdt_phase_expenses_cat($exp_cat_data, $columns);

	}

	// row for expenses totals
	render_project_detail_jdt_totals_row($phase_data, $columns, 'table-name-2', false, 'expense_totals', lang('all expenses'));
}


function render_project_detail_jdt_phase_expenses_cat($exp_cat_data, $columns) {

	// row for expenses title
	echo '<tr class="table-name-3"><td class="item-name" colspan="'.(count($columns) + 1).'">'.$exp_cat_data['group_name'].'</td></tr>';

	foreach ($exp_cat_data['actual_expenses'] as $exp_data) {
		
		$currency = Currencies::getCurrency($exp_data['currency_id']);
		$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

		echo '<tr>';
		foreach ($columns as $col) {
			$colspan = 1;
			$cell_class = '';
			$cell_value = '';
			switch ($col) {
				case 'item':
					$cell_class = 'item-name';
					$colspan = 2;
					$cell_value = $exp_data['description'];
					break;
				case 'date':
					$dt_val = new DateTimeValue(strtotime($exp_data['date']));
					$cell_class = 'date';
					$cell_value = format_date($dt_val);
					break;
				case 'hours':
					$cell_class = 'hours';
					break;
				case 'cost':
					$cell_class = 'amount';
					$cell_value = format_money_amount($exp_data['total_cost'], $currency_sym);
					break;
				case 'billing':
					$cell_class = 'amount';
					$cell_value = format_money_amount($exp_data['total_price'], $currency_sym);
					break;
			}
	
			echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
		}
		echo '</tr>';

	}

	// row for expenses totals
	render_project_detail_jdt_totals_row($exp_cat_data, $columns, 'table-name-3', false);
}




function render_project_detail_jdt_report_totals($jdt_report_data, $columns) {

	$totals = $jdt_report_data['totals'];

	$currency = Currencies::getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	echo '<table class="project-jdt-report-table grand-total">';

    // HEADER - project name
    echo '<thead>';
    echo '<tr class="table-name main"><td colspan="'.(count($columns)+1).'">'.lang('total')." ".$jdt_report_data['project_name'].'</td></tr>';
	// column headers
    echo '<tr>';
	foreach($columns as $c_name){
		$colspan = 1;
		if ($c_name == 'date') continue;
		if ($c_name == 'item') $colspan = 3;
        echo '<th colspan="'.$colspan.'">'. ($c_name == 'item' ? '' : lang($c_name)) .'</th>';
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody>';

	echo '<tr class="total table-name-3">';
	foreach ($columns as $col) {
		$colspan = 1;
		$draw_col = true;
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'item':
				$cell_class = 'item-name';
				$colspan = 3;
				$cell_value = $jdt_report_data['project_name'];
				break;
			case 'date':
				$draw_col = false;
				break;
			case 'hours':
				$cell_class = 'hours';
				$cell_value = $totals['hours'];
				break;
			case 'cost':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['cost'], $currency_sym);
				break;
			case 'billing':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['billing'], $currency_sym);
				break;
		}

		if ($draw_col) echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
	}

    echo '</tr></tbody></table>';
}


