<?php
/**
 * Purchase Order controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class PurchaseOrderController extends ApplicationController {

	var $plugin_name = "purchase_orders";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("PurchaseOrderManager.js", $this->plugin_name);
		ajx_current("panel", "purchase-order-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		if ($order == 'name') $order = 'number';
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		}
		
		$result = PurchaseOrders::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => false,
			"start" => $start,
			"limit" => $limit,
		));
		$purchase_orders = $result->objects;
		$object = $this->prepareObject($purchase_orders, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($purchase_orders, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"purchase_orders" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(PurchaseOrders::instance()->getObjectTypeId());
		$count = min(array($limit, count($purchase_orders)));
		for ($i = 0; $i < $count; $i++){
			if (isset($purchase_orders[$i])){
				$obj = $purchase_orders[$i];
				if ($obj instanceof PurchaseOrder){
					$info = $obj->getArrayInfo();
					$info['name'] = str_pad($info['number'], 7, '0', STR_PAD_LEFT);
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["purchase_orders"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["purchase_orders"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!PurchaseOrder::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("purchase_orders"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$purchase_order_data = array_var($_POST, 'purchase_order');
		
		if(!is_array($purchase_order_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new PurchaseOrder();
			$obj->setObjectTypeId(PurchaseOrders::instance()->getObjectTypeId());
			tpl_assign('purchase_order', $obj);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$purchase_order = $this->save_purchase_order($purchase_order_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($purchase_order, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add purchase order', $purchase_order->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$purchase_order = PurchaseOrders::findById(get_id());
		if(!($purchase_order instanceof PurchaseOrder)) {
			flash_error(lang('purchase_order dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$purchase_order->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!PurchaseOrder::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("purchase_orders"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$purchase_order_data = array_var($_POST, 'purchase_order');
		
		if(!is_array($purchase_order_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('purchase_order', $purchase_order);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$purchase_order = $this->save_purchase_order($purchase_order_data, $member_ids, $purchase_order);
				
				DB::commit();

				ApplicationLogs::createLog($purchase_order, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit purchase_order', $purchase_order->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_purchase_order($purchase_order_data, $member_ids, $purchase_order = null) {
		// Save purchase_order object
		$is_new = false;
		if (is_null($purchase_order) || !$purchase_order instanceof PurchaseOrder) {
			$purchase_order = new PurchaseOrder();
			$is_new = true;
			$max_number = DB::executeAll("SELECT MAX(`number`) as `max_num` FROM ".TABLE_PREFIX."purchase_orders");
			$purchase_order_data['number'] = $max_number[0]['max_num'] + 1;
		}
		
		$purchase_order_data['date'] = getDateValue($purchase_order_data['date']);
		
		$purchase_order->setFromAttributes($purchase_order_data);
		$purchase_order->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($purchase_order, $member_ids);
		}
		$object_controller->add_subscribers($purchase_order);
		$object_controller->link_to_new_object($purchase_order);
		$object_controller->add_custom_properties($purchase_order);
		
		return $purchase_order;
	}
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$purchase_order = PurchaseOrders::findById(get_id());
		if(!($purchase_order instanceof PurchaseOrder)) {
			flash_error(lang('purchase_order dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$purchase_order->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('purchase_order', $purchase_order);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$purchase_order = PurchaseOrders::findById($id);
					if ($purchase_order instanceof PurchaseOrder && $purchase_order->canDelete(logged_user())){
						try{
							DB::beginWork();
							$purchase_order->trash();
							DB::commit();
							ApplicationLogs::createLog($purchase_order, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed purchase_order', $purchase_order instanceof PurchaseOrder ? clean($purchase_order->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$purchase_order = PurchaseOrders::findById($id);
					if ($purchase_order instanceof PurchaseOrder && $purchase_order->canEdit(logged_user())){
						try{
							DB::beginWork();
							$purchase_order->archive();
							DB::commit();
							ApplicationLogs::createLog($purchase_order, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}