<?php
	class PurchaseOrder extends BasePurchaseOrder {
		
		function getObjectName() {
			return $this->getDescription();
		}
		
		function getArrayInfo() {
			$info = array();
			
			$columns = self::getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			$info['supplier_name'] = $this->getSupplierName();
			$info['ordered_by_name'] = $this->getOrderedByName();
			
			return $info;
		}
		
		
		function getOrderedByName() {
			$ordered_by = Contacts::findById($this->getOrderedBy());
			if ($ordered_by instanceof Contact) {
				return clean($ordered_by->getObjectName());
			}
			return "";
		}
		
		function getSupplierName() {
			$supplier = Contacts::findById($this->getSupplierId());
			if ($supplier instanceof Contact) {
				return clean($supplier->getObjectName());
			}
			return "";
		}
		
		
		function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, PurchaseOrders::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}