
og.PurchaseOrderManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.is_loading = false;
	
	this.fields = ['object_id', 'name', 'description', 'number', 'date', 'ordered_by', 'supplier_id', 'ordered_by_name', 'supplier_name',
	               'net_amount', 'vat_amount', 'gross_amount', 'estimated_cost', 'invoiced', 'memPath'];
	
  	var cps = og.custom_properties_by_type['purchase_order'] ? og.custom_properties_by_type['purchase_order'] : [];
  	var cp_names = [];
  	for (i=0; i<cps.length; i++) {
  		cp_names.push('cp_' + cps[i].id);
  	}
  	this.fields = this.fields.concat(cp_names);
   	
   	var dim_names = [];
   	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		dim_names.push('dim_' + did);
	}
   	this.fields = this.fields.concat(dim_names);
  	
	
	if (!og.PurchaseOrderManager.store) {
		og.PurchaseOrderManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('purchase_order', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'purchase_orders',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					
					var d = this.reader.jsonData;
					this.fireEvent('after list load', "");
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("purchase_orders"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("purchase_orders")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					
					var man = Ext.getCmp("purchase-order-manager");
					
					var sm = man.getSelectionModel();
					sm.clearSelections();
					
					og.eventManager.fireEvent('after grid panel load', {man:man, data:d});
					
					if (d && d.totals) {
						man.updateGridPagingToolbar({totalCount: d.totals.total_rows});
					} else {
						man.reloadGridPagingToolbar('purchase_order','list_all','purchase-order-manager');
					}
					man.is_loading = false;
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.PurchaseOrderManager.store.setDefaultSort('number', 'desc');
	}
	this.store = og.PurchaseOrderManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderDragHandle(value, p, r, ix) {
		if (r.data.object_id == -1) {
			return '';
		}
		return '<div class="img-grid-drag" title="' + lang('click to drag') + '" onmousedown="var sm = Ext.getCmp(\'purchase-order-manager\').getSelectionModel();if (!sm.isSelected('+ix+')) sm.clearSelections();sm.selectRow('+ix+', true);"></div>';
	}
	
	
	var readClass = 'read-unread-' + Ext.id();
	
	function renderDate(value, p, r) {
		return og.clean(value);
	}

	function renderIcon(value, p, r) {
		return '<div class="link-ico ico-purchase-order"></div>';
	}
	
	function renderLink(value, p, r) {
		if (isNaN(r.data.object_id)) {
			return '<span class="bold" id="'+r.data.object_id+'">'+ (value ? og.clean(value) : '') +'</span>';
		}
		var link_url = og.getUrl('purchase_order', 'view', {id:r.data.object_id});
		return '<a class="internalLink" onclick="og.openLink(\''+link_url+'\');" href="#">'+r.data.number+'</a>';
	}
	
	function renderInvoicedCheckbox(value, p, r) {
		if (isNaN(r.data.object_id)) return;
		
		var checked = value == 1 ? 'checked="checked"' : '';
		return '<input type="checkbox" '+checked+' onclick="return false;" style="opacity:0.75;"/>';
	}
	
	function renderAmount(value, p, r) {
		return '<span class="amount">'+ value +'</span>';
	}
	
	
	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.object_id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			return selections[0].data.object_id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange', function() {
		if (sm.getCount() <= 0) {
			actions.edit.setDisabled(true);
			actions.del.setDisabled(true);
		} else {
			actions.edit.setDisabled(false);
			actions.del.setDisabled(false);
		}
	});
	
	var cm_info = [
		sm,{
			/*id: 'draghandle',
			header: '&nbsp;',
			dataIndex: 'draghandle',
			width: 18,
        	renderer: renderDragHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{*/
			id: 'icon',
			header: '&nbsp;',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("number"),
			dataIndex: 'name',
			width: 40,
			renderer: renderLink,
			sortable: true
        },{
			id: 'date',
			header: lang("date"),
			dataIndex: 'date',
			width: 60,
			align: 'center',
			renderer: og.clean,
			sortable: true
        },{
			id: 'ordered_by_name',
			header: lang("ordered_by"),
			dataIndex: 'ordered_by_name',
			width: 60,
			renderer: og.clean
        },{
			id: 'description',
			header: lang("description"),
			dataIndex: 'description',
			width: 120,
			renderer: og.clean,
			sortable: true
        },{
			id: 'net_amount',
			header: lang("net amount"),
			dataIndex: 'net_amount',
			width: 60,
			align: 'right',
			renderer: og.clean
        /*},{
			id: 'vat_amount',
			header: lang("vat amount"),
			dataIndex: 'vat_amount',
			width: 60,
			align: 'right',
			renderer: og.clean*/
        },{
			id: 'gross_amount',
			header: lang('gross amount'),
			dataIndex: 'gross_amount',
			width: 60,
			align: 'right',
			renderer: og.clean
        },{
			id: 'estimated_cost',
			header: lang("estimated cost"),
			dataIndex: 'estimated_cost',
			width: 60,
			align: 'right',
			renderer: og.clean
        },{
			id: 'invoiced',
			header: lang("invoiced"),
			dataIndex: 'invoiced',
			width: 60,
			align: 'center',
			renderer: renderInvoicedCheckbox
        }];
	
	    
    // custom property columns
	var cps = og.custom_properties_by_type['purchase_order'] ? og.custom_properties_by_type['purchase_order'] : [];
	this.addCustomPropertyColumns(cps, cm_info, 'purchase-order-manager');

	// dimension columns
	// Commented because if these columns are before the purchase_order columns, when rendering the payments table cells are in the wrong order
	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		var key = 'lp_dim_' + did + '_show_as_column';
		if (og.preferences['listing_preferences'][key]) {
			cm_info.push({
				id: 'dim_' + did,
				header: og.dimensions_info[did].name,
				dataIndex: 'dim_' + did,
				sortable: true,
				renderer: og.renderDimCol
			});
			og.breadcrumbs_skipped_dimensions[did] = did;
		}
	}
	
	// create column model
    var cm = new Ext.grid.ColumnModel(cm_info);
    cm.on('hiddenchange', this.afterColumnShowHide, this);
    cm.defaultSortable = false;
		
	actions = {
		newCO: new Ext.Action({
			id: 'new_button_purchase_order',
			text: lang('new'),
            tooltip: lang('add new purchase order'),
            iconCls: 'ico-new new_button',
            hidden: og.replace_list_new_action && og.replace_list_new_action.purchase_order,
            handler: function() {
				og.render_modal_form('', {c:'purchase_order', a:'add'});
			}
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				og.render_modal_form('', {c:'purchase_order', a:'edit', params: {id:getFirstSelectedId()}});
			},
			scope: this
		}),
		del: new Ext.Action({
			text: lang('move to trash'),
            tooltip: lang('move selected objects to trash'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				var confirm_trash_config = parseInt(og.preferences['enableTrashConfirmation']);
				
				if (og.confirmNorification(lang('confirm move to trash'), confirm_trash_config)) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		})
    };
	
	var uid = Ext.id();
	
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		if (og.replace_list_new_action && og.replace_list_new_action.purchase_order) {
			tbar.push(og.replace_list_new_action.purchase_order);
		}
		
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.del);
	}
	
	og.PurchaseOrderManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		enableDrag: true,
		ddGroup: 'MemberDD',
		stateful: og.preferences['rememberGUIState'],
		id: 'purchase-order-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: true
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.PurchaseOrderManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}
		
		this.store.baseParams = {
			context: og.contextManager.plainContext()
		};
		
		this.updateColumnModelHiddenColumns();
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.reset();
		}
	},
	
	reset: function() {
		var params = {start:0};
		
		this.load(params);
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	reloadFiltering: function() {
		if (this.isLoading()) return;
		this.is_loading = true;
		
		var config = {
			start:0
		};
		
		this.load(config);
	},
	
	isLoading: function() {
		return this.is_loading;
	},
	
	trashObjects: function() {
		if (confirm(lang('confirm move to trash'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	},
	
	archiveObjects: function() {
		if (confirm(lang('confirm archive selected objects'))) {
			this.load({
				action: 'archive',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
});


Ext.reg("purchase-orders", og.PurchaseOrderManager);

/************************************************
Container for PurchaseOrderManager,
*************************************************/
og.PurchaseOrderManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.PurchaseOrderManager();
	
	og.PurchaseOrderManagerPanel.superclass.constructor.call(this, {
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.manager
		],
		closable: true
	});
}

Ext.extend(og.PurchaseOrderManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	}
});

Ext.reg("purchase-order-containerpanel", og.PurchaseOrderManagerPanel);

/*
Ext.grid.GridView.override({
	getRowClass: function (record, index, rowParams, store) {
		if (record.data.row_color) {
			return "purchase-order-row-"+record.data.row_color;
		}else{
			return "";
		}
	}
});
*/