<?php

use org\apache\maven\POM\_4_0_0\project;
use TSheets\TSheetsRestClient;
use TSheets\ObjectType;

define('SHORT_CODE_SERVICE_ITEM','si');
define('SHORT_CODE_CUSTOMER_TYPE','ctype');
define('SHORT_CODE_BILLABLE','b');
define('TOKEN_TIME_SHEETS', config_option('time_sheets_token')); // S.17__b8288fdbb98c3ac6f450b554b6d1b0795dadfbcc
$tsheets_client = null;

function getInstanteTS(){
    global $tsheets_client;

    if(!$tsheets_client){
        $tsheets_client = new TSheetsRestClient(1,TOKEN_TIME_SHEETS);
    }

    return $tsheets_client;
}

function get_id_custom_field_by_short_code($short_code){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::CustomFields);
    $response = $items['results']['customfields'];

    foreach($response as $item){
        if($item['short_code'] == $short_code){
            return $item['id'];
        }
    }
}

function getIdItemByName($name){
    $tsheets_client = getInstanteTS();
    $service_item_id = get_id_custom_field_by_short_code(SHORT_CODE_SERVICE_ITEM);

    if(!$service_item_id){
        $error = 'Not found custom field';
        // Logger::log_r("Error getting in getIdCustomFieldByShortCode");
        throw new Exception(build_qbo_error_message($error));
    }

    $items = $tsheets_client->get(ObjectType::CustomFieldItems, ['customfield_id'=>$service_item_id]);
    $response = $items['results']['customfielditems'];
    foreach($response as $item){
        if($item['name'] == $name){
            return $item['id'];
        }
    }
}

// for development purposes
// function get_ts_employees_by_name(){
//     $tsheets_client = getInstanteTS();
//     $items = $tsheets_client->get(ObjectType::Users,['usernames'=>'rtaylor@quinnesi.com']);
//     $response = reset($items['results']);
//     Logger::log_r($response);
// }

/*
    Currently the API does not allow to bring the user information by email
*/
function get_ts_employee_by_name_and_lastname($first_name, $last_name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::Users,['first_name'=>$first_name,'last_name'=>$last_name]);
    $response = reset($items['results']['users']);
    return $response;
}

function get_ts_employee_id_by_name($name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::Users,['usernames'=>$name]);
    $response = reset($items['results']['users']);

    return $response['id'];
}

function get_ts_employee_information_by_name($name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::Users,['usernames'=>$name]);
    $response = reset($items['results']['users']);
    
    return $response;
}

function get_ts_customer_id_by_name($name){
    if(empty($name)){ return 0;}

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::Jobcodes,['name'=>$name]);
    $response = reset($items['results']['jobcodes']);
    return $response['id'] ?? 0;
}

function save_ts_custom_field($name,$short_code,$type){
    $request[] = [
        "name" => $name,
        "required" => false,
        "active" => true,
        "applies_to" => "timesheet",
        "type" => $type,
        "show_to_all" => true,
        "short_code" => $short_code
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->add(ObjectType::CustomFields, $request);

    if(reset($items['results']['customfields'])['_status_code'] == 200){
        return reset($items['results']['customfields']);     
    }else{
        $error = 'Custom field not saved';
        // Logger::log_r("Error to save custom field");
        // Logger::log_r($items);
        throw new Exception(build_qbo_error_message($error));
    }   
}

function save_ts_custom_field_item($customfield_id, $name){
    $request[] = [
        "name" => $name,
        "customfield_id" => $customfield_id
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->add(ObjectType::CustomFieldItems, $request);

    if(reset($items['results']['customfielditems'])['_status_code'] == 200){
        return reset($items['results']['customfielditems']);     
    }else{
        $error = 'Custom field Item id not found';
        // Logger::log_r("Error in save  custom field item");
        throw new Exception(build_qbo_error_message($error));
    }
}

function get_ts_custom_field_item($customfield_id, $name){
    $request = [
        "customfield_id" => $customfield_id,
        "name" => $name
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(ObjectType::CustomFieldItems, $request);

    return reset($items['results']['customfielditems']) ?? [];
}

// function save_ts_project($name){
//     $item_information = get_ts_custom_field_item(1555648, $name);

//     if(!isset($item_information['name'])){
//         $item_information = save_ts_custom_field_item(1555648, $name);
//     }

//     return $item_information;
// }

function apply_permission_to_user($ts_user_id){
    $request[] = [
        "id" => $ts_user_id,
        "permissions" => [
            "time_tracking" => 1
        ]
    ];

    
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->put(ObjectType::Users, $request);
    if(reset($items['results']['users'])['_status_code'] == 200){
        return reset($items['results']['users']);     
    }

    return false;
}

function ts_get_labor_category($object) {
	$categories_dim = Dimensions::findByCode('hour_types');
	if (!$categories_dim) return;
	$cat_member = null;

	// get the labor category member of the object
	$members = $object->getMembers();
	foreach ($members as $m) {
		if ($m->getDimensionId() == $categories_dim->getId()) {
			$cat_member = $m;
			break;
		}
	}
    
    return $cat_member;
}

function save_timeslot_to_ts(Timeslot $timeslot){    
    $service_item_id = get_id_custom_field_by_short_code(SHORT_CODE_SERVICE_ITEM);
    $customer_type_id = get_id_custom_field_by_short_code(SHORT_CODE_CUSTOMER_TYPE);
    $billable_id = get_id_custom_field_by_short_code(SHORT_CODE_BILLABLE);
    $tsheets_client = getInstanteTS();
    $dataService = get_qbo_data_service();
    $ts_time_activity_id = $timeslot->getColumnValue('tsheets_online_id');
    $sync_error = '';

    $labor_information = ts_get_labor_category($timeslot);
    if($labor_information){
        // get the time entry item depending on the labor category
        $hours_item_ref = get_object_qbo_labor_item($timeslot);
        $name_item = $hours_item_ref->FullyQualifiedName;
        // $item_id = getIdItemByName($name_item);
        if(!$name_item){
            $sync_error = 'Not found item name';
            // Logger::log_r("Error getting in save timelot");
            // throw new Exception(build_qbo_error_message($error));
        }
    }else{
        $name_item = 'Hours';
    }

    $qbo_employee_obj = sync_contact_with_qbo($timeslot->getContactId());
    if(!isset($qbo_employee_obj->PrimaryEmailAddr->Address)){
        $sync_error = 'Employee not synced with quickbooks';
        throw new Exception($sync_error);
    }

    $ts_employee_data = get_ts_employee_by_name_and_lastname($qbo_employee_obj->GivenName, $qbo_employee_obj->FamilyName);
    $ts_employee_id = $ts_employee_data['id'];
    if(!empty($ts_employee_id)){
        if(empty($ts_employee_data['permissions']['time_tracking'])){
            $result_permission = apply_permission_to_user($ts_employee_id);
            if($result_permission){
                $sync_error = 'Problems to apply permission to employee';
            }
        }
    }else{
        $sync_error = 'Employee id not found';
    }

    $client_ot = ObjectTypes::findByName('customer');
	$project_ot = ObjectTypes::findByName('project');

    // find client and project members
	$feng_timeslot_members = $timeslot->getMembers();
	$client_member = null;
	$project_member = null;
    $job_id = null;
    $job_type = '';

	foreach ($feng_timeslot_members as $member) {

		if ($member->getObjectTypeId() == $client_ot->getId()) {
			$client_member = $member;
		}
		if ($member->getObjectTypeId() == $project_ot->getId()) {
			$project_member = $member;
		}
	}
    
    // get qbo customer for the evx client
	if($client_member){
		$customerRef = get_qbo_customer_object($dataService, $client_member);
        if(!empty($customerRef->DisplayName)){
            $custumer_id = get_ts_customer_id_by_name($customerRef->DisplayName);
            $job_id = $custumer_id;
            $job_type = 'Customer';

            if(empty($custumer_id)){
                $sync_error = 'Customer not found';
            }
        }
	}

	// get qbo job/project for the evx project
	if ($customerRef && $project_member instanceof Member) {
		$project_data = get_qbo_project_object($dataService, $project_member, $customerRef);

        if(!empty($project_data->DisplayName)){
            $project_id = get_ts_customer_id_by_name($project_data->DisplayName);
            $job_id = $project_id;
            $job_type = 'Project';

            if(empty($project_id)){
                $sync_error = 'Project not found';
                // Logger::log_r("Error getting project: ". $project_member->getName());
                // Logger::log_r($project_member);
                // throw new Exception(build_qbo_error_message($error)); 
            }
        }
	}

    if(!empty($sync_error)){
        // Logger::log_r("Error getting project: ". $project_member->getName());
        // Logger::log_r($project_member);
        $sync_error = "Error only when synchronizing: ". $sync_error; 
        throw new Exception($sync_error);
    }

    $type_entry = config_option('have_time_sheets') ? 'MANUAL' : 'REGULAR';

    if($type_entry == 'MANUAL'){
        $tz_offset = $timeslot->getTimezoneValue();
        $start_time = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
        $start_time->add('s', $tz_offset);
        $start_time_str = $start_time->toMySQL();
        $start_time_str = str_replace(' ', 'T', $start_time_str);
        $start_time_str = substr($start_time_str,0,10);
        $time_seconds = $timeslot->getColumnValue("worked_time") * 60;
    }else{
        $tz_offset = $timeslot->getTimezoneValue();
        $start_time = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
        $start_time->add('s', $tz_offset);
        $start_time_str = $start_time->toMySQL();
        $start_time_str = str_replace(' ', 'T', $start_time_str);
        $start_time_str = $start_time_str.'-00:00';
        $end_time = new DateTimeValue($timeslot->getEndTime()->getTimestamp());
        $end_time->add('s', $tz_offset);
        $end_time_str = $end_time->toMySQL();
        $end_time_str = str_replace(' ', 'T', $end_time_str);
        $end_time_str = $end_time_str.'-00:00';
    }

    $is_billable = "Yes";
    if($timeslot->getColumnValue("invoicing_status") == "non_billable"){
        $is_billable = 'No';
    }

    if(!empty($job_id) && !empty($customer_type_id)){
        $request_data = array(
            "user_id"=> $ts_employee_id,
            "duration"=> $time_seconds,
            "jobcode_id"=> $job_id,
            "notes"=> $timeslot->getDescription(),
            "customfields"=> [
                $billable_id => $is_billable,
                $service_item_id => $name_item,
                $customer_type_id => $job_type
            ]
        );

        if($type_entry == 'REGULAR'){
            $request_data['type'] = 'regular';
            $request_data['start'] = $start_time_str;
            $request_data['end'] = $end_time_str;
        }else{
            $request_data['type'] = 'manual';
            $request_data['date'] = $start_time_str;
        }

        // In process save
        if ($ts_time_activity_id == 0) {
            $request[] = $request_data;

            $result = $tsheets_client->add(ObjectType::Timesheets, $request);
            $result_save = reset($result['results']['timesheets']);

            // Logger::log_r($result_save);

            if($result_save['_status_code'] == 200){
                $timeslot->setColumnValue('tsheets_online_id', $result_save['id']);
                $timeslot->save();

                return true;
            }else{
                return false;
            }
        }else{
            $request_data["id"] = $ts_time_activity_id;
            $request[] = $request_data;                        
            $result_update = $tsheets_client->put(ObjectType::Timesheets, $request);
            
            if($result_update['_status_code'] == 406){
                return true;
            }else{
                return false;
            }
        }

        return true;
    }else{
        $error = 'Time lot no saved';
        // Logger::log_r("Error in save timelot");
        throw new Exception(build_qbo_error_message($error));    
    }
}

function delete_ts_timeslot(Timeslot $timeslot){
	$tsheets_client = getInstanteTS();

	$ts_timeslot_id = $timeslot->getColumnValue("tsheets_online_id");
    
    if($ts_timeslot_id){
        $result = $tsheets_client->delete(ObjectType::Timesheets, [$ts_timeslot_id]);
        $result_obj = reset($result['results']['timesheets']);
        
        $timeslot->setColumnValue("tsheets_online_id", 0);
        $timeslot->save();

        return $result_obj;
    }
}