<?php
Hook::register('quickbooks');

function quickbooks_on_plugin_deactivate($params, &$ignored) {

	if (array_var($params, 'plugin') == 'quickbooks') {
		Logger::log_r("ACA 2");
		$actual_expenses_ot = ObjectTypes::findByName('payment_receipt');
		if ($actual_expenses_ot instanceof ObjectType) {
			// disable mandatory custom properties added by this plugin
			$cp = CustomProperties::getCustomPropertyByCode($actual_expenses_ot->getId(), 'quickbooks_payment_method');
			if ($cp instanceof CustomProperty) {
				$cp->setIsDisabled(true);
				$cp->save();
			}
			$cp = CustomProperties::getCustomPropertyByCode($actual_expenses_ot->getId(), 'quickbooks_payment_account');
			if ($cp instanceof CustomProperty) {
				$cp->setIsDisabled(true);
				$cp->save();
			}
		}
	}
}

/**
 * Include the file that contains the cron event to process pending entites
 */
function quickbooks_additional_cron_function_files($ignored, &$files) {
	$files[] = ROOT . "/plugins/quickbooks/application/cron_functions.php";
}


/**
 * Column in invoices list to show the qbo sync status
 */
function quickbooks_invoices_list_additional_column_values($parameters, &$info) {
	$invoice = array_var($parameters, 'invoice');
	if ($invoice instanceof IncomeInvoice) {
		$info['qbo_sync_status'] = $invoice->getColumnValue('quickbooks_online_id') > 0;
	}
}

function quickbooks_general_object_array_info_additional_columns($parameters, &$info) {
	$object = array_var($parameters, 'object');
	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		if ($object instanceof FengObject && Plugins::instance()->isActivePlugin('invoice_payment_receipts' && $object->getObjectTypeId() == InvoicePaymentReceipts::instance()->getObjectTypeId())) {
			$pr = InvoicePaymentReceipts::instance()->findById($object->getId());
			$info['qbo_sync_status'] = $pr ? $pr->getColumnValue('quickbooks_online_id') > 0 : false;
		}
	}
}

/**
 * Connect or disconect from QBO section
 */
function quickbooks_config_category_before_options($params, &$ignored) {
	$category = array_var($params, 'category');
	
	if ($category->getName() == 'quickbooks_online') {
		
		Env::useLibrary('qbo_sdk', 'quickbooks');
		Env::useHelper('qbo_functions', 'quickbooks');
		
		try {
			$access_token = get_qbo_access_token(); 
			$is_valid_token = $access_token && is_valid_qbo_access_token($access_token);
		} catch (QuickBooksOnline\API\Exception\SdkException $e) {
			Logger::log_r($e->getMessage());
		}
		
		if ($access_token && $is_valid_token) {
			$data_service = get_qbo_data_service();
			$company_data = $data_service->getCompanyInfo();
			?>
			<div class="configCategoryOtpion">
				<div style="margin: 5px 0;">
				<?php echo lang('connected to quickbooks online company') ?> <span class="bold"><?php echo $company_data->CompanyName ?></span>
				</div>
				<a class="link-ico qbo-btn-white-sm" href="#" onclick="og.openLink(og.getUrl('quickbooks','admin_disconnect'));"
					style="background-size: 21px; margin: 10px 0px;"><?php echo lang('disconnect from quickbooks') ?></a>
			</div>
		<?php 
		} else {
			?>
			<div class="configCategoryOtpion">
				<div style="margin: 5px 0;">
				<?php echo lang('not connected at quickbooks online') ?>
				</div>
				<a class="link-ico qbo-btn-white-sm" href="#" onclick="og.openLink(og.getUrl('quickbooks','admin_connect'));"
					style="background-size: 21px; margin: 10px 0px;"><?php echo lang('connect to quickbooks') ?></a>
			</div>
		<?php 
		}
		
	}
}

/**
 * Add more sub sections to the QBO settings
 */
function quickbooks_more_config_category_sections($params, &$counter) {
	$category = array_var($params, 'category');
	
	if ($category->getName() == 'quickbooks_online') {
		?>
		
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','initial_setting_time_sheets'));"><?php echo lang('custom fields initial time entrie name')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('custom fields initial time entrie desc')?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','edit_custom_field_match'));"><?php echo lang('custom fields matching')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('custom fields matching desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','edit_hour_services_match'));"><?php echo lang('hour services matching')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('hour services matching desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','edit_items_and_categories_match'));"><?php echo lang('item categories matching')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('item categories matching desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','edit_taxes_match'));"><?php echo lang('taxes matching')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('taxes matching desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','sync_qbo_payment_methods'));"><?php echo lang('synchronize payment methods')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('synchronize payment methods desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','import_invoices_from_qbo'));"><?php echo lang('import invoices and payments from qbo')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('import invoices and payments from qbo desc') ?></div>
			</div>
		</div>
		<?php 
		$counter++;

		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('quickbooks','sync_qbo_accounts_and_payment_methods'));"><?php echo lang('qb sync account and methods')?></a></div>
				<div class="configOptionDescription desc"><?php echo lang('qb sync account and methods description')?></div>
			</div>
		</div>
		<?php
		$counter++;

		if (logged_user()->getEmailAddress() == 'support@evxsoftware.com') {
		?>
		<div class="configCategoryOtpion <?php echo $counter % 2 ? 'odd' : 'even' ?>">
			<div class="configOptionInfo">
				<div class="configOptionLabel"><a href="#" onclick="og.openLink(og.getUrl('qbo_viewer','index'));">QBO Viewer</a></div>
				<div class="configOptionDescription desc">View QBO entities (only for development and testing)</div>
			</div>
		</div>
		<?php 
		$counter++;
		}
	}
}

function quickbooks_check_integration_status($params, &$result){
	$object = array_var($params, 'object');
	if($object instanceof IncomeInvoice || $object instanceof InvoicePaymentReceipt){
		$qb_status = $object->getColumnValue('quickbooks_online_id') > 0;
    
		if($qb_status){
			Env::useLibrary('qbo_sdk', 'quickbooks');
			Env::useHelper('qbo_functions', 'quickbooks');

			try {
				$access_token = get_qbo_access_token(); 
				$is_valid_token = $access_token && is_valid_qbo_access_token($access_token);
			} catch (QuickBooksOnline\API\Exception\SdkException $e) {
				Logger::log_r($e->getMessage());
			}

			if (!($access_token && $is_valid_token)){
				$result = false;
				flash_error(lang('you are currently disconnected from quickbooks. please re-connect and try again'));
				return;
			}
		}
	}
}



function quickbooks_object_definition($type, &$ret) {
	if ($type == 'IncomeInvoice') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
	}
	if ($type == 'InvoicePaymentReceipt') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
	}
	if ($type == 'Member') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
	}
	if ($type == 'IncomeTax') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
	}
	$qb_plugin = Plugins::instance()->findOne(array("conditions" => "name='quickbooks'"));
	if ($qb_plugin->getVersion() >= 12) {
		if ($type == 'Contact') {
			$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
			$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
		}
	}
	if ($qb_plugin->getVersion() >= 13) {
		if ($type == 'Contact') {
			$ret['quickbooks_online_vendor_id'] = DATA_TYPE_INTEGER;
		}
	}
	if ($type == 'Timeslot') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
		$ret['tsheets_online_id'] = DATA_TYPE_INTEGER;
	}
	if ($type == 'PaymentReceipt') {
		$ret['quickbooks_online_id'] = DATA_TYPE_INTEGER;
		$ret['quickbooks_online_sync_token'] = DATA_TYPE_INTEGER;
		$ret['qbo_reimburse_charge_id'] = DATA_TYPE_INTEGER;
	}
}


function quickbooks_after_invoice_save($params, &$ignored) {
	// prevent sync loops and don't sync the invoices if batch invoicing process is running (they will be synchronized after the whole the batch process finishes)
	if (array_var($_SESSION, 'sync_incoming_objects') || array_var($_SESSION, 'batch_invoice_running')) return;
	
	$invoice = array_var($params, 'invoice');
	/* @var $invoice IncomeInvoice */
	if ($invoice instanceof IncomeInvoice && $invoice->getStatus() != 'canceled') {
		if (strlen($invoice->getInvoiceNumber()) > 21) {
			throw new Exception("Invoice number length cannot exceed 21 characters");
		}

		Env::useHelper('qbo_functions', 'quickbooks');
		if (check_if_object_can_be_sync($invoice)) {
			evt_add("save_invoice_to_quickbooks", array('id' => $invoice->getId()));
		}
		
	}
}


function quickbooks_after_invoice_payment_receipt_save($params, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'object');
	/* @var object InvoicePaymentReceipt */
	
	Env::useHelper('qbo_functions', 'quickbooks');
	if (check_if_object_can_be_sync($object)) {
		evt_add("save_invoice_payment_to_quickbooks", array('id' => $object->getId()));
	}
}


function quickbooks_after_content_object_trash($params, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'object');

	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($object)) return;

	if ($object instanceof IncomeInvoice && $object->getColumnValue("quickbooks_online_id") > 0 && $object->getStatus() != 'canceled') {
		$controller = new QuickbooksController();
		$controller->delete_invoice_from_quickbooks($object);
	}
	
	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		if ($object instanceof InvoicePaymentReceipt && $object->getColumnValue("quickbooks_online_id") > 0) {
			$controller = new QuickbooksController();
			$controller->delete_invoice_payment_from_quickbooks($object);
		}
	}

	if ($object instanceof Timeslot && $object->getColumnValue("quickbooks_online_id") > 0) {
		$controller = new QuickbooksController();
		$controller->delete_timeslot_from_quickbooks($object);
	}

	if ($object instanceof PaymentReceipt) {
		$controller = new QuickbooksController();
		$controller->delete_actual_expense_from_quickbooks($object);
	}
}

function quickbooks_after_object_controller_trash($ids, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	Env::useHelper('qbo_functions', 'quickbooks');
	
	if (!is_array($ids)) {
		$ids = explode(',', $ids);
	}
	$ids = array_filter($ids, 'is_numeric');
	foreach ($ids as $id) {
		$object = Objects::findObject($id);
		if (!check_if_object_can_be_sync($object)) continue;

		if ($object instanceof IncomeInvoice && $object->getColumnValue("quickbooks_online_id") > 0 && $object->getStatus() != 'canceled') {
			$controller = new QuickbooksController();
			$controller->delete_invoice_from_quickbooks($object);
		}
		
		if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
			if ($object instanceof InvoicePaymentReceipt && $object->getColumnValue("quickbooks_online_id") > 0) {
				$controller = new QuickbooksController();
				$controller->delete_invoice_payment_from_quickbooks($object);
			}
		}

		if ($object instanceof Timeslot && $object->getColumnValue("quickbooks_online_id") > 0) {
			// delete time entry at qbo
			delete_time_from_qbo($object);
		}
	}
}

function quickbooks_after_content_object_untrash($params, $ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	Logger::log_r("  UNTRASH !!!!");
	$object = array_var($params, 'object');

	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($object)) return;

	if ($object instanceof IncomeInvoice) {
		$controller = new QuickbooksController();
		$controller->save_invoice_to_quickbooks($object);
	}
	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		if ($object instanceof InvoicePaymentReceipt) {
			$controller = new QuickbooksController();
			$controller->save_invoice_payment_to_quickbooks($object);
		}
	}

	if ($object instanceof Timeslot && !$object->isOpen()) {
		$controller = new QuickbooksController();
		$controller->save_timeslots_to_quickbooks(array($object->getId()));
	}

	if ($object instanceof PaymentReceipt) {
		$controller = new QuickbooksController();
		$controller->save_actual_expense_to_quickbooks(array($object->getId()));
	}
}

function quickbooks_after_invoice_cancel($params, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'invoice');

	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($object)) return;

	if ($object instanceof IncomeInvoice && $object->getColumnValue("quickbooks_online_id") > 0) {
		$controller = new QuickbooksController();
		$controller->void_invoice_in_quickbooks($object);
	}
}

function quickbooks_after_invoice_reopen($params, &$ignored) {
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent sync loops
	
	$object = array_var($params, 'invoice');

	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($object)) return;

	if ($object instanceof IncomeInvoice && $object->getColumnValue("quickbooks_online_id") > 0) {
		$controller = new QuickbooksController();
		$controller->save_invoice_to_quickbooks($object);
	}
}

function quickbooks_render_page_actions($invoice, &$ret) {
	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($invoice)) return;

	if ($invoice instanceof IncomeInvoice && !$invoice->isTrashed() && $invoice->getColumnValue('quickbooks_online_id') == 0){
		
		PageActions::instance()->removeAction('ico-print');
		
		$js_function = "og.quickbooks.save_invoice_to_quickbooks(".$invoice->getId().", true)";
		
		add_page_action(lang('sync to quickbooks'), "javascript:$js_function", "link-ico qbo-btn-white-sm", null, null, true);
		
	}

	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {

		if ($invoice instanceof InvoicePaymentReceipt && !$invoice->isTrashed() && $invoice->getColumnValue('quickbooks_online_id') == 0){
			
			PageActions::instance()->removeAction('ico-print');
			
			$js_function = "og.quickbooks.save_invoice_payment_to_quickbooks(".$invoice->getId().", true)";
			
			add_page_action(lang('sync to quickbooks'), "javascript:$js_function", "link-ico qbo-btn-white-sm", null, null, true);
			
		}
	}
}


function quickbooks_after_generate_batch_invoices($params, &$ignored) {

	$invoices = array_var($params, 'invoices');
	
	if (count($invoices) > 0) {
		$controller = new QuickbooksController();
		$controller->save_several_invoices_to_quickbooks($invoices);
	}

}


// TIMESLOTS HOOKS
function quickbooks_timeslots_list_additional_column_values($parameters, &$info) {
	$timelot = array_var($parameters, 'timelot');
	if ($timelot instanceof Timeslot) {
		$remote_id = 0;

		if(config_option('have_time_sheets')){
			$remote_id = $timelot->getColumnValue('tsheets_online_id');
		}else{
			$remote_id = $timelot->getColumnValue('quickbooks_online_id');
		}

		if ($remote_id > 0) {
			$info['qbo_sync_status'] = '<div>'.lang('synchronized').'</div>';
		} else {
			$info['qbo_sync_status'] = '<div class="error">'.lang('not synchronized').'</div>';
		}

	}
}

function quickbooks_after_add_to_members($object, &$members){
	Env::useHelper('qbo_functions', 'quickbooks');
	if ($object instanceof Timeslot && check_if_object_can_be_sync($object) && !$object->isOpen()) {
		evt_add("save_timeslot_to_quickbook", array($object->getId())); 
	}
}

function quickbooks_after_timer_closed($object, &$members) {
	Env::useHelper('qbo_functions', 'quickbooks');
	if ($object instanceof Timeslot && !$object->isOpen()) {
		evt_add("save_timeslot_to_quickbook", array($object->getId())); 
	}
}

// update time status in quickbooks after setting it as unbilled in evx/feng
function quickbooks_after_change_object_inv_status($params, &$ignored) {
	
	$object = array_var($params, 'object');
	$old_status = array_var($params, 'old_status');
	// Check if object has column 'invoicing_status', if the value of the column changed and
	// if the old value was empty before, it means that the object is new.
	if ($object instanceof Timeslot && $object->getColumnValue('invoicing_status') != $old_status && $old_status != '') {

		// timeslot's status is pending -> mark as unbilled in quickbooks
		if ($object->getColumnValue('quickbooks_online_id') > 0 && $object->getColumnValue('invoicing_status') == 'pending') {

			$controller = new QuickbooksController();
			$controller->set_timeslot_as_unbilled($object);
		}
	}
}

function quickbooks_timeslot_list_additional_actions($params, &$actions) {
	$label = "";

	if(config_option('have_time_sheets')){
		$label = "Send to T-sheets"; //lang('recalculate billing')
	}else{
		$label = "Send to Quickbooks";
	}

	$actions[] = array(
		'url' => '#',
		'onclick' => "og.quickbooks.send_to_tsheets('".array_var($params, 'panel_id')."');",
		'cls' => 'ico-billing',
		'text' => $label
	);
}

// EXPENSES HOOKS

function quickbooks_after_payment_receipt_save($payment_receipt, &$info) {
	Env::useHelper('qbo_functions', 'quickbooks');
	if (!check_if_object_can_be_sync($payment_receipt)) return;

	if ($payment_receipt instanceof PaymentReceipt) {
		evt_add("save_actual_expense_to_quickbooks", array('id' => $payment_receipt->getId()));
	}	
}

function quickbooks_additional_system_columns($manager, &$columns) {
	if ($manager instanceof PaymentReceipts) {
		$columns[] = 'quickbooks_online_id';
		$columns[] = 'quickbooks_online_sync_token';
	}
}

function quickbooks_before_generate_payment_columns($payment_receipt, &$info) {
	$info[] = "quickbooks";
}

function quickbooks_payment_list_additional_column_values($parameters, &$info) {
	
	$payment = array_var($parameters, 'payment');
	if ($payment instanceof PaymentReceipt) {
		if (in_array($payment->getExpenseType(), config_option("expense_types_to_sync"))) {
			if ($payment->getColumnValue('quickbooks_online_id') > 0) {
				$info['quickbooks'] = '<div>'.lang('synchronized').'</div>';
			} else {
				$info['quickbooks'] = '<div class="error">'.lang('not synchronized').'</div>';
			}
		} else {
			$info['quickbooks'] = '<div>'.strtoupper(lang('n/a')).'</div>';
		}
	}
}



function quickbooks_override_list_custom_property_values($params, &$custom_property_values){
	$custom_property = array_var($params, 'cp');
	if ($custom_property instanceof CustomProperty && $custom_property->getCode() == 'quickbooks_payment_method') {
		$p_methods = QuickbooksPaymentMethods::findAll(array("conditions" => "visible=1"));
		//$pm_values = array('0@'.lang('none'));
		$pm_values = array();
		foreach($p_methods as $p_method){
			$pm_values[] = $p_method->getPaymentMethodId().'@'.str_replace(',', '', $p_method->getName());
		}
		$custom_property_values = implode(',', $pm_values);

	} else if ($custom_property instanceof CustomProperty && $custom_property->getCode() == 'quickbooks_payment_account') {
		$accounts = QuickbooksAccounts::findAll(array("conditions" => "visible=1 AND account_type_id IN (SELECT account_type_id FROM ".TABLE_PREFIX."quickbooks_account_types WHERE is_payment_account=1)"));
		$acc_values = array('0@'.lang('none'));
		//$acc_values = array();
		foreach($accounts as $account){
			$acc_values[] = $account->getAccountId().'@'.str_replace(',', '', $account->getName());
		}
		$custom_property_values = implode(',', $acc_values);
	}
}


function quickbooks_after_project_save($params, &$project) {

	if (logged_user()->getEmailAddress() == 'support@evxsoftware.com') {

		$project_data = array_var($params, 'project_data');
		$member = array_var($params, 'member');

		$member->setColumnValue("quickbooks_online_id", array_var($project_data, "quickbooks_online_id"));
		$member->save();

	}

}

function quickbooks_after_prepare_list_objects($params, &$invoices) {
	$from_date_sync = config_option("qbo_sync_entities_from_date");

	foreach($invoices['invoices'] as &$invoice){
		$invoice_information = IncomeInvoices::findById($invoice['object_id']);
		
		$can_sync = false;
		if ($from_date_sync instanceof DateTimeValue) {
			$can_sync = $invoice_information->getDate()->getTimestamp() >= $from_date_sync->getTimestamp();
		}

		if(!$can_sync && $from_date_sync && !$invoice_information->getColumnValue('quickbooks_online_id')){
			$invoice['qbo_sync_status'] = 'noApply';
		}else{
			$invoice['qbo_sync_status'] = $invoice_information->getColumnValue('quickbooks_online_id') > 0;
		}

	}
}


function quickbooks_define_colum_order_list($params, &$cols_data) {
	$order = array_var($params, 'order');

	if($order == 'qbo_sync_status'){
		$cols_data = '`quickbooks_online_id`';
	}
}