<?php

const TYPE_BILL = 'BILL';
const TYPE_CREDIT_CARD = "CREDIT CARD";


function build_export_file_contents_from_arrays($export_data) {
	
	$file_content = "";
	
	foreach ($export_data as $line_data) {
		$file_content .= implode(QB_COLUMN_SEPARATOR, $line_data) . QB_LINE_SEPARATOR;
	}
	
	return $file_content;
}

// General header function
function get_qb_export_timeslots_general_headers() {
	return array(
		'TIMERHDR' => array('TIMERHDR', 'VER', 'REL', 'COMPANYNAME', 'IMPORTEDBEFORE', 'FROMTIMER', 'COMPANYCREATETIME'),
		'HDR' => array('HDR', 'PROD', 'VER', 'REL', 'IIFVER', 'DATE', 'TIME', 'ACCNTNT', 'ACCNTNTSPLITTIME'),
	);
}

// Time activity headers
function get_qb_export_timeslots_line_headers() {
	return array('TIMEACT', 'DATE', 'JOB', 'EMP', 'ITEM', 'PITEM', 'DURATION', 'PROJ', 'NOTE', 'BILLINGSTATUS');
}


// Invoice general header
function get_qb_export_invoices_main_headers() {
	return array('TRNS','TRNSID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','TOPRINT','NAMEISTAXABLE','ADDR1','ADDR2','ADDR3','ADDR4','ADDR5','DUEDATE','TERMS','PAID','PAYMETH','SHIPVIA','SHIPDATE','OTHER1','REP','FOB','PONUM','INVTITLE','INVMEMO','SADDR1','SADDR2','SADDR3','SADDR4','SADDR5','PAYITEM','YEARTODATE','WAGEBASE','EXTRA','TOSEND','ISAJE');
}

// invoice line header
function get_qb_export_invoices_line_headers() {
	return array('SPL','SPLID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','QNTY','PRICE','INVITEM','PAYMETH','TAXABLE','VALADJ','REIMBEXP','SERVICEDATE','OTHER2','OTHER3','PAYITEM','YEARTODATE','WAGEBASE','EXTRA');
}

// end transaction header for invoices
function get_qb_export_invoices_end_headers() {
	return array('ENDTRNS');
}

// All invoice headers togheter
function get_qb_export_invoices_headers() {
	return array(
		get_qb_export_invoices_main_headers(),
		get_qb_export_invoices_line_headers(),
		get_qb_export_invoices_end_headers(),
	);
}


// Actual expenses general header
function get_qb_export_actual_expenses_main_headers() {
	return array('TRNS','TRNSTYPE','DATE','NAME','TIMESTAMP','DOCNUM','ACCNT');
}

// Actual expenses line header
function get_qb_export_actual_expenses_line_headers() {
	return array('SPL','TRNSTYPE','DATE','NAME','MEMO','CLASS','QNTY','PRICE','AMOUNT','INVITEM','ACCNT');
}

// end transaction header for Actual expenses
function get_qb_export_actual_expenses_end_headers() {
	return array('ENDTRNS');
}

// All invoice headers togheter
function get_qb_export_actual_expenses_headers() {
	return array(
		get_qb_export_actual_expenses_main_headers(),
		get_qb_export_actual_expenses_line_headers(),
		get_qb_export_actual_expenses_end_headers(),
	);
}

function get_qb_export_general_header_values($header_type) {
	
	$now = DateTimeValueLib::now();
	$today_str = $now->format('Y-m-d');
	$now_timestamp = $now->getTimestamp();
	$values = array($header_type);
	
	switch ($header_type) {
		case "TIMERHDR":
			$values = array($header_type, "8", "0", owner_company()->getObjectName(), QB_BOOLEAN_NO, QB_BOOLEAN_NO, owner_company()->getCreatedOn()->getTimestamp());
			break;
		case "HDR":
			$values = array($header_type, QB_PRODUCT_NAME, QB_PRODUCT_VER, QB_PRODUCT_REL, QB_IIF_VER, $today_str, $now_timestamp, QB_BOOLEAN_NO, "0");
			break;
		default:
			break;
	}
	
	return $values;
}

function get_qb_export_timeslots_line_values($headers, Timeslot $timeslot) {
	$values = array();
	
	foreach ($headers as $header) {
		$value = "";
		switch ($header) {
			case "TIMEACT": 
				$value = $header;
				break;
			case "DATE": 
				$value = format_date($timeslot->getStartTime(), null, null);
				break;
			case "JOB": 
				$value = get_qbd_project_name_for_feng_object($timeslot);
				break;
			case "EMP": 
				$value = $timeslot->getUser()->getObjectName();
				break;
			case "ITEM": // get the item name from the member of dimension Labor Categories (hour_types)
				$value = get_qbd_item_name_for_feng_object($timeslot);
				break;
			case "PITEM": 
				// @TODO: get the payroll item
				break;
			case "DURATION": 
				$total_minutes = $timeslot->getMinutes();
				$hh = str_pad(floor($total_minutes / 60), 2, '0', STR_PAD_LEFT);
				$mm = str_pad($total_minutes % 60, 2, '0', STR_PAD_LEFT);
				$value = "$hh:$mm";
				break;
			case "PROJ": // get the class name using the config option and custom properties defined for that purpose
				$value = get_qbd_class_for_feng_object($timeslot);
				break;
			case "NOTE": 
				$value = str_replace("\n", "/n", str_replace("\r", "", $timeslot->getDescription()));
				break;
			case "BILLINGSTATUS": 
				if ($timeslot->getColumnValue('invoicing_status') != 'non_billable') {
					if ($timeslot->getColumnValue('invoicing_status') == 'invoiced') {
						$value = "2";
					} else {
						$value = "1";
					}
				} else {
					$value = "0";
				}
				break;
			default: 
				break;
		}
		$values[] = str_replace("\t", " ", $value);
	}
	
	return $values;
}


/*
array('TRNS','TRNSID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','TOPRINT','NAMEISTAXABLE',
	'ADDR1','ADDR2','ADDR3','ADDR4','ADDR5','DUEDATE','TERMS','PAID','PAYMETH','SHIPVIA','SHIPDATE','OTHER1','REP','FOB','PONUM',
	'INVTITLE','INVMEMO','SADDR1','SADDR2','SADDR3','SADDR4','SADDR5','PAYITEM','YEARTODATE','WAGEBASE','EXTRA','TOSEND','ISAJE'),
array('SPL','SPLID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','QNTY','PRICE','INVITEM','PAYMETH','TAXABLE','VALADJ','REIMBEXP','SERVICEDATE','OTHER2','OTHER3','PAYITEM','YEARTODATE','WAGEBASE','EXTRA'),
array('ENDTRNS'),
*/


function get_qb_export_invoices_main_values(IncomeInvoice $object) {

	$headers = get_qb_export_invoices_main_headers();
	$values = array();
	
	foreach ($headers as $header) {
		$value = "";
		switch ($header) {
			case 'TRNS': 
				$value = $header;
				break;
			case 'TRNSID': 
				$value = $object->getId();
				break;
			case 'TRNSTYPE': 
				$value = "INVOICE";
				break;
			case 'DATE': 
				// $value = $object->getCreatedOn()->format('m/d/y');
				$value = $object->getColumnValue('date')->format('m/d/y');
				break;
			case 'ACCNT':
				$value = get_qbd_invoice_account($object);
				break;
			case 'NAME': 
				// @TODO: see if we can send the project here, see if we have to include project in the iif export before the invoice
				/*$client = Members::getMemberById($object->getCustomerId());
				$value = $client instanceof Member ? $client->getName() : "";*/
				$value = get_qbd_project_name_for_feng_object($object);
				break;
			case 'CLASS': 
				$value = get_qbd_class_for_feng_object($object);
				break;
			case 'AMOUNT': 
				$value = $object->getTotal();
				break;
			case 'DOCNUM': 
				$value = $object->getInvoiceNumber();
				break;
			case 'MEMO': 
				$value = str_replace("\n", "/n", str_replace("\r", "", $object->getAdditionalInformation()));
				$value = replace_qbd_simple_colon($value);
				break;
			case 'CLEAR': 
				$value = "N";
				break;
			case 'TOPRINT': 
				$value = "N";
				break;
			case 'NAMEISTAXABLE': 
				$value = "Y";
				break;
			case 'ADDR1': 
				$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerName())), 0, 40);
				break;
			case 'ADDR2': 
				$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerAddress())), 0, 40);
				break;
			case 'ADDR3': 
				$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerCity() . " " . $object->getCustomerState() . " " . $object->getCustomerZipcode())), 0, 40);
				break;
			case 'ADDR4': break;
			case 'ADDR5': break;
			case 'DUEDATE': 
				$due_date = $object->getExpirationDate();
				if ($due_date instanceof DateTimeValue) {
					$value = $due_date->format('m/d/y');
				}
				break;
			case 'TERMS': 
				$value = get_qbd_terms_for_invoice($object);
				break;
			case 'PAID': 
				if ($object->getColumnValue('paid_amount') == $object->getTotal()) {
					$value = "Y";
				} else {
					$value = "N";
				}
				break;
			case 'PAYMETH': break;
			case 'SHIPVIA': break;
			case 'SHIPDATE': break;
			case 'OTHER1': break;
			case 'REP': break;
			case 'FOB': break;
			case 'PONUM': break;
			case 'INVTITLE': break;
			case 'INVMEMO': break;
			case 'SADDR1': break;
			case 'SADDR2': break;
			case 'SADDR3': break;
			case 'SADDR4': break;
			case 'SADDR5': break;
			case 'PAYITEM': break;
			case 'YEARTODATE': break;
			case 'WAGEBASE': break;
			case 'EXTRA': break;
			case 'TOSEND': break;
			case 'ISAJE': break;
		}

		$values[] = $value;
	}

	return $values;
}

//'SPL','SPLID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','QNTY','PRICE','INVITEM','PAYMETH','TAXABLE','VALADJ','REIMBEXP',
//'SERVICEDATE','OTHER2','OTHER3','PAYITEM','YEARTODATE','WAGEBASE','EXTRA'),
function get_qb_export_invoices_line_values(IncomeInvoice $object) {

	$act_exp_obj_type = ObjectTypes::findByName('payment_receipt');
	$act_exp_obj_type_id = $act_exp_obj_type ? $act_exp_obj_type->getId() : -1;

	$headers = get_qb_export_invoices_line_headers();

	$invoice_lines = $object->getInvoiceLines();

	$lines_data = array();
	
	foreach ($invoice_lines as $line) {

		$values = array();
		
		foreach ($headers as $header) {
			$value = "";
			switch ($header) {
				case 'SPL': 
					$value = $header;
					break;
				case 'SPLID': 
					$value = $line->getOrder();
					break;
				case 'TRNSTYPE': 
					$value = "INVOICE";
					break;
				case 'DATE': 
					$value = $object->getColumnValue('date')->format('m/d/y');
					break;
				case 'ACCNT':
					break;
				case 'NAME': break;
				case 'CLASS': break;
				case 'AMOUNT': 
					$value = -1 * $line->getTotal();
					break;
				case 'DOCNUM': break;
				case 'MEMO': 

					if($line->getDetail()){
						$value = str_replace("\n", "/n", str_replace("\r", "", $line->getDetail()));
					}else{
						$category_id = $line->getColumnValue('category_id');
						$member_info = Members::getMemberById($category_id);
						if($member_info){
							$value = $member_info->getName();
						}else{
							$value = '';
						}
						
					}

					// if($category_id){
					// 	$member_info = Members::getMemberById($category_id);
					// 	$value = $member_info->getName();
					// }else{
					// 	$value = str_replace("\n", "/n", str_replace("\r", "", $line->getDetail()));
					// }

					break;
				case 'CLEAR': 
					$value = "N";
					break;
				case 'QNTY': 
					$value = -1 * $line->getAmount();
					break;
				case 'PRICE': 
					$value = $line->getUnitValue();
					break;
				case 'INVITEM': 
					$cat_member = Members::getMemberById($line->getCategoryId());
					$value = get_qbd_item_name_for_member($cat_member, $object, $line);
					if ($value == "") {
						$value = config_option("qbd_default_income_item");
					}

					$value = replace_qbd_simple_colon($value);
					break;
				case 'PAYMETH': break;
				case 'TAXABLE': 
					$value = "N";
					break;
				case 'VALADJ': break;
				case 'REIMBEXP': 
				//	if ($line->getFromObjectTypeId() == $act_exp_obj_type_id) {
						$value = "NOTHING";
				/*	} else {
						$value = "NONEED";
					}*/
					break;
				case 'SERVICEDATE': 
					$date = $line->getDate();
					if ($date instanceof DateTimeValue) {
						$value = $date->format('m/d/y');
					}
					break;
				case 'OTHER2': break;
				case 'OTHER3': break;
				case 'PAYITEM': break;
				case 'YEARTODATE': break;
				case 'WAGEBASE': break;
				case 'EXTRA': break;
			}
	
			$values[] = $value;
		}

		$lines_data[] = $values;
	}

	return $lines_data;
}

function get_qb_export_invoices_end_values($object) {
	return get_qb_export_invoices_end_headers();
}

function get_qb_export_actual_expenses_main_values(PaymentReceipt $object) {

	$act_exp_obj_type = ObjectTypes::findByName('payment_receipt');
	$headers = get_qb_export_actual_expenses_main_headers();
	$values = array();

	foreach ($headers as $header) {
		$value = "";
		switch ($header) {
			case 'TRNS': 
				$value = $header;
				break;
			case 'TRNSID': 
				// $value = $object->getId();
				break;
			case 'TRNSTYPE': 
				if($object->getExpenseType() == 'credit_card_purchase'){
					$value = TYPE_CREDIT_CARD;
				}else{
					$value = TYPE_BILL;
				}
				break;
			case 'DATE': 
				// $value = $object->getCreatedOn()->format('m/d/y');
				$value = $object->getColumnValue('date')->format('m/d/y');
				break;
			case 'ACCNT':
				if($object->getExpenseType() == 'credit_card_purchase'){
					$value = get_qbd_account_type(TYPE_CREDIT_CARD);
				}else{
					$value = get_qbd_account_type(TYPE_BILL);
				}

				break;
			case 'NAME':
				if($object->getExpenseType() == 'direct_to_vendor'){
					$value = get_qbd_vendor_name_for_feng_object($object);
				}else{
					$employee_id = $object->getColumnValue('paid_by_id');
					if($employee_id){
						// $feng_contact = Contacts::findById($employee_id);
						// $value = $feng_contact->getObjectName();
						$value = get_qbd_get_name_map_employee($employee_id);
					}
				}
				break;
			case 'CLASS': 
				$value = get_qbd_class_for_feng_object($object);
				break;
			case 'AMOUNT': 
				$value = $object->getAmount();
				break;
			case 'DOCNUM':
				// $invoice_id_cp = CustomProperties::getCustomPropertyByCode($act_exp_obj_type->getId(), "invoice_number");
				
				// $value = '';
				// if ($invoice_id_cp instanceof CustomProperty) {
				// 	$cpval = CustomPropertyValues::getCustomPropertyValue($object->getId(), $invoice_id_cp->getId());
				// 	if ($cpval instanceof CustomPropertyValue) {
				// 		$value = $cpval->getValue();
				// 	}
				// }

				// $value = substr($value, 0, 10);
				break;
			case 'MEMO': 
				$value = str_replace("\n", "/n", str_replace("\r", "", $object->getDescription()));
				break;
			case 'CLEAR': 
				//$value = "N";
				break;
			case 'TOPRINT': 
				//$value = "N";
				break;
			case 'NAMEISTAXABLE': 
				//$value = "Y";
				break;
			case 'ADDR1': 
	//			$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerName())), 0, 40);
				break;
			case 'ADDR2': 
	//			$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerAddress())), 0, 40);
				break;
			case 'ADDR3': 
	//			$value = substr_utf(str_replace("\n", "/n", str_replace("\r", "", $object->getCustomerCity() . " " . $object->getCustomerState() . " " . $object->getCustomerZipcode())), 0, 40);
				break;
			case 'ADDR4': break;
			case 'ADDR5': break;
			case 'DUEDATE': 
				//$due_date = $object->getDueDate();
				//if ($due_date instanceof DateTimeValue) {
				//	$value = $due_date->format('m/d/y');
				//}
				break;
			case 'TERMS': 
				//$value = get_qbd_terms_for_invoice($object);
				break;
			case 'PAID': 
				/*	if ($object->getColumnValue('paid_amount') == $object->getTotal()) {
					$value = "Y";
				} else {
					$value = "N";
				}*/
				break; 
			case 'TIMESTAMP': 
				$value = strtotime('now');
				break;
			case 'PAYMETH': break;
			case 'SHIPVIA': break;
			case 'SHIPDATE': break;
			case 'OTHER1': break;
			case 'REP': break;
			case 'FOB': break;
			case 'PONUM': break;
			case 'INVTITLE': break;
			case 'INVMEMO': break;
			case 'SADDR1': break;
			case 'SADDR2': break;
			case 'SADDR3': break;
			case 'SADDR4': break;
			case 'SADDR5': break;
			case 'PAYITEM': break;
			case 'YEARTODATE': break;
			case 'WAGEBASE': break;
			case 'EXTRA': break;
			case 'TOSEND': break;
			case 'ISAJE': break;
		}

		$values[] = $value;
	}

	return $values;
}

//'SPL','SPLID','TRNSTYPE','DATE','ACCNT','NAME','CLASS','AMOUNT','DOCNUM','MEMO','CLEAR','QNTY','PRICE','INVITEM','PAYMETH','TAXABLE','VALADJ','REIMBEXP',
//'SERVICEDATE','OTHER2','OTHER3','PAYITEM','YEARTODATE','WAGEBASE','EXTRA'),
function get_qb_export_actual_expenses_line_values(PaymentReceipt $object) {

	// Validate if is a valid type
	if ($object instanceof PaymentReceipt && !in_array($object->getExpenseType(), config_option("qbd_expense_types_to_sync"))) {
		// Logger::log_r(['OMITIDO', 'Payment is of rental type.', $object->getName(), $object->getExpenseType()]);
		return [];
	}
	
	$act_exp_obj_type = ObjectTypes::findByName('payment_receipt');
	$act_exp_obj_type_id = $act_exp_obj_type ? $act_exp_obj_type->getId() : -1;
	$exp_cat_dim = Dimensions::findByCode('expense_categories');

	$headers = get_qb_export_actual_expenses_line_headers();

	$lines_data = array();
	$values = array();

	foreach ($headers as $header) {
		$value = "";
		switch ($header) {
			case 'SPL': 
				$value = $header;
				break;
			case 'SPLID': 
				$value = "1";
				break;
			case 'TRNSTYPE': 
				if($object->getExpenseType() == 'credit_card_purchase'){
					$value = TYPE_CREDIT_CARD;
				}else{
					$value = TYPE_BILL;
				}
				break;
			case 'DATE': 
				$value = $object->getColumnValue('date')->format('m/d/y');
				break;
			case 'ACCNT':
				$cat_member = null;
				$members = $object->getMembers();
				foreach ($members as $m) {
					if ($m->getDimensionId() == $exp_cat_dim->getId()) {
						$cat_member = $m;
						break;
					}
				}
				
				if ($cat_member instanceof Member) {
					$value = get_qbd_item_name_for_member($cat_member, $object, null, 'qbd_expense_acct');
				}

				break;
			case 'NAME':
				// if($object->getExpenseType() == 'direct_to_vendor'){
				// 	$value = get_qbd_vendor_name_for_feng_object($object);
				// }else{
					$value = get_qbd_project_name_for_feng_object($object);

					// if(strlen($value)>41){
					// 	// $error = true;
					// 	// $error_msg = 'The NAME exeeds maxium characters';
					// }
					
					// if(empty($value)){
					// 	// $error = true;
					// 	// $error_msg = 'The NAME is required';
					// }
				// }
				break;
			case 'CLASS': 
				$value = get_qbd_class_for_feng_object($object);
				break;
			case 'AMOUNT': 
				$value = $object->getAmount();
				break;
			case 'MEMO': 
				if($object->getExpenseType() == 'credit_card_purchase'){
					$employee_id = $object->getColumnValue('paid_by_id');
					$feng_contact = Contacts::findById($employee_id);
					if($feng_contact){
						$value = $feng_contact->getObjectName() ?? '';
					}else{
						$value = '';
					}
				}else{
					$value = str_replace("\n", "/n", str_replace("\r", "", $object->getDescription()));
				}
				// $value .= ' - '.$object->getExpenseType();
				$value = replace_qbd_simple_colon($value);
				break;
			case 'CLEAR': 
				// $value = "N";
				break;
			case 'QNTY': 
				$value = $object->getQuantity();
				break;
			case 'PRICE': 
				$value = $object->getUnitCost();
				break;
			case 'INVITEM': 
				$cat_member = null;
				$members = $object->getMembers();
				foreach ($members as $m) {
					if ($m->getDimensionId() == $exp_cat_dim->getId()) {
						$cat_member = $m;
						break;
					}
				}
				if ($cat_member instanceof Member) {
					$value = get_qbd_item_name_for_member($cat_member, $object);
				}
				// if ($value == "") {
				// 	$value = config_option("qbd_default_income_item");
				// }

				$value = replace_qbd_simple_colon($value);
				break;
			case 'PAYMETH': break;
			case 'TAXABLE': break;
			case 'VALADJ': break;
			case 'REIMBEXP': break;
			case 'SERVICEDATE': break;
			case 'OTHER2': break;
			case 'OTHER3': break;
			case 'PAYITEM': break;
			case 'YEARTODATE': break;
			case 'WAGEBASE': break;
			case 'EXTRA': break;
		}

		$values[] = $value;
	}

	$lines_data[] = $values;

	return $lines_data;
}

function get_qb_export_actual_expenses_end_values($object) {
	return get_qb_export_invoices_end_headers();
}



/*******************************************************/
/****************** AUXILIAR FUNCTIONS *****************/
/*******************************************************/


function get_qbd_invoice_line_account(IncomeInvoice $object, IncomeInvoiceLine $line) {

	$account_name = "";

	// @TODO: try to get account from line's labor/expense category

	// if cant find account based in line's category use the default account defined in config option
	if ($account_name == "") {
		$account_name = config_option("qbd_default_income_account");
	}

	return $account_name;
}

function get_qbd_invoice_account(IncomeInvoice $object) {

	$account_name = "";

	// @TODO: try to get account from invoices's labor category

	// if cant find account based in line's category use the default account defined in config option
	if ($account_name == "") {
		$account_name = config_option("qbd_default_income_account");
	}

	return $account_name;
}

function get_qbd_invoiec_account(IncomeInvoice $object) {

	$account_name = "";

	// @TODO: try to get account from invoices's labor category

	// if cant find account based in line's category use the default account defined in config option
	if ($account_name == "") {
		$account_name = config_option("qbd_default_income_account");
	}

	return $account_name;
}

function get_qbd_expenses_account(PaymentReceipt $object) {
	$account_name = "";

	// @TODO: try to get account from invoices's labor category

	// if cant find account based in line's category use the default account defined in config option
	if ($account_name == "") {
		$account_name = config_option("qbd_default_expense_account");
	}

	return $account_name;
}

function get_qbd_terms_for_invoice(IncomeInvoice $object) {
	$terms_str = "";
	$terms_cp = CustomProperties::getCustomPropertyByCode($object->getObjectTypeId(), 'invoice_payment_period');
	if ($terms_cp) {
		$terms_cp_val = CustomPropertyValues::getCustomPropertyValue($object->getId(), $terms_cp->getId());
		if ($terms_cp_val) {
			$val = trim($terms_cp_val->getValue());
			if ($val == 'Due on receipt') {
				$terms_str = 'Due on recpt';
			} else {
				$terms_str = $val;
			}
		}
	}

	return $terms_str;
}

function get_qbd_project_name_for_feng_object(ContentDataObject $object) {
	$qbd_project_name = "";

	$project_ot = ObjectTypes::findByName('project');
	$customer_ot = ObjectTypes::findByName('customer');	

	$members = $object->getMembers();

	$customer_name = null;
	$project_name = null;
	$project_mem = null;
	$client_mem = null;

	foreach ($members as $member) {
		if ($member->getObjectTypeId() == $project_ot->getId()) {
			$project_name = $member->getName();
			$project_mem = $member;
		} else if ($member->getObjectTypeId() == $customer_ot->getId()) {
			$customer_name = $member->getName();
			$client_mem = $member;
		}
	}

	if (!$customer_name) {
		if ($project_mem) {
			$client_mem = get_client_member_of_project($project_mem);
			if ($client_mem) {
				$customer_name = $client_mem->getName();
			}
		}
	}
	if ($customer_name) {
		$qbd_project_name = $customer_name;
	}
	
	if ($project_name) {
		if (str_starts_with($project_name, $customer_name . ":") || strtolower($project_name) == strtolower($customer_name)) {
			$qbd_project_name = $project_name;
		} else {
			$qbd_project_name .= ($qbd_project_name == "" ? "" : ":") . $project_name;
		}
	}

	return $qbd_project_name;
}

function get_qbd_vendor_name_for_feng_object(ContentDataObject $object) {
	$supplier_ot = ObjectTypes::findByName('supplier');
	$members = $object->getMembers();
	
	foreach ($members as $member) {
		if ($member->getObjectTypeId() == $supplier_ot->getId()) {
			return $member->getName();
		}
	}

	return '';
}

function get_qbd_class_for_feng_object($feng_object) {
	
	$class_name = "";
	$class_member = null;
	$dimension_id_for_qbo_classes = config_option('qbd_dimension_for_classes');
	
	// check if the invoice is classified in any member of the dimension for classes
	foreach ($feng_object->getMembers() as $m) {
		if ($m->getDimensionId() == $dimension_id_for_qbo_classes) {
			$class_member = $m;
		}
	}

	if ($class_member instanceof Member) {
		$class_member_ot = ObjectTypes::findById($class_member->getObjectTypeId());
		if ($class_member_ot->getType() == 'dimension_group') {
			$class_cp = MemberCustomProperties::getCustomPropertyByCode($class_member->getObjectTypeId(), 'qbd_class');
			if ($class_cp) {
				$class_cp_val = MemberCustomPropertyValues::getMemberCustomPropertyValue($class_member->getId(), $class_cp->getId());
				if ($class_cp_val) $class_name = trim($class_cp_val->getValue());
			}
		} else {
			$class_cp = CustomProperties::getCustomPropertyByCode($class_member->getObjectTypeId(), 'qbd_class');
			if ($class_cp) {
				$class_cp_val = CustomPropertyValues::getCustomPropertyValue($class_member->getObjectId(), $class_cp->getId());
				if ($class_cp_val) $class_name = trim($class_cp_val->getValue());
			}
		}
	}

	return $class_name;
}

function get_qbd_item_name_for_feng_object($feng_object) {
	
	$cat_member = null;
	$dimension_id_for_qbo_items = Dimensions::findByCode('hour_types')->getId();
	
	foreach ($feng_object->getMembers() as $m) {
		if ($m->getDimensionId() == $dimension_id_for_qbo_items) {
			$cat_member = $m;
		}
	}

	return get_qbd_item_name_for_member($cat_member, $feng_object);
}

function get_qbd_item_name_for_member($cat_member, $feng_object = null, $invoice_line = null, $col = 'qbd_item') {
	
	$item_name = "";

	if ($cat_member instanceof Member) {

		$cat_member_ot = ObjectTypes::findById($cat_member->getObjectTypeId());

		if ($feng_object instanceof IncomeInvoice || $feng_object instanceof PaymentReceipt) {

			if ($cat_member_ot->getName() == 'hour_type') {
				
				// if category is for labor try to get the item name from the billing category given by the labor category and the other invoice's members
				Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
				Env::useHelper('functions', 'advanced_billing');
				
				$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
				$cant_decide_between_billing_definitions = null;
				$dim_members = get_task_dimension_members($allowed_dimensions, $feng_object->getId());
				$dim_members[$cat_member->getDimensionId()] = array($cat_member->getId());

				$billing_def = find_billing_def_by_user_and_members(0, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);
				if ($billing_def instanceof BillingDefinition) {
					$item_name = trim($billing_def->getColumnValue('qbd_item'));
				}

			} else if ($cat_member_ot->getName() == 'expense_category') {
				
				// if category is for expenses try to get the item name from the product type given by the expense category and the other invoice's members
				$product_type = null;
				if ($invoice_line instanceof IncomeInvoiceLine && $invoice_line->getProductTypeId() > 0) {
					$product_type = ProductTypes::getById($invoice_line->getProductTypeId());
				}

				if ($feng_object instanceof PaymentReceipt && $feng_object->getProductTypeId() > 0) {
					$product_type = ProductTypes::getById($feng_object->getProductTypeId());
				}

				if (!$product_type) {
					$object_members = $feng_object->getMembers();
					$object_members[] = $cat_member;
					$product_types = ProductTypes::getFilteredProductTypesData($object_members, null, true);
					$product_type = array_var($product_types, 0);
				}

				if ($product_type) {
					$item_name = trim($product_type->getColumnValue($col));// qbd_item
				}
			}
		}

		// if no item found using billing categories or product type definitions, then use the custom property defined in the category member
		if ($item_name == "") {
			if ($cat_member_ot->getType() == 'dimension_group') {
				$qbd_item_cp = MemberCustomProperties::getCustomPropertyByCode($cat_member->getObjectTypeId(), $col);
				if ($qbd_item_cp) {
					$qbd_item_cp_val = MemberCustomPropertyValues::getMemberCustomPropertyValue($cat_member->getId(), $qbd_item_cp->getId());
					if ($qbd_item_cp_val) $item_name = trim($qbd_item_cp_val->getValue());
				}
			} else {
				$qbd_item_cp = CustomProperties::getCustomPropertyByCode($cat_member->getObjectTypeId(), $col);
				if ($qbd_item_cp) {
					$qbd_item_cp_val = CustomPropertyValues::getCustomPropertyValue($cat_member->getObjectId(), $qbd_item_cp->getId());
					if ($qbd_item_cp_val) $item_name = trim($qbd_item_cp_val->getValue());
				}
			}
		}
	}

	return $item_name;
}

function get_qbd_account_type($type){
	$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbd_default_account_by_type WHERE  transaction_type = '". qbd_format_key($type) ."'");
	return $cf_matches_rows[0]['account'] ?? '';
}

function get_qbd_get_name_map_employee($eployee_id){
	$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbd_vendor_by_employee WHERE  eployee_id = '". $eployee_id ."'");
	return $cf_matches_rows[0]['vendor_name'] ?? '';
}

function qbd_format_key($key){
	return str_replace(' ', '_', strtolower($key));
}

function replace_qbd_simple_colon($text){
	return str_replace('"', '”', $text);
}