<?php
/**
 * Purchase Order controller
 *
 * @version 1.0
 *
 */
class SalesPurchaseOrderController extends ApplicationController {

	var $plugin_name = "sales_purchase_orders";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("SalesPurchaseOrderManager.js", $this->plugin_name);
		ajx_current("panel", "sales-purchase-order-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		if ($order == 'name') $order = 'number';
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		}
		
		$result = SalesPurchaseOrders::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => false,
			"start" => $start,
			"limit" => $limit,
		));
		$sales_purchase_orders = $result->objects;
		$object = $this->prepareObject($sales_purchase_orders, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($sales_purchase_orders, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"sales_purchase_orders" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(SalesPurchaseOrders::instance()->getObjectTypeId());
		$count = min(array($limit, count($sales_purchase_orders)));
		for ($i = 0; $i < $count; $i++){
			if (isset($sales_purchase_orders[$i])){
				$obj = $sales_purchase_orders[$i];
				if ($obj instanceof SalesPurchaseOrder){
					$info = $obj->getArrayInfo();
					$info['name'] = str_pad($info['number'], 7, '0', STR_PAD_LEFT);
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["sales_purchase_orders"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["sales_purchase_orders"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!SalesPurchaseOrder::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("sales_purchase_orders"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$sales_purchase_order_data = array_var($_POST, 'sales_purchase_order');
		
		if(!is_array($sales_purchase_order_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new SalesPurchaseOrder();
			$obj->setObjectTypeId(SalesPurchaseOrders::instance()->getObjectTypeId());
			tpl_assign('sales_purchase_order', $obj);

		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$sales_purchase_order = $this->save_sales_purchase_order($sales_purchase_order_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($sales_purchase_order, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add sales purchase order', $sales_purchase_order->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$sales_purchase_order = SalesPurchaseOrders::findById(get_id());
		if(!($sales_purchase_order instanceof SalesPurchaseOrder)) {
			flash_error(lang('sales_purchase_order dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$sales_purchase_order->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!SalesPurchaseOrder::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("sales_purchase_orders"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$sales_purchase_order_data = array_var($_POST, 'sales_purchase_order');
		
		if(!is_array($sales_purchase_order_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('sales_purchase_order', $sales_purchase_order);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$sales_purchase_order = $this->save_sales_purchase_order($sales_purchase_order_data, $member_ids, $sales_purchase_order);
				
				DB::commit();

				ApplicationLogs::createLog($sales_purchase_order, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit sales_purchase_order', $sales_purchase_order->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
/*	private function save_sales_purchase_order($sales_purchase_order_data, $member_ids, $sales_purchase_order = null) {
		// Save sales_purchase_order object
		$is_new = false;
		if (is_null($sales_purchase_order) || !$sales_purchase_order instanceof SalesPurchaseOrder) {
			$sales_purchase_order = new SalesPurchaseOrder();
			$is_new = true;
			$max_number = DB::executeAll("SELECT MAX(`number`) as `max_num` FROM ".TABLE_PREFIX."sales_purchase_orders");
			$sales_purchase_order_data['number'] = $max_number[0]['max_num'] + 1;
		}
		
		$sales_purchase_order_data['date'] = getDateValue($sales_purchase_order_data['date']);
		
		$sales_purchase_order->setFromAttributes($sales_purchase_order_data);
		$sales_purchase_order->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($sales_purchase_order, $member_ids);
		}
		$object_controller->add_subscribers($sales_purchase_order);
		$object_controller->link_to_new_object($sales_purchase_order);
		$object_controller->add_custom_properties($sales_purchase_order);
		
		return $sales_purchase_order;
	}*/


    function save_sales_purchase_order($sales_purchase_order_data, $member_ids, $sales_purchase_order = null) {
        try {
            DB::beginWork();
            //Create the purchase order
            $is_new = false;
            if (is_null($sales_purchase_order) || !$sales_purchase_order instanceof SalesPurchaseOrder) {
                $sales_purchase_order = new SalesPurchaseOrder();
                $is_new = true;
                $max_number = DB::executeAll("SELECT MAX(`number`) as `max_num` FROM ".TABLE_PREFIX."sales_purchase_orders");
                $sales_purchase_order_data['number'] = $max_number[0]['max_num'] + 1;
            }

            if ($sales_purchase_order_data['use_mysql_dates']) {
            	if (array_var($sales_purchase_order_data, 'date', '') != '') {
            		$event_ticket_data['date'] = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $sales_purchase_order_data['date']);
            	}
            } else {
           		$sales_purchase_order_data['date'] = getDateValue($sales_purchase_order_data['date']);
            }
            
            //Managed event
            $event_member = Members::findById(array_var($sales_purchase_order_data, 'event_id'));
            if ($event_member instanceof Member) {
                $member_ids[] = $event_member->getId();
                $member_ids = array_unique($member_ids);
            }
            
            // get the managed event content object id to save in related_object_id
            $member_ids = array_filter($member_ids);
            if (!$event_member instanceof Member && count($member_ids) > 0) {
            	$event_member = Members::findOne(array("conditions" => array(
            		"id IN (".implode(',',$member_ids).") AND object_type_id=?", ManagedEvents::instance()->getObjectTypeId()
            	)));
            }
            if ($event_member instanceof Member) {
            	$sales_purchase_order_data['related_object_id'] = $event_member->getObjectId();
            }
            // END Managed event

            //Ordered By - if requested_by_id not sent, try to find contact using "requested_by_email_address", if not found -> try to create the contact
            if (!array_var($sales_purchase_order_data, 'order_by') && array_var($sales_purchase_order_data, 'ordered_by_email')) {
                $ordered_by_email = array_var($sales_purchase_order_data, 'ordered_by_email');
                $ordered_by = Contacts::getByEmail($ordered_by_email);
                if (!$ordered_by instanceof Contact) {
                    $ordered_by = create_contact_from_sales_purchase_order_data($sales_purchase_order_data, $member_ids);
                }

                if ($ordered_by instanceof Contact) {
                    $sales_purchase_order_data['ordered_by'] = $ordered_by->getId();
                }
            }

            $sales_purchase_order->setFromAttributes($sales_purchase_order_data);
		    $sales_purchase_order->save();

            $object_controller = new ObjectController();
            if (!is_null($member_ids)) {
                $object_controller->add_to_members($sales_purchase_order, $member_ids);
            }
            $object_controller->add_subscribers($sales_purchase_order);
            $object_controller->link_to_new_object($sales_purchase_order);
            $object_controller->add_custom_properties($sales_purchase_order);


            //Event Tickets
            if($is_new && array_var($sales_purchase_order_data, 'event_tickets')){
                $event_tickets = array_var($sales_purchase_order_data, 'event_tickets');

                if(count($event_tickets) > 0){
                    $controller = new EventTicketController();

                    foreach ($event_tickets as $event_ticket) {

                        $event_ticket_data = array(
                            'ticket_type_id' => $event_ticket['ticket_type_id'],
                            'requested_by_id' => $sales_purchase_order->getOrderedBy(),
                            'requested_on' => $sales_purchase_order->getDate(),
                            'quantity' => $event_ticket['quantity'],
                            'use_mysql_dates' => true,
                        );

                        $controller->save_event_ticket($event_ticket_data, $member_ids);
                    }
                }
            }

            DB::commit();

        } catch (Exception $e) {
            DB::rollback();
            Logger::log("Error saving Sales Purchase Order: ".$e->getMessage()."\n".$e->getTraceAsString());
            throw $e;
        }
        return $sales_purchase_order;
    }
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$sales_purchase_order = SalesPurchaseOrders::findById(get_id());
		if(!($sales_purchase_order instanceof SalesPurchaseOrder)) {
			flash_error(lang('sales_purchase_order dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$sales_purchase_order->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('sales_purchase_order', $sales_purchase_order);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$sales_purchase_order = SalesPurchaseOrders::findById($id);
					if ($sales_purchase_order instanceof SalesPurchaseOrder && $sales_purchase_order->canDelete(logged_user())){
						try{
							DB::beginWork();
							$sales_purchase_order->trash();
							DB::commit();
							ApplicationLogs::createLog($sales_purchase_order, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed sales_purchase_order', $sales_purchase_order instanceof SalesPurchaseOrder ? clean($sales_purchase_order->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$sales_purchase_order = SalesPurchaseOrders::findById($id);
					if ($sales_purchase_order instanceof SalesPurchaseOrder && $sales_purchase_order->canEdit(logged_user())){
						try{
							DB::beginWork();
							$sales_purchase_order->archive();
							DB::commit();
							ApplicationLogs::createLog($sales_purchase_order, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}