<?php
Hook::register('status_dimension_timesheet');


function status_dimension_timesheet_after_add_to_members($object, &$members) {
	if ($object instanceof Timeslot) {

		$dim = Dimensions::findByCode('status_timesheet');
		
		Env::useHelper('functions', 'status_dimension_timesheet');
		classify_timeslot_in_invoiced_status_member($object);
		
		// if not classified in status dimension -> classify in default member if it is defined
		$default_value = DimensionOptions::instance()->getOptionValue($dim->getId(), 'default_value');
		if ($default_value > 0) {
			
			$ts_members = $object->getMembers();
			
			// if already classified in status dimension -> exit
			foreach ($ts_members as $member) {
				if ($member->getDimensionId() == $dim->getId()) return;
			}
			
			$default_member = Members::getMemberById($default_value);
			if ($default_member instanceof Member) {
				$object->addToMembers(array($default_member));
				$object->addToSharingTable();
			}
		}

	}
}



function status_dimension_timesheet_after_object_save($object, &$ret) {
	Env::useHelper('functions', 'status_dimension_timesheet');
	classify_timeslot_in_invoiced_status_member($object);
}

// add classification in approved member conditions when getting the expenses list to generate an invoice
function status_dimension_timesheet_expenses_for_invoices_modify_classification($params, &$member_ids) {
	$member_to_add = config_option('time_bulk_action_approved_member_id');
	if ($member_to_add && !in_array($member_to_add, $member_ids)) {
		$member_ids[] = $member_to_add;
	}
}

// add classification in approved member conditions when getting the timeslots list to generate an invoice
function status_dimension_timesheet_timeslots_for_invoices_modify_classification($params, &$member_ids) {
	$member_to_add = config_option('time_bulk_action_approved_member_id');
	if ($member_to_add && !in_array($member_to_add, $member_ids)) {
		$member_ids[] = $member_to_add;
	}
}

// classify in the invoicing status dimension after invoicing a payment receipt or after voiding an invoice
function status_dimension_timesheet_after_change_object_inv_status($params, &$ignored) {
	
	$object = array_var($params, 'object');
	$old_status = array_var($params, 'old_status');
	// Check if object has column 'invoicing_status', if the value of the column changed and
	// if the old value was empty before, it means that the object is new.
	if ($object instanceof ContentDataObject && in_array('invoicing_status', $object->manager()->getColumns())
			&& $object->getColumnValue('invoicing_status') != $old_status && $old_status != '') {
		
		$members_to_remove = array();
		$p_members = $object->getMembers();
		$dim = Dimensions::findByCode('status_timesheet');
		foreach ($p_members as $m) {
			if ($m->getDimensionId() == $dim->getId()) {
				$members_to_remove[] = $m->getId();
			}
		}
		
		if ($object->getColumnValue('invoicing_status') == 'invoiced') {
			$member_to_add = config_option('time_bulk_action_invoiced_member_id');
		} else if ($object->getColumnValue('invoicing_status') == 'pending') {
			$member_to_add = config_option('time_bulk_action_approved_member_id');
		}
		
		if (count($members_to_remove) > 0) {
			ObjectMembers::removeObjectFromMembers($object, logged_user(), $members_to_remove, $members_to_remove);
		}
		
		if ($member_to_add) {
			$m = Members::getMemberById($member_to_add);
			if ($m instanceof Member) {
				ObjectMembers::addObjectToMembers($object->getId(), array($m));
			}
		}
	}
	
}

