<?php
Hook::register("suppliers_in_reports");



/**
 * Add the supplier cp columns to the report view
 */
function suppliers_in_reports_custom_reports_fixed_additional_columns_def($args, &$results) {
	$ot = array_var($args, 'object_type');
	$field = array_var($args, 'field');

	if ($ot instanceof ObjectType && $ot->getName()=='payment') {
		
		$supplier_ot = ObjectTypes::findByName('supplier');
		$supplier_cps = CustomProperties::getAllCustomPropertiesByObjectType($supplier_ot->getId());
		
		foreach ($supplier_cps as $cp) {
			/* @var $cp CustomProperty */
			$f = 'supplier_cp_'.$cp->getId();
			if ($f == $field) {
				$cp_name = $cp->getName();
				if ($cp->getCode() != '') {
					$cp_lang = Localization::instance()->lang($cp->getCode());
					if ($cp_lang != '') {
						$cp_name = $cp_lang;
					}
				}
				$supplier_ot_name = $supplier_ot->getObjectTypeName();
				$name = "$supplier_ot_name - $cp_name";
				
				$results['columns']['names'][$field] = $name;
				$results['columns']['order'][] = $field;
			}
		}

	}
}

/**
 * Add the supplier cp columns to the add/edit custom report form
 */
function suppliers_in_reports_custom_reports_additional_columns($params, &$fields) {
	
	$ot = array_var($params, 'object_type');
	
	if ($ot->getName() == 'payment') {
		
		$supplier_ot = ObjectTypes::findByName('supplier');
		$supplier_cps = CustomProperties::getAllCustomPropertiesByObjectType($supplier_ot->getId());
		$supplier_ot_name = $supplier_ot->getObjectTypeName();
		
		foreach ($supplier_cps as $cp) {
			/* @var $cp CustomProperty */
			$cp_name = $cp->getName();
			if ($cp->getCode() != '') {
				$cp_lang = Localization::instance()->lang($cp->getCode());
				if ($cp_lang != '') {
					$cp_name = $cp_lang;
				}
			}
			$name = "$supplier_ot_name - $cp_name";
			
			$fields[] = array('id' => 'supplier_cp_'.$cp->getId(), 'name' => $name, 'type' => 'text');
		}
	}
}

/**
 * Add the supplier cp column values to the custom report result
 */
function suppliers_in_reports_custom_reports_override_column_format($parameters, &$value) {
	if (!Plugins::instance()->isActivePlugin('expenses')) return;
	
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($object instanceof Payment && str_starts_with($field, 'supplier_cp_')) {
		$supplier_ot = ObjectTypes::findByName('supplier');
		
		$cp_id = str_replace('supplier_cp_', '', $field);
		$supplier_members = get_object_members_by_type($object, $supplier_ot->getId());
		$supplier_member = array_var($supplier_members, 0);
		
		if ($supplier_member instanceof Member) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($supplier_member->getObjectId(), $cp_id);
			if ($cp_val instanceof CustomPropertyValue) {
				$value = trim($cp_val->getValue());
			}
		}
	}
}

