<?php
/**
 * Returns the list of files classified under $member_id, excluding the files that are classified in its submembers
 * @param integer $member_id
 * @return array of ProjectFile
 */
function get_files_list($member_id) {
	$dimension_id = config_option('webdav_dimension');
	
	$file_params = array(
		'ignore_context' => true,
		'limit' => 1000,
		'show_only_member_objects' => true,
	);
	if ($member_id > 0) {
		$file_params['member_ids'] = array($member_id);
	} else {
		$file_params['extra_conditions'] = "AND NOT EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
				INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
				WHERE m.dimension_id=$dimension_id AND om.object_id=o.id AND is_optimization=0)";
	}
	
	$all_files = ProjectFiles::instance()->listing($file_params);
	
	return $all_files->objects;
	
}

/**
 * Builds sql conditions from a member path (given in names)
 * @param array $parent_names
 * @return string The sql conditions to make the query 
 */
function build_file_path_conditions($parent_names) {
	$dimension_id = config_option('webdav_dimension');
	
	$member = null;
	$tmp_member = null;
	$parent_id = 0;
	
	foreach ($parent_names as $pname) {
		// get parent member
		$tmp_member = Members::findOne(array('conditions' => array("name=? AND dimension_id=? AND parent_member_id=?", $pname, $dimension_id, $parent_id)));
	
		// set parent id for next iteration
		if ($tmp_member instanceof Member) {
			$parent_id = $tmp_member->getId();
			$member = $tmp_member;
		} else {
			break;
		}
	}
	
	if ($member instanceof Member) {
		$cond = " AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
				WHERE om.member_id=".$member->getId()." AND om.object_id=o.id AND is_optimization=0)";
	} else {
		$cond = " AND NOT EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om
				INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
					WHERE m.dimension_id=$dimension_id AND om.object_id=o.id AND is_optimization=0)";
	}
	return $cond;
}

/**
 * Returns the ProjectFile object that is classified in the member given by the $path passed by parameter.
 * @param string $path - The webdav filesystem path to the file
 * @param Contact $user - the logged user
 * @return ProjectFile
 */
function get_file_from_path($path, $user) {
	$dimension_id = config_option('webdav_dimension');
	
	// explode parent path
	$parent_names = array_filter(explode (WEBDAV_PATH_SEPARATOR, $path));
	// get file name
	$file_name = end($parent_names);
	// remove file name
	array_splice($parent_names, count($parent_names)-1);
	
	$cond = build_file_path_conditions($parent_names);
	
	$file = ProjectFiles::findOne(array("conditions" => "name='$file_name' $cond"));
	if ($file instanceof ProjectFile && $file->canView($user)) {
		return $file;
	} else {
		return null;
	}
}