<?php

class WebDAVDirectory extends Sabre\DAV\FS\Directory {

  private $myPath;
  private $id;
  
  private $file_sizes;
  private $file_types;
  
  function __construct($myPath, $mem_id=0) {
  	
  	if ($myPath == "" || $myPath == WEBDAV_PATH_SEPARATOR){
    	$this->myPath = "";
    	$this->id = 0;
  	} else {
  		$this->myPath = urldecode($myPath);
    	
    	if (is_numeric($mem_id) && $mem_id>0) {
    		$this->id = $mem_id;
    	}
   		$member = $this->getMember();
    	
	    if(!$member){
	    	throw new Sabre\DAV\Exception\NotFound("Invalid Feng path.");
	    	$this->myPath = "";
	    } else {
	    	$this->id = $member->getId();
	    	$path = $member->getPath();
	    	$this->myPath = ($path=="" ? "" : WEBDAV_PATH_SEPARATOR.$path) . WEBDAV_PATH_SEPARATOR .$member->getName();
	    }
    }
  }
  
  /**
   * Get Member object from path or id (if this->id is valid)
   * If root return true, if invalid path return false.
   */
  private function getMember(){
  	if (is_numeric($this->id) && $this->id > 0) {
  		return Members::findById($this->id);
  	}
  	
  	$tmp = $this->myPath;
  	
    if($tmp == "" || $tmp == WEBDAV_PATH_SEPARATOR){
    	return true;
    } else {
    	$user = logged_user();
    	if(!$user instanceof Contact) {
    		return false;
    	}
    	
    	// get member from directory path
    	$member = get_member_from_path($tmp, $user);
    	return $member;
	}	
  }
  
  private function getMemberId(){
  	
  	if (is_numeric($this->id) && $this->id > 0) {
  		return $this->id;
  	} else {
  		return 0;
	  	$tmp = $this->getMember();
	  	return ($tmp instanceof Member ? $tmp->getId() : false);
  	}
  }
  
  /**
   * Returns the first level submembers of this member as directories 
   * and the files classified under this member (excluding the files classified in submembers) 
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Directory::getChildren()
   */
  function getChildren() {
  	set_time_limit(0);
  	ini_set('memory_limit', '512M');
  	
  	$user = logged_user();

  	$parent_id = $this->getMemberId();
  	
  	$children = array();
	
  	// get all members to build WebDAVDirectory for each one
 	$all_members = get_member_list($parent_id, $user);
 	foreach ($all_members as $mem) {
 		if ($mem instanceof Member) {
 			$children[] = $this->getChild($mem);
 		}
 	}
    
 	$file_ids = array();
 	
 	// get all files classified in this member to build WebDAVFile for each one
 	$all_files = get_files_list($parent_id);
 	if(is_array($all_files)){
 		foreach($all_files as $f) {
 			if ($f instanceof ProjectFile) {
 				$file_ids[] = $f->getId();
 			}
 		}
 		
 		// get attributes for all files
 		if (count($file_ids) > 0) {
 			$all_revision_data = DB::executeAll("SELECT file_id, filesize, type_string FROM ".TABLE_PREFIX."project_file_revisions f
    		WHERE f.file_id IN (".implode(',',$file_ids).") AND
    		f.revision_number=(SELECT max(f2.revision_number) FROM ".TABLE_PREFIX."project_file_revisions f2 WHERE f2.file_id=f.file_id)");
 			 
 			foreach ($all_revision_data as $rev) {
 				$this->file_sizes[$rev['file_id']] = $rev['filesize'];
 				$this->file_types[$rev['file_id']] = $rev['type_string'];
 			}
 		}
 		
		foreach($all_files as $f) {
			if ($f instanceof ProjectFile) {
				$children[] = $this->getChild($f);
			}
	    }
    }
 	
    // return the mixed array of WebDAVDirectory and WebDAVFile
	return $children;
  }

  /**
   * Creates a WebDAVDirectory or WebDAVFile from the given $object
   * If object is a string then a member is searched for the $path/$object, if not found then a file is searched for the $path/$object
   * @param $object - Source data to build a webdav object (can be a Member, ProjectFile or string)
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Directory::getChild()
   */
  function getChild($object) {

 	  if ($object instanceof Member){
 	  	
 	  	$myPath = $this->myPath . WEBDAV_PATH_SEPARATOR . $object->getName();
        return new WebDAVDirectory($myPath, $object->getId());
        
      } else if ($object instanceof ProjectFile) {
      	
      	$attributtes = array(
      		'size' => $this->file_sizes[$object->getId()], 
      		'last_modified' => $object->getUpdatedOn()->getTimestamp(),
      		'mime_type' => $this->file_types[$object->getId()],
      	);
      	
      	$myPath = $this->myPath . WEBDAV_PATH_SEPARATOR . $object->getFilename();
        return new WebDAVFile($myPath, $object->getId(), $attributtes);
          
      } else {
      		$mem = get_member_from_path($this->myPath . WEBDAV_PATH_SEPARATOR . $object, logged_user());
      		if ($mem instanceof Member) {
      			return new WebDAVDirectory($this->myPath . WEBDAV_PATH_SEPARATOR . $mem->getName(), $mem->getId());
      		} else {
      			$file = get_file_from_path($this->myPath . WEBDAV_PATH_SEPARATOR . $object, logged_user());
      			if ($file instanceof ProjectFile) {
      				$filesize = array_var($this->file_sizes, $file->getId());
      				if (!$filesize) {
      					$filesize = $file->getFileSize();
      				}
      				$attributtes = array(
      					'size' => $filesize,
      					'last_modified' => $file->getUpdatedOn()->getTimestamp(),
      					'mime_type' => $this->file_types[$file->getId()],
      				);
      				return new WebDAVFile($this->myPath . WEBDAV_PATH_SEPARATOR . $file->getFilename(), $file->getId(), $attributtes);
      			}
      		}


      }

	  // We have to throw a NotFound exception if the file didn't exist
	  throw new Sabre\DAV\Exception\NotFound('The file with name: ' . $object . ' could not be found');
  }
  

  /**
   * Returns the member name, ensure that is encoded in UTF-8
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::getName()
   */
  function getName() {
  	return basename_safe($this->myPath);
  }
  
  /**
   * Get the last modification date of the folder (member)
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::getLastModified()
   */
  function getLastModified(){
  	
  	$ws = $this->getMember();
  	if($ws instanceof Member){
  		return get_member_last_update_time($ws);
  	} else {
  		return false;
  	}
  }
  
  /**
   * Uploads a file with name = $name in the member given by this->path
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Directory::createFile()
   */
  function createFile($name, $data = null){

  	$user = logged_user();
  	if (!$user instanceof Contact) return;
  	
  	$members = array();
  	$m = $this->getMember();
  	if ($m instanceof Member) {
  		$members[] = ($m);
  	}
  	
  	// check write permissions
  	if (!can_write($user, $members, ProjectFiles::instance()->getObjectTypeId())) {
  		log_wd("User ".$user->getId()." cannot add files in ".$this->myPath);
  		throw new Sabre\DAV\Exception\Forbidden ("Cannot add files in ".$this->myPath);
  	}
  	
  	// get file contents from request
  	$content = stream_get_contents($data);
  	if (strlen($content) == 0) return;

  	// temp data to upload the file
  	$userid = $user ? $user->getId() : "0";
  	$tempFileName = ROOT ."/tmp/". $userid ."x". gen_id();
  	$fh = fopen($tempFileName, 'w') or die("Can't open file");
  	fwrite($fh, $content);
  	fclose($fh);
  	
  	$is_new_file = false;
  	
  	// if file exists in this path then update it, else create a new file in the current member 
  	$file = get_file_from_path($this->myPath. WEBDAV_PATH_SEPARATOR .$name, logged_user());
  	if (!$file instanceof ProjectFile) {
	  	$file = new ProjectFile();
	  	$file->setFilename($name);
	  	$file->setIsVisible(true);
	  	$file->setCreatedById($user->getId());
	  	$file->save();
	
		$object_controller = new ObjectController();
		$object_controller->add_to_members($file, array($this->getMemberId()));
		
		$is_new_file = true;
  	} else {
  		if (!$file->isCheckedOut() || $file->getCheckedOutById() == logged_user()->getId()) {
			// save the file to update the "updated_on" and "updated_by_id" attributes
	  		$file->save();
  		} else {
  			log_wd("Cannot edit file '$name', it is blocked by ".$file->getCheckedOutById());
  			throw new Sabre\DAV\Exception\Forbidden ("Cannot edit file '$name', it is blocked by ".$file->getCheckedOutById());
  		}
  	}
  	
  	// get mime type from file name
  	$mime_types_manager = new Mime_Types();
  	$mime_type = $mime_types_manager->get_type($name);
  	
  	// build file revision data
  	$fileToSave = array(
  			"name" => $name,
  			"type" => $mime_type,
  			"tmp_name" => $tempFileName,
  			"error" => 0,
  			"size" => filesize($tempFileName)
  	);
  	
  	// upload file revision
  	try {
	  	$revision = $file->handleUploadedFile($fileToSave, true, 'Uploaded using webdav');
	  	$revision->setCreatedById($user->getId());
	  	$revision->save();
	  	ApplicationLogs::createLog($file, $is_new_file ? ApplicationLogs::ACTION_ADD : ApplicationLogs::ACTION_EDIT);
  	} catch (Exception $e) {
  		log_wd($e->getMessage());
  		log_wd($e->getTraceAsString());
  	}
  } 
  
  /**
   * Returns the quota information
   *
   * This method MUST return an array with 2 values, the first being the total used space,
   * the second the available space (in bytes)
   */
  function getQuotaInfo() {
  
  	return array(1024*1024*1024*1024, 1024*1024*1024*1024);
  
  }
  
  /**
   * Creates a new member under current member
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Directory::createDirectory()
   */
  function createDirectory($name) {
	$user = logged_user();
  	if ($user instanceof Contact && can_manage_dimension_members($user)) {
  		$current_member = $this->getMember();
	  	create_webdav_member($name, $current_member);
  	}
  }
  
  /**
   * Changes member name if user has permissions
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::setName()
   */
  function setName($newName){
	$user = logged_user();
  	if ($user instanceof Contact && can_manage_dimension_members($user)) {
	  	$current_member = $this->getMember();
	  	if ($current_member instanceof Member) {
	  		$current_member->setName($newName);
	  		$current_member->save();
		  	return true;
	  	}
  	} else {
  		throw new Sabre\DAV\Exception\Forbidden ("Cannot rename Feng Office folders through WebDAV");
  	}  	
  }
  
  /**
   * Override delete funcion to prevent deletion.
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Directory::delete()
   */
  function delete(){
  	log_wd("Trying to delete webdav folder '".$this->myPath."' - user: ".(logged_user() instanceof Contact ? logged_user()->getId() : ''));
  	die();
  	//throw new Sabre\DAV\Exception\Forbidden ("Cannot delete Feng Office folders through WebDAV");	
  }
}
