<?php
/**
 * TimeRegistration controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TimeRegistrationController extends ApplicationController {

	var $plugin_name = "work_time_registration";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("TimeRegistrationManager.js", $this->plugin_name);
		ajx_current("panel", "time-registration-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function view() {
		
		$object = NewsDetails::findById(get_id());
		if (!$object instanceof NewsDetail) {
			flash_error(lang('news detail dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$object->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		tpl_assign('object', $object);
	}
	
	function list_all() {
		
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort','date');
		$order_dir = array_var($_GET,'dir','desc');
		$dim_order = null;
		$cp_order = null;
		
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		
		$action = array_var($_GET, 'action');
		//Resolve actions to perform
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$join_params = array();
		
		if ($order == 'date' || $order == 'hour' || $order == 'memPath' ) {
			$order = array('e.date', 'e.hour', 'e.minute');
		} else if ($order == 'person_name') {
			$order = array('jt.surname', 'jt.first_name');
			$join_params = array(
				'join_type' => 'inner',
				'table' => TABLE_PREFIX.'contacts',
				'jt_field' => 'object_id',
				'e_field' => 'person_id',
			);
		} else if ($order == 'hierarchy_name') {
			$order = array('jt.name');
			$join_params = array(
				'join_type' => 'inner',
				'table' => TABLE_PREFIX.'person_hierarchies',
				'jt_field' => 'id',
				'j_sub_q' => 'SELECT c.hierarchy_id FROM '.TABLE_PREFIX.'contacts c WHERE c.object_id=e.person_id',
			);
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		}
		$extra_conditions = "";
		
		// date filter
		$date_filter = array_var($_REQUEST, 'date_filter');
		if ($date_filter != null) {
			$date_obj = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), $date_filter);
			if ($date_obj instanceof DateTimeValue) {
				$extra_conditions .= " AND e.`date`='".$date_obj->format('Y-m-d')."'";
			}
		} else {
			$date_obj = DateTimeValueLib::now();
			$extra_conditions .= " AND e.`date`='".$date_obj->format('Y-m-d')."'";
		}
		
		$ws_ot = ObjectTypes::findByName('workspace');
		$selected_workspace = null;
		$active_context = active_context();
		foreach ($active_context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$selected_workspace = $selection;
			}
		}
		
		// status filter
		$summary_status_filter = array_var($_REQUEST, 'status_filter');
		
		if ($summary_status_filter != 'all') {
			if ($summary_status_filter == 'approved') $status_name = 'approved';
			else $status_name = 'pending';
			$ws_ot_id = $ws_ot->getId();
			
			$extra_conditions .= " AND '$status_name' = (SELECT ns.status FROM ".TABLE_PREFIX."news_summary ns WHERE `ns`.`date`='".$date_obj->format('Y-m-d')."' 
				AND `ns`.`internal_destiny`=(SELECT oms.member_id FROM ".TABLE_PREFIX."object_members oms WHERE oms.object_id=e.object_id AND oms.is_optimization=0
				AND $ws_ot_id=(select m.object_type_id from ".TABLE_PREFIX."members m where m.id=oms.member_id)))";
		}
		
		// status filter
		$news_type_filter = array_var($_REQUEST, 'type_filter');
		if ($news_type_filter != 'all') {
			$present = ($news_type_filter == 'present');
			$extra_conditions .= " AND news_type_id ".($present ? "" : "NOT")." IN (SELECT id FROM ".TABLE_PREFIX."news_types WHERE enabled=1 AND is_present_type=1)";
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		$result = NewsDetails::instance()->listing(array(
			"select_columns" => array("e.*", "o.*"),
			"join_params" => $join_params,
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$regs = $result->objects;
		$object = $this->prepareObject($regs, $start, $limit, $result->total);
		
		$object['date_filter'] = $date_obj->format(user_config_option('date_format'));
		
		ajx_extra_data($object);
		tpl_assign("listing", $object);
	}
	
	private function sortByHierarchy(&$object, $dir="asc") {
		$regs = $object["regs"];
		$ordered = array();
		$final = array();
		foreach ($regs as $reg) {
			$ordered[$reg['hierarchy_name']."_".$reg["person_name"]] = $reg;
		}
		ksort($ordered);
		if (strtolower($dir) == "desc") {
			$ordered = array_reverse($ordered);
		}
		foreach ($ordered as $reg) {
			$final[] = $reg;
		}
		$object["regs"] = $final;
		
		return $object;
	}
	
	private function sortByPersonName(&$object, $dir="asc") {
		$regs = $object["regs"];
		$ordered = array();
		$final = array();
		foreach ($regs as $reg) {
			$ordered[$reg["person_name"] . "_" . $reg["date"]] = $reg;
		}
		ksort($ordered);
		if (strtolower($dir) == "desc") {
			$ordered = array_reverse($ordered);
		}
		foreach ($ordered as $reg) {
			$final[] = $reg;
		}
		$object["regs"] = $final;
	
		return $object;
	}
	
	private function prepareObject($regs, $start, $limit, $total) {
		$object = array(
			"totalCount" => $total,
			"start" => $start,
			"regs" => array()
		);
		$count = min(array($limit, count($regs)));
		for ($i = 0; $i < $count; $i++){
			if (isset($regs[$i])){
				$reg = $regs[$i];
				if ($reg instanceof NewsDetail){
					$info = $reg->getArrayInfo(true);
					$info['date'] = $reg->getDate() instanceof DateTimeValue ? format_date($reg->getDate(), null, 0) : '';
					$c = Contacts::findById($reg->getPersonId());
					$info['person_name'] = $c instanceof Contact ? $c->getSurname() . ", " . $c->getFirstName() : '';
					$info['img_url'] = $c instanceof Contact ? $c->getPictureUrl() : '';
					$info['memPath'] = json_encode($reg->getMembersIdsToDisplayPath());
					$info['hierarchy_name'] = "";
					if ($c instanceof Contact) {
						$person_hierarchy = PersonHierarchies::findById($c->getColumnValue("hierarchy_id"));
						if ($person_hierarchy instanceof PersonHierarchy) {
							/*$hg = PersonHierarchyGroups::findById($person_hierarchy->getHierarchyGroupId());
							$info['hierarchy_name'] = ($hg instanceof PersonHierarchyGroup ? $hg->getName().", " : "") . $person_hierarchy->getName();*/
							$info['hierarchy_name'] = $person_hierarchy->getName();
						}
					}
					$nt = NewsTypes::instance()->findById($reg->getNewsTypeId());
					if ($nt instanceof NewsType) {
						$info['is_present_type'] = $nt->getIsPresentType() ? "1" : "";
					}
					
					$object["regs"][] = $info;
    			}
			}
		}
		return $object;
	}
	
	
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!NewsDetail::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else trim($notAllowedMember) == "" ? flash_error(lang('you must select where to keep', lang('the news detail'))) : flash_error(lang('no context permissions to add',lang("news_detail"), $notAllowedMember));
			ajx_current("empty");
			return;
		}
		
		$reg_data = array_var($_POST, 'detail');
		
		if(!is_array($reg_data)) {
			
			$det = new NewsDetail();
			$date = getDateValue(array_var($_GET, 'date'));
			$det->setDate($date);
			
			tpl_assign('detail', $det);
			tpl_assign('needs_observations', false);
			
		} else {
			// form submitted
			try {
				
				$date = getDateValue(array_var($reg_data, 'date', ''));
				$contact_id = array_var($reg_data, 'person_id');
				$c = Contacts::findById($contact_id);
				
				if (!config_option('allow_multiple_news_details_in_same_day')) {
					$tmp_det = $date instanceof DateTimeValue ? NewsDetails::findOne(array('conditions' => "date = '".$date->toMySQL()."' AND person_id = '$contact_id' AND trashed_by_id=0 AND archived_by_id=0")) : null;
					if ($tmp_det instanceof NewsDetail) {
						throw new Exception(lang('news detail already exists for date and person', $date->format('Y-m-d'), $c->getObjectName()));
					}
				}
				
				DB::beginWork();
				
				//$member_ids = json_decode(array_var($_POST, 'members'));
				$member_ids = array();
				if ($c instanceof Contact) $member_ids[] = $c->getColumnValue('internal_destiny');
				
				$reg = $this->save_news_detail($reg_data, $member_ids);
				
				DB::commit();
				
				ApplicationLogs::createLog($reg, ApplicationLogs::ACTION_ADD);

				flash_success(lang('success add new news detail', $reg->getObjectName()));
				ajx_current("back");
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				//flash_error($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	private function save_news_detail($reg_data, $member_ids, $reg = null) {
		// Save object
		$is_new = false;
		if (is_null($reg) || !$reg instanceof NewsDetail) {
			$reg = new NewsDetail();
			$is_new = true;
		}
		
		if (!isset($reg_data['name'])) $reg_data['name'] = array_var($reg_data, 'comment');
		
		$date = getDateValue(array_var($reg_data, 'date', ''));
		$time = getTimeValue(array_var($reg_data, 'hh_mm', ''));
		
		$c = Contacts::findById(array_var($reg_data, 'person_id'));
		if (!$c instanceof Contact) {
			throw new Exception(lang('you have to select a person'));
		}
		if (!$date instanceof DateTimeValue || $time == null) {
			throw new Exception(lang('you have to enter date and time'));
		}
		if (!is_numeric(array_var($reg_data, 'news_type_id'))) {
			throw new Exception(lang('you have to select news type'));
		}
		
		$reg_data['date'] = $date instanceof DateTimeValue ? $date->toMySQL() : EMPTY_DATETIME;
		$reg_data['hour'] = $time['hours'];
		$reg_data['minute'] = $time['mins'];
		
		$reg->setFromAttributes($reg_data);
		$reg->buildObjectName();
		$reg->save();
		
		$reg->addToSearchableObjects($is_new);
		
		$object_controller = new ObjectController();
		$object_controller->add_to_members($reg, $member_ids);
		$object_controller->add_subscribers($reg);
		$object_controller->link_to_new_object($reg);
		$object_controller->add_custom_properties($reg);
		// TODO: guardar en el historico de registros los cambios realizados
		
		$this->recalculate_summary($date, $member_ids);
		
		if ($this->check_absences_to_generate_alerts($reg)) {
			$nt = NewsTypes::instance()->findById($reg->getNewsTypeId());
			$contact = Contacts::instance()->findById($reg->getPersonId());
			flash_error(lang('person reached max number of absences of type', $contact->getObjectName(), $nt->getDescription()));
		}
		
		return $reg;
	}
	
	function edit() {
		$this->setTemplate('add');
		
		$reg = NewsDetails::findById(get_id());
		if(!($reg instanceof NewsDetail)) {
			flash_error(lang('news detail dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$reg->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$reg_data = array_var($_POST, 'detail');
		
		if(!is_array($reg_data)) {
			tpl_assign('detail', $reg);
			
			$contact = Contacts::findById($reg->getPersonId());
			$ws_id = $contact instanceof Contact ? $contact->getColumnValue('internal_destiny') : 0;
			$summary = null;
			if ($ws_id > 0) {
				$summary = NewsSummaries::findOne(array('conditions' => "`internal_destiny` = ".$ws_id." AND `date`='".format_date($reg->getDate(), DATE_MYSQL, 0)."'"));
			}
			tpl_assign('needs_observations', $summary instanceof NewsSummary && $summary->getStatus() == 'approved');
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				//$member_ids = json_decode(array_var($_POST, 'members'));
				$member_ids = array();
				$c = Contacts::findById(array_var($reg_data, 'person_id'));
				if ($c instanceof Contact) $member_ids[] = $c->getColumnValue('internal_destiny');
				
				$reg->addToHistory(logged_user(), 'edit');
				
				$reg = $this->save_news_detail($reg_data, $member_ids, $reg);
				
				DB::commit();

				ApplicationLogs::createLog($reg, ApplicationLogs::ACTION_EDIT);
				
				flash_success(lang('success edit new news detail', $reg->getObjectName()));
				ajx_current("back");
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				//flash_error($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$detail = NewsDetails::findById($id);
					if (isset($detail) && $detail->canDelete(logged_user())){
						try{
							DB::beginWork();
							$detail->addToHistory(logged_user(), 'delete');
							$detail->trash();
							$contact = Contacts::findById($detail->getPersonId());
							$this->recalculate_summary($detail->getDate(), array($contact->getColumnValue('internal_destiny')));
							DB::commit();
							ApplicationLogs::createLog($detail, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
					ajx_current('reload');
				}
				break;
			
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function check_needs_observations() {
		ajx_current("empty");

		if (array_var($_REQUEST, 'date')) {
			$date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', array_var($_REQUEST, 'date'));
		} else {
			$date = DateTimeValueLib::now();
		}
		$mem = Members::findById(array_var($_REQUEST, 'idest'));
		
		$needs_observations = false;
		if ($date instanceof DateTimeValue && $mem instanceof Member) {
			$summary = NewsSummaries::findOne(array('conditions' => "date = '".$date->toMySQL()."' AND internal_destiny=".$mem->getId()));
			$needs_observations = $summary instanceof NewsSummary && $summary->getStatus() == 'approved';
		}
		
		ajx_extra_data(array('needs_observations' => $needs_observations));
	}
	
	
	private function check_absences_to_generate_alerts(NewsDetail $detail) {
		$type = NewsTypes::instance()->findById($detail->getNewsTypeId());
		if ($type instanceof NewsType && $type->getGeneratesAbscence() && $type->getDaysToGenerateAbscence() > 0) {
			$count = 0;
			$date = $detail->getDate();
			while ($count < $type->getDaysToGenerateAbscence()) {
				$count++;
				$date = $date->add('d', -1);
				$tmp_det = NewsDetails::instance()->findOne(array(
					'conditions' => "news_type_id=".$detail->getNewsTypeId()." AND `date`='".$date->format('Y-m-d')."' AND person_id=".$detail->getPersonId())
				);
				if (!$tmp_det instanceof NewsDetail) {
					return false;
				}
			}
			if ($count >= $type->getDaysToGenerateAbscence()) {
				return true;
			}
		}
		return false;
	}
	
	
	function load_news_types() {
		ajx_current("empty");
		
		$news_types = NewsTypes::findAll();
		$news_types_info = array();
		foreach($news_types as $nt) {
			$news_types_info[$nt->getId()] = array('id' => $nt->getId(), 'name' => clean($nt->getDescription()));
		}
		
		$hierarchies = PersonHierarchies::findAll();
		$hierarchies_info = array();
		foreach ($hierarchies as $h) {
			$hierarchies_info[$h->getId()] = array('id' => $h->getId(), 'name' => clean($h->getName()));
		}
		
		ajx_extra_data(array('types' => $news_types_info, 'hierarchies' => $hierarchies_info));
	}
	
	
	function reload_news_type_selector() {
		ajx_current("empty");
		$contact_id = array_var($_REQUEST, 'p');
		$selected = array_var($_REQUEST, 'sel');
		$genid = array_var($_REQUEST, 'genid');
		
		$extra_cond = "";
		$contact = Contacts::findById($contact_id);
		if ($contact instanceof Contact) {
			$hid = $contact->getColumnValue('hierarchy_id');
			$h = PersonHierarchies::findById($hid);
			if ($h instanceof PersonHierarchy && $h->getHierarchyGroupId() > 0) {
				$extra_cond = "AND (hierarchy_group_id = ".$h->getHierarchyGroupId()." OR hierarchy_group_id = 0)";
			}
		}
		
		// get only options for the person's hierarchy group
		$html = news_type_select_box('detail[news_type_id]', $selected, array('style' => 'margin:0; height:22px;', 'id' => $genid.'newsDetailType'), false, $extra_cond);
		
		// get workspace info to show
		$dest_html = "";
		$m = Members::findById($contact->getColumnValue('internal_destiny'));
		if ($m instanceof Member) {
			$dest_html = '<span class="mname og-wsname-color-'.$m->getColor().'">'.$m->getName().'</span>';
			ajx_extra_data(array('html' => $html, 'dest' => $dest_html, 'dest_id' => $m->getId()));
		} else {
			ajx_extra_data(array('html' => $html, 'dest' => "", 'dest_id' => 0));
		}
		
	}
	
	
	function summary_panel() {
		ajx_current('empty');
		
		$context = active_context();
		$ws_ot = ObjectTypes::findByName('workspace');
		$workspace = null;
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$workspace = $selection;
				break;
			}
		}
		if ($workspace instanceof Member) {
			tpl_assign('workspace', $workspace);
			$view = 'summary_panel';
		} else {
			tpl_assign('workspace', null);
			$view = 'summary_panel_all';
		}
		
		if (array_var($_REQUEST, 'summary_date')) {
			$summary_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_REQUEST, 'summary_date'));
		} else {
			$summary_date = DateTimeValueLib::now();
		}
		tpl_assign('summary_date', $summary_date);
		tpl_assign('for_widget', array_var($_REQUEST, 'for_widget'));
		
		// get summary_data
		if ($view == 'summary_panel') {
			
			$extra_conditions = "AND `date`='".$summary_date->format('Y-m-d')."'";
			if ($workspace instanceof Member) {
				$extra_conditions .= " AND internal_destiny='" . $workspace->getId() ."'";
			}
			$result = NewsSummaries::instance()->listing(array(
				"extra_conditions" => $extra_conditions,
				"count_results" => false,
			));
			$summaries = $result->objects;
			
			$summary = null;
			if (count($summaries) > 0) {
				$summary = $summaries[0];
			}
			
			if ($summary instanceof NewsSummary) {
				
				$data = $this->calculate_summary_data_for_view($summary, $workspace, $summary_date);
				
				$totals_by_hierarchy = array_var($data, 'totals_by_hierarchy');
				$totals_by_news_type = array_var($data, 'totals_by_news_type');
				
				tpl_assign('totals_by_hierarchy', $totals_by_hierarchy);
				tpl_assign('totals_by_news_type', $totals_by_news_type);
				tpl_assign('summary', $summary);
			}
			
		} else {
			
			$all_member_ids = Members::findAll(array('id' => true));
			$extra_conditions = "AND `date`='".$summary_date->format('Y-m-d')."' AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om WHERE 
				om.member_id IN (SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.depth=1 AND m.object_type_id=".$ws_ot->getId().") AND om.object_id=o.id AND om.is_optimization=0)";
			
			$res_sum = NewsSummaries::instance()->listing(array(
				"extra_conditions" => $extra_conditions,
				"count_results" => false,
			));
			
			$totals_by_news_type = array();
			$totals_by_hierarchy = array();
			
			$summaries = $res_sum->objects;
			foreach ($summaries as $summary) {
				
				$workspace = Members::findById($summary->getInternalDestiny());
				$data = $this->calculate_summary_data_for_view($summary, $workspace, $summary_date);
				
				$ths = array_var($data, 'totals_by_hierarchy');
				$tns = array_var($data, 'totals_by_news_type');
				
				foreach ($ths as $k => &$th) {
					if (!isset($totals_by_hierarchy[$k])) {
						$totals_by_hierarchy[$k] = $th;
					} else {
						$totals_by_hierarchy[$k]->setTotal($totals_by_hierarchy[$k]->getTotal() + $th->getTotal());
					}
				}
				
				foreach ($tns as $k => $tn) {
					if (!isset($totals_by_news_type[$k])) $totals_by_news_type[$k] = 0;
					
					$totals_by_news_type[$k] = $totals_by_news_type[$k] + $tn;
				}
			}
			
			tpl_assign('totals_by_hierarchy', $totals_by_hierarchy);
			tpl_assign('totals_by_news_type', $totals_by_news_type);
		}
		
		ajx_extra_data(array('html' => tpl_fetch(get_template_path($view, 'time_registration', $this->plugin_name))));
	}


	private function calculate_summary_data_for_view($summary, $workspace, $summary_date) {
		
		$all_member_ids = $workspace->getAllChildrenIds(true);
		$all_member_ids[] = $workspace->getId();
		
		$totals_by_hierarchy = NewsSummaryTotalPersonss::findAll(array('conditions' => 'summary_id = '.$summary->getId()));
		$tmp = array();
		foreach ($totals_by_hierarchy as &$t) {
				
			$hs = PersonHierarchies::findAll(array('id' => true, 'conditions' => "enabled=1 and hierarchy_group_id=".$t->getHierarchyGroupId()));
			if (count($hs) == 0) $hs[] = 0;
			$contact_ids = Contacts::findAll(array('id' => true, 'conditions' => 'user_type=0 AND disabled=0 AND is_active=1 AND hierarchy_id IN ('.implode(',',$hs).') AND internal_destiny IN ('.implode(',', $all_member_ids).')'));
			if (count($contact_ids) == 0) $contact_ids[] = 0;
			
			//$absences = NewsDetails::count("`date`='".$summary_date->format('Y-m-d')."' AND person_id IN (".implode(',',$contact_ids).") AND trashed_by_id=0 AND archived_by_id=0");
			$sql_where = "`date`='".$summary_date->format('Y-m-d')."' AND person_id IN (".implode(',',$contact_ids).") AND trashed_by_id=0 AND archived_by_id=0 AND nt.generates_abscence<>0";
			$rows = DB::executeAll("SELECT count(distinct person_id) as num FROM ".TABLE_PREFIX."news_details e 
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=e.object_id
					INNER JOIN ".TABLE_PREFIX."news_types nt ON nt.id=e.news_type_id 
					WHERE $sql_where");
			$absences = $rows[0]['num'];
			$t->setTotalAbsences($absences);
				
			$tmp[$t->getHierarchyGroupId()] = $t;
		}
		$totals_by_hierarchy = $tmp;


		$tmp = NewsSummaryTotalAbss::findAll(array('conditions' => 'summary_id = '.$summary->getId()));
		$totals_by_news_type = array();
		foreach ($tmp as $t) {
			$group_id = 0;
			if ($t->getNewsTypeId() > 0) {
				$nt = NewsTypes::findById($t->getNewsTypeId());
				if ($nt instanceof NewsType) $group_id = $nt->getNewsTypeGroupId();
			}
			if (!isset($totals_by_news_type[$group_id])) $totals_by_news_type[$group_id] = 0;
			$totals_by_news_type[$group_id] = $totals_by_news_type[$group_id] + $t->getTotal();
		}
		
		return array('totals_by_hierarchy' => $totals_by_hierarchy, 'totals_by_news_type' => $totals_by_news_type);
	}
	
	
	
	private function recalculate_summary($date, $member_ids, $recursive=true) {
		
		$internal_destiny = $member_ids[0];
		
		$summary = NewsSummaries::findOne(array('conditions' => "date = '".$date->toMySQL()."' AND internal_destiny=$internal_destiny"));
		if (!$summary instanceof NewsSummary) {
			$summary = new NewsSummary();
			$summary->setDate($date);
			$summary->setStatus('pending');
			$summary->setInternalDestiny($internal_destiny);
			$summary->save();
			
			$oc = new ObjectController();
			$oc->add_to_members($summary, $member_ids);
		}
		
		$total_absences = array();
		
		// get all child members
		$actual_member = Members::findById($internal_destiny);
		$all_member_ids = $actual_member->getAllChildrenIds(true);
		$all_member_ids[] = $actual_member->getId();
		
		// get details for actual member and submembers
		$all_details = NewsDetails::findAll(array('conditions' => "date = '".$date->toMySQL()."' AND person_id IN (
			SELECT c.object_id FROM ".TABLE_PREFIX."contacts c WHERE c.internal_destiny IN (".implode(',', $all_member_ids).")) 
			AND trashed_by_id=0 AND archived_by_id=0"));
		
		$processed_contacts = array();
		// sum abscences grouped by type
		foreach ($all_details as $det) {
			if (isset($processed_contacts[$det->getPersonId()])) continue;
			
			$type = NewsTypes::findById($det->getNewsTypeId());
			if (!$type instanceof NewsType || !$type->getGeneratesAbscence() ) continue;
			
			$type_id = $det->getNewsTypeId();
			if (!isset($total_absences[$type_id])) {
				$total_absences[$type_id] = 0;
			}
			$total_absences[$type_id] = $total_absences[$type_id] + 1;
			$processed_contacts[$det->getPersonId()] = $det->getPersonId();
		}
		
		// save absences for summary
		NewsSummaryTotalAbss::delete('summary_id = '.$summary->getId());
		foreach ($total_absences as $type => $abs) {
			$newsabs = new NewsSummaryTotalAbs();
			$newsabs->setSummaryId($summary->getId());
			$newsabs->setNewsTypeId($type);
			$newsabs->setTotal($abs);
			$newsabs->save();
		}
		
		// delete previous data
		NewsSummaryTotalPersonss::delete('summary_id = '.$summary->getId());
		NewsSummaryTotalPersonsByHierarchies::delete('summary_id = '.$summary->getId());
		NewsSummaryTotalAbsByHierarchies::delete('summary_id = '.$summary->getId());
		
		// sum total persons and save them for summary
		$hgs = PersonHierarchyGroups::findAll(array('id' => true, 'conditions' => "enabled=1"));
		foreach ($hgs as $hg) {
			
			$hs = PersonHierarchies::findAll(array('id' => true, 'conditions' => "enabled=1 and hierarchy_group_id=".$hg));
			if (count($hs) == 0) $hs[] = 0;
			$total_hg = 0;
			
			// sum total persons by hierarchy and total absences by hierarchy
			foreach ($hs as $h_id) {
				$contacts = Contacts::findAll(array('id' => true, 'conditions' => 'user_type=0 AND disabled=0 AND is_active=1 AND hierarchy_id='.$h_id.' AND internal_destiny IN ('.implode(',', $all_member_ids).')'));
				$total = count($contacts);
				if ($total == 0) continue;
				
				$total_hg += $total;
				
				// total persons by hierarchy
				$newstotper_byh = new NewsSummaryTotalPersonsByHierarchy();
				$newstotper_byh->setSummaryId($summary->getId());
				$newstotper_byh->setHierarchyId($h_id);
				$newstotper_byh->setTotal($total);
				$newstotper_byh->save();
				
				// total absences by hierarchy
				$rows = DB::executeAll("SELECT count(distinct(d.person_id)) as num FROM ".TABLE_PREFIX."news_details d
						INNER JOIN ".TABLE_PREFIX."news_types nt on nt.id=d.news_type_id
						WHERE nt.generates_abscence>0 AND d.date = '".$date->toMySQL()."' AND d.person_id IN (".implode(',', $contacts).")");
				$count_by_h = $rows[0]['num'];
				$newstotabs_byh = new NewsSummaryTotalAbsByHierarchy();
				$newstotabs_byh->setSummaryId($summary->getId());
				$newstotabs_byh->setHierarchyId($h_id);
				$newstotabs_byh->setTotal($count_by_h);
				$newstotabs_byh->save();
			}
			
			if ($total_hg > 0) {
				$newstotper = new NewsSummaryTotalPersons();
				$newstotper->setSummaryId($summary->getId());
				$newstotper->setHierarchyGroupId($hg);
				$newstotper->setTotal($total_hg);
				$newstotper->save();
			}
		}
		
		// recalculate summary for parent
		if ($actual_member->getParentMemberId() > 0 && $recursive) {
			$this->recalculate_summary($date, array($actual_member->getParentMemberId()));
	/*	} else {
			$this->recalculate_general_summary($date);*/
		}
	}
	
	
	
	function change_summary_status() {
		ajx_current("empty");
		
		$summary = NewsSummaries::findById(get_id());
		if (!$summary instanceof NewsSummary) {
			flash_error(lang('news summary dnx'));
			return;
		} else {
			if (!in_array($_GET['st'], array('pending','approved'))) {
				flash_error(lang('invalid status'));
				return;
			}
			
			$st = $_GET['st'];
			
			if ($summary->getStatus() == 'approved') {
				if (!can_edit_approved_summaries(logged_user())) {
					flash_error(lang('cannot edit approved summaries'));
					return;
				}
				$success_msg = lang("success summary status edited");
			} else {
				if (!can_approve_summaries(logged_user())) {
					flash_error(lang('cannot approve summaries'));
					return;
				}
				
				$approve_subsummaries = array_var($_GET, 'approve_sub');
				if (!$approve_subsummaries) {
					$member = Members::findById($summary->getInternalDestiny());
					$child_ids = $member->getAllChildrenIds();
					foreach ($child_ids as $cid) {
						$sub_summary = NewsSummaries::findOne(array('conditions' => "`date`='".$summary->getDate()->toMySQL()."' AND internal_destiny = $cid"));
						if (!$sub_summary instanceof NewsSummary || $sub_summary->getStatus() == 'pending') {
							//flash_error(lang('cannot approve subsummaries pending'));
							ajx_extra_data(array('ok' => false, 'sub_sum_pending' => true));
							return;
						}
					}
				}
				
				$success_msg = lang("success summary approved");
			}
			try {
				DB::beginWork();
				
				$summary->setStatus($st);
				$summary->save();
				$this->create_present_type_news($summary);
				
				// change status of sub summaries
				if ($st == 'approved' && $approve_subsummaries) {
					$member = Members::findById($summary->getInternalDestiny());
					$child_ids = $member->getAllChildrenIds(true);
					foreach ($child_ids as $cid) {
						$sub_summary = NewsSummaries::findOne(array('conditions' => "`date`='".$summary->getDate()->toMySQL()."' AND status='pending' AND internal_destiny = $cid"));
						if (!$sub_summary instanceof NewsSummary) {
							$this->recalculate_summary($summary->getDate(), array($cid), false);
							$sub_summary = NewsSummaries::findOne(array('conditions' => "`date`='".$summary->getDate()->toMySQL()."' AND status='pending' AND internal_destiny = $cid"));
						}
						if ($sub_summary instanceof NewsSummary) {
							$sub_summary->setStatus($st);
							$sub_summary->save();
							$this->create_present_type_news($sub_summary);
						}
					}
					
					$success_msg = lang("success summary approved inlcuding subparts");
				}
				
				flash_success($success_msg);
				//ApplicationLogs::createLog($summary, 'edit');
				
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
		
		ajx_extra_data(array('ok' => true));
	}
	
	
	function create_approved_summary() {
		ajx_current("empty");
		
		if (!can_approve_summaries(logged_user())) {
			flash_error(lang('cannot approve summaries'));
			return;
		}
	
		$summary = new NewsSummary();
		$summary->setDate(DateTimeValueLib::dateFromFormatAndString('Y-m-d', array_var($_REQUEST, 'date')));
		$summary->setStatus('approved');
		
		$context = active_context();
		$ws_ot = ObjectTypes::findByName('workspace');
		$workspace = null;
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$workspace = $selection;
				break;
			}
		}
		
		if (!$workspace instanceof Member) {
			return;
		}
		$summary->setInternalDestiny($workspace->getId());
		
		$object_controller = new ObjectController();
		
		try {
			DB::beginWork();
			
			$summary->save();
			$object_controller->add_to_members($summary, array($workspace->getId()));
			$this->create_present_type_news($summary);
			
			// create emtpy approved summaries for subworkspaces
			$sub_workspaces = $workspace->getAllChildren(true);
			foreach ($sub_workspaces as $subw) {
				$subsum = new NewsSummary();
				$subsum->setDate(DateTimeValueLib::dateFromFormatAndString('Y-m-d', array_var($_REQUEST, 'date')));
				$subsum->setStatus('approved');
				$subsum->setInternalDestiny($subw->getId());
				$subsum->save();
				$object_controller->add_to_members($subsum, array($subw->getId()));
				$this->create_present_type_news($subsum);
			}
			
			flash_success(lang("success summary approved"));
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
		ajx_extra_data(array('ok' => true));
	}
	
	
	private function create_present_type_news(NewsSummary $summary) {
		$object_controller = new ObjectController();
		
		$date = new DateTimeValue($summary->getDate()->getTimestamp());
		$now = DateTimeValueLib::now();
		$now->add('m', logged_user()->getUserTimezoneValue()/60);
		
		$present_news_type = NewsTypes::instance()->findOne(array('conditions' => 'enabled=1 AND is_present_type=1'));
		if ($present_news_type instanceof NewsType) {
			$count = 0;

			$all_persons_in_dest = Contacts::findAll(array('conditions' => 'trashed_by_id=0 AND is_active=1 AND user_type=0 AND internal_destiny='.$summary->getInternalDestiny()));
			foreach ($all_persons_in_dest as $person) {
				
				$sql = "SELECT d.object_id FROM ".TABLE_PREFIX."news_details d 
						INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=d.object_id
						WHERE om.member_id=".$summary->getInternalDestiny()." AND om.is_optimization=0 AND d.person_id=".$person->getId()." AND `date`='".$date->toMySQL()."'";
				$rows = DB::executeAll($sql);
				
				if (!is_array($rows) || count($rows) == 0) {
					
					// crear novedad de tipo presente
					$det = new NewsDetail();
					$det->setDate($date);
					$det->setHour($now->getHour());
					$det->setMinute($now->getMinute());
					$det->setPersonId($person->getId());
					$det->setNewsTypeId($present_news_type->getId());
					$det->save();
					
					$object_controller->add_to_members($det, array($summary->getInternalDestiny()));
					$count++;
				}
			}
			// recalculate values
			$this->recalculate_summary($summary->getDate(), array($summary->getInternalDestiny()), false);
			
			if ($count > 0) {
				evt_add('reload tab panel', 'time-registration-panel');
			}
		}
	}
	
	
	function observations_panel() {
		ajx_current("empty");
		
		$context = active_context();
		$ws_ot = ObjectTypes::findByName('workspace');
		$workspace = null;
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$workspace = $selection;
				break;
			}
		}
		
		if (array_var($_REQUEST, 'summary_date')) {
			$date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_REQUEST, 'summary_date'));
		} else {
			$date = DateTimeValueLib::now();
		}
		
		$observations = array();
		if ($workspace instanceof Member) {
			// general observations
			$summary = NewsSummaries::findOne(array('conditions' => "`date`='".$date->toMySQL()."' AND internal_destiny = ".$workspace->getId()));
			if ($summary instanceof NewsSummary) {
				$observations[] = " - <span class='bold'>".lang('general observations').": </span>".$summary->getObservations();
			}
			
			// get all child members
			$all_member_ids = $workspace->getAllChildrenIds(true);
			$all_member_ids[] = $workspace->getId();
			
			// get details for actual member and submembers
			$all_details = NewsDetails::findAll(array('conditions' => "date = '".$date->toMySQL()."' AND observations <> '' AND person_id IN (
				SELECT c.object_id FROM ".TABLE_PREFIX."contacts c WHERE c.internal_destiny IN (".implode(',', $all_member_ids).")) 
				AND trashed_by_id=0 AND archived_by_id=0"));
			
			foreach ($all_details as $det) {
				$contact = Contacts::findById($det->getPersonId());
				if ($contact instanceof Contact) {
					$observations[] = " - <span class='bold'>".$contact->getObjectName().": </span>".$det->getObservations();
				}
			}
		}
		
		tpl_assign('observations', $observations);
		ajx_extra_data(array('html' => tpl_fetch(get_template_path('observations_panel', 'time_registration', $this->plugin_name))));
	}
	
	
	function administration_settings() {

		$resp_id = config_option('summary_general_responsible');
		if ($resp_id > 0) {
			$resp = Contacts::findById($resp_id);
			$resp_name = $resp->getObjectName();
		} else {
			$resp_name = "";
		}
		
		$aux_id = config_option('summary_general_auxiliar');
		if ($aux_id > 0) {
			$aux = Contacts::findById($aux_id);
			$aux_name = $aux->getObjectName();
		} else {
			$aux_name = "";
		}
		
		tpl_assign('allow_multiple_news_details', config_option('allow_multiple_news_details_in_same_day'));
		tpl_assign('selected_resp_id', $resp_id);
		tpl_assign('selected_resp_name', $resp_name);
		tpl_assign('selected_aux_id', $aux_id);
		tpl_assign('selected_aux_name', $aux_name);
		
	}
	
	function save_administration_setting() {
		ajx_current("empty");
		
		if (array_var($_REQUEST, 'type') == 'general_responsible' || array_var($_REQUEST, 'type') == 'general_auxiliar') {
			// check if contact exists
			$contact = Contacts::findbyId(array_var($_REQUEST, 'contact_id'));
			if (!$contact instanceof Contact) {
				return;
			}
		}
		
		// save option
		if (array_var($_REQUEST, 'type') == 'general_responsible') {
			
			set_config_option('summary_general_responsible', $contact->getId());
			
		} else if (array_var($_REQUEST, 'type') == 'general_auxiliar') {
			
			set_config_option('summary_general_auxiliar', $contact->getId());
			
		} else if (array_var($_REQUEST, 'type') == 'allow_multiple_nd') {
			
			$allow = array_var($_REQUEST, 'allow') > 0;
			set_config_option('allow_multiple_news_details_in_same_day', $allow);
			
		}
	}
	
	
	
	function generate_from_last_day() {
		
		$date_to = getDateValue(array_var($_REQUEST, 'date_to'));

		$date_from = new DateTimeValue($date_to->getTimestamp());
		$date_from->add('d', -1);
		$working_days = explode(',', config_option('working_days'));
		while (!in_array($date_from->format('w'), $working_days)) {
			$date_from->add('d', -1);
		}
		
		$context = active_context();
		$ws_ot = ObjectTypes::findByName('workspace');
		$workspace = null;
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$workspace = $selection;
				break;
			}
		}
		
		if (!$workspace instanceof Member) {
			flash_error(lang("internal destiny must be selected to copy"));
			ajx_current("empty");
			return;
		}
		
		$extra_conditions = " AND `date`='".$date_from->toMySQL()."'";
		$extra_conditions .= " AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om WHERE om.object_id=o.id AND om.member_id=".$workspace->getId()." AND om.is_optimization=0)";
		
		$result = NewsDetails::instance()->listing(array(
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
		));
		
		$old_details = $result->objects;
		
		$object_controller = new ObjectController();
		try {
			
			DB::beginWork();
			$count = 0;
			foreach ($old_details as $det) {
				$nd = NewsDetails::findOne(array('conditions' => "person_id=".$det->getPersonId()." AND `date`='".$date_to->toMySQL()."' AND 
					EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om WHERE om.object_id=e.object_id AND om.member_id=".$workspace->getId()." AND om.is_optimization=0)"));
				
				if ($nd instanceof NewsDetail) continue;
				
				$new_det = $det->copy();
				$new_det->setDate($date_to);
				$new_det->save();
				$object_controller->add_to_members($new_det, array($workspace->getId()));
				$count++;
			}
			
			$this->recalculate_summary($date_to, array($workspace->getId()));
			
			DB::commit();
			flash_success(lang("success copying news details from date in internal destiny", $count, format_date($date_to, null, 0), $workspace->getName()));
			ajx_current("reload");
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
		
	}
	
	
	
	function edit_summary_observations() {
		$date = getDateValue(array_var($_REQUEST, 'date'));
		
		$context = active_context();
		$ws_ot = ObjectTypes::findByName('workspace');
		$workspace = null;
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $ws_ot->getId()) {
				$workspace = $selection;
				break;
			}
		}
		
		if (!$workspace instanceof Member) {
			flash_error(lang("internal destiny not selected"));
			ajx_current("empty");
			return;
		}
		
		$summary = NewsSummaries::findOne(array('conditions' => "`date`='".$date->toMySQL()."' AND internal_destiny = ".$workspace->getId()));
		if (!$summary instanceof NewsSummary) {
			flash_error(lang('news summary dnx for date and ws', format_date($date,null,0), $workspace->getName()));
			ajx_current("empty");
			return;
		}
		
		if ($summary->getStatus() != 'pending') {
			flash_error(lang('cannot edit observations in approved news summary', format_date($date,null,0), $workspace->getName()));
			ajx_current("empty");
			return;
		}
		
		tpl_assign('date', $date);
		tpl_assign('workspace', $workspace);
		tpl_assign('observations', $summary->getObservations());
		
		if (isset($_REQUEST['observations'])) {
			$obs_text = $_REQUEST['observations'];
			try {
				DB::beginWork();
				
				$summary->setObservations($obs_text);
				$summary->save();
				
				DB::commit();
				
				ApplicationLogs::createLog($summary, ApplicationLogs::ACTION_EDIT);
				
				flash_success(lang('success edit summary observations'));
				ajx_current("back");
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
		}
		
	}
	
}






















