<?php
/**
 * TimeRegistrationReports controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TimeRegistrationReportsController extends ApplicationController {

	var $plugin_name = "work_time_registration";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function summary_report_parameters() {
		
	}
	
	function summary_report() {
		$report_data = array(
				'date_start' => array_var($_REQUEST, 'date_start'), 'date_end' => array_var($_REQUEST, 'date_end'), 'show_details' => array_var($_REQUEST, 'show_details'),
				'show_present_news' => array_var($_REQUEST, 'show_present_news'), 'include_subdest' => array_var($_REQUEST, 'include_subdest'), 
				'news_type_id' => array_var($_REQUEST, 'news_type_id'), 'person_id' => array_var($_REQUEST, 'person_id')
		);
		
		$_SESSION['summary_report'] = $report_data;
		
		try {
			$summaries_info = array();
			
			$date_start = getDateValue(array_var($_REQUEST, 'date_start'));
			$date_end = getDateValue(array_var($_REQUEST, 'date_end'));
			
			// if destiny not specified take it from context
			$internal_destiny = array_var($_REQUEST, 'internal_destiny');
			if (!$internal_destiny) {
				$ws_ot = ObjectTypes::findByName('workspace');
				$context = active_context();
				foreach ($context as $sel) {
					if ($sel instanceof Member && $sel->getObjectTypeId()) {
						$internal_destiny = $sel->getId();
						break;
					}
				}
			}
			if (!$internal_destiny) {
				throw new Exception(lang('internal destiny not selected'));
			}
			
			$internal_destiny_to_show = array($internal_destiny);
			if (array_var($_REQUEST, 'include_subdest') == 'checked') {
				$member = Members::findById($internal_destiny);
				$child_ids = $member->getAllChildrenIds(true);
				
				$internal_destiny_to_show = array_merge($internal_destiny_to_show, $child_ids);
			}
			
			// get info for each day
			$date = new DateTimeValue($date_start->getTimestamp());
			while ($date->getTimestamp() <= $date_end->getTimestamp()) {
				
				$date_str = format_date($date, 'Y-m-d', 0);
				foreach ($internal_destiny_to_show as $id) {
					
					$member_ids = array($id);
					$m = Members::findById($id);
					if ($m instanceof Member) $member_ids = array_merge($member_ids, $m->getAllChildrenIds(true));
					
					$people_count = Contacts::instance()->count("user_type=0 AND internal_destiny IN (".implode(',', $member_ids).")");
					if ($people_count == 0) continue;
					
					$info = $this->get_summary_info_by_day($date, $id, array_var($report_data, 'news_type_id'), array_var($report_data, 'person_id'), array_var($report_data, 'show_present_news')=='checked');
					if (is_array($info)) {
						if (!isset($summaries_info[$date_str])) $summaries_info[$date_str] = array();
						$summaries_info[$date_str][$id] = $info;
					}
					
				}
				
				// advance to next day
				$date->add('d', 1);
			}
			
			if (count($summaries_info) == 0) {
				throw new Exception(lang('there are no summaries to print from to', format_date($date_start, null, 0), format_date($date_end, null, 0)));
			}
			
					
			tpl_assign('summaries_info', $summaries_info);
			tpl_assign('date_start', $date_start);
			tpl_assign('date_end', $date_end);
			
			tpl_assign('title', lang('summary report'));
			tpl_assign('allow_export', false);
			if (array_var($_REQUEST, 'from_list')) {
				tpl_assign('template_name', 'summary_report_from_list');
			} else {
				tpl_assign('template_name', 'summary_report');
			}
			$this->setTemplate('report_wrapper');
			
		} catch (Exception $e) {
			ajx_current("empty");
			flash_error($e->getMessage());
		}
	}
	
	/**
	 * Returns the summary information for an internal destiny in a specified date
	 */
	private function get_summary_info_by_day(DateTimeValue $date, $internal_destiny, $news_type_filter=null, $person_id=null, $show_present_news=false) {
		
		$summary = NewsSummaries::findOne(array('conditions' => array('`date` = ? AND `internal_destiny` = ?', $date, $internal_destiny)));
		if (!$summary instanceof NewsSummary) return;
		
		$workspace = Members::findById($internal_destiny);
		$all_member_ids = $workspace->getAllChildrenIds(true);
		$all_member_ids[] = $workspace->getId();

		$totals_by_hierarchy = NewsSummaryTotalPersonss::findAll(array('conditions' => 'summary_id = '.$summary->getId()));
		$tmp = array();
		foreach ($totals_by_hierarchy as &$t) {
			
			$hs = PersonHierarchies::findAll(array('id' => true, 'conditions' => "enabled=1 and hierarchy_group_id=".$t->getHierarchyGroupId()));
			if (count($hs) == 0) $hs[] = 0;
			
			if (is_numeric($person_id) && $person_id > 0) {
				$contact_ids[] = $person_id;
			} else {
				$contact_ids = Contacts::findAll(array('id' => true, 'conditions' => 'user_type=0 AND disabled=0 AND is_active=1 AND hierarchy_id IN ('.implode(',',$hs).') AND internal_destiny IN ('.implode(',', $all_member_ids).')'));
				if (count($contact_ids) == 0) $contact_ids[] = 0;
			}
			
			$sql_where = "`date`='".$date->format('Y-m-d')."' AND person_id IN (".implode(',',$contact_ids).") AND trashed_by_id=0 AND archived_by_id=0 AND nt.generates_abscence<>0";
			
			$rows = DB::executeAll("SELECT count(distinct person_id) as num FROM ".TABLE_PREFIX."news_details e 
					INNER JOIN ".TABLE_PREFIX."objects o ON o.id=e.object_id
					INNER JOIN ".TABLE_PREFIX."news_types nt ON nt.id=e.news_type_id  
					WHERE $sql_where");
			$absences = $rows[0]['num'];
			$t->setTotalAbsences($absences);
			
			$tmp[$t->getHierarchyGroupId()] = $t;
		}
		$totals_by_hierarchy = $tmp;


		$tmp = NewsSummaryTotalAbss::findAll(array('conditions' => 'summary_id = '.$summary->getId()));
		$totals_by_news_type_groups = array();
		foreach ($tmp as $t) {
			$group_id = 0;
			if ($t->getNewsTypeId() > 0) {
				$nt = NewsTypes::findById($t->getNewsTypeId());
				if ($nt instanceof NewsType) $group_id = $nt->getNewsTypeGroupId();
			}
			if (!isset($totals_by_news_type_groups[$group_id])) $totals_by_news_type_groups[$group_id] = 0;
			$totals_by_news_type_groups[$group_id] = $totals_by_news_type_groups[$group_id] + $t->getTotal();
		}
		
		// get details
		$extra_conditions = " AND `date`='".$date->toMySQL()."' AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om 
			WHERE om.member_id=".$internal_destiny." AND om.object_id=o.id )";

		if (is_numeric($news_type_filter) && $news_type_filter > 0) {
			$extra_conditions .= " AND e.news_type_id='$news_type_filter'";
		}
		
		if (is_numeric($person_id) && $person_id > 0) {
			$extra_conditions .= " AND e.person_id='$person_id'";
		}
		
		if (!$show_present_news) {
			$extra_conditions .= " AND EXISTS (SELECT nt.is_present_type FROM ".TABLE_PREFIX."news_types nt WHERE nt.id=e.news_type_id AND nt.is_present_type=0)";
		}
		
		$result = NewsDetails::instance()->listing(array(
			"raw_data" => true,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => false,
		));
		$details = $result->objects;
		
		$ws_ot_id = ObjectTypes::findByName('workspace')->getId();
		foreach ($details as &$detail) {
			$sql = "SELECT m.id, m.name FROM ".TABLE_PREFIX."object_members om INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id 
				WHERE om.object_id=".$detail['id']." AND om.is_optimization = 0 AND m.object_type_id=$ws_ot_id";
			$rows = DB::executeAll($sql);
			
			if (count($rows) > 0) {
				$detail['internal_destiny_id'] = $rows[0]['id'];
				$detail['internal_destiny_name'] = $rows[0]['name'];
			} else {
				$detail['internal_destiny_id'] = 0;
				$detail['internal_destiny_name'] = '';
			}
		}
		
		return array(
			'totals_by_hg' => $totals_by_hierarchy,
			'totals_by_ntg' => $totals_by_news_type_groups,
			'details' => $details,
			'observations' => $summary->getObservations(),
		);
	}

	
	function absences_by_hierarchy_group_parameters() {
		
	}
	
	
	function absences_by_hierarchy_group() {
		try {
			
			$report_data = array_var($_REQUEST, 'report');
			$_SESSION['absences_by_hierarchy_group'] = $report_data;
			
			$dates = $this->parse_date_parameter(array_var($report_data, 'date_type'), array_var($report_data, 'date_start_value'), array_var($report_data, 'date_end_value'));
			$date_start = array_var($dates, 'st');
			$date_end = array_var($dates, 'et');
			$main_idest = null;
			
			// if destiny not specified take it from context
			$ws_ot = ObjectTypes::findByName('workspace');
			$internal_destiny = array_var($report_data, 'internal_destiny');
			if (!$internal_destiny) {
				$context = active_context();
				foreach ($context as $sel) {
					if ($sel instanceof Member && $sel->getObjectTypeId()) {
						$internal_destiny = $sel->getId();
						break;
					}
				}
			}
			if (!$internal_destiny) {
				$internal_destiny_to_show = Members::findAll(array('id' => true, 'conditions' => 'object_type_id = '.$ws_ot->getId().' AND depth=1'));
			} else {
				$main_idest = Members::findById($internal_destiny);
				$internal_destiny_to_show = array($internal_destiny);
			}
			
			
			if (array_var($report_data, 'include_subdest') == 'checked') {
				$parent_ids = $internal_destiny_to_show;
				foreach ($parent_ids as $id) {
					$member = Members::findById($id);
					$child_ids = $member->getAllChildrenIds(true);
					$internal_destiny_to_show = array_merge($internal_destiny_to_show, $child_ids);
				}
			}
			
			$extra_conditions = " AND `date` >= '".$date_start->toMySQL()."' AND `date` <= '".$date_end->toMySQL()."'";
			
			$member_cond = " AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om 
				WHERE om.member_id IN (".implode(',',$internal_destiny_to_show).") AND om.object_id=o.id AND om.is_optimization=0)";
			
			if (array_var($report_data, 'news_type_id') > 0) {
				$extra_conditions .= " AND `news_type_id` = '".array_var($report_data, 'news_type_id')."'";
			}
			
			if (array_var($report_data, 'person_id') > 0) {
				$extra_conditions .= " AND `person_id` = '".array_var($report_data, 'person_id')."'";
			}
			
			$result = NewsDetails::instance()->listing(array(
				"raw_data" => true,
				"extra_conditions" => $extra_conditions . $member_cond,
				"ignore_context" => true,
				"count_results" => false,
				"only_count_results" => false,
			));
			$details = $result->objects;
			
			if (count($details) == 0) {
				throw new Exception(lang('there are no summaries to print from to', format_date($date_start, null, 0), format_date($date_end, null, 0)));
			}
			
			$days_count = array();
			$count_by_hg = array();
			
			foreach ($details as $det) {
				$nt = NewsTypes::findById($det['news_type_id']);
				if ($nt instanceof NewsType && !$nt->getGeneratesAbscence()) continue;
				
				$c = Contacts::findById($det['person_id']);
				$h = PersonHierarchies::findById($c->getColumnValue('hierarchy_id'));
				
				$hg_id = $h->getHierarchyGroupId();
				if (!isset($count_by_hg[$hg_id])) $count_by_hg[$hg_id] = array('id' => array(), 'nt' => array());
				
				// set total by internal destiny
				$rows = DB::executeAll("SELECT om.member_id FROM ".TABLE_PREFIX."object_members om
					INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
					WHERE om.object_id=".$det['object_id']." AND om.is_optimization=0 AND m.object_type_id=".$ws_ot->getId());
				
				if (count($rows) == 0) continue;
				$mid = $rows[0]['member_id'];
				
				if (!isset($count_by_hg[$hg_id]['id'][$mid])) $count_by_hg[$hg_id]['id'][$mid] = 0;
				$count_by_hg[$hg_id]['id'][$mid] = $count_by_hg[$hg_id]['id'][$mid] + 1;
				
				// set total by news type
				$nt_id = $det['news_type_id'];
				
				if (!isset($count_by_hg[$hg_id]['nt'][$nt_id])) $count_by_hg[$hg_id]['nt'][$nt_id] = 0;
				$count_by_hg[$hg_id]['nt'][$nt_id] = $count_by_hg[$hg_id]['nt'][$nt_id] + 1;
				
				$days_count[$det['date']] = 1;
			}
			
			tpl_assign('main_idest', $main_idest);
			tpl_assign('days_count', count($days_count));
			tpl_assign('count_by_hg', $count_by_hg);
			tpl_assign('date_start', $date_start);
			tpl_assign('date_end', $date_end);
			
			tpl_assign('title', lang('absences by hierarchy group report'));
			tpl_assign('allow_export', false);
			tpl_assign('template_name', 'absences_by_hierarchy_group');
			$this->setTemplate('report_wrapper');
			
		} catch (Exception $e) {
			ajx_current("empty");
			flash_error($e->getMessage());
		}
	}
	
	
	function presents_by_hierarchy_group_parameters() {
		
	}
	
	function presents_by_hierarchy_group() {
		try {
			
			$report_data = array_var($_REQUEST, 'report');
			$_SESSION['presents_by_hierarchy_group'] = $report_data;
		
			$date = getDateValue(array_var($report_data, 'date'));
			$main_idest = null;
			
			// if destiny not specified take it from context
			$ws_ot = ObjectTypes::findByName('workspace');
			$internal_destiny = array_var($report_data, 'internal_destiny');
			if (!$internal_destiny) {
				$context = active_context();
				foreach ($context as $sel) {
					if ($sel instanceof Member && $sel->getObjectTypeId()) {
						$internal_destiny = $sel->getId();
						break;
					}
				}
			}
			if (!$internal_destiny) {
				$internal_destiny_to_show = Members::findAll(array('id' => true, 'conditions' => 'object_type_id = '.$ws_ot->getId().' AND depth=1'));
				if (count($internal_destiny_to_show) == 0) $internal_destiny_to_show = array(-1);
			} else {
				$main_idest = Members::findById($internal_destiny);
				$internal_destiny_to_show = array($internal_destiny);
			}
			
			if (array_var($report_data, 'include_subdest') == 'checked') {
				$parent_ids = $internal_destiny_to_show;
				foreach ($parent_ids as $id) {
					$member = Members::findById($id);
					$child_ids = $member->getAllChildrenIds();
					$internal_destiny_to_show = array_merge($internal_destiny_to_show, $child_ids);
				}
			}
			
			$count_by_hierarchy_group = array();
			$hierarchies_by_group = array();
			
			$summaries = NewsSummaries::findAll(array('conditions' => "`date`='".$date->toMySQL()."' AND `internal_destiny` IN (".implode(',',$internal_destiny_to_show).")"));
			foreach ($summaries as $summary) {
				$idest = $summary->getInternalDestiny();
				$persons_by_h = NewsSummaryTotalPersonsByHierarchies::findAll(array('conditions' => 'summary_id = '.$summary->getId()));
				foreach ($persons_by_h as $person_by_h) {
					$h_id = $person_by_h->getHierarchyId();
					$total = $person_by_h->getTotal();
					
					$hierarchy = PersonHierarchies::instance()->findById($h_id);
					
					$hg = $hierarchy->getHierarchyGroupId();
					if (!isset($hierarchies_by_group[$hg])) $hierarchies_by_group[$hg] = array();
					$hierarchies_by_group[$hg][$h_id] = $hierarchy;
					
					if (!isset($count_by_hierarchy_group[$hg])) $count_by_hierarchy_group[$hg] = array();
					
					if (!isset($count_by_hierarchy_group[$hg][$idest])) $count_by_hierarchy_group[$hg][$idest] = array();
					
					// get details
					$extra_conditions = " AND `date`='".$date->toMySQL()."' AND EXISTS (SELECT om.object_id FROM ".TABLE_PREFIX."object_members om 
						WHERE om.member_id=".$idest." AND om.object_id=o.id AND om.is_optimization=0)";
					$extra_conditions .= " AND `person_id` IN (SELECT c.object_id FROM ".TABLE_PREFIX."contacts c WHERE c.hierarchy_id=$h_id AND c.user_type=0 AND c.disabled=0 AND c.is_active=1)";
					
					$result = NewsDetails::instance()->listing(array(
						"raw_data" => true,
						"extra_conditions" => $extra_conditions,
						"count_results" => false,
						"only_count_results" => false,
					));
					$details = $result->objects;
					$count = array();
					if (is_array($count)) {
						foreach ($details as $det) {
							$nt = NewsTypes::findById($det['news_type_id']);
							if ($nt instanceof NewsType && $nt->getGeneratesAbscence()) {
								$count[$det['person_id']] = 1;
							}
						}
					}
					
					$count_by_hierarchy_group[$hg][$idest][$h_id] = $total - count($count);
				}
			}
			
			tpl_assign('date', $date);
			tpl_assign('main_idest', $main_idest);
			tpl_assign('hierarchies_by_group', $hierarchies_by_group);
			tpl_assign('count_by_hierarchy_group', $count_by_hierarchy_group);
			
			tpl_assign('title', lang('presents by hierarchy group report'));
			tpl_assign('allow_export', false);
			tpl_assign('template_name', 'presents_by_hierarchy_group');
			$this->setTemplate('report_wrapper');
			
		} catch (Exception $e) {
			ajx_current("empty");
			flash_error($e->getMessage());
		}
	}
	
	
	
	private function parse_date_parameter($date_type, $date_start, $date_end) {
		$now = DateTimeValueLib::now();
		switch ($date_type) {
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Next week
				$monday = $now->getMondayOfWeek()->add('w',1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Next month
				$now->add('M',1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue($date_start);
				$st = $st->beginningOfDay();
				
				$et = getDateValue($date_end);
				$et = $et->endOfDay();
				break;
		}
		
		return array('st' => $st, 'et' => $et);
	}
	
	
	
}