<?php
	
	function print_summary_info_totals_by_news_type_group($totals_by_ntg, $ntgs) {
	
		$html = '<table><tr><th class="border">'.lang('news type group').'</th>'.'<th class="center">'.lang('total').'</th></tr>';
		$report_total = 0;
		
		foreach ($totals_by_ntg as $ntg => $tot) {
			$name = $ntg == 0 ? lang('without cause') : $ntgs[$ntg]->getName();
			$html .= '<tr><td>'.$name.'</td><td class="center">'.$tot.'</td></tr>';
			$report_total += $tot;
		}
	
		$html .= '<tr class="bold totals"><td>'.lang('all').'</td><td class="center">'.$report_total.'</td></tr>';
		$html .= '</table>';
	
		echo $html;
	}
	
	
	function print_summary_info_details($details, $hgs, $ntgs, $nts) {
		$hgs_by_code = array();
		foreach ($hgs as $hg) $hgs_by_code[$hg->getCode()] = $hg;
		
		echo '<table><tr class="border"><th>'.lang('date').'</th><th>'.lang('person hierarchy').'</th><th>'.lang('surname').'</th><th>'.lang('first_name').'</th><th>'
			.lang('news detail').'</th><th>'.lang('internal_destiny').'</th><th>'.lang('observation').'</th></tr>';
		
		$grouped_details = group_details_by_hierarchy_group($details, $hgs, $nts);
		
		foreach ($grouped_details as $hg_code => $hg_info) {
			
			foreach ($hg_info as $h_code => $h_info) {
				foreach ($h_info as $cname => $info) {
					$d = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $info['det']['date']);
					echo '<tr><td>'. ($d instanceof DateTimeValue ? format_date($d,null,0) : '') .'</td>';
					echo '<td>'. $info['h_name'] .'</td>';
					echo '<td>'. $info['contact']['surname'] .'</td>';
					echo '<td>'. $info['contact']['firstname'] .'</td>';
					$nt_name = $info['det']['news_type_id'] == 0 ? lang('without cause') : $nts[$info['det']['news_type_id']]->getDescription();
					echo '<td>'. $nt_name .'</td>';
					echo '<td>'. $info['det']['internal_destiny_name'] .'</td>';
					echo '<td>'. $info['det']['observations'] .'</td>';
					echo '</tr>';
				}
			}
		}
		
		echo '</table>';
		
	}
	
	function print_summary_info_obs($obs) {
		if ($obs != "") {
			echo '<h1>'.lang('general observations').'</h1>';
			echo $obs;
		}
	}
	
	function group_details_by_hierarchy_group($details, $hgs, $nts) {
		$grouped_details = array();
		
		foreach ($details as $detail) {
			
			$contact = Contacts::findById($detail['person_id']);
			$h = PersonHierarchies::findById($contact->getColumnValue('hierarchy_id'));
			$hg_id = $h->getHierarchyGroupId();
			$hg = $hgs[$hg_id];
			
			if (!isset($grouped_details[$hg->getCode()])) $grouped_details[$hg->getCode()] = array();
			
			$date_str = str_replace('00:00:00', '', $detail['date']) . ' ' . str_pad($detail['hour'], 2, '0') . ':' .str_pad($detail['minute'], 2, '0');
			$nt_name = $detail['news_type_id'] == 0 ? lang('without cause') : ($nts[$detail['news_type_id']] instanceof NewsType ? $nts[$detail['news_type_id']]->getDescription() : "");
			
			$contact_name = strtolower($contact->getSurname().' '.$contact->getFirstName())." $date_str $nt_name";
			
			$grouped_details[$hg->getCode()][$h->getCode()][$contact_name] = array(
				'det' => $detail,
				'h_name' => $h->getName(),
				'contact' => array('surname' => $contact->getSurname(), 'firstname' => $contact->getFirstName())
			);
		}
		
		// order by hierarchy group code, hierarchy code, person surname and firstname
		foreach ($grouped_details as $k => &$gd) {
			// order by person surname, firstname
			foreach ($gd as &$d) ksort($d, SORT_STRING);
			// order by hierarchy code
			ksort($gd, SORT_STRING);
		}
		// order by hierarchy group code
		ksort($grouped_details, SORT_STRING);
		
		return $grouped_details;
	}

	$date_format = user_config_option('date_format');

?>
<div class="summary-report-container">
<?php
	
	// hierarchy groups
	$hgs = array();
	$hgs_tmp = PersonHierarchyGroups::findAll(array('conditions' => 'enabled=1'));
	foreach ($hgs_tmp as $hg) {
		$hgs[$hg->getId()] = $hg;
	}
	// news type groups
	$ntgs = array();
	$ntgs_tmp = NewsTypeGroups::findAll(array('conditions' => 'enabled=1'));
	foreach ($ntgs_tmp as $ntg) {
		$ntgs[$ntg->getId()] = $ntg;
	}
	// news types
	$nts = array();
	$nts_groups = array();
	$nts_tmp = NewsTypes::findAll(array('conditions' => 'enabled=1'));
	foreach ($nts_tmp as $nt) {
		$nts[$nt->getId()] = $nt;
		$nts_groups[$nt->getId()] = $nt->getNewsTypeGroupId();
	}
	
	
	$totals_by_news_type_group = array();
	$all_details = array();
	
	// accumulate totals by news type group
	foreach ($summaries_info as $k => $group) {
		
		foreach ($group as $key => $info) {
			$details_array = array_var($info, 'details');
			foreach ($details_array as $detail) {
				$nt_group = array_var($nts_groups, $detail['news_type_id']);
				
				if (!isset($totals_by_news_type_group[$nt_group])) $totals_by_news_type_group[$nt_group] = 0;
				$totals_by_news_type_group[$nt_group] = $totals_by_news_type_group[$nt_group] + 1;

				// concat all details
				$all_details[] = $detail;
			}
			
		}
	}
	
	// print totals by news type group
	print_summary_info_totals_by_news_type_group($totals_by_news_type_group, $ntgs);
	
	// print news details
	$options = array_var($_SESSION, 'summary_report', array());
	if (array_var($options, 'show_details') == 'checked') {
		print_summary_info_details($all_details, $hgs, $ntgs, $nts);
	}
	
?>
</div>



