<?php
Hook::register("work_time_registration");

function work_time_registration_member_contact_cp_filters($params, &$filters) {
	$cp = array_var($params, 'cp');
	$member = array_var($params, 'member');
	
	if (($cp->getCode() == 'responsible' || $cp->getCode() == 'auxiliar') && $member instanceof Member) {
		if (!is_array($filters)) $filters = array();
		//$filters['internal_destiny'] = $member->getId();
		$filters['is_user'] = 1;
		$filters['has_permissions'] = $member->getId();
	}
}

function work_time_registration_render_administration_icons($ignored, &$icons){
	if (logged_user() instanceof Contact) {
		$icons[] = array(
			'ico' => 'ico-large-news-type-group',
			'name' => lang('news summary configuration'),
			'url' => get_url('time_registration', 'administration_settings'),
		);
		
		/*
		$icons[] = array(
			'ico' => 'ico-large-news-type-group',
			'name' => lang('news type groups'),
			'url' => get_url('news_type_group', 'list_all'),
		);
		$icons[] = array(
			'ico' => 'ico-large-news-type',
			'name' => lang('news types'),
			'url' => get_url('news_type', 'list_all'),
		);
		$icons[] = array(
			'ico' => 'ico-large-person-hierarchy-group',
			'name' => lang('person hierarchy groups'),
			'url' => get_url('person_hierarchy_group', 'list_all'),
		);
		$icons[] = array(
			'ico' => 'ico-large-person-hierarchy',
			'name' => lang('person hierarchies'),
			'url' => get_url('person_hierarchy', 'list_all'),
		);*/
	}
}


function work_time_registration_object_definition($type, &$ret) {
	if ($type == 'Contact') {
		
		$ret['nii'] = DATA_TYPE_INTEGER;
		$ret['hierarchy_id'] = DATA_TYPE_INTEGER;
		$ret['document_type'] = DATA_TYPE_STRING;
		$ret['document_number'] = DATA_TYPE_STRING;
		$ret['speciality'] = DATA_TYPE_STRING;
		$ret['is_active'] = DATA_TYPE_BOOLEAN;
		$ret['internal_destiny'] = DATA_TYPE_INTEGER; // workspace
     	
	} else if ($type == 'SystemPermission') {
		
		$ret['can_approve_summaries'] = DATA_TYPE_BOOLEAN;
		$ret['can_edit_approved_summaries'] = DATA_TYPE_BOOLEAN;
		
	}
}

function work_time_registration_object_edit_categories($object, &$categories) {
	if ($object instanceof Contact) {
		tpl_assign('contact', $object);
		$categories[] = array(
			'id' => 'additional_person_data',
			'name' => lang('additional person data'),
			'required' => false,
			'visible' => true,
			'content' => tpl_fetch(get_template_path('person_form_additional_data', 'person', 'work_time_registration')),
		);
	}
}

function work_time_registration_after_add_contact($contact, &$ret) {
	__work_time_registration_classify_in_internal_destiny($contact);
}
function work_time_registration_after_edit_contact($contact, &$ret) {
	__work_time_registration_classify_in_internal_destiny($contact);
}

function __work_time_registration_classify_in_internal_destiny($contact) {
	if ($contact instanceof Contact) {
		$contact_data = array_var($_POST, 'contact');

		$internal_destiny = array_var($contact_data, 'internal_destiny');
		if (!is_numeric($internal_destiny)) {
			$decoded = json_decode($internal_destiny, true);
			if (is_array($decoded) && count($decoded) > 0) {
				$internal_destiny = $decoded[0];
			}
		}
		
		if (is_numeric($internal_destiny) && $internal_destiny > 0) {
			$m = Members::findById($internal_destiny);
			if ($m instanceof Member) {
				$contact->addToMembers(array($m));
				$contact->addToSharingTable();
			}
		}
	}
}


function work_time_registration_after_member_save($params, &$ret) {
	$member = array_var($params, 'member');
	// dar permisos para novedades al responsable y auxiliar
	$resp_contact_id = 0;
	$aux_contact_id = 0;
	
	$mcp_controller = new MemberCustomPropertiesController();
	$mcp_controller->add_custom_properties($member);
	
	$resp_cp = CustomProperties::findOne(array('conditions' => "code='responsible'"));
	if ($resp_cp instanceof CustomProperty) {
		$val = CustomPropertyValues::getCustomPropertyValue($member->getId(), $resp_cp->getId());
		if ($val instanceof CustomPropertyValue) {
			$resp_contact_id = $val->getValue();
			$contact = Contacts::findById($resp_contact_id);
			if ($contact instanceof Contact && $contact->getUserType() > 0) {
				work_time_registration_save_module_permissions($contact, $member);
			}
		}
	}
	
	$aux_cp = CustomProperties::findOne(array('conditions' => "code='auxiliar'"));
	if ($aux_cp instanceof CustomProperty) {
		$val = CustomPropertyValues::getCustomPropertyValue($member->getId(), $aux_cp->getId());
		if ($val instanceof CustomPropertyValue) {
			$aux_contact_id = $val->getValue();
			if ($aux_contact_id > 0 && $aux_contact_id != $resp_contact_id) {
				$contact = Contacts::findById($aux_contact_id);
				if ($contact instanceof Contact && $contact->getUserType() > 0) {
					work_time_registration_save_module_permissions($contact, $member);
				}
			}
		}
	}
}

function work_time_registration_save_module_permissions($contact, $member) {
	$ot_d = ObjectTypes::findByName('news_detail');
	$ot_s = ObjectTypes::findByName('news_summary');
	$ws_dim = Dimensions::findByCode('workspaces');
	
	// set system permissions to edit summaries
	$sys_perm = SystemPermissions::instance()->findOne(array('conditions' => 'permission_group_id='.$contact->getPermissionGroupId()));
	if (!$sys_perm->getColumnValue('can_approve_summaries') || !$sys_perm->getColumnValue('can_edit_approved_summaries')) {
		$sys_perm->setColumnValue('can_approve_summaries', true);
		$sys_perm->setColumnValue('can_edit_approved_summaries', true);
		$sys_perm->save();
	}
	
	// set tab permission
	$tpp = TabPanelPermissions::instance()->findOne(array('conditions' => array("permission_group_id=? AND tab_panel_id='time-registration-panel'", $contact->getPermissionGroupId())));
	if (!$tpp instanceof TabPanelPermission) {
		$tpp = new TabPanelPermission();
		$tpp->setPermissionGroupId($contact->getPermissionGroupId());
		$tpp->setTabPanelId('time-registration-panel');
		$tpp->save();
	}
	
	// add r/w/d permissions in $member for this user
	DB::execute("INSERT INTO ".TABLE_PREFIX."contact_member_permissions (permission_group_id, member_id, object_type_id, can_delete, can_write) VALUES
		(".$contact->getPermissionGroupId().",".$member->getId().",".$ot_d->getId().",1,1)
		ON DUPLICATE KEY UPDATE can_delete=1, can_write=1");
	DB::execute("INSERT INTO ".TABLE_PREFIX."contact_member_permissions (permission_group_id, member_id, object_type_id, can_delete, can_write) VALUES
		(".$contact->getPermissionGroupId().",".$member->getId().",".$ot_s->getId().",1,1)
		ON DUPLICATE KEY UPDATE can_delete=1, can_write=1");
	DB::execute("INSERT INTO ".TABLE_PREFIX."contact_dimension_permissions (permission_group_id, dimension_id, permission_type) VALUES
		(".$contact->getPermissionGroupId().",".$ws_dim->getId().",'check')
		ON DUPLICATE KEY UPDATE permission_type='check'");
}


function work_time_registration_additional_member_node_class($member, &$class) {
	$ws_ot = ObjectTypes::findByName('workspace');
	if ($member instanceof Member && $member->getObjectTypeId() == $ws_ot->getId()) {
		$now = DateTimeValueLib::now()->format('Y-m-d') . ' 00:00:00';
		$summary = NewsSummaries::findOne(array('conditions' => "`date`='$now' AND `status`='approved' AND `internal_destiny`=".$member->getId()));
		if (!$summary instanceof NewsSummary) {
			$class = 'pending-summary-node';
		}
	}
}


function work_time_registration_contact_selector_plugin_filters($plugin_filters, &$plugin_conditions) {
	foreach ($plugin_filters as $col => $val) {
		if ($col == 'internal_destiny') {
			$ws = Members::getMemberById($val);
			if ($ws instanceof Member) {
				$member_ids = $ws->getAllChildrenIds(true);
				$member_ids[] = $ws->getId();
				$plugin_conditions .= " AND `internal_destiny` IN (".implode(',', $member_ids).")";
			}
		}
	}
}