og.TimeRegistrationManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.actual_status_filter = 'all',
	this.actual_type_filter = 'all',
	this.date_filter = '';
	
	if (!og.TimeRegistrationManager.store) {
		og.TimeRegistrationManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('time_registration', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'regs',
				totalProperty: 'totalCount',
				id: 'object_id',
				fields: [
					'object_id', 'person_id', 'person_name', 'date', 'hour', 'minute', 'comment', 'news_type_id', 'memPath', 'observations', 'img_url', 'hierarchy_name', 'is_present_type'
				]
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					
					var d = this.reader.jsonData;
					
					this.fireEvent('after list load', "");
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("news details"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("news details")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					
					Ext.getCmp('time-registration-manager').date_filter = d.date_filter;
					
					var man = Ext.getCmp("time-registration-manager");
					
					var status_filter = Ext.getCmp('ogNewsDetailsStatusFilterCombo');
					status_filter.setValue(man.actual_status_filter);
					status_filter.collapse();
					
					var type_filter = Ext.getCmp('ogNewsDetailsTypeFilterCombo');
					type_filter.setValue(man.actual_type_filter);
					type_filter.collapse();
					
					var sm = Ext.getCmp('time-registration-manager').getSelectionModel();
					sm.clearSelections();
					
					if (d && d.totals) {
						man.updateGridPagingToolbar({totalCount: d.totals.total_rows});
					} else {
						Ext.getCmp('time-registration-manager').reloadGridPagingToolbar('time_registration','list_all','time-registration-manager');
					}
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.TimeRegistrationManager.store.setDefaultSort('date', 'desc');
	}
	this.store = og.TimeRegistrationManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderDragHandle(value, p, r, ix) {
		return '';
		//return '<div class="img-grid-drag" title="' + lang('click to drag') + '" onmousedown="var sm = Ext.getCmp(\'time-registration-manager\').getSelectionModel();if (!sm.isSelected('+ix+')) sm.clearSelections();sm.selectRow('+ix+', true);"></div>';
	}
	
	var readClass = 'read-unread-' + Ext.id();
	
	function renderIcon(value, p, r) {
		return String.format('<div class="work-time-registration picture-file-small"><img src="{0}" alt="{1}" /></div>', r.data.img_url, og.clean(r.data.name));
	}
	
	function renderType(value, p, r) {
		var t = og.work_time_registration.news_types[value];
		var str = '';
		if (t && t.name) str = t.name;
		
		return str;
	}
	
	function renderComment(value, p, r) {
		return og.clean(value);
	}
	
	function renderDate(value, p, r) {
		return value;
	}
	
	function renderTime(value, p, r) {
		var h = r.data.hour;
		if (h < 10) h = '0' + h;
		var m = r.data.minute;
		if (m < 10) m = '0' + m;
		
		return h + ':' + m;
	}
	
	function renderContact(value, p, r) {
		var onclick = 'og.openLink(og.getUrl(\'time_registration\', \'view\', {id: '+r.data.object_id+'})); return false;';
		return String.format('<a href="{1}" onclick="{2}" title="{0}">{0}</a>', value, og.getUrl('time_registration', 'view', {id: r.data.object_id}), onclick);
	}
	
	function renderDestiny(value, p, r) {
		mem_path = "";
		if (r.data.memPath){
			var mpath = Ext.util.JSON.decode(r.data.memPath);
		}
		if (mpath){ 
			mem_path = "<div class='breadcrumb-container' style='display: inline-block;'>";
			mem_path += og.getEmptyCrumbHtml(mpath, '.breadcrumb-container', og.breadcrumbs_skipped_dimensions);
			mem_path += "</div>";
		}
		
		return mem_path;
	}

	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.object_id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			return selections[0].data.object_id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange', function() {
		if (sm.getCount() <= 0) {
			actions.edit.setDisabled(true);
			actions.del.setDisabled(true);
			//actions.archive.setDisabled(true);
		} else {
			actions.edit.setDisabled(false);
			actions.del.setDisabled(false);
			//actions.archive.setDisabled(false);
		}
	});
	
	var cm = new Ext.grid.ColumnModel([
		sm,{
		/*	id: 'draghandle',
			header: '&nbsp;',
			dataIndex: 'draghandle',
			width: 18,
        	renderer: renderDragHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{*/
			id: 'icon',
			header: '&nbsp;',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'date',
			header: lang("date"),
			dataIndex: 'date',
			width: 80,
			renderer: renderDate,
			sortable:true
		},{
			id: 'time',
			header: lang("hour"),
			dataIndex: 'hour',
			width: 80,
			renderer: renderTime,
			sortable:true
		},{
			id: 'hierarchy_name',
			header: lang("person hierarchy"),
			dataIndex: 'hierarchy_name',
			width: 150,
			renderer: renderComment,
			sortable:true
		},{
			id: 'person_name',
			header: lang("person"),
			dataIndex: 'person_name',
			width: 100,
			renderer: renderContact,
			sortable:true
		},{
			id: 'news_type_id',
			header: lang("news type"),
			dataIndex: 'news_type_id',
			width: 100,
			renderer: renderType,
			sortable:true
		},{
			id: 'observations',
			header: lang("observations"),
			dataIndex: 'observations',
			width: 150,
			renderer: og.clean,
			sortable:true
		},{
			id: 'memPath',
			header: lang("internal destiny"),
			dataIndex: 'memPath',
			width: 150,
			renderer: renderDestiny,
			sortable:false
        }]);
	cm.defaultSortable = false;
    cm.on('hiddenchange', this.afterColumnShowHide, this);

		
	actions = {
		newCO: new Ext.Action({
			id: 'new_button_news_detail',
			text: lang('new'),
            tooltip: lang('add news detail'),
            iconCls: 'ico-new',
            hidden: og.replace_list_new_action && og.replace_list_new_action.news_detail,
            handler: function() {
				var url = og.getUrl('time_registration', 'add', {date: Ext.getCmp('time-registration-manager').date_filter});
				og.openLink(url, null);
			}
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				var url = og.getUrl('time_registration', 'edit', {id:getFirstSelectedId()});
				og.openLink(url, null);
			},
			scope: this
		}),
		del: new Ext.Action({
			text: lang('move to trash'),
            tooltip: lang('move selected objects to trash'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				var confirm_trash_config = parseInt(og.preferences['enableTrashConfirmation']);
				
				if (og.confirmNorification(lang('confirm move to trash'), confirm_trash_config)) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		}),
		approve: new Ext.Action({
			text: lang('approve summary'),
            tooltip: lang('approve summary'),
        //    hidden: true,
            handler: function() {
				if (confirm(lang('confirm approve summary'))) {
					og.openLink(og.getUrl('time_registration', 'approve_summary'));
				}
			},
			scope: this
		}),
		print: new Ext.Action({
			text: lang('print summary'),
            tooltip: lang('print summary'),
            iconCls: 'ico-print',
			handler: function() {
				var date = Ext.getCmp('time-registration-manager').date_filter;
				og.openLink(og.getUrl('time_registration_reports', 'summary_report', {from_list:1, date_start: date, date_end: date, show_details:'checked'}), {caller:'reporting'});
			},
			scope: this
		}),
		copy_last_day_details: new Ext.Action({
			text: lang('copy last day details'),
            tooltip: lang('copy last day details title'),
            iconCls: 'ico-copy',
			handler: function() {
				Ext.Msg.show({
					title: '',
					msg: lang('are you sure you want to copy details from yesterday')+'<br />'+lang('existing news details will not be overwritten'),
					buttons: Ext.Msg.YESNO,
					fn: function(button, text){
						if (button == 'yes') {
							var date = Ext.getCmp('time-registration-manager').date_filter;
							og.openLink(og.getUrl('time_registration', 'generate_from_last_day', {date_to: date}));
						}
					},
					icon: Ext.MessageBox.QUESTION
				});
			},
			scope: this
		}),
		edit_obs: new Ext.Action({
			text: lang('edit summary observations'),
            tooltip: lang('edit summary observations'),
            iconCls: 'ico-properties',
			handler: function() {
				var date = Ext.getCmp('time-registration-manager').date_filter;
				og.openLink(og.getUrl('time_registration', 'edit_summary_observations', {date: date}));
			},
			scope: this
		})
    };
	
	filters = {
		status_filter: new Ext.form.ComboBox({
	    	id: 'ogNewsDetailsStatusFilterCombo',
	    	store: new Ext.data.SimpleStore({
		        fields: ['value', 'text'],
		        data : [['all', lang('show all novelties')], ['approved', lang('show only approved')], ['pending', lang('show only pending')]]
		    }),
		    displayField: 'text',
	        mode: 'local',
	        triggerAction: 'all',
	        selectOnFocus: true,
	        width: 185,
	        valueField: 'value',
	        valueNotFoundText: '',
	        listeners: {
	        	'select' : function(combo, record) {
					var man = Ext.getCmp("time-registration-manager");
					man.actual_status_filter = combo.getValue();
					man.reloadFiltering();
	        	}
	        }
		}),
		type_filter: new Ext.form.ComboBox({
	    	id: 'ogNewsDetailsTypeFilterCombo',
	    	store: new Ext.data.SimpleStore({
		        fields: ['value', 'text'],
		        data : [['all', lang('all people')], ['present', lang('presents')], ['no_present', lang('not presents')]]
		    }),
		    displayField: 'text',
	        mode: 'local',
	        triggerAction: 'all',
	        selectOnFocus: true,
	        width: 125,
	        valueField: 'value',
	        valueNotFoundText: '',
	        listeners: {
	        	'select' : function(combo, record) {
					var man = Ext.getCmp("time-registration-manager");
					man.actual_type_filter = combo.getValue();
					man.reloadFiltering();
	        	}
	        }
		})
	}
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		if (og.replace_list_new_action && og.replace_list_new_action.news_detail) {
			tbar.push(og.replace_list_new_action.news_detail);
		}
		
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.del);
		tbar.push('-');
		tbar.push(actions.print);
		tbar.push(actions.copy_last_day_details);
		tbar.push('-');
		tbar.push(filters.status_filter);
		tbar.push(filters.type_filter);
		tbar.push('-');
		tbar.push(actions.edit_obs);
	}
	
	var parentHeight = document.getElementById('time-registration-panel').offsetHeight;
	
	og.TimeRegistrationManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		enableDrag: true,
		ddGroup: 'MemberDD',
		stateful: og.preferences['rememberGUIState'],
		id: 'time-registration-manager',
		stripeRows: true,
		closable: true,
		height: parentHeight - 115 - 50,
		loadMask: false,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: true
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
	$(window).resize(function() {
		og.timeRegResizePanel();
	});
};

og.timeRegResizePanel = function() {
	var parentHeight = document.getElementById('time-registration-panel').offsetHeight;
	var h = parentHeight - 115 - 50;
	
	$("#time-registration-manager .x-grid3-scroller").css('height', h + 'px');
	$("#time-registration-manager .x-panel-body.x-panel-body-noheader").css('height', (h-58) + 'px');
	$("#time-registration-manager .x-panel-body.x-panel-body-noheader .x-grid3").css('height', (h-58) + 'px');
	
}

Ext.extend(og.TimeRegistrationManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}
		
		this.store.baseParams = {
			context: og.contextManager.plainContext(),
			status_filter: params.status_filter ? params.status_filter : this.actual_status_filter,
			type_filter: params.type_filter ? params.type_filter : this.actual_type_filter,
			date_filter: params.date_filter ? params.date_filter : this.date_filter
		};
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
		og.timeRegResizePanel();
	},
	
	resetVars: function(){
		this.viewUnclassified = false;
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.load({start:0});
		}
	},
	
	reset: function() {
		this.load({start:0});
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	reloadFiltering: function() {
		this.load({
			start:0, 
			status_filter: this.actual_status_filter,
			type_filter: this.actual_type_filter,
			date_filter: this.date_filter
		});
	},
	
	trashObjects: function() {
		if (confirm(lang('confirm move to trash'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	},
	
	archiveObjects: function() {
		if (confirm(lang('confirm archive selected objects'))) {
			this.load({
				action: 'archive',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
});


Ext.reg("time-registration-panel", og.TimeRegistrationManager);

/************************************************
Container for TimeRegistrationManager,
*************************************************/
og.TimeRegistrationManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.TimeRegistrationManager();
	this.summary = new og.HtmlPanel({
		html: '',
		id: 'time-registration-summary',
		style: 'height: 115px;'
	});
	this.obs = new og.HtmlPanel({
		html: '',
		id: 'time-registration-obs',
		style: 'height: 50px;'
	});
	
	og.TimeRegistrationManagerPanel.superclass.constructor.call(this, {
		id: 'time-reg-manager-container',
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.summary,
			this.manager,
			this.obs
		]
	});
}

Ext.extend(og.TimeRegistrationManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
		var url_params = {};
		if (this.manager.date_filter) url_params.summary_date = this.manager.date_filter;
		
		og.openLink(og.getUrl('time_registration', 'summary_panel', url_params), {
			callback: function(success, data) {
				var summary_panel = Ext.getCmp('time-registration-summary');
				summary_panel.load(data.html);
			}
		});
		og.openLink(og.getUrl('time_registration', 'observations_panel', url_params), {
			callback: function(success, data) {
				var summary_panel = Ext.getCmp('time-registration-obs');
				summary_panel.load(data.html);
			}
		});
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
		var url_params = {};
		if (this.manager.date_filter) url_params.summary_date = this.manager.date_filter;
		
		og.openLink(og.getUrl('time_registration', 'summary_panel', url_params), {
			callback: function(success, data) {
				var summary_panel = Ext.getCmp('time-registration-summary');
				summary_panel.load(data.html);
			}
		});
		og.openLink(og.getUrl('time_registration', 'observations_panel', url_params), {
			callback: function(success, data) {
				var summary_panel = Ext.getCmp('time-registration-obs');
				summary_panel.load(data.html);
			}
		});
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	},
	setDateFilter: function(d) {
		this.manager.date_filter = d;
	}
});

Ext.reg("time-registration-containerpanel", og.TimeRegistrationManagerPanel);


Ext.grid.GridView.override({
	getRowClass: function (record, index, rowParams, store) {
		if (record.data.is_present_type) {
			return "time-reg-row-status-present";
		} else if (record.data.observations) {
			return "time-reg-row-status"+(record.data.observations=='' ? '' : '-obs');
		} else {
			return "";
		}
	}/*,
	focusCell : function(row, col, hscroll){
		this.syncFocusEl(this.ensureVisible(row, col, hscroll));
		this.focusEl.focus.defer(1, this.focusEl);
	},

    syncFocusEl : function(row, col, hscroll){
        var xy = row;
        if(!Ext.isArray(xy)){
            row = Math.min(row, Math.max(0, this.getRows().length-1));
            //xy = this.getResolvedXY(this.resolveCell(row, col, hscroll));
        }
        this.focusEl.setXY(xy||this.scroller.getXY());
    }*/
});

